/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.model;

import java.awt.event.ActionEvent;
import java.util.EventObject;

import javax.swing.ActionMap;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.*;

import com.jgoodies.binding.beans.Model;
import com.jgoodies.skeleton.domain.Description;
import com.jgoodies.skeleton.domain.Project;
import com.jgoodies.skeleton.domain.Shaft;
import com.jgoodies.skeleton.model.node.NavigationNode;
import com.jgoodies.skeleton.model.node.RootNode;
import com.jgoodies.uif.application.Action;
import com.jgoodies.uif.application.Application;

/**
 * Provides bound properties, operations and Actions
 * for the navigation tree.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.23 $
 */
public final class NavigationModel extends Model {

    // Names of the Bound Bean Properties *************************************

    public static final String PROPERTYNAME_SELECTION = "selection";
    public static final String PROPERTYNAME_SELECTION_TYPE = "selectionType";
    public static final String PROPERTYNAME_NAVIGATION_TREE_MODEL = "navigationTreeModel";


    // Action Resource Keys ***************************************************

    public static final String ACTION_ADD_SHAFT   = "addShaft";
    public static final String ACTION_ADD_FLANGE  = "addFlange";
    public static final String ACTION_ADD_SEGMENT = "addSegment";
    public static final String ACTION_DELETE_ITEM = "deleteItem";


    // Instance Fields ********************************************************

    /**
     * Refers to the parent main model.
     * Used to lookup the default parent frame
     */
    private final MainModel mainModel;


    /**
     * Holds the current selection, an instance of Model.
     *
     * @see #getSelection()
     * @see #getSelectionType()
     */
    private Object selection;

    /**
     * Holds the class of the current selection.
     *
     * @see #getSelection()
     * @see #getSelectionType()
     */
    private Class selectionType;


    /**
     * Holds the node that refers to the current selection.
     * Used to report a potential change to its hosting tree model
     * when the selection changes. This is because the node's label
     * may have changed as part of the editor change.<p>
     *
     * This is a poor workaround that is based on the weak assumption
     * that this class's tree selection listener is the first to know
     * about the node change.<p>
     *
     * TODO: Let all nodes observe the relevant domain data that
     * is rendered in the tree view to report a node change.
     */
    private NavigationNode selectedNode;


    /**
     * Holds the model for the navigation tree.
     * It changes everytime this model's project changes.
     *
     * @see #getNavigationTreeModel()
     * @see #getNavigationTreeSelectionModel()
     */
    private TreeModel navigationTreeModel;


    /**
     * Refers to the single selection model for the navigation tree.
     * The selection is reset if the navigation tree model changes.
     *
     * @see #getNavigationTreeSelectionModel()
     * @see #getNavigationTreeModel()
     */
    private final TreeSelectionModel navigationTreeSelectionModel;


    // Instance Creation ******************************************************

    /**
     * Constructs a NavigationModel that has no selection and no tree model.
     *
     * @param mainModel  the parent model, the MainModel
     */
    public NavigationModel(MainModel mainModel) {
        this.mainModel = mainModel;
        setNavigationTreeModel(createNavigationTreeModel(new Project("Empty")));
        navigationTreeSelectionModel = new DefaultTreeSelectionModel();
        navigationTreeSelectionModel.setSelectionMode(
                TreeSelectionModel.SINGLE_TREE_SELECTION);
        initEventHandling();
    }


    private void initEventHandling() {
        navigationTreeSelectionModel.addTreeSelectionListener(
                new NavigationTreeSelectionChangeHandler());
        updateSelectionActionEnablement(null);
    }


    // Accessing the Selection and Selection Type *****************************

    /**
     * Returns the current selection, a domain object. This selection
     * will be updated if the selection in the navigation tree changes.
     * During the transition from one tree selection to another this
     * value may hold the old or new selection.
     *
     * @return the domain object selected in the navigation tree.
     *
     * @see #getSelectionType()
     */
    public Object getSelection() {
        return selection;
    }


    /**
     * Sets a new domain object as selection. This method is invoked
     * by the NavigationTreeSelectionChangeHandler that observes changes in
     * the selection of the navigation tree.
     *
     * @param newSelection   the selection to set
     *
     * @see #getSelection()
     */
    private void setSelection(Object newSelection) {
        Object oldSelection = getSelection();
        selection = newSelection;
        setSelectionType(newSelection.getClass());
        firePropertyChange(PROPERTYNAME_SELECTION, oldSelection, newSelection);
        mainModel.getEditorModel().selectionChanged(oldSelection, newSelection);
    }


    /**
     * Returns this model's selected node.
     *
     * @return this model's selected node.
     */
    private NavigationNode getSelectedNode() {
        return selectedNode;
    }


    /**
     * Sets the selection and reports the previously selected node as changed.<p>
     *
     * <strong>Note:</strong> This code requires a special order. It assumes
     * that domain changes happen when the selection changes.<p>
     *
     * TODO: Remove this code and let the nodes listen to domain properties.
     *
     * @param newSelectedNode   the new node selection
     */
    private void setSelectedNode(NavigationNode newSelectedNode) {
        NavigationNode oldSelectedNode = selectedNode;
        setSelection(newSelectedNode.getModel());
        selectedNode = newSelectedNode;
        ((DefaultTreeModel) getNavigationTreeModel()).nodeChanged(oldSelectedNode);
    }


    /**
     * Returns the class of the selected domain object.
     *
     * @return the class of the selected domain object.
     *
     * @see #getSelection()
     */
    public Class getSelectionType() {
        return selectionType;
    }


    /**
     * Sets a new selection type. This method is invoked in
     * <code>#setSelection</code> with the class of the new selection.
     *
     * @param newSelectionType   the selection type to set
     *
     * @see #getSelectionType()
     */
    private void setSelectionType(Class newSelectionType) {
        Class oldSelectionType = getSelectionType();
        selectionType = newSelectionType;
        firePropertyChange(PROPERTYNAME_SELECTION_TYPE, oldSelectionType,
                                                        newSelectionType);
        mainModel.getHelpModel().updateHelpSet(getSelection());
    }


    // Managing the Navigation Tree Model *************************************

    /**
     * Returns the tree model for the navigation tree.
     *
     * @return the tree model for the navigation tree.
     */
    public TreeModel getNavigationTreeModel() {
        return navigationTreeModel;
    }


    /**
     * Sets a new tree model for the navigation tree.<p>
     *
     * TODO: The model shall set the tree selection to the first top-level
     * node, the project description. Currently this is done in a view,
     * see <code>NavigationView#updateModel(TreeModel)</code>.
     *
     * @param newModel  the new tree model to set
     */
    private void setNavigationTreeModel(TreeModel newModel) {
        TreeModel oldModel = getNavigationTreeModel();
        navigationTreeModel = newModel;
        firePropertyChange(PROPERTYNAME_NAVIGATION_TREE_MODEL, oldModel, newModel);
    }


    /**
     * Returns the fixed selection model for the navigation tree.
     *
     * @return the fixed selection model for the navigation tree.
     */
    public TreeSelectionModel getNavigationTreeSelectionModel() {
        return navigationTreeSelectionModel;
    }


    /**
     * Creates a new navigation tree model and sets it as
     * this class' tree model. Invoked by the parent MainModel
     * if the project has changed.
     */
    void updateNavigationTreeModel() {
        setNavigationTreeModel(createNavigationTreeModel(mainModel.getProject()));
    }


    /**
     * Creates and returns a tree model for the given project.
     * Constructs a <code>RootNode</code> with the project and
     * returns a <code>DefaultTreeModel</code> with this root.
     *
     * @param aProject   the project to create the tree for
     * @return a TreeModel for the given project
     */
    private TreeModel createNavigationTreeModel(Project aProject) {
        return new DefaultTreeModel(new RootNode(aProject));
    }


    // Actions ***************************************************************

    @Action
    public void addShaft(ActionEvent e) {
        // mainModel().addShaft();
        showMessage(e, "Add shaft performed.");
    }

    @Action
    public void addFlange(ActionEvent e) {
        // mainModel().addFlangeNode();
        showMessage(e, "Add flange performed.");
    }

    @Action
    public void addSegment(ActionEvent e) {
        // mainModel().addSegmentNode();
        showMessage(e, "Add segment performed.");
    }

    @Action
    public void deleteItem(ActionEvent e) {
        // mainModel().removeSelectedItem();
        showMessage(e, "Item removed.");
    }

    private void showMessage(EventObject e, String what) {
        mainModel.showMessage(e, what);
    }


    // Event Handling *********************************************************

    /**
     * Listens to changes in the tree node selection and updates the selection.
     */
    private final class NavigationTreeSelectionChangeHandler implements TreeSelectionListener {

        /**
         * The selected node in the navidation tree has changed.
         * Updates this model's selected domain object and in turn
         * the selection type.<p>
         *
         * Does nothing if the new tree selection is {@code null}.
         *
         * @param evt    the event that describes the change
         */
        public void valueChanged(TreeSelectionEvent evt) {
            TreePath path = navigationTreeSelectionModel.getSelectionPath();
            NavigationNode aSelectedNode = path == null
                ? null
                : (NavigationNode) path.getLastPathComponent();
            updateSelectionActionEnablement(aSelectedNode);
            if (NavigationModel.this.equals(getSelectedNode(), aSelectedNode)) {
                return;
            }
            acceptOrRejectChange(path, aSelectedNode, evt);
        }


        private void acceptOrRejectChange(TreePath path, NavigationNode aSelectedNode, EventObject evt) {
            if (aSelectedNode == null)
                return;

            String validationResult = validate(getSelection());
            if (validationResult == null) {
                setSelectedNode(aSelectedNode);
            } else {
                // Revert to the old selection path
                navigationTreeSelectionModel.removeSelectionPath(path);
                setSelectionPathFor(getSelectedNode());
                // Update the tree node
                ((DefaultTreeModel) getNavigationTreeModel()).nodeChanged(getSelectedNode());
                // Show a warning
                mainModel.showValidationMessage(evt, validationResult, false);
            }
        }


        private String validate(Object model) {
            return mainModel.getEditorModel().validateBeforeSelectionChange(model);
        }


        private void setSelectionPathFor(TreeNode node) {
            Object[] pathToRoot =
                ((DefaultTreeModel) navigationTreeModel).getPathToRoot(getSelectedNode());
            navigationTreeSelectionModel.addSelectionPath(new TreePath(pathToRoot));
        }


    }


    // Updating the Action Enablement *****************************************

    /**
     * Updates the enablement state actions related to the node selection.
     *
     * @param aSelectedNode   the selected node
     */
    private void updateSelectionActionEnablement(NavigationNode aSelectedNode) {
        ActionMap map = Application.getActionMap(this);
        Object aSelection = aSelectedNode == null
            ? null
            : aSelectedNode.getModel();

        boolean canAddSubsystem = aSelection != null;
        boolean canAddComponent = aSelection instanceof Shaft;
        boolean canDelete =
            aSelection != null && !(aSelection instanceof Description);

        map.get(ACTION_ADD_SHAFT).setEnabled(canAddSubsystem);
        map.get(ACTION_ADD_FLANGE).setEnabled(canAddComponent);
        map.get(ACTION_ADD_SEGMENT).setEnabled(canAddComponent);
        map.get(ACTION_DELETE_ITEM).setEnabled(canDelete);
    }


}
