/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.model.help;

import java.net.URL;

import javax.swing.Icon;

import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;

/**
 * Instances of this class define help nodes in the dynamic help.
 * A help node is either a chapter or topic, where only topics have
 * an attached URL.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.12 $
 */

public final class HelpNode {

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(HelpNode.class);

    private static final Icon CHAPTER_ICON =
        RESOURCES.getIcon("com.jgoodies.help.openBook.icon");

    private static final Icon TOPIC_ICON =
        RESOURCES.getIcon("com.jgoodies.help.topic.icon");

    public static final int CHAPTER_ITEM = 0;
    public static final int TOPIC_ITEM = 1;

    private final String name;
    private final int type;
    private final URL url;


    // Instance Creation ******************************************************

    /**
     * Constructs a <code>HelpNode</code> with the given name, type, and URL.
     */
    private HelpNode(String name, int type, URL url) {
        this.name = name;
        this.type = type;
        this.url = url;
    }

    /**
     * Creates and returns a chapter help node with the given name.
     */
    static HelpNode createChapter(String name) {
        return new HelpNode(name, CHAPTER_ITEM, null);
    }

    /**
     * Creates and returns a topic help node with the given name and path.
     */
    static HelpNode createTopic(String name, String path) {
        URL url = HelpNode.class.getResource(path);
        return new HelpNode(name, TOPIC_ITEM, url);
    }

    /**
     * Creates and returns the help root node.
     */
    static HelpNode createRoot() {
        return createChapter("root");
    }


    // Accessors **************************************************************

    public boolean isChapter() {
        return type == CHAPTER_ITEM;
    }

    @Override
    public String toString() {
        return name;
    }

    public URL getURL() {
        return url;
    }

    boolean matches(URL aUrl) {
        return aUrl.equals(getURL());
    }

    public Icon getIcon(boolean sel) {
        return isChapter() ? CHAPTER_ICON : TOPIC_ICON;
    }

}