/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.model.help;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;

import com.jgoodies.skeleton.domain.Description;
import com.jgoodies.skeleton.domain.Flange;
import com.jgoodies.skeleton.domain.Segment;
import com.jgoodies.skeleton.domain.Shaft;
import com.jgoodies.skeleton.model.DynamicHelpModel;

/**
 * This class provides help sets (TreeNodes) for each domain class.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.9 $
 *
 * @see     com.jgoodies.skeleton.model.DynamicHelpModel
 */

public final class HelpSets  {

	public static final TreeNode WELCOME_HELP_SET     = createWelcomeHelpSet();
    public static final TreeNode DESCRIPTION_HELP_SET = createDescriptionHelpSet();
    public static final TreeNode SHAFT_HELP_SET       = createShaftHelpSet();
    public static final TreeNode SEGMENT_HELP_SET     = createSegmentHelpSet();
    public static final TreeNode FLANGE_HELP_SET      = createFlangeHelpSet();

    private static final String  URL_PREFIX           = "/com/jgoodies/skeleton/resources/help/dynamic/";


	private HelpSets() {
        // Suppresses default constructor, ensuring non-instantiability.
    }


	/**
	 * Registers all help sets with the DynamicHelpModel.
     *
     * @param helpModel   holds a map from selection types to help sets
	 */
	public static void registerHelpSets(DynamicHelpModel helpModel) {
        helpModel.registerHelp(Description.class, DESCRIPTION_HELP_SET);
        helpModel.registerHelp(Shaft.class,       SHAFT_HELP_SET);
        helpModel.registerHelp(Segment.class,     SEGMENT_HELP_SET);
        helpModel.registerHelp(Flange.class,      FLANGE_HELP_SET);
	}


	/**
	 * Creates and answers the welcome help set.
	 */
	private static TreeNode createWelcomeHelpSet() {
		DefaultMutableTreeNode root = new DefaultMutableTreeNode();
		DefaultMutableTreeNode chapter;

		chapter = createChapter("Help");
		chapter.add(createTopic("Welcome", 					"welcome/welcome"));
		chapter.add(createTopic("How to use dynamic help", 	"welcome/help"));
		root.add(chapter);

		chapter = createChapter("Getting Started");
		chapter.add(createTopic("Create a new project",  	"welcome/new"));
		chapter.add(createTopic("Open existing project", 	"welcome/open"));
		root.add(chapter);

		return root;
	}


	/**
	 * Creates and answers the description help set.
	 */
	private static TreeNode createDescriptionHelpSet() {
		DefaultMutableTreeNode root = new DefaultMutableTreeNode();
		DefaultMutableTreeNode chapter;

		chapter = createChapter("Help");
		chapter.add(createTopic("Project", 		"description/project"));
		chapter.add(createTopic("Manufacturer", "description/manufacturer"));
		chapter.add(createTopic("Inspector", 	"description/inspector"));
		chapter.add(createTopic("Ship", 		"description/ship"));
		root.add(chapter);

		return root;
	}


	/**
	 * Creates and answers the shaft help set.
	 */
	private static TreeNode createShaftHelpSet() {
		DefaultMutableTreeNode root = new DefaultMutableTreeNode();
		DefaultMutableTreeNode chapter;

		chapter = createChapter("Help");
		chapter.add(createTopic("Shaft", 	"shaft/general"));
		chapter.add(createTopic("Comments", "shaft/comments"));
		root.add(chapter);

		chapter = createChapter("Samples");
		chapter.add(createTopic("Sample1", "shaft/sample1"));
		chapter.add(createTopic("Sample2", "shaft/sample2"));
		root.add(chapter);

		return root;
	}



	/**
	 * Creates and answers the segment help set.
	 */
	private static TreeNode createSegmentHelpSet() {
		DefaultMutableTreeNode root = new DefaultMutableTreeNode();
		DefaultMutableTreeNode chapter;

		chapter = createChapter("Help");
		chapter.add(createTopic("Segment", 		"segment/general"));
		chapter.add(createTopic("Diameters", 	"segment/diameters"));
		chapter.add(createTopic("Criteria", 	"segment/criteria"));
		root.add(chapter);

		chapter = createChapter("Samples");
		chapter.add(createTopic("Sample1", 		"segment/sample1"));
		root.add(chapter);

		return root;
	}



	/**
	 * Creates and answers the flange help set.
	 */
	private static TreeNode createFlangeHelpSet() {
		DefaultMutableTreeNode root = new DefaultMutableTreeNode();
		DefaultMutableTreeNode chapter;

		chapter = createChapter("Help");
		chapter.add(createTopic("Flange", 		"flange/general"));
		chapter.add(createTopic("Diameters",	"flange/diameters"));
		chapter.add(createTopic("Criteria", 	"flange/criteria"));
		chapter.add(createTopic("Bolts", 		"flange/bolts"));
		root.add(chapter);

		chapter = createChapter("Samples");
		chapter.add(createTopic("Sample1", 		"flange/sample1"));
		chapter.add(createTopic("Sample2", 		"flange/sample2"));
		root.add(chapter);

		return root;
	}


	/**
	 * Creates and returns a topic node.
	 */
	private static DefaultMutableTreeNode createTopic(String name, String path) {
		String fullPath = URL_PREFIX + path + ".html";
		return new DefaultMutableTreeNode(HelpNode.createTopic(name, fullPath));
	}


	/**
	 * Creates and returns a chapter node.
	 */
	private static DefaultMutableTreeNode createChapter(String name) {
		return new DefaultMutableTreeNode(HelpNode.createChapter(name));
	}

}