/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view.editor;

import javax.swing.Icon;
import javax.swing.JToolBar;

import com.jgoodies.uif.AbstractView;

/**
 * The abstract superclass of all <code>Editor</code> implementations.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.12 $
 */
public abstract class AbstractEditor extends AbstractView implements Editor {

    /**
     * Holds the icon displayed in the title bar.
     * This Editor implementation reused the icon for all
     * domain objects of a specific type. For example all Flanges
     * use the same icon. Other implementors may choose to change
     * the icon with every individual domain instance.
     *
     * @see #getIcon()
     */
    private final Icon icon;

    /**
     * Holds the prefix of the title displayed in the title bar.
     * The prefix is shared for a specific type, for example
     * all Shafts use the same prefix.
     *
     * @see #getTitle()
     */
    private final String titlePrefix;

    /**
     * The suffix for the title displayed in the title bar.
     * This typically changes with the edited object.
     *
     * @see #setTitleSuffix(String)
     */
    private String titleSuffix;


    // Instance Creation ****************************************************

    /**
     * Constructs an AbstractEditor with the specified icon.
     *
     * @param icon          the icon to be used for this editor
     */
    public AbstractEditor(Icon icon) {
        this(icon, "");
    }


    /**
     * Constructs an AbstractEditor with the specified icon and title prefix.
     *
     * @param icon          the icon to be used for this editor
     * @param titlePrefix   the prefix used by this editor, e.g. "Flange"
     */
    public AbstractEditor(Icon icon, String titlePrefix) {
        this.icon = icon;
        this.titlePrefix = titlePrefix;
    }


    /**
     * Constructs an AbstractEditor with the specified title prefix.
     *
     * @param titlePrefix   the prefix used by this editor, e.g. "Flange"
     */
    public AbstractEditor(String titlePrefix) {
        this(null, titlePrefix);
    }


    // Implementing the Viewer Interface ************************************

    /**
     * Returns this editor's icon. The icon is shared for
     * all Flanges, all Shafts, etc.
     *
     * @return this editor's icon
     *
     * @see Editor#getIcon()
     */
    public Icon getIcon() {
        return icon;
    }


    /**
     * Returns this editor's title. The title changes
     * with every individual edited domain object.
     * It is composed of a prefix that is shared for all Flanges,
     * all Shafts, etc. and a suffix that changed with the edited instance.
     *
     * @return this editor's title.
     */
    public String getTitle() {
        return titlePrefix + ' ' + titleSuffix;
    }


    /**
     * Returns this editor's tool bar.
     * This default implementation specifies that no tool bar is used;
     * subclasses may override this behavior.
     *
     * @return this editor's tool bar
     */
    public JToolBar getToolBar() {
        return null;
    }


    /**
     * Activates this editor. This method is invoked
     * if the selection type in the navigation panel has changed
     * and this editor is associated with the new selection.
     * For example if the selection changes from type Flange
     * to type Shaft, this method will be invoked. It will not
     * be invoked if the selection changes from Flange1 to Flange2.<p>
     *
     * This implementation assumes that only editors change domain objects.
     * In other words, we can assume that a domain object is unchanged
     * since its last edit. If the domain objects can be changed by third
     * parties, you may need to update the view if an editor becomes active.<p>
     *
     * Sublcasses may also need to perform additional actions
     * on activation. For example, scroll a list or table to make
     * the current selection visible.
     */
    public void activate() {
        // Do nothing by default; subclasses may override.
    }


    /**
     * Deactivates the editor. This method is invoked
     * if the selection type in the navigation panel has changed
     * and this editor is associated with the old selection.<p>
     *
     * This default implementation does nothing.
     * Subclasses my change the presentation.
     */
    public void deactivate() {
        // Do nothing. The model has been updated before.
    }


    // Abstract Behavior ****************************************************

    /**
     * Sets a new title suffix that is used to compose this editor's title.
     * Typically this is set in <code>#updateView</code>.
     *
     * @param newSuffix    the suffix to be used
     * @see #getTitle()
     */
    protected final void setTitleSuffix(String newSuffix) {
        titleSuffix = newSuffix;
    }


}