/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view.preferences;

import javax.swing.JCheckBox;
import javax.swing.JComponent;

import com.jgoodies.binding.PresentationModel;
import com.jgoodies.binding.adapter.BasicComponentFactory;
import com.jgoodies.binding.beans.PropertyAdapter;
import com.jgoodies.binding.value.BufferedValueModel;
import com.jgoodies.binding.value.ValueModel;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.skeleton.model.UISettingsModel;
import com.jgoodies.uif.AbstractView;
import com.jgoodies.uif.builder.PanelBuilder2;
import com.jgoodies.uifextras.convenience.DefaultTipOfTheDayModel;
import com.jgoodies.uifextras.convenience.TipOfTheDayModel;


/**
 * Builds the <em>General</em> tab in the preferences dialog.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.11 $
 *
 * @see com.jgoodies.uif.AbstractDialog
 * @see com.jgoodies.binding.PresentationModel
 */
public final class GeneralTabView extends AbstractView {

    private final PresentationModel<UISettingsModel> model;

    private JCheckBox showTipsBox;
    private JCheckBox autoExpandBox;
    private JCheckBox autoCollapseBox;


    // Instance Creation ****************************************************

    /**
     * Constructs the <i>General</i> tab for the preferences dialog.
     *
     * @param settingsModel  holds the presentation settings and vends
     *     models for the bound bean properties of the settings object
     */
    public GeneralTabView(PresentationModel<UISettingsModel> settingsModel) {
        this.model = settingsModel;
    }


    // Initialization *********************************************************

    /**
     * Creates and binds the UI components. Creates three check boxes
     * that are bound to properties of the <code>TipOfTheDayDialog</code>
     * and the given <code>UISettingsModel</code>.
     * All models are buffered and change the underlying model properties
     * only if the PresentationModel's <code>triggerChannel</code> indicates that values
     * shall be commited.
     */
    private void initComponents() {
        TipOfTheDayModel tipModel = DefaultTipOfTheDayModel.getInstance();
        ValueModel showTipsModel = new BufferedValueModel(
                new PropertyAdapter(tipModel, TipOfTheDayModel.SHOWING_TIPS_PROPERTY),
                model.getTriggerChannel());
        showTipsBox = BasicComponentFactory.createCheckBox(
                showTipsModel,
                "Show tip of the day");
        showTipsBox.setMnemonic('S');
        showTipsBox.setContentAreaFilled(false);

        autoExpandBox = BasicComponentFactory.createCheckBox(
                model.getBufferedModel(UISettingsModel.PROPERTYNAME_EXPAND_SELECTED_PATHS),
                "Auto expand selected node");
        autoExpandBox.setMnemonic('e');
        autoExpandBox.setContentAreaFilled(false);

        autoCollapseBox = BasicComponentFactory.createCheckBox(
                model.getBufferedModel(UISettingsModel.PROPERTYNAME_COLLAPSE_DESELECTED_PATHS),
                "Auto collapse deselected nodes");
        autoCollapseBox.setMnemonic('c');
        autoCollapseBox.setContentAreaFilled(false);
    }


    // Building *************************************************************

    /**
     * Builds and returns the general panel using the previously
     * created components. Uses a FormLayout to lay out the panel,
     * and a PanelBuilder to add the UI components.
     *
     * @return the general tab
     */
    @Override
    protected JComponent buildPanel() {
        initComponents();

        FormLayout layout = new FormLayout(
                "pref",
                "p, $lg, p, $lg, p");
        PanelBuilder builder = new PanelBuilder2(layout);
        builder.setBorder(Borders.DLU14_BORDER);
        builder.setOpaque(false);
        CellConstraints cc = new CellConstraints();

        builder.add(showTipsBox,      cc.xy(1, 1));
        builder.add(autoExpandBox,    cc.xy(1, 3));
        builder.add(autoCollapseBox,  cc.xy(1, 5));
        return builder.getPanel();
    }


}