/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.animation.extras;

import java.awt.*;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.SwingUtilities;

import com.jgoodies.animation.Animation;
import com.jgoodies.animation.Animations;
import com.jgoodies.animation.animations.BasicTextAnimation;
import com.jgoodies.animation.animations.GlyphAnimation;
import com.jgoodies.animation.components.BasicTextLabel;
import com.jgoodies.animation.components.GlyphLabel;

/**
 * Renders animations on top of the application GUI.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.6 $
 */
public final class Prompter {

    public static final int NORTH  = 0;
    public static final int EAST   = 1;
    public static final int SOUTH  = 2;
    public static final int WEST   = 3;
    public static final int CENTER = 4;

    private static final Color INFO_COLOR   = Color.DARK_GRAY;
    private static final Color PROMPT_COLOR = Color.RED;

    private final JFrame frame;
    private final PrompterGlassPane glassPane;

    public Prompter(JFrame frame) {
        this.frame = frame;
        glassPane = new PrompterGlassPane();
    }

    // The Public API *******************************************************

    /**
     * Sets the prompter's glass pane to visible or invisible.
     */
    public void setVisible(boolean b) {
        glassPane().setVisible(b);
    }

    /**
     * Sets a list of hidden items (rectangles or components).
     */
    public void setHiddenItems(List hiddenItems) {
        glassPane().setHiddenItems(hiddenItems);
        glassPane().repaint();
    }

    /**
     * Returns the prompter's glass pane.
     */
    public PrompterGlassPane glassPane() {
        return glassPane;
    }

    // Animation Creation: Set Bounds ***************************************

    /**
     * Creates and answer a type here animation with the given duration and text.
     */
    public Animation createTypeAnimation(
        long duration,
        String text,
        Component destination,
        Dimension size) {
        Animation setBounds =
            new SetBoundsAnimation(glyphLabel(), destination, size);
        Animation on = new VisibilityAnimation(glyphLabel(), true);
        Animation glyph =
            new GlyphAnimation(glyphLabel(), duration, 100, text);
        Animation off = new VisibilityAnimation(glyphLabel(), false);
        Animation pause = Animations.pause(1);
        return Animations.sequential(
            new Animation[] {setBounds, on, glyph, off, pause});
    }

    /**
     * Creates and answer a click here animation with the given duration and text.
     */
    public Animation createClickAnimation(
        long duration,
        String text,
        Component destination,
        Dimension size) {
        Animation setBounds =
            new SetBoundsAnimation(textLabel(), destination, size);
        Animation scale =
            BasicTextAnimation.defaultScale(
                textLabel(),
                duration,
                text,
                PROMPT_COLOR);
        return Animations.sequential(setBounds, Animations.reverse(scale));
    }

    /**
     * Creates and answers a fade animation with the given duration and text.
     */
    public Animation createFade(
        long duration,
        String text,
        Component destination,
        Dimension size,
        int orientation) {
        Animation setBounds =
            new SetBoundsAnimation(
                textLabel(),
                destination,
                size,
                orientation);
        Animation fade =
            BasicTextAnimation.defaultFade(
                textLabel(),
                duration,
                text,
                INFO_COLOR);
        return Animations.sequential(setBounds, fade);
    }

    /**
     * Creates and answers a scale animation with the given duration and text.
     */
    public Animation createScale(
        long duration,
        String text,
        Component destination,
        Dimension size) {
        Animation setBounds =
            new SetBoundsAnimation(textLabel(), destination, size);
        Animation scale =
            BasicTextAnimation.defaultScale(
                textLabel(),
                duration,
                text,
                PROMPT_COLOR);
        return Animations.sequential(setBounds, scale);
    }

    /**
     * Creates and returns a space animation with the given duration and text.
     */
    public Animation createSpace(
        long duration,
        String text,
        Component dest,
        Dimension size,
        int anchor) {
        Animation setBounds =
            new SetBoundsAnimation(textLabel(), dest, size, anchor);
        Animation space =
            BasicTextAnimation.defaultSpace(
                textLabel(),
                duration,
                text,
                PROMPT_COLOR);
        return Animations.sequential(setBounds, space);
    }

    // Animation Creation: Hide Areas ***************************************

    public Animation createShowAllAnimation() {
        return new HideAnimation(Collections.EMPTY_LIST);
    }

    public Animation createHideAnimation(Component c) {
        List list = new LinkedList();
        list.add(c);
        return new HideAnimation(list);
    }

    public Animation createHideAnimation(Component c1, Component c2) {
        List list = new LinkedList();
        list.add(c1);
        list.add(c2);
        return new HideAnimation(list);
    }

    public Animation createHideAnimation(
        Component c1,
        Component c2,
        Component c3) {
        List list = new LinkedList();
        list.add(c1);
        list.add(c2);
        list.add(c3);
        return new HideAnimation(list);
    }

    // Helper Code **********************************************************

    private Rectangle getTranslatedBounds(
        Component dest,
        Dimension size,
        int anchor) {
        int w = dest.getWidth();
        int h = dest.getHeight();
        Point sourceCenter =
            SwingUtilities.convertPoint(
                dest,
                new Point(0, 0),
                frame.getRootPane());
        int x = sourceCenter.x;
        int y = sourceCenter.y;
        switch (anchor) {
            case NORTH :
                x += (w - size.width) / 2;
                break;

            case WEST :
                y += (h - size.height) / 2;
                break;

            case SOUTH :
                x += (w - size.width) / 2;
                y += (h - size.height);
                break;

            case EAST :
                x += (w - size.width);
                y += (h - size.height) / 2;
                break;

            default : // CENTER
                x += (w - size.width) / 2;
                y += (h - size.height) / 2;
                break;
        }
        return new Rectangle(x, y, size.width, size.height);
    }

    // Accessing Collaborators **********************************************

    private BasicTextLabel textLabel() {
        return glassPane().textLabel();
    }

    private GlyphLabel glyphLabel() {
        return glassPane().glyphLabel();
    }

    // Animation Implementations ********************************************

    /**
     * An animation that sets the bounds of a given component.
     */
    private final class SetBoundsAnimation
        extends Animations.OneTimeAnimation {

        private final Component source;
        private final Component destination;
        private final Dimension size;
        private final int anchor;

        private SetBoundsAnimation(
            Component source,
            Component dest,
            Dimension size) {
            this(source, dest, size, CENTER);
        }

        private SetBoundsAnimation(
            Component source,
            Component destination,
            Dimension size,
            int anchor) {
            this.source = source;
            this.size = size;
            this.destination = destination;
            this.anchor = anchor;
        }

        @Override
        protected void applyEffect(long time) {
            source.setBounds(getTranslatedBounds(destination, size, anchor));
        }

    }


    /**
     * An animation that sets a list of hidden areas in the PromperGlassPane.
     */
    private final class HideAnimation extends Animations.OneTimeAnimation {

        private final List itemsToHide;

        private HideAnimation(List itemsToHide) {
            this.itemsToHide = itemsToHide;
        }

        @Override
        protected void applyEffect(long time) {
            setHiddenItems(itemsToHide);
        }

    }


    /**
     * An animation that just sets a components visibility.
     */
    private static final class VisibilityAnimation
        extends Animations.OneTimeAnimation {

        private final Component component;
        private final boolean visible;

        private VisibilityAnimation(Component c, boolean visible) {
            this.component = c;
            this.visible = visible;
        }

        @Override
        protected void applyEffect(long time) {
            component.setVisible(visible);
        }
    }

    /**
     * An animation that changes the radius and alpha value of
     * a translucent circle.
     *
     * @author Karsten Lentzsch
     */
//    private static class CircleAnimation extends AbstractAnimation {
//
//        private final CircleComponent target;
//        private final AnimationFunctions.FloatFunction radiusFunction;
//        private final AnimationFunction colorFunction;
//
//        /**
//         * Constructs a circle animation using the given
//         * target, duration, radius function, and color function.
//         */
//        private CircleAnimation(
//            CircleComponent target,
//            long duration,
//            AnimationFunction radiusFunction,
//            AnimationFunction colorFunction) {
//            super(duration);
//            this.target = target;
//            this.radiusFunction = AnimationFunctions.asFloat(radiusFunction);
//            this.colorFunction = colorFunction;
//        }
//
//        /**
//         * Applies the effect: sets the radius and color.
//         */
//        protected void applyEffect(long time) {
//            target.setRadius((int) radiusFunction.valueAt(time));
//            // 12 + (int) (time / 60)
//            target.setColor((Color) colorFunction.valueAt(time));
//            target.repaint();
//        }
//    }

}