/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.domain;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Describes a book as returned by the Amazon.com web services.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.8 $
 */
public final class Book {

    // Names of the Bean Properties *******************************************

    public static final String PROPERTYNAME_TITLE            = "title";
    public static final String PROPERTYNAME_AUTHORS          = "authors";
    public static final String PROPERTYNAME_LIST_PRICE       = "listPrice";

    public static final String PROPERTYNAME_BINDING          = "binding";
    public static final String PROPERTYNAME_PAGES            = "pages";
    public static final String PROPERTYNAME_PUBLISHER        = "publisher";
    public static final String PROPERTYNAME_PUBLICATION_DATE = "publicationDate";
    public static final String PROPERTYNAME_ISBN             = "ISBN";
    public static final String PROPERTYNAME_SMALL_IMAGE_URL  = "smallImageURL";
    public static final String PROPERTYNAME_MEDIUM_IMAGE_URL = "mediumImageURL";
    public static final String PROPERTYNAME_REVIEWS          = "reviews";


    // Instance Fields ********************************************************

    private String title;
    private List   authors;
    private String listPrice;

    private String binding;
    private String pages;
    private String publisher;
    private String publicationDate;
    private String isbn;
    private URL    smallImageURL;
    private URL    mediumImageURL;
    private List   reviews;


    // Instance Creation ****************************************************

    public Book() {
        authors = new LinkedList();
        reviews = new LinkedList();
    }


    // Accessors ************************************************************

    /**
     * Returns the ISBN.
     *
     * @return  the ISBN
     */
    public String getISBN() {
        return isbn;
    }

    /**
     * Returns the product name.
     *
     * @return String
     */
    public String getTitle() {
        return title;
    }

    /**
     * Returns the list of authors.
     *
     * @return   the list of authors
     */
    public List getAuthors() {
        return authors;
    }

    /**
     * Returns the release date.
     *
     * @return the release date
     */
    public String getPublicationDate() {
        return publicationDate;
    }

    /**
     * Returns the manufacturer.
     *
     * @return String
     */
    public String getPublisher() {
        return publisher;
    }

    /**
     * Returns the URL of the small image.
     * @return  a URL for the small image
     */
    public URL getSmallImageURL() {
        return smallImageURL;
    }

    /**
     * Returns the URL of the medium sized image.
     *
     * @return String
     */
    public URL getMediumImageURL() {
        return mediumImageURL;
    }

    /**
     * Returns the list of reviews.
     *
     * @return the list of reviews
     */
    public List getReviews() {
        return reviews;
    }

    /**
     * Returns our price.
     *
     * @return our price
     */
    public String getListPrice() {
        return listPrice;
    }

    /**
     * Sets the ISBN.
     *
     * @param isbn  The ISBN to set
     */
    public void setISBN(String isbn) {
        this.isbn = isbn;
    }

    /**
     * Sets the product name.
     *
     * @param productName The product name to set
     */
    public void setTitle(String productName) {
        this.title = productName;
    }

    /**
     * Sets the authors.
     *
     * @param authors   the List of authors
     */
    public void setAuthors(List authors) {
        this.authors = authors;
    }

    /**
     * Sets the release date.
     *
     * @param releaseDate  The release date to set
     */
    public void setPublicationDate(String releaseDate) {
        this.publicationDate = releaseDate;
    }

    /**
     * Sets the small image URL.
     *
     * @param imageUrlSmall The imageUrlMedium to set
     */
    public void setSmallImageURL(String imageUrlSmall) {
        try {
            this.smallImageURL = new URL(imageUrlSmall);
        } catch (MalformedURLException e) {
            // Can't do anything useful
        }
    }

    /**
     * Sets the medium image URL.
     *
     * @param imageUrlMedium The imageUrlMedium to set
     */
    public void setMediumImageURL(String imageUrlMedium) {
        try {
            this.mediumImageURL = new URL(imageUrlMedium);
        } catch (MalformedURLException e) {
            // Can't do anything useful
        }
    }

    /**
     * Sets the publisher.
     *
     * @param publisher The publisher to set
     */
    public void setPublisher(String publisher) {
        this.publisher = publisher;
    }

    /**
     * Sets the list price.
     *
     * @param listPrice  The price to set
     */
    public void setListPrice(String listPrice) {
        this.listPrice = listPrice;
    }

    /**
     * Sets the reviews.
     *
     * @param reviews  The reviews to set
     */
    public void setReviews(List reviews) {
        this.reviews = reviews == null
            ? new LinkedList()
            : reviews;
    }


    // Convenience Accessors ************************************************

    /**
     * Returns the editorial review.
     *
     * @return the editorial review text
     */
    public String getEditorialReview() {
        return "This book is a strong buy.";
    }


    /**
     * Computes and returns the first author of the given details
     * followed by an optional <em>, et al</em>.
     *
     * @return a string that consists of the first author and
     *     optional <em>, et al</em>.
     */
    public String getAuthorsEtAl() {
        StringBuilder result = new StringBuilder();
        if (getAuthors().size() > 0) {
            result.append(getAuthors().get(0));
        }
        if (getAuthors().size() > 1) {
            result.append(", et al");
        }
        return result.toString();
    }

    /**
     * Computes and returns the list of authors of the given details instance
     * as concatenated strings, separated by a colon.
     *
     * @return a string that consists of the concatenated authors
     */
    public String getConcatenatedAuthors() {
        StringBuilder result = new StringBuilder();
        Iterator i = getAuthors().iterator();
        if (i.hasNext()) {
            result.append(i.next());
            while (i.hasNext()) {
                result.append(", ");
                result.append(i.next());
            }
        }
        return result.toString();
    }


    public float getPrice() {
        if (listPrice == null)
            return -1;
        try {
            return Float.parseFloat(listPrice.substring(1));
        } catch (NumberFormatException e) {
            return -1;
        }
    }


    public int getYear() {
        if (publicationDate == null) {
            return -1;
        }
        String year = publicationDate.substring(publicationDate.length() - 4);
        try {
            return Integer.parseInt(year);
        } catch (NumberFormatException e) {
            return -1;
        }
    }


    /**
     * @return the binding.
     */
    public String getBinding() {
        return binding;
    }


    /**
     * @param binding the binding to set
     */
    public void setBinding(String binding) {
        this.binding = binding;
    }


    /**
     * @return the pages.
     */
    public String getPages() {
        return pages;
    }


    /**
     * @param pages the pages to set
     */
    public void setPages(String pages) {
        this.pages = pages;
    }



}
