/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.domain;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.prefs.Preferences;

import com.jgoodies.binding.beans.Model;

/**
 * Used to filter books in a list of books as done in the
 * <code>SearchResultModel</code>.
 * The filtering is performed by the <code>#accept(Book)</code>
 * and <code>#filter(Collection)</code> methods.<p>
 *
 * The FilterDialog presents and edits this class' properties.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.5 $
 *
 * @see com.jgoodies.bookfinder.model.SearchResultModel
 * @see com.jgoodies.bookfinder.view.dialog.FilterDialog
 */
public final class BookFilter extends Model {


    // Names of the Bound Properties *****************************************

    public static final String PROPERTYNAME_LIMIT          = "limit";
    public static final String PROPERTYNAME_LIMIT_ENABLED  = "limitEnabled";
    public static final String PROPERTYNAME_PRICE          = "price";
    public static final String PROPERTYNAME_PRICE_ENABLED  = "priceEnabled";
    public static final String PROPERTYNAME_YEAR           = "year";
    public static final String PROPERTYNAME_YEAR_ENABLED   = "yearEnabled";


    // Default Values ********************************************************

    private static final boolean DEFAULT_LIMIT_ENABLED  = false;
    private static final boolean DEFAULT_PRICE_ENABLED  = false;
    private static final boolean DEFAULT_YEAR_ENABLED   = false;
    private static final int     DEFAULT_LIMIT      	   =  100;
    private static final int     DEFAULT_PRICE      	   =   50;
    private static final int     DEFAULT_YEAR       	   = 1990;


    // Fields ***************************************************************

    public boolean limitEnabled  = DEFAULT_LIMIT_ENABLED;
    public boolean priceEnabled  = DEFAULT_PRICE_ENABLED;
    public boolean yearEnabled   = DEFAULT_YEAR_ENABLED;

    public int     limit     = DEFAULT_LIMIT;
    public int     price     = DEFAULT_PRICE;
    public int     year      = DEFAULT_YEAR;


    // Filtering ************************************************************

    /**
     * Tests and answers if the specified details should be displayed
     * in the overview.
     *
     * @param details   the details to check
     * @return true if accepted, false if rejected
     */
    public boolean accept(Book details) {
        float aPrice = details.getPrice();
        if (priceEnabled && (aPrice != -1) && (aPrice > this.price))
            return false;

        int aYear = details.getYear();
        if (yearEnabled && (aYear != -1) && (aYear < this.year))
            return false;

        return true;
    }


    /**
     * Filters the given elements and returns a list of elements
     * that are accepted by this filter. The size of the filtered
     * list is bound by this filter's limit.
     *
     * @param allElements  the element list to be filtered
     * @return a list of accepted elements
     */
    public List filter(Collection allElements) {
        List acceptedElements = new ArrayList();
        int index = 1;

        for (Iterator i = allElements.iterator(); i.hasNext();) {
            Book details = (Book) i.next();
            if (limitEnabled && (index > limit))
                break;
            if (accept(details)) {
                acceptedElements.add(details);
                index++;
            }
        }
        return acceptedElements;
    }


    // Accessing Bound Properties *******************************************


    /**
     * Returns this filter's limit.
     *
     * @return the limit
     */
    public int getLimit() {
        return limit;
    }


    /**
     * Sets a new limit.
     *
     * @param limit  The limit to set.
     */
    public void setLimit(int limit) {
        int oldValue = getLimit();
        this.limit = limit;
        firePropertyChange(PROPERTYNAME_LIMIT, oldValue, limit);
    }


    /**
     * Returns whether this filter takes the limit into account.
     *
     * @return true if the limit is used to filter, false otherwise
     */
    public boolean isLimitEnabled() {
        return limitEnabled;
    }


    /**
     * Enables or disables the limit filter.
     *
     * @param limitEnabled  true to enable the limit filter
     */
    public void setLimitEnabled(boolean limitEnabled) {
        boolean oldValue = isLimitEnabled();
        this.limitEnabled = limitEnabled;
        firePropertyChange(PROPERTYNAME_LIMIT_ENABLED, oldValue, limitEnabled);
    }


    /**
     * Returns this filter's price limit.
     *
     * @return the price limit used to filter elements
     */
    public int getPrice() {
        return price;
    }


    /**
     * Sets a new price limit.
     *
     * @param price The price limit to set.
     */
    public void setPrice(int price) {
        int oldValue = getPrice();
        this.price = price;
        firePropertyChange(PROPERTYNAME_PRICE, oldValue, price);
    }


    /**
     * Returns whether the price filter is enabled.
     *
     * @return true if prices are used to filter
     */
    public boolean isPriceEnabled() {
        return priceEnabled;
    }


    /**
     * Enables or disables the price limit filter.
     *
     * @param priceEnabled  true to filter prices
     */
    public void setPriceEnabled(boolean priceEnabled) {
        boolean oldValue = isPriceEnabled();
        this.priceEnabled = priceEnabled;
        firePropertyChange(PROPERTYNAME_PRICE_ENABLED, oldValue, priceEnabled);
    }


    /**
     * Returns this filter's year limit.
     *
     * @return this filter's year limit
     */
    public int getYear() {
        return year;
    }


    /**
     * Sets a new year limit.
     *
     * @param year The year limit to set.
     */
    public void setYear(int year) {
        int oldValue = getYear();
        this.year = year;
        firePropertyChange(PROPERTYNAME_YEAR, oldValue, year);
    }


    /**
     * Returns whether the year filter is enabled.
     *
     * @return true if elements are filted by the year
     */
    public boolean isYearEnabled() {
        return yearEnabled;
    }


    /**
     * Enables or disables the year filter.
     *
     * @param yearEnabled true to enable the year filter
     */
    public void setYearEnabled(boolean yearEnabled) {
        boolean oldValue = isYearEnabled();
        this.yearEnabled = yearEnabled;
        firePropertyChange(PROPERTYNAME_YEAR_ENABLED, oldValue, yearEnabled);
    }


    // Storing and Restoring ************************************************

    private static final String KEY_LIMIT_ENABLED  = "filter.limitEnabled";
    private static final String KEY_PRICE_ENABLED  = "filter.priceEnabled";
    private static final String KEY_YEAR_ENABLED   = "filter.yearEnabled";
    private static final String KEY_LIMIT          = "filter.limit";
    private static final String KEY_PRICE          = "filter.price";
    private static final String KEY_YEAR           = "filter.year";

    /**
     * Restores the persistent data from the specified <code>Preferences</code>.
     */
    public void restoreFrom(Preferences prefs) {
        limitEnabled  = prefs.getBoolean(KEY_LIMIT_ENABLED,  DEFAULT_LIMIT_ENABLED);
        priceEnabled  = prefs.getBoolean(KEY_PRICE_ENABLED,  DEFAULT_PRICE_ENABLED);
        yearEnabled   = prefs.getBoolean(KEY_YEAR_ENABLED,   DEFAULT_YEAR_ENABLED);
        limit         = prefs.getInt    (KEY_LIMIT,          DEFAULT_LIMIT);
        price         = prefs.getInt    (KEY_PRICE,          DEFAULT_PRICE);
        year          = prefs.getInt    (KEY_YEAR,           DEFAULT_YEAR);
    }

    /**
     * Stores the persistent data in the specified <code>Preferences</code>.
     */
    public void storeIn(Preferences prefs) {
        prefs.putBoolean(KEY_LIMIT_ENABLED,  limitEnabled);
        prefs.putBoolean(KEY_PRICE_ENABLED,  priceEnabled);
        prefs.putBoolean(KEY_YEAR_ENABLED,   yearEnabled);
        prefs.putInt    (KEY_LIMIT,          limit);
        prefs.putInt    (KEY_PRICE,          price);
        prefs.putInt    (KEY_YEAR,           year);
    }


}