/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.model;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.util.EventObject;
import java.util.List;
import java.util.prefs.Preferences;

import javax.swing.JOptionPane;

import com.jgoodies.binding.beans.BeanAdapter;
import com.jgoodies.binding.beans.Model;
import com.jgoodies.binding.list.SelectionInList;
import com.jgoodies.binding.value.ValueHolder;
import com.jgoodies.binding.value.ValueModel;
import com.jgoodies.bookfinder.domain.BookFilter;
import com.jgoodies.bookfinder.model.UISettingsModel.OverviewMode;
import com.jgoodies.bookfinder.request.AmazonAccessor;
import com.jgoodies.bookfinder.request.WebService;
import com.jgoodies.bookfinder.request.query.SimulatedKeywordQuery;
import com.jgoodies.bookfinder.request.query.XMLQuery;
import com.jgoodies.bookfinder.view.dialog.FilterDialog;
import com.jgoodies.bookfinder.view.dialog.PreferencesDialog;
import com.jgoodies.bookfinder.view.main.MainFrame;
import com.jgoodies.bookfinder.view.main.Tour;
import com.jgoodies.uif.action.ToggleAction;
import com.jgoodies.uif.application.*;
import com.jgoodies.uif.util.WindowUtils;
import com.jgoodies.uifextras.convenience.DefaultAboutDialog;
import com.jgoodies.uifextras.help.HelpBroker;

/**
 * References all application-level data or sub models
 * and provides convenience accessors.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.22 $
 *
 * @see SearchModel
 * @see SearchResultModel
 * @see DetailsModel
 * @see com.jgoodies.bookfinder.request.WebService
 * @see com.jgoodies.bookfinder.domain.BookFilter
 */
public final class MainModel extends Model implements ExitListener {


    // Names of the Bound Bean Properties *************************************

    public static final String PROPERTYNAME_ACTIVITY = "activity";


    // Action Resource Keys ***************************************************

    public static final String ACTION_OPEN_ABOUT_DIALOG   = "openAboutDialog";
    public static final String ACTION_OPEN_FILTER_DIALOG  = "openFilterDialog";
    public static final String ACTION_OPEN_PREFERENCES    = "openPreferences";
    public static final String ACTION_EXIT                = "exit";
    public static final String ACTION_OPEN_HELP_CONTENTS  = "openHelpContents";
    public static final String ACTION_START_TOUR          = "startTour";


    // Other Static Fields ****************************************************

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(MainModel.class);


    // Fields *****************************************************************

    /**
     * Refers to the sub model that provides the search models and actions.
     *
     * @see #getSearchModel()
     */
    private final SearchModel searchModel;

    /**
     * Refers to the sub model that provides the search result models.
     *
     * @see #getSearchResultModel()
     */
    private final SearchResultModel searchResultModel;

    /**
     * Refers to the sub model that provides the search models and actions.
     *
     * @see #getDetailsModel()
     */
    private final DetailsModel detailsModel;

    /**
     * Refers to the presentation settings that provide bound Bean properties
     * for a bunch of presentation related settings.
     *
     * @see #getUISettingsModel()
     */
    private final UISettingsModel presentationSettings;

    /**
     * Refers to the object that describes the current web service.
     *
     * @see #getWebService()
     */
    private final WebService webService;

    /**
     * Refers to the object that access the Amazon.com web service.
     *
     * @see #getAmazonAccessor()
     */
    private final AmazonAccessor amazonAccessor;

    /**
     * Holds the current book filter.
     *
     * @see #getFilter()
     */
    private final BookFilter filter;

    /**
     * Holds the filtered list and the (single) selection.
     */
    private final SelectionInList selectionInFilteredBooks;


    /**
     * Holds a text that describes the current activity.
     *
     * @see #setActivity(String)
     */
    private String activity;


    // Toggle Actions *********************************************************

    private ToggleAction animateHeaderAction;
    private ToggleAction summaryModeAction;
    private ToggleAction listModeAction;
    private ToggleAction tableModeAction;
    private ToggleAction showDetailsAction;


    // Instance Creation ****************************************************

    /**
     * Constructs a <code>MainModel</code>.
     */
    public MainModel() {
        searchModel         = new SearchModel(this);
        searchResultModel   = new SearchResultModel(this);
        detailsModel        = new DetailsModel(this);
        presentationSettings = new UISettingsModel();

        webService = new WebService();
        amazonAccessor = new AmazonAccessor(this);
        filter     = new BookFilter();

        selectionInFilteredBooks = new SelectionInList(
                new ValueHolder(searchResultModel.getFilteredBooks(), true),
                detailsModel.getBookHolder());

        searchResultModel.initEventHandling();

        initializeToggleActions();
        restoreState();

        ApplicationContext.getInstance().getApplication().addExitListener(this);
    }


    // Accessors ************************************************************

    /**
     * Returns the submodel that provides the search models and actions.
     *
     * @return the submodel that provides the search models and actions.
     */
    public SearchModel getSearchModel() {
        return searchModel;
    }


    /**
     * Returns the submodel that provides the search result models.
     *
     * @return the submodel that provides the search result models.
     */
    public SearchResultModel getSearchResultModel() {
        return searchResultModel;
    }


    /**
     * Returns the sub model that provides the details models and actions.
     *
     * @return the sub model that provides the details models and actions.
     */
    public DetailsModel getDetailsModel() {
        return detailsModel;
    }


    /**
     * Returns the presentation settings that provides bound Bean
     * properties for view related state.
     *
     * @return the presentation settings.
     */
    public UISettingsModel getUISettingsModel() {
        return presentationSettings;
    }


    /**
     * Returns the description of the web service.
     *
     * @return the description of the web service.
     */
    public WebService getWebService() {
        return webService;
    }


    public AmazonAccessor getAmazonAccessor() {
        return amazonAccessor;
    }


    /**
     * Returns the current filter that is used to filter elements
     * from the raw list to build up the filtered details list.
     *
     * @return the current filter
     */
    public BookFilter getFilter() {
        return filter;
    }


    /**
     * Returns a ListModel of filtered books with a selection.
     *
     * @return a ListModel of filtered books with a selection.
     */
    public SelectionInList getSelectionInFilteredBook() {
        return selectionInFilteredBooks;
    }


    // Application ExitListener Implementation **************************

    /**
     * Stores this model's state if the application is closing.
     */
    public void applicationExiting() {
        storeState();
    }


    public boolean applicationExitAllowed(EventObject event) {
        return true;
    }


    public ToggleAction getAnimateHeaderAction() {
        return animateHeaderAction;
    }

    public ToggleAction getSummaryModeAction() {
        return summaryModeAction;
    }

    public ToggleAction getListModeAction() {
        return listModeAction;
    }

    public ToggleAction getTableModeAction() {
        return tableModeAction;
    }

    public ToggleAction getShowDetailsAction() {
        return showDetailsAction;
    }


    // Action Operations ******************************************************

    /**
     * Opens the about dialog.
     */
    @Action
    public void openAboutDialog(ActionEvent e) {
        new DefaultAboutDialog((Frame) WindowUtils.getParent(e)).open();
    }

    /**
     * Opens the filter dialog.
     */
    @Action
    public void openFilterDialog(ActionEvent e) {
        new FilterDialog((Frame) WindowUtils.getParent(e), getFilter()).open();
    }

    /**
     * Opens the preferences dialog.
     */
    @Action
    public void openPreferences(ActionEvent e) {
        new PreferencesDialog((Frame) WindowUtils.getParent(e), this).open();
    }


    @Action
    public void exit(ActionEvent e) {
        ApplicationContext.getInstance().getApplication().exit(e);
    }


    @Action
    public void openHelpContents(ActionEvent e) {
        HelpBroker.openDefault();
    }

    /**
     * Starts the integrated tour.
     */
    @Action
    public void startTour(ActionEvent e) {
        new Tour(this, (MainFrame) WindowUtils.getParent(e)).start();
    }


    // Operations *************************************************************

    /**
     * Clears the books list and resets the message.
     */
    public void reset() {
        getSearchResultModel().reset();
    }


    /**
     * Enables or disables the display mode actions.
     */
    public void setDisplayModeActionsEnabled(boolean b) {
        summaryModeAction.setEnabled(b);
        listModeAction.setEnabled(b);
        tableModeAction.setEnabled(b);
    }

    /**
     * Sets author and keywords to blank strings.
     */
    public void setSearchContentsBlank() {
        getSearchModel().setAuthor("");
        getSearchModel().setKeywords("");
    }

    /**
     * Sets the given message in the search result model.
     *
     * @param message  the message to be set
     */
    public void setMessage(String message) {
        getSearchResultModel().setMessage(message);
    }

    /**
     * Shows a message.
     */
    public void showActivity(String prefix, String suffix) {
        setActivity(prefix + "@" + suffix);
    }

    /**
     * Makes the details panel visible.
     */
    public void showDetails() {
        getUISettingsModel().setShowingDetails(true);
    }


    /**
     * Adds books to the existing list of all books.
     *
     * @param moreBooks  the list of books to add
     */
    public void addBooks(List moreBooks) {
        getSearchResultModel().setBooks(moreBooks);
    }


    // Property Accessors *****************************************************

    private String getActivity() {
        return activity;
    }

    public void setActivity(String newActivity) {
        String oldActivity = getActivity();
        activity = newActivity;
        firePropertyChange(PROPERTYNAME_ACTIVITY, oldActivity, newActivity);
    }


    // Storing and Restoring ************************************************

    /**
     * Restores the persistent data from the specified <code>Preferences</code>.
     */
    private void restoreState() {
        Preferences prefs = ApplicationContext.getInstance().getUserPreferences();
        getUISettingsModel().restoreFrom(prefs);
        getWebService().restoreFrom(prefs);
        getFilter().restoreFrom(prefs);
    }

    /**
     * Stores the persistent data in the specified <code>Preferences</code>.
     */
    private void storeState() {
        Preferences prefs = ApplicationContext.getInstance().getUserPreferences();
        getUISettingsModel().storeIn(prefs);
        getWebService().storeIn(prefs);
        getFilter().storeIn(prefs);
    }


    // Helper Code ************************************************************

    /**
     * Ensures that we have a token and answers whether we have one or not.
     * Does nothing, if we already have a token. Otherwise, it pops up
     * a message dialog and asks the user to enter a token.
     */
    private boolean ensureHasToken() {
        if (getWebService().hasToken())
            return true;

        showActivity("Token", "Required\u2026");
        String result =
            JOptionPane.showInputDialog(
                null,
                "In order to use this tool, you will first need"
                    + "\nto obtain a developer token from Amazon.com;"
                    + "\nSee https://associates.amazon.com/exec/panama/"
                    + "\nassociates/join/developer/application.html/"
                    + "\n\nIf you just press OK, the tool will simulate the access."
                    + "\n\nPlease enter the token.",
                "Enter Web Service Token",
                JOptionPane.INFORMATION_MESSAGE);
        if (result == null || result.trim().length() == 0)
            return false;

        getWebService().setAccessKey(result);
        return true;
    }

    /**
     * Finds books for the given query utilizing the AmazonAccessor,
     * which in turn answers a cached result or connects Amazon.com.
     */
    void startSearch(XMLQuery query) {
        boolean isSimulation = !ensureHasToken();
        XMLQuery usedQuery =
            isSimulation ? SimulatedKeywordQuery.getSampleQuery() : query;
        reset();
        amazonAccessor.find(usedQuery);
    }



    // Action Implementation **************************************************


    /**
     * Initializes the toggle actions. Uses an adapter that vends ValueModels
     * that convert bean properties of the UISettingsModel to ValueModel.
     */
    private void initializeToggleActions() {
        BeanAdapter presentationAdapter = new BeanAdapter(
                getUISettingsModel(),
                true);
        ValueModel overviewModeHolder = presentationAdapter.getValueModel(
                UISettingsModel.PROPERTYNAME_OVERVIEW_MODE);


        animateHeaderAction = ToggleAction.createCheck(
                "animateHeader",
                RESOURCES,
                presentationAdapter.getValueModel(
                    UISettingsModel.PROPERTYNAME_ANIMATING_HEADER));

        summaryModeAction = ToggleAction.createRadio(
                "overviewMode.summary",
                RESOURCES,
                overviewModeHolder,
                OverviewMode.SUMMARY);

        listModeAction = ToggleAction.createRadio(
                "overviewMode.list",
                RESOURCES,
                overviewModeHolder,
                OverviewMode.LIST);

        tableModeAction = ToggleAction.createRadio(
                "overviewMode.table",
                RESOURCES,
                overviewModeHolder,
                OverviewMode.TABLE);
        setDisplayModeActionsEnabled(false);

        showDetailsAction = ToggleAction.createCheck(
                "showDetails",
                RESOURCES,
                presentationAdapter.getValueModel(
                    UISettingsModel.PROPERTYNAME_SHOWING_DETAILS));
    }


}
