/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.model;

import com.jgoodies.binding.beans.Model;
import com.jgoodies.bookfinder.request.query.AuthorQuery;
import com.jgoodies.bookfinder.request.query.KeywordQuery;
import com.jgoodies.uif.application.Action;

/**
 * Provides bound Bean properties and search actions: keywords, an author,
 * a keywords-search action, and an author-search action.<p>
 *
 * As an alternative to the bound Bean properties, we could implement
 * the author and keywords model using two <code>ValueHolder</code>s.
 * I favor bound Bean properties, because it allows to perform
 * additional operations in the getter and setter. Here we trim
 * the strings before we set them.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.9 $
 */
public final class SearchModel extends Model {


    // Names of the bound properties ******************************************

    public static final String PROPERTYNAME_AUTHOR   = "author";
    public static final String PROPERTYNAME_KEYWORDS = "keywords";


    // Action Resource Keys ***************************************************

    public static final String ACTION_FIND_BY_KEYWORDS = "findByKeywords";
    public static final String ACTION_FIND_BY_AUTHOR   = "findByAuthor";


    // Fields *****************************************************************

    private final MainModel mainModel;

    /**
     * Holds the current search author.
     */
    private String author;

    /**
     * Holds the current search keywords.
     */
    private String keywords;


    // Instance Creation ******************************************************

    /**
     * Constructs a <code>SearchModel</code> initialized with
     * empty author and keywords.
     */
    SearchModel(MainModel mainModel) {
        this.mainModel = mainModel;
        author   = "";
        keywords = "";
    }


    // Accessors **************************************************************

    /**
     * Returns the author used for the author search.
     *
     * @return the author used for the author search.
     */
    public String getAuthor() {
        return author;
    }

    /**
     * Sets a new author to be searched for.
     * The author string will be trimmed before it is set as new value.
     *
     * @param newUntrimmedAuthor   the author to be set, will be trimmed
     */
    public void setAuthor(String newUntrimmedAuthor) {
        String newAuthor = newUntrimmedAuthor.trim();
        String oldAuthor = getAuthor();
        author = newAuthor;
        firePropertyChange(PROPERTYNAME_AUTHOR, oldAuthor, newAuthor);
    }


    /**
     * Returns the keywords used for the keywords search.
     *
     * @return the keywords used for the keywords search.
     */
    public String getKeywords() {
        return keywords;
    }

    /**
     * Sets a new keywords to be searched for.
     * The keywords string will be trimmed before it is set as new value.
     *
     * @param newUntrimmedKeywords   the author to be set, will be trimmed
     */
    public void setKeywords(String newUntrimmedKeywords) {
        String newKeywords = newUntrimmedKeywords.trim();
        String oldKeywords = getKeywords();
        keywords = newKeywords;
        firePropertyChange(PROPERTYNAME_KEYWORDS, oldKeywords, newKeywords);
    }


    // Actions ****************************************************************

    /**
     * Searches and retrieves a SearchResult for the keywords.
     */
    @Action
    public void findByKeywords() {
        if (keywords.length() == 0)
            return;

        mainModel.startSearch(new KeywordQuery(
                mainModel.getWebService(),
                keywords.toUpperCase()));
    }


    /**
     * Searches and retrieves a SearchResult for the author.
     */
    @Action
    public void findByAuthor() {
        if (author.length() == 0)
            return;

        mainModel.startSearch(new AuthorQuery(
                mainModel.getWebService(),
                author.toUpperCase()));
    }


}
