/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.model;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.ListModel;

import com.jgoodies.binding.beans.Model;
import com.jgoodies.binding.list.ArrayListModel;
import com.jgoodies.binding.list.ObservableList;

/**
 * References all application-level data or sub models
 * and provides convenience accessors.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.5 $
 */
public final class SearchResultModel extends Model {

    // Names of the bound properties ****************************************

    public static final String PROPERTYNAME_MESSAGE        = "message";
    public static final String PROPERTYNAME_ALL_BOOKS      = "allBooks";
    public static final String PROPERTYNAME_FILTERED_BOOKS = "filterBooks";


    // Constants ************************************************************

    private static final String NO_MESSAGE = "";


    // Fields ***************************************************************

    /**
     * Refers to the parent model that holds the filter.
     */
    private final MainModel mainModel;

    /**
     * Holds the a user message - if any.
     */
    private String message = NO_MESSAGE;

    /**
     * Holds the List of all books found in the last search.
     */
    private List books = Collections.EMPTY_LIST;

    /**
     * Holds an observable List of all books that were accepted by
     * the filter - if any.
     */
    private final ObservableList filteredBooks;



    // Instance Creation ****************************************************

    /**
     * Constructs a <code>SearchResultModel</code> with the given parent.
     *
     * @param mainModel   this model's parent model
     */
    SearchResultModel(MainModel mainModel) {
        this.mainModel = mainModel;
        filteredBooks = new ArrayListModel();
    }

    void initEventHandling() {
        PropertyChangeListener handler = new UpdateFilteredBooksHandler();
        addPropertyChangeListener(PROPERTYNAME_ALL_BOOKS, handler);
        mainModel.getFilter().addPropertyChangeListener(handler);
    }


    // Readers and Writers for Bound Properties *******************************

    /**
     * Returns the list of all books. A subset of these elements
     * are available as filtered books.
     *
     * @return the list of all books found during the last search
     */
    private List getBooks() {
        return books;
    }

    /**
     * Sets the unfiltered list of books.
     *
     * @param newBooks  the list of books to be set
     */
    void setBooks(List newBooks) {
        List oldDetails = getBooks();
        books = newBooks;
        firePropertyChange(PROPERTYNAME_ALL_BOOKS, oldDetails, newBooks);
    }

    /**
     * Returns the number of all books.
     */
    public int getBooksCount() {
        return getBooks().size();
    }

    /**
     * Checks and answers if we have books.
     */
    public boolean hasBooks() {
        return getBooksCount() > 0;
    }

    /**
     * Returns the number of books accepted by the filter.
     *
     * @return the number of books accepted by the filter.
     */
    public int getFilteredBooksCount() {
        return filteredBooks.getSize();
    }

    /**
     * Returns the ListModel of books accepted by the filter.<p>
     *
     * TODO: Return an unmodifiable ListModel
     *
     * @return the ListModel of books accepted by the filter.
     */
    public ListModel getFilteredBooks() {
        return filteredBooks;
    }


    /**
     * Returns the current user message text. Such messages are
     * available if we cannot connect
     *
     * @return the text of the current user message.
     */
    public String getMessage() {
        return message;
    }

    /**
     * Sets the given text as new stored message.
     */
    void setMessage(String newText) {
        String oldText = getMessage();
        message = newText;
        firePropertyChange(PROPERTYNAME_MESSAGE, oldText, newText);
    }

    /**
     * Checks and answers if we have a message.
     */
    public boolean hasMessage() {
        return !message.equals(NO_MESSAGE);
    }


    // Operations *************************************************************

    /**
     * Clears the books list and resets the message.
     */
    void reset() {
        setMessage(NO_MESSAGE);
        setBooks(Collections.EMPTY_LIST);
    }


    // Event Handling *********************************************************

    private void updateFilteredBooks(List newList) {
        List oldList = new ArrayList(filteredBooks);
        filteredBooks.clear();
        filteredBooks.addAll(newList);
        firePropertyChange(PROPERTYNAME_FILTERED_BOOKS, oldList, newList);
    }


    private class UpdateFilteredBooksHandler implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent evt) {
            updateFilteredBooks(mainModel.getFilter().filter(getBooks()));
        }
    }


}