/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.model;

import java.util.prefs.Preferences;

import com.jgoodies.binding.beans.Model;
import com.jgoodies.uif.util.SystemUtils;

/**
 * Provides bound Bean properties for the presentation related settings.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.7 $
 */
public final class UISettingsModel extends Model {

    // Names of the bound properties ******************************************

    public static final String PROPERTYNAME_ANIMATING_HEADER = "animatingHeader";
    public static final String PROPERTYNAME_OVERVIEW_MODE    = "overviewMode";
    public static final String PROPERTYNAME_SHOWING_DETAILS  = "showingDetails";


    // Constants ************************************************************

    private static final boolean DEFAULT_ANIMATING_HEADER =
        SystemUtils.IS_OS_WINDOWS;

    private static final boolean DEFAULT_SHOWING_DETAILS = true;


    // Enumeration ************************************************************

    public enum OverviewMode {
        SUMMARY, LIST, TABLE
    }

    // GUI State Value Holders ************************************************

    private boolean animatingHeader = DEFAULT_ANIMATING_HEADER;
    private OverviewMode overviewMode   = OverviewMode.TABLE;
    private boolean showingDetails  = DEFAULT_SHOWING_DETAILS;


    // Instance Creation ****************************************************

    /**
     * Constructs UI related settings.
     */
    UISettingsModel() {
        // Override the default constructor to reduce the visibility.
    }


    // Accessors ************************************************************

    /**
     * Returns whether we animate changes of the header label.
     *
     * @return true if the label is animated, false if not
     */
    public boolean isAnimatingHeader() {
        return animatingHeader;
    }

    /**
     * Enables or disables the animation in the header area.
     *
     * @param newValue  true to animate the header, false for plain switching
     */
    public void setAnimatingHeader(boolean newValue) {
        boolean oldValue = isAnimatingHeader();
        animatingHeader = newValue;
        firePropertyChange(PROPERTYNAME_ANIMATING_HEADER, oldValue, newValue);
    }


    /**
     * Returns the current overview mode that indicates how
     * the overview is presentated: summary, list or table.
     *
     * @return the current overview mode
     */
    public OverviewMode getOverviewMode() {
        return overviewMode;
    }

    public void setOverviewMode(OverviewMode newMode) {
        OverviewMode oldMode = getOverviewMode();
        overviewMode = newMode;
        firePropertyChange(PROPERTYNAME_OVERVIEW_MODE, oldMode, newMode);
    }


    /**
     * Returns if we show or hide the details view.
     *
     * @return true if details are shown, false if hidden
     */
    public boolean isShowingDetails() {
        return showingDetails;
    }

    /**
     * Shows or hides the details view.
     *
     * @param newValue  true to show details, false to hide them
     */
    public void setShowingDetails(boolean newValue) {
        boolean oldValue = isShowingDetails();
        showingDetails = newValue;
        firePropertyChange(PROPERTYNAME_SHOWING_DETAILS, oldValue, newValue);
    }


    // Property Persistency ***************************************************

    private static final String KEY_ANIMATE_HEADER  = "state.animateHeader";
    private static final String KEY_OVERVIEW_MODE   = "state.overviewMode";
    private static final String KEY_SHOWING_DETAILS = "state.showDetails";


    /**
     * Restores the persistent properties from the specified Preferences.
     *
     * @param prefs   the Preferences object that holds the property values
     */
    void restoreFrom(Preferences prefs) {
        setAnimatingHeader(
            prefs.getBoolean(KEY_ANIMATE_HEADER, DEFAULT_ANIMATING_HEADER));

        setOverviewMode(OverviewMode.valueOf(
                prefs.get(KEY_OVERVIEW_MODE,
                          OverviewMode.SUMMARY.name())));

        setShowingDetails(
            prefs.getBoolean(KEY_SHOWING_DETAILS, DEFAULT_SHOWING_DETAILS));
    }


    /**
     * Stores the persistent properties in the specified Preferences.
     *
     * @param prefs   the Preferences object that holds the property values
     */
    void storeIn(Preferences prefs) {
        prefs.putBoolean(KEY_ANIMATE_HEADER, isAnimatingHeader());
        prefs.put(KEY_OVERVIEW_MODE, getOverviewMode().name());
        prefs.putBoolean(KEY_SHOWING_DETAILS, isShowingDetails());
    }


}