/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.request;

import java.io.IOException;
import java.net.ConnectException;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import org.jdom.JDOMException;

import com.jgoodies.bookfinder.domain.SearchResult;
import com.jgoodies.bookfinder.model.MainModel;
import com.jgoodies.bookfinder.request.query.AuthorQuery;
import com.jgoodies.bookfinder.request.query.XMLQuery;
import com.jgoodies.uif.util.SafeWorker;

/**
 * An accessor to the Amazon.com web services that caches
 * results and performs multiple asynchron queries.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.8 $
 */
public final class AmazonAccessor {

    private static final Logger LOGGER = Logger.getLogger("AmazonAccessor");

    /**
     * Provides all high-level models.
     * Used to add books, enable actions, and show messages.
     *
     * @see #getMainModel()
     */
    private final MainModel mainModel;

    /**
     * A cache that maps keywords to keyword search results.
     */
    private final Map keywordResultCache;

    /**
     * A cache that maps author names to author search results.
     */
    private final Map authorResultCache;

    /** A cache that maps URLs to small item images.					*/
    //private final Map smallImageCache;

    /** A cache that maps URLs to medium item images.					*/
    //private final Map mediumImageCache;


    /**
     * Constructs the <code>AmazonAccessor</code>.
     *
     * @param mainModel  the model that holds all relevant models
     */
    public AmazonAccessor(MainModel mainModel) {
        this.mainModel = mainModel;
        keywordResultCache = new HashMap();
        authorResultCache  = new HashMap();
        //smallImageCache    = new HashMap();
        //mediumImageCache   = new HashMap();
    }


    // The Public API *******************************************************

    /**
     * Fetches the result for the specified query and sets the result and/or
     * error messages via the controller. Firstly, checks if there's
     * a valid result entry in the cache associated with the query.
     */
    public synchronized void find(XMLQuery query) {
        List cachedResult = lookupCachedResult(query);
        if (cachedResult != null)
            setResult(cachedResult, query);
        else
            fetch(query);
    }

    /**
     * Returns the small image for the specified url.
     */
//    private Image smallImage(URL url) {
//        return null;
//    }


    /**
     * Returns the medium image for the specified url.
     */
//    private Image mediumImage(URL url) {
//        return null;
//    }


    // Core Accessor Loop ***************************************************

    /**
     * Finds books for the given query in a background thread.
     */
    private void fetch(final XMLQuery query) {
        getMainModel().setDisplayModeActionsEnabled(false);
        showMessage("Searching", query.getSearchData() + "\u2026");

        new SearchWorker(query).execute();
    }


    /**
     * Fetches the product info for the given query and sets errors,
     * if the result had an error message, or if an exceptions has been thrown.
     */
    private QueryResult execute(XMLQuery query) {
        long t0 = System.currentTimeMillis();
        LOGGER.info("Executing query " + query);
        try {
            SearchResult info = query.executeQuery();
            if (info == null) {
                LOGGER.warning("Error in query " + query);
                return new QueryResult(
                    null,
                    new IllegalStateException("Program error"));
            }
            return new QueryResult(info, null);
        } catch (JDOMException e) {
            e.printStackTrace();
            return new QueryResult(null, e);
        } catch (IOException e) {
            return new QueryResult(null, e);
        } finally {
            long t1 = System.currentTimeMillis();
            LOGGER.info("Finished query after " + (t1 - t0) + "ms.");
        }
    }


    // Setting The Result ***************************************************

    /**
     * Sets a finished result via the main model.
     */
    private void setResult(List items, XMLQuery query) {
        showMessage(query.getSearchType(), query.getSearchData());
        if (items.isEmpty()) {
            getMainModel().setMessage(
                "Could not find a result for your query.");
        }
        getMainModel().addBooks(items);
        getMainModel().setDisplayModeActionsEnabled(!items.isEmpty());
    }


    /**
     * Sets a failure via the main model.
     */
    private void setFailure(Throwable throwable) {
        showMessage("search", "failed");
        String userMessage = userMessage(throwable);
        if (userMessage == null) {
            LOGGER.info(throwable.toString());
            getMainModel().setMessage(throwable.toString());
        } else {
            getMainModel().setMessage(userMessage);
        }
        getMainModel().setDisplayModeActionsEnabled(false);
    }

    /**
     * Sets a message via the main model.
     */
    private void setMessage(String message) {
        showMessage("search", "problem");
        getMainModel().setMessage(message);
        getMainModel().setDisplayModeActionsEnabled(false);
    }


    // Helper Code **********************************************************************

    /**
     * Looks for an entry in the cache associated with the given query.
     * Returns the cache entry if it's valid, or null if invalid or absent.
     */
    private List lookupCachedResult(XMLQuery query) {
        Map cache = cache(query);
        String key = query.getSearchData();
        QueryCacheEntry entry = (QueryCacheEntry) cache.get(key);

        if (entry == null)
            return null;

        if (entry.isValid())
            return entry.details();

        cache.remove(key);
        return null;
    }


    /**
     * Looks for an entry in the cache associated with the given query.
     * Returns the cache entry if it's valid, or null if invalid or absent.
     */
    private void updateCache(XMLQuery query, List details) {
        Map cache = cache(query);
        String key = query.getSearchData();
        cache.put(key, new QueryCacheEntry(details));
    }


    /**
     * Returns the cache related to the given query.
     */
    private Map cache(XMLQuery query) {
        return query instanceof AuthorQuery
            ? authorResultCache
            : keywordResultCache;
    }


    /**
     * Shows a message in the header panel.
     */
    private void showMessage(String prefix, String suffix) {
        getMainModel().showActivity(prefix, suffix);
    }


    /**
     * Tries to find and answer a user readable message for the given Throwable.
     */
    private String userMessage(Throwable t) {
        if (t instanceof UnknownHostException) {
            return "Could not contact Amazon.com."
                + "\n\nMake sure you have established an Internet connection.";
        }
        if (t instanceof ConnectException) {
            return "Could not connect the Amazon.com service."
                + "\n\nMake sure you have established an Internet connection.";
        }
        if (t instanceof SocketException) {
            return "The Amazon.com network is unreachable."
                + "\n\nMake sure you have established an Internet connection."
                + "\nAlso, verify the Web service data in the preferences.";
        }
        return null;
    }


    // Accessing Collaborators **********************************************

    private MainModel getMainModel() {
        return mainModel;
    }


    // Helper Classes *******************************************************

    /**
     * Describes the result of a query:
     * result details, result message, or failure.
     */
    private static final class QueryResult {

        private final SearchResult searchResult;
        private final Throwable   throwable;

        private QueryResult(SearchResult searchResult, Throwable throwable) {
            this.searchResult = searchResult;
            this.throwable   = throwable;
        }

        private boolean hasFailure() {
            return throwable != null;
        }

        private boolean isEmpty() {
            return books().size() == 0;
        }

        private boolean hasMessage() {
            return !isEmpty() && (message() != null);
        }

        private List books() {
            return searchResult.getBooks();
        }

        private String message() {
            return searchResult.getErrorMsg();
        }

        private Throwable throwable() {
            return throwable;
        }

    }


    private final class SearchWorker extends SafeWorker<QueryResult, Object> {

        private final XMLQuery query;


        SearchWorker(XMLQuery query) {
            this.query = query;
        }

        @Override
        protected QueryResult doInBackground() {
            return AmazonAccessor.this.execute(query);
        }

        @Override
        protected void succeeded(QueryResult result) {
            if (result.hasFailure()) {
                setFailure(result.throwable());
            } else if (!result.hasMessage() || result.isEmpty()) {
                List books = result.books();
                updateCache(query, books);
                setResult(books, query);
            } else {
                setMessage(result.message());
            }
        }

    }


    /**
     * Describes an entry in the query caches;
     * becomes invalid an hour after construction.
     */
    private static final class QueryCacheEntry {

        private static final long ONE_HOUR = 60 * 60 * 1000; // ms

        private final List details;
        private final long constructionTime;

        private QueryCacheEntry(List details) {
            this.details = details;
            this.constructionTime = System.currentTimeMillis();
        }

        private boolean isValid() {
            long now = System.currentTimeMillis();
            return now - constructionTime < ONE_HOUR;
        }

        private List details() {
            return details;
        }

    }


}