/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.request;

import java.util.prefs.Preferences;

import com.jgoodies.binding.beans.Model;

/**
 * Describes an Amazon web service and provides bound properties
 * for the host, path, tag and token.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.7 $
 */
public final class WebService extends Model {

    // Names for Bound Bean Properties **************************************

    public static final String PROPERTYNAME_HOST       = "host";
    public static final String PROPERTYNAME_PATH       = "path";
    public static final String PROPERTYNAME_TAG        = "tag";
    public static final String PROPERTYNAME_ACCESS_KEY = "accessKey";


    // Default Values *******************************************************

    private static final String DEFAULT_HOST       = "webservices.amazon.com";
    private static final String DEFAULT_PATH       = "/onca/xml";
    private static final String DEFAULT_TAG        = "AWSECommerceService";
    private static final String DEFAULT_ACCESS_KEY = "05D95Y5AFTMKJ9DJ8ZR2";


    // Fields ***************************************************************

    private String host;
    private String path;
    private String tag;
    private String accessKey;


    // Public Interface *****************************************************

    /**
     * Returns the web services' host.
     *
     * @return the web services' host
     */
    public String getHost() {
        return host;
    }

    /**
     * Returns the web services' path.
     *
     * @return the web services' path.
     */
    public String getPath() {
        return path;
    }

    /**
     * Returns the tag.
     *
     * @return the tag
     */
    public String getTag() {
        return tag;
    }

    /**
     * Returns the access key used to identify the access at amazon.
     *
     * @return the access key
     */
    public String getAccessKey() {
        return accessKey;
    }


    /**
     * Checks and answers if we have an access key.
     *
     * @return true indicates we have a key, false if none is available
     */
    public boolean hasToken() {
        return getAccessKey().trim().length() > 0;
    }


    /**
     * Sets a new host.
     *
     * @param newHost    the new host to set
     */
    public void setHost(String newHost) {
        String oldHost = getHost();
        host = newHost;
        firePropertyChange(PROPERTYNAME_HOST, oldHost, newHost);
    }


    /**
     * Sets a new path.
     *
     * @param newPath   the new path to set
     */
    public void setPath(String newPath) {
        String oldPath = getPath();
        path = newPath;
        firePropertyChange(PROPERTYNAME_PATH, oldPath, newPath);
    }


    /**
     * Sets a new tag.
     *
     * @param newTag     the new tag to set
     */
    public void setTag(String newTag) {
        String oldTag = getTag();
        tag = newTag;
        firePropertyChange(PROPERTYNAME_TAG, oldTag, newTag);
    }


    /**
     * Sets the given string as new access key.
     *
     * @param newAccessKey   the new access key to set
     */
    public void setAccessKey(String newAccessKey) {
        String oldAccessKey = getAccessKey();
        accessKey = newAccessKey;
        firePropertyChange(PROPERTYNAME_ACCESS_KEY, oldAccessKey, newAccessKey);
    }


    // Property Persistency ***************************************************

    private static final String KEY_HOST  = "webservice.newhost";
    private static final String KEY_PATH  = "webservice.newpath";
    private static final String KEY_TAG   = "webservice.tag";
    private static final String KEY_TOKEN = "webservice.token";

    /**
     * Restores the persistent properties from the specified Preferences.
     *
     * @param prefs   the Preferences object that holds the property values
     */
    public void restoreFrom(Preferences prefs) {
        setHost (prefs.get(KEY_HOST,  DEFAULT_HOST));
        setPath (prefs.get(KEY_PATH,  DEFAULT_PATH));
        setTag  (prefs.get(KEY_TAG,   DEFAULT_TAG));
        setAccessKey(prefs.get(KEY_TOKEN, DEFAULT_ACCESS_KEY));
    }

    /**
     * Stores the persistent properties in the specified Preferences.
     *
     * @param prefs   the Preferences object that holds the property values
     */
    public void storeIn(Preferences prefs) {
        prefs.put(KEY_HOST,  getHost());
        prefs.put(KEY_PATH,  getPath());
        prefs.put(KEY_TAG,   getTag());
        prefs.put(KEY_TOKEN, getAccessKey());
    }

}