/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.request.query;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.StringTokenizer;
import java.util.logging.Logger;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import com.jgoodies.bookfinder.domain.SearchResult;
import com.jgoodies.bookfinder.request.Unmarshaller;
import com.jgoodies.bookfinder.request.WebService;

/**
 * An abstract superclass for XML query to the Amazon.com web services.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.9 $
 */
abstract class AbstractXMLQuery implements XMLQuery {

    private final WebService webService;
    private final String searchData;
    private final int page;


    // Instance Creation ****************************************************

    /**
     * Constructs an abstract XML query with the given search type and data.
     */
    AbstractXMLQuery(WebService webService, String searchData) {
        this.webService = webService;
        this.searchData = searchData;
        page = 1;
    }


    // Public API ***********************************************************

    /**
     * Executes this XML query and returns the <code>SearchResult</code> object
     * generated by the query.
     */
    public SearchResult executeQuery() throws IOException, JDOMException {
        return executeQuery(page);
    }


    /**
     * Returns the query's search data, for example: 'gui', or 'arno schmidt'.
     */
    public String getSearchData() {
        return searchData;
    }


    // Abstract Behavior ****************************************************

    /**
     * Returns the query's search type, for example: 'author', 'key', or ISBN.
     */
    public abstract String getSearchType();


    /**
     * Returns this query's operations as used in the query URL,
     * for example: 'ItemSearch'.
     *
     * @return this query's search code
     */
    protected abstract String getOperation();


    // Default Behavior *****************************************************

    /**
     * Executes this XML query using the specified page number and
     * returns the <code>SearchResult</code> object generated by the query.
     */
    protected SearchResult executeQuery(int pageNo)
        throws IOException, JDOMException {
        URL url = getQueryURL(pageNo);
        if (null == url)
            return null;

        SAXBuilder builder = new SAXBuilder();
        Document document = builder.build(url.openStream());
        Element  root     = document.getRootElement();
        return Unmarshaller.searchResult(root);
    }


    /**
     * Computes and returns the URL used to execute this query for the
     * specified page no.
     *
     * @return the URL used to execute this query
     */
    protected URL getQueryURL(int pageNo) {
        String queryString = constructQueryString();
        try {
            return new URL(queryString);
        } catch (MalformedURLException e) {
            Logger.getLogger("AbstractXMLQuery").warning(
                "Malformed URL: " + queryString);
            return null;
        }
    }


    protected String getResponseGroup() {
        return "Medium"; // or "lite";
    }


    protected String getSearchIndex() {
        return "Books";
    }


    // Helper Code **********************************************************

    /**
     * Constructs and answers the query's string that is used to build the URL.
     */
    private String constructQueryString() {
        StringBuilder builder = new StringBuilder(200);
        builder.append("http://");
        builder.append(webService.getHost());
        builder.append(webService.getPath());
        builder.append("?Service=");
        builder.append(webService.getTag());
        builder.append("&AWSAccessKeyId=");
        builder.append(webService.getAccessKey());
        builder.append("&Operation=");
        builder.append(getOperation());
        builder.append("&SearchIndex=");
        builder.append(getSearchIndex());
        builder.append('&');
        builder.append(getSearchType());
        builder.append('=');
        builder.append(wrapBlanks(getSearchData()));
        builder.append("&ResponseGroup=");
        builder.append(getResponseGroup());
        return builder.toString();
    }


    /**
     * Returns a string where blanks are replaced by a code suitable for an URI.
     */
    private String wrapBlanks(String str) {
        StringBuilder builder = new StringBuilder();
        StringTokenizer tokenizer = new StringTokenizer(str, " ");
        builder.append(tokenizer.nextToken());
        while (tokenizer.hasMoreElements()) {
            builder.append("%20");
            builder.append(tokenizer.nextToken());
        }
        return builder.toString();
    }


    @Override
    public String toString() {
        return constructQueryString();
    }


}
