/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.view.dialog;

import java.awt.BorderLayout;
import java.awt.Frame;
import java.text.Format;
import java.text.NumberFormat;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JPanel;

import com.jgoodies.binding.PresentationModel;
import com.jgoodies.binding.adapter.BasicComponentFactory;
import com.jgoodies.bookfinder.domain.BookFilter;
import com.jgoodies.bookfinder.view.Resources;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif.AbstractDialog;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.util.Resizer;
import com.jgoodies.uifextras.panel.HeaderPanel;

/**
 * Provides the filter dialog.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.9 $
 *
 * @see com.jgoodies.bookfinder.domain.BookFilter
 * @see com.jgoodies.binding.value.BufferedValueModel
 * @see com.jgoodies.binding.adapter.BasicComponentFactory
 */
public final class FilterDialog extends AbstractDialog {

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(FilterDialog.class);

    /**
     * Provides ValueModel that adapt the filter's bound bean properties.
     */
    private final PresentationModel filterModel;

//  private JCheckBox  limitCheck;
//  private JComponent limitField;
    private JCheckBox  priceCheck;
    private JComponent priceField;
    private JCheckBox  yearCheck;
    private JComponent yearField;


    // Instance Creation ****************************************************

    /**
     * Constructs a FilterDialog for the given owner and filter.
     *
     * @param owner   the dialogs owner
     * @param filter  the filter to be edited
     */
    public FilterDialog(Frame owner, BookFilter filter) {
        super(owner);
        this.filterModel = new PresentationModel(filter, getTriggerChannel());
    }


    // Overriding Superclass Building ***************************************

    /**
     * Builds and returns the dialog's header panel.
     */
    @Override
    protected JComponent buildHeader() {
        return new HeaderPanel(
            "BookFilter Editor",
            "You can see and edit the tool's filter settings, which"
                + "\nwill be used to filter items in the overview panel.",
            RESOURCES.getIcon(Resources.FILTER_ICON_ID));
    }

    /**
     * Builds and returns the filter dialog's content pane.
     */
    @Override
    protected JComponent buildContent() {
        JPanel content = new JPanel(new BorderLayout());
        content.add(buildMainPanel(), BorderLayout.CENTER);
        content.add(buildButtonBarWithOKCancelApply(), BorderLayout.SOUTH);
        return content;
    }

    @Override
    protected void resizeHook(JComponent component) {
        Resizer.DEFAULT.resizeDialogContent(component);
    }


    // Initialization *********************************************************

    /**
     * Creates, binds and configures the UI components.
     */
    private void initComponents() {
//      limitCheck = createCheckBox(
//              BookFilter.PROPERTYNAME_LIMIT_ENABLED,
//              "Limit visible items to ");

        priceCheck = BasicComponentFactory.createCheckBox(
                filterModel.getBufferedModel(BookFilter.PROPERTYNAME_PRICE_ENABLED),
                "Where price is ");
        priceCheck.setMnemonic('p');

        yearCheck = BasicComponentFactory.createCheckBox(
                filterModel.getBufferedModel(BookFilter.PROPERTYNAME_YEAR_ENABLED),
                "Where publishing year is ");
        yearCheck.setMnemonic('y');

//      limitField = BasicComponentFactory.createFormattedTextField(
//              filterModel.getBufferedModel(BookFilter.PROPERTYNAME_LIMIT),
//              createIntegerFormat(3)); //,  0, 10000);

        priceField = BasicComponentFactory.createFormattedTextField(
                filterModel.getBufferedModel(BookFilter.PROPERTYNAME_PRICE),
                createIntegerFormat(1));  //,  0,  1000);

        yearField  = BasicComponentFactory.createFormattedTextField(
                filterModel.getBufferedModel(BookFilter.PROPERTYNAME_YEAR),
                createIntegerFormat(4));  //,  0,  2067);
    }


    // Building *************************************************************

    /**
     * Builds and returns the tabbed pane.
     */
    private JComponent buildMainPanel() {
        initComponents();

        FormLayout layout = new FormLayout(
            "7dlu, pref, pref, $lcgap, 25dlu, $lcgap, pref, 0:grow",
            "p, $lg, p, $lg, p, 14dlu, 40dlu");
        PanelBuilder builder = new PanelBuilder(layout);
        CellConstraints cc = new CellConstraints();

//      builder.addSeparator("General", 	    cc.xyw(1, 1, 8);
//      builder.add(limitCheck,				    cc.xy (2, 3));
//      builder.add(limitField,				    cc.xy (5, 3));

        builder.addSeparator("Show only books", cc.xyw(1, 1, 8));
        builder.add(priceCheck,                 cc.xy (2, 3));
        builder.addLabel("<=",                  cc.xy (3, 3));
        builder.add(priceField,                 cc.xy (5, 3));
        builder.addLabel("US$",                 cc.xy (7, 3));

        builder.add(yearCheck,                  cc.xy (2, 5));
        builder.addLabel(">=",                  cc.xy (3, 5));
        builder.add(yearField,                  cc.xy (5, 5));

        return builder.getPanel();
    }


    // Misc *******************************************************************

    @Override
    protected void doCloseWindow() {
        doCancel();
    }


    // Helper Code ************************************************************

    /**
     * @param cols  the number of columns used for the integer format
     * @return a non-grouping integer format with cols minimum digits
     */
    private Format createIntegerFormat(int cols) {
        NumberFormat format = NumberFormat.getIntegerInstance();
        format.setMinimumIntegerDigits(cols);
        format.setGroupingUsed(false);
        return format;
    }


}