/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.view.main;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.net.URL;
import java.util.Iterator;

import javax.swing.*;

import com.jgoodies.bookfinder.domain.Book;
import com.jgoodies.bookfinder.domain.EditorialReview;
import com.jgoodies.bookfinder.model.DetailsModel;
import com.jgoodies.bookfinder.view.Resources;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.looks.Options;
import com.jgoodies.uif.AbstractView;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.builder.ToolBarBuilder;
import com.jgoodies.uif.component.ToolBarButton;
import com.jgoodies.uif.panel.CardPanel;
import com.jgoodies.uif.panel.SimpleInternalFrame;
import com.jgoodies.uif.util.Modes;
import com.jgoodies.uif.util.SystemUtils;
import com.jgoodies.uifextras.util.HTMLUtils;
import com.jgoodies.uifextras.util.UIFactory;

/**
 * Builds and updates the BookFinder's details panel.<p>
 *
 * <strong>Note:</strong> Unlike the other panels in this application,
 * this details panels uses labels that are not bound to individual
 * Book properties. Instead, the BookChangeHandler listens to changes
 * of the selected book and updates the labels in method #updateContent.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.16 $
 */

final class DetailsView extends AbstractView {

    private static final String EMPTY_CARD   = "empty";
    private static final String CONTENT_CARD = "content";

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(DetailsView.class);


    private final DetailsModel detailsModel;

    private CardPanel   cardPanel;
    private JTabbedPane tabbedPane;
    private JLabel      iconLabel;
    private JLabel      productNameRow1Label;
    private JLabel      productNameRow2Label;
    private JLabel      authorsLabel;
    private JLabel      publisherLabel;
    private JLabel      releaseDateLabel;
    private JLabel      isbnLabel;
    private JLabel      priceLabel;


    // Instance Creation ****************************************************

    /**
     * Constructs a <code>DetailsView</code> on the given DetailsModel.
     *
     * @param detailsModel  provides the book holder and buy now actions
     */
    DetailsView(DetailsModel detailsModel) {
        this.detailsModel = detailsModel;
    }

    private void initEventHandling() {
        detailsModel.getBookHolder().addValueChangeListener(new BookChangeHandler());
    }

    @Override
    protected JComponent buildPanel() {
        initEventHandling();

        SimpleInternalFrame panel = new SimpleInternalFrame("Book");
        panel.setToolBar(buildToolBar());
        panel.setContent(buildContent());
        panel.setSelected(true);
        return panel;
    }


    // Building *************************************************************

    /**
     * Builds and returns the tool bar.
     */
    private JToolBar buildToolBar() {
        Action action = Application.getAction(
                detailsModel, DetailsModel.ACTION_HIDE_DETAILS);
        ToolBarBuilder builder = new ToolBarBuilder();
        builder.add(new CloseButton(action));
        return builder.getToolBar();
    }

    /**
     * Builds the content, which is a <code>CardPanel</code>
     * wrapped by a border stripped <code>JScrollPane</code>.
     */
    private JComponent buildContent() {
        cardPanel = new CardPanel();
        cardPanel.add(buildEmptyCard(), EMPTY_CARD);
        cardPanel.add(buildContentCard(), CONTENT_CARD);
        return cardPanel;
    }

    /**
     * Builds and returns an empty panel.
     */
    private JComponent buildEmptyCard() {
        FormLayout layout = new FormLayout(
            "pref, 7dlu, 0:grow",
            "pref");
        PanelBuilder builder = new PanelBuilder(layout);
        builder.setDefaultDialogBorder();
        CellConstraints cc = new CellConstraints();

        builder.addLabel("No item selected.", cc.xy(3, 1));
        return builder.getPanel();
    }

    /**
     * Creates and configures the UI components for the content panel.
     */
    private void initComponents() {
        iconLabel =
            new JLabel(RESOURCES.getIcon(Resources.LOGO_POWERED_BY_ID));
        productNameRow1Label = UIFactory.createBoldLabel("Product name");
        productNameRow2Label = UIFactory.createBoldLabel("");
        authorsLabel = new JLabel("Authors");
        publisherLabel = new JLabel("Publisher");
        releaseDateLabel = new JLabel("December, 1967");
        isbnLabel    = new JLabel("ISBN:");
        priceLabel   = UIFactory.createBoldLabel("$25", 0, Color.BLACK, false);
    }

    /**
     * Builds and returns the content card.
     */
    private JComponent buildContentCard() {
        initComponents();

        tabbedPane = new JTabbedPane(JTabbedPane.BOTTOM);
        tabbedPane.putClientProperty(Options.EMBEDDED_TABS_KEY, Boolean.TRUE);
        tabbedPane.addTab("Summary", buildSummaryTab());
        return tabbedPane;
    }

    /**
     * Builds and returns the overview tab.
     */
    private JComponent buildSummaryTab() {
        FormLayout layout = new FormLayout(
            "pref, 4dlu, 0:grow",
            "p, p, p, 4dlu, p, p, p, 4dlu, p, 4dlu, p, 0:grow");
        PanelBuilder builder = new PanelBuilder(layout, new WhitePanel());
        builder.setDefaultDialogBorder();
        CellConstraints cc = new CellConstraints();

        builder.add(iconLabel,            cc.xywh(1,  1, 1, 12, "fill, top"));
        builder.add(productNameRow1Label, cc.xy  (3,  2));
        builder.add(productNameRow2Label, cc.xy  (3,  3));
        builder.add(authorsLabel,         cc.xy  (3,  5));
        builder.add(publisherLabel,       cc.xy  (3,  6));
        builder.add(releaseDateLabel,     cc.xy  (3,  7));
        builder.add(isbnLabel,            cc.xy  (3,  9));
        builder.add(priceLabel,           cc.xy  (3, 11));

        return builder.getPanel();
    }

    /**
     * Creates, builds and returns a review tab.
     */
    private JComponent createReviewTab(EditorialReview review) {
        JEditorPane reviewPane = UIFactory.createHTMLPane(false, false);
        HTMLUtils.addDefaultStyleSheet(reviewPane);
        reviewPane.setEditable(false);
        reviewPane.setOpaque(false);
        reviewPane.setText(review.getContent());

        FormLayout layout = new FormLayout(
            "200dlu:grow",
            "fill:pref:grow");
        PanelBuilder builder = new PanelBuilder(
                layout,
                SystemUtils.IS_OS_MAC
                    ? new JPanel(null)
                    : new WhitePanel());
        builder.setDefaultDialogBorder();
        CellConstraints cc = new CellConstraints();

        builder.add(reviewPane, cc.xy(1, 1));
        final JScrollPane scrollPane =
            UIFactory.createStrippedScrollPane(builder.getPanel());
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                scrollPane.getVerticalScrollBar().setValue(0);
            }
        });
        return scrollPane;
    }


    // Updating the Content ***************************************************

    private void updateContent(Book book) {
        URL url = book.getMediumImageURL();
        Icon baseIcon = url == null
                ? null
                : Utils.dropIcon(
                        Utils.asynchronousIcon(url, getPanel()), false);
        iconLabel.setIcon(Utils.minimumSizeIcon(baseIcon, 125, 140));
        setWrappedProductName(book.getTitle());
        authorsLabel.setText(book.getConcatenatedAuthors());
        publisherLabel.setText(book.getPublisher());
        releaseDateLabel.setText(book.getPublicationDate());
        isbnLabel.setText("ISBN: " + book.getISBN());
        priceLabel.setText(book.getListPrice());

        while (tabbedPane.getTabCount() > 1) {
            tabbedPane.removeTabAt(1);
        }
        for (Iterator iter = book.getReviews().iterator(); iter.hasNext();) {
            EditorialReview review = (EditorialReview) iter.next();
            String reviewTitle = getReviewName(review.getSource());
            JComponent reviewTab = createReviewTab(review);
            tabbedPane.addTab(reviewTitle, reviewTab);
        }
    }


    private String getReviewName(String name) {
        return name.equals("Amazon.com")
            ? "Editorial Review"
            : name;
    }


    private void setWrappedProductName(String productName) {
        // Try to put all in one label.
        productNameRow1Label.setText(productName);
        int width = productNameRow1Label.getPreferredSize().width;
        //System.out.println(productName + ".width=" + width);
        if (width < 400) {
            productNameRow2Label.setText(" ");
            return;
        }
        // Try to find and split at a the first colon.
        int colonIndex = productName.indexOf(':', 30);
        if (colonIndex != -1 && productName.charAt(colonIndex+1) == ' ') {
            colonIndex++;
        }
        int length     = productName.length();
        int half       = length / 2;
        int spaceIndex = productName.indexOf(' ', half - 3);
        if (width < 800) {
            if ((colonIndex != -1) && (colonIndex < length - 10)) {
                productNameRow1Label.setText(productName.substring(0, colonIndex+1));
                productNameRow2Label.setText(productName.substring(colonIndex+1, length));
                return;
            } if (spaceIndex != -1) {
                productNameRow1Label.setText(productName.substring(0, spaceIndex+1));
                productNameRow2Label.setText(productName.substring(spaceIndex+1, length));
                return;
            }
        }
        productNameRow1Label.setText(productName.substring(0, half-1));
        productNameRow2Label.setText(productName.substring(half-1, length));
    }


    private class BookChangeHandler implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent evt) {
            Book newBook = (Book) evt.getNewValue();
            if (newBook == null) {
                cardPanel.showCard(EMPTY_CARD);
            } else {
                cardPanel.showCard(CONTENT_CARD);
                updateContent(newBook);
            }
        }
    }


    // Helper Code **********************************************************

    /**
     * A panel with a white background.
     */
    private static final class WhitePanel extends JPanel {

        private WhitePanel() {
            super(null);
        }

        @Override
        public void updateUI() {
            super.updateUI();
            if (Modes.isLafAqua()) {
                setOpaque(false);
            } else {
                setBackground(UIManager.getColor("text"));
            }
        }
    }


    /**
     * A button that resets the rollover state after the
     * action has been performed and if the button is not showing.
     */
    private final class CloseButton extends ToolBarButton {
        private CloseButton(Action action) {
            super(action);
        }
        @Override
        protected void fireActionPerformed(ActionEvent event) {
            super.fireActionPerformed(event);
            if (!isShowing()) {
                getModel().setRollover(false);
            }
        }
    }


}