/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.view.main;

import java.awt.BorderLayout;
import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;

import com.jgoodies.animation.*;
import com.jgoodies.bookfinder.model.MainModel;
import com.jgoodies.bookfinder.view.Resources;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.looks.LookUtils;
import com.jgoodies.uif.AbstractView;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.util.SystemUtils;
import com.jgoodies.uifextras.panel.GradientBackgroundPanel;
import com.jgoodies.uifextras.util.UIFactory;

/**
 * The header panel of the Book Finder application.
 * Provides an animated blend effect to switch messages.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.11 $
 *
 * @see MainModel
 */
final class HeaderView extends AbstractView {

    private static final int ANIMATION_DURATION = 200;
    private static final int ANIMATION_FPS      = 40;

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(HeaderView.class);

    private final MainModel mainModel;

    private JLabel[] prefixLabels;
    private JLabel[] suffixLabels;
    private JPanel[] panels;
    private JLabel iconLabel;
    private int foreground = 0;
    private int background = 1;
    private Animator animator;


    // Instance Creation ****************************************************

    /**
     * Constructs a <code>HeaderView</code> on the given MainModel.
     * Registers a listener with the MainModel to handle changes
     * in its activity text.
     *
     * @param mainModel   provides the header text model
     */
    HeaderView(MainModel mainModel) {
        this.mainModel = mainModel;
    }


    // Public API ***********************************************************

    /**
     * Displays a generic message.
     *
     * @param prefix    the message's first part
     * @param suffix    the message's last part
     */
    private synchronized void showMessage(String prefix, String suffix) {
        if (isAnimated()) {
            // Ensure a previous animation is stopped before we start a new one.
            if (animator != null) {
                animator.stop();
            }
        }
        String fullPrefix = prefix.length() == 0
            ? ""
            : prefix.toUpperCase() + ": ";
        prefixLabels[background].setText(fullPrefix);
        suffixLabels[background].setText(suffix.toUpperCase());
        foreground = 1 - foreground;
        background = 1 - background;
        if (!isAnimated()) {
            blendOverCompleted();
            return;
        }
        panels[background].setVisible(true);
        panels[foreground].setVisible(false);
        Animation blendOver = new BlendOverAnimation(ANIMATION_DURATION,
                prefixLabels[foreground], suffixLabels[foreground],
                prefixLabels[background], suffixLabels[background]);
        animator = new Animator(blendOver, ANIMATION_FPS);
        blendOver.addAnimationListener(new AnimationAdapter() {
            @Override
            public void animationStarted(AnimationEvent e) {
                panels[foreground].setVisible(true);
            }
            @Override
            public void animationStopped(AnimationEvent e) {
                blendOverCompleted();
            }
        });
        animator.start();
    }

    private void blendOverCompleted() {
        prefixLabels[foreground].setForeground(getGray(255));
        prefixLabels[background].setForeground(getGray(0));
        suffixLabels[foreground].setForeground(getOrange(255));
        suffixLabels[background].setForeground(getOrange(0));
        panels[background].setVisible(false);
        panels[foreground].setVisible(true);
    }


    // Building ************************************************************************

    /**
     * Creates and configures the UI components.
     */
    private void initComponents() {
        int fontExtraSize = SystemUtils.IS_OS_MAC
            ? 8
            : (LookUtils.IS_LOW_RESOLUTION ? 9 : 8);
        prefixLabels = new JLabel[2];
        prefixLabels[foreground] =
            UIFactory.createBoldLabel("", fontExtraSize, getGray(255), true);
        prefixLabels[background] =
            UIFactory.createBoldLabel("", fontExtraSize, getGray(255), true);
        suffixLabels = new JLabel[2];
        suffixLabels[foreground] =
            UIFactory.createBoldLabel("", fontExtraSize, getOrange(255), true);
        suffixLabels[background] =
            UIFactory.createBoldLabel("", fontExtraSize, getOrange(255), true);
        panels = new JPanel[2];

        iconLabel =
            new JLabel(RESOURCES.getIcon(Resources.LOGO_POWERED_BY_ID));

    }

    private void initEventHandling() {
        mainModel.addPropertyChangeListener(
                MainModel.PROPERTYNAME_ACTIVITY,
                new ActivityChangeHandler(this));
    }


    /**
     * Creates and answers the header area.
     */
    @Override
    protected JComponent buildPanel() {
        initComponents();
        initEventHandling();

        FormLayout layout = new FormLayout(
            "4dlu, pref:grow, 7dlu, pref, 4dlu",
            "43, 2");
        PanelBuilder builder = new PanelBuilder(layout, new GradientBackgroundPanel());
        CellConstraints cc = new CellConstraints();

        builder.add(buildForegroundPanel(), cc.xy (2, 1));
        builder.add(buildBackgroundPanel(), cc.xy (2, 1));
        builder.add(iconLabel,              cc.xy (4, 1));
        builder.add(new JSeparator(),       cc.xyw(1, 2, 5));

        return builder.getPanel();
    }


    private JComponent buildForegroundPanel() {
        JPanel panel = new JPanel(new BorderLayout());
        panel.setOpaque(false);
        panel.add(prefixLabels[foreground], BorderLayout.WEST);
        panel.add(suffixLabels[foreground], BorderLayout.CENTER);
        return panels[foreground] = panel;
    }

    private JComponent buildBackgroundPanel() {
        JPanel panel = new JPanel(new BorderLayout());
        panel.setOpaque(false);
        panel.add(prefixLabels[background], BorderLayout.WEST);
        panel.add(suffixLabels[background], BorderLayout.CENTER);
        return panels[background] = panel;
    }

    // Helper Methods *******************************************************************

    private boolean isAnimated() {
        return mainModel.getUISettingsModel().isAnimatingHeader();
    }

    private static Color getGray(int alpha) {
        return new Color(64, 64, 64, alpha);
    }

    private static Color getOrange(int alpha) {
        return new Color(255, 142, 12, alpha);
    }


    // Animation Class ******************************************************************

    /**
     * An animation that changes the colors of overlapping labels
     * to implement a blend over effect.
     *
     * @author Karsten Lentzsch
     */
    private final static class BlendOverAnimation extends AbstractAnimation {

        private final JLabel oldPrefix;
        private final JLabel oldSuffix;
        private final JLabel newPrefix;
        private final JLabel newSuffix;

        /**
         * Constructs an animation that changes the colors of the
         * prefix and suffix labels over the duration.
         */
        public BlendOverAnimation(long duration, JLabel oldPrefix, JLabel oldSuffix, JLabel newPrefix, JLabel newSuffix) {
            super(duration, true);
            this.oldPrefix = oldPrefix;
            this.oldSuffix = oldSuffix;
            this.newPrefix = newPrefix;
            this.newSuffix = newSuffix;
        }

        /**
         * Applies the effect: sets the text and time.
         */
        @Override
        protected final void applyEffect(long time) {
            // Work around freeze bug - do nothing if time == 0;
            if ((time == 0))
                return;

            int foregroundAlpha = (int) (255 * time / duration());
            int backgroundAlpha = 255 - foregroundAlpha;
            //System.out.print(foregroundAlpha + ";");
            oldPrefix.setForeground(getGray(foregroundAlpha));
            oldSuffix.setForeground(getOrange(foregroundAlpha));
            newPrefix.setForeground(getGray(backgroundAlpha));
            newSuffix.setForeground(getOrange(backgroundAlpha));
        }
    }


    // Event Handling *********************************************************

    private final static class ActivityChangeHandler implements PropertyChangeListener {

        private final HeaderView view;

        ActivityChangeHandler(HeaderView view) {
            this.view = view;
        }

        /**
         * The header text has changed. Updates the header.
         * First decomposes the compoung header text,
         * then invokes <code>#showMessage</code>.
         *
         * @param evt   the event that refers to the new text
         */
        public void propertyChange(PropertyChangeEvent evt) {
            String compoundText = (String) evt.getNewValue();
            int atIndex = compoundText.indexOf('@');
            int length  = compoundText.length();
            String prefix = compoundText.substring(0, atIndex);
            String suffix = compoundText.substring(atIndex + 1, length);
            view.showMessage(prefix, suffix);
        }
    }

}