/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.view.main;

import java.awt.Dimension;

import javax.swing.JComponent;

import com.jgoodies.animation.extras.Prompter;
import com.jgoodies.bookfinder.model.MainModel;
import com.jgoodies.uif.AbstractFrame;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ApplicationContext;
import com.jgoodies.uif.application.ResourceMap;

/**
 * The main frame of the Book Finder application.
 * It creates the menu bar, header, and all subpanels.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.12 $
 *
 * @see com.jgoodies.bookfinder.model.MainModel
 * @see com.jgoodies.bookfinder.view.main.MainPageView
 * @see Tour
 */
public final class MainFrame extends AbstractFrame {

    private static final Dimension MINIMUM_SIZE = new Dimension(500, 400);

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(MainFrame.class);

    /**
     * Refers to the mainModel that provides all high-level models.
     * Used to build this frame's main page.
     *
     * @see #buildContentPane()
     */
    private final MainModel mainModel;


    private MainPageView mainPageView;
    private final Prompter prompter;


    // Instance Creation ******************************************************

    /**
     * Constructs the main frame of the Book Finder.
     *
     * @param mainModel   provides bound properties and high-level models
     */
    public MainFrame(MainModel mainModel) {
        super(RESOURCES.getString("application.title"));
        setName("main");
        this.mainModel = mainModel;
        prompter = new Prompter(this);
    }


    // View Sections For the Tour *********************************************

    MainPageView getMainPageView() {
        return mainPageView;
    }

    JComponent getSearchPanel() {
        return mainPageView.getSearchPanel();
    }

    JComponent getResultPanel() {
        return mainPageView.getResultPanel();
    }

    JComponent getDetailsPanel() {
        return mainPageView.getDetailsPanel();
    }

    Prompter getPrompter() {
        return prompter;
    }


    // Building *************************************************************

    /**
     * Builds this MainFrame. Sets the prompter's glass pane first,
     * then executes the superclass behavior, and finally sets the menu bar.
     */
    @Override
    public void build() {
        setGlassPane(prompter.glassPane());
        super.build();
        setJMenuBar(new MenuBarView(mainModel).getMenuBar());
    }

    /**
     * Builds and returns this frame's content pane.
     */
    @Override
    protected JComponent buildContentPane() {
        mainPageView = new MainPageView(mainModel);
        return mainPageView.getPanel();
    }


    // Implementing Abstract Behavior ****************************************

    /**
     * Returns the frame's minimum size. It is used by the WindowUtils
     * to resize the window if the user has shrinked the window below
     * this given size.
     *
     * @return the frame's minimum size
     */
    public Dimension getWindowMinimumSize() {
        return MINIMUM_SIZE;
    }


    @Override
    protected void configureCloseOperation() {
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(
                ApplicationContext.getInstance().getApplication().getApplicationExitOnWindowClosingHandler());
    }


    // Storing and Restoring State ******************************************

    /**
     * Restores the frame's state from the user preferences.
     */
    @Override
    protected void restoreState() {
        super.restoreState();
        mainPageView.restoreFrom(ApplicationContext.getInstance().getUserPreferences());
    }

    /**
     * Stores the main page's state in the user preferences.
     */
    @Override
    protected void storeState() {
        super.storeState();
        mainPageView.storeIn(ApplicationContext.getInstance().getUserPreferences());
    }


}