/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.view.main;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.LayoutManager;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.prefs.Preferences;

import javax.swing.*;

import com.jgoodies.bookfinder.model.MainModel;
import com.jgoodies.bookfinder.model.UISettingsModel;
import com.jgoodies.uif.AbstractView;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.util.ComponentTreeUtils;
import com.jgoodies.uifextras.util.UIFactory;

/**
 * This is the main panel of the Book Finder application.
 * It creates the finder, overview, details view, and status bar.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.14 $
 *
 * @see com.jgoodies.bookfinder.model.MainModel
 */

final class MainPageView extends AbstractView {

    private static final Dimension PREFERRED_SIZE = new Dimension(700, 540);

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(MainPageView.class);

    private final MainModel mainModel;

    private HeaderView headerView;
    private SearchView searchView;
    private ResultView resultView;
    private DetailsView detailsView;
    private JSplitPane mainSplitPane;
    private JSplitPane overviewDetailsSplitPane;


    // Instance Creation ******************************************************

    MainPageView(MainModel mainModel) {
        this.mainModel = mainModel;
    }


    // Exposing View for the Tour *********************************************

    JComponent getHeaderPanel() {
        return headerView.getPanel();
    }

    JComponent getSearchPanel() {
        return searchView.getPanel();
    }

    SearchView getSearchView() {
        return searchView;
    }

    JComponent getResultPanel() {
        return resultView.getPanel();
    }

    JComponent getDetailsPanel() {
        return detailsView.getPanel();
    }


    // ************************************************************************

    /**
     * Answers if the details panel is visible.
     */
    public boolean isDetailsVisible() {
        return overviewDetailsSplitPane.getTopComponent() != null;
    }

    /**
     * Sets the visiblity of the dynamic help contents panel.
     */
    private void setDetailsVisible(boolean b) {
        if (isDetailsVisible() == b) {
            return;
        }
        if (b) {
            mainSplitPane.setRightComponent(overviewDetailsSplitPane);
            overviewDetailsSplitPane.setTopComponent(resultView.getPanel());
        } else {
            overviewDetailsSplitPane.setTopComponent(null);
            mainSplitPane.setRightComponent(resultView.getPanel());
        }
    }

    // Building *************************************************************

    private void initComponents() {
        headerView  = new HeaderView(mainModel);
        searchView  = new SearchView(mainModel.getSearchModel());
        resultView  = new ResultView(mainModel);
        detailsView = new DetailsView(mainModel.getDetailsModel());
    }


    private void initEventHandling() {
        mainModel.getUISettingsModel().addPropertyChangeListener(
                UISettingsModel.PROPERTYNAME_SHOWING_DETAILS,
                new ShowingDetailsChangeHandler());
    }


    /**
     * Builds this panel with the horizontal <code>JSplitPane</code> in the center,
     * and a status bar in the south.
     */
    @Override
    protected JComponent buildPanel() {
        initComponents();
        initEventHandling();

        JPanel panel = new RefreshedPanel(new BorderLayout());
        panel.add(headerView.getPanel(), BorderLayout.NORTH);
        panel.add(buildMainSplitPane(),  BorderLayout.CENTER);
        panel.add(buildStatusBar(),      BorderLayout.SOUTH);
        panel.setPreferredSize(PREFERRED_SIZE);

        return panel;
    }

    /**
     * Builds and answers the main <code>JSplitPane</code> that contains
     * the navigation elements on the left, and the view panels on the right.
     */
    private JComponent buildMainSplitPane() {
        mainSplitPane =
            UIFactory.createStrippedSplitPane(
                JSplitPane.HORIZONTAL_SPLIT,
                searchView.getPanel(),
                buildOverviewDetailsPanel(),
                0);
        mainSplitPane.setBorder(BorderFactory.createEmptyBorder(6, 4, 0, 4));
        return mainSplitPane;
    }

    /**
     * Builds the <code>ViewerPanel</code>, the <code>HelpView</code>
     * and answers them wrapped by a stripped <code>JSplitPane</code>.
     */
    private JComponent buildOverviewDetailsPanel() {
        resultView.getPanel().setMinimumSize(new Dimension(200, 50));
        resultView.getPanel().setPreferredSize(new Dimension(300, 100));

        overviewDetailsSplitPane =
            UIFactory.createStrippedSplitPane(
                JSplitPane.VERTICAL_SPLIT,
                resultView.getPanel(),
                detailsView.getPanel(),
                0.2);
        return overviewDetailsSplitPane;
    }

    /**
     * Builds and returns the status bar.
     */
    private JPanel buildStatusBar() {
        JLabel statusField = UIFactory.createPlainLabel(
                RESOURCES.getString("application.copyright"));
        JPanel panel = new JPanel(new BorderLayout());
        panel.setBorder(BorderFactory.createEmptyBorder(2, 6, 2, 3));
        panel.add(statusField, BorderLayout.WEST);
        return panel;
    }


    // Storing and Restoring State ******************************************

    private static final String KEY_MAIN_DIVIDER_LOCATION =
        "mainDividerLocation";
    private static final String KEY_VERTICAL_DIVIDER_LOCATION =
        "verticalDividerLocation";

    /**
     * Restores the frame's state from the user preferences.
     */
    protected void restoreFrom(Preferences userPrefs) {
        setDetailsVisible(mainModel.getUISettingsModel().isShowingDetails());

        int mainDividerLocation =
            userPrefs.getInt(KEY_MAIN_DIVIDER_LOCATION, -1);
        int verticalDividerLocation =
            userPrefs.getInt(KEY_VERTICAL_DIVIDER_LOCATION, -1);

        if ((mainDividerLocation > 100)
            && (mainDividerLocation < getPanel().getWidth() - 50)) {
           // mainSplitPane.setDividerLocation(mainDividerLocation);
        }
        if ((verticalDividerLocation > 100)
            && (verticalDividerLocation < getPanel().getHeight() - 50)) {
           // overviewDetailsSplitPane.setDividerLocation(
             //   verticalDividerLocation);
        }
    }

    /**
     * Stores the frame's state in the user preferences.
     */
    protected void storeIn(Preferences userPrefs) {
        int mainDividerLocation = mainSplitPane.getDividerLocation();
        int verticalDividerLocation =
            overviewDetailsSplitPane.getDividerLocation();
        userPrefs.putInt(
            KEY_MAIN_DIVIDER_LOCATION,
            mainDividerLocation);
        userPrefs.putInt(
            KEY_VERTICAL_DIVIDER_LOCATION,
            verticalDividerLocation);
    }

    // Misc *******************************************************************

    /**
     * In addition to the superclass behavior this class updates the UI
     * of the overviewDetails that may not be in the component tree.
     */
    private final class RefreshedPanel extends JPanel {

        private RefreshedPanel(LayoutManager layoutManager) {
            super(layoutManager);
        }

        /**
         * Explictly updates the UIs of invisible panels.
         */
        @Override
        public void updateUI() {
            super.updateUI();
            if (overviewDetailsSplitPane != null && !isDetailsVisible()) {
                ComponentTreeUtils.updateComponentTreeUI(overviewDetailsSplitPane);
            }
        }
    }


    // Event Handling *********************************************************

    private class ShowingDetailsChangeHandler implements PropertyChangeListener {

        public void propertyChange(PropertyChangeEvent evt) {
            boolean visible = Boolean.TRUE.equals(evt.getNewValue());
            setDetailsVisible(visible);
        }

    }

}