/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.view.main;

import java.awt.Color;
import java.awt.Component;
import java.awt.image.ImageObserver;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import com.jgoodies.binding.adapter.BasicComponentFactory;
import com.jgoodies.bookfinder.domain.Book;
import com.jgoodies.bookfinder.model.MainModel;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uifextras.util.UIFactory;

/**
 * Builds the list that displays the search results.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.4 $
 *
 * @see MainModel
 */
final class ResultListView {

    private JList list;


    /**
     * Builds, binds, configures and returns this overview list.
     *
     * @param mainModel   used to access the result SelectionInList
     * @return the overview list
     */
    JComponent build(MainModel mainModel) {
        list = BasicComponentFactory.createList(
                mainModel.getSelectionInFilteredBook(),
                new OverviewListCellRenderer(list));
        return list;
    }


    // Renderer ***************************************************************

    /**
     * A renderer that has an icon in the left, and three rows in the right:
     * Product name, authors, price.
     */
    private static final class OverviewListCellRenderer extends JPanel
            implements
                ListCellRenderer {

        private static final Border NO_FOCUS_BORDER = new EmptyBorder(1, 1, 1,
                1);

        private final ImageObserver imageObserver;
        private final JLabel iconLabel;
        private final JLabel productNameLabel;
        private final JLabel authorsLabel;
        private final JLabel priceLabel;

        private OverviewListCellRenderer(ImageObserver imageObserver) {
            super(null);
            this.imageObserver = imageObserver;
            iconLabel = new JLabel();
            productNameLabel = UIFactory.createBoldLabel("Product name");
            authorsLabel = new JLabel("Author");
            priceLabel = UIFactory.createBoldLabel("Price");
            build();
        }

        private void build() {
            iconLabel.setHorizontalTextPosition(JLabel.CENTER);
            FormLayout layout = new FormLayout("60px, 4dlu, pref:grow",
                    "6px, pref, pref, pref, 25px");
            PanelBuilder builder = new PanelBuilder(layout, this);
            CellConstraints cc = new CellConstraints();

            builder.add(iconLabel,        cc.xywh(1, 1, 1, 5));
            builder.add(productNameLabel, cc.xy(3, 2));
            builder.add(authorsLabel,     cc.xy(3, 3));
            builder.add(priceLabel,       cc.xy(3, 4));
        }

        public Component getListCellRendererComponent(JList list, Object value,
                int index, boolean isSelected, boolean cellHasFocus) {
            Color foreground;
            if (isSelected) {
                setBackground(list.getSelectionBackground());
                foreground = list.getSelectionForeground();
            } else {
                setBackground(list.getBackground());
                foreground = list.getForeground();
            }
            productNameLabel.setForeground(foreground);
            authorsLabel.setForeground(foreground);
            priceLabel.setForeground(foreground);

            Book details = (Book) value;
            Icon baseIcon = details.getSmallImageURL() == null ? null : Utils
                    .dropIcon(
                            Utils.asynchronousIcon(details
                                    .getSmallImageURL(), imageObserver),
                            isSelected);
            iconLabel.setIcon(Utils.minimumSizeIcon(baseIcon, 50, 70));
            productNameLabel.setText(details.getTitle());
            authorsLabel.setText(details.getAuthorsEtAl());
            priceLabel.setText(details.getListPrice());

            //setEnabled(list.isEnabled());
            //setFont(list.getFont());
            setBorder(cellHasFocus ? UIManager.getBorder(
                    "List.focusCellHighlightBorder") : NO_FOCUS_BORDER);
            return this;
        }

        /**
		 * Overridden for performance reasons. See the <a href="#override">
		 * Implementation Note</a> for more information.
		 */
        @Override
        protected void firePropertyChange(String propertyName, Object oldValue,
                Object newValue) {
            // Strings get interned...
            if (propertyName == "text")
                super.firePropertyChange(propertyName, oldValue, newValue);
        }

        /**
		 * Overridden for performance reasons. See the <a href="#override">
		 * Implementation Note</a> for more information.
		 */
        @Override
        public void firePropertyChange(String propertyName, byte oldValue,
                byte newValue) {
            // Overridden for performance reasons.
        }

        /**
		 * Overridden for performance reasons. See the <a href="#override">
		 * Implementation Note</a> for more information.
		 */
        @Override
        public void firePropertyChange(String propertyName, char oldValue,
                char newValue) {
            // Overridden for performance reasons.
        }

        /**
		 * Overridden for performance reasons. See the <a href="#override">
		 * Implementation Note</a> for more information.
		 */
        @Override
        public void firePropertyChange(String propertyName, short oldValue,
                short newValue) {
            // Overridden for performance reasons.
        }

        /**
		 * Overridden for performance reasons. See the <a href="#override">
		 * Implementation Note</a> for more information.
		 */
        @Override
        public void firePropertyChange(String propertyName, int oldValue,
                int newValue) {
            // Overridden for performance reasons.
        }

        /**
		 * Overridden for performance reasons. See the <a href="#override">
		 * Implementation Note</a> for more information.
		 */
        @Override
        public void firePropertyChange(String propertyName, long oldValue,
                long newValue) {
            // Overridden for performance reasons.
        }

        /**
		 * Overridden for performance reasons. See the <a href="#override">
		 * Implementation Note</a> for more information.
		 */
        @Override
        public void firePropertyChange(String propertyName, float oldValue,
                float newValue) {
            // Overridden for performance reasons.
        }

        /**
		 * Overridden for performance reasons. See the <a href="#override">
		 * Implementation Note</a> for more information.
		 */
        @Override
        public void firePropertyChange(String propertyName, double oldValue,
                double newValue) {
            // Overridden for performance reasons.
        }

        /**
		 * Overridden for performance reasons. See the <a href="#override">
		 * Implementation Note</a> for more information.
		 */
        @Override
        public void firePropertyChange(String propertyName, boolean oldValue,
                boolean newValue) {
            // Overridden for performance reasons.
        }

    }

}