/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.view.main;

import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.ActionMap;
import javax.swing.JComponent;
import javax.swing.JToolBar;

import com.jgoodies.bookfinder.model.MainModel;
import com.jgoodies.bookfinder.model.SearchResultModel;
import com.jgoodies.bookfinder.model.UISettingsModel;
import com.jgoodies.bookfinder.model.UISettingsModel.OverviewMode;
import com.jgoodies.uif.AbstractView;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.builder.ToolBarBuilder;
import com.jgoodies.uif.panel.CardPanel;
import com.jgoodies.uif.panel.SimpleInternalFrame;
import com.jgoodies.uifextras.util.UIFactory;

/**
 * A container that wraps <code>Viewer</code> instances with a
 * <code>JScrollPane</code>, which in turn is wrapped by
 * a <code>SimpleInternalFrame</code>.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.8 $
 *
 * @see com.jgoodies.bookfinder.model.MainModel
 */
final class ResultView extends AbstractView {

	private static final String TITLE_PREFIX = "Overview";
    private static final String MESSAGE_CARD = "Message";

    private final MainModel mainModel;

	private CardPanel cardPanel;


    // Instance Creation ******************************************************

	/**
	 * Constructs an <code>ResultView</code>.
	 */
	ResultView(MainModel mainModel) {
        this.mainModel = mainModel;
    }

    @Override
    protected JComponent buildPanel() {
        SimpleInternalFrame panel = new SimpleInternalFrame(TITLE_PREFIX);
		panel.setToolBar(buildToolBar());
        panel.setContent(buildContent());
        panel.setSelected(true);

        updateCard();
        initEventHandling();
        return panel;
	}


	private void initEventHandling() {
        PropertyChangeListener modeChangeHandler = new ModeChangeHandler();
        mainModel.getSearchResultModel().addPropertyChangeListener(
                SearchResultModel.PROPERTYNAME_MESSAGE,
                modeChangeHandler);
        mainModel.getUISettingsModel().addPropertyChangeListener(
                UISettingsModel.PROPERTYNAME_OVERVIEW_MODE,
                modeChangeHandler);
        mainModel.getSearchResultModel().addPropertyChangeListener(
                SearchResultModel.PROPERTYNAME_FILTERED_BOOKS,
                new TitleChangeHandler());
	}


	// Building *************************************************************

	/**
	 * Builds and returns the tool bar.
	 */
	private JToolBar buildToolBar() {
        ActionMap map = Application.getActionMap(mainModel);
        ToolBarBuilder builder = new ToolBarBuilder();
        builder.add(map.get("openFilterDialog"));
		builder.addGap();
        builder.addToggle(mainModel.getSummaryModeAction());
        builder.addToggle(mainModel.getListModeAction());
        builder.addToggle(mainModel.getTableModeAction());
		return builder.getToolBar();
	}


	/**
	 * Builds the content, which is a <code>CardPanel</code>
	 * wrapped by a border stripped <code>JScrollPane</code>.
	 */
	private JComponent buildContent() {
		cardPanel = new CardPanel();
		cardPanel.add(buildMessageCard(), MESSAGE_CARD);
		cardPanel.add(buildSummaryCard(), OverviewMode.SUMMARY.name());
		cardPanel.add(buildListCard(),	  OverviewMode.LIST.name());
		cardPanel.add(buildTableCard(),	  OverviewMode.TABLE.name());
		cardPanel.setMinimumSize  (new Dimension(200, 200));
		return cardPanel;
	}


	/**
	 * Builds and returns the overview message card.
	 */
	private JComponent buildMessageCard() {
		return UIFactory.createStrippedScrollPane(
                new ResultMessageView(mainModel.getSearchResultModel()).build());
	}


	/**
	 * Builds and returns the overview summary card.
	 */
	private JComponent buildSummaryCard() {
		return UIFactory.createStrippedScrollPane(
                new ResultListSummaryView().build(mainModel));
	}


	/**
	 * Builds and returns the overview list card.
	 */
	private JComponent buildListCard() {
		return UIFactory.createStrippedScrollPane(
                new ResultListView().build(mainModel));
	}


	/**
	 * Builds and returns the overview table card.
	 */
	private JComponent buildTableCard() {
		return UIFactory.createStrippedScrollPane(
                new ResultTableView().build(mainModel));
	}


	// Helper Code  *********************************************************

	private String getDetailsTitle() {
		int rawCount	  = mainModel.getSearchResultModel().getBooksCount();
		int filteredCount = mainModel.getSearchResultModel().getFilteredBooksCount();
		String suffix = rawCount == filteredCount
				? ""
				: " (BookFilter matched " + filteredCount + " of " + rawCount + " books)";
        return TITLE_PREFIX + suffix;
	}

    private void updateCard() {
        String cardName = mainModel.getSearchResultModel().hasMessage()
            ? MESSAGE_CARD
            : mainModel.getUISettingsModel().getOverviewMode().name();
        cardPanel.showCard(cardName);
    }


    // Handling Changes *******************************************************

    private class ModeChangeHandler implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent e) {
            updateCard();
        }
    }

    private class TitleChangeHandler implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent e) {
            ((SimpleInternalFrame) getPanel()).setTitle(getDetailsTitle());
        }
    }


}