/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.view.main;

import javax.swing.*;

import com.jgoodies.binding.adapter.BasicComponentFactory;
import com.jgoodies.binding.beans.BeanAdapter;
import com.jgoodies.bookfinder.model.SearchModel;
import com.jgoodies.bookfinder.view.Resources;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif.AbstractView;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.builder.ToolBarBuilder;
import com.jgoodies.uif.panel.SimpleInternalFrame;
import com.jgoodies.uif.util.ComponentUtils;
import com.jgoodies.uifextras.util.GradientSeparator;

/**
 * Builds a search panel for the author and keywords searches.
 * Consists of two sections:
 * one for the keywords search, one for the author search.
 * Each section consists of a label, an input field and a GO button.<p>
 *
 * The panel is bound to an instance of <code>SearchModel</code>.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.13 $
 *
 * @see com.jgoodies.bookfinder.model.SearchModel
 * @see Tour
 */
final class SearchView extends AbstractView {

    /**
     * Refers to the searchModel that provides the models and actions.
     */
    private final SearchModel searchModel;

    private JTextField keywordsField;
    private JButton keywordsGoButton;
    private JTextField authorField;
    private JButton authorGoButton;


    // Instance Creation **************************************************

    /**
     * Constructs a <code>SearchView</code> for the given SearchModel.
     *
     * @param searchModel   provides the models and actions for this panel
     */
    SearchView(SearchModel searchModel) {
        this.searchModel = searchModel;
    }

    @Override
    protected JComponent buildPanel() {
        SimpleInternalFrame panel = new SimpleInternalFrame("Search");
        panel.setToolBar(buildToolBar());
        panel.setContent(buildContent());
        return panel;
    }


    // Exposing Components for the Tour ***************************************

    /*
     * TODO: The Tour requests the panel components to get their bounds.
     * Return the component bounds, not the components.
     */


    /**
     * Returns the keyword field.
     */
    JTextField getKeywordField() {
        return keywordsField;
    }

    /**
     * Returns the keyword go button.
     */
    JButton getKeywordGoButton() {
        return keywordsGoButton;
    }


    // Building ***************************************************************

    /**
     * Builds and returns the toolbar.
     */
    private JToolBar buildToolBar() {
        ToolBarBuilder builder = new ToolBarBuilder();
        builder.add(Box.createVerticalStrut(23));
        return builder.getToolBar();
    }

    /**
     * Creates, binds and configures the UI components.
     */
    private void initComponents() {
        BeanAdapter beanAdapter = new BeanAdapter(searchModel, true);

        authorField = BasicComponentFactory.createTextField(
                beanAdapter.getValueModel(SearchModel.PROPERTYNAME_AUTHOR),
                false);

        keywordsField = BasicComponentFactory.createTextField(
                beanAdapter.getValueModel(SearchModel.PROPERTYNAME_KEYWORDS),
                false);

        authorField.  setColumns(10);
        keywordsField.setColumns(10);

        ActionMap searchMap = Application.getActionMap(searchModel);
        Action findByAuthorAction   = searchMap.get(SearchModel.ACTION_FIND_BY_AUTHOR);
        Action findByKeywordsAction = searchMap.get(SearchModel.ACTION_FIND_BY_KEYWORDS);
        authorGoButton   = new JButton(findByAuthorAction);
        keywordsGoButton = new JButton(findByKeywordsAction);

        ComponentUtils.addEnterAction(authorField, findByAuthorAction);
        ComponentUtils.addEnterAction(keywordsField, findByKeywordsAction);
    }


    /**
     * Builds and returns the content, a panel with two sections:
     * one for the keywords search, one for the author search.
     * Each section consists of a label, an input field and a GO button.
     */
    private JComponent buildContent() {
        initComponents();
        String keywordsText = "Find books by keywords";
        String authorText   = "Find books by author";
        JComponent separator =
            GradientSeparator.createHorizontalGradient(
                Resources.AMAZON_ORANGE);

        FormLayout layout = new FormLayout(
            "pref:grow, 3dlu, pref",
            "pref, 2dlu, pref, 15dlu, 2px, 13dlu, pref, 2dlu, pref");

        PanelBuilder builder = new PanelBuilder(layout);
        builder.setBorder(Borders.createEmptyBorder("14dlu, 5dlu, 4dlu, 4dlu"));
        CellConstraints cc = new CellConstraints();

        builder.addLabel(keywordsText,  cc.xyw(1, 1, 3));
        builder.add(keywordsField,      cc.xy (1, 3));
        builder.add(keywordsGoButton,   cc.xy (3, 3));
        builder.add(separator,          cc.xyw(1, 5, 3));
        builder.addLabel(authorText,    cc.xyw(1, 7, 3));
        builder.add(authorField,        cc.xy (1, 9));
        builder.add(authorGoButton,     cc.xy (3, 9));

        return builder.getPanel();
    }


}
