/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.view.main;

import java.awt.Dimension;
import java.util.LinkedList;
import java.util.List;

import javax.swing.Action;
import javax.swing.JComponent;

import com.jgoodies.animation.*;
import com.jgoodies.animation.extras.Prompter;
import com.jgoodies.bookfinder.model.MainModel;
import com.jgoodies.uif.application.Application;

/**
 * Provides the Book Finder's integrated tour.
 * Uses an animation to drive the Prompter.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.9 $
 *
 * @see com.jgoodies.animation.Animation
 * @see com.jgoodies.animation.Animator
 */

public final class Tour {

    private static final long STEP_DURATION = 2500;


    private final MainModel mainModel;

    private final MainFrame mainFrame;

    public Tour(MainModel mainModel, MainFrame mainFrame) {
        this.mainModel = mainModel;
        this.mainFrame  = mainFrame;
    }

    /**
     * Starts the animated tour. Prepares the UI, creates animations,
     * creates an animator, and finally launches the animator.
     */
    public void start() {
        final Action startTourAction =
            Application.getAction(mainModel, MainModel.ACTION_START_TOUR);
        startTourAction.setEnabled(false);
        getPrompter().setVisible(true);
        getMainModel().setSearchContentsBlank();
        getMainModel().showDetails();
        getMainModel().showActivity("", "");

        Animation animation = createComposedAnimation();

        Animator animator = new Animator(animation, 25);
        animation.addAnimationListener(new AnimationAdapter() {
            @Override
            public void animationStopped(AnimationEvent e) {
                getPrompter().setVisible(false);
                startTourAction.setEnabled(true);
            }

        });
        animator.start();
    }

    /**
     * Creates the tour animation that is composed of several steps.
     */
    private Animation createComposedAnimation() {
        List parts = new LinkedList();
        parts.add(getPrompter().createHideAnimation(getMainPage()));
        parts.add(Animations.pause(500));
        parts.add(
            getPrompter().createFade(
                3000,
                "THE BOOK FINDER TOUR",
                getMainPage(),
                new Dimension(300, 40),
                Prompter.CENTER));

        parts.add(Animations.pause(500));
        parts.add(
            getPrompter().createFade(
                STEP_DURATION,
                "STEP 1",
                getMainPageView().getSearchPanel(),
                new Dimension(100, 40),
                Prompter.NORTH));
        parts.add(Animations.pause(500));
        parts.add(
            getPrompter().createHideAnimation(
                getMainPageView().getHeaderPanel(),
                getMainPageView().getResultPanel(),
                getMainPageView().getDetailsPanel()));
        parts.add(Animations.pause(1000));
        parts.add(
            getPrompter().createTypeAnimation(
                3000,
                "TYPE HERE",
                getMainPageView().getSearchView().getKeywordField(),
                new Dimension(140, 40)));
        parts.add(Animations.pause(1000));
        parts.add(
            getPrompter().createClickAnimation(
                3000,
                "CLICK HERE",
                getMainPageView().getSearchView().getKeywordGoButton(),
                new Dimension(200, 40)));
        parts.add(Animations.pause(1000));
        parts.add(
            getPrompter().createSpace(
                2500,
                "LOOK HERE",
                getMainPageView().getHeaderPanel(),
                new Dimension(220, 40),
                Prompter.WEST));

        parts.add(
            getPrompter().createHideAnimation(
                getMainPageView().getResultPanel(),
                getMainPageView().getDetailsPanel()));
        parts.add(Animations.pause(500));
        parts.add(
            getPrompter().createFade(
                STEP_DURATION,
                "STEP 2",
                getMainPageView().getResultPanel(),
                new Dimension(100, 40),
                Prompter.NORTH));
        parts.add(
            getPrompter().createClickAnimation(
                3000,
                "CHOOSE HERE",
                getMainPageView().getResultPanel(),
                new Dimension(210, 40)));

        parts.add(getPrompter().createHideAnimation(
                getMainPageView().getDetailsPanel()));
        parts.add(Animations.pause(500));
        parts.add(
            getPrompter().createFade(
                STEP_DURATION,
                "STEP 3",
                getMainPageView().getDetailsPanel(),
                new Dimension(100, 40),
                Prompter.NORTH));
        parts.add(
            getPrompter().createSpace(
                3000,
                "BROWSE HERE",
                getMainPageView().getDetailsPanel(),
                new Dimension(240, 40),
                Prompter.CENTER));
        parts.add(getPrompter().createShowAllAnimation());

        return Animations.sequential(parts);
    }


    // Collaborator Access **************************************************************

    private MainModel getMainModel() {
        return mainModel;
    }

    private MainFrame getMainFrame() {
        return mainFrame;
    }

    private MainPageView getMainPageView() {
        return getMainFrame().getMainPageView();
    }

    private JComponent getMainPage() {
        return getMainPageView().getPanel();
    }

    private Prompter getPrompter() {
        return getMainFrame().getPrompter();
    }

}
