/*
 * Copyright (c) 2002-2004 Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.bookfinder.view.main;

import java.awt.*;
import java.awt.image.ImageObserver;
import java.net.URL;

import javax.swing.Icon;

/**
 * Consists only of static convenience methods for the icon presentation.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.2 $
 *
 * @see     javax.swing.Icon
 * @see     java.awt.image.ImageObserver
 */

final class Utils {

    private Utils() {
        // Overrides default constructor; prevents instantiation.
    }


    // Factory Methods ********************************************************

    /**
     * Constructs an asynchronously loaded image icon from the given URL,
     * using the specified imageObserver for notification of image updates.
     */
    static Icon asynchronousIcon(URL url, ImageObserver imageObserver) {
        return asynchronousIcon(
                Toolkit.getDefaultToolkit().getImage(url),
                imageObserver);
    }


    /**
     * Constructs an asynchronously loaded image icon from the given image,
     * using the specified imageObserver for notification of image updates.
     */
    static Icon asynchronousIcon(Image image, ImageObserver imageObserver) {
        return new AsynchronousImageIcon(image, imageObserver);
    }


	/**
	 * Returns a wrapped icon that has minimum width and height.
	 */
	static Icon minimumSizeIcon(Icon icon, int minimumWidth, int minimumHeight) {
		return new MinimumSizedIcon(icon, minimumWidth, minimumHeight);
	}


	/**
	 * Returns a wrapped icon that provides a drop shadow.
	 */
	static Icon dropIcon(Icon icon, boolean selected) {
		return new DropShadowIcon(icon, selected);
	}


	// Helper Classes *******************************************************************

	/**
     * An icon that has a minimum size, either horizontally, vertically, or  both.
     */
	private static final class MinimumSizedIcon implements Icon {

		private final Icon icon;
		private final int  minimumWidth;
		private final int  minimumHeight;


		private MinimumSizedIcon(Icon icon, int minimumWidth, int minimumHeight) {
			this.icon          = icon;
			this.minimumWidth  = minimumWidth;
			this.minimumHeight = minimumHeight;
		}


		public int getIconWidth()	{
			int iconWidth = icon == null ? 0 : icon.getIconWidth();
			return Math.max(iconWidth,  minimumWidth);
		}

		public int getIconHeight() {
			int iconHeight = icon == null ? 0 : icon.getIconHeight();
			return Math.max(iconHeight, minimumHeight);
		}


		public void paintIcon(Component c, Graphics g, int x, int y) {
			if (icon == null)
				return;
			int xOffset = Math.max(0, (getIconWidth()  - icon.getIconWidth())  / 2);
			int yOffset = Math.max(0, (getIconHeight() - icon.getIconHeight()) / 2);
			icon.paintIcon(c, g, x + xOffset, y + yOffset);
		}
	}


	/**
     * An Icon that paints a drop shadow around a base icon.
     */
	private static final class DropShadowIcon implements Icon {

		private static final Color INNER_SHADOW = new Color(187, 187, 187);
		private static final Color MID_SHADOW	= new Color(220, 220, 220);
		private static final Color OUTER_SHADOW = new Color(244, 244, 244);

		private final Icon	  icon;
		private final boolean selected;


		private DropShadowIcon(Icon icon, boolean selected) {
			this.icon	  = icon;
			this.selected = selected;
		}

		public  int getIconWidth()	{ return icon.getIconWidth() + 4; }
		public  int getIconHeight()	{ return icon.getIconHeight()+ 4; }

		public void paintIcon(Component c, Graphics g, int x, int y) {
			icon.paintIcon(c, g, x + 1, y + 1);
			int width = getIconWidth();
			int height = getIconHeight();
			if (selected || width < 10)
				return;

			Graphics2D g2 = (Graphics2D) g;
			g2.translate(x, y);
			int w = width - 4;
			int h = height - 4;
			g2.setColor(OUTER_SHADOW);
			g2.fillRect(1, 0, w, 1);
			g2.fillRect(0, 1, 1, h);
			g2.fillRect(width - 1, 3, 1, h-1);
			g2.fillRect(3, height - 1, w-1, 1);
			g2.setColor(MID_SHADOW);
			g2.fillRect(width - 2, 2, 1, h);
			g2.fillRect(2, height - 2, w, 1);
			g2.setColor(INNER_SHADOW);
			g2.fillRect(width - 3, 1, 1, h);
			g2.fillRect(1, height - 3, w +1, 1);
			g2.translate(-x, -y);
		}

	}


    private static class AsynchronousImageIcon implements Icon {

        private final Image         image;
        private final ImageObserver imageObserver;

        /**
         * Constructs an asynchronously loaded image icon from the given image,
         * using the specified imageObserver for notification of image updates.
         */
        AsynchronousImageIcon(Image image, ImageObserver imageObserver) {
            this.image         = image;
            this.imageObserver = imageObserver;
        }

        public int getIconWidth()  { return image.getWidth(null);   }
        public int getIconHeight() { return image.getHeight(null);  }

        public void paintIcon(Component c, Graphics g, int x, int y) {
            g.drawImage(image, x, y, imageObserver);
        }

    }

}
