/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.forms.demo.domain;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.jgoodies.forms.tutorial.QuickStartExample;
import com.jgoodies.forms.tutorial.basics.*;
import com.jgoodies.forms.tutorial.building.*;
import com.jgoodies.forms.tutorial.factories.ButtonBarFactoryExample;
import com.jgoodies.forms.tutorial.pitfalls.GrowingTextAreaExample;
import com.jgoodies.forms.tutorial.pitfalls.VerticalGrowthExample;

/**
 * Creates the list of examples that is used in the Examples bar.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.8 $
 */

public final class ExampleFactory {

    private ExampleFactory() {
        // Override the default constructor; prevents instantiation.
    }

    // Public API ***********************************************************

    /**
     * Creates and returns an unmodifiable list of <code>Example</code>
     * instances.
     *
     * @return an unmodifiable list of <code>Example</code> instances
     */
    public static List<Example> createExampleList() {
        List<Example> examples = new ArrayList<Example>();

        //Quick Start ------------------------------------------------------

        examples.add(
            new Example(
                "Quick Start",
                "Demonstrates the typical steps how to work with the FormLayout:\n"
                    + "create a FormLayout instance, group columns and rows, "
                    + "create a non-visual builder, add a standardized border, "
                    + "and finally add the UI components.",
                new QuickStartExample()));

        // Layout Basics ------------------------------------------------------

        examples.add(
            new Example(
                "Basics 1 - Alignments",
                "Demonstrates the FormLayout alignment options.\n\n" +
                "Column alignments are: left, center, right, fill, default\n" +
                "Row alignments are: top, center, bottom, fill, default",
                new AlignmentExample()));

        examples.add(
            new Example(
                "Basics 2 - Basic Sizes",
                "Demonstrates the basic FormLayout sizing options:\n" +
                "constant, minimum and preferred.\n\n" +
                "The text components have different minimum and preferred sizes.",
                new BasicSizesExample()));

        examples.add(
            new Example(
                "Basics 3 - Growing",
                "Demonstrates the FormLayout growing options:\n" +
                "none, default, weighted.",
                new GrowingExample()));

        examples.add(
            new Example(
                "Basics 4 - Span",
                "Demonstrates how components can span multiple columns and rows.",
                new SpanExample()));

        examples.add(
            new Example(
                "Basics 5 - Grouping",
                "Demonstrates how to give columns and rows equal sizes.\n" +
                "This is an essential feature for balanced and elegant design.",
                new GroupingExample()));

        examples.add(
            new Example(
                "Basics 6 - Units",
                "Demonstrates the different sizing units supported by the FormLayout:\n" +
                "Pixel, Dialog Units (dlu), Point, Millimeter, Centimeter and Inch.\n\n" +
                "Pt, mm, cm, and in honor the screen resolution; dlus honor the font's pixel size too.",
                new UnitsExample()));

        examples.add(
            new Example(
                "Basics 7 - Cell Alignments",
                "Demonstrates how FormLayout applies the default column and row " +
                "alignments to cells - and how to override the defaults." +
                "The buttons show the cell constraint alignments.\n\n" +
                "Cells that specify no alignment use the column and row defaults." +
                "Column default is Fill, row default is Center.",
                new CellAlignmentExample()));

        examples.add(
            new Example(
                "Basics 8 - Component Sizes",
                "Demonstrates the three component sizes: Min, Default, Pref.\n\n" +
                "Min and Pref measure the components minimum and preferred size," +
                "where the Default size behaves like Pref but shrinks if the  " +
                "container space is scarce.",
                new ComponentSizesExample()));

        examples.add(
            new Example(
                "Basics 9 - Bounded Sizes",
                "Demonstrates how to specify lower and upper bounds for " +
                "column and row sizes.\n\n" +
                "This is useful to give components or related panels " +
                "a more stable layout.\n" +
                "The first two tabs below jump, " +
                "whereas the remaining tabs are stable.",
                new BoundedSizesExample()));


        // Different Building Styles ------------------------------------------

        examples.add(
            new Example(
                "Building - Plain",
                "Demonstrates the very basic approach to add components to a panel.\n\n" +
                "Defines columns and rows statically, " +
                "and adds components to the container without helper classes.",
                new PlainExample()));

        examples.add(
            new Example(
                "Building - PanelBuilder",
                "Demonstrates a basic approach to add components to a panel.\n\n" +
                "Defines columns and rows statically, " +
                "and adds components to the container using a PanelBuilder.",
                new PanelBuilderExample()));

/*
        examples.add(
            new Example(
                "Building - String Constraints",
                "Demonstrates how to build a form with String cell constraints\n\n" +
                "This style is not recommended.",
                "com.jgoodies.forms.tutorial.building.StringConstraintsExample"));

*/
        examples.add(
            new Example(
                "Building - Row Counter",
                "Demonstrates how to fill a FormLayout using a PanelBuilder.\n\n" +
                "Defines columns and rows statically, " +
                "adds components to a PanelBuilder, " +
                "and uses a variable to track the current row.",
                new RowCounterExample()));

        examples.add(
            new Example(
                "Building - Dynamic Rows",
                "Demonstrates how to add rows dynamically to a FormLayout.\n\n" +
                "Defines columns statically and rows dynamically, " +
                "and adds components via the PanelBuilder.",
                new DynamicRowsExample()));

        examples.add(
            new Example(
                "Building - Default Form",
                "Demonstrates how to fill a FormLayout using the DefaultFormBuilder.\n\n" +
                "Defines columns statically and rows dynamically, " +
                "and appends components via the DefaultFormBuilder.",
                new DefaultFormBuilderExample()));

        examples.add(
            new Example(
                "Building - Custom Rows",
                "Shows three ways to add custom rows to a DefaultFormBuilder:\n" +
                "1) single custom row, 2) standard + custom row, 3) multiple standard rows.\n\n" +
                "They differ in the position of the 'Comment' label," +
                " the alignment of font baselines and the height of the custom row.",
                new DefaultFormWithCustomRowsExample()));

        examples.add(
            new Example(
                "Building - Custom Areas",
                "Compares approaches how to append a custom area at the end of" +
                " a panel built with the DefaultFormBuilder.\n\n" +
                "1) Using two custom rows to align the leading label, " +
                "2) using a single custom row with label on top, " +
                "3) using a separator.",
                new DefaultFormWithCustomAreasExample()));

        examples.add(
            new Example(
                "Building - Indent Column",
                "Demonstrates how to build a Form with a leading indent column.\n\n" +
                "Uses and configures the DefaultFormBuilder to automatically " +
                "skip the leading column if you add components.",
                new IndentColumnExample()));

        examples.add(
            new Example(
                "Building - Debug a Panel",
                "Exhibits layout details using the FormDebugPanel and the FormDebugUtils.\n\n" +
                "The example also demonstrates efficient panel building " +
                "with the DefaultFormBuilder. The builder has been configured " +
                "to use a leading indent column.",
                new FormDebugExample()));

        examples.add(
            new Example(
                "Building - Orientation",
                "Demonstrates building panels from left to right or right to left.\n\n" +
                "Uses the DefaultFormBuilder configured with three different orientations: " +
                "fixed left-to-right, fixed right-to-left, and platform default.",
                new ComponentOrientationExample()));

        examples.add(
            new Example(
                "Building - Button Bars",
                "Demonstrates how to build button bars " +
                "that comply with style guides.\n\n" +
                "The ButtonBarBuilder ensures minimum widths, " +
                "uses logical gaps for related and unrelated buttons, " +
                "and allows to add gridded or ungridded buttons " +
                "- with default or narrow margins.",
                new ButtonBarsExample()));

        examples.add(
            new Example(
                "Building - Button Stacks",
                "Demonstrates how to build button stacks " +
                "that comply with style guides.\n\n" +
                "The ButtonStackBuilder ensures minimum widths, " +
                "uses logical gaps for related and unrelated buttons, " +
                "and allows to add gridded or ungridded buttons " +
                "- with default or narrow margins.",
                new ButtonStacksExample()));

        examples.add(
            new Example(
                "Building - Button Order",
                "Demonstrates how to build button bars " +
                "with a fixed button order or with a button order " +
                "that honors the platform's style.",
                new ButtonOrderExample()));


        // Factories ----------------------------------------------------------

        examples.add(
            new Example(
                "Factories - Button Bars",
                "Presents the button bars as provided by the ButtonBarFactory.",
                new ButtonBarFactoryExample()));

/*

       examples.add(
            new Example(
                "Factories - Wizard Bars",
                "Presents the different wizard bars as provided by the WizardBarFactory.",
                "com.jgoodies.forms.tutorial.factories.WizardBarFactoryExample"));

*/

        // Pitfalls ----------------------------------------------------------

        examples.add(
            new Example(
                "Pitfalls - Vertical Growth",
                "Demonstrates a frequent pitfall when specifying a growing row.\n" +
                "In this layout a row grows, but the text area in that row\n"  +
                "is centered and doesn't 'grow'. In other words, the area\n" +
                "doesn't fill the available vertical space.",
                new VerticalGrowthExample()));

        examples.add(
            new Example(
                "Pitfalls - Growing Text Area",
                "Demonstrates a pitfall for JTextAreas in resizable containers\n" +
                "that are not wrapped by a JScrollPane.\n" +
                "If configured correctly , the area grows and shrinks,\n" +
                "if not it'll grow but never shrink again.",
                new GrowingTextAreaExample()));


        return Collections.unmodifiableList(examples);
    }

}
