/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.forms.demo.view;

import java.awt.Color;
import java.awt.Font;

import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;

import com.jgoodies.animation.Animation;
import com.jgoodies.animation.Animations;
import com.jgoodies.animation.animations.BasicTextAnimation;
import com.jgoodies.animation.animations.BasicTextAnimations;
import com.jgoodies.animation.components.BasicTextLabel;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.demo.model.MainModel;
import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.looks.LookUtils;
import com.jgoodies.uif.AbstractView;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.component.UIFButton;
import com.jgoodies.uif.util.Modes;
import com.jgoodies.uif.util.SystemUtils;

/**
 * The view that displays the Forms Demo's intro animation.
 *
 * @author Karsten Lentzsch
 */

final class IntroPageView extends AbstractView {

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(IntroPageView.class);

    /**
     * The presentation model that provides the skip Action.
     */
    private final MainModel model;

    private JLabel         gridLabel;
    private BasicTextLabel label1;
    private BasicTextLabel label2;
    private UIFButton      skipButton;
    private Animation      animation;


    // Instance Creation ******************************************************

    IntroPageView(MainModel model) {
        this.model = model;
    }


    // API ********************************************************************

    /**
     * Returns the intro animation.
     */
    Animation animation() {
        return animation;
    }


    // Building ***************************************************************

    /**
     * Creates and configures the UI components.
     */
    private void initComponents() {
        ImageIcon icon = RESOURCES.getImageIcon("intro.image");
        gridLabel = new JLabel(icon);

        Font font = getAnimationFont();
        label1 = new BasicTextLabel(" ");
        label1.setFont(font);
        label1.setBounds(0, 0, 350, 100);
        label1.setOpaque(false);

        label2 = new BasicTextLabel(" ");
        label2.setFont(font);
        label2.setBounds(0, 0, 350, 100);
        label2.setOpaque(false);

        Action skipAction = Application.getAction(
                model, MainModel.ACTION_SKIP_INTRO);
        skipButton = new UIFButton(skipAction);
        skipButton.setFocusPainted(false);
        skipButton.setOpaqueMode(Modes.LAF_NON_AQUA);
    }


    /**
     * Builds this panel with the horizontal <code>JSplitPane</code> in the center,
     * and a status bar in the south.
     */
    @Override
    protected JComponent buildPanel() {
        initComponents();

        // I cannot use Borders.DLU7, just because
        // horizontal separators span the whole page.
        FormLayout layout = new FormLayout(
                "15px:grow, fill:[200dlu,pref], 0:grow, 15px",
                "15px:grow, fill:[30dlu,pref], 30px:grow, 15px");
        PanelBuilder builder = new PanelBuilder(layout);
        builder.setBackground(Color.WHITE);

        CellConstraints cc = new CellConstraints();
        builder.add(label1,           cc.xy (2, 2));
        builder.add(label2,           cc.xy (2, 2));
        builder.add(gridLabel,        cc.xy (2, 2));
        builder.add(buildButtonBar(), cc.xyw(2, 3, 2, "right, bottom"));

        animation = createAnimation();

        return builder.getPanel();
    }

    private JComponent buildButtonBar() {
        JComponent bar = ButtonBarFactory.buildRightAlignedBar(skipButton);
        bar.setOpaque(false);
        return bar;
    }


    // Animation Creation *****************************************************

    /**
     * Creates and returns a composed animation for the intro.
     */
    private Animation createAnimation() {
        Animation welcome =
            BasicTextAnimation.defaultFade(
                label1,
                2500,
                "Welcome To",
                Color.DARK_GRAY);

        Animation theJGoodiesForms =
            BasicTextAnimation.defaultFade(
                label1,
                3000,
                "The JGoodies Forms",
                Color.DARK_GRAY);

        Animation makesSimpleThingsEasy =
            BasicTextAnimations.defaultFade(
                label1,
                label2,
                4000,
                -100,
                "Forms makes simple things easy\u2026|" +
                "\u2026 and the hard stuff possible.",
                Color.DARK_GRAY);

        Animation makesGoodDesignEasy =
            BasicTextAnimations.defaultFade(
                label1,
                label2,
                4000,
                -500,
                "It makes good design easy\u2026|" +
                "\u2026 and the bad difficult.",
                Color.DARK_GRAY);

        Animation a5 =
            BasicTextAnimations.defaultFade(
                label1,
                label2,
                4000,
                500,
                "Main Benefits:",
                Color.DARK_GRAY);

        Animation a6 =
            BasicTextAnimations.defaultFade(
                label1,
                label2,
                3000,
                500,
                "Powerful, flexible and precise Layout|" +
                "Easy to work with|Quite easy to understand|" +
                "Faster UI Production|" +
                "Better UI code readability|" +
                "Forces style guide compliance",
                Color.DARK_GRAY);

        Animation all =
            Animations.sequential(
                new Animation[] {
                    Animations.pause(1000),
                    welcome,
                    Animations.pause(1000),
                    theJGoodiesForms,
                    Animations.pause(1000),
                    makesSimpleThingsEasy,
                    Animations.pause(1000),
                    makesGoodDesignEasy,
                    Animations.pause(1000),
                    a5,
                    a6,
                    Animations.pause(1500),
                    });

        return all;
    }

    // Helper Code **********************************************************

    /**
     * Looks up and answer a <code>Font</code> used for the animations.
     * Tries to get a bold <i>Tahoma</i> on Windows, and uses
     * <i>MetaBold</i> on other platforms.
     *
     * @return the font used for the animation
     */
    private Font getAnimationFont() {
        Font font = null;
        // Try to get Tahoma on Windows.
        if (SystemUtils.IS_OS_WINDOWS) {
            int fontSize = LookUtils.IS_LOW_RESOLUTION ? 18 : 22;
            font = new Font("Tahoma", Font.BOLD, fontSize);
        }
        // Use the logical dialog font as a fall back.
        if (font == null) {
            int fontSize = LookUtils.IS_LOW_RESOLUTION ? 19 : 24;
            font = new Font("dialog", Font.BOLD, fontSize);
        }
        return font;
    }

}