/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.forms.demo.view;

import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JComponent;

import com.jgoodies.animation.Animation;
import com.jgoodies.animation.AnimationUtils;
import com.jgoodies.animation.Animator;
import com.jgoodies.forms.demo.model.MainModel;
import com.jgoodies.forms.demo.model.MainModel.ViewMode;
import com.jgoodies.uif.AbstractFrame;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ApplicationContext;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.panel.CardPanel;

/**
 * The main frame of the JGoodies Forms Demo that builds
 * the menu bar, menus, intro, main page, and its subviews.
 *
 * @author Karsten Lentzsch
 */
public final class MainFrame extends AbstractFrame {

    private static final Dimension MINIMUM_SIZE = new Dimension(525, 425);

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(MainFrame.class);

    private final MainModel model;

    private MenuBarView   menuBarView;
    private IntroPageView introPageView;
    private MainPageView  mainPageView;
    private CardPanel pages;


    // Instance Creation ****************************************************

    /**
     * Constructs the main frame.
     */
    public MainFrame(MainModel model) {
        super(RESOURCES.getString("application.title"));
        setName("main");
        this.model = model;
    }


    // Building *************************************************************

    private void initComponents() {
        menuBarView   = new MenuBarView(model);
        introPageView = new IntroPageView(model);
        mainPageView  = new MainPageView(model);
    }


    @Override
    protected void initEventHandling() {
        super.initEventHandling();
        model.addPropertyChangeListener(
                MainModel.PROPERTY_VIEW_MODE,
                new ViewModeChangeHandler());
    }


    /**
     * Builds the content pane, starts the intro page animation
     * and registers an action for the animation stop event.
     */
    @Override
    protected JComponent buildContentPane() {
        initComponents();

        pages = new CardPanel();
        pages.add(introPageView.getPanel(), MainModel.ViewMode.INTRO.name());
        pages.add(mainPageView.getPanel(),  MainModel.ViewMode.MAIN.name());

        Animation intro = introPageView.animation();
        AnimationUtils.invokeOnStop(intro, model.getSkipRunnable());
        new Animator(introPageView.animation(), 25).start();

        return pages;
    }


    // Implementing Abstract Behavior ****************************************

    /**
     * Returns the frame's minimum size. It is used by the WindowUtils
     * to resize the window if the user has shrinked the window below
     * this given size.
     *
     * @return the frame's minimum size
     */
    public Dimension getWindowMinimumSize() {
        return MINIMUM_SIZE;
    }


    @Override
    protected void configureCloseOperation() {
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(
                ApplicationContext.getInstance().getApplication().getApplicationExitOnWindowClosingHandler());
    }


    // Helper Classes *********************************************************

    /**
     * Updates the card and sets or removes the menu bar.
     */
    private final class ViewModeChangeHandler implements PropertyChangeListener {

        public void propertyChange(PropertyChangeEvent evt) {
            ViewMode viewMode = model.getViewMode();
            if (viewMode == ViewMode.MAIN) {
                setJMenuBar(menuBarView.getMenuBar());
            } else {
                setJMenuBar(null);
            }
            pages.showCard(viewMode.name());
        }
    }

}