/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.chart;

import java.awt.*;
import java.text.Format;

import javax.swing.JComponent;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.plaf.UIResource;

/**
 * The abstract superclass of all chart classes; provides
 * caching behavior and code common to all subclasses.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.9 $
 *
 * @param <T>  the type of this chart's model data
 */
public abstract class AbstractChart<T> extends JComponent {

    // Names of the Bound Bean Properties *************************************

    public static final String PROPERTYNAME_MODEL     = "model";
    public static final String PROPERTYNAME_FORMAT    = "format";
    public static final String PROPERTYNAME_SHOW_SIZE = "showSize";


    // Constants **************************************************************

    protected static final Color OUTER_SHADOW_COLOR = new Color(0, 0, 0, 25);
    protected static final Color INNER_SHADOW_COLOR = new Color(0, 0, 0, 42);


    // Instance Fields ********************************************************

    private final ChangeListener modelChangeHandler;
    protected ChartModel<T> model;
    private Format format;

    private boolean cacheIsValid = false;
    private boolean showSize = true;
    protected boolean printingDevice = false;


    // Instance Creation ******************************************************

    protected AbstractChart() {
        this(null);
    }


    protected AbstractChart(Format format) {
        this.format = format;
        this.modelChangeHandler = new ChartModelChangeHandler();
        setFocusable(false);
    }


    // Accessors **************************************************************

    public final ChartModel<T> getModel() {
        return model;
    }


    public final void setModel(ChartModel<T> newModel) {
        ChartModel<T> oldModel = getModel();
        if (oldModel != null) {
            oldModel.removeChangeListener(modelChangeHandler);
        }
        this.model = newModel;
        if (newModel != null) {
            newModel.addChangeListener(modelChangeHandler);
        }
        if (oldModel == newModel) {
            return;
        }
        firePropertyChange(PROPERTYNAME_MODEL, oldModel, newModel);
        invalidateCache();
        repaint();
    }


    public final Format getFormat() {
        return format;
    }


    public final void setFormat(Format newFormat) {
        Format oldFormat = getFormat();
        this.format = newFormat;
        firePropertyChange(PROPERTYNAME_FORMAT, oldFormat, newFormat);
        invalidateCache();
        repaint();
    }


    public final boolean getShowSize() {
        return showSize;
    }


    public final void setShowSize(boolean newValue) {
        boolean oldValue = getShowSize();
        showSize = newValue;
        firePropertyChange(PROPERTYNAME_SHOW_SIZE, oldValue, newValue);
        invalidateCache();
        repaint();
    }


    /**
     * Returns the <code>ChartData</code> at the given position.
     */
    public final T getDataAt(Point position) {
        int index = getIndexAt(position);
        return -1 == index ? null : model.getData(index);
    }


    // Abstract Behavior ****************************************************

    /**
     * Returns the index of the chart data for the given position.
     */
    protected abstract int getIndexAt(Point position);


    /**
     * Renders the chart on the specified <code>Graphics2D</code>.
     */
    protected abstract void render(Graphics2D g);


    protected abstract void validateCache();


    // Helper Code ************************************************************

    /**
     * Formats the chart data's value and returns the formatted string.
     * Subclasses will typically override this.
     */
    protected final String formatSize(long value) {
        return format != null
                ? format.format(Long.valueOf(value))
                : String.valueOf(value);
    }


    protected final boolean hasEmptyModel() {
        return (model == null) || (model.getItemCount() == 0);
    }


    // Caching ****************************************************************

    protected final void invalidateCache() {
        cacheIsValid = false;
    }


    protected final boolean cacheIsValid() {
        return cacheIsValid;
    }


    protected final void ensureValidCache() {
        if (!cacheIsValid()) {
            validateCache();
            cacheIsValid = true;
        }
    }


    // Painting and Printing **************************************************

    @Override
    public void paint(Graphics g) {
        g.setFont(getFont());
        paint0(g);
    }


    private void paint0(Graphics g) {
        if (hasEmptyModel()) {
            renderNoData((Graphics2D) g);
            return;
        }
        ensureValidCache();

        render((Graphics2D) g);
    }


    @Override
    public void print(Graphics g) {
        printingDevice = true;

        Font storedFont = g.getFont();
        g.setFont(new Font("dialog", Font.PLAIN, storedFont.getSize()));
        paint0(g);
        g.setFont(storedFont);

        printingDevice = false;
    }


    @Override
    public void updateUI() {
        super.updateUI();
        Font font = getFont();
        if (font instanceof UIResource) {
            setFont(UIManager.getFont("Table.font"));
        }
    }


    private void renderNoData(Graphics2D g2) {
        FontMetrics fm = g2.getFontMetrics();
        String str = "No Data";

        int x = (getSize().width - fm.stringWidth(str)) / 2;
        int y = (getSize().height + fm.getAscent()) / 2;

        g2.setColor(Color.BLACK);
        RenderingUtils.drawString(this, g2, str, x, y);
    }


    // Event Handling *********************************************************

    private final class ChartModelChangeHandler implements ChangeListener {

        public void stateChanged(ChangeEvent e) {
            invalidateCache();
            repaint();
        }
    }


}
