/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.chart;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Reduces the effort required to implement the ChartModel interface.
 * Implements the listener registration, de-registration and can fire
 * change events.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.5 $
 *
 * @param <T>  the type of the chart model data
 */
public abstract class AbstractChartModel<T> implements ChartModel<T> {

    private final List<ChangeListener> listenerList;


    // Instance Creation ******************************************************

    protected AbstractChartModel() {
        this.listenerList = new CopyOnWriteArrayList<ChangeListener>();
    }


    // Managing Listeners *****************************************************

    /**
     * Adds the given listener to the list of observers.
     *
     * @param listener    the listener to be added
     *
     * @throws NullPointerException if the listener is {@code null}
     */
    public final void addChangeListener(ChangeListener listener) {
        if (listener == null) {
            throw new NullPointerException("The listener to be added must not be null.");
        }
        listenerList.add(listener);
    }


    /**
     * Removes the given listener fromthe list of observers.
     * Does nothing if the listener is null.
     *
     * @param listener    the listener to be added
     */
    public final void removeChangeListener(ChangeListener listener) {
        if (listener != null) {
            listenerList.remove(listener);
        }
    }


    /**
     * Notifies all ChangeListeners registered with this model
     * that the state changed. The listeners need to look up
     * the changed data from this model.
     */
    public final void fireStateChanged() {
        ChangeEvent event = null;
        for (ChangeListener listener : listenerList) {
            if (event == null) {
                event = new ChangeEvent(this);
            }
            listener.stateChanged(event);
        }
    }


}
