/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.chart;

import java.awt.*;

import javax.swing.JComponent;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


/**
 * Paints a tree as inner pie chart with outer ring segments.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.7 $
 */
public abstract class AbstractTreeChart<T> extends JComponent {

    // Names for the bound properties *****************************************

    public static final String PROPERTYNAME_MODEL = "model";


    // Instance Fields ********************************************************

    private final ChangeListener modelChangeHandler;
    protected TreeChartModel<T> model;


    // Instance Creation ******************************************************

    public AbstractTreeChart(TreeChartModel<T> model) {
        this.modelChangeHandler = new TreeChartModelChangeHandler();
        setModel(model);
        setFocusable(false);
    }


    // Accessors **************************************************************

    public final TreeChartModel<T> getModel() {
        return model;
    }

    public final void setModel(TreeChartModel<T> newModel) {
        TreeChartModel<T> oldModel = getModel();
        if (oldModel != null) {
            oldModel.removeChangeListener(modelChangeHandler);
        }
        this.model = newModel;
        if (newModel != null) {
            newModel.addChangeListener(modelChangeHandler);
        }
        if (oldModel == newModel) {
            return;
        }
        firePropertyChange(PROPERTYNAME_MODEL, oldModel, newModel);
        invalidateCache();
        repaint();
    }


    // Abstract Behavior ******************************************************

    protected abstract void render(Graphics2D g2, T root);


    // Helper Code ************************************************************

    private boolean hasEmptyModel() {
        return (model == null) || (model.getChildCount(model.getRoot()) == 0);
    }


    private void invalidateCache() {
        // TODO invalidate a cache
    }

    private void ensureValidCache() {
        // TODO ensure a valid cache
    }

    protected final int[] computePartitions(T node, int totalWidth) {
        long totalSize = model.getSize(node);
        int elements = model.getChildCount(node);
        int[] result = new int[elements];
        if (elements == 1) {
            result[0] = totalWidth;
            return result;
        }
        double lastRoundingRest = 0;
        int usedWidth = 0;
        for (int i=0; i < elements; i++) {
            double size = model.getSize(model.getChild(node, i));
            double fraction = totalSize == 0 ? 1d : size / totalSize;
            double width = fraction * totalWidth;
            int roundedWidth = (int) Math.round(width + lastRoundingRest);
            double roundingRest = width - roundedWidth;
            lastRoundingRest = roundingRest;
            //System.out.println("i=" + i + "; fraction=" + fraction + "; width=" + width + "; roundingRest=" + roundingRest + "; usedWidth=" + usedWidth);
            result[i] = (i < elements - 1)
                ? roundedWidth
                : totalWidth - usedWidth;
            usedWidth += roundedWidth;
        }
        return result;
    }


    // Painting ***************************************************************

    @Override
    public final void paint(Graphics g) {
        g.setFont(getFont());
        paint0((Graphics2D) g);
    }


    private void paint0(Graphics2D g2) {
        if (hasEmptyModel()) {
            renderNoData(g2);
            return;
        }
        ensureValidCache();

//        g2.setRenderingHint(
//            RenderingHints.KEY_RENDERING,
//            RenderingHints.VALUE_RENDER_QUALITY);
//        g2.setRenderingHint(
//            RenderingHints.KEY_FRACTIONALMETRICS,
//            RenderingHints.VALUE_FRACTIONALMETRICS_ON);

        Object newAAHint = RenderingHints.VALUE_ANTIALIAS_ON;
        Object oldAAHint = g2.getRenderingHint(RenderingHints.KEY_ANTIALIASING);
        if (newAAHint != oldAAHint) {
            g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, newAAHint);
        } else {
            oldAAHint = null;
        }
        render(g2, model.getRoot());
        if (oldAAHint != null) {
            g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, oldAAHint);
        }
    }


    private void renderNoData(Graphics2D g2) {
        FontMetrics fm = g2.getFontMetrics();
        String str = "No Tree Data";

        int x = (getSize().width - fm.stringWidth(str)) / 2;
        int y = (getSize().height + fm.getAscent()) / 2;

        g2.setColor(Color.BLACK);
        g2.drawString(str, x, y);
    }


    // Event Handling *********************************************************

    private final class TreeChartModelChangeHandler implements ChangeListener {

        public void stateChanged(ChangeEvent e) {
            invalidateCache();
            repaint();
        }
    }


}
