/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.chart;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.text.Format;

import javax.swing.Icon;
import javax.swing.UIManager;

/**
 * Renders chart data in horizontal lines; from left to right it paints:
 * <ol>
 * <li> an optional icon, which is requested by #getIcon()
 * <li> the data's label, aligned left or right
 * <li> a colored bar with a size label in or besides it.
 * </ol>
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.6 $
 */
public final class BarChart<T> extends AbstractChart<T> {

    // Names of the Bound Bean Properties *************************************

    public static final String PROPERTYNAME_LEFT_ALIGN_TEXT = "leftAlignText";


    // Constants **************************************************************

    private static final int   MINIMUM_TEXT_ICON_WIDTH = 40;
    private static final int   MINIMUM_BAR_WIDTH = 60;
    private static final int   ICON_TEXT_PAD = 4;
    private static final int   TEXT_BAR_PAD = 8;
    private static final int   SIZE_LABEL_PAD = 2;
    private static final int   SHADOW_WIDTH = 3;
    private static final float LINE_TO_BAR_HEIGHT = 0.85f;

    // Bound Property
    private boolean leftAlignText;

    // Cached data
    private long maxSize;
    private int maxIconWidth;

    // Rendering data
    private int[] barY;
    private int[] barWidths;
    private int textWidth;
    private int lineHeight;
    private int barHeight;


    // Instance Creation ******************************************************

    public BarChart(Format format) {
        this(format, true);
    }


    public BarChart(Format format, boolean leftAlignText) {
        super(format);
        this.leftAlignText = leftAlignText;
    }


    // Accessors **************************************************************

    public boolean getLeftAlignText() {
        return leftAlignText;
    }


    public void setLeftAlignText(boolean newValue) {
        boolean oldValue = getLeftAlignText();
        leftAlignText = newValue;
        firePropertyChange(PROPERTYNAME_LEFT_ALIGN_TEXT, oldValue, newValue);
        invalidateCache();
        repaint();
    }


    // Implementing Abstract Behavior *****************************************

    /**
     * Returns the chart data index for the given position.
     * Checks for a valid cache.
     */
    @Override
    protected int getIndexAt(Point position) {
        if (hasEmptyModel() || !cacheIsValid()) return -1;

        int line = (int) position.getY() / lineHeight;
        return line < getModel().getItemCount() ? line : -1;
    }


    /**
     * Renders the chart to the given graphics object.
     */
    @Override
    protected void render(Graphics2D g2) {
        FontMetrics fm = g2.getFontMetrics();
        lineHeight = fm.getHeight() + 4;
        textWidth = computeTextWidth(fm);

        int xStart = textWidth + TEXT_BAR_PAD;
        int xEnd = getWidth() - 1;
        int yStart = 1;

        computeTextAndBarWidths(xStart, xEnd, yStart);

        renderAllLines(g2, xStart, xEnd, fm);
    }


    /**
     * Validates the cache.
     */
    @Override
    protected void validateCache() {
        maxSize = 0;

        int itemCount = model.getItemCount();
        for (int index = 0; index < itemCount; index++) {
            long dataSize = model.getSize(index);
            if (dataSize > maxSize) maxSize = dataSize;
        }
        maxIconWidth = computeMaxIconWidth();
    }


    // Layout *****************************************************************

    /**
     * Computes and returns the maximum width of all icons.
     */
    private int computeMaxIconWidth() {
        int maxWidth = 0;
        int itemCount = model.getItemCount();
        for (int index = 0; index < itemCount; index++) {
            Icon icon = model.getIcon(index);
            if (icon != null) {
                int width = icon.getIconWidth();
                if (width > maxWidth) maxWidth = width;
            }
        }
        return 0 == maxWidth ? maxWidth : maxWidth + ICON_TEXT_PAD;
    }


    /**
     * Computes and returns the maximum width of texts using the given font metrics.
     */
    private int computeMaxTextWidth(FontMetrics fm) {
        int maxWidth = 0;
        int itemCount = model.getItemCount();
        for (int index = 0; index < itemCount; index++) {
            int width = fm.stringWidth(model.getText(index));
            if (width > maxWidth) maxWidth = width;
        }
        return maxWidth;
    }


    /**
     * Computes and returns the maximum width of texts and bars.
     */
    private void computeTextAndBarWidths(int xStart, int xEnd, int yStart) {
        int paneWidth = xEnd - xStart;
        int maxBarWidth = paneWidth - SHADOW_WIDTH;

        barHeight = Math.round(lineHeight * LINE_TO_BAR_HEIGHT);
        int barYOffset = yStart;

        int itemCount = model.getItemCount();
        barWidths = new int[itemCount];
        barY = new int[itemCount];

        for (int i = 0; i < itemCount; i++) {
            barY[i] = barYOffset + i * lineHeight;
            int plainWidth = 0 == maxSize
                ? 0
                : Math.round((float) maxBarWidth * model.getSize(i) / maxSize);
            barWidths[i] = Math.max(1, plainWidth);
        }
    }


    /**
     * Computes and returns the text width.
     */
    private int computeTextWidth(FontMetrics fm) {
        int freeWidth = getWidth() - MINIMUM_BAR_WIDTH - TEXT_BAR_PAD;
        int maxTextWidth = computeMaxTextWidth(fm);

        return Math.max(MINIMUM_TEXT_ICON_WIDTH,
                        Math.min(maxIconWidth + maxTextWidth, freeWidth));
    }


    // Rendering ************************************************************

    /**
     * Renders all lines to the given graphics object, using the specified layout data.
     */
    private void renderAllLines(Graphics2D g2, int xStart, int xEnd,
            FontMetrics fm) {
        final int textOffset = (barHeight + fm.getAscent()) / 2 - 1;
        Color foreground = UIManager.getColor("controlText");

        int itemCount = model.getItemCount();
        for (int i = 0; i < itemCount; i++) {
            int y = barY[i];
            int labelY = y + textOffset;
            int barWidth = barWidths[i];

            renderIcon (g2, model.getIcon(i), 0, y - 1);
            renderLabel(g2, fm, foreground, model.getText(i), xStart, labelY);
            renderBar  (g2, model.getColor(i), xStart, y, barWidth, barHeight);

            if (getShowSize()) {
                renderSize(g2, fm, foreground, model.getSize(i),
                        xStart, labelY, barWidth, xEnd - xStart);
            }
        }
    }


    /**
     * Renders a single icon using the specified layout data.
     */
    private void renderIcon(Graphics2D g2, Icon icon, int x, int y) {
        if (icon != null) {
            icon.paintIcon(this, g2,
                    x, y + (lineHeight - icon.getIconHeight()) / 2);
        }
    }


    /**
     * Renders a single label using the specified layout data.
     */
    private void renderLabel(Graphics2D g2, FontMetrics fm, Color foreground,
            String text, int x, int y) {
        final int textEnd = x - TEXT_BAR_PAD;
        String clippedText = ChartUtils.clipText(text, fm, textEnd - maxIconWidth);
        g2.setColor(foreground);
        int labelX = leftAlignText
            ? maxIconWidth
            : textEnd - fm.stringWidth(clippedText);
        RenderingUtils.drawString(this, g2, clippedText, labelX, y);
    }


    /**
     * Renders a single bar using the specified layout data.
     */
    private void renderBar(Graphics2D g2, Color background, int x, int y,
            int width, int height) {
        if (!printingDevice) renderShadow(g2, x, y, width, height);

        g2.setColor(background);
        g2.fillRect(x, y, width, height);
        g2.setColor(Color.DARK_GRAY);
        g2.drawRect(x, y, width, height);
    }


    /**
     * Renders a shadow for the bar, using transparency and translucency,
     * and avoids sharp edges.
     */
    private void renderShadow(Graphics2D g2, int x, int y, int width, int height) {
        g2.setColor(OUTER_SHADOW_COLOR);
        g2.fillRoundRect(x + 3, y + 3, width, height, 2, 2);
        g2.setColor(INNER_SHADOW_COLOR);
        g2.fillRoundRect(x + 2, y + 2, width, height, 2, 2);
    }


    /**
     * Renders the bar's size label using the specified layout data.
     */
    private void renderSize(Graphics2D g2, FontMetrics fm, Color foreground,
            long size, int x, int y, int barWidth, int width) {
        String formattedSize = formatSize(size);
        int sizeWidth = fm.stringWidth(formattedSize);
        int widthAndPad = sizeWidth + 2 * SIZE_LABEL_PAD;
        int widthBehindBar = width - barWidth;

        if (barWidth > widthAndPad) {
            // Paint the size centered inside the bar.
            int sizeX = (barWidth - sizeWidth) / 2;
            g2.setColor(Color.BLACK);
            RenderingUtils.drawString(this, g2, formattedSize, x + sizeX, y);
        } else if (widthBehindBar > widthAndPad) {
            // Paint the size behind the bar if there's enough space.
            int sizeX = barWidth + 4;
            g2.setColor(foreground);
            RenderingUtils.drawString(this, g2, formattedSize, x + sizeX, y);
        }
    }


}
