/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.chart;

import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Rectangle;

import javax.swing.JComponent;

/**
 * Models and renders the labels around pie charts; they are composed
 * of a textual label, and an optional size label, which will be displayed
 * if there's enough space.<p>
 *
 * The layout is done by the {@link PieChartLabels} class.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.7 $
 */
final class ChartLabel {
    private static final double DEGREE_TO_RADIAN = Math.PI / 180f;
    private static final int    LABEL_TO_PIE_GAP = 5;

    private final String  label;
    private final String  sizeStr;
    private final int     startAngle;
    private final int     stopAngle;
    private final boolean isLeftAligned;

    private Rectangle bounds;
    private boolean showSize;

    /**
     * Constructs a <code>ChartLabel</code> instance.
     */
    ChartLabel(
        String label,
        String sizeStr,
        int startAngle,
        int stopAngle,
        boolean isLeftAligned) {
        this.label = label;
        this.sizeStr = sizeStr;
        this.startAngle = startAngle;
        this.stopAngle = stopAngle;
        this.isLeftAligned = isLeftAligned;
    }

    /**
     * Computes and answers if the label has enough space to show the size.
     */
    private boolean computeShowSize(
        int groupSize,
        int radius,
        int labelAndSizeHeight) {
        if (null == sizeStr)
            return false;

        if (groupSize == 1)
            return true;

        if (stopAngle - startAngle > 30)
            return true;

        double radian1 = DEGREE_TO_RADIAN * startAngle;
        double radian2 = DEGREE_TO_RADIAN * stopAngle;
        float h1 = (float) Math.cos(radian1) * radius;
        float h2 = (float) Math.cos(radian2) * radius;
        float availableHeight = Math.abs(h1 - h2);
        int pad = 8;

        return labelAndSizeHeight + pad < availableHeight;
    }

    /**
     * Initializes this composed chart label and fills some caches.
     */
    void initializeFrom(
        FontMetrics fm,
        int width,
        int height,
        int radius,
        int groupSize) {
        int xCenter = width / 2;
        int yCenter = height / 2;

        int angle = (startAngle + stopAngle) / 2;
        double radian = DEGREE_TO_RADIAN * angle;
        float sin = (float) Math.sin(radian);
        float cos = (float) Math.cos(radian);

        double distance = radius + LABEL_TO_PIE_GAP;
        int x = xCenter + (int) (sin * distance);
        int y = yCenter - (int) (cos * distance);

        //int labelWidth = fm.stringWidth(label);
        int labelHeight = fm.getHeight();
        int labelAscent = fm.getAscent();
        //int totalWidth;
        int totalHeight;

        showSize = computeShowSize(groupSize, radius, 2 * labelAscent);

        if (showSize) {
            //totalWidth  = Math.max(labelWidth, fm.stringWidth(sizeStr));
            totalHeight = 2 * labelAscent;
        } else {
            //totalWidth  = labelWidth;
            totalHeight = labelHeight;
        }

        int availableWidth;
        if (x < xCenter) {
            availableWidth = x;
            x = Math.max(0, x - xCenter);
        } else {
            availableWidth = width - x - 1;
        }
        if (y < yCenter - radius * 0.9f) {
            y -= totalHeight;
        } else if (y < yCenter + radius * 0.9f) {
            y -= totalHeight / 2;
            int shift = Math.round(Math.abs(cos) * totalHeight / 2);
            availableWidth -= shift;
            if (x > xCenter)
                x += shift;
        } else
            y += 0;

        bounds = new Rectangle(x, y, availableWidth, totalHeight);
    }

    /**
     * Paints the composed label to the given graphics object.
     */
    void paint(JComponent c, Graphics g) {
        //g.drawRect(bounds.x, bounds.y, bounds.width, bounds.height);
        FontMetrics fm = g.getFontMetrics();
        String clippedText = ChartUtils.clipText(label, fm, bounds.width);
        int labelX =
            isLeftAligned ? 0 : bounds.width - fm.stringWidth(clippedText);
        int ascent = fm.getAscent();
        g.translate(bounds.x, bounds.y);

        RenderingUtils.drawString(c, g, clippedText, labelX, ascent);
        if (showSize) {
            String clippedSizeStr = ChartUtils.clipText(sizeStr, fm, bounds.width);
            int sizeX =
                isLeftAligned
                    ? 0
                    : bounds.width - fm.stringWidth(clippedSizeStr);
            RenderingUtils.drawString(c, g, clippedSizeStr, sizeX, ascent + 1 + ascent);
        }

        g.translate(-bounds.x, -bounds.y);
    }
}