/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.chart;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;

import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.EventListenerList;

/**
 * A chart container that provides convenience code and handles events and
 * changes the cursor.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.7 $
 */
public final class ChartView<T> extends JPanel {

    private final AbstractChart<T> chart;

    private final EventListenerList listenerList = new EventListenerList();
    private ChangeEvent changeEvent;
    private T selection;

    /**
     * Constructs a ChartView for the given chart.
     *
     * @param chart   the underlying chart
     */
    public ChartView(AbstractChart<T> chart) {
        super(null);
        this.chart = chart;
        buildContent();
    }

    // Building *************************************************************

    /**
     * Builds the content of the composed chart view.
     */
    protected void buildContent() {
        setLayout(new BorderLayout());
        chart.addMouseListener(createChartSelectionAdapter());
        add(chart, BorderLayout.CENTER);
        setOpaque(false);
        setCursor(Cursor.getDefaultCursor());
    }


    // Event Handling *******************************************************

    /**
     * Adds a ChangeListener to the chart.
     */
    public void addChangeListener(ChangeListener l) {
        listenerList.add(ChangeListener.class, l);
    }

    /**
     * Removes a ChangeListener from the chart.
     */
    public void removeChangeListener(ChangeListener l) {
        listenerList.remove(ChangeListener.class, l);
    }

    /**
     * Adds the default mouse motion adapter.
     */
    public void addMotionAdapters() {
        chart.addMouseMotionListener(createChartMotionAdapter());
    }


    /**
     * Creates and answers the chart motion adapter that changes the cursor.
     */
    private MouseMotionAdapter createChartMotionAdapter() {
        return new MouseMotionAdapter() {
            @Override
            public void mouseMoved(MouseEvent e) {
                setChartCursor(e.getPoint());
            }
        };
    }

    /**
     * Creates and answers the mouse adapter that sets the selected value and
     * changes the cursor.
     */
    private MouseAdapter createChartSelectionAdapter() {
        return new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent e) {
                if (   !SwingUtilities.isLeftMouseButton(e)
                    || e.isPopupTrigger()
                    || e.isConsumed())
                    return;

                AbstractChart<T> aChart = (AbstractChart<T>) e.getSource();
                Point p = e.getPoint();
                T data = aChart.getDataAt(p);
                if (!isSelectable(data))
                    return;
                setSelectedValue(data);
                doSelect();
                setChartCursor(p);
            }
        };
    }

    /*
     * Notify all listeners that have registered interest for
     * notification on this event type.  The event instance
     * is lazily created using the parameters passed into
     * the fire method.
     * @see EventListenerList
     */
    protected void fireStateChanged() {
        // Guaranteed to return a non-null array
        Object[] listeners = listenerList.getListenerList();
        // Process the listeners last to first, notifying
        // those that are interested in this event
        for (int i = listeners.length - 2; i >= 0; i -= 2) {
            if (listeners[i] == ChangeListener.class) {
                // Lazily create the event:
                if (changeEvent == null)
                    changeEvent = new ChangeEvent(this);
                ((ChangeListener) listeners[i + 1]).stateChanged(changeEvent);
            }
        }
    }

    /**
     * Performs a select operation.
     */
    protected void doSelect() {
        chart.setCursor(Cursor.getDefaultCursor());
        fireStateChanged();
    }


    // Public Interface *****************************************************

    /**
     * Returns the chart.
     */
    public AbstractChart<T> getChart() {
        return chart;
    }

    /**
     * Returns the chart's current selection.
     */
    public T getSelectedValue() {
        return selection;
    }

    /**
     * Checks and answer if the specified <code>ChartData</code> can be selected.
     */
    public boolean isSelectable(T data) {
        return null != data;
    }

    /**
     * Sets the chart's cursor for the given position.
     */
    void setChartCursor(Point p) {
        T data = chart.getDataAt(p);
        int cursorType =
            isSelectable(data) ? Cursor.HAND_CURSOR : Cursor.DEFAULT_CURSOR;
        chart.setCursor(Cursor.getPredefinedCursor(cursorType));
    }

    /**
     * Sets a new selection, i.e. an instance of <code>ChartData</code>.
     */
    public void setSelectedValue(T newSelection) {
        selection = newSelection;
    }

    /**
     * Enables or disables the tool tips.
     */
    public void setToolTipEnabled(boolean enabled) {
        getChart().setToolTipText(enabled ? "" : null);
    }

}
