/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.chart;

import java.awt.Color;

import javax.swing.Icon;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * A wrapper for ChartModels that colorizes the uncolored wrapped items.
 * More precisely, if {@code m} is the wrapped ChartModel, and {@code w}
 * is this wrapper, then {@code w.getItemCount() == m.getItemCount()},
 * and for all indices {@code i}, {@code w} and {@code m} return the same
 * data, icon, text, and size. And {@code w.getColor(i) == m.getColor(i)},
 * iff {@code m.getColor(i) == null}.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.4 $
 *
 * @see ChartModel
 */
public final class ColoredChartModel<T> extends AbstractChartModel<T> {

    private final ChartModel<T> delegate;
    private Color[] colors;

    private int   colorCount;
    private float saturation;
    private float brightness;


    // Instance Creation ******************************************************

    /**
     * Constructs a wrapper ChartModel on the given model.
     * The wrapper has a color set for all indexes where
     * the wrapped model has a {@code null} color.
     *
     * @param chartModel    the chart model to colorize
     * @param colorCount    the number of colors to compute
     * @param saturation    the saturation of the computed colors
     * @param brightness    the brightness of the computed colors
     *
     * @throws NullPointerException if the chartModel is {@code null}
     */
    public ColoredChartModel(
            ChartModel<T> chartModel,
            int colorCount,
            float saturation,
            float brightness) {
        this.delegate = chartModel;
        this.colorCount = colorCount;
        this.saturation = saturation;
        this.brightness = brightness;
        colorize();
        delegate.addChangeListener(new DelegateChangeHandler());
    }


    // Accessors **************************************************************

    public int getColorCount() {
        return colorCount;
    }

    public void setColorCount(int newColorCount) {
        this.colorCount = newColorCount;
        changed();
    }


    public float getSaturation() {
        return saturation;
    }

    public void setSaturation(float newSaturation) {
        this.saturation = newSaturation;
        changed();
    }


    public float getBrightness() {
        return brightness;
    }

    public void setBrightness(float newBrightness) {
        this.brightness = newBrightness;
        changed();
    }


    // ChartModel Implementation **********************************************

    /**
     * {@inheritDoc}
     */
    public int getItemCount()        { return delegate.getItemCount(); }

    /**
     * {@inheritDoc}
     */
    public T getData(int index)      { return delegate.getData(index); }

    /**
     * {@inheritDoc}
     */
    public Icon getIcon(int index)   { return delegate.getIcon(index); }

    /**
     * {@inheritDoc}
     */
    public String getText(int index) { return delegate.getText(index); }

    /**
     * {@inheritDoc}
     */
    public long getSize(int index)   { return delegate.getSize(index); }

    /**
     * {@inheritDoc}
     *
     * Returns the delegate's color if non-{@code null},
     * the computed otherwise.
     */
    public Color getColor(int index) {
        Color color = delegate.getColor(index);
        return color != null ? color : colors[index];
    }


    // Colorization ***********************************************************

    private void colorize() {
        int itemCount = getItemCount();
        int colored = 0;
        colors = new Color[itemCount];
        for (int index = 0; index < itemCount; index++) {
            Color delegateColor = delegate.getColor(index);
            if (delegateColor == null) {
                colors[index] = color(colored++, colorCount, saturation, brightness);
            }
        }
    }

    private Color color(int index, int count, float sat, float bright) {
        return Color.getHSBColor(((float) index) / (count + 1), sat, bright);
    }


    // Events *****************************************************************

    private void changed() {
        colorize();
        fireStateChanged();
    }


    private final class DelegateChangeHandler implements ChangeListener {
        public void stateChanged(ChangeEvent e) {
            changed();
        }
    }

}
