/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.chart;

import java.awt.Color;
import java.util.Arrays;
import java.util.Comparator;

import javax.swing.Icon;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * A wrapper for ChartModels that limits the number of elements.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.7 $
 *
 * @see ChartModel
 */
public final class FilteredChartModel<T> extends AbstractChartModel<T> {

    private final ChartModel<T> delegate;

    private int itemLimit;
    private float minimumFraction;
    private String collapsedText;
    private Color collapsedColor;

    private int[] mappedIndices;
    private long collapsedSize;


    // Instance Creation ******************************************************

    /**
     * Constructs a wrapper ChartModel on the given model.
     * The wrapper has a limited number of items, and all items
     * have a size larger than the specified minimum sizes.
     * Cropped elements are collapsed to a new item with
     * the given {@code collapsedText} and {@code collapsedColor}.
     *
     * @param chartModel    the chart model to colorize
     * @param itemLimit     the maximum number of items in the wrapper
     * @param minimumFraction   the minimum size of items in the wrapper
     * @param collapsedText  the text of the collapsed items
     * @param collapsedColor the color of the collapsed items
     *
     * @throws NullPointerException if the chartModel is {@code null}
     */
    public FilteredChartModel(
            ChartModel<T> chartModel,
            int itemLimit,
            float minimumFraction,
            String collapsedText,
            Color collapsedColor) {
        assertValidItemLimit(itemLimit);
        assertValidMinimumFraction(minimumFraction);
        this.delegate = chartModel;
        this.itemLimit = itemLimit;
        this.minimumFraction = minimumFraction;
        this.collapsedText = collapsedText;
        this.collapsedColor = collapsedColor;
        filter();
        delegate.addChangeListener(new DelegateChangeHandler());
    }


    // Accessors **************************************************************

    public int getItemLimit() {
        return itemLimit;
    }

    public void setItemLimit(int newItemLimit) {
        assertValidItemLimit(newItemLimit);
        this.itemLimit = newItemLimit;
        changed();
    }


    public float getMinimumFraction() {
        return minimumFraction;
    }

    public void setMinimumFraction(float newMinimumFraction) {
        assertValidMinimumFraction(newMinimumFraction);
        this.minimumFraction = newMinimumFraction;
        changed();
    }


    public String getCollapsedText() {
        return collapsedText;
    }

    public void setCollapsedText(String newCollapsedText) {
        this.collapsedText = newCollapsedText;
        changed();
    }


    public Color getCollapsedColor() {
        return collapsedColor;
    }

    public void setCollapsedColor(Color newCollapsedColor) {
        this.collapsedColor = newCollapsedColor;
        changed();
    }


    // ChartModel Implementation **********************************************

    /**
     * {@inheritDoc}
     */
    public int getItemCount() {
        return isFiltering()
            ? itemLimit
            : delegate.getItemCount();
    }

    /**
     * {@inheritDoc}
     */
    public T getData(int index) {
        return delegate.getData(mappedIndex(index));
    }

    /**
     * {@inheritDoc}
     */
    public Icon getIcon(int index) {
        return isFiltering() && (index == itemLimit - 1)
            ? null
            : delegate.getIcon(mappedIndex(index));
    }

    /**
     * {@inheritDoc}
     */
    public String getText(int index) {
        return isFiltering() && (index == itemLimit - 1)
            ? getCollapsedText()
            : delegate.getText(mappedIndex(index));
    }

    /**
     * {@inheritDoc}
     */
    public long getSize(int index) {
        return isFiltering() && (index == itemLimit - 1)
            ? collapsedSize
            : delegate.getSize(mappedIndex(index));
    }

    /**
     * {@inheritDoc}
     */
    public Color getColor(int index) {
        return isFiltering() && (index == itemLimit - 1)
            ? collapsedColor
            : delegate.getColor(mappedIndex(index));
    }


    // Filtering **************************************************************

    private void filter() {
        mappedIndices = null;
        int delegateItemCount = delegate.getItemCount();
        if (delegateItemCount <= itemLimit) {
            return;
        }

        collapsedSize = 0;
        Integer[] sortedIndices = new Integer[delegateItemCount];
        for (int i = 0; i < delegateItemCount; i++) {
            sortedIndices[i] = Integer.valueOf(i);
        }
        Arrays.sort(sortedIndices, new IndexComparator(delegate));
        int[] reversedSorted = new int[delegateItemCount];
        for (int i = 0; i < delegateItemCount; i++) {
            reversedSorted[sortedIndices[i].intValue()] = i;
        }
        mappedIndices = new int[itemLimit -1 ];

        int collapsedCount = 0;
        int mappedIndexCount = 0;
        for (int index = 0; index < delegateItemCount; index++) {
            long size = delegate.getSize(index);
            int reversedSortedIndex = reversedSorted[index];
            if (reversedSortedIndex < itemLimit - 1) {
                mappedIndices[mappedIndexCount++] = index;
            } else {
                collapsedSize += size;
                collapsedCount++;
            }
        }
    }


    private boolean isFiltering() {
        return mappedIndices != null;
    }


    private int mappedIndex(int index) {
        if (!isFiltering()) {
            return index;
        }
        return index == itemLimit - 1
            ? index
            : mappedIndices[index];
    }


    // Helper Code ************************************************************

    private void assertValidItemLimit(int limit) {
        if (limit < 3) {
            throw new IllegalArgumentException("The item limit must be >= 2.");
        }
    }

    private void assertValidMinimumFraction(float fraction) {
        if (fraction < 0.00001f) {
            throw new IllegalArgumentException("The minimum fraction must be >= 0.001%");
        }
    }


    // Events *****************************************************************

    private void changed() {
        filter();
        fireStateChanged();
    }


    private final class DelegateChangeHandler implements ChangeListener {
        public void stateChanged(ChangeEvent e) {
            changed();
        }
    }


    // Comparator *************************************************************

    private static final class IndexComparator implements Comparator<Integer> {

        private final ChartModel<?> source;

        IndexComparator(ChartModel<?> source) {
            this.source = source;
        }

        public int compare(Integer o1, Integer o2) {
            long size1 = source.getSize(o1.intValue());
            long size2 = source.getSize(o2.intValue());
            return size1 > size2
                 ? -1
                 : (size1 == size2 ? 0 : 1);
        }
    }

}
