/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.chart;

import java.awt.*;
import java.text.Format;
import java.text.NumberFormat;

/**
 * Renders a pie chart.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.7 $
 */
public final class PieChart<T> extends AbstractChart<T> {

    // Names of the Bound Bean Properties *************************************

    public static final String PROPERTYNAME_SHOW_TOTAL = "showTotal";
    public static final String PROPERTYNAME_TOTAL_TEXT = "totalText";

    // Constants **************************************************************

    private static final int   MINIMUM_RADIUS = 50;
    private static final int   PREFERRED_TEXT_WIDTH = 80;
    private static final int   PREFERRED_TEXT_HEIGHT = 40;
    private static final float RADIUS_TEXT_GROWTH_WEIGHT = 0.94f;

    private static final double RADIAN_TO_DEGREE = 180f / Math.PI;
    private static final double EPSILON = 1E-7;

    private static final NumberFormat PERCENT_FORMATTER = createPercentFormatter();

    // Bound Properties
    private boolean showTotal;
    private String totalText;

    // Model related data
    private long sizeTotal;
    private String[] percentString;
    private int[] angle;
    private PieChartLabels chartLabels;

    // Bounds related data
    private int xCenter;
    private int yCenter;
    private int radius;


    // Instance Creation ******************************************************

    public PieChart(Format format, String totalText) {
        super(format);
        this.totalText = totalText;
        this.showTotal = true;
    }


    // Accessors **************************************************************

    public boolean getShowTotal() {
        return showTotal;
    }


    public void setShowTotal(boolean newValue) {
        boolean oldValue = getShowTotal();
        showTotal = newValue;
        firePropertyChange(PROPERTYNAME_SHOW_TOTAL, oldValue, newValue);
        invalidateCache();
        repaint();
    }


    public String getTotalText() {
        return totalText;
    }


    public void setTotalText(String newText) {
        if (newText == null) {
            throw new NullPointerException("The total text must not be null.");
        }
        this.totalText = newText;
    }


    // Implementing Abstract Behavior *****************************************

    /**
     * Returns the chart data index for the given position.
     * Checks for a valid cache.
     */
    @Override
    public int getIndexAt(Point position) {
        if (hasEmptyModel()) return -1;
        ensureValidCache();

        int positionAngle = getAngleAt(position);
        if (positionAngle == -1) return -1;

        int itemCount = model.getItemCount();
        for (int index = 1; index < itemCount; index++) {
            if (positionAngle < angle[index]) return index - 1;
        }
        return itemCount - 1;
    }


    /**
     * Creates and answers the formatter that is used for rendering percent strings.
     */
    private static NumberFormat createPercentFormatter() {
        NumberFormat formatter = NumberFormat.getPercentInstance();
        formatter.setMaximumFractionDigits(1);
        formatter.setMinimumFractionDigits(1);
        formatter.setMinimumIntegerDigits(1);
        return formatter;
    }


    // Layout ***************************************************************

    /**
     * Sets the chart's bounds.
     */
    @Override
    public void setBounds(int x, int y, int width, int height) {
        super.setBounds(x, y, width, height);
        xCenter = width / 2;
        yCenter = height / 2;
        int xRadius = Math
                .round(((width / 2 - PREFERRED_TEXT_WIDTH) - MINIMUM_RADIUS)
                        * RADIUS_TEXT_GROWTH_WEIGHT);
        int yRadius = Math
                .round(((height / 2 - PREFERRED_TEXT_HEIGHT) - MINIMUM_RADIUS)
                        * RADIUS_TEXT_GROWTH_WEIGHT);
        radius = MINIMUM_RADIUS + Math.max(0, Math.min(xRadius, yRadius));
    }


    /**
     * Computes and answers the angle for the given position.
     */
    private int getAngleAt(Point position) {
        double dX = position.x - xCenter;
        double dY = yCenter - position.y;
        double r = Math.sqrt(dX * dX + dY * dY);

        // Return -1 if the position is out of the circle.
        if (r > radius) return -1;

        double theta = Math.atan2(dX, dY);
        int rawDegrees = (int) Math.round(theta * RADIAN_TO_DEGREE);
        int positiveDegrees = (rawDegrees + 360) % 360;
        return positiveDegrees;
    }


    // Rendering ************************************************************

    /**
     * Renders the pie chart to the given graphics object.
     */
    @Override
    protected void render(Graphics2D g2) {
        chartLabels.doLayout(g2.getFontMetrics(), getWidth(), getHeight(),
                radius);

        //        g2.setRenderingHint(
        //            RenderingHints.KEY_RENDERING,
        //            RenderingHints.VALUE_RENDER_QUALITY);
        //        g2.setRenderingHint(
        //            RenderingHints.KEY_FRACTIONALMETRICS,
        //            RenderingHints.VALUE_FRACTIONALMETRICS_ON);
        Object newAAHint = RenderingHints.VALUE_ANTIALIAS_ON;
        Object oldAAHint = g2.getRenderingHint(RenderingHints.KEY_ANTIALIASING);
        if (newAAHint != oldAAHint) {
            g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, newAAHint);
        } else {
            oldAAHint = null;
        }

        // Outer border
        int cXOrigin = xCenter - radius;
        int cYOrigin = yCenter - radius;
        int cWidth = 2 * radius - 1;
        if (!printingDevice) {
            renderShadow(g2, cXOrigin, cYOrigin, cWidth);
            //renderShadowFast(g2, cXOrigin, cYOrigin, cWidth);
        }

        int startAngle = 0;
        int stopAngle;
        int arcAngle;
        int itemCount = model.getItemCount();
        for (int pieNo = 0; pieNo < itemCount; pieNo++) {
            startAngle = angle[pieNo];
            stopAngle = (pieNo < itemCount - 1) ? angle[pieNo + 1] : 360;
            arcAngle = stopAngle - startAngle;

            // Arc
            g2.setColor(model.getColor(pieNo));
            renderArc(g2, startAngle, arcAngle);

            // Label and percent
            //paintPercent(g, percentString[pieNo], startAngle, arcAngle);
            //paintLabel(g, model[pieNo].getLabel(), startAngle, arcAngle);

            // Line
            g2.setColor(Color.GRAY);
            if (itemCount > 1) renderRadialLine(g2, startAngle);
        }
        g2.drawOval(cXOrigin, cYOrigin, cWidth, cWidth);

        if (oldAAHint != null) {
            g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, oldAAHint);
        }
        if (getShowTotal()) {
            renderTotal(g2);
        }
        chartLabels.paint(this, g2);
    }


    /**
     * Renders an arc to the given graphics using the specified start- and arc angles.
     */
    private void renderArc(Graphics2D g2, int startAngle, int arcAngle) {
        g2.fillArc(xCenter - radius, yCenter - radius, 2 * radius - 1,
                2 * radius - 1, -startAngle + 90, -arcAngle);
    }


    /**
     * Renders the line from center to the outline.
     */
    private void renderRadialLine(Graphics2D g2, int anAngle) {
        double radian = Math.toRadians(anAngle);
        int x = xCenter + (int) ((float) Math.sin(radian) * (radius - 0.5f));
        int y = yCenter - (int) ((float) Math.cos(radian) * (radius - 0.5f));
        g2.drawLine(xCenter - 1, yCenter - 1, x - 1, y - 1);
    }


    /**
     * Renders an aesthetic pie's shadow using translucency.
     */
    protected void renderShadow(Graphics2D g2, int cXOrigin, int cYOrigin,
            int cWidth) {
        g2.setColor(OUTER_SHADOW_COLOR);
        g2.fillOval(cXOrigin + 2, cYOrigin + 2, cWidth, cWidth);
        g2.setColor(INNER_SHADOW_COLOR);
        g2.fillOval(cXOrigin + 1, cYOrigin + 1, cWidth, cWidth);
    }


    /**
     * Renders a shadow using a faster but less aesthetic method - paints outlines only.
     */
    protected void renderShadowFast(Graphics2D g2, int cXOrigin, int cYOrigin,
            int cWidth) {
        g2.setColor(OUTER_SHADOW_COLOR);
        g2.drawOval(cXOrigin + 2, cYOrigin + 2, cWidth, cWidth);
        g2.setColor(INNER_SHADOW_COLOR);
        g2.drawOval(cXOrigin + 1, cYOrigin + 1, cWidth, cWidth);
    }


    /**
     * Renders the total label in the pie's center, if there's enough space.
     */
    private void renderTotal(Graphics2D g2) {
        FontMetrics fm = g2.getFontMetrics();
        String sizeStr = formatSize(sizeTotal);

        final int width1 = fm.stringWidth(totalText);
        final int width2 = fm.stringWidth(sizeStr);
        final int width3 = fm.stringWidth("123,1 MB");
        final int height1 = fm.getHeight();
        final int hpad = 6;
        final int vpad = 2;
        final int width = 2 * hpad + Math.max(width3, Math.max(width1, width2));
        final int height = 2 * vpad + height1 * 2;

        if (width > radius) return;

        final int x0 = xCenter - width / 2;
        final int y0 = yCenter - height / 2;

        g2.setColor(Color.WHITE);
        g2.fillRect(x0, y0, width, height);
        g2.setColor(Color.DARK_GRAY);
        g2.drawRect(x0, y0, width, height);

        g2.setColor(Color.BLACK);
        //        g2.setRenderingHint(
        //            RenderingHints.KEY_ANTIALIASING,
        //            RenderingHints.VALUE_ANTIALIAS_DEFAULT);
        RenderingUtils.drawString(this, g2, totalText, xCenter - width1 / 2,
                yCenter - vpad);
        RenderingUtils.drawString(this, g2, sizeStr, xCenter - width2 / 2,
                yCenter + height1 - vpad);
        //        g2.setRenderingHint(
        //            RenderingHints.KEY_ANTIALIASING,
        //            RenderingHints.VALUE_ANTIALIAS_ON);
    }


    // Caching **************************************************************

    /**
     * Validates the cache. In addition to the superclass behavior,
     * we compute size totals and the (stripped) strings and their angles.
     */
    @Override
    protected void validateCache() {
        sizeTotal = 0;
        int itemCount = model.getItemCount();
        for (int i = 0; i < itemCount; i++) {
            sizeTotal += model.getSize(i);
        }
        cacheStringsAndAngles();
    }


    /**
     * Computes and caches the stripped string labels and their angles.
     */
    private void cacheStringsAndAngles() {
        int itemCount = model.getItemCount();
        percentString = new String[itemCount];
        angle = new int[itemCount];
        chartLabels = new PieChartLabels();

        int startAngle = 0;
        for (int i = 0; i < itemCount; i++) {
            long size = model.getSize(i);
            float percent = sizeTotal < EPSILON
                    ? 1f / itemCount
                    : (float) size / sizeTotal;
            percentString[i] = PERCENT_FORMATTER.format(percent);
            int arcAngle = Math.round(360 * percent);
            int stopAngle = startAngle + arcAngle;
            angle[i] = startAngle;

            String formattedSize = getShowSize() ? formatSize(size) : null;
            chartLabels.add(model.getText(i), formattedSize, startAngle,
                    stopAngle);

            startAngle = stopAngle;
        }
    }

}