/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.chart;

import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.UIManager;

/**
 * Helper class that lays out the labels around a pie chart.
 * Therefore it keeps four lists that contain the labels for different sectors,
 * clockwise: north-east, south-east, south-west, and north-west.<p>
 *
 * A future version shall reduce the number of overlapping labels,
 * by moving them in case they overlap.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.4 $
 */
final class PieChartLabels {

    private static final int PAINT_LABEL_ARC_THRESHOLD = 10;

    private final List<ChartLabel> northEast;
    private final List<ChartLabel> southEast;
    private final List<ChartLabel> southWest;
    private final List<ChartLabel> northWest;


    // Instance Creation ******************************************************

    /**
     * Constructs a <code>PieChartLabels</code> instance.
     */
    PieChartLabels() {
        northEast = new LinkedList<ChartLabel>();
        southEast = new LinkedList<ChartLabel>();
        southWest = new LinkedList<ChartLabel>();
        northWest = new LinkedList<ChartLabel>();
    }


    // API ********************************************************************

    /**
     * Adds a string using the given start- and stop angles.
     */
    void add(String label, String sizeStr, int startAngle, int stopAngle) {
        int arcAngle = stopAngle - startAngle;
        // [Pending:] Use a bow threshold instead of arc threshold.
        if (arcAngle < PAINT_LABEL_ARC_THRESHOLD) return;

        int angle = (startAngle + stopAngle) / 2;
        ChartLabel cLabel = new ChartLabel(label, sizeStr, startAngle,
                stopAngle, angle <= 180);

        if (angle <= 90) {
            northEast.add(cLabel);
        } else if (angle <= 180) {
            southEast.add(cLabel);
        } else if (angle <= 270) {
            southWest.add(cLabel);
        } else
            northWest.add(cLabel);
    }


    /**
     * Starts the layout of all labels.
     */
    void doLayout(FontMetrics fm, int width, int height, int radius) {
        doLayout(northEast, fm, width, height, radius);
        doLayout(southEast, fm, width, height, radius);
        doLayout(southWest, fm, width, height, radius);
        doLayout(northWest, fm, width, height, radius);
    }


    /**
     * Lays out a set of labels for a given sector.
     */
    private void doLayout(List<ChartLabel> chartLabels, FontMetrics fm,
            int width, int height, int radius) {
        int groupSize = chartLabels.size();
        for (ChartLabel label : chartLabels) {
            label.initializeFrom(fm, width, height, radius, groupSize);
        }
    }


    /**
     * Paints all labels to the specified graphics object.
     */
    void paint(JComponent c, Graphics2D g2) {
        g2.setColor(UIManager.getColor("controlText"));

        paint(northEast, c, g2);
        paint(southEast, c, g2);
        paint(southWest, c, g2);
        paint(northWest, c, g2);
    }


    /**
     * Paints a list of labels to the specified graphics object.
     */
    private void paint(List<ChartLabel> chartLabels, JComponent c, Graphics2D g2) {
        for (ChartLabel label : chartLabels) {
            label.paint(c, g2);
        }
    }

}
