/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.chart;

import java.awt.Color;
import java.awt.Graphics2D;


/**
 * Paints a tree as inner pie chart with outer ring segments.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.9 $
 */
public final class TreePieChart<T> extends AbstractTreeChart<T> {

    // Constants **************************************************************

    //private static final double EPSILON = 1E-7;
    private static final int MAX_LEVEL = 5;
    private static final Color LEAF_COLOR = new Color(225, 225, 225);


    // Instance Fields ********************************************************

    // Bounds related data
    private int xCenter;
    private int yCenter;
    private int radius;


    // Instance Creation ******************************************************

    public TreePieChart(TreeChartModel<T> model) {
        super(model);
    }


    // Layout *****************************************************************

    /**
     * Sets the chart's bounds.
     */
    @Override
    public void setBounds(int x, int y, int width, int height) {
        super.setBounds(x, y, width, height);
        xCenter = width / 2;
        yCenter = height / 2;
        int xRadius = Math.round((width  - 10) / 2);
        int yRadius = Math.round((height - 10) / 2);
        radius = Math.max(0, Math.min(xRadius, yRadius));
    }



    // Painting ***************************************************************

    @Override
    protected void render(Graphics2D g2, T root) {
        int level = 0;
        int startAngle = 0;
        int stopAngle = 360;
        boolean allParentsSingletons = false;
        float startHue = 0f;
        float stopHue = 1f;
        float saturation = 0.5f;
        float brightness = 0.9f;
        int leafStartLevel = 0;
        int trailStartLevel = 0;
        render(g2, root, level, startAngle, stopAngle, allParentsSingletons,
                leafStartLevel, trailStartLevel,
                startHue, stopHue, saturation, brightness);
    }


    private void render(Graphics2D g2, T node, int level,
            int startAngle, int angle,
            boolean allParentsSingletons,
            int leadStartLevel,
            int trailStartLevel,
            float startHue, float stopHue,
            float saturation, float brightness) {
//        System.out.println();
//        System.out.print("Level=" + level);
//        if (node instanceof AbstractNode) {
//            AbstractNode aNode = (AbstractNode) node;
//            System.out.println("; name=" + aNode.getName());
//        }
//        System.out.println("StartAngle=" + startAngle + "; stopAngle=" + stopAngle);
//        System.out.println("StartHue=" + startHue + "; stopHue=" + stopHue);
        int childCount = model.getChildCount(node);
        if (level < MAX_LEVEL) {
            int[] angles = computePartitions(node, angle);
            int childStartAngle = startAngle;
            double totalHue = stopHue - startHue;
            for (int i = 0; i < childCount; i++) {
                boolean firstChild = i == 0;
                boolean lastChild = i == childCount - 1;
                int childAngle = angles[i];
                double nextChildStartHue = startHue + (totalHue * (i     / 14f));
                double nextChildStopHue  = startHue + (totalHue * ((i+1) / 14f));
                if (angle > 0) {
                    render(g2, model.getChild(node, i), level+1,
                            childStartAngle,
                            childAngle,
                            allParentsSingletons,
                            firstChild ? leadStartLevel  : level,
                            lastChild  ? trailStartLevel : level,
                            (float) nextChildStartHue, (float) nextChildStopHue,
                            saturation*0.8f,
                            brightness*1.02f);
                    }
                childStartAngle += childAngle;
            }
        }
        if (level > 0) {
            Color fillColor = model.isLeaf(node)
                ? LEAF_COLOR
                : Color.getHSBColor(startHue, saturation, brightness);
            g2.setColor(fillColor);
            fillArc(g2, startAngle, angle, level);
        }
        if ((level < MAX_LEVEL) && (childCount > 0) && (angle > 0d)) {
            // Draw my children's outer segment line, or a circle in the
            g2.setColor(Color.GRAY);
            drawArc(g2, startAngle, angle, level+1);
        }
        if (((angle != 0) && (angle != 360)) || (level == MAX_LEVEL)) {
            g2.setColor(Color.GRAY);
            int stopAngle = startAngle + angle;
            renderRadialLine(g2, startAngle, leadStartLevel, level);
            renderRadialLine(g2, stopAngle, trailStartLevel, level);
        }
    }


    /**
     * Renders an arc to the given graphics using the specified start- and arc angles.
     */
    private void drawArc(Graphics2D g2, int startAngle, int arcAngle, int level) {
        int arcRadius = (int) ((double) radius * level / MAX_LEVEL - 1);
        g2.drawArc(xCenter - arcRadius, yCenter - arcRadius,
                2 * arcRadius ,
                2 * arcRadius , -startAngle + 90, -arcAngle);
    }


    /**
     * Renders an arc to the given graphics using the specified start- and arc angles.
     */
    private void fillArc(Graphics2D g2, int startAngle, int arcAngle, int level) {
        int arcRadius = (int) ((double) radius * level / MAX_LEVEL - 1);
        g2.fillArc(xCenter - arcRadius, yCenter - arcRadius,
                2 * arcRadius,
                2 * arcRadius, -startAngle + 90, -arcAngle);
    }


    /**
     * Renders the line from center to the outline.
     */
    private void renderRadialLine(Graphics2D g2, int anAngle, int innerLevel, int outerLevel) {
        double radian = Math.toRadians(anAngle);
        double innerRadius = (double) radius * innerLevel / MAX_LEVEL - 1d;
        double outerRadius = (double) radius * outerLevel / MAX_LEVEL - 0.5d;
        int xInner = xCenter + (int) (Math.sin(radian) * (innerRadius - 0.5d));
        int yInner = yCenter - (int) (Math.cos(radian) * (innerRadius - 0.5d));
        int xOuter = xCenter + (int) (Math.sin(radian) * (outerRadius - 0.5d));
        int yOuter = yCenter - (int) (Math.cos(radian) * (outerRadius - 0.5d));
        g2.drawLine(xInner, yInner, xOuter, yOuter);
    }


}
