/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport;

import java.awt.Dimension;
import java.awt.KeyboardFocusManager;
import java.io.File;
import java.util.ResourceBundle;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.plaf.DimensionUIResource;

import com.jgoodies.jdiskreport.domain.Root;
import com.jgoodies.jdiskreport.domain.ScanConfiguration;
import com.jgoodies.jdiskreport.domain.ScanResult;
import com.jgoodies.jdiskreport.model.MainModel;
import com.jgoodies.jdiskreport.model.PersistencyManager;
import com.jgoodies.jdiskreport.model.node.RootNode;
import com.jgoodies.jdiskreport.model.scan.ConsoleScanLog;
import com.jgoodies.jdiskreport.model.scan.DepthFirstScanner;
import com.jgoodies.jdiskreport.model.scan.Scanner.ScanLog;
import com.jgoodies.jdiskreport.view.MainFrame;
import com.jgoodies.looks.LookUtils;
import com.jgoodies.looks.Options;
import com.jgoodies.looks.plastic.PlasticXPLookAndFeel;
import com.jgoodies.looks.windows.WindowsLookAndFeel;
import com.jgoodies.uif.application.ActionReader;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.osx.OSXApplicationMenu;
import com.jgoodies.uif.util.Modes;
import com.jgoodies.uif.util.UIFFocusTraversalPolicy;


/**
 * Starts JDiskReport and launches the GUI if necessary. First, looks up
 * the system properties for path, configuration, input and output files,
 * and decides whether a command line version is used or the GUI shall be shown.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.6 $
 */
public final class JDiskReport extends Application {


    private static Root root;


    public static void main(String[] args) {
        // TODO: Test with English
        // Locale.setDefault(Locale.ENGLISH);
        OSXApplicationMenu.setAboutName("JDiskReport");
        new JDiskReport().dispatch(args);
    }


    private void dispatch(String[] args) {
        if (args.length != 0) {
            printUsage();
            return;
        }
        String path = System.getProperty("path");
        String config = System.getProperty("config");
        String input = System.getProperty("input");
        String output = System.getProperty("output");
//        System.out.println("path="   + format(path));
//        System.out.println("config=" + format(config));
//        System.out.println("input="  + format(input));
//        System.out.println("output=" + format(output));
        boolean hasPath = path != null;
        boolean hasConfig = config != null;
        boolean hasPathOrConfig = hasPath || hasConfig;
        boolean hasInput = input != null;
        boolean hasOutput = output != null;
        // Path or Configuration ----------------------------
        if (hasPathOrConfig) {
            // Either path or configuration, not both
            if (hasPath && hasConfig) {
                printUsage();
                return;
            }
            // Must not mix path or configuration with input
            if (hasPathOrConfig && hasInput) {
                printUsage();
                return;
            }
            ScanConfiguration scanConfiguration = hasPath
                ? createConfiguration(path)
                : readConfiguration(config);
            if (scanConfiguration == null) {
                return;
            }
            if (hasOutput) {
                scanAndSaveReport(scanConfiguration, output);
            } else {
                scanAndShowGUI(scanConfiguration);
            }
            return;
        }
        if (hasInput) {
            // Must not have input and output at the same time
            if (hasOutput) {
                printUsage();
                return;
            }
            openReportAndShowGUI(input);
            return;
        }
        // No parameter -> Show GUI
        choosePathAndScanAndShowGUI();
    }


    private void choosePathAndScanAndShowGUI() {
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        int result = fileChooser.showDialog(null, "Scan");
        if (result == JFileChooser.CANCEL_OPTION) {
            return;
        }
        String scanPath = fileChooser.getSelectedFile().getAbsolutePath();
        if (scanPath == null) {
            return;
        }
        checkPathAndScanAndShowGUI(scanPath);
    }


    private void checkPathAndScanAndShowGUI(String scanTarget) {
        if (!validScanPath(scanTarget)) {
            return;
        }
        ScanConfiguration configuration = new ScanConfiguration(scanTarget);
        ScanLog log = new ConsoleScanLog();
        ScanResult result = new DepthFirstScanner().scan(configuration, log);
        showGUI(result.getRoot());
    }


    private void scanAndShowGUI(ScanConfiguration configuration) {
        ScanLog log = new ConsoleScanLog();
        ScanResult result = new DepthFirstScanner().scan(configuration, log);
        showGUI(result.getRoot());
    }


    private void scanAndSaveReport(ScanConfiguration configuration, String outputPath) {
        ScanLog log = new ConsoleScanLog();
        ScanResult result = new DepthFirstScanner().scan(configuration, log);
        new PersistencyManager().writeScanResult(new File(outputPath), result);
    }


    private ScanConfiguration createConfiguration(String scanTarget) {
        return validScanPath(scanTarget) ? new ScanConfiguration(scanTarget) : null;
    }


    private ScanConfiguration readConfiguration(String configPath) {
        if (!validInputFile(configPath)) {
            return null;
        }
        ScanConfiguration configuration = new PersistencyManager().readScanConfiguration(new File(configPath));
        return configuration;
    }


    private void openReportAndShowGUI(String inputPath) {
        File inputFile = new File(inputPath);
        if (!inputFile.exists()) {
            System.out.println("The input file " + inputFile + " does not exists.");
            return;
        }
        if (!inputFile.isFile()) {
            System.out.println("The input path " + inputFile + " is not a file.");
            return;
        }
        ScanResult result = new PersistencyManager().readScanResult(inputFile);
        showGUI(result.getRoot());
    }


    private void showGUI(Root aRoot) {
        root = aRoot;
        Application.launch(JDiskReport.class);
    }

    @Override
    protected void startup(String[] args) {
        RootNode node = RootNode.createFrom(root);
        printMemoryUsage();
        System.out.print("Computing statistics...");
        long t0 = System.currentTimeMillis();
        node.computeAndSetStatistics();
        long t1 = System.currentTimeMillis();
        System.out.println("done in " + duration(t0, t1));
        printMemoryUsage();
        System.out.println("Clearing cached files");
        node.clearCachedFiles();
        printMemoryUsage();

        configureUI();

        MainModel mainModel = new MainModel(node);
        JFrame frame = new MainFrame(mainModel);
        frame.setVisible(true);
    }


    // Launching the Application **********************************************

    /**
     * Configures the user interface.
     */
    private void configureUI() {
        Options.setDefaultIconSize(new Dimension(18, 18));
        Options.setPopupDropShadowEnabled(true);
        UIManager.put("ToolBar.separatorSize",  new DimensionUIResource(6, 18));
        ActionReader.setPlatformResourcesEnabled(true);

        try {
            if (Modes.OS_WINDOWS.enabled()) {
                UIManager.setLookAndFeel(new WindowsLookAndFeel());
            } else if (Modes.OS_NON_MAC.enabled()) {
                UIManager.setLookAndFeel(new PlasticXPLookAndFeel());
            }
        } catch (UnsupportedLookAndFeelException e) {
            e.printStackTrace();
        }

        System.setProperty("apple.laf.useScreenMenuBar", "true");

        //ExtUIManager.setup();
        UIManager.put("ClassLoader", LookUtils.class.getClassLoader());
        KeyboardFocusManager.getCurrentKeyboardFocusManager().
            setDefaultFocusTraversalPolicy(UIFFocusTraversalPolicy.DEFAULT);
    }


    // Helper Code ************************************************************

    private boolean validScanPath(String scanPath) {
        File scanFile = new File(scanPath);
        if (!scanFile.exists()) {
            System.out.println("File " + scanFile + " does not exist.");
            return false;
        }
        if (!scanFile.isDirectory()) {
            System.out.println(scanFile + " is not a directory.");
            return false;
        }
        return true;
    }


    private boolean validInputFile(String path) {
        File file = new File(path);
        if (!file.exists()) {
            System.out.println("File " + file + " does not exist.");
            return false;
        }
        if (file.isDirectory()) {
            System.out.println(file + " is a directory.");
            return false;
        }
        return true;
    }


//    private String format(String s) {
//        return s == null
//            ? "null"
//            : "'" + s + "'";
//    }


    private void printUsage() {
        ResourceBundle bundle =
            ResourceBundle.getBundle("com.jgoodies.jdiskreport.resources.JDiskReport");
        System.out.println(bundle.getString("Application.usage"));
    }


    private String duration(long start, long stop) {
        long millis = stop - start;
        return String.valueOf(millis / 1000L) + "s";
    }


    private void printMemoryUsage() {
        System.gc();
        long free = Runtime.getRuntime().freeMemory();
        long total = Runtime.getRuntime().totalMemory();
        long used = total - free;
        System.out.println("Total memory=" + (total / 1024L) + "KB");
        //System.out.println("Free memory =" + (free  / 1024L) + "KB");
        System.out.println("Used memory =" + (used  / 1024L) + "KB");
        System.out.flush();
    }

}
