/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport.domain;

import java.io.File;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import com.jgoodies.binding.beans.Model;



/**
 * Describes the target, inclusions, exclusions, and other options
 * for scanning a file tree.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.6 $
 */
public final class ScanConfiguration extends Model {

    private static final long serialVersionUID = 1L;

    // Names of the Bound Bean Properties *************************************

    public static final String PROPERTYNAME_TARGET       = "target";
    public static final String PROPERTYNAME_NAME         = "name";
    public static final String PROPERTYNAME_CHECK_CYLES  = "checkCycles";
    public static final String PROPERTYNAME_INCLUSIONS   = "inclusions";
    public static final String PROPERTYNAME_EXCLUSIONS   = "exclusions";
    public static final String PROPERTYNAME_MIN_MODIFIED = "minModified";
    public static final String PROPERTYNAME_MAX_MODIFIED = "maxModified";


    // Fields for Bound Properties ********************************************

    private File target;
    private String name;
    private boolean checkCycles;
    private List<String> inclusions;
    private List<String> exclusions;
    private long minModified;
    private long maxModified;
    //private long minFileSize;
    //private long maxFileSize;


    // Instance Creation ******************************************************

    /**
     * Constructs a scan configuration for a given path.
     */
    public ScanConfiguration(String targetPath) {
        this(new File(targetPath));
    }


    /**
     * Constructs a scan configuration for a given path.
     */
    public ScanConfiguration(File target) {
        if (target == null) {
            throw new NullPointerException("The target must not be null.");
        }
        this.target = target;
        this.name = target.getAbsolutePath();
        this.checkCycles = !SystemUtils.IS_OS_WINDOWS;
        this.inclusions = null;
        this.exclusions = null;
        this.minModified = -1;
        this.maxModified = -1;
    }


    public static ScanConfiguration from(Properties properties) {
        String targetPath = properties.getProperty(PROPERTYNAME_TARGET);
        ScanConfiguration configuration = new ScanConfiguration(targetPath);
        configuration.setName(properties.getProperty(PROPERTYNAME_NAME));
        configuration.setCheckCycles(
                Boolean.parseBoolean(
                        properties.getProperty(PROPERTYNAME_CHECK_CYLES,
                        SystemUtils.IS_OS_WINDOWS ? "false" : "true")));
        configuration.setInclusions(decode(properties.getProperty(PROPERTYNAME_INCLUSIONS)));
        configuration.setExclusions(decode(properties.getProperty(PROPERTYNAME_EXCLUSIONS)));
        configuration.setMinModified(
                parseTime(properties.getProperty(PROPERTYNAME_MIN_MODIFIED)));
        configuration.setMaxModified(
                parseTime(properties.getProperty(PROPERTYNAME_MAX_MODIFIED)));
        return configuration;
    }

    private static long parseTime(String time) {
        if (time == null || time.trim().length() == 0) {
            return -1;
        }
        return Long.parseLong(time.trim());
    }


    // Accessors **************************************************************

    public File getTarget() {
        return target;
    }

    public void setTarget(File newTarget) {
        if (newTarget == null) {
            throw new NullPointerException("The target must not be null.");
        }
        File oldTarget = getTarget();
        target = newTarget;
        firePropertyChange(PROPERTYNAME_TARGET, oldTarget, newTarget);
    }


    public String getName() {
        return name;
    }

    public void setName(String newValue) {
        Object oldValue = getName();
        name = newValue;
        firePropertyChange(PROPERTYNAME_NAME, oldValue, newValue);
    }



    public boolean getCheckCycles() {
        return checkCycles;
    }

    public void setCheckCycles(boolean newValue) {
        boolean oldValue = getCheckCycles();
        checkCycles = newValue;
        firePropertyChange(PROPERTYNAME_CHECK_CYLES, oldValue, newValue);
    }


    public List<String> getInclusions() {
        return inclusions;
    }


    public void setInclusions(List<String> newValue) {
        List<String> oldValue = getInclusions();
        inclusions = newValue;
        firePropertyChange(PROPERTYNAME_INCLUSIONS, oldValue, newValue);
    }


    public boolean hasInclusions() {
        return (inclusions != null) && !inclusions.isEmpty();
    }


    public List<String> getExclusions() {
        return exclusions;
    }

    public void setExclusions(List<String> newValue) {
        List<String> oldValue = getExclusions();
        exclusions = newValue;
        firePropertyChange(PROPERTYNAME_EXCLUSIONS, oldValue, newValue);
    }


    public boolean hasExclusions() {
        return (exclusions != null) && !exclusions.isEmpty();
    }


    public long getMinModified() {
        return minModified;
    }

    public void setMinModified(long newValue) {
        long oldValue = getMinModified();
        minModified = newValue;
        firePropertyChange(PROPERTYNAME_MIN_MODIFIED, oldValue, newValue);
    }


    public boolean hasMinModifiedDate() {
        return minModified != -1;
    }


    public long getMaxModified() {
        return maxModified;
    }

    public void setMaxModified(long newValue) {
        long oldValue = getMaxModified();
        maxModified = newValue;
        firePropertyChange(PROPERTYNAME_MAX_MODIFIED, oldValue, newValue);
    }


    public boolean hasMaxModifiedDate() {
        return maxModified != -1;
    }


    public boolean hasFilter() {
        return hasInclusions()
            || hasExclusions()
            || hasMinModifiedDate()
            || hasMaxModifiedDate();
    }

    // Filtering Files ********************************************************

    public boolean accept(String canonicalName) {
        return true;
    }


    // Conversion To/From Properties ******************************************

    public Properties asProperties() {
        Properties properties = new Properties();
        properties.setProperty(
                PROPERTYNAME_TARGET,
                getTarget().getAbsolutePath());
        properties.setProperty(
                PROPERTYNAME_NAME,
                getName());
        properties.setProperty(
                PROPERTYNAME_CHECK_CYLES,
                String.valueOf(getCheckCycles()));
        String encodedInclusions = hasInclusions()
            ? encode(getInclusions())
            : "";
        properties.setProperty(
                PROPERTYNAME_INCLUSIONS,
                encodedInclusions);
        String encodedExclusions = hasExclusions()
            ? encode(getExclusions())
            : "";
        properties.setProperty(
            PROPERTYNAME_EXCLUSIONS,
            encodedExclusions);
        properties.setProperty(
            PROPERTYNAME_MIN_MODIFIED,
            hasMinModifiedDate() ? String.valueOf(getMinModified()) : "");
        properties.setProperty(
                PROPERTYNAME_MAX_MODIFIED,
                hasMaxModifiedDate() ? String.valueOf(getMaxModified()) : "");
        return properties;
    }


    // String Representation **************************************************

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder("Scan Configuration: ");
        builder.append("\nTarget=");
        builder.append(getTarget());
        builder.append("\nName=");
        builder.append(getName());
        builder.append("\nCheck cycles=");
        builder.append(getCheckCycles());
        builder.append("\nInclusions=");
        builder.append(hasInclusions() ? encode(getInclusions()) : "none");
        builder.append("\nExclusions=");
        builder.append(hasExclusions() ? encode(getExclusions()) : "none");
        builder.append("\nMin modified date=");
        builder.append(hasMinModifiedDate() ? formatDate(getMinModified()) : "none");
        builder.append("\nMax modified date=");
        builder.append(hasMaxModifiedDate() ? formatDate(getMaxModified()) : "none");
        return builder.toString();
    }


    private static String encode(List<String> patterns) {
        StringBuilder builder = new StringBuilder();
        for (String pattern : patterns) {
            if (builder.length() > 0) {
                builder.append(", ");
            }
            builder.append(pattern);
        }
        return builder.toString();
    }


    private static List<String> decode(String patterns) {
        List<String> result = new ArrayList<String>();
        if ((patterns == null) || (patterns.trim().length() == 0)) {
            return result;
        }
        String[] items = patterns.split(",");
        for (String item : items) {
            String trimmedItem = item.trim();
            if (trimmedItem.length() > 0) {
                result.add(trimmedItem);
            }
        }
        return result;
    }


    private String formatDate(long time) {
        return DateFormat.getDateTimeInstance(DateFormat.LONG, DateFormat.LONG).format(new Date(time));
    }

}
