/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport.domain;

import java.io.Serializable;
import java.text.DateFormat;
import java.util.Date;



/**
 * An immutable description of a scan that consists of the configuration,
 * the root of the file system tree, and the start and end time.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.5 $
 */
public final class ScanResult implements Serializable {

    private static final long serialVersionUID = 1L;

    private final ScanConfiguration configuration;
    private final Root root;
    private final long fileCount;
    private final long folderCount;
    private final long startTime;
    private final long endTime;


    // Instance Creation ******************************************************

    /**
     * Constructs a scan result for the given configuration, root, and summary.
     */
    public ScanResult(
            ScanConfiguration configuration,
            Root root,
            long fileCount,
            long folderCount,
            long startTime,
            long endTime) {
        if (configuration == null) {
            throw new NullPointerException("The configuration must not be null.");
        }
        if (root == null) {
            throw new NullPointerException("The root must not be null.");
        }
        if (startTime < 0) {
            throw new IllegalArgumentException("The start time must be positive.");
        }
        if (endTime < startTime) {
            throw new IllegalArgumentException("The end time must be after the start time.");
        }
        this.configuration = configuration;
        this.root = root;
        this.fileCount = fileCount;
        this.folderCount = folderCount;
        this.startTime = startTime;
        this.endTime = endTime;
    }


    // Accessors **************************************************************

    public ScanConfiguration getConfiguration() {
        return configuration;
    }


    public Root getRoot() {
        return root;
    }


    public Date getStartTime() {
        return new Date(startTime);
    }


    public Date getEndTime() {
        return new Date(endTime);
    }


    public long getScanDuration() {
        return endTime - startTime;
    }


    // String Representation **************************************************

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder("Scan result for: ");
        builder.append(configuration.getTarget());
        builder.append("\nFiles=");
        builder.append(fileCount);
        builder.append("\nFolders=");
        builder.append(folderCount);
        builder.append("\nStarted=");
        builder.append(formatDate(getStartTime()));
        builder.append("\nFinished=");
        builder.append(formatDate(getEndTime()));
        builder.append("\nDuration=");
        builder.append(formatTime(getScanDuration()));
        return builder.toString();
    }


    private String formatDate(Date time) {
        return DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.MEDIUM).format(time);
    }


    private String formatTime(long time) {
        long seconds = time / 1000;
        if (seconds < 60) {
            return seconds + " seconds";
        }
        long minutes = seconds / 60;
        seconds = seconds % 60;
        if (minutes < 60) {
            return minutes + " minutes, " + seconds + " seconds";
        }
        long hours = minutes / 60;
        minutes = minutes % 60;
        return hours + " hours, " + minutes + " minutes";
    }


}
