/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport.model;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import com.jgoodies.binding.beans.Model;
import com.jgoodies.jdiskreport.model.node.DirectoryNode;
import com.jgoodies.jdiskreport.model.node.RootNode;
import com.jgoodies.jdiskreport.model.node.DirectoryNode.FileTreeModel;
import com.jgoodies.jdiskreport.model.settings.GUISettings;
import com.jgoodies.jdiskreport.model.settings.GUISettings.SortMode;

/**
 * Provides bound properties, operations and Actions
 * for the navigation tree.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.7 $
 */
public final class NavigationModel extends Model {

    // Names of the Bound Bean Properties *************************************

    public static final String PROPERTYNAME_SELECTION = "selection";


    // Instance Fields ********************************************************

    /**
     * Refers to the parent main model.
     */
    private final MainModel mainModel;


    /**
     * Holds the current directory selection as chosen in the navigation tree.
     *
     * @see #getSelection()
     */
    private DirectoryNode selection;

    /**
     * Holds the model for the navigation tree.
     *
     * @see #getNavigationTreeModel()
     */
    private final FileTreeModel navigationTreeModel;


    /**
     * Refers to the single selection model for the navigation tree.
     * The selection is reset if the navigation tree model changes.
     *
     * @see #getNavigationTreeSelectionModel()
     * @see #getNavigationTreeModel()
     */
    private final TreeSelectionModel navigationTreeSelectionModel;


    // Instance Creation ******************************************************

    /**
     * Constructs a NavigationModel that has no selection and no tree model.
     *
     * @param mainModel  this model's parent model
     */
    NavigationModel(MainModel mainModel) {
        this.mainModel = mainModel;
        RootNode rootNode = mainModel.getRootNode();
        // TreeModel
        SortMode sortMode = mainModel.guiSettings().getSortMode();
        navigationTreeModel = mainModel.getRootNode().getTreeModel(sortMode);
        // TreeSelectionModel
        navigationTreeSelectionModel = new DefaultTreeSelectionModel();
        navigationTreeSelectionModel.setSelectionMode(
                TreeSelectionModel.SINGLE_TREE_SELECTION);
        initEventHandling();

        // Select the root
        navigationTreeSelectionModel.addSelectionPath(new TreePath(rootNode));
    }


    private void initEventHandling() {
        navigationTreeSelectionModel.addTreeSelectionListener(
                new NavigationTreeSelectionChangeHandler());
        mainModel.guiSettings().addPropertyChangeListener(
                new GUISettingsChangeHandler());
    }


    // Accessors **************************************************************

    /**
     * Returns the current selection, a directory node. This selection
     * will be updated if the selection in the navigation tree changes.
     * During the transition from one tree selection to another this
     * value may hold the old or new selection.
     *
     * @return the navigation tree selection
     */
    public DirectoryNode getSelection() {
        return selection;
    }


    /**
     * Sets a new directory node as selection. This method is invoked
     * by the NavigationTreeSelectionChangeHandler that observes changes in
     * the selection of the navigation tree.
     *
     * @param newSelection   the selection to set
     *
     * @see #getSelection()
     */
    private void setSelection(DirectoryNode newSelection) {
        DirectoryNode oldSelection = getSelection();
        selection = newSelection;
        firePropertyChange(PROPERTYNAME_SELECTION, oldSelection, newSelection);
    }


    // Managing the Navigation Tree Model *************************************

    /**
     * Returns the tree model for the navigation tree.
     *
     * @return the tree model for the navigation tree.
     */
    public TreeModel getNavigationTreeModel() {
        return navigationTreeModel;
    }


    /**
     * Returns the fixed selection model for the navigation tree.
     *
     * @return the fixed selection model for the navigation tree.
     */
    public TreeSelectionModel getNavigationTreeSelectionModel() {
        return navigationTreeSelectionModel;
    }


    /**
     * Creates a new navigation tree model and sets it as
     * this class' tree model. Invoked by the parent MainModel
     * if the project has changed.
     */
    void updateTreeSortMode() {
        SortMode sortMode = mainModel.guiSettings().getSortMode();
        navigationTreeModel.setSortMode(sortMode);
    }


    // Event Handling *********************************************************

    private final class GUISettingsChangeHandler implements PropertyChangeListener {

        public void propertyChange(PropertyChangeEvent evt) {
            String propertyName = evt.getPropertyName();
            if (GUISettings.PROPERTYNAME_SORT_MODE.equals(propertyName)) {
                updateTreeSortMode();
            }
        }

    }


    /**
     * Listens to changes in the tree node selection and updates the selection.
     */
    private final class NavigationTreeSelectionChangeHandler implements TreeSelectionListener {

        /**
         * The selected node in the navidation tree has changed.
         * Updates this model's selected domain object and in turn
         * the selection type.<p>
         *
         * Does nothing if the new tree selection is {@code null}.
         *
         * @param evt    the event that describes the change
         */
        public void valueChanged(TreeSelectionEvent evt) {
            TreePath path = navigationTreeSelectionModel.getSelectionPath();
            DirectoryNode newSelection = path == null
                ? null
                : (DirectoryNode) path.getLastPathComponent();
            if (newSelection == null) {
                return;
            }
            // TODO: Activate updateSelectionActionEnablement(aSelectedNode);
            setSelection(newSelection);
        }


    }


    // Updating the Action Enablement *****************************************

//    /**
//     * Updates the enablement state actions realted to the node selection.
//     *
//     * @param aSelectedNode   the selected node
//     */
//    private void updateSelectionActionEnablement(DirectoryNode aSelectedNode) {
//        boolean canAddSubsystem = aSelection != null;
//        boolean canDelete = aSelection != null;
//    }


}
