/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport.model;

import java.io.*;
import java.util.Enumeration;
import java.util.Properties;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import com.jgoodies.jdiskreport.domain.ScanConfiguration;
import com.jgoodies.jdiskreport.domain.ScanResult;



/**
 * Saves and loads scan configurations (text files) and
 * compressed binary scan results.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.9 $
 */
public final class PersistencyManager {

    // Scan Configuration *****************************************************

    /**
     * Pre-considtion: the input file exists.
     */
    public ScanConfiguration readScanConfiguration(File inputFile) {
        if (inputFile == null) {
            throw new NullPointerException("The input file must not be null.");
        }
        FileInputStream in;
        try {
            in = new FileInputStream(inputFile);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            System.err.println("The input file "
                    + "does not exists, "
                    + "or is a directory rather than a regular file, "
                    + "or cannot be opened for any other reasons.");
            return null;
        }
        try {
            Properties properties = new Properties();
            properties.load(in);
            return ScanConfiguration.from(properties);
        } catch (IOException e) {
            e.printStackTrace();
            System.err.println("Failed to read the scan configuration from file: " + inputFile);
            return null;
        } finally {
            try {
                in.close();
            } catch (IOException e) {
                e.printStackTrace();
                System.err.println("Failed to close the input file: " + inputFile);
            }
        }
    }


    public void writeScanConfiguration(File outputFile, ScanConfiguration configuration) {
        if (outputFile == null) {
            throw new NullPointerException("The output file must not be null.");
        }
        if (configuration == null) {
            throw new NullPointerException("The scan configuration must not be null.");
        }
        FileOutputStream out;
        try {
            out = new FileOutputStream(outputFile);
        } catch (FileNotFoundException e) {
            System.out.println("The output file "
                    + "exists but is a directory rather than a regular file, "
                    + "does not exist but cannot be created, "
                    + "or cannot be opened for any other reason.");
            e.printStackTrace();
            return;
        }
        Properties properties = configuration.asProperties();
        try {
            properties.store(out, "JDiskReport2 scan configuration properties");
        } catch (IOException e) {
            e.printStackTrace();
            System.out.println(
                "Write failed: " + outputFile.getName());
        } finally {
            try {
                out.close();
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        }
    }



    // Scan Results ***********************************************************

    /**
     * Reads a scan result from the specified input file.
     *
     * @param inputFile   the file to read the result from
     * @return the scan result that holds the configuration, the file tree,
     *    and summary
     *
     * @throws NullPointerException   if the input file is null
     */
    public ScanResult readScanResult(File inputFile) {
        if (inputFile == null) {
            throw new NullPointerException("The input file must not be null.");
        }
        if (!inputFile.exists()) {
            System.out.println("The input file " + inputFile + " does not exist.");
            return null;
        }
        if (!inputFile.isFile()) {
            System.out.println("The input file " + inputFile + " + is not a file.");
            return null;
        }
        ZipFile zipFile = null;
        try {
            zipFile = new ZipFile(inputFile, ZipFile.OPEN_READ);
            Enumeration<? extends ZipEntry> entries = zipFile.entries();
            if (!entries.hasMoreElements()) {
                System.out.println("Missing entry.");
                return null;
            }
            ZipEntry entry = entries.nextElement();
            if (entries.hasMoreElements()) {
                System.out.println("Warning: a report file should contain a single entry only.");
            }
            return readScanResult(zipFile.getInputStream(entry));
        } catch (IOException e) {
            e.printStackTrace();
            System.out.println("The input file " + inputFile
                    + " cannot be opened.");
            return null;
        } finally {
            if (zipFile != null) {
                try {
                    zipFile.close();
                } catch (IOException e) {
                    e.printStackTrace();
                    System.out.println("Failed to close the input zip file.");
                }
            }
        }
    }

    private ScanResult readScanResult(InputStream in) {
        ObjectInputStream objectIn = null;
        try {
            objectIn = new ObjectInputStream(in);
            return (ScanResult) objectIn.readObject();
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        } catch (ClassNotFoundException f) {
            f.printStackTrace();
            return null;
        } finally {
            if (objectIn != null) {
                try {
                    objectIn.close();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
        }
    }


    /**
     * Saves the given scan result with its configuration and
     * file tree to the specified output file.
     *
     * @param outputFile   the file to store the scan result
     * @param result       the scan result with configuration and file tree
     *     to be saved
     *
     * @throws NullPointerException   if the output file or root is null
     */
    public void writeScanResult(File outputFile, ScanResult result) {
        if (outputFile == null) {
            throw new NullPointerException("The output file must not be null.");
        }
        if (result == null) {
            throw new NullPointerException("The scan result must not be null.");
        }
        FileOutputStream out;
        try {
            out = new FileOutputStream(outputFile);
        } catch (FileNotFoundException e) {
            System.out.println("The output file "
                    + "exists but is a directory rather than a regular file, "
                    + "does not exist but cannot be created, "
                    + "or cannot be opened for any other reason.");
            e.printStackTrace();
            return;
        }
        ZipOutputStream zipOut = new ZipOutputStream(out);
        ZipEntry zipEntry = new ZipEntry(outputFile.getName()+".uncompressed");
        ObjectOutputStream objectOut = null;
        try {
            zipOut.putNextEntry(zipEntry);
            objectOut = new ObjectOutputStream(zipOut);
            objectOut.writeObject(result);
            zipOut.closeEntry();
        } catch (IOException e) {
            e.printStackTrace();
            System.out.println(
                "Write failed: " + outputFile.getName());
        } finally {
            try {
                zipOut.close();
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        }
    }


    // Helper Class ***********************************************************

    public static final class ReadWriteException extends Exception {

        ReadWriteException(String message, Throwable cause) {
            super(message, cause);
        }

    }

}
