/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport.model;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.Icon;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

import com.jgoodies.binding.beans.Model;
import com.jgoodies.chart.*;
import com.jgoodies.jdiskreport.model.node.AbstractNode;
import com.jgoodies.jdiskreport.model.node.DirectoryNode;
import com.jgoodies.jdiskreport.model.settings.ChartSettings;
import com.jgoodies.jdiskreport.model.settings.GUISettings;
import com.jgoodies.jdiskreport.model.settings.GUISettings.SizeMode;
import com.jgoodies.jdiskreport.model.settings.GUISettings.SortMode;
import com.jgoodies.jdiskreport.model.statistic.Statistics;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;

/**
 * Provides chart and table models for the different presentations.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.12 $
 */
public final class PresentationModel extends Model {


    private static final ResourceMap RESOURCES =
        Application.getResourceMap(PresentationModel.class);


    // Instance Fields ********************************************************

    /**
     * Refers to the parent main model.
     * Used to lookup the default parent frame
     */
    private final MainModel mainModel;

    private final DelegatingTreeChartModel<AbstractNode> treeChartModel;

    private final DelegatingTableModel sizeTableModel;
    private final ChartModel<AbstractNode> sizeChartModel;
    private final FilteredChartModel<AbstractNode> filteredSizeChartModel;
    private final ColoredChartModel<AbstractNode> coloredSizeChartModel;

    private final DelegatingTableModel largestFilesTableModel;
    private final DelegatingTableModel oldestFilesTableModel;
    private final DelegatingTableModel newestFilesTableModel;

    private final DelegatingTableModel sizeDistributionTableModel;
    private final ChartModel<Object> sizeDistributionChartModel;
    private final ColoredChartModel<Object> coloredSizeDistributionChartModel;

    private final DelegatingTableModel timeDistributionTableModel;
    private final ChartModel<Object> timeDistributionChartModel;
    private final ColoredChartModel<Object> coloredTimeDistributionChartModel;

    private final DelegatingTableModel typeDistributionTableModel;
    private final ChartModel<Object> typeDistributionChartModel;
    private final FilteredChartModel<Object> filteredTypeDistributionChartModel;
    private final ColoredChartModel<Object> coloredTypeDistributionChartModel;


    // Instance Creation ******************************************************

    /**
     * Constructs a NavigationModel that has no selection and no tree model.
     *
     * @param mainModel  this model's parent model
     */
    PresentationModel(MainModel mainModel) {
        this.mainModel = mainModel;

        treeChartModel = new DelegatingTreeChartModel<AbstractNode>(
                getTreeChartModel(mainModel.getRootNode()));
        String collapsedText =
            RESOURCES.getString("PresentationModel.filter.collapsedText");
        Color collapsedColor =
            RESOURCES.getColor("PresentationModel.filter.collapsedColor");
       ChartSettings chartSettings = mainModel.guiSettings().chartSettings();
       int itemLimit = chartSettings.getElementLimit();
       float itemThreshold = chartSettings.getElementThreshold();
       float saturation = chartSettings.getSaturation();
       float brightness = chartSettings.getBrightness();

        sizeTableModel = new DelegatingTableModel(
                getSizeTableModel(mainModel.getRootNode()));
        sizeChartModel = new SizeChartModel(
                    sizeTableModel,
                    mainModel.guiSettings());
        filteredSizeChartModel = new FilteredChartModel<AbstractNode>(
                sizeChartModel, itemLimit, itemThreshold, collapsedText, collapsedColor);
        coloredSizeChartModel = new ColoredChartModel<AbstractNode>(
                filteredSizeChartModel,
                itemLimit,
                saturation,
                brightness);

        Statistics rootStatistics = mainModel.getRootNode().getStatistics();
        largestFilesTableModel = new DelegatingTableModel(
                rootStatistics.getLargestFiles().getTableModel());
        oldestFilesTableModel = new DelegatingTableModel(
                rootStatistics.getOldestFiles().getTableModel());
        newestFilesTableModel = new DelegatingTableModel(
                rootStatistics.getNewestFiles().getTableModel());

        sizeDistributionTableModel = new DelegatingTableModel(
                rootStatistics.getSizeDistribution().getTableModel());
        sizeDistributionChartModel =
            new DistributionChartModel(
                    sizeDistributionTableModel,
                    mainModel.guiSettings());
        coloredSizeDistributionChartModel = new ColoredChartModel<Object>(
                sizeDistributionChartModel,
                itemLimit,
                saturation,
                brightness);

        timeDistributionTableModel = new DelegatingTableModel(
                rootStatistics.getTimeDistribution().getTableModel());
        timeDistributionChartModel =
            new DistributionChartModel(
                    timeDistributionTableModel,
                    mainModel.guiSettings());
        coloredTimeDistributionChartModel = new ColoredChartModel<Object>(
                timeDistributionChartModel,
                itemLimit,
                saturation,
                brightness);

        typeDistributionTableModel = new DelegatingTableModel(
                rootStatistics.getTypeDistribution().getTableModelBySize());
        typeDistributionChartModel =
            new DistributionChartModel(
                    typeDistributionTableModel,
                    mainModel.guiSettings());
        filteredTypeDistributionChartModel = new FilteredChartModel<Object>(
                typeDistributionChartModel, itemLimit, itemThreshold, collapsedText, collapsedColor);
        coloredTypeDistributionChartModel = new ColoredChartModel<Object>(
                filteredTypeDistributionChartModel,
                itemLimit,
                saturation,
                brightness);

        initEventHandling();
    }


    // Accessors **************************************************************

    public TreeChartModel<AbstractNode> getTreeChartModel() {
        return treeChartModel;
    }


    public ChartModel<AbstractNode> getSizeChartModel() {
        return coloredSizeChartModel;
    }


    public TableModel getSizeTableModel() {
        return sizeTableModel;
    }


    public TableModel getLargestFilesTableModel() {
        return largestFilesTableModel;
    }


    public TableModel getOldestFilesTableModel() {
        return oldestFilesTableModel;
    }


    public TableModel getNewestFilesTableModel() {
        return newestFilesTableModel;
    }


    public ChartModel<Object> getSizeDistributionChartModel() {
        return coloredSizeDistributionChartModel;
    }


    public TableModel getSizeDistributionTableModel() {
        return sizeDistributionTableModel;
    }


    public ChartModel<Object> getTimeDistributionChartModel() {
        return coloredTimeDistributionChartModel;
    }


    public TableModel getTimeDistributionTableModel() {
        return timeDistributionTableModel;
    }


    public ChartModel<Object> getTypeDistributionChartModel() {
        return coloredTypeDistributionChartModel;
    }


    public TableModel getTypeDistributionTableModel() {
        return typeDistributionTableModel;
    }


    // Helper Code ************************************************************

    private TableModel getSizeTableModel(DirectoryNode directoryNode) {
        SortMode sortMode = mainModel.guiSettings().getSortMode();
        SizeMode sizeMode = mainModel.guiSettings().getSizeMode();
        boolean showFiles = mainModel.guiSettings().getShowFiles();
        return directoryNode.getTableModel(sortMode, sizeMode, showFiles);
    }


    private TreeChartModel<AbstractNode> getTreeChartModel(DirectoryNode directoryNode) {
        SortMode sortMode = mainModel.guiSettings().getSortMode();
        SizeMode sizeMode = mainModel.guiSettings().getSizeMode();
        boolean showFiles = mainModel.guiSettings().getShowFiles();
        return directoryNode.getTreeChartModel(sortMode, sizeMode, showFiles);
    }


    // Event Handling *********************************************************

    private void initEventHandling() {
        mainModel.guiSettings().addPropertyChangeListener(
                new GUISettingsChangeHandler());
        mainModel.guiSettings().chartSettings().addPropertyChangeListener(
                new ChartSettingsChangeHandler());
        mainModel.navigationModel().addPropertyChangeListener(
                NavigationModel.PROPERTYNAME_SELECTION,
                new NavigationSelectionChangeHandler());
        updateSelection(mainModel.navigationModel().getSelection());
    }


    private void updateSelection(DirectoryNode selection) {
        TreeChartModel<AbstractNode> aTreeChartModel = getTreeChartModel(selection);
        treeChartModel.setDelegate(aTreeChartModel);

        TableModel aSizeTableModel = getSizeTableModel(selection);
        sizeTableModel.setDelegate(aSizeTableModel);

        Statistics statistics = selection.getStatistics();
        if (statistics != null) {
            largestFilesTableModel.setDelegate(statistics.getLargestFiles().getTableModel());
            oldestFilesTableModel.setDelegate(statistics.getOldestFiles().getTableModel());
            newestFilesTableModel.setDelegate(statistics.getNewestFiles().getTableModel());

            TableModel sizeDistTableModel = statistics.getSizeDistribution().getTableModel();
            sizeDistributionTableModel.setDelegate(sizeDistTableModel);

            TableModel timeDistTableModel = statistics.getTimeDistribution().getTableModel();
            timeDistributionTableModel.setDelegate(timeDistTableModel);

            TableModel typeDistTableModel = statistics.getTypeDistribution().getTableModelBySize();
            typeDistributionTableModel.setDelegate(typeDistTableModel);
        }
    }


    private void updateColoredModels() {
        ChartSettings chartSettings = mainModel.guiSettings().chartSettings();
        int colorCount = chartSettings.getElementLimit();
        float saturation = chartSettings.getSaturation();
        float brightness = chartSettings.getBrightness();
        coloredSizeChartModel.setColorCount(colorCount);
        coloredSizeChartModel.setSaturation(saturation);
        coloredSizeChartModel.setBrightness(brightness);

        coloredSizeDistributionChartModel.setColorCount(colorCount);
        coloredSizeDistributionChartModel.setSaturation(saturation);
        coloredSizeDistributionChartModel.setBrightness(brightness);

        coloredTimeDistributionChartModel.setColorCount(colorCount);
        coloredTimeDistributionChartModel.setSaturation(saturation);
        coloredTimeDistributionChartModel.setBrightness(brightness);

        coloredTypeDistributionChartModel.setColorCount(colorCount);
        coloredTypeDistributionChartModel.setSaturation(saturation);
        coloredTypeDistributionChartModel.setBrightness(brightness);
    }


    private void updateFilteredModels() {
        ChartSettings chartSettings = mainModel.guiSettings().chartSettings();
        int elementLimit = chartSettings.getElementLimit();
        filteredSizeChartModel.setItemLimit(elementLimit);
    }


    private final class GUISettingsChangeHandler implements PropertyChangeListener {

        public void propertyChange(PropertyChangeEvent evt) {
            String propertyName = evt.getPropertyName();
            if (   GUISettings.PROPERTYNAME_SORT_MODE.equals(propertyName)
                || GUISettings.PROPERTYNAME_SIZE_MODE.equals(propertyName)
                || GUISettings.PROPERTYNAME_SHOW_FILES.equals(propertyName)) {
                updateSelection(mainModel.navigationModel().getSelection());
            }
        }

    }


    private final class ChartSettingsChangeHandler implements PropertyChangeListener {

        public void propertyChange(PropertyChangeEvent evt) {
            String propertyName = evt.getPropertyName();
            if (   ChartSettings.PROPERTYNAME_BRIGHTNESS.equals(propertyName)
                || ChartSettings.PROPERTYNAME_SATURATION.equals(propertyName)) {
                updateColoredModels();
            } else if (ChartSettings.PROPERTYNAME_ELEMENT_LIMIT.equals(propertyName)) {
                updateFilteredModels();
            }
        }

    }


    private final class NavigationSelectionChangeHandler implements PropertyChangeListener {

        public void propertyChange(PropertyChangeEvent evt) {
            DirectoryNode selection = (DirectoryNode) evt.getNewValue();
            updateSelection(selection);
        }

    }


    // TableModel Implementation **********************************************

    private static final class DelegatingTableModel extends AbstractTableModel {

        private TableModel delegate;

        // Instance Creation ----------------------------

        DelegatingTableModel(TableModel delegate) {
            this.delegate = delegate;
        }

        // Changing the Delegate ------------------------

        void setDelegate(TableModel newDelegate) {
            if (delegate == newDelegate) {
                return;
            }
            this.delegate = newDelegate;
            fireTableDataChanged();
        }

        // TableModel Implementation ---------------------

        public int getColumnCount() {
            return delegate.getColumnCount();
        }

        @Override
        public String getColumnName(int columnIndex) {
            return delegate.getColumnName(columnIndex);
        }

        @Override
        public Class<?> getColumnClass(int columnIndex) {
            return delegate.getColumnClass(columnIndex);
        }

        public int getRowCount() {
            return delegate.getRowCount();
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            return delegate.getValueAt(rowIndex, columnIndex);
        }

    }


    // ChartModel Implementation **********************************************

    private static final class SizeChartModel extends AbstractChartModel<AbstractNode> {

        private final TableModel tableModel;
        private final GUISettings guiSettings;


        SizeChartModel(TableModel tableModel, GUISettings guiSettings) {
            this.tableModel = tableModel;
            this.guiSettings = guiSettings;
            tableModel.addTableModelListener(new ChartUpdater(this));
        }


        // ChartModel Implementation ------------------------------------------

        public int getItemCount() {
            return tableModel.getRowCount();
        }

        public AbstractNode getData(int index) {
            return (AbstractNode) tableModel.getValueAt(index, 0);
        }

        public Icon getIcon(int index) {
            return getData(index).getIcon(false);
        }

        public String getText(int index) {
            return getData(index).getName();
        }

        public long getSize(int index) {
            SizeMode sizeMode = guiSettings.getSizeMode();
            return sizeMode == SizeMode.SIZE
                ? getData(index).getSize()
                : getData(index).getFileCount();
        }

        public Color getColor(int index) {
            return null;
        }

    }


    private static final class DistributionChartModel extends AbstractChartModel<Object> {

        private final TableModel tableModel;
        private final GUISettings guiSettings;


        DistributionChartModel(TableModel tableModel, GUISettings guiSettings) {
            this.tableModel = tableModel;
            this.guiSettings = guiSettings;
            tableModel.addTableModelListener(new ChartUpdater(this));
        }


        // ChartModel Implementation ------------------------------------------

        public int getItemCount() {
            return tableModel.getRowCount();
        }

        public Object getData(int index) {
            return null;
        }

        public Icon getIcon(int index) {
            return null;
        }

        public String getText(int index) {
            return (String) tableModel.getValueAt(index, 0);
        }

        public long getSize(int index) {
            SizeMode sizeMode = guiSettings.getSizeMode();
            int columnIndex = sizeMode == SizeMode.SIZE ? 1 : 3;
            return ((Long) tableModel.getValueAt(index, columnIndex)).longValue();
        }

        public Color getColor(int index) {
            return null;
        }

    }


    /**
     * Fires a ChartModel state change for all table model events.
     */
    private static final class ChartUpdater implements TableModelListener {

        private final AbstractChartModel<?> chartModel;

        ChartUpdater(AbstractChartModel<?> chartModel) {
            this.chartModel = chartModel;
        }

        public void tableChanged(TableModelEvent e) {
            chartModel.fireStateChanged();
        }


    }


    // TreeChartModel Implementation ******************************************

    private static final class DelegatingTreeChartModel<T> extends AbstractTreeChartModel<T> {

        private TreeChartModel<T> delegate;

        // Instance Creation ------------------------------------------------

        DelegatingTreeChartModel(TreeChartModel<T> delegate) {
            this.delegate = delegate;
        }


        // Changing the Delegate --------------------------------------------

        void setDelegate(TreeChartModel<T> newDelegate) {
            TreeChartModel<T> oldDelegate = delegate;
            if (oldDelegate == newDelegate) {
                return;
            }
            delegate = newDelegate;
            fireStateChanged();
        }


        // TreeChartModel Implementation ------------------------------------
        public T getRoot() {
            return delegate.getRoot();
        }

        public int getChildCount(T node) {
            return delegate.getChildCount(node);
        }

        public T getChild(T node, int index) {
            return delegate.getChild(node, index);
        }

        public long getSize(T node) {
            return delegate.getSize(node);
        }

        public boolean isLeaf(T node) {
            return delegate.isLeaf(node);
        }

    }


}
