/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport.model.node;

import java.util.*;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.swing.event.TableModelListener;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.table.TableModel;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import com.jgoodies.chart.TreeChartModel;
import com.jgoodies.jdiskreport.domain.Directory;
import com.jgoodies.jdiskreport.domain.FileInfo;
import com.jgoodies.jdiskreport.model.settings.GUISettings.SizeMode;
import com.jgoodies.jdiskreport.model.settings.GUISettings.SortMode;
import com.jgoodies.jdiskreport.model.statistic.Statistics;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;


/**
 * Describes directories.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.18 $
 */
public class DirectoryNode extends AbstractNode {

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(DirectoryNode.class);

    private static final Comparator<AbstractNode> FILE_SIZE_COMPARATOR =
        new FileSizeComparator();

    private static final Comparator<AbstractNode> FILE_COUNT_COMPARATOR =
        new FileCountComparator();

    private static final Comparator<AbstractNode> NAME_COMPARATOR =
        new NameComparator();


    // Instance Fields ********************************************************

    private final Directory directory;
    private final long size;
    private final long directoryCount;
    private final long fileCount;
    private final DirectoryNode[] childDirectories;
    private final FileNode[] childFiles;

    private List<AbstractNode> sortedChildren;
    private CollapsedFilesNode collapsedFilesNode;

    /**
     * An optional object that contains the top 50 lists of largest, oldest,
     * and newest files, as well as the size, time, and type distributions.
     *
     * To reduce the memory needed by a tree of directory and file nodes,
     * the statistics may be available for a limited number of levels
     * (starting at the root of the tree). In case the statistics are
     * not available, the following three fields are filled.
     */
    private Statistics statistics;

    /**
     * Holds the largest file if the statistics is null.
     */
    private FileNode largestFile;

    /**
     * Holds the newest file if the statistics is null.
     */
    private FileNode oldestFile;

    /**
     * Holds the oldest file if the statistics is null.
     */
    private FileNode newestFile;


    // Instance Creation ******************************************************

    /**
     * Constructs a DirectoryNode for the given name, size, file count,
     * child directories and files.
     * Sets this node as parent for all its children.
     */
    protected DirectoryNode(
            Directory directory,
            long size,
            long directoryCount,
            long fileCount,
            DirectoryNode[] childDirectories,
            FileNode[] childFiles) {
        this.directory = directory;
        this.size = size;
        this.directoryCount = directoryCount;
        this.fileCount = fileCount;
        this.childDirectories = childDirectories;
        this.childFiles = childFiles;
        for (DirectoryNode child : childDirectories) {
            child.setParent(this);
        }
        for (FileNode child : childFiles) {
            child.setParent(this);
        }
    }


    static DirectoryNode createFrom(Directory directory) {
        long size = 0;
        long directoryCount = 1;
        long fileCount = 0;

        Directory[] childDirs = directory.getDirectories();
        DirectoryNode[] childDirNodes = new DirectoryNode[childDirs.length];
        for (int i = 0; i < childDirNodes.length; i++) {
            DirectoryNode childNode = DirectoryNode.createFrom(childDirs[i]);
            childDirNodes[i] = childNode;
            size += childNode.getSize();
            directoryCount += childNode.getDirectoryCount();
            fileCount += childNode.getFileCount();
        }

        FileInfo[] childFiles = directory.getFiles();
        FileNode[] childFileNodes = new FileNode[childFiles.length];
        for (int i = 0; i < childFileNodes.length; i++) {
            FileNode childNode = new FileNode(childFiles[i]);
            childFileNodes[i] = childNode;
            size += childNode.getSize();
            fileCount += childNode.getFileCount();
        }

        return new DirectoryNode(
                directory, size, directoryCount, fileCount,
                childDirNodes, childFileNodes);
    }


    // Accessors **************************************************************

    @Override
    public String getName() {
        return directory.getName();
    }


    @Override
    public boolean isFileLeaf() {
        return false;
    }


    /**
     * Returns the name displayed by directory tree nodes.
     *
     * @return the file name for directories, the path for root nodes
     */
    public String getNodeName() {
        return getName();
    }


    public String getPath() {
        return getFile().getPath();
    }


    @Override
    public long getSize() {
        return size;
    }


    public long getDirectoryCount() {
        return directoryCount;
    }


    @Override
    public long getFileCount() {
        return fileCount;
    }


    // Structure **************************************************************

    public DirectoryNode[] getChildDirectories() {
        return childDirectories;
    }


    public FileNode[] getChildFiles() {
        return childFiles;
    }


    // Statistics *************************************************************

    public Statistics getStatistics() {
        return statistics;
    }


    private void setStatistics(Statistics statistics) {
        this.statistics = statistics;
    }


    public FileNode getLargestFile() {
        return statistics != null
            ? statistics.getLargestFiles().first()
            : largestFile;
    }


    public FileNode getOldestFile() {
        return statistics != null
            ? statistics.getOldestFiles().first()
            : oldestFile;
    }


    public FileNode getNewestFile() {
        return statistics != null
            ? statistics.getNewestFiles().first()
            : newestFile;
    }


    protected void computeAndSetStatistics(int level) {
        DirectoryNode[] childDirs = getChildDirectories();
        List<Statistics> allStatistics = new ArrayList<Statistics>(childDirs.length);
        for (DirectoryNode directoryNode : childDirs) {
            directoryNode.computeAndSetStatistics(level - 1);
            Statistics nodeStatistics = directoryNode.getStatistics();
            allStatistics.add(nodeStatistics);
            if (level <= 0) {
                if (directoryNode.getFileCount() > 0) {
                    directoryNode.largestFile = directoryNode.getLargestFile();
                    directoryNode.oldestFile  = directoryNode.getOldestFile();
                    directoryNode.newestFile  = directoryNode.getNewestFile();
                }
                directoryNode.setStatistics(null);
                directoryNode.clearCachedFiles();
            }
        }
        allStatistics.add(Statistics.createFrom(getChildFiles()));
        setStatistics(Statistics.createFrom(allStatistics));
        for (Statistics s : allStatistics) {
            s.getTypeDistribution().replaceMapByArrays();
        }
        Arrays.sort(getChildDirectories(), FILE_SIZE_COMPARATOR);
        Arrays.sort(getChildFiles(),       FILE_SIZE_COMPARATOR);
    }


    @Override
    public void clearCachedFiles() {
        super.clearCachedFiles();
        for (DirectoryNode directoryNode : getChildDirectories()) {
            directoryNode.clearCachedFiles();
        }
        for (FileNode fileNode : getChildFiles()) {
            fileNode.clearCachedFiles();
        }
    }


    // TreeModel Methods ******************************************************

    /**
     * Returns the number of children <code>TreeNode</code>s the receiver
     * contains.
     */
    final int getChildCount() {
        return childDirectories.length;
    }


    /**
     * Returns the child <code>TreeNode</code> at index
     * <code>childIndex</code>.
     */
    final DirectoryNode getChildAt(int childIndex, SortMode sortMode) {
        ensureChildDirectoriesSortedBy(sortMode);
        return childDirectories[childIndex];
    }


    /**
     * Returns the index of <code>node</code> in the receivers children.
     * If the receiver does not contain <code>node</code>, -1 will be
     * returned.
     */
    final int getIndex(DirectoryNode node, SortMode sortMode) {
        ensureChildDirectoriesSortedBy(sortMode);
        for (int i = 0; i < getChildCount(); i++) {
            if (node == childDirectories[i]) {
                return i;
            }
        }
        return -1;
    }

    private void ensureChildDirectoriesSortedBy(SortMode sortMode) {
        if (sortMode == lastSortMode) {
            return;
        }
        // TODO: Move the next line to #invalidateCache
        sortedChildren = null;
        Comparator<AbstractNode> comparator = sortMode == SortMode.BY_SIZE
            ? FILE_SIZE_COMPARATOR
            : NAME_COMPARATOR;
        Arrays.sort(childDirectories, comparator);
        lastSortMode = sortMode;
    }


    // Table Model ************************************************************

    private SortMode lastSortMode = null;
    private SizeMode lastSizeMode = null;
    private Boolean lastShowFiles = null;

    public final TableModel getTableModel(
            SortMode sortMode,
            SizeMode sizeMode,
            boolean showFiles) {
        return new SizeTableModel(getSortedChildren(sortMode, sizeMode, showFiles));
    }


    final List<AbstractNode> getSortedChildren(
            SortMode sortMode,
            SizeMode sizeMode,
            boolean showFiles) {
        if (   (sortedChildren != null)
            && (lastSortMode  == sortMode)
            && (lastSizeMode  == sizeMode)
            && (lastShowFiles == Boolean.valueOf(showFiles))) {
            return sortedChildren;
        }
        sortedChildren = createSortedChildren(sortMode, sizeMode, showFiles);
        lastSortMode = sortMode;
        lastSizeMode = sizeMode;
        lastShowFiles = Boolean.valueOf(showFiles);
        return sortedChildren;
    }

    private List<AbstractNode> createSortedChildren(
            SortMode sortMode,
            SizeMode sizeMode,
            boolean showFiles) {
        int length = childDirectories.length
                   + (showFiles ? childFiles.length : 1);
        List<AbstractNode> children =
            new ArrayList<AbstractNode>(length);
        for (DirectoryNode childDirectory : childDirectories) {
            children.add(childDirectory);
        }
        if (showFiles) {
            for (FileNode childFile : childFiles) {
                children.add(childFile);
            }
        } else if (childFiles.length > 0) {
            System.out.println("Adding a collapsed files node. Size: " + (getCollapsedFilesNode().getSize()));
            children.add(getCollapsedFilesNode());
        }
        Collections.sort(children,
                sortMode == SortMode.BY_NAME
                    ? NAME_COMPARATOR
                    : (sizeMode == SizeMode.SIZE ? FILE_SIZE_COMPARATOR : FILE_COUNT_COMPARATOR));
        return children;
    }

    private CollapsedFilesNode getCollapsedFilesNode() {
        if (collapsedFilesNode == null) {
            System.out.println("computing collapsed files\u2026");
            int collapsedSize = 0;
            for (FileNode childFile : childFiles) {
                collapsedSize += childFile.getSize();
            }
            collapsedFilesNode = new CollapsedFilesNode(this, collapsedSize, childFiles.length);
        }
        return collapsedFilesNode;
    }


    // TreeChartModel *********************************************************

    public final TreeChartModel<AbstractNode> getTreeChartModel(
            SortMode sortMode,
            SizeMode sizeMode,
            boolean showFiles) {
        return new FileTreeChartModel(this, sortMode, sizeMode, showFiles);
    }


    // Protected Access *******************************************************

    protected Directory getDirectory() {
        return directory;
    }


    // TableModel *************************************************************

    final static class SizeTableModel implements TableModel {

        private static final String COLUMN_NAME_KEY_PREFIX =
            DirectoryNode.class.getSimpleName();

        private static final String[] COLUMN_NAME_KEY_SUFFIXES =
            { "name", "size", "files", "relativeSize", "absoluteSize" };


        private static final Class<?>[] COLUMN_CLASSES =
            { AbstractNode.class, Long.class, Long.class, Float.class, Float.class };


        private final List<AbstractNode> children;

        SizeTableModel(List<AbstractNode> children) {
            this.children = children;
        }

        public int getColumnCount() {
            return COLUMN_NAME_KEY_SUFFIXES.length;
        }

        public String getColumnName(int columnIndex) {
            String key = COLUMN_NAME_KEY_PREFIX
                + ".columnName."
                + COLUMN_NAME_KEY_SUFFIXES[columnIndex];
            return RESOURCES.getString(key);
        }

        public Class<?> getColumnClass(int columnIndex) {
            return COLUMN_CLASSES[columnIndex];
        }

        public int getRowCount() {
            return children.size();
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            AbstractNode node = children.get(rowIndex);
            switch (columnIndex) {
                case 0 :
                    return node;

                case 1 :
                    return Long.valueOf(node.getSize());

                case 2 :
                    return Long.valueOf(node.getFileCount());

                case 3 :
                    return Float.valueOf(node.getRelativeSize());

                case 4 :
                    return Float.valueOf(node.getAbsoluteSize());

                default :
                    throw new IllegalStateException("Unknown column index: " + columnIndex);
            }

        }

        public void setValueAt(Object value, int rowIndex, int columnIndex) {
            throw new UnsupportedOperationException("Size table models are not editable.");
        }

        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return false;
        }

        public void addTableModelListener(TableModelListener l) {
            // Do nothing; this model is immutable.
        }

        public void removeTableModelListener(TableModelListener l) {
            // Do nothing; this model is immutable.
        }

    }


    // TreeModel **************************************************************

    public static final class FileTreeModel implements TreeModel {

        private final List<TreeModelListener> listenerList;

        private final RootNode root;
        private SortMode sortMode;

        // Instance Creation --------------------------------------------------

        FileTreeModel(RootNode root, SortMode sortMode) {
            this.root = root;
            this.sortMode = sortMode;
            this.listenerList = new CopyOnWriteArrayList<TreeModelListener>();
        }


        // Accessor -----------------------------------------------------------

        public void setSortMode(SortMode newSortMode) {
            if (sortMode == newSortMode) {
                return;
            }
            sortMode = newSortMode;
            fireTreeStructureChanged();
        }

        // TreeModel Implementation -------------------------------------------

        public Object getRoot() {
            return root;
        }

        public int getChildCount(Object parent) {
            DirectoryNode node = (DirectoryNode) parent;
            return node.getChildCount();
        }

        public Object getChild(Object parent, int index) {
            DirectoryNode node = (DirectoryNode) parent;
            return node.getChildAt(index, sortMode);
        }

        public int getIndexOfChild(Object parent, Object child) {
            DirectoryNode node = (DirectoryNode) parent;
            return node.getIndex((DirectoryNode) child, sortMode);
        }

        public boolean isLeaf(Object node) {
            DirectoryNode directoryNode = (DirectoryNode) node;
            return directoryNode.getChildCount() == 0;
        }

        public void addTreeModelListener(TreeModelListener listener) {
            if (listener == null) {
                throw new NullPointerException("The listener to be added must not be null.");
            }
            listenerList.add(listener);
        }

        public void removeTreeModelListener(TreeModelListener listener) {
            if (listener == null) {
                throw new NullPointerException("The listener to be added must not be null.");
            }
            listenerList.add(listener);
        }

        /**
         * Notifies all ChangeListeners registered with this model
         * that the state changed. The listeners need to look up
         * the changed data from this model.
         */
        private void fireTreeStructureChanged() {
            TreeModelEvent event = new TreeModelEvent(this, new TreePath(getRoot()), null, null);
            for (TreeModelListener listener : listenerList) {
                listener.treeStructureChanged(event);
            }
        }


        // Unsupported Operations ---------------------------------------------

        public void valueForPathChanged(TreePath path, Object newValue) {
            // Ignore. FileTreeModel don't fire events.;
        }

    }


    // Comparators ************************************************************

    private static final class FileSizeComparator implements Comparator<AbstractNode> {

        public int compare(AbstractNode node1, AbstractNode node2) {
            // First compare the sizes
            long size1 = node1.getSize();
            long size2 = node2.getSize();
            if (size1 < size2) {
                return 1;
            } else if (size1 > size2) {
                return -1;
            } else {
                return node1.getFile().compareTo(node2.getFile());
            }
        }
    }


    private static final class FileCountComparator implements Comparator<AbstractNode> {

        public int compare(AbstractNode node1, AbstractNode node2) {
            // First compare the number of files
            long size1 = node1.getFileCount();
            long size2 = node2.getFileCount();
            if (size1 < size2) {
                return 1;
            } else if (size1 > size2) {
                return -1;
            } else {
                return node1.getFile().compareTo(node2.getFile());
            }
        }
    }


    private static final class NameComparator implements Comparator<AbstractNode> {

        public int compare(AbstractNode node1, AbstractNode node2) {
            return node1.getFile().compareTo(node2.getFile());
        }
    }


}
