/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport.model.scan;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import com.jgoodies.jdiskreport.domain.*;
import com.jgoodies.jdiskreport.model.util.Formats;



/**
 * Scans a file system in depth-first order and returns a scan result,
 * which consists of the scan configuration and a tree of Directory instances
 * and FileInfos that represent the scanned tree.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.5 $
 */
public final class DepthFirstScanner implements Scanner {

    private ScanConfiguration configuration;
    private ScanLog log;

    private long directoryCount;
    private long fileCount;

    private boolean checkLinks;
    private boolean hasFilter;


    // Scanner Implementation *************************************************

    public synchronized ScanResult scan(ScanConfiguration scanConfiguration, ScanLog scanLog) {
        this.configuration = scanConfiguration;
        this.log = scanLog;
        if (!scanConfiguration.getTarget().exists()) {
            scanLog.logWarning("The scan target directory does not exist: " + scanConfiguration.getTarget());
            return null;
        }
        File canonicalTarget;
        try {
            canonicalTarget = scanConfiguration.getTarget().getCanonicalFile();
        } catch (IOException e) {
            scanLog.logWarning("Can't get the canonical form of the scan target directory.");
            return null;
        }

        scanLog.logConfiguration(scanConfiguration);
        checkLinks = configuration.getCheckCycles();
        hasFilter = configuration.hasFilter();
        directoryCount = 0;
        fileCount = 0;
        long startTime = System.currentTimeMillis();
        Root rootDir = scan(canonicalTarget);
        long endTime = System.currentTimeMillis();
        ScanResult result = new ScanResult(
                scanConfiguration,
                rootDir,
                fileCount,
                directoryCount,
                startTime,
                endTime);
        scanLog.logSummary(result);
        return result;
    }


    // Scan *******************************************************************

    private Root scan(File root) {
        Directory dir = scan(root, 0);
        return new Root(root, dir.getDirectories(), dir.getFiles());
    }


    private Directory scan(File directory, int level) {
        log.logDirectory(directory, level);
        File[] children = directory.listFiles();
        if (children == null) {
            log.logWarning("Can't access: " + directory);
            return new Directory(directory.getName());
        }
        List<Directory> directories = new ArrayList<Directory>();
        List<FileInfo> files = new ArrayList<FileInfo>(children.length);
        boolean link;
        boolean accepted;
        for (File file : children) {
            String name = file.getName();
            if (checkLinks) {
                String canonicalPath = FileUtils.getCanonicalPath(file);
                link = FileUtils.isLink(file, canonicalPath);
                accepted = !hasFilter || configuration.accept(canonicalPath);
            } else {
                link = false;
                accepted = !hasFilter || configuration.accept(FileUtils.getCanonicalPath(file));
            }
            if (accepted) {
                if (!link && file.isDirectory()) {
                    directoryCount++;
                    Directory childDir = scan(file, level + 1);
                    if (childDir != null) {
                        directories.add(childDir);
                    }
                } else { // We may distinguish non-regular files: if (file.isFile()) {
                    fileCount++;
                    long length = link ? FileUtils.linkLength(file) : file.length();
                    if (length < 0) {
                        log.logWarning("Negative file size (" + length + ") added as 0: " + file);
                        length = 0;
                    }
                    long lastModified = file.lastModified();
                    if (lastModified < 0) {
                        log.logWarning("Illegal modification date (" + Formats.formatDate(lastModified) + "):" + file);
                    }
                    files.add(new FileInfo(name, length, lastModified));
                }
            }
        }
        return new Directory(
                directory.getName(),
                directories.toArray(new Directory[directories.size()]),
                files.toArray(new FileInfo[files.size()])
                );
    }


}
