/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport.model.settings;

import java.util.prefs.Preferences;

import com.jgoodies.binding.beans.Model;

/**
 * Provides bound properties for chart related settings.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.7 $
 */
public final class ChartSettings extends Model {

    // Names of the Bound Bean Properties *************************************

    public static final String PROPERTYNAME_BRIGHTNESS        = "brightness";
    public static final String PROPERTYNAME_DEPTH             = "depth";
    public static final String PROPERTYNAME_SATURATION        = "saturation";
    public static final String PROPERTYNAME_ELEMENT_LIMIT     = "elementLimit";
    public static final String PROPERTYNAME_ELEMENT_THRESHOLD = "elementThreshold";


    // Constants **************************************************************

    public  static final float MIN_BRIGHTNESS            = 0.3f;
    private static final float DEFAULT_BRIGHTNESS        = 0.9f;
    public  static final float MAX_BRIGHTNESS            = 1.0f;

    public  static final int   MIN_DEPTH                 =  1;
    private static final int   DEFAULT_DEPTH             =  5;
    public  static final int   MAX_DEPTH                 = 10;

    /**
     * The default value for property 'elementLimit'.
     */
    public  static final int   MIN_ELEMENT_LIMIT         =  5;
    private static final int   DEFAULT_ELEMENT_LIMIT     = 13;
    public  static final int   MAX_ELEMENT_LIMIT         = 20;

    public  static final float MIN_ELEMENT_THRESHOLD     = 0.00f;
    private static final float DEFAULT_ELEMENT_THRESHOLD = 0.01f;
    public  static final float MAX_ELEMENT_THRESHOLD     = 0.10f;

    public  static final float MIN_SATURATION            = 0.1f;
    private static final float DEFAULT_SATURATION        = 0.5f;
    public  static final float MAX_SATURATION            = 1.0f;



    // Fields ***************************************************************

    /**
     * Describes the brightness of colors used in the charts.
     */
    private float brightness = DEFAULT_BRIGHTNESS;

    /**
     * Describes the depth of the hierachical charts - currently the ring chart.
     */
    private int depth = DEFAULT_DEPTH;

    /**
     * The upper bound for the number of individual elements in a chart.
     */
    private int elementLimit = DEFAULT_ELEMENT_LIMIT;

    /**
     * The lower bound for the size of elements that shall be displayed.
     */
    private float elementThreshold = DEFAULT_ELEMENT_THRESHOLD;

    /**
     * Describes the saturation of colors used in the charts.
     */
    private float saturation = DEFAULT_SATURATION;


    // Instance Creation ******************************************************

    ChartSettings() {
        // Override default constructor to reduce the visibility.
    }


    // Readers and Writers for Bound Properties *******************************

    /**
     * Returns the brightness of the colors used in the charts.
     *
     * @return the brightness of the colors used in the charts
     */
    public float getBrightness() {
        return brightness;
    }

    /**
     * Sets a new chart color brightness.
     *
     * @param newBrightness   the brightness to be set
     */
    public void setBrightness(float newBrightness) {
        float oldBrightness = getBrightness();
        brightness = newBrightness;
        firePropertyChange(PROPERTYNAME_BRIGHTNESS, oldBrightness, newBrightness);
    }


    /**
     * Returns the depth of hierachical charts. Currently only the ring chart
     * can honor this property.
     */
    public int getDepth() {
        return depth;
    }

    /**
     * Sets a new depth for the hierachical charts.
     *
     * @param newDepth  the depth to be set
     */
    public void setDepth(int newDepth) {
        int oldDepth = getDepth();
        depth = newDepth;
        firePropertyChange(PROPERTYNAME_DEPTH, oldDepth, newDepth);
    }


    /**
     * Returns the maximum number of individual elements per chart.
     *
     * @return the maximum number of individual elements per chart
     */
    public int getElementLimit() {
        return elementLimit;
    }

    /**
     * Sets a new upper bound for the number of individual elements per chart.
     *
     * @param newValue  the new upper bound to be set
     */
    public void setElementLimit(int newValue) {
        float oldValue = getElementLimit();
        elementLimit = newValue;
        firePropertyChange(PROPERTYNAME_ELEMENT_LIMIT, oldValue, newValue);
    }


    /**
     * Returns the threshold for elements to be displayed in a chart.
     * Elements that have a percentual size smaller than the threshold
     * will be hidden - otherwise they are shown.
     *
     * @return the threshold for elements to be displayed in a chart
     */
    public float getElementThreshold() {
        return elementThreshold;
    }

    /**
     * Sets a new threshold for elements to be displayed in charts.
     *
     * @param newThreshold   the threshold to be set
     */
    public void setElementThreshold(float newThreshold) {
        float oldThreshold = getElementThreshold();
        elementThreshold = newThreshold;
        firePropertyChange(PROPERTYNAME_ELEMENT_THRESHOLD, oldThreshold, newThreshold);
    }


    /**
     * Returns the saturation of the colors used for the chart elements.
     *
     * @return the saturation of the color used for the chart elements
     */
    public float getSaturation() {
        return saturation;
    }

    /**
     * Sets a new saturation for the colors used for the chart elements.
     *
     * @param newSaturation   the satuation to be set
     */
    public void setSaturation(float newSaturation) {
        float oldSaturation = getSaturation();
        saturation = newSaturation;
        firePropertyChange(PROPERTYNAME_SATURATION, oldSaturation, newSaturation);
    }


    // Property Persistency ***************************************************


    private static final String KEY_BRIGHTNESS =
        "chart.brightness";

    private static final String KEY_DEPTH =
        "chart.depth";

    private static final String KEY_ELEMENT_LIMIT =
        "chart.elementLimit";

    private static final String KEY_ELEMENT_THRESHOLD =
        "chart.element_threshold";

    private static final String KEY_SATURATION =
        "chart.saturation";


    /**
     * Restores the persistent properties from the specified Preferences.
     *
     * @param prefs   the Preferences object that holds the property values
     */
    public void restoreFrom(Preferences prefs) {
        setBrightness(
            boundedValue(
                prefs.getFloat(KEY_BRIGHTNESS, -1),
                MIN_BRIGHTNESS,
                MAX_BRIGHTNESS,
                DEFAULT_BRIGHTNESS));

        setDepth(
            boundedValue(
                prefs.getInt(KEY_DEPTH, -1),
                MIN_DEPTH,
                MAX_DEPTH,
                DEFAULT_DEPTH));

        setElementLimit(
            boundedValue(
                prefs.getInt(KEY_ELEMENT_LIMIT, -1),
                MIN_ELEMENT_LIMIT,
                MAX_ELEMENT_LIMIT,
                DEFAULT_ELEMENT_LIMIT));

        setElementThreshold(
            boundedValue(
                prefs.getFloat(KEY_ELEMENT_THRESHOLD, -1),
                MIN_ELEMENT_THRESHOLD,
                MAX_ELEMENT_THRESHOLD,
                DEFAULT_ELEMENT_THRESHOLD));

        setSaturation(
            boundedValue(
                prefs.getFloat(KEY_SATURATION, -1),
                MIN_SATURATION,
                MAX_SATURATION,
                DEFAULT_SATURATION));
    }

    /**
     * Stores the persistent properties in the specified Preferences.
     *
     * @param prefs   the Preferences object that holds the property values
     */
    public void storeIn(Preferences prefs) {
        prefs.putFloat(KEY_BRIGHTNESS,        getBrightness());
        prefs.putInt(  KEY_DEPTH,             getDepth());
        prefs.putInt(  KEY_ELEMENT_LIMIT,     getElementLimit());
        prefs.putFloat(KEY_ELEMENT_THRESHOLD, getElementThreshold());
        prefs.putFloat(KEY_SATURATION,        getSaturation());
    }

    // Helper Code ************************************************************

    /**
     * Returns a value that is in the interval as specified
     * by the given lower and upper bounds. Uses a default value
     * if the given value is outside the interval.
     *
     * @param f             the input value
     * @param min           the lower bound
     * @param max           the upper bound
     * @param defaultValue  the value used if the value is outside the bounds
     * @return a value in the given interval
     */
    private static float boundedValue(float f, float min, float max, float defaultValue) {
        return min <= f && f <= max
            ? f
            : defaultValue;
    }

    /**
     * Returns a value that is in the interval as specified
     * by the given lower and upper bounds. Uses a default value
     * if the given value is outside the interval.
     *
     * @param i             the input value
     * @param min           the lower bound
     * @param max           the upper bound
     * @param defaultValue  the value used if the value is outside the bounds
     * @return a value in the given interval
     */
    private static int boundedValue(int i, int min, int max, int defaultValue) {
        return min <= i && i <= max
            ? i
            : defaultValue;
    }


}