/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport.model.settings;

import java.util.prefs.Preferences;

import com.jgoodies.binding.beans.BeanAdapter;
import com.jgoodies.binding.beans.Model;
import com.jgoodies.uif.action.ToggleAction;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;

/**
 * Provides bound properties for GUI related settings.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.11 $
 */
public final class GUISettings extends Model {

    // Names of the Bound Bean Properties *************************************

    public static final String PROPERTYNAME_BOLDNESS_THRESHOD =
        "boldnessThreshold";
    public static final String PROPERTYNAME_COLLAPSE_DESELECTED_PATHS =
        "collapseDeselectedPaths";
    public static final String PROPERTYNAME_EXPAND_SELECTED_PATHS =
        "expandSelectedPaths";
    public static final String PROPERTYNAME_STATISTICS_DEPTH =
        "statisticsDepth";

    public static final String PROPERTYNAME_SORT_MODE  = "sortMode";
    public static final String PROPERTYNAME_SIZE_MODE  = "sizeMode";
    public static final String PROPERTYNAME_SHOW_FILES = "showFiles";
    public static final String PROPERTYNAME_NODE_MODE  = "nodeMode";
    public static final String PROPERTYNAME_VIEW_MODE  = "viewMode";


    // Resources **************************************************************

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(GUISettings.class);


    // Enumerations ***********************************************************

    public enum SortMode { BY_SIZE, BY_NAME }

    public enum SizeMode { SIZE, COUNT }

    public enum NodeMode { PLAIN, SIZE, PERCENT }

    public enum ViewMode {

        OVERVIEW,
        SIZE_TREE_PIE,
        SIZE_TREE_MAP,

        SIZE_PIE,
        SIZE_BAR,
        SIZE_TABLE,

        TOP50_LARGEST,
        TOP50_OLDEST,
        TOP50_NEWEST,

        SIZE_DISTRIBUTION_PIE,
        SIZE_DISTRIBUTION_BAR,
        SIZE_DISTRIBUTION_TABLE,

        TIME_DISTRIBUTION_PIE,
        TIME_DISTRIBUTION_BAR,
        TIME_DISTRIBUTION_TABLE,

        TYPE_DISTRIBUTION_PIE,
        TYPE_DISTRIBUTION_BAR,
        TYPE_DISTRIBUTION_TABLE }


    // Constants **************************************************************

    /**
     * The default value (0.10f) for property 'boldnessThreshold'.
     */
    private static final float DEFAULT_BOLDNESS_THRESHOLD = 0.10f;

    /**
     * The lower bound for the boldness threshold.
     */
    static final float MIN_BOLDNESS_THRESHOLD = 0.00f;

    /**
     * The upper bound for the boldness threshold.
     */
    static final float MAX_BOLDNESS_THRESHOLD = 1.00f;


    private static final boolean DEFAULT_COLLAPSE_DESELECTED_PATHS = true;
    private static final boolean DEFAULT_EXPAND_SELECTED_PATHS   = true;

    private static final SortMode DEFAULT_SORT_MODE = SortMode.BY_SIZE;
    private static final SizeMode DEFAULT_SIZE_MODE = SizeMode.SIZE;
    private static final NodeMode DEFAULT_NODE_MODE = NodeMode.PLAIN;
    private static final ViewMode DEFAULT_VIEW_MODE = ViewMode.SIZE_PIE;
    private static final boolean  DEFAULT_SHOW_FILES = true;

    public  static final int MIN_STATISTICS_DEPTH     = 2;
    private static final int DEFAULT_STATISTICS_DEPTH = 3;
    public  static final int MAX_STATISTICS_DEPTH     = 8;


    // Fields ***************************************************************

    private final ChartSettings chartSettings;

    /**
     * Describes a threshold that is used to render size nodes plain or bold.
     */
    private float boldnessThreshold = DEFAULT_BOLDNESS_THRESHOLD;

    /**
     * Describes whether a path shall be collapsed on deselection.
     */
    private boolean collapseDeselectedPaths = DEFAULT_COLLAPSE_DESELECTED_PATHS;

    /**
     * Describes whether a path shall be expanded on selection.
     */
    private boolean expandSelectedPaths = DEFAULT_EXPAND_SELECTED_PATHS;

    private SortMode sortMode = DEFAULT_SORT_MODE;
    private SizeMode sizeMode = DEFAULT_SIZE_MODE;

    /**
     * Describes how directory nodes are rendered: plain, size, percent.
     */
    private NodeMode nodeMode = DEFAULT_NODE_MODE;

    /**
     * Holds a representation of the chosen presentation.
     */
    private ViewMode viewMode = DEFAULT_VIEW_MODE;

    /**
     * Describes whether files are shown or collapsed in the views.
     */
    private boolean showFiles = DEFAULT_SHOW_FILES;

    /**
     * Describes for how many levels we collect and store
     * the statistics: top lists, distributions and types.
     */
    private int statisticsDepth = DEFAULT_STATISTICS_DEPTH;


    // Toggle Actions *********************************************************

    private ToggleAction sortModeSizeAction;
    private ToggleAction sortModeNameAction;
    private ToggleAction sizeModeSizeAction;
    private ToggleAction sizeModeCountAction;
    private ToggleAction showFilesAction;


    // Instance Creation ******************************************************

    public GUISettings() {
        this.chartSettings = new ChartSettings();
        initializeToggleActions();
    }


    private void initializeToggleActions() {
        BeanAdapter<GUISettings> adapter = new BeanAdapter<GUISettings>(this, true);

        sortModeSizeAction = ToggleAction.createRadio(
                "sortMode.bySize",
                RESOURCES,
                adapter.getValueModel(PROPERTYNAME_SORT_MODE),
                SortMode.BY_SIZE);
        sortModeNameAction = ToggleAction.createRadio(
                "sortMode.byName",
                RESOURCES,
                adapter.getValueModel(PROPERTYNAME_SORT_MODE),
                SortMode.BY_NAME);
        sizeModeSizeAction = ToggleAction.createRadio(
                "sizeMode.size",
                RESOURCES,
                adapter.getValueModel(PROPERTYNAME_SIZE_MODE),
                SizeMode.SIZE);
        sizeModeCountAction = ToggleAction.createRadio(
                "sizeMode.count",
                RESOURCES,
                adapter.getValueModel(PROPERTYNAME_SIZE_MODE),
                SizeMode.COUNT);
        showFilesAction = ToggleAction.createCheck(
                "showFiles",
                RESOURCES,
                adapter.getValueModel(PROPERTYNAME_SHOW_FILES));
    }


    // Exposing Sub Models *****************************************************

    /**
     * Returns the Bean that describes the ChartSettings.
     *
     * @return the Bean that describes the chart properties
     */
    public ChartSettings chartSettings() {
        return chartSettings;
    }


    // Exposing Toggle Actions ************************************************

    public ToggleAction getSortModeSizeAction() {
        return sortModeSizeAction;
    }


    public ToggleAction getSortModeNameAction() {
        return sortModeNameAction;
    }


    public ToggleAction getSizeModeSizeAction() {
        return sizeModeSizeAction;
    }


    public ToggleAction getSizeModeCountAction() {
        return sizeModeCountAction;
    }


    public ToggleAction getShowFilesAction() {
        return showFilesAction;
    }


    // Accessors for Bound Properties *****************************************

    /**
     * Returns the threshold that indicates when a directory node
     * shall be emphasized. Nodes with a percentage large than
     * this threshold will be rendered with a bold font.
     *
     * @return a size threshold used to render nodes plain or bold
     */
    public float getBoldnessThreshold() {
        return boldnessThreshold;
    }

    /**
     * Sets a new boldness threshold.
     *
     * @param newValue  the boldness threshold to be set
     */
    public void setBoldnessThreshold(float newValue) {
        float oldValue = getBoldnessThreshold();
        boldnessThreshold = newValue;
        firePropertyChange(PROPERTYNAME_BOLDNESS_THRESHOD, oldValue, newValue);
    }


    /**
     * Returns whether the navigator collapses deselected paths.
     */
    public boolean getCollapseDeselectedPaths() {
        return collapseDeselectedPaths;
    }

    /**
     * Enables or disables that deselected paths are collapsed automatically.
     *
     * @param newValue   true to enable the auto collapse, false to disable it
     */
    public void setCollapseDeselectedPaths(boolean newValue) {
        boolean oldValue = getCollapseDeselectedPaths();
        collapseDeselectedPaths = newValue;
        firePropertyChange(PROPERTYNAME_COLLAPSE_DESELECTED_PATHS, oldValue, newValue);
    }


    /**
     * Returns if the navigator expands paths on selection.
     *
     * @return true if selected paths are expanded, false otherwise
     */
    public boolean getExpandSelectedPaths() {
        return expandSelectedPaths;
    }

    /**
     * Enables or disables the automatic expansion of selected paths.
     *
     * @param newValue   true to enable the auto expansion, false to disable it
     */
    public void setExpandSelectedPaths(boolean newValue) {
        boolean oldValue = getExpandSelectedPaths();
        expandSelectedPaths = newValue;
        firePropertyChange(PROPERTYNAME_EXPAND_SELECTED_PATHS, oldValue, newValue);
    }


    /**
     * Returns the current sort mode, one of: by name, by size or unsorted.
     *
     * @return the current sort mode
     */
    public SortMode getSortMode() {
        return sortMode;
    }

    /**
     * Sets a new sort mode: by name, by size or unsorted.
     *
     * @param newMode  the sort mode to be set
     */
    public void setSortMode(SortMode newMode) {
        SortMode oldMode = getSortMode();
        sortMode = newMode;
        firePropertyChange(PROPERTYNAME_SORT_MODE, oldMode, newMode);
    }


    /**
     * Returns the current size mode: size vs. count.
     *
     * @return the current size mode
     */
    public SizeMode getSizeMode() {
        return sizeMode;
    }

    /**
     * Sets a new size mode: size vs. count.
     *
     * @param newMode  the size mode to be set
     */
    public void setSizeMode(SizeMode newMode) {
        SizeMode oldMode = getSizeMode();
        sizeMode = newMode;
        firePropertyChange(PROPERTYNAME_SIZE_MODE, oldMode, newMode);
    }


    /**
     * Returns the mode that indicates how directory nodes are rendered
     * in the navigation tree: plain, percent, size.
     *
     * @return the mode that indicates how directory nodes are rendered
     */
    public NodeMode getNodeMode() {
        return nodeMode;
    }

    /**
     * Sets a new mode for the rendering of directory nodes.
     *
     * @param newMode  the mode to be set
     */
    public void setNodeMode(NodeMode newMode) {
        NodeMode oldMode = getNodeMode();
        nodeMode = newMode;
        firePropertyChange(PROPERTYNAME_NODE_MODE, oldMode, newMode);
    }


    /**
     * Returns the current view mode used to determine which
     * presentation is shown.
     *
     * @return the current view mode
     */
    public ViewMode getViewMode() {
        return viewMode;
    }

    /**
     * Sets a new view mode.
     *
     * @param newMode  the view mode to be set
     */
    public void setViewMode(ViewMode newMode) {
        ViewMode oldMode = getViewMode();
        viewMode = newMode;
        firePropertyChange(PROPERTYNAME_VIEW_MODE, oldMode, newMode);
    }


    /**
     * Returns if individual files are shown in the relevant views.
     *
     * @return true if files are shown, false if they are collapsed
     *    to a synthetic node "Files in this folder".
     */
    public boolean getShowFiles() {
        return showFiles;
    }

    /**
     * Sets whether individual files shall be shown in the relevant views,
     * or if they shall be collapsed to a synthetic node "Files in this folder".
     *
     * @param newValue   true to show leafs, false to hide leafs
     */
    public void setShowFiles(boolean newValue) {
        boolean oldValue = getShowFiles();
        showFiles = newValue;
        firePropertyChange(PROPERTYNAME_SHOW_FILES, oldValue, newValue);
    }


    /**
     * Returns the current statistics depth that indicates
     * for how many levels we collect and store the cumulated
     * statistics: top lists, distributions and types.
     *
     * @return the number of levels we provide the statistics
     */
    public int getStatisticsDepth() {
        return statisticsDepth;
    }

    /**
     * Sets a new statistics depth.
     *
     * @param newDepth   the statistics depth to be set
     */
    public void setStatisticsDepth(int newDepth) {
        int oldDepth = getStatisticsDepth();
        statisticsDepth = newDepth;
        firePropertyChange(PROPERTYNAME_STATISTICS_DEPTH, oldDepth, newDepth);
    }


    // Persistency ************************************************************

    private static final String KEY_BOLDNESS_THRESHOLD =
        "boldness_threshold";

    private static final String KEY_COLLAPSE_DESELECTED_PATHS =
        "collapse_deselected_paths";

    private static final String KEY_EXPAND_SELECTED_PATHS =
        "expand_selected_paths";

    private static final String KEY_SORT_MODE = "sort_mode";
    private static final String KEY_SIZE_MODE = "size_mode";
    private static final String KEY_NODE_MODE = "node_mode";
    private static final String KEY_VIEW_MODE = "view_mode";

    private static final String KEY_SHOW_FILES = "show_files";

    private static final String KEY_STATISTICS_DEPTH = "statistics.depth";

    /**
     * Restores the persistent properties from the specified Preferences.
     *
     * @param prefs   the Preferences object that holds the property values
     */
    public void restoreFrom(Preferences prefs) {
        chartSettings().restoreFrom(prefs);

        setBoldnessThreshold(boundedValue(
                prefs.getFloat(KEY_BOLDNESS_THRESHOLD, -1),
                MIN_BOLDNESS_THRESHOLD,
                MAX_BOLDNESS_THRESHOLD,
                DEFAULT_BOLDNESS_THRESHOLD));

        setCollapseDeselectedPaths(prefs.getBoolean(
                KEY_COLLAPSE_DESELECTED_PATHS,
                DEFAULT_COLLAPSE_DESELECTED_PATHS));

        setExpandSelectedPaths(prefs.getBoolean(
                KEY_EXPAND_SELECTED_PATHS,
                DEFAULT_EXPAND_SELECTED_PATHS));

        setSortMode(SortMode.valueOf(prefs.get(KEY_SORT_MODE, "")));
        setSizeMode(SizeMode.valueOf(prefs.get(KEY_SIZE_MODE, "")));
        setNodeMode(NodeMode.valueOf(prefs.get(KEY_NODE_MODE, "")));
        setViewMode(ViewMode.valueOf(prefs.get(KEY_VIEW_MODE, "")));

        setShowFiles(prefs.getBoolean(
                KEY_SHOW_FILES,
                DEFAULT_SHOW_FILES));

        setStatisticsDepth(boundedValue(
                prefs.getInt(KEY_STATISTICS_DEPTH, -1),
                MIN_STATISTICS_DEPTH,
                MAX_STATISTICS_DEPTH,
                DEFAULT_STATISTICS_DEPTH));
    }

    /**
     * Stores the persistent properties in the specified Preferences.
     *
     * @param prefs   the Preferences object that holds the property values
     */
    public void storeIn(Preferences prefs) {
        chartSettings().storeIn(prefs);

        prefs.putFloat(KEY_BOLDNESS_THRESHOLD,
            getBoldnessThreshold());

        prefs.putBoolean(KEY_COLLAPSE_DESELECTED_PATHS,
            getCollapseDeselectedPaths());

        prefs.putBoolean(KEY_EXPAND_SELECTED_PATHS,
            getExpandSelectedPaths());

        prefs.put(KEY_SORT_MODE, getSortMode().name());
        prefs.put(KEY_SIZE_MODE, getSizeMode().name());
        prefs.put(KEY_NODE_MODE, getNodeMode().name());
        prefs.put(KEY_VIEW_MODE, getViewMode().name());

        prefs.putBoolean(KEY_SHOW_FILES, getShowFiles());

        prefs.put(KEY_SORT_MODE, getSortMode().name());

        prefs.putInt(KEY_STATISTICS_DEPTH, getStatisticsDepth());
    }

    // Helper Code ************************************************************

    /**
     * Returns a value that is in the interval as specified
     * by the given lower and upper bounds. Uses a default value
     * if the given value is outside the interval.
     *
     * @param f             the input value
     * @param min           the lower bound
     * @param max           the upper bound
     * @param defaultValue  the value used if the value is outside the bounds
     * @return a value in the given interval
     */
    private static float boundedValue(float f, float min, float max, float defaultValue) {
        return min <= f && f <= max
            ? f
            : defaultValue;
    }

    /**
     * Returns a value that is in the interval as specified
     * by the given lower and upper bounds. Uses a default value
     * if the given value is outside the interval.
     *
     * @param i             the input value
     * @param min           the lower bound
     * @param max           the upper bound
     * @param defaultValue  the value used if the value is outside the bounds
     * @return a value in the given interval
     */
    private static int boundedValue(int i, int min, int max, int defaultValue) {
        return min <= i && i <= max
            ? i
            : defaultValue;
    }


}