/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport.model.statistic;

import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import com.jgoodies.jdiskreport.model.node.FileNode;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;


/**
 * An abstract superclass for the size and modified distributions.
 * These information are redundant and can be computed
 * from the domain tree of directories and file infos.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.10 $
 */
public abstract class AbstractDistribution {


    private static final ResourceMap RESOURCES =
        Application.getResourceMap(AbstractDistribution.class);


    // Instance Fields ********************************************************

    private final long[] sizes;
    private final long[] counts;
    private long sizeTotal;
    private long countTotal;
    private TableModel tableModel;


    // Instance Creation ******************************************************

    /**
     * Constructs an empty distribution with sizes and counts set to 0.
     */
    protected AbstractDistribution() {
        this.sizes  = new long[getRowCount()];
        this.counts = new long[getRowCount()];
    }


    // Access *****************************************************************

    /**
     * Lazily creates and returns a table model for this size distribution.
     *
     * @return a TableModel that represents this SizeDistribution
     */
    public final TableModel getTableModel() {
        if (tableModel == null) {
            tableModel = new ArrayDistributionTableModel(this);
        }
        return tableModel;
    }

    final long sizeTotal() {
        return sizeTotal;
    }

    final long sizeAt(int rowIndex) {
        return sizes[rowIndex];
    }


    final long countTotal() {
        return countTotal;
    }


    final long countAt(int rowIndex) {
        return counts[rowIndex];
    }


    // Abstract Behavior ******************************************************

    protected abstract int getRowCount();


    String getKeyPrefix() {
        return getClass().getSimpleName();
    }


    protected abstract void add(FileNode fileNode);

    // Updating ***************************************************************


    final void add(AbstractDistribution distribution) {
        int rowCount = getRowCount();
        for (int i = 0; i < rowCount; i++) {
            add(i, distribution.sizeAt(i), distribution.countAt(i));
        }
    }


    protected final void add(int rowIndex, long size, long count) {
        sizes [rowIndex] += size;
        counts[rowIndex] += count;
        sizeTotal  += size;
        countTotal += count;
    }


    // String Representation **************************************************

    @Override
    public String toString() {
        TableModel model = getTableModel();
        StringBuilder builder = new StringBuilder();
        builder.append(getClass().getName());
        for (int row = 0; row < getRowCount(); row++) {
            builder.append("\n");
            builder.append(model.getValueAt(row, 0));
            builder.append(": size=");
            builder.append(model.getValueAt(row, 1));
            builder.append("; count=");
            builder.append(model.getValueAt(row, 3));
        }
        builder.append("\nTotal: size=" + sizeTotal());
        builder.append("; count=" + countTotal());
        return builder.toString();
    }


    // TableModel *************************************************************

    final static class ArrayDistributionTableModel implements TableModel {

        private static final String[] COLUMN_NAME_KEY_SUFFIXES =
            { "title", "size", "sizePercent", "count", "countPercent" };


        private static final Class<?>[] COLUMN_CLASSES =
            { String.class, Long.class, Float.class, Long.class, Float.class };


        private final AbstractDistribution distribution;

        ArrayDistributionTableModel(AbstractDistribution distribution) {
            this.distribution = distribution;
        }

        public int getColumnCount() {
            return COLUMN_NAME_KEY_SUFFIXES.length;
        }

        public String getColumnName(int columnIndex) {
            String key = distribution.getKeyPrefix()
                + ".columnName."
                + COLUMN_NAME_KEY_SUFFIXES[columnIndex];
            return RESOURCES.getString(key);
        }

        public Class<?> getColumnClass(int columnIndex) {
            return COLUMN_CLASSES[columnIndex];
        }

        public int getRowCount() {
            return distribution.getRowCount();
        }

        private String getRowName(int rowIndex) {
            String key = distribution.getKeyPrefix()
                + ".rowName["
                + rowIndex + "]";
            return RESOURCES.getString(key);
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            switch (columnIndex) {
                case 0 :
                    return getRowName(rowIndex);

                case 1 :
                    return Long.valueOf(distribution.sizeAt(rowIndex));

                case 2 :
                    float size = distribution.sizeAt(rowIndex);
                    long sizeTotal = distribution.sizeTotal;
                    return Float.valueOf(sizeTotal == 0L ? 1f : (size / sizeTotal));

                case 3 :
                    return Long.valueOf(distribution.countAt(rowIndex));

                case 4 :
                    float count = distribution.countAt(rowIndex);
                    long countTotal = distribution.countTotal;
                    return Float.valueOf(countTotal == 0L ? 1f : (count / countTotal));

                default :
                    throw new IllegalStateException("Unknown column index: " + columnIndex);
            }
        }

        public void setValueAt(Object value, int rowIndex, int columnIndex) {
            throw new UnsupportedOperationException("Distribution table models are not editable.");
        }

        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return false;
        }

        public void addTableModelListener(TableModelListener l) {
            // Do nothing; this model is immutable.
        }

        public void removeTableModelListener(TableModelListener l) {
            // Do nothing; this model is immutable.
        }

    }


}
