/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport.model.statistic;

import java.util.List;

import com.jgoodies.jdiskreport.model.node.FileNode;


/**
 * Holds the top 50 largest, oldest, and newest files,
 * as well as the distributions for size, time, and type.
 * All these information are redundant and can be computed
 * from the domain tree of directories and file infos.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.6 $
 *
 * @see TopList
 * @see AbstractDistribution
 */
public final class Statistics {

    private final TopList largestFiles;
    private final TopList oldestFiles;
    private final TopList newestFiles;

    private final AbstractDistribution sizeDistribution;
    private final AbstractDistribution timeDistribution;
    private final TypeDistribution typeDistribution;


    // Instance Creation ******************************************************

    /**
     * Constructs a statistics object for the given substatistics.
     */
    private Statistics(
            TopList largestFiles,
            TopList oldestFiles,
            TopList newestFiles,
            AbstractDistribution sizeDistribution,
            AbstractDistribution timeDistribution,
            TypeDistribution typeDistribution) {
        this.largestFiles = largestFiles;
        this.oldestFiles  = oldestFiles;
        this.newestFiles  = newestFiles;
        this.sizeDistribution = sizeDistribution;
        this.timeDistribution = timeDistribution;
        this.typeDistribution = typeDistribution;
    }


    public static Statistics createFrom(FileNode[] allFiles) {
        LimitedTreeSet<FileNode> top50LargestSet =
            new LimitedTreeSet<FileNode>(50, TopList.LARGEST_COMPARATOR);
        LimitedTreeSet<FileNode> top50OldestSet =
            new LimitedTreeSet<FileNode>(50, TopList.OLDEST_COMPARATOR);
        LimitedTreeSet<FileNode> top50NewestSet =
            new LimitedTreeSet<FileNode>(50, TopList.NEWEST_COMPARATOR);

        AbstractDistribution sizeDistribution = new SizeDistribution();
        AbstractDistribution timeDistribution = new TimeDistribution();
        TypeDistribution typeDistribution = new TypeDistribution();

        for (FileNode fileNode : allFiles) {
            top50LargestSet.add(fileNode);
            top50OldestSet.add(fileNode);
            top50NewestSet.add(fileNode);
            sizeDistribution.add(fileNode);
            timeDistribution.add(fileNode);
            typeDistribution.add(fileNode);
        }
        return new Statistics(
                new TopList(top50LargestSet),
                new TopList(top50OldestSet),
                new TopList(top50NewestSet),
                sizeDistribution,
                timeDistribution,
                typeDistribution);
    }


    public static Statistics createFrom(List<Statistics> allStatistics) {
        LimitedTreeSet<FileNode> top50LargestSet =
            new LimitedTreeSet<FileNode>(50, TopList.LARGEST_COMPARATOR);
        LimitedTreeSet<FileNode> top50OldestSet =
            new LimitedTreeSet<FileNode>(50, TopList.OLDEST_COMPARATOR);
        LimitedTreeSet<FileNode> top50NewestSet =
            new LimitedTreeSet<FileNode>(50, TopList.NEWEST_COMPARATOR);

        AbstractDistribution sizeDistribution = new SizeDistribution();
        AbstractDistribution timeDistribution = new TimeDistribution();
        TypeDistribution typeDistribution = new TypeDistribution();

        for (Statistics statistics : allStatistics) {
            for (FileNode fileNode : statistics.getLargestFiles().getFiles()) {
                if (!top50LargestSet.addItem(fileNode)) {
                    break;
                }
            }
            for (FileNode fileNode : statistics.getOldestFiles().getFiles()) {
                if (!top50OldestSet.addItem(fileNode)) {
                    break;
                }
            }
            for (FileNode fileNode : statistics.getNewestFiles().getFiles()) {
                if (!top50NewestSet.addItem(fileNode)) {
                    break;
                }
            }
            sizeDistribution.add(statistics.getSizeDistribution());
            timeDistribution.add(statistics.getTimeDistribution());
            typeDistribution.add(statistics.getTypeDistribution());
        }
        return new Statistics(
                new TopList(top50LargestSet),
                new TopList(top50OldestSet),
                new TopList(top50NewestSet),
                sizeDistribution,
                timeDistribution,
                typeDistribution);
    }


    // Accessors **************************************************************

    public TopList getLargestFiles() {
        return largestFiles;
    }


    public TopList getOldestFiles() {
        return oldestFiles;
    }


    public TopList getNewestFiles() {
        return newestFiles;
    }


    public AbstractDistribution getSizeDistribution() {
        return sizeDistribution;
    }


    public AbstractDistribution getTimeDistribution() {
        return timeDistribution;
    }


    public TypeDistribution getTypeDistribution() {
        return typeDistribution;
    }


    // Building Statistics ****************************************************

//    private void addAll(FileNode[] allFiles) {
//        getTop50Largest().addAll(allFiles);
//        getTop50Oldest().addAll(allFiles);
//        getTop50Newest().addAll(allFiles);
//        getSizeDistribution().addAll()
//    }


}
