/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport.model.statistic;

import java.util.Calendar;
import java.util.Date;

import com.jgoodies.jdiskreport.model.node.FileNode;


/**
 * Implements the modified distributions.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.6 $
 */
final class TimeDistribution extends AbstractDistribution {


    // Constants **************************************************************

    private static final long[] ROW_BOUNDS = computeRowBounds();


    // Instance Creation ******************************************************

    /**
     * Constructs an empty ModifiedDistribution with sizes and counts set to 0.
     */
    TimeDistribution() {
        // Just invoke the super constructor.
    }


    // Implementing Abstract Behavior *****************************************

    @Override
    protected int getRowCount() {
        return ROW_BOUNDS.length;
    }


    @Override
    protected void add(FileNode fileNode) {
        long lastModified = fileNode.getLastModified();
        long size = fileNode.getSize();
        int rowCount = getRowCount();
        for (int i = rowCount - 1; i >= 0; i--) {
            if (lastModified > ROW_BOUNDS[i]) {
                add(i, size, 1L);
                return;
            }
        }
    }


    // Helper Code ************************************************************

    private static long[] computeRowBounds() {
        long today;
        long yesterday;
        long seven_days_ago;
        long thirty_days_ago;
        long ninety_days_ago;
        long hundredeighty_days_ago;
        long one_year_ago;
        long two_years_ago;
        long three_years_ago;
        long four_years_ago;
        long five_years_ago;
        long ten_years_ago;
        long twenty_years_ago;

        long now = System.currentTimeMillis();
        Calendar cal = Calendar.getInstance();
        Calendar calToday = Calendar.getInstance();
        cal.setTime(new Date(now));

        // Today
        calToday.set(Calendar.HOUR_OF_DAY, 0);
        calToday.set(Calendar.MINUTE, 0);
        calToday.set(Calendar.SECOND, 0);
        cal.setTime(calToday.getTime());
        today = cal.getTime().getTime();

        // Yesterday
        cal.add(Calendar.DATE, -1);
        yesterday = cal.getTime().getTime();

        // 7 days ago
        cal.add(Calendar.DATE, -6);
        seven_days_ago = cal.getTime().getTime();

        // 30 days ago
        cal.add(Calendar.DATE, -23);
        thirty_days_ago = cal.getTime().getTime();

        // 90 days ago
        cal.add(Calendar.DATE, -60);
        ninety_days_ago = cal.getTime().getTime();

        // 180 days ago
        cal.add(Calendar.DATE, -90);
        hundredeighty_days_ago = cal.getTime().getTime();

        // 1 year ago
        cal.setTime(calToday.getTime());
        cal.add(Calendar.YEAR, -1);
        one_year_ago = cal.getTime().getTime();

        // 2 years ago
        cal.add(Calendar.YEAR, -1);
        two_years_ago = cal.getTime().getTime();

        // 3 years ago
        cal.add(Calendar.YEAR, -1);
        three_years_ago = cal.getTime().getTime();

        // 4 years ago
        cal.add(Calendar.YEAR, -1);
        four_years_ago = cal.getTime().getTime();

        // 5 days ago
        cal.add(Calendar.YEAR, -1);
        five_years_ago = cal.getTime().getTime();

        // 10 years ago
        cal.add(Calendar.YEAR, -5);
        ten_years_ago = cal.getTime().getTime();

        // 20 years ago
        cal.add(Calendar.YEAR, -10);
        twenty_years_ago = cal.getTime().getTime();

        return new long[]{
                0,
                twenty_years_ago,
                ten_years_ago,
                five_years_ago,
                four_years_ago,
                three_years_ago,
                two_years_ago,
                one_year_ago,
                hundredeighty_days_ago,
                ninety_days_ago,
                thirty_days_ago,
                seven_days_ago,
                yesterday,
                today};
    }


}
