/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport.model.statistic;

import java.util.*;

import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import com.jgoodies.jdiskreport.model.node.DirectoryNode;
import com.jgoodies.jdiskreport.model.node.FileNode;
import com.jgoodies.jdiskreport.model.util.Formats;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;



/**
 * Describes a list with a limited amount of elements.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.10 $
 */
public final class TopList {

    public static final Comparator<FileNode> LARGEST_COMPARATOR =
        new LargestComparator();

    public static final Comparator<FileNode> OLDEST_COMPARATOR =
        new OldestComparator();

    public static final Comparator<FileNode> NEWEST_COMPARATOR =
        new NewestComparator();


    private static final ResourceMap RESOURCES =
        Application.getResourceMap(TopList.class);


    private final List<FileNode> files;

    private TableModel tableModel;


    // Instance Creation ******************************************************

    TopList(SortedSet<FileNode> sortedSet) {
        ArrayList<FileNode> sortedList = new ArrayList<FileNode>(sortedSet);
        sortedList.trimToSize();
        this.files = Collections.unmodifiableList(sortedList);
    }


    // Accessors **************************************************************

    List<FileNode> getFiles() {
        return files;
    }


    public FileNode first() {
        return files.get(0);
    }


    /**
     * Lazily creates and returns a table model for this top list.
     *
     * @return a TableModel that represents this top list
     */
    public TableModel getTableModel() {
        if (tableModel == null) {
            tableModel = new TopListTableModel(getFiles());
        }
        return tableModel;
    }


    // String Representation **************************************************

    @SuppressWarnings("boxing")
    @Override
    public String toString() {
        TableModel model = getTableModel();
        StringBuilder builder = new StringBuilder();
        builder.append(getClass().getName());
        for (int row = 0; row < model.getRowCount(); row++) {
            builder.append("\n");
            builder.append(model.getValueAt(row, 0));
            builder.append(": name=");
            builder.append(model.getValueAt(row, 1));
            builder.append("; size=");
            builder.append(Formats.formatSize((Long) model.getValueAt(row, 2)));
            builder.append("; modified=");
            builder.append(Formats.formatDate((Long) model.getValueAt(row, 3)));
            builder.append("; file=");
            builder.append(model.getValueAt(row, 4));
        }
        return builder.toString();
    }


    // TableModel *************************************************************

    final static class TopListTableModel implements TableModel {

        private static final String COLUMN_NAME_KEY_PREFIX =
            TopList.class.getSimpleName();

        private static final String[] COLUMN_NAME_KEY_SUFFIXES =
            { "name", "size", "modified", "path" };


        private static final Class<?>[] COLUMN_CLASSES =
            { FileNode.class, Long.class, Long.class, DirectoryNode.class };


        private final List<FileNode> files;

        TopListTableModel(List<FileNode> files) {
            this.files = files;
        }

        public int getColumnCount() {
            return COLUMN_NAME_KEY_SUFFIXES.length;
        }

        public String getColumnName(int columnIndex) {
            String key = COLUMN_NAME_KEY_PREFIX
                + ".columnName."
                + COLUMN_NAME_KEY_SUFFIXES[columnIndex];
            return RESOURCES.getString(key);
        }

        public Class<?> getColumnClass(int columnIndex) {
            return COLUMN_CLASSES[columnIndex];
        }

        public int getRowCount() {
            return files.size();
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            FileNode node = files.get(rowIndex);
            switch (columnIndex) {
                case 0 :
                    return node;

                case 1 :
                    return Long.valueOf(node.getSize());

                case 2 :
                    return Long.valueOf(node.getLastModified());

                case 3 :
                    return node.getParent();

                default :
                    throw new IllegalStateException("Unknown column index: " + columnIndex);
            }
        }

        public void setValueAt(Object value, int rowIndex, int columnIndex) {
            throw new UnsupportedOperationException("TopList table models are not editable.");
        }

        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return false;
        }

        public void addTableModelListener(TableModelListener l) {
            // Do nothing; this model is immutable.
        }

        public void removeTableModelListener(TableModelListener l) {
            // Do nothing; this model is immutable.
        }

    }


    // Comparators ************************************************************

    private static final class LargestComparator implements Comparator<FileNode> {

        public int compare(FileNode node1, FileNode node2) {
            // First compare the sizes
            long size1 = node1.getSize();
            long size2 = node2.getSize();
            if (size1 < size2) {
                return 1;
            } else if (size1 > size2) {
                return -1;
            } else {
                return node1.getFile().compareTo(node2.getFile());
            }
        }
    }


    private static final class OldestComparator implements Comparator<FileNode> {

        public int compare(FileNode node1, FileNode node2) {
            // First compare the modification dates
            long size1 = node1.getLastModified();
            long size2 = node2.getLastModified();
            if (size1 < size2) {
                return -1;
            } else if (size1 > size2) {
                return 1;
            } else {
                return node1.getFile().compareTo(node2.getFile());
            }
        }
    }


    private static final class NewestComparator implements Comparator<FileNode> {

        public int compare(FileNode node1, FileNode node2) {
            // First compare the modification dates
            long size1 = node1.getLastModified();
            long size2 = node2.getLastModified();
            if (size1 < size2) {
                return 1;
            } else if (size1 > size2) {
                return -1;
            } else {
                return node1.getFile().compareTo(node2.getFile());
            }
        }
    }


}
