/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport.model.statistic;

import java.util.*;

import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import com.jgoodies.jdiskreport.model.node.FileNode;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;


/**
 * An abstract superclass for the size and modified distributions.
 * These information are redundant and can be computed
 * from the domain tree of directories and file infos.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.8 $
 */
public final class TypeDistribution {


    private static final ResourceMap RESOURCES =
        Application.getResourceMap(TypeDistribution.class);


    // Instance Fields ********************************************************

    private Map<String, FileType> map;
    private FileType[] typesBySize;
    private FileType[] typesByCount;
    private long sizeTotal;
    private long countTotal;
    private TableModel tableModelBySize;
    private TableModel tableModelByCount;


    // Instance Creation ******************************************************

    /**
     * Constructs an empty type distribution that contains no file types.
     */
    protected TypeDistribution() {
        this.map = new HashMap<String, FileType>();
    }


    // Access *****************************************************************

    /**
     * Lazily creates and returns a table model for this type distribution
     * sorted by size.
     *
     * @return a TableModel for file types sorted by size
     */
    public TableModel getTableModelBySize() {
        if (tableModelBySize == null) {
            tableModelBySize = new ArrayDistributionTableModel(this, typesBySize);
        }
        return tableModelBySize;
    }

    /**
     * Lazily creates and returns a table model for this type distribution
     * sorted by size.
     *
     * @return a TableModel for file types sorted by size
     */
    public TableModel getTableModelByCount() {
        if (tableModelByCount == null) {
            tableModelByCount = new ArrayDistributionTableModel(this, typesByCount);
        }
        return tableModelByCount;
    }

    long sizeTotal() {
        return sizeTotal;
    }

    long countTotal() {
        return countTotal;
    }


    String getKeyPrefix() {
        return getClass().getSimpleName();
    }


    void add(FileNode fileNode) {
        String suffix = fileNode.getSuffix();
        FileType type = map.get(suffix);
        if (type == null) {
            type = new FileType(suffix, fileNode.getSize(), fileNode.getFileCount());
            map.put(suffix, type);
        } else {
            type.add(fileNode.getSize());
        }
    }


    void add(TypeDistribution distribution) {
        for (Map.Entry<String, FileType> entry : distribution.map.entrySet()) {
            String entrySuffix = entry.getKey();
            FileType entryType = entry.getValue();
            FileType type = map.get(entrySuffix);
            if (type == null) {
                map.put(entrySuffix, entryType);
            } else {
                type.add(entryType.size, entryType.count);
            }
        }
    }


    public void replaceMapByArrays() {
        int size = map.size();
        List<FileType> types = new ArrayList<FileType>(size);
        for (FileType type : map.values()) {
            types.add(type);
        }
        Collections.sort(types, SIZE_COMPARATOR);
        typesBySize = types.toArray(new FileType[types.size()]);
        Collections.sort(types, COUNT_COMPARATOR);
        typesByCount = types.toArray(new FileType[types.size()]);
        map = null;
    }


    private static final Comparator<FileType> SIZE_COMPARATOR =
        new SizeComparator();

    private static final Comparator<FileType> COUNT_COMPARATOR =
        new CountComparator();

    private static final class SizeComparator implements Comparator<FileType> {
        public int compare(FileType t1, FileType t2) {
            long value1 = t1.size;
            long value2 = t2.size;
            return value1 < value2 ? 1 : (value1 > value2 ? -1 : 0);
        }
    }

    private static final class CountComparator implements Comparator<FileType> {
        public int compare(FileType t1, FileType t2) {
            long value1 = t1.count;
            long value2 = t2.count;
            return value1 < value2 ? 1 : (value1 > value2 ? -1 : 0);
        }
    }


    // String Representation **************************************************

    @Override
    public String toString() {
        TableModel model = getTableModelBySize();
        StringBuilder builder = new StringBuilder();
        builder.append(getClass().getName());
        for (int row = 0; row < model.getRowCount(); row++) {
            builder.append("\n");
            builder.append(model.getValueAt(row, 0));
            builder.append(": size=");
            builder.append(model.getValueAt(row, 1));
            builder.append("; count=");
            builder.append(model.getValueAt(row, 3));
        }
        builder.append("\nTotal: size=" + sizeTotal());
        builder.append("; count=" + countTotal());
        return builder.toString();
    }


    // Map Values *************************************************************

    private static final class FileType {
        final String suffix;
        long size;
        long count;

        FileType(String suffix, long size, long count) {
            this.suffix = suffix;
            this.size   = size;
            this.count  = count;
        }

        void add(long newSize) {
            size += newSize;
            count++;
        }

        void add(long newSize, long newCount) {
            size  += newSize;
            count += newCount;
        }
    }


    // TableModel *************************************************************

    final static class ArrayDistributionTableModel implements TableModel {

        private static final String[] COLUMN_NAME_KEY_SUFFIXES =
            { "title", "size", "sizePercent", "count", "countPercent" };


        private static final Class<?>[] COLUMN_CLASSES =
            { String.class, Long.class, Float.class, Long.class, Float.class };


        private final TypeDistribution distribution;
        private final FileType[] types;
        private final String noExtension;

        ArrayDistributionTableModel(TypeDistribution distribution, FileType[] types) {
            if (types == null) {
                throw new NullPointerException("The types array must not be null.");
            }
            this.distribution = distribution;
            this.types = types;
            this.noExtension = RESOURCES.getString("TypeDistribution.noExtension");
        }

        public int getColumnCount() {
            return COLUMN_NAME_KEY_SUFFIXES.length;
        }

        public String getColumnName(int columnIndex) {
            String key = distribution.getKeyPrefix()
                + ".columnName."
                + COLUMN_NAME_KEY_SUFFIXES[columnIndex];
            return RESOURCES.getString(key);
        }

        public Class<?> getColumnClass(int columnIndex) {
            return COLUMN_CLASSES[columnIndex];
        }

        public int getRowCount() {
            return types.length;
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            switch (columnIndex) {
                case 0 :
                    String suffix = types[rowIndex].suffix;
                    return suffix == null ? noExtension : suffix;

                case 1 :
                    return Long.valueOf(types[rowIndex].size);

                case 2 :
                    float size = types[rowIndex].size;
                    long sizeTotal = distribution.sizeTotal;
                    return Float.valueOf(sizeTotal == 0L ? 1f : (size / sizeTotal));

                case 3 :
                    return Long.valueOf(types[rowIndex].count);

                case 4 :
                    float count = types[rowIndex].count;
                    long countTotal = distribution.countTotal;
                    return Float.valueOf(countTotal == 0L ? 1f : (count / countTotal));

                default :
                    throw new IllegalStateException("Unknown column index: " + columnIndex);
            }
        }

        public void setValueAt(Object value, int rowIndex, int columnIndex) {
            throw new UnsupportedOperationException("Distribution table models are not editable.");
        }

        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return false;
        }

        public void addTableModelListener(TableModelListener l) {
            // Do nothing; this model is immutable.
        }

        public void removeTableModelListener(TableModelListener l) {
            // Do nothing; this model is immutable.
        }

    }


}
