/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport.model.util;

import static com.jgoodies.jdiskreport.model.util.ByteSizeConstants.ONE_GB;
import static com.jgoodies.jdiskreport.model.util.ByteSizeConstants.ONE_KB;
import static com.jgoodies.jdiskreport.model.util.ByteSizeConstants.ONE_MB;
import static com.jgoodies.jdiskreport.model.util.ByteSizeConstants.ONE_TB;

import java.text.*;
import java.util.Date;

import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;

/**
 * Consists only of static methods that format model values.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.9 $
 *
 * @see Format
 */
public final class Formats {

    public static final Format KILOBYTE_FORMAT = new KilobytesFormat();
    public static final Format FILE_COUNT_FORMAT = new FileCountFormat();

    private static final Format DATE_FORMAT = createDateFormat();
    private static final Format DATE_TIME_FORMAT = createDateTimeFormat();
    private static final Format PERCENT_FORMAT = createPercentFormat();
    private static final Format SIZE_FORMAT_LONG = createSizeFormatShort();
    private static final Format SIZE_FORMAT_FRACTION  = createSizeFormatFraction();
    public static final Format COUNT_FORMAT_LONG = createCountFormatShort();
    private static final Format COUNT_FORMAT_FRACTION  = createCountFormatFraction();


    private static final ResourceMap RESOURCES =
        Application.getResourceMap(Formats.class);


    private Formats() {
        // Overrides default constructor; prevents instantiation.
    }


    // API ********************************************************************

    public static String formatDate(long modified) {
        return DATE_FORMAT.format(new Date(modified));
    }


    public static String formatDateAndTime(Long modified) {
        return DATE_TIME_FORMAT.format(new Date(modified.longValue()));
    }


    public static String formatPercent(float value) {
        return value == 1.0f ? "100%" : PERCENT_FORMAT.format(Float.valueOf(value));
    }


    public static String formatSize(long size) {
        final long kilobytes = (ONE_KB - 1 + size) / ONE_KB;
        return formatFileSize0(kilobytes) + " " + sizeUnit(kilobytes);
    }


    public static String formatSizeForCell(Long size) {
        final long kilobytes = (ONE_KB - 1 + size.longValue()) / ONE_KB;
        return formatFileSize0(kilobytes) + sizeCellSpace(kilobytes) + sizeUnit(kilobytes);
    }


    public static String formatCount(long count) {
        if (count < 1000000L) {
            return formatCount0(count);
        } else if (count < 1000000000L) {
            return formatCount0((float) count / 1000000L) + " " + RESOURCES.getString("Formats.million");
        } else {
            return formatCount0((float) count / 1000000000L) + " " + RESOURCES.getString("Formats.billion");
        }
    }


    public static String formatCountForCell(Long count) {
        return formatCount0(count.longValue());
    }


    // Implementation *********************************************************

    private static String formatFileSize0(long kilobytes) {
        if (kilobytes < 1000L) {
            return formatSize0(kilobytes);
        } else if (kilobytes < 1000000L) {
            return formatSize0((float) kilobytes / ONE_KB);
        } else if (kilobytes < 1000000000L) {
            return formatSize0((float) kilobytes / ONE_MB);
        } else if (kilobytes < 1000000000000L) {
            return formatSize0((float) kilobytes / ONE_GB);
        } else {
            return formatSize0((float) kilobytes / ONE_TB);
        }
    }


    private static String formatSize0(long kilobytes) {
        return SIZE_FORMAT_LONG.format(Long.valueOf(kilobytes));
    }


    private static String formatSize0(float sizeFraction) {
        return SIZE_FORMAT_FRACTION.format(Float.valueOf(sizeFraction));
    }


    private static String formatCount0(long count) {
        return COUNT_FORMAT_LONG.format(Long.valueOf(count));
    }


    private static String formatCount0(float countFraction) {
        return COUNT_FORMAT_FRACTION.format(Float.valueOf(countFraction));
    }


    private static String sizeCellSpace(long kilobytes) {
        if (kilobytes < 1000L) {
            return " ";
        } else if (kilobytes < 1000000L) {
            return "     ";
        } else if (kilobytes < 1000000000L) {
            return "             ";
        } else if (kilobytes < 1000000000000L) {
            return "                     ";
        } else {
            return "                             ";
        }
    }

    private static String sizeUnit(long kilobytes) {
        if (kilobytes < 1000L)
            return "KB";
        else if (kilobytes < 1000000L)
            return "MB";
        else if (kilobytes < 1000000000L)
            return "GB";
        else if (kilobytes < 1000000000000L)
            return "TB";
        else
            return "PB";
    }


    // Format Creation ********************************************************

    private static Format createDateFormat() {
        return DateFormat.getDateInstance(DateFormat.MEDIUM);
    }


    private static Format createDateTimeFormat() {
        return DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.SHORT);
    }


    private static Format createPercentFormat() {
        NumberFormat format = NumberFormat.getPercentInstance();
        format.setMaximumFractionDigits(1);
        format.setMinimumFractionDigits(1);
        // format.setMinimumIntegerDigits(2);
        return format;
    }


    private static Format createSizeFormatShort() {
        NumberFormat format = NumberFormat.getNumberInstance();
        format.setGroupingUsed(true);
        format.setMaximumFractionDigits(0);
        return format;
    }


    private static Format createSizeFormatFraction() {
        NumberFormat format = NumberFormat.getNumberInstance();
        format.setGroupingUsed(true);
        format.setMaximumFractionDigits(1);
        format.setMinimumFractionDigits(1);
        return format;
    }


    private static Format createCountFormatShort() {
        NumberFormat format = NumberFormat.getIntegerInstance();
        format.setGroupingUsed(true);
        return format;
    }


    private static Format createCountFormatFraction() {
        NumberFormat format = NumberFormat.getIntegerInstance();
        format.setGroupingUsed(true);
        format.setMaximumFractionDigits(1);
        format.setMinimumFractionDigits(1);
        return format;
    }


    // Format Implementations *************************************************

    private static final class KilobytesFormat extends Format {

        @Override
        public StringBuffer format(Object obj, StringBuffer toAppendTo,
                FieldPosition pos) {
            toAppendTo.append(formatSize(((Long) obj).longValue()));
            return toAppendTo;
        }

        @Override
        public Object parseObject(String source, ParsePosition pos) {
            throw new UnsupportedOperationException("KilobytesFormat can't parse.");
        }
    }


    private static final class FileCountFormat extends Format {

        @Override
        public StringBuffer format(Object obj, StringBuffer toAppendTo,
                FieldPosition pos) {
            toAppendTo.append(formatCount(((Long) obj).longValue()));
            toAppendTo.append(' ');
            toAppendTo.append(RESOURCES.getString("Formats.files"));
            return toAppendTo;
        }

        @Override
        public Object parseObject(String source, ParsePosition pos) {
            throw new UnsupportedOperationException("KilobytesFormat can't parse.");
        }
    }


}
