/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport.view;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JComponent;
import javax.swing.JLabel;

import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.factories.DefaultComponentFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.jdiskreport.model.MainModel;
import com.jgoodies.jdiskreport.model.NavigationModel;
import com.jgoodies.jdiskreport.model.node.DirectoryNode;
import com.jgoodies.jdiskreport.model.util.Formats;
import com.jgoodies.uif.AbstractView;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.builder.I15dPanelBuilder2;

/**
 * Builds the details view and updates it if the selection changes.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.12 $
 */
final class DetailsView extends AbstractView {

    private final static ResourceMap RESOURCES =
        Application.getResourceMap(DetailsView.class);

    private final MainModel mainModel;

    private JLabel folderLabel;
    private JLabel folderNameLabel;
    private JLabel folderTypeLabel;
    private JLabel fileCountLabel;
    private JLabel directoryCountLabel;
    private JLabel totalSizeLabel;
    private JLabel largestFileLabel;
    private JLabel absoluteSizeLabel;
    private JLabel relativeSizeLabel;
    private JLabel oldestFileLabel;
    private JLabel newestFileLabel;


    // Instance Creation ******************************************************

    DetailsView(MainModel mainModel) {
        this.mainModel = mainModel;
    }


    // Building ***************************************************************

    private void initComponents() {
        folderLabel = new JLabel(RESOURCES.getIcon("folder.icon"));
        folderNameLabel     = createLabel();
        folderTypeLabel     = createLabel();
        fileCountLabel      = createLabel();
        directoryCountLabel = createLabel();
        totalSizeLabel      = createLabel();
        largestFileLabel    = createLabel();
        absoluteSizeLabel   = createLabel();
        relativeSizeLabel   = createLabel();
        oldestFileLabel     = createLabel();
        newestFileLabel     = createLabel();
    }

    @Override
    protected JComponent buildPanel() {
        initComponents();
        initEventHandling();

        FormLayout layout = new FormLayout(
                "pref, 6dlu, r:p, 2dlu, l:10dlu:grow, 9dlu, r:p, 2dlu, r:[28dlu,p], 20dlu, r:p, 2dlu, r:[30dlu,p], 20dlu, r:p, 2dlu, r:[23dlu,p], 20dlu, r:p, 2dlu, l:[38dlu,p]",
                "2dlu, p, 2dlu, p");
        I15dPanelBuilder2 builder = new I15dPanelBuilder2(layout, RESOURCES);
        builder.setBorder(Borders.createEmptyBorder("5dlu, 7dlu, 7dlu, 7dlu"));
        builder.setBackground(RESOURCES.getColor("details.background"));
        builder.setComponentFactory(new MyComponentFactory());
        CellConstraints cc = new CellConstraints();
        builder.add(folderLabel,                  cc.xywh(1, 1, 1, 3, "fill, center"));
        builder.addI15dLabel("name.text",         cc.xy( 3, 2));
        builder.add(folderNameLabel,              cc.xy( 5, 2));
        builder.addI15dLabel("type.text",         cc.xy( 3, 4));
        builder.add(folderTypeLabel,              cc.xy( 5, 4));

        builder.addI15dLabel("files.text",        cc.xy( 7, 2));
        builder.add(fileCountLabel,               cc.xy( 9, 2));
        builder.addI15dLabel("directories.text",  cc.xy( 7, 4));
        builder.add(directoryCountLabel,          cc.xy( 9, 4));

        builder.addI15dLabel("totalSize.text",    cc.xy(11, 2));
        builder.add(totalSizeLabel,               cc.xy(13, 2));
        builder.addI15dLabel("largestFile.text",  cc.xy(11, 4));
        builder.add(largestFileLabel,             cc.xy(13, 4));

        builder.addI15dLabel("absoluteSize.text", cc.xy(15, 2));
        builder.add(absoluteSizeLabel,            cc.xy(17, 2));
        builder.addI15dLabel("relativeSize.text", cc.xy(15, 4));
        builder.add(relativeSizeLabel,            cc.xy(17, 4));

        builder.addI15dLabel("oldestFile.text",   cc.xy(19, 2));
        builder.add(oldestFileLabel,              cc.xy(21, 2));
        builder.addI15dLabel("newestFile.text",   cc.xy(19, 4));
        builder.add(newestFileLabel,              cc.xy(21, 4));

        return builder.getPanel();
    }


    // Event Handling *********************************************************

    private void initEventHandling() {
        mainModel.navigationModel().addPropertyChangeListener(
                NavigationModel.PROPERTYNAME_SELECTION,
                new NavigationSelectionChangeHandler());
        update(mainModel.navigationModel().getSelection());
    }


    private void update(DirectoryNode selection) {
        folderLabel.setIcon(selection.getIcon(true));
        folderNameLabel.setText(selection.getNodeName());
        folderTypeLabel.setText("Ordner");
        fileCountLabel.setText(Formats.formatCount(selection.getFileCount()));
        directoryCountLabel.setText(Formats.formatCount(selection.getDirectoryCount()));
        totalSizeLabel.setText(Formats.formatSize(selection.getSize()));
        absoluteSizeLabel.setText(Formats.formatPercent(selection.getAbsoluteSize()));
        relativeSizeLabel.setText(Formats.formatPercent(selection.getRelativeSize()));

        boolean noFiles = selection.getFileCount() == 0;
        String largestText = noFiles
            ? ""
            : Formats.formatSize(selection.getLargestFile().getSize());
        String oldestText = noFiles
            ? ""
            : Formats.formatDate(selection.getOldestFile().getLastModified());
        String newestText = noFiles
            ? ""
            : Formats.formatDate(selection.getNewestFile().getLastModified());
        largestFileLabel.setText(largestText);
        oldestFileLabel.setText(oldestText);
        newestFileLabel.setText(newestText);
    }


    private final class NavigationSelectionChangeHandler implements PropertyChangeListener {

        public void propertyChange(PropertyChangeEvent evt) {
            DirectoryNode selection = (DirectoryNode) evt.getNewValue();
            update(selection);
        }

    }


    // Helper Code ************************************************************

    private JLabel createLabel() {
        JLabel label = new JLabel();
        return label;
    }


    private static final class MyComponentFactory extends DefaultComponentFactory {
        @Override
        public JLabel createLabel(String textWithMnemonic) {
            JLabel label = super.createLabel(textWithMnemonic);
            label.setForeground(RESOURCES.getColor("details.foreground"));
            return label;
        }
    }


}
