/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport.view;

import java.awt.Component;

import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.border.Border;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;

import com.jgoodies.forms.factories.Borders;
import com.jgoodies.jdiskreport.model.node.AbstractNode;
import com.jgoodies.jdiskreport.model.node.DirectoryNode;
import com.jgoodies.jdiskreport.model.util.Formats;

/**
 * Consists only of static variables that hold table cell renderers.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.5 $
 */
final class TableCellRenderers {

    // Constants **************************************************************

    static final TableCellRenderer DEFAULT_RENDERER =
        new DefaultRenderer();

    static final TableCellRenderer RIGHT_ALIGNED_DEFAULT_RENDERER =
        new RightAlignedDefaultRenderer();

    static final TableCellRenderer NODE_RENDERER =
        new NodeRenderer();

    static final TableCellRenderer DIRECTOY_PATH_RENDERER =
        new DirectoryPathRenderer();

    static final TableCellRenderer DATE_TIME_RENDERER =
        new DateTimeRenderer();

    static final TableCellRenderer NUMBER_RENDERER =
        new NumberRenderer();

    static final TableCellRenderer SIZE_RENDERER =
        new SizeRenderer();

    static final TableCellRenderer COUNT_RENDERER =
        new CountRenderer();

    static final TableCellRenderer PERCENT_RENDERER =
        new PercentRenderer();


    // Instance Creation ******************************************************

    private TableCellRenderers() {
        // Overrides default constructor; prevents instantiation.
    }


    // Implementations ********************************************************

    /**
     * Has a wider left and right border than its superclass.
     **/
    private static class DefaultRenderer extends DefaultTableCellRenderer {

        private static final Border EMPTY_BORDER = Borders.createEmptyBorder("0, 3dlu, 0, 3dlu");

        @Override
        public Component getTableCellRendererComponent(JTable table, Object value,
                boolean isSelected, boolean hasFocus, int row, int column) {
            JLabel label =
                (JLabel) super.getTableCellRendererComponent(table, value, isSelected, false, row, column);
            label.setBorder(EMPTY_BORDER);
            return label;
        }
    }


    /**
     * Right aligned default renderer.
     **/
    private static final class RightAlignedDefaultRenderer extends DefaultRenderer {
        public RightAlignedDefaultRenderer() {
            setHorizontalAlignment(JLabel.RIGHT);
        }
    }


    /**
     * Renders AbstractNodes with icon and name.
     **/
    private static final class NodeRenderer extends DefaultRenderer {
        @Override
        public void setValue(Object value) {
            AbstractNode node = (AbstractNode) value;
            setIcon(node == null ? null : node.getIcon(false));
            setText(node == null ? ""   : node.getName());
        }
    }


    /**
     * Renders DirectoryNode with icon and path.
     **/
    private static final class DirectoryPathRenderer extends DefaultRenderer {
        @Override
        public void setValue(Object value) {
            DirectoryNode node = (DirectoryNode) value;
            setIcon(node == null ? null : node.getIcon(false));
            setText(node == null ? ""   : node.getPath());
        }
    }


    /**
     * Has a wider border than the default number renderer.
     **/
    private static final class NumberRenderer extends DefaultRenderer {
        public NumberRenderer() {
            setHorizontalAlignment(JLabel.RIGHT);
        }
    }


    /**
     * Renders sizes with unit and identation.
     **/
    private static final class SizeRenderer extends DefaultRenderer {
        public SizeRenderer() {
            setHorizontalAlignment(JLabel.RIGHT);
        }
        @Override
        public void setValue(Object value) {
            setText(value == null ? "" : Formats.formatSizeForCell((Long) value));
        }
    }


    /**
     * Renders counts without unit.
     **/
    private static final class CountRenderer extends DefaultRenderer {
        public CountRenderer() {
            setHorizontalAlignment(JLabel.RIGHT);
        }
        @Override
        public void setValue(Object value) {
            setText(value == null ? "" : Formats.formatCountForCell((Long) value));
        }
    }


    /**
     * Renders modification dates with time.
     **/
    private static final class DateTimeRenderer extends DefaultRenderer {
        @Override
        public void setValue(Object value) {
            setText(value == null ? "" : Formats.formatDateAndTime((Long) value));
        }
    }


    /**
     * Renders percent floats.
     **/
    private static final class PercentRenderer extends DefaultRenderer {
        public PercentRenderer() {
            setHorizontalAlignment(JLabel.RIGHT);
        }
        @SuppressWarnings("boxing")
        @Override
        public void setValue(Object value) {
            setText(value == null ? "" : Formats.formatPercent((Float) value));
        }
    }


}
