/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.jdiskreport.view.component;

import java.awt.*;

import javax.swing.*;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.plaf.basic.BasicGraphicsUtils;

import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;

/**
 * The toggle button used to choose a JDiskReport presentation.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.8 $
 */
public final class NavigationButton extends JToggleButton {

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(NavigationButton.class);


    // Instance Creation ******************************************************

    public NavigationButton(Icon icon, String text) {
        super(text);
        setIcon(icon);
        setFocusPainted(false);
        setContentAreaFilled(false);
        setHorizontalAlignment(SwingConstants.LEADING);
        setBorder(new CompoundBorder(
                new NavigationButtonBorder(),
                new EmptyBorder(1, 9, 1, 9)));
        setForeground(getCurrentColor());
        getModel().addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                setForeground(getCurrentColor());
            }
        });
    }

    @Override
    protected void paintComponent(Graphics g) {
        //ButtonModel model = getModel();
        boolean isActive  = model.isRollover() || model.isPressed();

        // Paint background
        if (model.isArmed() && model.isPressed()) {
            paintPressed(g);
        } else if (isActive) {
            paintRollover(g);
        } else if (model.isSelected()) {
            paintSelected(g);
        }

        if (isFocusPainted() && hasFocus()) {
            paintFocus(g);
        }
        super.paintComponent(g);
    }

    protected void paintPressed(Graphics g) {
        Color startColor = RESOURCES.getColor("NavigationButton.pressed.gradient.start");
        Color endColor   = RESOURCES.getColor("NavigationButton.pressed.gradient.stop");
        paintGradient(g, startColor, endColor);
    }


    private void paintRollover(Graphics g) {
        Color startColor = RESOURCES.getColor("NavigationButton.rollover.gradient.start");
        Color endColor   = RESOURCES.getColor("NavigationButton.rollover.gradient.stop");
        paintGradient(g, startColor, endColor);
    }

    private void paintSelected(Graphics g) {
        Color startColor = RESOURCES.getColor("NavigationButton.selected.gradient.start");
        Color endColor   = RESOURCES.getColor("NavigationButton.selected.gradient.stop");
        paintGradient(g, startColor, endColor);
    }

    private void paintGradient(Graphics g, Color startColor, Color endColor) {
        int width  = getWidth();
        int height = getHeight();
        Insets insets = NavigationButtonBorder.INSETS;
        int x, y, w, h;
        Graphics2D g2 = (Graphics2D) g;
        Rectangle clip = g2.getClipBounds();
        if (clip != null) {
            x = clip.x; y = clip.y; w = clip.width; h = clip.height;
            x = Math.max(x, insets.left);
            y = Math.max(y, insets.top);
            int right = width - insets.right;
            if (x + w > right) {
                w = right - x;
            }
            int bottom = height - insets.bottom;
            if (y + h > bottom) {
                h = bottom - y;
            }
        } else {
            x = insets.left;
            y = insets.top;
            w = width  - (insets.left + insets.right);
            h = height - (insets.top  + insets.bottom);
        }
        paintGradient(g2, x, y, w, h, width, height, startColor, endColor);
    }


    private void paintGradient(Graphics2D g2,
            int x, int y, int w, int h, int width, int height,
            Color startColor, Color endColor) {
        g2.setPaint(new GradientPaint(0, 0, startColor, 0, height, endColor, true));
        g2.fillRect(x, y, w, h);
    }


    private void paintFocus(Graphics g){
        int width  = getWidth();
        int height = getHeight();
        Insets insets = NavigationButtonBorder.INSETS;
        int x = insets.left;
        int y = insets.top;
        int w = width  - (insets.left + insets.right);
        int h = height - (insets.top  + insets.bottom);
        Color focusColor = UIManager.getColor("Button.focus");
        g.setColor(focusColor);
        BasicGraphicsUtils.drawDashedRect(g, x, y, w, h);
    }

    private Color getCurrentColor() {
        ButtonModel m = getModel();
        String foregroundKey = m.isEnabled()
            ? "NavigationButton.foreground"
            : "NavigationButton.disabledForeground";
        return RESOURCES.getColor(foregroundKey);
    }

}