/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.looks.demo;

import java.awt.Component;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractButton;
import javax.swing.JComponent;
import javax.swing.JFrame;

import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.component.ToolBarButton;
import com.jgoodies.uif.component.ToolBarToggleButton;
import com.jgoodies.uif.util.Modes;
import com.jgoodies.uif.util.ScreenUtils;
import com.jgoodies.uifextras.convenience.DefaultAboutDialog;
import com.jgoodies.uifextras.convenience.ResourceIDs;
import com.jgoodies.uifextras.help.HelpBroker;
import com.jgoodies.uifextras.panel.HeaderPanel;

/**
 * Builds the main frame in the full JGoodies Looks Demo.
 * Demonstrates and tests different multi-platform issues by
 * showing a variety of Swing widgets in different configurations.
 * Also, this frame contains examples for Swing misuse,
 * that can be automatically corrected by ClearLook.<p>
 *
 * This class overrides the protected methods provided by its
 * superclass <code>DemoFrame</code> to vend components and
 * builders from the JGoodies UI framework that better handle
 * different platforms.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.13 $
 *
 * @see com.jgoodies.uif.component.ToolBarButton
 * @see com.jgoodies.uif.component.ToolBarToggleButton
 */
public final class DemoFrameUIF extends DemoFrame {

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(DemoFrameUIF.class);



    private static DemoFrameUIF instance;

    /**
     * Constructs a <code>DemoFrame</code>, configures the UI,
     * and builds the content.
     */
    protected DemoFrameUIF(Settings settings) {
        super(settings);
    }


    /**
     * Creates and opens an <code>DemoFrameUIF</code> that can access
     * the given <code>Launcher</code>; uses the specified bounds,
     * if non-null.
     */
    public static DemoFrameUIF openOn(Settings settings, Rectangle bounds) {
        instance = new DemoFrameUIF(settings);
        if (bounds == null) {
            instance.setSize(PREFERRED_SIZE);
            instance.locateOnScreen(instance);
        } else
            instance.setBounds(bounds);
        instance.setVisible(true);
        return instance;
    }

    static DemoFrameUIF getInstance() {
        return instance;
    }

    // Overriding Superclass Behavior *****************************************

    @Override
    protected String getWindowTitle() {
        return "Sample Frame - The JGoodies Looks Demo";
    }

    /**
     * Locates the given component on the screen's east.
     */
    @Override
    protected void locateOnScreen(Component component) {
        ScreenUtils.locateOnScreenEast(component);
    }

    /**
     * Creates and returns a builder that builds the menu.
     * Unlike the superclass we use a menu builder that uses the JGoodies
     * UI Framework.
     *
     * @return the builder that builds the menu bar
     */
    @Override
    protected MenuBarView createMenuBuilder() {
        return new MenuBarViewUIF();
    }

    /**
     * Creates and returns a JButton configured for use in a JToolBar.<p>
     *
     * Unlike the superclass we use the JGoodies UI framework's ToolBarButton.
     */
    @Override
    protected AbstractButton createToolBarButton(String iconName, String toolTipText) {
        ToolBarButton button = new ToolBarButton(readImageIcon(iconName));
        button.setToolTipText(toolTipText);
        button.setBorderPaintedMode(Modes.LAF_NON_AQUA);
        button.setWideMarginMode(Modes.LAF_AQUA);
        return button;
    }

    /**
     * Creates and returns a JToggleButton configured for use in a JToolBar.<p>
     *
     * Unlike the superclass we use the JGoodies UI framework's ToolBarButton.
     */
    @Override
    protected AbstractButton createToolBarRadioButton(String iconName, String toolTipText) {
    	ToolBarToggleButton button = new ToolBarToggleButton(readImageIcon(iconName));
        button.setToolTipText(toolTipText);
    	button.setBorderPaintedMode(Modes.LAF_NON_AQUA);
    	button.setWideMarginMode(Modes.LAF_AQUA);
    	return button;
    }



    // Helper Code ************************************************************

    /**
     * Creates and returns an ActionListener that opens the help viewer.
     */
    @Override
    protected ActionListener createHelpActionListener() {
        return new HelpActionHandler();
    }

    /**
     * Creates and returns an ActionListener that opens the about dialog.
     */
    @Override
    protected ActionListener createAboutActionListener() {
        return new AboutActionHandler();
    }


    /**
     * A helper class that overrides the height of the header panel.
     */
    private static final class AboutDialog extends DefaultAboutDialog {

        private AboutDialog(JFrame owner) {
            super(owner);
        }

        @Override
        protected JComponent buildHeader() {
            return new HeaderPanel(
                "About " + RESOURCES.getString("application.name"),
                "You can click on the tabs below to see information\n" +
                "about the demo's version and the system's state.",
                RESOURCES.getIcon(ResourceIDs.ABOUT_ICON));
        }

    }

    public static final class AboutActionHandler implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            new AboutDialog(getInstance()).open();
        }
    }

    private static final class HelpActionHandler implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            HelpBroker.openDefault();
        }
    }

}