/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.looks.launcher;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.event.*;
import java.util.LinkedList;
import java.util.List;

import javax.swing.*;
import javax.swing.plaf.metal.MetalLookAndFeel;

import com.jgoodies.looks.demo.DemoFrameUIF;
import com.jgoodies.looks.plastic.Plastic3DLookAndFeel;
import com.jgoodies.looks.plastic.PlasticLookAndFeel;
import com.jgoodies.looks.plastic.PlasticTheme;
import com.jgoodies.looks.plastic.PlasticXPLookAndFeel;
import com.jgoodies.uif.AbstractFrame;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ApplicationContext;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.laf.LookChoiceStrategies;
import com.jgoodies.uif.util.ComponentTreeUtils;
import com.jgoodies.uif.util.Resizer;
import com.jgoodies.uif.util.ScreenUtils;
import com.jgoodies.uif.util.SystemUtils;

/**
 * Launcher for the JGoodies Looks Demo. Provides an interface
 * to choose a look and theme and many settings.
 * Launches the <code>DemoFrameUIF</code>.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.25 $
 */
public final class Launcher extends AbstractFrame {

    private static final String WINLAF_CLASS_NAME =
        "net.java.plaf.windows.WindowsLookAndFeel";

    private static final String FROG_CLASS_NAME =
        "com.sap.plaf.frog.FrogLookAndFeel";

    private static final String GTK_CLASS_NAME =
        "com.sun.java.swing.plaf.gtk.GTKLookAndFeel";

    private static final String KUNSTSTOFF_CLASS_NAME =
        "com.incors.plaf.kunststoff.KunststoffLookAndFeel";

    private static final String ALLOY_OBFUSCATED_CLASS_NAME =
        "com.incors.plaf.alloy.bv";

    private static final String XENON_CLASS_NAME =
        "de.ergosign.plaf.xenon.XenonLookAndFeel";

    private static final String ALIGNED_WINLAF_CLASS_NAME =
        "com.jgoodies.alignedwinlaf.WindowsLookAndFeel";

    private static final String[] OPTIONAL_LOOKS_CLASS_NAMES = {
            ALIGNED_WINLAF_CLASS_NAME,
            WINLAF_CLASS_NAME,
            FROG_CLASS_NAME,
            GTK_CLASS_NAME,
            KUNSTSTOFF_CLASS_NAME,
            ALLOY_OBFUSCATED_CLASS_NAME,
            XENON_CLASS_NAME};

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(Launcher.class);


    private LauncherPageView  launcherPageView;
    private DemoFrameUIF      demoFrame;
    private final WindowListener    restoreListener;
    private MouseListener     doubleClickHandler;

    /**
     * Constructs the launcher (main) frame.
     */
    public Launcher() {
        super(RESOURCES.getString("application.title"));
        setTitleSuffix(RESOURCES.getString("application.title"));
        setTitlePrefix("Launcher");
        setName("main");
        restoreListener = new WindowAdapter() {
            @Override
            public void windowClosed(WindowEvent e) {
                restoreLauncher();
            }
        };
    }

    /**
     * Opens the frame. In addition to the superclass behavior we ensure that
     * the selection in the theme panel is visible.
     */
    @Override
    public void open() {
        super.open();
        launcherPageView.ensureThemeIsVisible();
    }


    /**
     * Builds and returns the content pane; delegated to
     * the <code>LauncherPageView</code> to enable Applet support later.
     */
    @Override
    public JComponent buildContentPane() {
        configureUI();
        launcherPageView = new LauncherPageView(this);
        doubleClickHandler = new DoubleClickHandler();
        launcherPageView.addLookAndFeelListDoubleClickListener(
                doubleClickHandler);
        return launcherPageView.getPanel();
    }


    // Implementing Abstract Behavior ****************************************

    /**
     * Returns the frame's minimum size. Used to ensure a minimum size
     * as requested by the <code>WindowUtils</code>.
     *
     * @return the frame's minimum size
     */
    public Dimension getWindowMinimumSize() {
        return new Dimension(400, 400);
    }


    /**
     * Configures the close behavior to close the application
     * on Window closing.
     *
     * @see JFrame#setDefaultCloseOperation(int)
     * @see Application#getApplicationExitOnWindowClosingHandler()
     */
    @Override
    protected void configureCloseOperation() {
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(
                ApplicationContext.getInstance().getApplication().getApplicationExitOnWindowClosingHandler());
    }


    /**
     * Resizes the specified component and  is called during the #build process.
     * Unlike the default, we try to resize to a heigh/width ratio of 1.
     */
    @Override
    protected void resizeHook(JComponent component) {
        Resizer.ONE2ONE.resizeDialogContent(component);
    }

    /**
     * Locates the frame on the screen. Unlike the default,
     * we locate the launcher on the left side.
     */
    @Override
    protected void locateOnScreen() {
        ScreenUtils.locateOnScreenWest(this);
    }

    // Launch Process *******************************************************************

    /**
     * Launches the demo frame: stores bounds of an existing frame,
     * configures the demo, opens the demo frame, iconifies the launcher,
     * and finally listens to demo frame close events.
     */
    void doLaunch() {
        launcherPageView.removeLookAndFeelListDoubleClickListener(doubleClickHandler);
        Rectangle bounds = null;
        if (demoFrame != null) {
            bounds = demoFrame.getBounds();
            demoFrame.dispose();
        }
        demoFrame = DemoFrameUIF.openOn(launcherPageView.getSettings(), bounds);
        setState(Frame.ICONIFIED);
        demoFrame.addWindowListener(restoreListener);
    }

    /**
     * Restores the launcher to reset the original look&feel.
     */
    private void restoreLauncher() {
        if (isWinLAF()) {
            reportWinLAFProblemAndExit();
        }
        demoFrame.removeWindowListener(restoreListener);
        PlasticLookAndFeel.setPlasticTheme(
            PlasticLookAndFeel.createMyDefaultTheme());
        configureUI();
        ComponentTreeUtils.updateAllUIs();
        setState(Frame.NORMAL);
        launcherPageView.addLookAndFeelListDoubleClickListener(doubleClickHandler);
    }


    // Winlaf Code ************************************************************

    private boolean isWinLAF() {
        String lafName = UIManager.getLookAndFeel().getClass().getName();
        return WINLAF_CLASS_NAME.equals(lafName);
    }

    private void reportWinLAFProblemAndExit() {
        JOptionPane.showMessageDialog(null,
                "Since WinLAF doesn't uninstall properly"
                + "\nthis Looks Demo will be shut down."
                + "\n\nTo try other look&feels, restart this demo.");
        System.exit(0);
    }


    // UI Configuration for the Launcher Itself *****************************************

    /**
     * Configures the UI before we try to install the look and feel.
     */
    private void configureUI() {
        try {
            UIManager.setLookAndFeel(defaultLookAndFeelClassName());
        } catch (Exception e) {
            System.out.println("Can't change L&F: " + e);
        }
    }

    /**
     * Computes and returns the class name for the default look and feel.
     */
    String defaultLookAndFeelClassName() {
        return LookChoiceStrategies.SYSTEM.getDefaultLookClassName();
    }

    // Helper Methods *******************************************************************

    /**
     * Computes and returns an array of available and supported look&feel implementations.
     */
    LookAndFeel[] computeLooks() {
        List<LookAndFeel> looks = new LinkedList<LookAndFeel>();
        looks.add(new PlasticXPLookAndFeel());
        looks.add(new Plastic3DLookAndFeel());
        looks.add(new PlasticLookAndFeel());
        looks.add(new MetalLookAndFeel());
        if (SystemUtils.IS_OS_WINDOWS) {
            looks.add(new com.jgoodies.looks.windows.WindowsLookAndFeel());
            looks.add(new com.sun.java.swing.plaf.windows.WindowsLookAndFeel());
        }
        if (SystemUtils.IS_OS_MAC) {
            addLook(looks, UIManager.getSystemLookAndFeelClassName());
        }
        for (String className : OPTIONAL_LOOKS_CLASS_NAMES) {
            addLook(looks, className);
        }
        return looks.toArray(new LookAndFeel[looks.size()]);
    }

    private void addLook(List looks, String className) {
        try {
            Class lookClass = Class.forName(className);
            LookAndFeel laf = (LookAndFeel) lookClass.newInstance();
            if (laf.isSupportedLookAndFeel()) {
                looks.add(laf);
            }
        } catch (Exception e) {
            // Do nothing if this look is absent
        }
    }

    /**
     * Computes and answers an array of available Plastic themes.
     */
    PlasticTheme[] computeThemes() {
        List themes = PlasticLookAndFeel.getInstalledThemes();
        return (PlasticTheme[]) themes.toArray(
            new PlasticTheme[themes.size()]);
    }


    // Double-Click Handler ***************************************************

    private final class DoubleClickHandler extends MouseAdapter {
        @Override
        public void mouseClicked(MouseEvent e) {
            super.mouseClicked(e);
            if (e.getClickCount() == 2)
                doLaunch();
        }
    }

}
