/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.looks.launcher;

import java.awt.Component;
import java.awt.event.MouseListener;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.builder.ButtonBarBuilder2;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.looks.BorderStyle;
import com.jgoodies.looks.HeaderStyle;
import com.jgoodies.looks.demo.Settings;
import com.jgoodies.looks.plastic.Plastic3DLookAndFeel;
import com.jgoodies.looks.plastic.PlasticLookAndFeel;
import com.jgoodies.looks.plastic.PlasticTheme;
import com.jgoodies.uif.AbstractView;
import com.jgoodies.uif.application.Action;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.builder.PanelBuilder2;
import com.jgoodies.uif.laf.ExtUIManager;
import com.jgoodies.uifextras.convenience.ResourceIDs;
import com.jgoodies.uifextras.help.HelpBroker;
import com.jgoodies.uifextras.panel.HeaderPanel;

/**
 * Builds the launcher page of the JGoodies Looks Demo.
 * Provides an interface to choose a look, theme and set options.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.19 $
 */
public final class LauncherPageView extends AbstractView {

    private static final String ACTION_OPEN_TAB_HELP = "openTabHelp";
    private static final String ACTION_LAUNCH_DEMO   = "launchDemo";

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(LauncherPageView.class);


    private final Launcher launcher;

    private JList lookAndFeelList;
    private JList themeList;

    private JCheckBox   useNarrowMarginsCheckBox;
    private JCheckBox   tabIconsEnabledCheckBox;
    private ButtonGroup popupDropShadowGroup;
    private ButtonGroup plasticTabStyleGroup;
    private ButtonGroup plasticFocusContrastGroup;

    private ButtonGroup menuBarHeaderStyleGroup;
    private ButtonGroup menuBarWindowsBorderStyleGroup;
    private ButtonGroup menuBarPlasticBorderStyleGroup;
    private ButtonGroup menuBarPlastic3DHintGroup;
    private ButtonGroup toolBarHeaderStyleGroup;
    private ButtonGroup toolBarWindowsBorderStyleGroup;
    private ButtonGroup toolBarPlasticBorderStyleGroup;
    private ButtonGroup toolBarPlastic3DHintGroup;

    private JTabbedPane tabbedPane;
    private JButton     helpButton;
    private JButton     launchButton;
    private Map<JComponent, URL> helpMap;


    // Instance Creation ****************************************************

    /**
     * Constructs and builds the <code>LauncherPageView</code>.
     */
    LauncherPageView(Launcher launcher) {
        this.launcher = launcher;
        initComponents();
    }

    // Access Used By the DemoFrame ****************************************

    Settings getSettings() {
        Settings settings = Settings.createDefault();
        settings.setSelectedLookAndFeel(selectedLookAndFeel());
        settings.setSelectedTheme(selectedTheme());
        settings.setUseNarrowButtons(useNarrowMarginsCheckBox.isSelected());
        settings.setTabIconsEnabled(tabIconsEnabledCheckBox.isSelected());
        settings.setPopupDropShadowEnabled(popupDropShadowEnabled());
        settings.setPlasticTabStyle(plasticTabStyle());
        settings.setPlasticHighContrastFocusEnabled(plasticHighContrastFocusEnabled());
        settings.setMenuBarHeaderStyle(getHeaderStyle(menuBarHeaderStyleGroup));
        settings.setMenuBarPlasticBorderStyle(getBorderStyle(menuBarPlasticBorderStyleGroup));
        settings.setMenuBarWindowsBorderStyle(getBorderStyle(menuBarWindowsBorderStyleGroup));
        settings.setMenuBar3DHint(menuBar3DHint());
        settings.setToolBarHeaderStyle(getHeaderStyle(toolBarHeaderStyleGroup));
        settings.setToolBarPlasticBorderStyle(getBorderStyle(toolBarPlasticBorderStyleGroup));
        settings.setToolBarWindowsBorderStyle(getBorderStyle(toolBarWindowsBorderStyleGroup));
        settings.setToolBar3DHint(toolBar3DHint());
        return settings;
    }

    void addLookAndFeelListDoubleClickListener(MouseListener l) {
        lookAndFeelList.addMouseListener(l);
    }

    void removeLookAndFeelListDoubleClickListener(MouseListener l) {
        lookAndFeelList.removeMouseListener(l);
    }

    // Getters Used by #getSettings *****************************************

    private LookAndFeel selectedLookAndFeel() {
        LookAndFeel laf = (LookAndFeel) lookAndFeelList.getSelectedValue();
        if (laf != null) {
            return laf;
        }
        String className = launcher.defaultLookAndFeelClassName();
        laf = ExtUIManager.createLookAndFeelInstance(className);
        return laf != null ? laf : new Plastic3DLookAndFeel();
    }

    private PlasticTheme selectedTheme() {
        return (PlasticTheme) themeList.getSelectedValue();
    }

    private String plasticTabStyle() {
        return plasticTabStyleGroup.getSelection().getActionCommand();
    }

    private Boolean popupDropShadowEnabled() {
        String name = popupDropShadowGroup.getSelection().getActionCommand();
        if (name.equalsIgnoreCase("On"))
            return Boolean.TRUE;
        else if (name.equalsIgnoreCase("Off"))
            return Boolean.FALSE;
        else
            return null;
    }

    private boolean plasticHighContrastFocusEnabled() {
        return plasticFocusContrastGroup.getSelection().getActionCommand().
        equalsIgnoreCase("high");
    }

    private Boolean menuBar3DHint() {
        String name =
            menuBarPlastic3DHintGroup.getSelection().getActionCommand();
        if (name.equals("None"))
            return null;
        else if (name.equals("On"))
            return Boolean.TRUE;
        else
            return Boolean.FALSE;
    }

    private Boolean toolBar3DHint() {
        String name =
            toolBarPlastic3DHintGroup.getSelection().getActionCommand();
        if (name.equals("None"))
            return null;
        else if (name.equals("On"))
            return Boolean.TRUE;
        else
            return Boolean.FALSE;
    }

    private HeaderStyle getHeaderStyle(ButtonGroup group) {
        String name = group.getSelection().getActionCommand();
        if (name.equals("None"))
            return null;
        else if (name.equals("Single"))
            return HeaderStyle.SINGLE;
        else
            return HeaderStyle.BOTH;
    }

    private BorderStyle getBorderStyle(ButtonGroup group) {
        String name = group.getSelection().getActionCommand();
        if (name.equals("None"))
            return null;
        else if (name.equals("Empty"))
            return BorderStyle.EMPTY;
        else if (name.equals("Etched"))
            return BorderStyle.ETCHED;
        else
            return BorderStyle.SEPARATOR;
    }


    // Building *************************************************************

    /**
     * Creates and configures UI components.
     */
    private void initComponents() {
        tabbedPane = new JTabbedPane();

        lookAndFeelList = new JList(launcher.computeLooks());
        lookAndFeelList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lookAndFeelList.addListSelectionListener(createLookListener());
        lookAndFeelList.setCellRenderer(createLookRenderer());

        themeList = new JList(launcher.computeThemes());
        themeList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        themeList.setCellRenderer(createThemeRenderer());
        //updateThemeEnablement();

        useNarrowMarginsCheckBox = new JCheckBox("Use narrow margins");
        useNarrowMarginsCheckBox.setContentAreaFilled(false);
        useNarrowMarginsCheckBox.setSelected(true);
        tabIconsEnabledCheckBox = new JCheckBox("Use icons in JTabbedPane");
        tabIconsEnabledCheckBox.setContentAreaFilled(false);
        tabIconsEnabledCheckBox.setSelected(true);
        popupDropShadowGroup = new ButtonGroup();
        plasticTabStyleGroup = new ButtonGroup();
        plasticFocusContrastGroup = new ButtonGroup();

        menuBarHeaderStyleGroup = new ButtonGroup();
        menuBarWindowsBorderStyleGroup = new ButtonGroup();
        menuBarPlasticBorderStyleGroup = new ButtonGroup();
        menuBarPlastic3DHintGroup = new ButtonGroup();
        toolBarHeaderStyleGroup = new ButtonGroup();
        toolBarWindowsBorderStyleGroup = new ButtonGroup();
        toolBarPlasticBorderStyleGroup = new ButtonGroup();
        toolBarPlastic3DHintGroup = new ButtonGroup();

        ActionMap map = Application.getActionMap(this);
        helpButton = new JButton(map.get(ACTION_OPEN_TAB_HELP));
        launchButton = new JButton(map.get(ACTION_LAUNCH_DEMO));
    }

    @Action
    public void openTabHelp() {
        Object tab = tabbedPane.getSelectedComponent();
        URL url = helpMap.get(tab);
        HelpBroker.openURL(url);
    }

    @Action
    public void launchDemo() {
        launcher.doLaunch();
    }

    /**
     * Builds and returns the launcher page.
     */
    @Override
    protected JComponent buildPanel() {
        FormLayout layout = new FormLayout(
                "4dlu, fill:pref:grow, 4dlu",
                "p, 4dlu, fill:p:grow, $ug, p, 5dlu");

        PanelBuilder builder = new PanelBuilder(layout);
        CellConstraints cc = new CellConstraints();
        builder.add(buildHeader(),     cc.xyw(1, 1, 3));
        builder.add(buildTabbedPane(), cc.xy (2, 3));
        builder.add(buildButtonBar(),  cc.xy (2, 5));

        return builder.getPanel();
    }

    private Component buildHeader() {
        return new HeaderPanel(
            "The JGoodies Looks Demo",
            "Choose a look, set options, then launch the demo.\n" +
            "Each tab provides a context-sensitive help button.",
            RESOURCES.getIcon(ResourceIDs.ABOUT_ICON));
    }

    private Component buildTabbedPane() {
        helpMap = new HashMap<JComponent, URL>();
        addTab(
            tabbedPane,
            buildLookAndThemeTab(),
            "Look & Theme",
            "look");
        addTab(
            tabbedPane,
            buildGeneralOptionsTab(),
            "General Options",
            "general");
        addTab(tabbedPane, buildBarOptionsTab(),
                "Menu- & Tool Bar",
                "bars");
        return tabbedPane;
    }

    /**
     * Adds a card to the given JTabbedPane using the specified data.
     */
    private void addTab(
        JTabbedPane parent,
        JComponent card,
        String label,
        String help) {
        parent.add(card, label);
        String path = "/com/jgoodies/looks/demo/resources/help/launcher/" + help + ".html";
        URL url = getClass().getResource(path);
        helpMap.put(card, url);
    }

    /**
     * Creates and answers the button bar with help and launch button.
     */
    private JComponent buildButtonBar() {
        ButtonBarBuilder2 builder = new ButtonBarBuilder2();
        builder.addButton(helpButton);
        builder.addUnrelatedGap();
        builder.addGlue();
        builder.addButton(launchButton);
        return builder.getPanel();
    }

    // Building Tabs ********************************************************************

    private JComponent buildLookAndThemeTab() {
        FormLayout layout = new FormLayout(
                "min:grow, 6dlu, min:grow",
                "p, 2dlu, fill:100dlu:grow");

        PanelBuilder builder = new PanelBuilder2(layout);
        builder.setDefaultDialogBorder();
        builder.setOpaque(false);
        CellConstraints cc = new CellConstraints();

        builder.addLabel("&Look && Feel:",            cc.xy(1, 1));
        builder.add(new JScrollPane(lookAndFeelList), cc.xy(1, 3));

        builder.addLabel("&Plastic Theme:",           cc.xy(3, 1));
        builder.add(new JScrollPane(themeList),       cc.xy(3, 3));
        preselectLookAndTheme();
        return builder.getPanel();
    }


    private JComponent buildGeneralOptionsTab() {
        FormLayout layout = new FormLayout(
                "7dlu, left:max(50dlu;pref), 4dlu, max(36dlu;p), 1dlu, p, 1dlu, p, 1dlu, p, 0:grow",
                "pref, 2dlu, pref, 4dlu, pref, 4dlu, pref, 11dlu, "
              + "pref, 2dlu, pref, 4dlu, pref, 0:grow");
        layout.setColumnGroups(new int[][] {{4, 6, 8, 10 }});
        PanelBuilder builder = new PanelBuilder(layout);
        builder.setDefaultDialogBorder();
        builder.setOpaque(false);
        CellConstraints cc = new CellConstraints();

        builder.addSeparator("Global Options",  cc.xyw(1, 1, 11));

        builder.addLabel("Button Margins:",     cc.xy (2, 3));
        builder.add(useNarrowMarginsCheckBox,   cc.xyw(4, 3, 5));

        builder.addLabel("Tab Icons:",          cc.xy (2, 5));
        builder.add(tabIconsEnabledCheckBox,    cc.xyw(4, 5, 5));

        builder.addLabel("Popup Shadow:",       cc.xy (2, 7));
        addRadio("Default", popupDropShadowGroup, builder, 4, 7).setSelected(true);
        addRadio("On",      popupDropShadowGroup, builder, 6, 7);
        addRadio("Off",     popupDropShadowGroup, builder, 8, 7);

        builder.addSeparator("Plastic Options", cc.xyw(1, 9, 11));

        boolean useMetalTabs =
            PlasticLookAndFeel.getTabStyle().equalsIgnoreCase("Metal");
        builder.addLabel("Tab Style:",               cc.xy(2, 11));
        addRadio("Default", plasticTabStyleGroup, builder, 4, 11).setSelected(
            !useMetalTabs);
        addRadio("Metal", plasticTabStyleGroup,   builder, 6, 11).setSelected(
            useMetalTabs);

        boolean highContrastEnabled =
            PlasticLookAndFeel.getHighContrastFocusColorsEnabled();
        builder.addLabel("Focus Contrast:",            cc.xy(2, 13));
        addRadio("Low",  plasticFocusContrastGroup, builder, 4, 13).setSelected(
            !highContrastEnabled);
        addRadio("High", plasticFocusContrastGroup, builder, 6, 13).setSelected(
            highContrastEnabled);

        return builder.getPanel();
    }


    private JComponent buildBarOptionsTab() {
        FormLayout layout = new FormLayout(
                "7dlu, left:max(50dlu;pref), 4dlu, max(36dlu;p), 1dlu, p, 1dlu, p, 1dlu, p, 4dlu, pref:grow",
                "pref, 2dlu, pref, 4dlu, pref, 4dlu, pref, 4dlu, pref, 11dlu, "
                    + "pref, 2dlu, pref, 4dlu, pref, 4dlu, pref, 4dlu, pref, 0:grow");
        layout.setColumnGroups(new int[][] {{4, 6, 8, 10, 12 } });
        PanelBuilder builder = new PanelBuilder(layout);
        builder.setDefaultDialogBorder();
        builder.setOpaque(false);
        CellConstraints cc = new CellConstraints();

        // Menu Bar Options -------------------------------------------------------------
        builder.addSeparator("Menu Bar Style Options", cc.xyw(1, 1, 12));

        builder.addLabel("Header:",          cc.xy(2, 3));
        addHeaderStyleGroup(builder, 3, menuBarHeaderStyleGroup);

        builder.addLabel("Windows Border:",  cc.xy(2, 5));
        addBorderStyleGroup(builder, 5, menuBarWindowsBorderStyleGroup);

        builder.addLabel("Plastic Border:",  cc.xy(2, 7));
        addBorderStyleGroup(builder, 7, menuBarPlasticBorderStyleGroup);

        builder.addLabel("Plastic 3D Hint:", cc.xy(2, 9));
        addMenuBar3DHintGroup(builder, 9);

        // Tool Bar Options -------------------------------------------------------------
        builder.addSeparator("Tool Bar Style Options", cc.xyw(1, 11, 12));

        builder.addLabel("Header:", cc.xy(2, 13));
        addHeaderStyleGroup(builder, 13, toolBarHeaderStyleGroup);

        builder.addLabel("Windows Border:", cc.xy(2, 15));
        addBorderStyleGroup(builder, 15, toolBarWindowsBorderStyleGroup);

        builder.addLabel("Plastic Border:", cc.xy(2, 17));
        addBorderStyleGroup(builder, 17, toolBarPlasticBorderStyleGroup);

        builder.addLabel("Plastic 3D Hint:", cc.xy(2, 19));
        addToolBar3DHintGroup(builder, 19);

        return builder.getPanel();
    }

    private void preselectLookAndTheme() {
        LookAndFeel defaultLaf = UIManager.getLookAndFeel();
        ListModel model = lookAndFeelList.getModel();
        int size = model.getSize();
        for (int i = 0; i < size; i++) {
            LookAndFeel laf = (LookAndFeel) model.getElementAt(i);
            if (laf.getClass().equals(defaultLaf.getClass())) {
                lookAndFeelList.setSelectedIndex(i);
                break;
            }
        }
        // Preselect the default theme.
        PlasticTheme defaultTheme =
            PlasticLookAndFeel.createMyDefaultTheme();
        model = themeList.getModel();
        size = model.getSize();
        for (int i = 0; i < size; i++) {
            PlasticTheme theme = (PlasticTheme) model.getElementAt(i);
            if (theme.getClass().equals(defaultTheme.getClass())) {
                themeList.setSelectedIndex(i);
                break;
            }
        }

    }

    private void addMenuBar3DHintGroup(PanelBuilder builder, int row) {
        addRadio(
            "None",
            menuBarPlastic3DHintGroup,
            builder,
            4,
            row).setSelected(
            true);
        addRadio("On", menuBarPlastic3DHintGroup, builder, 6, row);
        addRadio("Off", menuBarPlastic3DHintGroup, builder, 8, row);
    }

    private void addToolBar3DHintGroup(PanelBuilder builder, int row) {
        addRadio(
            "None",
            toolBarPlastic3DHintGroup,
            builder,
            4,
            row).setSelected(
            true);
        addRadio("On", toolBarPlastic3DHintGroup, builder, 6, row);
        addRadio("Off", toolBarPlastic3DHintGroup, builder, 8, row);
    }

    private void addHeaderStyleGroup(
        PanelBuilder builder,
        int row,
        ButtonGroup group) {
        addRadio("None", group, builder, 4, row).setSelected(true);
        addRadio("Single", group, builder, 6, row);
        addRadio("Both", group, builder, 8, row);
    }

    private void addBorderStyleGroup(
        PanelBuilder builder,
        int row,
        ButtonGroup group) {
        addRadio("None", group, builder, 4, row).setSelected(true);
        addRadio("Empty", group, builder, 6, row);
        addRadio("Etched", group, builder, 8, row);
        addRadio("Separator", group, builder, 10, row);
    }


    // Helper Methods *******************************************************

    private ListSelectionListener createLookListener() {
        return new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (e.getValueIsAdjusting())
                    return;
                updateThemeEnablement();
            }
        };
    }

    private ListCellRenderer createLookRenderer() {
        return new DefaultListCellRenderer() {
            @Override
            public Component getListCellRendererComponent(
                JList list,
                Object value,
                int index,
                boolean isSelected,
                boolean cellHasFocus) {
                JLabel label =
                    (JLabel) super.getListCellRendererComponent(
                        list,
                        value,
                        index,
                        isSelected,
                        cellHasFocus);
                LookAndFeel laf = (LookAndFeel) value;
                label.setText(laf.getName());
                return label;
            }
        };
    }

    private ListCellRenderer createThemeRenderer() {
        return new DefaultListCellRenderer() {
            @Override
            public Component getListCellRendererComponent(
                JList list,
                Object value,
                int index,
                boolean isSelected,
                boolean cellHasFocus) {
                JLabel label =
                    (JLabel) super.getListCellRendererComponent(
                        list,
                        value,
                        index,
                        isSelected,
                        cellHasFocus);
                PlasticTheme theme = (PlasticTheme) value;
                label.setText(theme.getName());
                return label;
            }
        };
    }

    /**
     * Updates the theme comboBox enablement according to the look and feel selection.
     */
    private void updateThemeEnablement() {
        themeList.setEnabled(
            selectedLookAndFeel() instanceof PlasticLookAndFeel);
    }

    void ensureThemeIsVisible() {
        int index = themeList.getSelectedIndex();
        if (!themeList.isEnabled() || index == -1)
            return;
        themeList.ensureIndexIsVisible(index);
    }


    // Builder Methods ******************************************************

    /**
     * Creates a JRadioButton for the give label and adds it to the button group.
     */
    private JRadioButton addRadio(
        String label,
        ButtonGroup group,
        PanelBuilder builder,
        int col,
        int row) {
        JRadioButton radio = new JRadioButton(label);
        radio.setContentAreaFilled(false);
        radio.setActionCommand(label);
        group.add(radio);
        CellConstraints cc = new CellConstraints();
        builder.add(radio, cc.xy(col, row, "l, c"));
        return radio;
    }

}