/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.metamorphosis;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Rectangle;

import javax.swing.*;
import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;

import com.jgoodies.looks.BorderStyle;
import com.jgoodies.looks.HeaderStyle;
import com.jgoodies.looks.Options;
import com.jgoodies.looks.plastic.Plastic3DLookAndFeel;
import com.jgoodies.looks.plastic.PlasticLookAndFeel;
import com.jgoodies.looks.windows.WindowsLookAndFeel;
import com.jgoodies.uif.osx.OSXApplicationMenu;
import com.jgoodies.uif.util.ScreenUtils;
import com.jgoodies.uif.util.SystemUtils;

/**
 * Main class for the Fakeclipse frame.
 *
 * @author Karsten Lentzsch
 */
public final class Fakeclipse extends JFrame {

    /**
     * Constructs a <code>Fakeclipse</code>, configures the UI,
     * and builds the content.
     */
    private Fakeclipse() {
        configureUI();
        build();
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
    }

    /**
     * Creates and opens a <code>DemoFrame</code> that can access
     * the given <code>Launcher</code>; uses the specified bounds,
     * if non-null.
     */
    static Fakeclipse openOn(Style style, Rectangle bounds) {
        Style.current = style;
        Fakeclipse instance = new Fakeclipse();
        if (bounds == null) {
            instance.setSize(800, 600);
            ScreenUtils.locateOnOpticalScreenCenter(instance);
        } else
            instance.setBounds(bounds);
        instance.setVisible(true);
        return instance;
    }

    /**
     * Sets a style and opens the Fakeclipse frame.
     */
    public static void main(String[] args) {
        OSXApplicationMenu.setAboutName("Fakeclipse");
        String styleName =
            System.getProperty("style", Style.ELEGANT.toString());
        openOn(Style.valueOf(styleName), null);
    }

    /**
     * Configures the user interface: sets look and theme.
     */
    private void configureUI() {
        Options.setDefaultIconSize(new Dimension(18, 18));
        try {
            if (Style.getCurrent().useExtendedLookAndFeel()) {
                Options.setUseSystemFonts(true);
                if (Utils.isWindowsPlatform()
                    && Style.getCurrent().forceSystemLookAndFeel()) {
                    UIManager.setLookAndFeel(new WindowsLookAndFeel());
                } else {
                    MetalLookAndFeel.setCurrentTheme(
                        PlasticLookAndFeel.createMyDefaultTheme());
                    UIManager.setLookAndFeel(new Plastic3DLookAndFeel());
                }
            } else {
                if (Style.getCurrent().forceSystemLookAndFeel())
                    UIManager.setLookAndFeel(
                        UIManager.getSystemLookAndFeelClassName());
                else {
                    if (Style.getCurrent().forceBetterColorTheme()) {
                        MetalLookAndFeel.setCurrentTheme(
                            PlasticLookAndFeel.createMyDefaultTheme());
                    } else {
                        MetalLookAndFeel.setCurrentTheme(
                            new DefaultMetalTheme());
                    }
                    UIManager.setLookAndFeel(
                        UIManager.getCrossPlatformLookAndFeelClassName());
                }
            }
        } catch (Exception e) {
            System.out.println("Can't change L&F: " + e);
        }
    }

    /**
     * Builds the <code>Fakeclipse</code> frame.
     */
    private void build() {
        setContentPane(buildContentPane());
        setTitle(Style.getCurrent().toString() + " - Fakeclipse Platform");
        setJMenuBar(new MenuBarView().getMenuBar());
        setIconImage(Utils.getIcon(
            SystemUtils.IS_OS_MAC
                ? "logo64x64.gif"
                : "logo16x16.gif").getImage());
    }

    /**
     * Builds and answers the content pane.
     */
    private JComponent buildContentPane() {
        JPanel panel = new JPanel(new BorderLayout());
        panel.add(buildToolBar(), BorderLayout.NORTH);
        panel.add(buildSideBar(), BorderLayout.WEST);
        panel.add(buildMainStatus(), BorderLayout.CENTER);
        return panel;
    }

    /**
     * Builds and answers the tool bar.
     */
    private Component buildToolBar() {
        JLabel label = new JLabel(Utils.getIcon("topbar.gif"));
        label.setHorizontalAlignment(SwingConstants.LEFT);

        JToolBar toolBar = new JToolBar();
        toolBar.putClientProperty(Options.HEADER_STYLE_KEY, HeaderStyle.BOTH);
        toolBar.putClientProperty(
            WindowsLookAndFeel.BORDER_STYLE_KEY,
            BorderStyle.SEPARATOR);
        toolBar.updateUI();
        toolBar.add(label);
        if (Style.getCurrent() != Style.FRANKEN)
            return toolBar;
        JPanel panel = new JPanel(new BorderLayout());
        panel.add(toolBar, BorderLayout.NORTH);
        panel.add(new JSeparator(), BorderLayout.CENTER);
        return panel;
    }

    /**
     * Builds and answers the perspective and view side bar.
     */
    private Component buildSideBar() {
        JLabel label = new JLabel(Utils.getIcon("sidebar.gif"));
        label.setVerticalAlignment(SwingConstants.TOP);
        JPanel panel = new JPanel(new BorderLayout(2, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(new JSeparator(SwingConstants.VERTICAL), BorderLayout.CENTER);
        return panel;
    }

    /**
     * Builds and answers the edit panel.
     */
    private Component buildEditPanel() {
        JPanel panel =
            UIFactory.createEditorPanel(
                Utils.getIcon("javasrc.gif"),
                "description.txt");
        JTextArea area = new JTextArea();
        area.setPreferredSize(new Dimension(80, 200));
        area.setLineWrap(true);
        area.setWrapStyleWord(true);
        area.setText(Style.getCurrent().getDescription());
        JPanel barAndArea = new JPanel(new BorderLayout());
        barAndArea.add(Box.createHorizontalStrut(12), BorderLayout.WEST);
        barAndArea.add(area, BorderLayout.CENTER);
        JScrollPane scrollPane = UIFactory.createScrollPane(barAndArea);
        panel.add(scrollPane, BorderLayout.CENTER);
        return panel;
    }

    /**
     * Builds and answers the main panel plus status bar.
     */
    private Component buildMainStatus() {
        JPanel panel = new JPanel(new BorderLayout());
        JSplitPane split =
            UIFactory.createSplitPane(
                JSplitPane.HORIZONTAL_SPLIT,
                new PackagesPanel().build(),
                buildEditOutlineTasks(),
                0.3);
        if (Style.getCurrent().reduceBorders()) {
            split.setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 1));
        }

        panel.add(split, BorderLayout.CENTER);
        panel.add(buildStatusBar(), BorderLayout.SOUTH);
        return panel;
    }

    /**
     * Builds and answers the edit, outline, and tasks panels separated by split panes.
     */
    private Component buildEditOutlineTasks() {
        return UIFactory.createSplitPane(
            JSplitPane.VERTICAL_SPLIT,
            buildEditOutline(),
            new TasksPanel().build(),
            1);
    }

    /**
     * Builds and answers the edit and outline panel separated by a split pane.
     */
    private Component buildEditOutline() {
        return UIFactory.createSplitPane(
            JSplitPane.HORIZONTAL_SPLIT,
            buildEditPanel(),
            buildOutlinePanel(),
            0.7d);
    }

    /**
     * Builds and returns the outline panel.
     */
    private Component buildOutlinePanel() {
        JPanel panel =
            UIFactory.createEclipsePanel(
                Utils.getIcon("outline.gif"),
                "Outline",
                Utils.getIcon("outline_tools.gif"),
                false);
        JPanel content = new JPanel(new BorderLayout());
        content.add(
            new JLabel("An outline is not availiable."),
            BorderLayout.NORTH);
        content.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

        panel.add(UIFactory.createScrollPane(content), BorderLayout.CENTER);
        panel.setPreferredSize(new Dimension(80, 300));
        return panel;
    }

    /**
     * Builds and returns the status bar.
     */
    private Component buildStatusBar() {
        JLabel label =
            new JLabel(" com/jgoodies/metamorphosis/description.txt");
        label.setBorder(
            BorderFactory.createCompoundBorder(
                new Borders.ThinStatusBorder(),
                BorderFactory.createEmptyBorder(2, 2, 2, 2)));
        return label;
    }

}