/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.metamorphosis;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JPanel;

import com.jgoodies.animation.Animation;
import com.jgoodies.animation.AnimationUtils;
import com.jgoodies.animation.Animations;
import com.jgoodies.animation.animations.BasicTextAnimation;
import com.jgoodies.animation.animations.BasicTextAnimations;
import com.jgoodies.animation.animations.GlyphAnimation;
import com.jgoodies.animation.components.BasicTextLabel;
import com.jgoodies.animation.components.GlyphLabel;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif.util.SystemUtils;

/**
 * Builds the Metamorphosis intro panel and provides the animation.
 *
 * @author Karsten Lentzsch
 */
final class IntroPageView {

    private final Metamorphosis metamorphosis;

    private BasicTextLabel label1;
    private BasicTextLabel label2;
    private GlyphLabel     glyphLabel;
    private JButton        skipButton;
    private Animation      animation;

    /**
     * Constructs an <code>IntroPanel</code> for a given Metamorphosis instance.
     */
    IntroPageView(Metamorphosis metamorphosis) {
        this.metamorphosis = metamorphosis;
    }


    // API ******************************************************************

    /**
     * Returns the intro animation.
     */
    Animation animation() {
        return animation;
    }


    // Building *************************************************************

    /**
     * Creates and configures the UI components.
     */
    private void initComponents() {
        label1 = new BasicTextLabel(" ");
        label1.setFont(Metamorphosis.META_FONT);
        label1.setBounds(0, 0, 400, 100);
        label1.setOpaque(false);

        label2 = new BasicTextLabel(" ");
        label2.setFont(Metamorphosis.META_FONT);
        label2.setBounds(0, 0, 400, 100);
        label2.setOpaque(false);

        glyphLabel = new GlyphLabel(" ", 5000, 100);
        glyphLabel.setFont(Metamorphosis.META_FONT);
        glyphLabel.setBounds(0, 0, 400, 100);
        glyphLabel.setOpaque(false);

        skipButton = new JButton("Skip Intro");
        skipButton.setFocusPainted(false);
        skipButton.setOpaque(!SystemUtils.IS_OS_MAC);
        skipButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                goToStart();
            }
        });
    }

    /**
     * Builds and answers the intro panel.
     */
    Component build() {
        initComponents();

        // I cannot use Borders.DLU7, just because
        // the horizontal separators span the whole page.
        FormLayout layout = new FormLayout(
                "8dlu, fill:max(200dlu;pref):grow, 8dlu",
                "8dlu, 23dlu:grow, 3dlu, fill:max(30dlu;pref):grow, 3dlu, 23dlu:grow, 8dlu");
        PanelBuilder builder = new PanelBuilder(layout);
        builder.setBackground(Color.WHITE);
        CellConstraints cc = new CellConstraints();

        builder.add(createColoredBar(), cc.xyw(1, 3, 3));
        builder.add(label1,             cc.xy (2, 4));
        builder.add(label2,             cc.xy (2, 4));
        builder.add(glyphLabel,         cc.xy (2, 4));
        builder.add(createColoredBar(), cc.xyw(1, 5, 3));
        builder.add(skipButton,         cc.xy (2, 6, "right, bottom"));

        animation = createAnimation();
        return builder.getPanel();
    }

    /**
     * Creates and answers a gray bar.
     */
    private Component createColoredBar() {
        JPanel panel = new JPanel(null);
        panel.setBackground(Color.GRAY);
        return panel;
    }


    // Animation Creation ***************************************************

    /**
     * Creates and answers a composed animation for the intro.
     */
    private Animation createAnimation() {
        Animation a0 =
            BasicTextAnimations.defaultFade(
                label1,
                label2,
                3000,
                100,
                "Karsten Lentzsch|presents|The Swing Metamorphosis|A Short Course In",
                Color.DARK_GRAY);

        Animation a1 =
            new GlyphAnimation(glyphLabel, 5000, 1, "Swing Visual Design");

        Animation a2 =
            BasicTextAnimations.defaultFade(
                label1,
                label2,
                2000,
                -400,
                "Five Steps|From Ugly|To Elegant.",
                Color.DARK_GRAY);

        Animation a3 =
            BasicTextAnimation.defaultScale(
                label1,
                2500,
                "Step 1:",
                Color.DARK_GRAY);

        BasicTextAnimation a4 =
            BasicTextAnimation.defaultFade(
                label1,
                4000,
                "METAL L&F FRANKEN UI",
                Color.red.darker());
        a4.setOffsetEnabled(true);

        Animation a5 =
            BasicTextAnimation.defaultScale(
                label1,
                2500,
                "Step 2:",
                Color.DARK_GRAY);

        BasicTextAnimation a6 =
            BasicTextAnimation.defaultFade(
                label1,
                4000,
                "A ROOKIE UI",
                Color.DARK_GRAY);

        Animation a7 =
            BasicTextAnimation.defaultScale(
                label1,
                2500,
                "Step 3:",
                Color.DARK_GRAY);

        BasicTextAnimation a8 =
            BasicTextAnimation.defaultFade(
                label1,
                4000,
                "A STANDARD UI",
                Color.DARK_GRAY);

        Animation a9 =
            BasicTextAnimation.defaultScale(
                label1,
                2500,
                "Step 4:",
                Color.DARK_GRAY);

        Animation a10 =
            new GlyphAnimation(glyphLabel, 5000, 1, "AN ADVANCED UI");

        Animation a11 =
            BasicTextAnimation.defaultScale(
                label1,
                2500,
                "Step 5:",
                Color.DARK_GRAY);

        Animation a12 =
            BasicTextAnimation.defaultSpace(
                label1,
                6500,
                "AN ELEGANT SWING UI",
                Color.DARK_GRAY);

        Animation all =
            Animations.sequential(
                new Animation[] {
                    Animations.pause(1000),
                    a0,
                    a1,
                    Animations.pause(1000),
                    a2,
                    Animations.pause(1000),
                    a3,
                    a4,
                    a5,
                    a6,
                    a7,
                    a8,
                    a9,
                    a10,
                    a11,
                    a12,
                    Animations.pause(2000),
                    });

        Runnable runnable = new Runnable() {
            public void run() {
                goToStart();
            }
        };
        AnimationUtils.invokeOnStop(all, runnable);

        return all;
    }

    // Helper Methods *******************************************************

    /**
     * Goes to the start panel.
     */
    private void goToStart() {
        metamorphosis.goToStart();
    }

}