/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.metamorphosis;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.jgoodies.looks.BorderStyle;
import com.jgoodies.looks.HeaderStyle;
import com.jgoodies.looks.Options;
import com.jgoodies.looks.windows.WindowsLookAndFeel;

/**
 * Builds a menu bar that demonstrates a bunch of menus
 * with a mix different menu items and menu item states.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.5 $
 */
final class MenuBarView {

    /**
     * Holds the lazily created menu bar.
     */
    private JMenuBar menuBar;


    // API ********************************************************************

    /**
     * Lazily builds and returns this view's menu bar.
     *
     * @return the lazily built menu bar
     */
    JMenuBar getMenuBar() {
        if (menuBar == null) {
            menuBar = buildMenuBar();
        }
        return menuBar;
    }



    // Building ***************************************************************

    /**
     * Builds and returns the menu bar.
     */
    private JMenuBar buildMenuBar() {
        JMenuBar bar = new JMenuBar();
        bar.putClientProperty(Options.HEADER_STYLE_KEY, HeaderStyle.BOTH);
        bar.putClientProperty(
            WindowsLookAndFeel.BORDER_STYLE_KEY,
            BorderStyle.SEPARATOR);
        JMenu menu;

        bar.add(buildFileMenu());

        menu = new JMenu("Edit");
        menu.setMnemonic('E');
        addRadioMenuItems(menu);
        bar.add(menu);

        menu = new JMenu("Source");
        menu.setMnemonic('S');
        addCheckMenuItems(menu);
        bar.add(menu);

        bar.add(buildStubMenu("Refactor", 't'));
        bar.add(buildStubMenu("Navigate", 'N'));
        bar.add(buildStubMenu("Search", 'a'));
        bar.add(buildStubMenu("Project", 'P'));
        bar.add(buildStubMenu("Run", 'R'));
        bar.add(buildStubMenu("Window", 'W'));
        bar.add(buildStubMenu("Help", 'H'));
        return bar;
    }

    /**
     * Builds and answers the file menu.
     */
    JMenu buildFileMenu() {
        JMenuItem item;

        JMenu menu = new JMenu("File");
        menu.setMnemonic('F');

        // Build a submenu that has the noIcons hint set.
        JMenu submenu = new JMenu("New");
        submenu.putClientProperty(Options.NO_ICONS_KEY, Boolean.TRUE);
        submenu.add(new JMenuItem("Project\u2026"));
        submenu.add(new JMenuItem("Folder\u2026"));
        submenu.add(new JMenuItem("Document\u2026"));
        submenu.addSeparator();
        submenu.add(new JMenuItem("This submenu has no icons"));

        menu.add(submenu);
        menu.addSeparator();
        item = new JMenuItem("Close");
        item.setAccelerator(KeyStroke.getKeyStroke("ctrl F4"));
        menu.add(item);
        item = new JMenuItem("Close All");
        item.setAccelerator(KeyStroke.getKeyStroke("ctrl shift F4"));
        menu.add(item);
        menu.addSeparator();
        item =
            new JMenuItem(
                "Save description.txt",
                Utils.getIcon("save_edit.gif"));
        item.setAccelerator(KeyStroke.getKeyStroke("ctrl S"));
        item.setEnabled(false);
        menu.add(item);
        menu.add(
            new JMenuItem(
                "Save description.txt As\u2026",
                Utils.getIcon("saveas_edit.gif")));
        item = new JMenuItem("Save All", Utils.getIcon("saveall_edit.gif"));
        item.setAccelerator(KeyStroke.getKeyStroke("ctrl shift S"));
        item.setEnabled(false);
        menu.add(item);
        menu.add(new JMenuItem("Revert"));
        menu.addSeparator();
        menu.add(new JMenuItem("Move\u2026"));
        menu.add(new JMenuItem("Rename\u2026"));
        menu.addSeparator();
        menu.add(new JMenuItem("Refresh"));
        menu.addSeparator();
        item = new JMenuItem("Print", Utils.getIcon("print.gif"));
        item.setAccelerator(KeyStroke.getKeyStroke("ctrl P"));
        menu.add(item);
        menu.addSeparator();
        menu.add(new JMenuItem("Import", Utils.getIcon("import_wiz.gif")));
        menu.add(new JMenuItem("Export", Utils.getIcon("export_wiz.gif")));
        menu.addSeparator();
        item = new JMenuItem("Properties");
        item.setAccelerator(KeyStroke.getKeyStroke("alt ENTER"));
        menu.add(item);
        menu.addSeparator();
        menu.add(new JMenuItem("1 WinXPMenuItemUI.java"));
        menu.add(new JMenuItem("2 WinXPUtils.java"));
        menu.add(new JMenuItem("3 WinXPBorders.java"));
        menu.add(new JMenuItem("4 WinXPLookAndFeel.java"));
        menu.addSeparator();
        menu.add(new JMenuItem("Exit"));
        return menu;
    }

    /**
     * Creates and answer a stub menu for the given name.
     */
    private JMenu buildStubMenu(String name, char mnemonic) {
        JMenu menu = new JMenu(name);
        menu.setMnemonic(mnemonic);
        menu.add(new JMenuItem("Fake"));
        return menu;
    }

    /**
     * Builds and answers a menu with different JRadioButtonMenuItems.
     */
    void addRadioMenuItems(JMenu menu) {
        JRadioButtonMenuItem item;

        // Default icon
        ButtonGroup group1 = new ButtonGroup();
        item = createRadioItem(true, false);
        group1.add(item);
        menu.add(item);
        item = createRadioItem(true, true);
        group1.add(item);
        menu.add(item);

        menu.addSeparator();

        item = createRadioItem(false, false);
        menu.add(item);
        item = createRadioItem(false, true);
        menu.add(item);

        menu.addSeparator();

        // Custom icon
        ButtonGroup group2 = new ButtonGroup();
        item = createRadioItem(true, false);
        item.setIcon(Utils.getIcon("pie_mode.gif"));
        group2.add(item);
        menu.add(item);
        item = createRadioItem(true, true);
        item.setIcon(Utils.getIcon("bar_mode.gif"));
        group2.add(item);
        menu.add(item);

        menu.addSeparator();

        item = createRadioItem(false, false);
        item.setIcon(Utils.getIcon("alphab_sort.png"));
        //item.setDisabledIcon(Utils.getIcon("alphab_sort_gray.png"));
        menu.add(item);
        item = createRadioItem(false, true);
        item.setIcon(Utils.getIcon("size_sort.png"));
        item.setDisabledIcon(Utils.getIcon("size_sort_gray.png"));
        menu.add(item);
    }

    /**
     * Builds and answers a menu with different JCheckBoxMenuItems.
     */
    void addCheckMenuItems(JMenu menu) {
        JCheckBoxMenuItem item;

        // Default icon
        menu.add(createCheckItem(true, false));
        menu.add(createCheckItem(true, true));
        menu.addSeparator();
        menu.add(createCheckItem(false, false));
        menu.add(createCheckItem(false, true));

        menu.addSeparator();

        // Custom icon
        item = createCheckItem(true, false);
        item.setIcon(Utils.getIcon("check.gif"));
        item.setSelectedIcon(Utils.getIcon("check_selected.gif"));
        menu.add(item);
        item = createCheckItem(true, true);
        item.setIcon(Utils.getIcon("check.gif"));
        item.setSelectedIcon(Utils.getIcon("check_selected.gif"));
        menu.add(item);

        menu.addSeparator();

        item = createCheckItem(false, false);
        item.setIcon(Utils.getIcon("check.gif"));
        item.setSelectedIcon(Utils.getIcon("check_selected.gif"));
        menu.add(item);
        item = createCheckItem(false, true);
        item.setIcon(Utils.getIcon("check.gif"));
        item.setSelectedIcon(Utils.getIcon("check_selected.gif"));
        item.setDisabledSelectedIcon(
            Utils.getIcon("check_disabled_selected.gif"));
        menu.add(item);
    }

    /**
     * Creates and answers a radio button with the given enablement
     * and selection state.
     */
    private JRadioButtonMenuItem createRadioItem(
        boolean enabled,
        boolean selected) {
        JRadioButtonMenuItem item =
            new JRadioButtonMenuItem(
                getToggleLabel(enabled, selected),
                selected);
        item.setEnabled(enabled);
        item.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                JRadioButtonMenuItem source =
                    (JRadioButtonMenuItem) e.getSource();
                source.setText(
                    getToggleLabel(source.isEnabled(), source.isSelected()));
            }
        });
        return item;
    }

    /**
     * Creates and answers a check box with the given enablement
     * and selection state.
     */
    private JCheckBoxMenuItem createCheckItem(
        boolean enabled,
        boolean selected) {
        JCheckBoxMenuItem item =
            new JCheckBoxMenuItem(
                getToggleLabel(enabled, selected),
                selected);
        item.setEnabled(enabled);
        item.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                JCheckBoxMenuItem source = (JCheckBoxMenuItem) e.getSource();
                source.setText(
                    getToggleLabel(source.isEnabled(), source.isSelected()));
            }
        });
        return item;
    }

    /**
     * Return a string label for the given enablement and selection state.
     */
    private String getToggleLabel(boolean enabled, boolean selected) {
        String prefix = enabled ? "Enabled" : "Disabled";
        String suffix = selected ? "Selected" : "Deselected";
        return prefix + " and " + suffix;
    }

}