/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */
package com.jgoodies.metamorphosis;

import java.awt.Font;
import java.awt.FontFormatException;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.InputStream;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.UIManager;

import com.jgoodies.animation.Animator;
import com.jgoodies.looks.plastic.PlasticLookAndFeel;
import com.jgoodies.looks.plastic.PlasticXPLookAndFeel;
import com.jgoodies.uif.osx.OSXApplicationMenu;
import com.jgoodies.uif.panel.CardPanel;
import com.jgoodies.uif.util.ScreenUtils;
import com.jgoodies.uif.util.SystemUtils;

/**
 * The Metamorphosis main class.
 *
 * @author Karsten Lentzsch
 */
public final class Metamorphosis extends JFrame {

    private static final String VERSION    = "1.0.3 (July 30, 2008)";
    private static final String COPYRIGHT  = "\u00a9 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.";

             static final Font   META_FONT  = getFont("resources/font.ttf", 24);
    private static final String INTRO_CARD = "intro";
    private static final String START_CARD = "start";

    private IntroPageView intro;
    private StartPageView start;
    private CardPanel cardPanel;

    private static Metamorphosis instance;
    private Fakeclipse fakeclipse;

    /**
     * Constructs the Metamorphosis (main) frame.
     */
    private Metamorphosis() {
        // Override default constructor; prevents instantiation.
    }

    /**
     * Instantiates Metamorphosis, builds, and opens the frame and starts the intro.
     */
    public static void main(String[] args) {
        OSXApplicationMenu.setAboutName("Metamorphosis");
        System.setProperty("apple.laf.useScreenMenuBar", "true");
        OSXApplicationMenu.register(
            new AboutActionHandler(),
            null,
            new QuitActionHandler());
        instance = new Metamorphosis();
        instance.build();
    }

    /**
     * Creates and configures the UI components.
     */
    private void initComponents() {
        intro     = new IntroPageView(this);
        start     = new StartPageView(this);
        cardPanel = new CardPanel();
    }

    /**
     * Builds the <code>Fakeclipse</code> frame.
     */
    private void build() {
        initComponents();
        setContentPane(buildContentPane());
        setTitle("The Swing Metamorphosis");
        setIconImage(Utils.getIcon(
            SystemUtils.IS_OS_MAC
                ? "logo64x64.gif"
                : "logo16x16.gif").getImage());
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        setSize(500, 330);
        ScreenUtils.locateOnOpticalScreenCenter(this);
        setVisible(true);

        new Animator(intro.animation(), 25).start();
    }

    /**
     * Builds and answers the content pane:
     * an intro and a start panel composed via a card panel.
     */
    public JComponent buildContentPane() {
        configureUI();
        cardPanel.add(intro.build(), INTRO_CARD);
        cardPanel.add(start.build(), START_CARD);
        return cardPanel;
    }

    /**
     * Configures the UI: sets font size hints and tries to set Plastic3D.
     */
    private static void configureUI() {
        try {
            if (SystemUtils.IS_OS_MAC) {
                UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            } else {
                UIManager.setLookAndFeel(new PlasticXPLookAndFeel());
                PlasticLookAndFeel.setPlasticTheme(
                    PlasticLookAndFeel.createMyDefaultTheme());
            }
        } catch (Exception e) {
            System.out.println("Can't change L&F: " + e);
        }
    }

    /**
     * Goes to the start panel.
     */
    void goToStart() {
        cardPanel.showCard(START_CARD);
        start.start();
    }

    /**
     * Launches Fakeclipse: stores bounds of an existing frame,
     * configures the fake, opens the fake, and finally listens to
     * fake frame close events.
     */
    void doLaunch(Style style) {
        Rectangle bounds = null;
        if (fakeclipse != null) {
            bounds = fakeclipse.getBounds();
            //fakeclipse.removeWindowListener(restoreListener);
            fakeclipse.dispose();
        }
        fakeclipse = Fakeclipse.openOn(style, bounds);
        //fakeclipse.addWindowListener(restoreListener);
    }

    /**
     * Looks up and answer a font for the specified filename and size.
     */
    private static Font getFont(String filename, int size) {
        InputStream in = Metamorphosis.class.getResourceAsStream(filename);
        try {
            Font font = Font.createFont(Font.TRUETYPE_FONT, in);
            return font.deriveFont(Font.PLAIN, size);
        } catch (IOException e) {
            System.out.println("Can't load font.");
        } catch (FontFormatException e) {
            System.out.println("Font has bad format.");
        }
        return getBoldDialogFont(size);
    }

    private static Font getBoldDialogFont(int size) {
        return new Font("dialog", Font.BOLD, size);
    }

    // *************************************************************

    private static class AboutActionHandler implements ActionListener {

        public void actionPerformed(ActionEvent evt) {
            JOptionPane.showMessageDialog(instance,
                "The Swing Metamorphosis\n"
                + COPYRIGHT + "\n\nVersion " + VERSION);
        }
    }

    private static class QuitActionHandler implements ActionListener {

        public void actionPerformed(ActionEvent evt) {
            System.exit(0);
        }
    }


}