/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.metamorphosis;

import java.awt.Component;

import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.table.*;

/**
 * This extension of class JTable provides behavior for a default
 * alignment and sizing of table columns. Therefore it utilizes
 * two arrays and a List for the column alignment, header alignment,
 * and the column size hints.
 *
 * @author Karsten Lentzsch
 */

final class SpecialReadOnlyTable extends JTable {

    //private int[] columnAlignments = null;
    private int[] headerAlignments = null;
    private int[] columnSizeHints;

    public SpecialReadOnlyTable() {
        if (Style.getCurrent() != Style.FRANKEN)
            setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

        // Set the selection configuration.
        setRowSelectionAllowed(false);
        setColumnSelectionAllowed(false);
        setCellSelectionEnabled(false);
        setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        setEnabled(false);
    }

    private TableCellRenderer createLeftAlignedDefaultHeaderRenderer() {
        DefaultTableCellRenderer label = new DefaultTableCellRenderer() {
            @Override
            public Component getTableCellRendererComponent(
                JTable table,
                Object value,
                boolean isSelected,
                boolean hasFocus,
                int row,
                int column) {
                if (table != null) {
                    JTableHeader header = table.getTableHeader();
                    if (header != null) {
                        this.setForeground(header.getForeground());
                        this.setBackground(header.getBackground());
                        this.setFont(header.getFont());
                    }
                }
                setText((value == null) ? "" : value.toString());
                this.setBorder(UIManager.getBorder("TableHeader.cellBorder"));
                return this;
            }
        };
        label.setHorizontalAlignment(SwingConstants.LEFT);
        return label;
    }

    // Modify the column's header to be aligned left.
    private void leftAlignColumn(TableColumn column) {
        TableCellRenderer headerRenderer = column.getHeaderRenderer();
        if (headerRenderer instanceof DefaultTableCellRenderer)
            (
                (
                    DefaultTableCellRenderer) headerRenderer)
                        .setHorizontalAlignment(
                SwingConstants.LEFT);
        else
            column.setHeaderRenderer(
                createLeftAlignedDefaultHeaderRenderer());
    }

    private void configureColumnHeaders() {
        if (null == headerAlignments)
            return;

        for (int col = 0; col < getModel().getColumnCount(); col++) {
            if (headerAlignments[col] == SwingConstants.LEFT)
                leftAlignColumn(getColumnModel().getColumn(col));
        }
    }

    private void configureColumnSizes() {
        for (int col = 0; col < getModel().getColumnCount(); col++) {
            int sizeHint = columnSizeHints[col];
            if (sizeHint != -1)
                getColumnModel().getColumn(col).setPreferredWidth(sizeHint);
        }
    }

    public void setColumnSizes(int[] columnSizeHints) {
        this.columnSizeHints = columnSizeHints;
    }

    public void setHeaderAlignments(int[] alignments) {
        this.headerAlignments = alignments;
    }

    @Override
    public void setModel(TableModel model) {
        super.setModel(model);

        if (Style.getCurrent() != Style.FRANKEN) {
            configureColumnHeaders();
            configureColumnSizes();
        }
        revalidate();
    }

    @Override
    public void updateUI() {
        super.updateUI();
        configureEnclosingScrollPane();
        setRowHeight(Math.max(UIManager.getInt("Tree.rowHeight"), 16) - 1);
    }
}