/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.metamorphosis;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.util.LinkedList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;

import com.jgoodies.animation.Animation;
import com.jgoodies.animation.Animations;
import com.jgoodies.animation.Animator;
import com.jgoodies.animation.animations.BasicTextAnimations;
import com.jgoodies.animation.animations.FanAnimation;
import com.jgoodies.animation.components.BasicTextLabel;
import com.jgoodies.animation.components.FanComponent;
import com.jgoodies.animation.components.GlyphLabel;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.looks.plastic.PlasticLookAndFeel;
import com.jgoodies.uif.component.UIFLabel;
import com.jgoodies.uif.util.SystemUtils;


/**
 * Builds the Metamorphosis start panel and provides the fan animation.
 *
 * @author Karsten Lentzsch
 */
final class StartPageView {

	private final Metamorphosis metamorphosis;

    private final List<JButton> buttons;
	private FanComponent	fan;
	private BasicTextLabel label1;
	private BasicTextLabel label2;
	private GlyphLabel     glyphLabel;
	private Animator		animator;


	/**
	 * Constructs the start panel.
	 */
	StartPageView(Metamorphosis metamorphosis) {
		this.metamorphosis = metamorphosis;
		buttons = new LinkedList<JButton>();
	}


	/**
	 * Starts the fan animation.
	 */
	void start() { animator.start(); }


	// Building *************************************************************************

    /**
     * Creates and configures the UI components.
     */
    private void initComponents() {
        Color baseColor = Color.LIGHT_GRAY;
        fan = new FanComponent(10, baseColor);
        fan.addMouseMotionListener(new MouseMotionAdapter() {
            @Override
            public void mouseMoved(MouseEvent e) {
                fan.setOrigin(e.getPoint());
            }
        });

        label1 = new BasicTextLabel(" ");
        label1.setFont(Metamorphosis.META_FONT);

        label2 = new BasicTextLabel(" ");
        label2.setFont(Metamorphosis.META_FONT);

        glyphLabel = new GlyphLabel(" ", 5000, 100);
        glyphLabel.setFont(Metamorphosis.META_FONT);
    }

	/**
	 * Builds and returns the start panel component.
	 */
	Component build() {
        initComponents();

        FormLayout layout = new FormLayout(
                "25px, 0:grow, right:pref, 4dlu, pref, 0:grow, 25px",
                "25px, 0:grow, pref, 4dlu, pref, 4dlu, pref, 4dlu, pref, 4dlu, pref, 25px, pref, 25px");
		PanelBuilder builder = new PanelBuilder(layout);
        CellConstraints cc = new CellConstraints();
        builder.setBackground(Color.WHITE);

        Component label  = createLabel("Step 1:");
        Component button = createButton("Show Franken UI", Style.FRANKEN);
        builder.add(label,                 cc.xy(3, 3));
        builder.add(button,                cc.xy(5, 3));

        label  = createLabel("Step 2:");
        button = createButton("Show Rookie UI", Style.ROOKIE);
        builder.add(label,                 cc.xy(3, 5));
        builder.add(button,                cc.xy(5, 5));

        label  = createLabel("Step 3:");
        button = createButton("Show Standard UI", Style.STANDARD);
        builder.add(label,                 cc.xy(3, 7));
        builder.add(button,                cc.xy(5, 7));

        label  = createLabel("Step 4:");
        button = createButton("Show Advanced UI", Style.ADVANCED);
        builder.add(label,                 cc.xy(3, 9));
        builder.add(button,                cc.xy(5, 9));

        label  = createLabel("Step 5:");
        button = createButton("Show Elegant UI", Style.ELEGANT);
        builder.add(label,                 cc.xy(3, 11));
        builder.add(button,                cc.xy(5, 11));

		builder.add(buildAnimationPanel(), cc.xyw(2, 13, 5));

		animator = new Animator(createAnimation(), 30);

		return builder.getPanel();
	}


	/**
	 * Builds and answers the animation panel that consists of
	 * the fan component.
	 */
	private Component buildAnimationPanel() {
        FormLayout layout = new FormLayout("fill:200px:grow", "fill:50px");
        PanelBuilder builder = new PanelBuilder(layout);
        CellConstraints cc = new CellConstraints();
        builder.setOpaque(false);
        builder.setBorder(BorderFactory.createLineBorder(Color.GRAY));

		builder.add(label1,     cc.xy(1, 1));
		builder.add(label2,     cc.xy(1, 1));
		builder.add(glyphLabel, cc.xy(1, 1));
		builder.add(fan,        cc.xy(1, 1));

		return builder.getPanel();
	}


	/**
	 * Creates and answers a label that is large, anti-aliased, and uses a custom font.
	 */
	private Component createLabel(String text) {
		JLabel label = new UIFLabel(text, Font.PLAIN, 0, true);
		label.setFont(Metamorphosis.META_FONT);
		label.setForeground(Color.DARK_GRAY);
		return label;
	}


	/**
	 * Creates and answers a button that launches the specified style.
	 */
	private JButton createButton(String label, final Style style) {
		JButton button = new JButton(label);
		button.putClientProperty(PlasticLookAndFeel.IS_3D_KEY, Boolean.TRUE);
		button.setFocusPainted(false);
        button.setOpaque(!SystemUtils.IS_OS_MAC);
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				metamorphosis.doLaunch(style);
			}
		});
		buttons.add(button);
		return button;
	}


	/*
	private JButton createCustomizeButton() {
		JButton button = new JButton("Show Custom UI\u2026");
		button.addActionListener(
		new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				metamorphosis.doLaunch(null);
			}
		});
		return button;
	}
	*/


	// Animation Creation ***************************************************************

	/**
	 * Creates and answers a composed animation with a fan animation in
	 * the background and a sequential text animation in the foreground.
	 */
	private Animation createAnimation() {
		String text  = "Karsten Lentzsch|UI DESIGN|UI IMPLEMENTATION|www.JGoodies.com";
		Color  color = Color.DARK_GRAY;

		Animation fade1  = BasicTextAnimations.defaultFade(
			label1, label2, 4500, 0, text, color);

		Animation fade2  = BasicTextAnimations.defaultFade(
			label1, label2, 4500, -250, text, color);

		Animation fade3  = BasicTextAnimations.defaultFade(
			label1, label2, 4500, -500, text, color);

		Animation space = BasicTextAnimations.defaultSpace(
			label1, label2, 4500, 0, text, color);

		Animation fanAnimation = FanAnimation.defaultFan(fan, 20000);
		Animation typo = Animations.sequential(new Animation[]{fade1, fade2, fade3, space});

		return Animations.repeat(1000f, Animations.parallel(
			Animations.repeat(1000f, fanAnimation),
			Animations.repeat(1000f, typo)));
	}


}
