/*
 * Copyright (c) 2002-2006 Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of  Karsten Lentzsch. Use is
 * subject to license terms.
 *
 */

package com.jgoodies.metamorphosis;

import java.util.LinkedList;
import java.util.List;
import java.util.Locale;

/**
 * This class provides information about the choose UI style,
 * e.g. which icon or border style should be applied.
 *
 * @author Karsten Lentzsch
 */

final class Style {

	private static final List<Style> ALL = new LinkedList<Style>();

	static final Style FRANKEN   = new Style("Franken",  IconStyle.MIXED, false, false, false, false, false, false);
	static final Style ROOKIE    = new Style("Rookie",   IconStyle.JFA,   true,  true,  false, false, false, false);
	static final Style STANDARD  = new Style("Standard", IconStyle.WIN,   true,  true,  true,  false, false, false);
	static final Style ADVANCED  = new Style("Advanced", IconStyle.WIN,   true,  true,  true,  true,  false, false);
	static final Style ELEGANT   = new Style("Elegant",  IconStyle.WIN,   true,  true,  true,  true,  true,  true);
	static final Style STANDARDX = new Style("StandardX",IconStyle.WIN,   true,  true,  false, false, false, false);
	static final Style ADVANCEDX = new Style("AdvancedX",IconStyle.WIN,   true,  true,  false, true,  false, false);
	static final Style ELEGANTX  = new Style("ElegantX", IconStyle.WIN,   true,  true,  false, true,  true,  true);

	static Style current = ELEGANT;


	static final class IconStyle {
		static final IconStyle MIXED = new IconStyle("Mixed");
		static final IconStyle JFA   = new IconStyle("JFA");
		static final IconStyle WIN   = new IconStyle("Windows");

		private final String styleName;
		private IconStyle(String name) { this.styleName = name; }
		@Override
        public  String toString()		{ return "IconStyle: " + styleName;		}
	}


	private final String    name;
	private final IconStyle	iconStyle;
	private final boolean  	reduceBorders;
	private final boolean	forceBetterColorTheme;
	private final boolean	forceSystemLookAndFeel;
	private final boolean	useExtendedLookAndFeel;
	private final boolean	useEclipseBorder;
	private final boolean	useEclipseGradient;


	Style(
		String		name,
		IconStyle	iconStyle,
		boolean	reduceBorders,
		boolean	forceBetterColorTheme,
		boolean	forceSystemLookAndFeel,
		boolean	useExtendedLookAndFeel,
		boolean	useEclipseBorder,
		boolean	useEclipseGradient) {

		this.name					= name;
		this.iconStyle				= iconStyle;
		this.reduceBorders			= reduceBorders;
		this.forceBetterColorTheme	= forceBetterColorTheme;
		this.forceSystemLookAndFeel	= forceSystemLookAndFeel;
		this.useExtendedLookAndFeel	= useExtendedLookAndFeel;
		this.useEclipseBorder	    = useEclipseBorder;
		this.useEclipseGradient		= useEclipseGradient;

		ALL.add(this);
	}

	public static Style getCurrent()	          { return current; }
	public static void setCurrent(Style newStyle) { current = newStyle; }

	public static Style valueOf(String styleName) {
		String loweredName = styleName.toLowerCase(Locale.ENGLISH);
		for (Style s : ALL) {
			if (s.toString().toLowerCase(Locale.ENGLISH).equals(loweredName))
				return s;
		}
		throw new IllegalArgumentException("Unknown style name: " + styleName);
	}


	public IconStyle iconStyle()			  {	return iconStyle;				}
	public boolean	 reduceBorders()		  {	return reduceBorders;			}
	public boolean	 forceBetterColorTheme()  {	return forceBetterColorTheme;	}
	public boolean	 forceSystemLookAndFeel() {	return forceSystemLookAndFeel;	}
	public boolean	 useExtendedLookAndFeel() {	return useExtendedLookAndFeel;	}
	public boolean	 useEclipseBorder()		  {	return useEclipseBorder;		}
	public boolean	 useEclipseGradient()	  {	return useEclipseGradient;		}

	@Override
    public String 		toString() 			  {	return name; 					}

	public String getDescription() {
		StringBuilder builder = new StringBuilder(200);

		// Comment
		builder.append("THE OVERALL APPEARANCE ");
		if (this == FRANKEN) {
			builder.append("sucks! It demos a lot of common GUI design errors " +
			"that one can see so often in Java clients.");
		} else if (this == ROOKIE) {
			builder.append("avoids the most dramatic graphical bloopers. However, it still sucks.");
		} else if (this == STANDARD) {
			builder.append("has been improved a bit. " +
			"Nevertheless, it still lags behind the look and feel of recent native applications.");
		} else if (this == ADVANCED) {
			builder.append("can compete with native applications. It avoids the most common " +
			"Java gui bloopers and works around some Swing look&feel deficiencies.");
		} else if (this == ELEGANT) {
			builder.append("is elegant; " +
			"it has been copied from the eclipse workbench, see www.eclipse.org.");
		} else {
			builder.append("has been customized by you, see the following details.");
		}

		// Icon style
		builder.append("\n\nTHE ICON SET ");
		if (iconStyle == IconStyle.MIXED) {
			builder.append("is a mixture of several icon styles with different sizes, colors schemes " +
			"and symbol styles. Most icons use oversatured colors.");
		} else if (iconStyle == IconStyle.JFA) {
			builder.append("is consistent. Nevertheless, it suffers from oversaturated colors " +
			"and uncommon symbols.");
		} else {
			builder.append("is consistent and is well suited for most platforms, especially Windows. ");
		}

		// Borders
		builder.append("\n\nBORDERS ");
		if (useEclipseBorder) {
			builder.append("around panels have been replaced by Eclipse-like borders.");
		} else if (!reduceBorders) {
			builder.append("are used heavily and clutter the user interface. " +
			"A source of trouble is the JSplitPane that comes with three borders for both components and the divider.");
		} else {
			builder.append("have been removed from all JSplitPanes.");
		}

		// Color theme
		builder.append("\n\nTHE COLOR SCHEME ");
		if (forceSystemLookAndFeel) {
			builder.append("is based on system colors.");
		} else if (!forceBetterColorTheme) {
			builder.append("conflicts with the Windows default desktop colors.");
		} else {
			builder.append("works well with the majority of default desktop settings, "
						 +"including Windows default colors.");
		}


		// Look&Feel
		builder.append("\n\nTHE LOOK&FEEL ");
		if (!forceSystemLookAndFeel) {
			builder.append("is the Java look&feel ");
		} else {
			builder.append("is the system look&feel ");
		}
		if (!useExtendedLookAndFeel()) {
			builder.append("and so, uses non-standard fonts on most Windows environments." +
			"");
		} else {
			builder.append("as implemented by JGoodies; " +
			"fonts, menus, trees, tables, and text panes have been tweaked " +
			"to more precisely emulate the native widgetry.");
		}


		builder.append("\n\nTHE MENU ITEMS in the File, Edit, and Source menus are ");
		if (!useExtendedLookAndFeel)
			builder.append("not ");
		builder.append("aligned.");

		// Eclipse character
		if (useEclipseGradient) {
			builder.append("\n\nIN ADDITION, the tasks panel uses a gradient.");
		}

		return builder.toString();
	}

}
