/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.metamorphosis;

import java.awt.*;

import javax.swing.*;
import javax.swing.border.Border;

import com.jgoodies.looks.Options;
import com.jgoodies.uif.component.UIFSplitPane;

/**
 * The UIFactory class can create a bunch of panels, useful in Fakeclipse.
 *
 * @author Karsten Lentzsch
 */
final class UIFactory {

    private UIFactory() {
        // Overrides default constructor; prevents instantiation.
    }

    private static class EclipseHeaderPanel extends JPanel {

        EclipseHeaderPanel(LayoutManager lm) {
            super(lm);
        }

        @Override
        public void paint(Graphics g) {
            if (Style.getCurrent().useEclipseGradient()) {
                Color start =
                    UIManager.getColor(
                        Utils.isWindowsPlatform()
                            && Style.getCurrent().forceSystemLookAndFeel()
                                ? "textHighlight"
                                : "ScrollBar.thumb");
                Color end = UIManager.getColor("control");
                Color middle = new Color(167, 202, 239);

                Graphics2D g2 = (Graphics2D) g;
                int width = getWidth();
                int height = getHeight();

                int endX = width;
                int splitX = endX / 2;
                g2.setPaint(
                    new GradientPaint(0, 0, start, splitX, 0, middle));
                g2.fillRect(0, 0, splitX, height);
                g2.setPaint(
                    new GradientPaint(splitX, 0, middle, endX, 0, end));
                g2.fillRect(splitX, 0, endX, height);
            }
            super.paint(g);
        }
    }

    static JPanel createEclipsePanel(
        Icon leftIcon,
        String label,
        Icon rightIcon,
        boolean selected,
        boolean edit) {
        JPanel topLeft = new EclipseHeaderPanel(new BorderLayout());
        JLabel left = new JLabel(label);
        if (selected && Style.getCurrent().useEclipseGradient())
            left.setForeground(
                Utils.isWindowsPlatform()
                    && Style.getCurrent().forceSystemLookAndFeel()
                        ? UIManager.getColor("textHighlightText")
                        : Color.WHITE);
        left.setIcon(leftIcon);
        left.setOpaque(false); //!selected);
        JLabel right = new JLabel(rightIcon);
        right.setOpaque(!selected);
        right.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 4));
        topLeft.add(left, BorderLayout.WEST);
        topLeft.setOpaque(edit || !selected);
        Border pad = BorderFactory.createEmptyBorder(3, 3, 3, 3);
        if (edit) {
            topLeft.setBorder(
                BorderFactory.createCompoundBorder(
                    new Borders.EclipseTabBorder(),
                    pad));
            topLeft.setBackground(UIManager.getColor("controlLtHighlight"));
        } else {
            topLeft.setBorder(pad);
        }
        JPanel top = new JPanel(new BorderLayout());
        top.add(topLeft, edit ? BorderLayout.WEST : BorderLayout.CENTER);
        top.add(right, BorderLayout.EAST);
        if (edit) {
            if (Style.getCurrent().useEclipseBorder()) {
                top.setBorder(new Borders.EclipseTabHeaderBorder());
            } else {
                top.setBorder(new Borders.FlatHeaderBorder());
            }
        } else if (Style.getCurrent().useEclipseBorder()) {
            top.setBorder(new Borders.EclipseHeaderBorder());
        } else {
            top.setBorder(new Borders.FlatHeaderBorder());
        }
        top.setOpaque(!selected);

        JPanel panel = new JPanel(new BorderLayout());
        panel.add(top, BorderLayout.NORTH);
        if (Style.getCurrent().useEclipseBorder()) {
            panel.setBorder(new Borders.EclipsePanelBorder());
        }
        return panel;
    }

    static JPanel createEclipsePanel(
        Icon leftIcon,
        String label,
        Icon rightIcon,
        boolean selected) {
        return createEclipsePanel(
            leftIcon,
            label,
            rightIcon,
            selected,
            false);
    }

    static JPanel createEditorPanel(Icon leftIcon, String label) {
        return createEclipsePanel(leftIcon, label, null, false, true);
    }

    static JSplitPane createSplitPane(
        int orientation,
        Component comp1,
        Component comp2,
        double resizeWeight) {
            JSplitPane split =
                Style.getCurrent().reduceBorders()
        // && !Style.getCurrent().forceSystemLookAndFeel()
    ? new UIFSplitPane(orientation, comp1, comp2)
    : new JSplitPane(orientation, comp1, comp2);
        split.setResizeWeight(resizeWeight);
        if (Style.getCurrent().reduceBorders()) {
            split.setBorder(BorderFactory.createEmptyBorder());
        }
        return split;
    }

    static JScrollPane createScrollPane(Component component) {
        JScrollPane scrollPane = new JScrollPane(component);
        scrollPane.putClientProperty(Options.IS_ETCHED_KEY, Boolean.TRUE);
        scrollPane.updateUI();
        if (Style.getCurrent().useEclipseBorder()) {
            scrollPane.setBorder(BorderFactory.createEmptyBorder());
        }
        return scrollPane;
    }

}