/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton;

import java.awt.Dimension;

import javax.swing.UIManager;

import com.jgoodies.forms.layout.LayoutMap;
import com.jgoodies.forms.layout.Sizes;
import com.jgoodies.looks.LookUtils;
import com.jgoodies.looks.Options;
import com.jgoodies.skeleton.model.MainModel;
import com.jgoodies.skeleton.view.MainFrame;
import com.jgoodies.uif_lite.application.ResourceMap;

/**
 * The main class of the Skeleton application.
 * Initializes singletons, configures the UI,
 * and builds and opens the main frame.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.25 $
 *
 * @see MainModel
 * @see MainFrame
 */
public final class Skeleton {

    private static final ResourceMap RESOURCES =
        new ResourceMap(null, Skeleton.class);


    public static void main(String[] arguments) {
        new Skeleton().boot();
    }


    /**
     * Boots the application; catches fatal errors and tries
     * to perform a clean up and to free resources.
     */
    private void boot() {
        try {
            load();
        } catch (Throwable t) {
            System.err.println("Load failed:" + t);
            t.printStackTrace();
            System.exit(0); // exit the application
        } finally {
            // Clean up and free resources;
        }
    }

    /**
     * Loads the application: prepares the <code>ActionManager</code>
     * and <code>ResourceUtils</code>, then configures the UI,
     * registers <code>Action</code>s, builds the main frame,
     * registers listeners, and finally makes the main frame visible.<p>
     *
     * The JGoodies Swing Suite provides a more fine grained application
     * startup process with many hooks that one can override.
     */
    private void load() {
        configureUI();

        // Create the model that provides all high-level models.
        MainModel mainModel = new MainModel();

        // Create and build the main frame.
        MainFrame mainFrame = new MainFrame(
                RESOURCES.getString("application.title"), mainModel);
        mainFrame.build();

        // Set the main frame as default parent for dialogs.
        mainModel.setDefaultParentFrame(mainFrame);

        // Show the main frame.
        mainFrame.setVisible(true);
    }

    /**
     * Configures the UI: sets the class loader, configures fonts,
     * chooses a look&amp;feel and sets it in the <code>UIManager</code>.
     * Chooses the JGoodies Windows L&amp;f on Windows and the
     * <code>PlasticXPLookAndFeel</code> on other OS.<p>
     *
     * The JGoodies Swing Suite's <code>ApplicationStarter</code>,
     * <code>ExtUIManager</code>, and <code>LookChoiceStrategies</code>
     * classes provide a much more fine grained algorithm to choose and
     * restore a look and theme.<p>
     *
     * Since 1.1 the JGoodies L&amp;Fs do not set the Swing <em>ClassLoader</em>
     * property automatically. Therefore we set the Swing class loader
     * to the one used for the <code>LookUtils</code> class.
     */
    private void configureUI() {
        UIManager.put("ClassLoader", LookUtils.class.getClassLoader());
        Options.setDefaultIconSize(new Dimension(18, 18));
        try {
            UIManager.setLookAndFeel(Options.getSystemLookAndFeelClassName());
        } catch (Exception e) {
            System.err.println("Can't set look & feel:" + e);
        }
        // Use DLU as default unit.
        Sizes.setDefaultUnit(null);
        configureLayoutMap();
    }


    protected void configureLayoutMap() {
        LayoutMap map = LayoutMap.getRoot();
        map.columnPut("indent",     "14");
        map.columnPut("label",      "right:[45, pref]");
        map.columnPut("component",  "80");
        map.columnPut("column-gap", "14");
        map.columnPut("colgap",     "${column-gap}");
    }



}