/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.domain;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;


/**
 * References all relevant project data: the project description and a
 * list of shafts. The shafts refer to their child components: segments
 * and flanges
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.7 $
 */

public class Project extends Model {

    /**
     * Refers to <code>Description</code> object that describes this project.
     */
    private final Description description;

    /**
     * Holds a list of shafts.
     */
    private final List shafts;

    /**
     * The file used to save this project to.
     */
    private File file;


    // Instance Creation ******************************************************

    /**
     * Constructs a <code>Project</code> with the given name and
     * and empty shaft list.
     *
     * @param name  the project's name
     */
    public Project(String name) {
        this.description = new Description(name);
        this.shafts      = new ArrayList();
    }


    // The public API *********************************************************

    public String getName() {
        return getDescription().getName();
    }

    public Description getDescription() {
        return description;
    }


    // Managing Shafts ********************************************************

    /**
     * Returns an unmodifiable list of this project's shafts.
     *
     * @return an unmodifiable list of this project's shafts.
     */
    public List getShafts() {
        return Collections.unmodifiableList(shafts);
    }


    /**
     * Adds the given shaft to this project's shafts.
     *
     * @param shaft   the shaft to add
     */
    public void addShaft(Shaft shaft) {
        shafts.add(shaft);
    }


    /**
     * Removes the given shaft from this project's shafts.
     *
     * @param shaft   the shaft to remove
     */
    public void removeShaft(Shaft shaft) {
        shafts.remove(shaft);
    }


    // Reading and Saving *****************************************************


    /**
     * Reads, creates and returns a Project from the given <code>File</code>.<p>
     * A real app would parse the file.
     *
     * @param f   the file to read the project from
     * @return the Project read from the given file
     */
    public static Project readFrom(File f) {
        Project model = ProjectFactory.createSample();
        model.setFile(f);
        return model;
    }


    /**
     * Saves this project to its associated <code>File</code>.
     */
    public void save() {
        if (!hasFile()) {
            throw new IllegalStateException("Can't save project without file.");
        }
        saveAs(file);
    }


    /**
     * Saves this project to the given <code>File</code>.
     *
     * @param aFile   the file to save to
     */
    public void saveAs(File aFile) {
        setFile(aFile);
    }


    /**
     * Checks and answers whether this project has an associated file
     * as backing store.
     *
     * @return true if this project has an associated file
     */
    public boolean hasFile() {
        return file != null;
    }


    /**
     * Returns the name of this project's associated filename,
     * or &quote;Unsaved&quote; if it has no file associated.
     *
     * @return this project's filename or a default if it has no file
     */
    public String getFileName() {
        return hasFile() ? file.getName() : "Unsaved";
    }


    /**
     * Sets a file as this project' backing store.
     *
     * @param aFile  the new associated file
     */
    private void setFile(File aFile) {
        this.file = aFile;
    }


    // Misc *******************************************************************

    /**
     * Returns a string representation for this project.
     * Currently it prints the class info and file name.
     *
     * @return a string representation for this project
     */
    @Override
    public String toString() {
        return super.toString() + ':' + getFileName();
    }


}