/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.domain;


import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Describes shafts and provides bound bean properties for the
 * shaft and associated comments. Also holds a list of children:
 * segments and flanges.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.7 $
 *
 * @see Segment
 * @see Flange
 */

public class Shaft extends Model {

    // Names of the Bound Bean Properties *************************************

    public static final String PROPERTYNAME_IDENTIFIER = "identifier";
    public static final String PROPERTYNAME_POWER = "power";
    public static final String PROPERTYNAME_SPEED = "speed";
    public static final String PROPERTYNAME_MATERIAL = "material";
    public static final String PROPERTYNAME_ICE_CLASS = "iceClass";
    public static final String PROPERTYNAME_LENGTH = "length";
    public static final String PROPERTYNAME_MACHINERY = "machinery";
    public static final String PROPERTYNAME_INSPECTION = "inspection";


    // Instance Fields ********************************************************

    /**
     * Holds a list of segments and flanges.
     */
    private final List components;

    private String identifier;
    private String power;
    private String speed;
    private String material;
    private String iceClass;
    private long   length;
    private String machinery;
    private String inspection;


    // Instance Creation ******************************************************

    /**
     * Constructs a <code>Shaft</code> with the given identifier.
     * The list of child components is empty.
     *
     * @param identifier    the initial name
     */
    public Shaft(String identifier) {
        this.identifier = identifier;
        this.components = new ArrayList();
    }


    // Accessing and Managing Components **************************************

    /**
     * Returns an unmodifiable List of this shaft's child components:
     * segments and flanges.
     *
     * @return An unmodifiable List of this shaft's child components.
     */
    public List getComponents() {
        return Collections.unmodifiableList(components);
    }


    /**
     * Adds the given segment to this shaft's components.
     *
     * @param segment  the segment to add
     */
    public void addComponent(Segment segment) {
        components.add(segment);
    }


    /**
     * Adds the given flange to this shaft's components.
     *
     * @param flange  the flange to add
     */
    public void addComponent(Flange flange) {
        components.add(flange);
    }


    /**
     * Removes the given segment from this shaft's components.
     *
     * @param segment  the segment to remove
     */
    public void removeComponent(Segment segment) {
        components.remove(segment);
    }


    /**
     * Removes the given flange from this shaft's components.
     *
     * @param flange    the flange to remove
     */
    public void removeComponent(Flange flange) {
        components.remove(flange);
    }


    // Property Accessors *****************************************************

    /**
     * @return the ice class.
     */
    public String getIceClass() {
        return iceClass;
    }


    /**
     * Returns this shaft's identifier.
     *
     * @return the identifier.
     */
    public String getIdentifier() {
        return identifier;
    }


    /**
     * @return the inspection.
     */
    public String getInspection() {
        return inspection;
    }


    /**
     * @return the length.
     */
    public long getLength() {
        return length;
    }


    /**
     * @return the machinery.
     */
    public String getMachinery() {
        return machinery;
    }


    /**
     * @return the material.
     */
    public String getMaterial() {
        return material;
    }


    /**
     * @return the power.
     */
    public String getPower() {
        return power;
    }


    /**
     * @return the speed.
     */
    public String getSpeed() {
        return speed;
    }


    /**
     * @param newIceClass The ice class to set.
     */
    public void setIceClass(String newIceClass) {
        String oldIceClass = getIceClass();
        this.iceClass = newIceClass;
        firePropertyChange(PROPERTYNAME_ICE_CLASS, oldIceClass, newIceClass);
    }


    /**
     * @param newIdentifier the identifier to set
     */
    public void setIdentifier(String newIdentifier) {
        String oldIdentifier = getIdentifier();
        identifier = newIdentifier;
        firePropertyChange(PROPERTYNAME_IDENTIFIER, oldIdentifier, newIdentifier);
    }


    /**
     * @param newInspection The inspection to set.
     */
    public void setInspection(String newInspection) {
        String oldInspection = getInspection();
        this.inspection = newInspection;
        firePropertyChange(PROPERTYNAME_INSPECTION, oldInspection, newInspection);
    }


    /**
     * @param newLength The length to set.
     */
    public void setLength(long newLength) {
        long oldLength = getLength();
        this.length = newLength;
        firePropertyChange(PROPERTYNAME_LENGTH, oldLength, newLength);
    }


    /**
     * @param newMachinery The machinery to set.
     */
    public void setMachinery(String newMachinery) {
        String oldMachinery = getMachinery();
        this.machinery = newMachinery;
        firePropertyChange(PROPERTYNAME_MACHINERY, oldMachinery, newMachinery);
    }


    /**
     * @param newMaterial The material to set.
     */
    public void setMaterial(String newMaterial) {
        String oldMaterial = getMaterial();
        this.material = newMaterial;
        firePropertyChange(PROPERTYNAME_MATERIAL, oldMaterial, newMaterial);
    }


    /**
     * @param newPower The power to set.
     */
    public void setPower(String newPower) {
        String oldPower = getPower();
        this.power = newPower;
        firePropertyChange(PROPERTYNAME_POWER, oldPower, newPower);
    }


    /**
     * @param newSpeed The speed to set.
     */
    public void setSpeed(String newSpeed) {
        String oldSpeed = getSpeed();
        this.speed = newSpeed;
        firePropertyChange(PROPERTYNAME_SPEED, oldSpeed, newSpeed);
    }


    // Misc *******************************************************************

    /**
     * Returns a string representation for this shaft.
     * Currently it prints the class info and identifier.
     *
     * @return a string representation for this shaft
     */
    @Override
    public String toString() {
        return super.toString() + ':' + getIdentifier();
    }


}