/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.model;

import java.awt.Component;
import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.io.File;
import java.util.ResourceBundle;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JOptionPane;

import com.jgoodies.skeleton.domain.Model;
import com.jgoodies.skeleton.domain.Project;
import com.jgoodies.skeleton.domain.ProjectFactory;
import com.jgoodies.uif_lite.application.ActionManager;

/**
 * References all application-level data or submodels and provides
 * convenience accessors.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.18 $
 */
public final class MainModel extends Model {

    // Names of the Bound Bean Properties *************************************

    public static final String PROPERTYNAME_PROJECT = "project";


    // References to Submodels ************************************************

    /**
     * Refers to the navigation submodel that provides
     * bound properties, operations and Actions for the
     * navigation tree.
     *
     * @see #getNavigationModel()
     */
    private final NavigationModel navigationModel;

    /**
     * Refers to the editor submodel that provides
     * bound properties for the edited and validated object.
     */
    private final EditorModel editorModel;


    // Model State Fields *****************************************************

    /**
     * Holds the current project.
     *
     * @see #getProject()
     * @see #setProject(Project)
     */
    private Project project;

    /**
     * Holds a frame that is used as default parent frame for message dialogs.
     *
     * @see #setDefaultParentFrame(Frame)
     */
    private Frame defaultParentFrame;


    // Instance Creation ******************************************************

    /**
     * Constructs a <code>MainModel</code> that has no project set,
     * no selection and no tree model.
     */
    public MainModel() {
        navigationModel = new NavigationModel(this);
        editorModel = new EditorModel(this);
        registerActions();
        updateProjectActionEnablement(null);
    }


    // Accessing Submodels ****************************************************

    /**
     * Returns the navigation submodel.
     *
     * @return the navigation submodel.
     */
    public NavigationModel getNavigationModel() {
        return navigationModel;
    }


    /**
     * Returns the editor submodel.
     *
     * @return the editor submodel.
     */
    public EditorModel getEditorModel() {
        return editorModel;
    }


    // Managing Projects ******************************************************

    /**
     * Returns the current project.
     *
     * @return the current project.
     */
    public Project getProject() {
        return project;
    }


    /**
     * Sets a new project.
     *
     * @param newProject   the project to set
     *
     * @throws NullPointerException if the new project is null
     */
    public void setProject(Project newProject) {
        if (newProject == null)
            throw new NullPointerException("The project must not be null.");

        Project oldProject = getProject();
        project = newProject;
        firePropertyChange(PROPERTYNAME_PROJECT, oldProject, newProject);
        getNavigationModel().updateNavigationTreeModel();
        updateProjectActionEnablement(newProject);
    }


    /**
     * Checks and answers if a project is loaded.
     *
     * @return true if a project is loaded.
     */
    public boolean hasProject() {
        return getProject() != null;
    }


    /**
     * Checks and answers whether the project's file path is valid.
     *
     * @return true if the project's file path is valid.
     */
    public boolean isProjectFilePathValid() {
        return false;
    }


    // Application Shutdown *************************************************

    /**
     * Leaves the application if nobody vetos against the close request.
     */
    public void aboutToExitApplication(Object eventSource) {
        if (requestForWindowClose()) {
            exitApplication();
        }
    }

    /**
     * Checks and answers whether we accept a close request.
     * The default behavior just says: true = OK to close.
     * You may ask: "Do you really want to close the application?".<p>
     *
     * TODO: Warn if the edited object is invalid and offer
     * to cancel the exit operation.
     *
     * @return true to accept the window close, false to reject it
     */
    private boolean requestForWindowClose() {
        return true;
    }

    /**
     * Shuts down the application, in this case just exit the System.
     * Larger application may need to perform additional operations
     * before exiting the System, for example store state, save preferences, etc.
     */
    private void exitApplication() {
        System.exit(0);
    }


    // Window Management ******************************************************

    void showMessage(String what) {
        JOptionPane.showMessageDialog(getDefaultParentFrame(), what);
    }


    /**
     * Opens a message dialog that shows the validation result,
     * things the user must fix, before the selection can change.
     *
     * @param validationResult   the validation result to be displayed
     * @param error              true for an error message, false for a warning
     */
    void showValidationMessage(String validationResult, boolean error) {
        Component parent   = getDefaultParentFrame();
        String messageText = validationResult + "\n\n";
        String titleText   = "Validation " + (error ? "Error" : "Warning");
        int messageType    = error ? JOptionPane.ERROR_MESSAGE : JOptionPane.WARNING_MESSAGE;
        JOptionPane.showMessageDialog(parent, messageText, titleText, messageType);
    }


    /**
     * Saves the given object to a database or whatever.
     */
    void save(Object model) {
        // save the given model.
        // System.out.println("Saving " + model);
    }


    private Frame getDefaultParentFrame() {
        return defaultParentFrame;
    }

    /**
     * Sets a frame as the default parent for this controller's dialogs.
     *
     * @param defaultParentFrame    the default parent
     */
    public void setDefaultParentFrame(Frame defaultParentFrame) {
        this.defaultParentFrame = defaultParentFrame;
    }


    // Actions **************************************************************

    public Action getNewProjectAction() {
        return ActionManager.get(NEW_PROJECT_ID);
    }


    public Action getOpenProjectAction() {
        return ActionManager.get(OPEN_PROJECT_ID);
    }


    public Action getSaveAction() {
        return ActionManager.get(SAVE_ID);
    }


    public Action getSaveAsAction() {
        return ActionManager.get(SAVE_AS_ID);
    }


    public Action getPrintAction() {
        return ActionManager.get(PRINT_ID);
    }


    public Action getOpenPrintDialogAction() {
        return ActionManager.get(OPEN_PRINT_DIALOG_ID);
    }


    public Action getExitAction() {
        return ActionManager.get(EXIT_ID);
    }


    public Action getOpenAboutDialogAction() {
        return ActionManager.get(OPEN_ABOUT_DIALOG_ID);
    }


    // Action Behavior ******************************************************

    /**
     * Creates a new project and sets it in the main model.
     * Uses a sample project for demoing purposes.
     */
    private void doNewProject(ActionEvent e) {
        setProject(ProjectFactory.createSample());
    }


    /**
     * Opens a file chooser, loads a project from a file,
     * and sets it in the main model.
     * Uses a sample project in this demo.
     */
    private void doOpenProject(ActionEvent e) {
        FileDialog dialog = new FileDialog(getDefaultParentFrame(), "Open project file");
        dialog.setVisible(true);
        if (dialog.getFile() == null)
            return;

        System.out.println(
            "You've chosen to open this file: " + dialog.getFile());
        setProject(Project.readFrom(new File(dialog.getFile())));
    }


    /**
     * Saves the project to a file.
     */
    private void doSave(ActionEvent e) {
        showMessage("Save performed.");
    }


    private void doSaveAs(ActionEvent e) {
        showMessage("Save as performed.");
    }


    private void doPrint(ActionEvent e) {
        showMessage("Print performed.");
    }


    private void doOpenPrintDialog(ActionEvent e) {
        showMessage("Open print dialog performed.");
    }


    private void doExit(ActionEvent e) {
        aboutToExitApplication(e.getSource());
    }


    private void doOpenAboutDialog(ActionEvent e) {
        ResourceBundle bundle = ResourceBundle.getBundle(
            "com/jgoodies/skeleton/resources/Skeleton");

        showMessage(
            "The JGoodies Swing Skeleton Application"
            + "\n" + bundle.getString("application.copyright") + ". All Rights Reserved."
            + "\n\nVersion " + bundle.getString("application.fullVersion"));
    }


    // Action IDs *************************************************************

    private static final String NEW_PROJECT_ID       = "newProject";
    private static final String OPEN_PROJECT_ID      = "openProject";
    private static final String SAVE_ID              = "save";
    private static final String SAVE_AS_ID           = "saveAs";
    private static final String PRINT_ID             = "print";
    private static final String OPEN_PRINT_DIALOG_ID = "openPrintDialog";
    private static final String EXIT_ID              = "exit";
    private static final String OPEN_ABOUT_DIALOG_ID = "openAboutDialog";

    /**
     * Register actions in the <code>ActionManager</code>
     * using instances of a dispatching <code>Action</code> implementation.
     */
    private void registerActions() {
        register(NEW_PROJECT_ID);
        register(OPEN_PROJECT_ID);
        register(SAVE_ID, false);
        register(SAVE_AS_ID, false);
        register(PRINT_ID, false);
        register(OPEN_PRINT_DIALOG_ID, false);
        register(EXIT_ID);
        register(OPEN_ABOUT_DIALOG_ID);
    }

    // Helper Code **********************************************************

    /**
     * Registers a <code>DispatchingAction</code> with the
     * <code>ActionManager</code> using the given id.
     */
    private void register(String id) {
        register(id, true);
    }

    /**
     * Registers a <code>DispatchingAction</code> with the
     * <code>ActionManager</code> using the given id and enablement.
     */
    private void register(String id, boolean enabled) {
        Action action = new DispatchingAction(id);
        ActionManager.register(id, action);
        action.setEnabled(enabled);
    }


    // Updating the Action Enablement *****************************************

    /**
     * Updates the enablement of actions that are related to the project state.
     *
     * @param aProject    the current project
     */
    private void updateProjectActionEnablement(Project aProject) {
        boolean enabled = aProject != null;
        ActionManager.get(SAVE_ID).setEnabled(enabled);
        ActionManager.get(SAVE_AS_ID).setEnabled(enabled);
        ActionManager.get(PRINT_ID).setEnabled(enabled);
        ActionManager.get(OPEN_PRINT_DIALOG_ID).setEnabled(enabled);
    }


    /**
     * An implementation of the Action interface that dispatches using an ID.
     */
    private final class DispatchingAction extends AbstractAction {

        private final String id;

        /**
         * Constructs a <code>DispatchingAction</code>.
         */
        private DispatchingAction(String id) { this.id = id; }

        /**
         * Dispatches using the stored id, then delegates to the
         * appropriate method of the application controller,
         * that finally does the real work.
         */
        public void actionPerformed(ActionEvent event) {
            if (id.equals(NEW_PROJECT_ID)) {
                doNewProject(event);
            } else if (id.equals(OPEN_PROJECT_ID)) {
                doOpenProject(event);
            } else if (id.equals(SAVE_ID)) {
                doSave(event);
            } else if (id.equals(SAVE_AS_ID)) {
                doSaveAs(event);
            } else if (id.equals(PRINT_ID)) {
                doPrint(event);
            } else if (id.equals(OPEN_PRINT_DIALOG_ID)) {
                doOpenPrintDialog(event);
            } else if (id.equals(EXIT_ID)) {
                doExit(event);
            } else if (id.equals(OPEN_ABOUT_DIALOG_ID)) {
                doOpenAboutDialog(event);
            } else {
                System.err.println("Unknown action id: " + id);
            }
        }

    }





}