/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.model;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.*;

import com.jgoodies.skeleton.domain.Description;
import com.jgoodies.skeleton.domain.Model;
import com.jgoodies.skeleton.domain.Project;
import com.jgoodies.skeleton.domain.Shaft;
import com.jgoodies.skeleton.model.node.NavigationNode;
import com.jgoodies.skeleton.model.node.RootNode;
import com.jgoodies.uif_lite.application.ActionManager;

/**
 * A presentation model that provides bound properties, operations
 * and Actions for the navigation tree.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.14 $
 */
public final class NavigationModel extends Model {

    // Names of the Bound Bean Properties *************************************

    public static final String PROPERTYNAME_SELECTION = "selection";
    public static final String PROPERTYNAME_SELECTION_TYPE = "selectionType";
    public static final String PROPERTYNAME_NAVIGATION_TREE_MODEL = "navigationTreeModel";


    // Instance Fields ********************************************************

    /**
     * Refers to the parent main model.
     * Used to lookup the default parent frame
     */
    private final MainModel mainModel;


    /**
     * Holds the current selection, an instance of Model.
     *
     * @see #getSelection()
     * @see #getSelectionType()
     */
    private Object selection;

    /**
     * Holds the class of the current selection.
     *
     * @see #getSelection()
     * @see #getSelectionType()
     */
    private Class selectionType;


    /**
     * Holds the node that refers to the current selection.
     * Used to report a potential change to its hosting tree model
     * when the selection changes. This is because the node's label
     * may have changed as part of the editor change.<p>
     *
     * This is a poor workaround that is based on the weak assumumption
     * that this class's tree selection listener is the first to know
     * about the node change.<p>
     *
     * TODO: Let all nodes observe the relevant domain data that
     * is rendered in the tree view to report a node change.
     */
    private NavigationNode selectedNode;


    /**
     * Holds the model for the navigation tree.
     * It changes everytime this model's project changes.
     *
     * @see #getNavigationTreeModel()
     * @see #getNavigationTreeSelectionModel()
     */
    private TreeModel navigationTreeModel;


    /**
     * Refers to the single selection model for the navigation tree.
     * The selection is reset if the navigation tree model changes.
     *
     * @see #getNavigationTreeSelectionModel()
     * @see #getNavigationTreeModel()
     */
    private final TreeSelectionModel navigationTreeSelectionModel;


    // Instance Creation ******************************************************

    /**
     * Constructs a NavigationModel that has no selection and no tree model.
     *
     * @param mainModel  the parent model, the MainModel
     */
    public NavigationModel(MainModel mainModel) {
        this.mainModel = mainModel;
        setNavigationTreeModel(createNavigationTreeModel(new Project("Empty")));
        navigationTreeSelectionModel = new DefaultTreeSelectionModel();
        navigationTreeSelectionModel.setSelectionMode(
                TreeSelectionModel.SINGLE_TREE_SELECTION);
        registerActions();
        initEventHandling();
    }


    private void initEventHandling() {
        navigationTreeSelectionModel.addTreeSelectionListener(
                new NavigationTreeSelectionChangeHandler());
        updateSelectionActionEnablement(null);
    }


    // Accessing the Selection and Selection Type *****************************

    /**
     * Returns the current selection, a domain object. This selection
     * will be updated if the selection in the navigation tree changes.
     * During the transition from one tree selection to another this
     * value may hold the old or new selection.
     *
     * @return the domain object selected in the navigation tree.
     *
     * @see #getSelectionType()
     */
    public Object getSelection() {
        return selection;
    }


    /**
     * Sets a new domain object as selection. This method is invoked
     * by the NavigationTreeSelectionChangeHandler that observes changes in
     * the selection of the navigation tree.
     *
     * @param newSelection   the selection to set
     *
     * @see #getSelection()
     */
    private void setSelection(Object newSelection) {
        Object oldSelection = getSelection();
        selection = newSelection;
        setSelectionType(newSelection.getClass());
        firePropertyChange(PROPERTYNAME_SELECTION, oldSelection, newSelection);
        mainModel.getEditorModel().selectionChanged(oldSelection, newSelection);
    }


    /**
     * Returns this model's selected node.
     *
     * @return this model's selected node.
     */
    private NavigationNode getSelectedNode() {
        return selectedNode;
    }


    /**
     * Sets the selection and reports the previously selected node as changed.<p>
     *
     * <strong>Note:</strong> This code requires a special order. It assumes
     * that domain changes happen when the selection changes.<p>
     *
     * TODO: Remove this code and let the nodes listen to domain properties.
     *
     * @param newSelectedNode   the new node selection
     */
    private void setSelectedNode(NavigationNode newSelectedNode) {
        NavigationNode oldSelectedNode = selectedNode;
        selectedNode = newSelectedNode;
        setSelection(newSelectedNode.getModel());
        ((DefaultTreeModel) getNavigationTreeModel()).nodeChanged(oldSelectedNode);
    }


    /**
     * Returns the class of the selected domain object.
     *
     * @return the class of the selected domain object.
     *
     * @see #getSelection()
     */
    public Class getSelectionType() {
        return selectionType;
    }


    /**
     * Sets a new selection type. This method is invoked in
     * <code>#setSelection</code> with the class of the new selection.
     *
     * @param newSelectionType   the selection type to set
     *
     * @see #getSelectionType()
     */
    private void setSelectionType(Class newSelectionType) {
        Class oldSelectionType = getSelectionType();
        selectionType = newSelectionType;
        firePropertyChange(PROPERTYNAME_SELECTION_TYPE, oldSelectionType,
                                                        newSelectionType);
    }


    // Managing the Navigation Tree Model *************************************

    /**
     * Returns the tree model for the navigation tree.
     *
     * @return the tree model for the navigation tree.
     */
    public TreeModel getNavigationTreeModel() {
        return navigationTreeModel;
    }


    /**
     * Sets a new tree model for the navigation tree.<p>
     *
     * TODO: The model shall set the tree selection to the first top-level
     * node, the project description. Currently this is done in a view,
     * see <code>NavigationView#updateModel(TreeModel)</code>.
     *
     * @param newModel  the new tree model to set
     */
    private void setNavigationTreeModel(TreeModel newModel) {
        TreeModel oldModel = getNavigationTreeModel();
        navigationTreeModel = newModel;
        firePropertyChange(PROPERTYNAME_NAVIGATION_TREE_MODEL, oldModel, newModel);
    }


    /**
     * Returns the fixed selection model for the navigation tree.
     *
     * @return the fixed selection model for the navigation tree.
     */
    public TreeSelectionModel getNavigationTreeSelectionModel() {
        return navigationTreeSelectionModel;
    }


    /**
     * Creates a new navigation tree model and sets it as
     * this class' tree model. Invoked by the parent MainModel
     * if the project has changed.
     */
    void updateNavigationTreeModel() {
        setNavigationTreeModel(createNavigationTreeModel(mainModel.getProject()));
    }


    /**
     * Creates and returns a tree model for the given project.
     * Constructs a <code>RootNode</code> with the project and
     * returns a <code>DefaultTreeModel</code> with this root.
     *
     * @param aProject   the project to create the tree for
     * @return a TreeModel for the given project
     */
    private TreeModel createNavigationTreeModel(Project aProject) {
        return new DefaultTreeModel(new RootNode(aProject));
    }


    // Exposing Actions *******************************************************

    public Action getAddFlangeAction() {
        return ActionManager.get(ADD_FLANGE_ID);
    }

    public Action getAddSegmentAction() {
        return ActionManager.get(ADD_SEGMENT_ID);
    }

    public Action getAddShaftAction() {
        return ActionManager.get(ADD_SHAFT_ID);
    }

    public Action getDeleteItemAction() {
        return ActionManager.get(DELETE_ITEM_ID);
    }


    // Action Behavior ********************************************************

    private void doAddShaft(ActionEvent e) {
        // mainModel().addShaft();
        showMessage("Add shaft performed.");
    }

    private void doAddFlange(ActionEvent e) {
        // mainModel().addFlangeNode();
        showMessage("Add flange performed.");
    }

    private void doAddSegment(ActionEvent e) {
        // mainModel().addSegmentNode();
        showMessage("Add segment performed.");
    }

    private void doDeleteItem(ActionEvent e) {
        // mainModel().removeSelectedItem();
        showMessage("Item removed.");
    }

    private void showMessage(String what) {
        mainModel.showMessage(what);
    }


    // Event Handling *********************************************************

    /**
     * Listens to changes in the tree node selection and updates the selection.
     */
    private final class NavigationTreeSelectionChangeHandler implements TreeSelectionListener {

        /**
         * The selected node in the navidation tree has changed.
         * Updates this model's selected domain object and in turn
         * the selection type.<p>
         *
         * Does nothing if the new tree selection is {@code null}.
         *
         * @param evt    the event that describes the change
         */
        public void valueChanged(TreeSelectionEvent evt) {
            TreePath path = navigationTreeSelectionModel.getSelectionPath();
            NavigationNode aSelectedNode = path == null
                ? null
                : (NavigationNode) path.getLastPathComponent();
            updateSelectionActionEnablement(aSelectedNode);
            if (NavigationModel.this.equals(getSelectedNode(), aSelectedNode)) {
                return;
            }
            acceptOrRejectChange(path, aSelectedNode);
        }


        private void acceptOrRejectChange(TreePath path, NavigationNode aSelectedNode) {
            if (aSelectedNode == null)
                return;

            String validationResult = validate(getSelection());
            if (validationResult == null) {
                setSelectedNode(aSelectedNode);
            } else {
                // Revert to the old selection path
                navigationTreeSelectionModel.removeSelectionPath(path);
                setSelectionPathFor(getSelectedNode());
                // Update the tree node
                ((DefaultTreeModel) getNavigationTreeModel()).nodeChanged(getSelectedNode());
                // Show a warning
                mainModel.showValidationMessage(validationResult, false);
            }
        }


        private String validate(Object model) {
            return mainModel.getEditorModel().validateBeforeSelectionChange(model);
        }


        private void setSelectionPathFor(TreeNode node) {
            Object[] pathToRoot =
                ((DefaultTreeModel) navigationTreeModel).getPathToRoot(getSelectedNode());
            navigationTreeSelectionModel.addSelectionPath(new TreePath(pathToRoot));
        }


    }


    // Updating the Action Enablement *****************************************

    /**
     * Updates the enablement state actions realted to the node selection.
     *
     * @param aSelectedNode   the selected node
     */
    private void updateSelectionActionEnablement(NavigationNode aSelectedNode) {
        Object aSelection = aSelectedNode == null
            ? null
            : aSelectedNode.getModel();
        boolean canAddSubsystem = aSelection != null;
        boolean canAddComponent = aSelection instanceof Shaft;
        boolean canDelete       = aSelection != null
                               && !(aSelection instanceof Description);

        ActionManager.get(ADD_SHAFT_ID).setEnabled(canAddSubsystem);
        ActionManager.get(ADD_FLANGE_ID).setEnabled(canAddComponent);
        ActionManager.get(ADD_SEGMENT_ID).setEnabled(canAddComponent);
        ActionManager.get(DELETE_ITEM_ID).setEnabled(canDelete);
    }


    // Action Implementation **************************************************

    private static final String ADD_SHAFT_ID   = "addShaft";
    private static final String ADD_FLANGE_ID  = "addFlange";
    private static final String ADD_SEGMENT_ID = "addSegment";
    private static final String DELETE_ITEM_ID = "deleteItem";

    /**
     * Register actions in the <code>ActionManager</code>
     * using instances of a dispatching <code>Action</code> implementation.
     */
    private void registerActions() {
        register(ADD_SHAFT_ID);
        register(ADD_FLANGE_ID);
        register(ADD_SEGMENT_ID);
        register(DELETE_ITEM_ID);
    }

    /**
     * Registers a <code>DispatchingAction</code> with the
     * <code>ActionManager</code> using the given id.
     */
    private void register(String id) {
        register(id, true);
    }

    /**
     * Registers a <code>DispatchingAction</code> with the
     * <code>ActionManager</code> using the given id and enablement.
     */
    private void register(String id, boolean enabled) {
        Action action = new DispatchingAction(id);
        ActionManager.register(id, action);
        action.setEnabled(enabled);
    }


    /**
     * An implementation of the Action interface that dispatches using an ID.
     */
    private final class DispatchingAction extends AbstractAction {

        private final String id;

        /**
         * Constructs a <code>DispatchingAction</code>.
         */
        private DispatchingAction(String id) { this.id = id; }

        /**
         * Dispatches using the stored id, then delegates to the
         * appropriate method of the application controller,
         * that finally does the real work.
         */
        public void actionPerformed(ActionEvent event) {
            if (id.equals(ADD_SHAFT_ID)) {
                doAddShaft(event);
            } else if (id.equals(ADD_FLANGE_ID)) {
                doAddFlange(event);
            } else if (id.equals(ADD_SEGMENT_ID)) {
                doAddSegment(event);
            } else if (id.equals(DELETE_ITEM_ID)) {
                doDeleteItem(event);
            } else {
                System.err.println("Unknown action id: " + id);
            }
        }

    }



}