/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  o Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  o Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  o Neither the name of JGoodies Karsten Lentzsch nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.jgoodies.skeleton.model.node;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;

import javax.swing.Icon;
import javax.swing.UIManager;
import javax.swing.tree.TreeNode;

/**
 * The abstract superclass for all tree node classes in Skeleton (Pro).
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.5 $
 */

public abstract class AbstractTreeNode implements NavigationNode {

    /**
     * Refers to this node's parent node.
     *
     * @see #getParent()
     */
    private final NavigationNode parent;

    /**
     * Holds the list of this node's child nodes.
     *
     * @see #getChildAt(int)
     * @see #getChildCount()
     * @see #children()
     */
    private final List children;

    /**
     * Refers to the model that is associated with this node.
     * For example a FlangeNode has an associated Flange instance.
     *
     * @see #getModel()
     */
    private final Object model;


    // Instance Creation ******************************************************

    /**
     * Constructs a tree node for the given parent and model.
     *
     * @param parent   this node's parent
     * @param model    the domain object associated with this node
     */
    public AbstractTreeNode(NavigationNode parent, Object model) {
        this.parent = parent;
        this.model = model;
        children = new ArrayList();
    }


    // Implementing the TreeNode Interface ************************************

    /**
     * Returns true if the receiver allows children.
     *
     * @return true if the receiver allows children
     */
    public boolean getAllowsChildren() {
        return true;
    }

    /**
     * Returns the child <code>TreeNode</code> at index
     * <code>childIndex</code>.
     *
     * @return the child at the specified index
     */
    public TreeNode getChildAt(int childIndex) {
        return (TreeNode) children.get(childIndex);
    }

    /**
     * Returns the number of children <code>TreeNode</code>s the receiver
     * contains.
     *
     * @return the number of children
     */
    public int getChildCount() {
        return children.size();
    }

    /**
     * Returns the children of the receiver as an <code>Enumeration</code>.
     *
     * @return the children of the receiver as an <code>Enumeration</code>.
     */
    public Enumeration children() {
        return Collections.enumeration(children);
    }

    /**
     * Returns the index of <code>node</code> in the receivers children.
     * If the receiver does not contain <code>node</code>, -1 will be
     * returned.
     *
     * @return the index of the given node, or -1 if it is not a child
     */
    public int getIndex(TreeNode node) {
        return children.indexOf(node);
    }

    /**
     * Returns the parent <code>TreeNode</code> of the receiver.
     *
     * @return the parent <code>TreeNode</code> of the receiver.
     */
    public TreeNode getParent() {
        return parent;
    }

    /**
     * Returns true if the receiver is a leaf.
     *
     * @return true if the receiver is a leaf.
     */
    public boolean isLeaf() {
        return getChildCount() == 0;
    }


    // Implementing the NavigationNode Interface ******************************

    /**
     * Returns this node's name. Subclasses typically implement this method
     * by returning the model's name or identifier.
     *
     * @return this node's name
     */
    public abstract String getName();


    /**
     * Returns this node's icon for the given selection state.
     *
     * @return this node's icon for the given selection state
     */
    public Icon getIcon(boolean selected) {
        return UIManager.getIcon(
            selected ? "Tree.openIcon" : "Tree.closedIcon");
    }


    /**
     * Returns this node's model. Subclasses may provide a type-safe method
     * with a more specific return type that is the model's class. For example,
     * the <code>FlangeNode</code> may implement <code>getFlange</code>.
     *
     * @return this node's model
     */
    public Object getModel() {
        return model;
    }


    // Misc *******************************************************************

    /**
     * Adds the given child to the list of this node's children.
     *
     * @param child the node to add as this node's child
     */
    public void add(NavigationNode child) {
        children.add(child);
    }


    /**
     * Returns a string representation for this node.
     * Currently it prints the class info and node name.
     *
     * @return a string representation for this node
     */
    @Override
    public String toString() {
        return super.toString() + ':' + getName();
    }


}