/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  o Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  o Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  o Neither the name of JGoodies Karsten Lentzsch nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.jgoodies.skeleton.model.node;

import java.util.Iterator;

import javax.swing.Icon;

import com.jgoodies.skeleton.domain.Flange;
import com.jgoodies.skeleton.domain.Project;
import com.jgoodies.skeleton.domain.Segment;
import com.jgoodies.skeleton.domain.Shaft;

/**
 * Describes the root node in the Skeleton (Pro) navigation tree.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.6 $
 */

public final class RootNode extends AbstractTreeNode {


    // Instance Creation ******************************************************

    /**
     * Creates a root node for the specified project.
     *
     * @param project    the associated project
     */
    public RootNode(Project project) {
        super(null, project);
        buildNodesFrom(project);
    }


    // Building the Tree ******************************************************

    /**
     * Builds nodes from the specified <code>Project</code>.
     *
     * @param project    the project to build the tree from
     */
    private void buildNodesFrom(Project project) {
        add(new DescriptionNode(this, project.getDescription()));

        for (Iterator i = project.getShafts().iterator(); i.hasNext();) {
            Shaft shaft = (Shaft) i.next();
            ShaftNode node = new ShaftNode(this, shaft);
            add(node);

            for (Iterator j = shaft.getComponents().iterator(); j.hasNext();) {
                Object component = j.next();
                if (component instanceof Segment) {
                    node.add(new SegmentNode(node, (Segment) component));
                } else if (component instanceof Flange) {
                    node.add(new FlangeNode(node, (Flange) component));
                } else
                    throw new IllegalStateException("Unknown node type.");

            }
        }
    }


    // ************************************************************************

    /**
     * Returns this node's associated Description instance.
     *
     * @return this node's associated Description instance.
     * @see NavigationNode#getModel()
     */
    public Project getProject() {
        return (Project) getModel();
    }


    /**
     * Returns this node's icon. Since the root node will be hidden
     * by the tree, we can return {@code null}.
     *
     * @return null
     */
    @Override
    public Icon getIcon(boolean sel) {
        return null;
    }


    /**
     * Returns this node's name. Since the root node will be hidden
     * by the tree, we can return {@code null}.
     *
     * @return null
     */
    @Override
    public String getName() {
        return null;
    }


    /**
     * Returns a string representation for this node.
     * Currently it prints the class info and node name.
     *
     * @return a string representation for this node
     */
    @Override
    public String toString() {
        return super.toString() + ':' + getProject().toString();
    }

}