/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.ResourceBundle;

import javax.swing.*;

import com.jgoodies.looks.LookUtils;
import com.jgoodies.skeleton.domain.Description;
import com.jgoodies.skeleton.domain.Flange;
import com.jgoodies.skeleton.domain.Segment;
import com.jgoodies.skeleton.domain.Shaft;
import com.jgoodies.skeleton.model.MainModel;
import com.jgoodies.skeleton.view.editor.*;
import com.jgoodies.uif_lite.AbstractView;
import com.jgoodies.uif_lite.component.Factory;

/**
 * Builds the main page of the Skeleton application:
 * the tool bar, navigation, all editors and the status bar.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.11 $
 *
 * @see MainModel
 * @see MainFrame
 */
final class MainPageView extends AbstractView {

    private static final Dimension PREFERRED_SIZE = LookUtils.IS_LOW_RESOLUTION
        ? new Dimension(620, 510)
        : new Dimension(760, 570);

    /**
     * Refers to the mainModel that provides all high-level models.
     * Used to build this frame's main page.
     *
     * @see #buildPanel()
     */
    private final MainModel mainModel;

    private ToolBarView    toolBarView;
    private NavigationView navigationView;
    private EditorView     editorView;


    // Instance Creation ******************************************************

    /**
     * Constructs a MainPageView for the given main mainModel.
     *
     * @param mainModel   provides high-level models
     */
    MainPageView(MainModel mainModel) {
        this.mainModel = mainModel;
    }


    // Building Panels ******************************************************

    private void initComponents() {
        toolBarView    = new ToolBarView(mainModel);
        navigationView = new NavigationView(mainModel);
        editorView     = new EditorView(mainModel);
        initEditorView();
    }


    /**
     * Adds and registers editors to the editor view,
     * and sets the active editor to the Welcome panel.
     */
    private void initEditorView() {
        WelcomeView welcomeView = new WelcomeView(mainModel);

        editorView.addEditor(null,              welcomeView);
        editorView.addEditor(Description.class, new DescriptionEditor());
        editorView.addEditor(Shaft.class,       new ShaftEditor());
        editorView.addEditor(Segment.class,     new SegmentEditor());
        editorView.addEditor(Flange.class,      new FlangeEditor());

        editorView.setActiveEditor(welcomeView);
    }


    /**
     * Builds this panel with the horizontal split pane
     * in the center and a status bar in the south.
     *
     * @return the built main page
     */
    @Override
    protected JComponent buildPanel() {
        initComponents();

        JPanel panel = new JPanel(new BorderLayout());
        panel.add(toolBarView.getToolBar(), BorderLayout.NORTH);
        panel.add(buildMainSplitPane(),     BorderLayout.CENTER);
        panel.add(buildStatusBar(),         BorderLayout.SOUTH);
        panel.setPreferredSize(PREFERRED_SIZE);
        return panel;
    }

    /**
     * Builds and returns the main <code>JSplitPane</code> that contains
     * the navigation elements on the left, and the view panels on the right.
     *
     * @return the split pane that consists of the navigator and editor panel
     */
    private JComponent buildMainSplitPane() {
        JSplitPane mainSplitPane =
            Factory.createStrippedSplitPane(
                JSplitPane.HORIZONTAL_SPLIT,
                navigationView.getPanel(),
                editorView.getPanel(),
                0.25);
        mainSplitPane.setBorder(BorderFactory.createEmptyBorder(6, 4, 0, 4));
        return mainSplitPane;
    }

    /**
     * Builds and returns the status bar.
     *
     * @return the status bar.
     */
    private JPanel buildStatusBar() {
        ResourceBundle bundle = ResourceBundle.getBundle(
            "com/jgoodies/skeleton/resources/Skeleton");

        JLabel statusField = new JLabel(bundle.getString("application.copyright"));
        JPanel panel = new JPanel(new BorderLayout());
        panel.setBorder(BorderFactory.createEmptyBorder(2, 6, 2, 3));
        panel.add(statusField, BorderLayout.WEST);
        return panel;
    }

}