/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view;

import javax.swing.JMenu;
import javax.swing.JMenuBar;

import com.jgoodies.looks.BorderStyle;
import com.jgoodies.looks.HeaderStyle;
import com.jgoodies.looks.Options;
import com.jgoodies.looks.plastic.PlasticLookAndFeel;
import com.jgoodies.looks.windows.WindowsLookAndFeel;
import com.jgoodies.skeleton.model.MainModel;
import com.jgoodies.skeleton.model.NavigationModel;

/**
 * Builds Skeleton's JMenuBar and pull-down menus.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.8 $
 *
 * @see MainFrame
 * @see MainModel
 */
final class MenuBarView {

    /**
     * Refers to this view's presentation mainModel that provides
     * the Actions used to build the menu items.
     */
    private final MainModel mainModel;

    /**
     * Holds the lazily created menu bar.
     */
    private JMenuBar menuBar;


    // Instance Creation ******************************************************

    /**
     * Constructs a MenuBarView for the given mainModel that
     * provides the Actions used to build the menu items.
     *
     * @param mainModel    provides the Actions used to build the menu items
     */
    MenuBarView(MainModel mainModel) {
        this.mainModel = mainModel;
    }


    // API ********************************************************************

    /**
     * Lazily builds and returns this view's menu bar.
     *
     * @return the lazily built menu bar
     */
    JMenuBar getMenuBar() {
        if (menuBar == null) {
            menuBar = buildMenuBar();
        }
        return menuBar;
    }


    // Building ***************************************************************


    /**
     * Configures, composes, and returns the menu bar.
     *
     * @return the menu bar
     */
    private JMenuBar buildMenuBar() {
        JMenuBar bar = new JMenuBar();

        // Set a hint so that JGoodies Looks will detect it
        // as being in the header.
        bar.putClientProperty(Options.HEADER_STYLE_KEY, HeaderStyle.BOTH);
        // Unlike the default, use a separator border.
        bar.putClientProperty(
            WindowsLookAndFeel.BORDER_STYLE_KEY,
            BorderStyle.SEPARATOR);
        bar.putClientProperty(
            PlasticLookAndFeel.BORDER_STYLE_KEY,
            BorderStyle.SEPARATOR);

        bar.add(buildFileMenu());
        bar.add(buildNavigatorMenu());
        bar.add(buildHelpMenu());
        return bar;
    }


    /**
     * Builds and returns the File menu.
     *
     * @return the File menu
     */
    private JMenu buildFileMenu() {
        JMenu menu = new JMenu("File");
        menu.setMnemonic('F');
        menu.add(mainModel.getNewProjectAction());
        menu.add(mainModel.getOpenProjectAction());
        menu.addSeparator();
        menu.add(mainModel.getSaveAction());
        menu.add(mainModel.getSaveAsAction());
        menu.addSeparator();
        menu.add(mainModel.getOpenPrintDialogAction());
        menu.addSeparator();
        menu.add(mainModel.getExitAction());
        return menu;
    }


    /**
     * Builds and returns the components menu.
     *
     * @return the Navigator menu
     */
    private JMenu buildNavigatorMenu() {
        NavigationModel model = mainModel.getNavigationModel();
        JMenu menu = new JMenu("Components");
        menu.setMnemonic('C');
        menu.add(model.getAddShaftAction());
        menu.add(model.getAddFlangeAction());
        menu.add(model.getAddSegmentAction());
        menu.add(model.getDeleteItemAction());
        return menu;
    }


    /**
     * Builds and returns the Help menu.
     *
     * @return the Help menu
     */
    private JMenu buildHelpMenu() {
        JMenu menu = new JMenu("Help");
        menu.setMnemonic('H');
        // Since no menu item has an icon, set a hint to the menu.
        menu.putClientProperty(Options.NO_ICONS_KEY, Boolean.TRUE);
        menu.add(mainModel.getOpenAboutDialogAction());
        return menu;
    }

}