/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.*;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeModel;

import com.jgoodies.skeleton.model.MainModel;
import com.jgoodies.skeleton.model.NavigationModel;
import com.jgoodies.skeleton.model.node.NavigationNode;
import com.jgoodies.uif_lite.AbstractView;
import com.jgoodies.uif_lite.component.Factory;
import com.jgoodies.uif_lite.panel.CardPanel;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

/**
 * Builds the navigation panel that is primarily intended to display
 * a tree of <code>NavigationNode</code> instances. Since Skeleton Pro
 * has no tree data at application startup time, there's an empty white panel
 * that is displayed as long as there's no data.
 * The first time the tree gets a tree model, the empty panel is replaced
 * by the tree. The empty panel and the tree are switched in a CardPanel.<p>
 *
 * This panel is embedded in a stripped scrollpane that in turn is contained
 * in a <code>SimpleInternalFrame</code> that in turn is contained
 * in the main page.<p>
 *
 * Observes the MainModel's navigation tree model to set it as tree model
 * in the navigation panel's tree.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.9 $
 *
 * @see MainModel
 * @see MainPageView
 */
final class NavigationView extends AbstractView {

    private static final String NO_TREE_CARD = "NoTree";
    private static final String TREE_CARD    = "Tree";

    /**
     * Refers to the main model that refers to the navigation model
     * that in turn holds the tree model and tree selection model.
     */
    private final MainModel mainModel;

    /**
     * Refers to the tree that displays the navigation nodes.
     *
     * @see #initComponents()
     */
    private JTree tree;

    /**
     * Holds two cards: a stub that shows "no project" and
     * a second card for the scrollable navigation tree.
     */
    private CardPanel cardPanel;


    // Instance Creation ******************************************************

    /**
     * Constructs a <code>NavigationView</code> for the given mainModel.
     *
     * @param mainModel  provides the tree model and tree selection model
     */
    NavigationView(MainModel mainModel) {
        this.mainModel = mainModel;
    }


    // Initialization *********************************************************

    /**
     * Creates, binds and configures the navigation tree
     * used to display the <code>NavigationNodes</code>.
     * Requests the tree model and the tree selection model
     * from the MainModel.
     */
    private void initComponents() {
        tree = new RefreshedTree(mainModel.getNavigationModel().getNavigationTreeModel());
        tree.setSelectionModel(mainModel.getNavigationModel().getNavigationTreeSelectionModel());
        tree.putClientProperty("JTree.lineStyle", "None");
        tree.setRootVisible(false);
        tree.setShowsRootHandles(true);
        tree.setCellRenderer(new TreeCellRenderer());

        cardPanel = new CardPanel();
    }

    private void initEventHandling() {
        mainModel.getNavigationModel().addPropertyChangeListener(
                NavigationModel.PROPERTYNAME_NAVIGATION_TREE_MODEL,
                new TreeModelChangeHandler());
    }


    // Building ***************************************************************

    /**
     * Builds and returns a CardPanel with two cards: one for the
     * navigation tree, another for the empty card.
     *
     * @return the built panel
     */
    @Override
    protected JComponent buildPanel() {
        initComponents();
        initEventHandling();

        SimpleInternalFrame panel = new SimpleInternalFrame("Navigator");
        panel.setContent(buildContent());
        panel.setSelected(true);
        panel.setMinimumSize  (new Dimension(100, 100));
        panel.setPreferredSize(new Dimension(160, 200));
        return panel;
    }


    private JComponent buildContent() {
        cardPanel.add(buildNoTreeCard(), NO_TREE_CARD);
        cardPanel.add(tree,              TREE_CARD);
        return Factory.createStrippedScrollPane(cardPanel);
    }


    /**
     * Builds and returns a stub panel that is used
     * as long as no <code>Project</code> has been set.
     */
    private JComponent buildNoTreeCard() {
        JPanel panel = new JPanel(new BorderLayout());
        panel.add(
            new JLabel("No project has been loaded."),
            BorderLayout.NORTH);
        panel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        panel.setPreferredSize(new Dimension(150, 200));
        panel.setBackground(Color.WHITE);
        return panel;
    }


    // Event Handling *********************************************************

    /**
     * Sets a new tree model, expand the top level nodes (shafts)
     * and pre-selects the first node (project data).
     *
     * @param newModel   the new tree model
     */
    private void updateModel(TreeModel newModel) {
        tree.setModel(newModel);
        int topLevelNodeCount = newModel.getChildCount(newModel.getRoot());
        // Skeleton Pro uses TreeUtils.expandTopLevel(tree).
        for (int i = topLevelNodeCount; i > 0; i--) {
            tree.expandRow(i);
        }
        tree.setSelectionRow(0);
    }


    /**
     * Listens to changes in the mainModel's navigation tree model and rebuilds the tree.
     */
    private final class TreeModelChangeHandler implements PropertyChangeListener {

        /**
         * The mainModel's navigation tree model has changed.
         * Update this navigator's tree model switch to the tree card
         * and expand the top-level nodes.
         *
         * @param evt   describes the property change
         */
        public void propertyChange(PropertyChangeEvent evt) {
            TreeModel newModel = (TreeModel) evt.getNewValue();
            updateModel(newModel);
            cardPanel.showCard(TREE_CARD);
        }
    }


    // Helper Classes *********************************************************

    /**
     * Renders icons and labels for the NavigationNodes.
     */
    private static final class TreeCellRenderer extends DefaultTreeCellRenderer {

        @Override
        public Component getTreeCellRendererComponent(JTree tree, Object value,
                                                      boolean sel,
                                                      boolean expanded,
                                                      boolean leaf, int row,
                                                      boolean focus) {
            super.getTreeCellRendererComponent(tree, value, selected, expanded,
                    false, row, focus);
            NavigationNode node = (NavigationNode) value;
            this.setForeground(sel
                    ? getTextSelectionColor()
                    : getTextNonSelectionColor());
            selected = sel;
            this.setIcon(node.getIcon(sel));
            this.setText(node.getName());
            return this;
        }
    }


    /**
     * Refreshes the selection path and cell renderer after UI changes.
     */
    private static final class RefreshedTree extends JTree {

        private RefreshedTree(TreeModel treeModel) {
            super(treeModel);
        }

        /**
         * In addition to the superclass behavior this methods
         * updates the selection path and cell renderer.
         */
        @Override
        public void updateUI() {
            super.updateUI();
            setSelectionPath(getSelectionPath());
            setCellRenderer(new TreeCellRenderer());
        }

    }
}