/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view;

import javax.swing.AbstractButton;
import javax.swing.Action;
import javax.swing.JToolBar;

import com.jgoodies.looks.BorderStyle;
import com.jgoodies.looks.HeaderStyle;
import com.jgoodies.looks.Options;
import com.jgoodies.looks.plastic.PlasticLookAndFeel;
import com.jgoodies.looks.windows.WindowsLookAndFeel;
import com.jgoodies.skeleton.model.MainModel;
import com.jgoodies.skeleton.model.NavigationModel;
import com.jgoodies.uif_lite.component.Factory;

/**
 * Builds the tool bar of the Skeleton application.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.8 $
 *
 * @see MainFrame
 * @see MainModel
 */
final class ToolBarView {

    /**
     * Refers to this view's presentation mainModel that provides
     * the Actions used to build the menu items.
     */
    private final MainModel mainModel;

    /**
     * Holds the lazily created tool bar.
     */
    private JToolBar toolBar;


    // Instance Creation ******************************************************

    /**
     * Constructs a ToolBarView for the given mainModel that
     * provides the Actions used to build the menu items.
     *
     * @param mainModel    provides the Actions used to build the menu items
     */
    ToolBarView(MainModel mainModel) {
        this.mainModel = mainModel;
    }


    // API ********************************************************************

    /**
     * Lazily builds and returns this view's tool bar.
     *
     * @return the lazily built menu bar
     */
    JToolBar getToolBar() {
        if (toolBar == null) {
            toolBar = buildToolBar();
        }
        return toolBar;
    }


    // Building ***************************************************************


    /**
     * Creates, configures, composes and returns the tool bar.
     *
     * @return the application's toolbar.
     */
    private JToolBar buildToolBar() {
        NavigationModel navigationModel = mainModel.getNavigationModel();
        JToolBar bar = new JToolBar();

        // Set the rollover property.
        bar.putClientProperty("JToolBar.isRollover", Boolean.TRUE);
        // Set a hint so that JGoodies Looks detect it as being in the header.
        bar.putClientProperty(Options.HEADER_STYLE_KEY, HeaderStyle.BOTH);
        // Unlike the default, use a separator border.
        bar.putClientProperty(
            WindowsLookAndFeel.BORDER_STYLE_KEY,
            BorderStyle.SEPARATOR);
        bar.putClientProperty(
            PlasticLookAndFeel.BORDER_STYLE_KEY,
            BorderStyle.SEPARATOR);

        bar.add(createToolBarButton(mainModel.getOpenProjectAction()));
        bar.add(createToolBarButton(mainModel.getSaveAction()));
        bar.add(createToolBarButton(mainModel.getPrintAction()));
        bar.addSeparator();
        bar.add(createToolBarButton(navigationModel.getAddShaftAction()));
        bar.add(createToolBarButton(navigationModel.getAddFlangeAction()));
        bar.add(createToolBarButton(navigationModel.getAddSegmentAction()));
        return bar;
    }


    /**
     * Creates and returns a button suitable for use in a tool bar.<p>
     *
     * The JGoodies Swing Suite comes with a special button component for
     * tool bars.
     */
    private AbstractButton createToolBarButton(Action action) {
        return Factory.createToolBarButton(action);
    }

}