/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view.editor;

import javax.swing.*;
import javax.swing.text.JTextComponent;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.skeleton.domain.Description;
import com.jgoodies.skeleton.model.Resources;
import com.jgoodies.uif_lite.builder.PanelBuilder2;

/**
 * An implementation of the {@link Editor} interface that presents the general
 * project data, i.e. instances of {@link Description}.<p>
 *
 * This container uses a <code>FormLayout</code> and the panel building
 * is done with the <code>PanelBuilder</code> class.
 * Columns and rows are specified before the panel is filled with components.
 * Unlike the {@link ShaftEditor} this class uses a hard coded row numbers.
 * This is easier to read with the downside that inserting a row requires
 * extra work.<p>
 *
 * This panel building style is simple and works quite well. However, you may
 * consider using a more sophisticated form builder to fill the panel and/or
 * add rows dynamically; see the {@link SegmentEditor} and {@link FlangeEditor}.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.13 $
 *
 * @see com.jgoodies.skeleton.domain.Description
 * @see	ShaftEditor
 * @see	SegmentEditor
 * @see	FlangeEditor
 */
public final class DescriptionEditor extends AbstractEditor {

    private JTextComponent  identifierField;
    private JTextComponent  companyNameField;
    private JTextComponent  contactPersonField;
    private JTextComponent  orderNoField;
    private JTextComponent  inspectorField;
    private JTextComponent  referenceNoField;
    private JComboBox       statusComboBox;
    private JTextComponent  shipYardField;
    private JTextComponent  registerNoField;
    private JTextComponent  hullNumbersField;
    private JRadioButton    typeNewBuildingRadio;
    private JRadioButton    typeConversionRadio;
    private JRadioButton    typeRepairRadio;


    // Instance Creation ****************************************************

    /**
     * Constructs a <code>DescriptionEditor</code>.
     */
    public DescriptionEditor() {
        super(Resources.DESCRIPTION_ICON);
    }

    // Abstract Behavior ****************************************************

    /**
     * Returns this <code>Editor</code>'s title.
     */
    @Override
    public String getTitle() {
        return "General Project Data";
    }


    // Storing and Restoring Models *****************************************

    /**
     * Writes editor contents to the given model.
     *
     * @param model  the object to write this editor's value to
     */
    public void updateModel(Object model) {
        Description description = (Description) model;

        description.setName(identifierField.getText());
        description.setCompanyName(companyNameField.getText());
        description.setContactPerson(contactPersonField.getText());
        description.setOrderNo(orderNoField.getText());
        description.setInspector(inspectorField.getText());
        description.setReferenceNo(referenceNoField.getText());
        description.setStatus((String) statusComboBox.getSelectedItem());
        description.setShipyard(shipYardField.getText());
        description.setRegisterNo(registerNoField.getText());
        description.setHullNumbers(hullNumbersField.getText());

        String type;
        if (typeNewBuildingRadio.isSelected())
            type = Description.TYPE_NEW_BUILDING;
        else if (typeConversionRadio.isSelected())
            type = Description.TYPE_CONVERSION;
        else
            type = Description.TYPE_REPAIR;
        description.setProjectType(type);
    }


    /**
     * Reads the editor contents from the given model.
     *
     * @param model    the object to read the values from
     */
    public void updateView(Object model) {
        Description description = (Description) model;

        identifierField.setText(description.getName());
        companyNameField.setText(description.getCompanyName());
        contactPersonField.setText(description.getContactPerson());
        orderNoField.setText(description.getOrderNo());
        inspectorField.setText(description.getInspector());
        referenceNoField.setText(description.getReferenceNo());
        statusComboBox.setSelectedItem(description.getStatus());
        shipYardField.setText(description.getShipyard());
        registerNoField.setText(description.getRegisterNo());
        hullNumbersField.setText(description.getHullNumbers());

        Object type = description.getProjectType();
        if (Description.TYPE_NEW_BUILDING.equals(type))
            typeNewBuildingRadio.setSelected(true);
        else if (Description.TYPE_CONVERSION.equals(type))
            typeConversionRadio.setSelected(true);
        else
            typeRepairRadio.setSelected(true);
    }


    // Component Creation and Configuration **********************************

    /**
     *  Creates and configures the UI components.
     */
    private void initComponents() {
        identifierField      = new JTextField();
        companyNameField     = new JTextField();
        contactPersonField   = new JTextField();
        orderNoField         = new JTextField();
        inspectorField       = new JTextField();
        referenceNoField     = new JTextField();
        statusComboBox       = new JComboBox(Description.STATUS_ARRAY);
        shipYardField        = new JTextField();
        registerNoField      = new JTextField();
        hullNumbersField     = new JTextField();

        typeNewBuildingRadio = new JRadioButton(Description.TYPE_NEW_BUILDING);
        typeNewBuildingRadio.setMnemonic('N');
        typeNewBuildingRadio.setContentAreaFilled(false);
        typeConversionRadio  = new JRadioButton(Description.TYPE_CONVERSION);
        typeConversionRadio.setMnemonic('v');
        typeConversionRadio.setContentAreaFilled(false);
        typeRepairRadio      = new JRadioButton(Description.TYPE_REPAIR);
        typeRepairRadio.setMnemonic('R');
        typeRepairRadio.setContentAreaFilled(false);
        ButtonGroup typeGroup = new ButtonGroup();
        typeGroup.add(typeNewBuildingRadio);
        typeGroup.add(typeConversionRadio);
        typeGroup.add(typeRepairRadio);
    }



    // Building *************************************************************

    /**
     * Builds and returns this editor's panel.
     */
    @Override
    protected JComponent buildPanel() {
        initComponents();

        FormLayout layout = new FormLayout(
                "$label, $lcgap, $component, $colgap, "
              + "$label, $lcgap, $component",
                "p, $lg, p, $pg, " +
                "3*(p, $lg), p, $pg, " +
                "3*(p, $lg), p, $pg, " +
                "3*(p, $lg), p, $lg, p");

        PanelBuilder builder = new PanelBuilder2(layout);
        builder.setDefaultDialogBorder();

        CellConstraints cc = new CellConstraints();
        builder.addSeparator("Project",      cc.xyw(1,  1, 7));
        builder.addLabel("Id&entifier:",     cc.xy (1,  3));
        builder.add(identifierField,         cc.xy (3,  3));

        builder.addSeparator("Manufacturer", cc.xyw(1,  5, 7));
        builder.addLabel("Comp&any:",        cc.xy (1,  7));
        builder.add(companyNameField,        cc.xyw(3,  7, 5));
        builder.addLabel("Con&tact:",        cc.xy (1,  9));
        builder.add(contactPersonField,      cc.xyw(3,  9, 5));
        builder.addLabel("Or&der no:",       cc.xy (1, 11));
        builder.add(orderNoField,            cc.xy (3, 11));

        builder.addSeparator("Inspector",    cc.xyw(1, 13, 7));
        builder.addLabel("Na&me:",           cc.xy (1, 15));
        builder.add(inspectorField,          cc.xyw(3, 15, 5));
        builder.addLabel("Reference n&o:",   cc.xy (1, 17));
        builder.add(referenceNoField,        cc.xy (3, 17));
        builder.addLabel("Stat&us:",         cc.xy (1, 19));
        builder.add(statusComboBox,          cc.xy (3, 19));

        builder.addSeparator("Ship",         cc.xyw(1, 21, 7));
        builder.addLabel("Sh&ipyard:",       cc.xy (1, 23));
        builder.add(shipYardField,           cc.xyw(3, 23, 5));
        builder.addLabel("Regi&ster no:",    cc.xy (1, 25));
        builder.add(registerNoField,         cc.xy (3, 25));
        builder.addLabel("Hull num&bers:",   cc.xy (1, 27));
        builder.add(hullNumbersField,        cc.xy (3, 27));
        builder.addLabel("Type:",            cc.xy (1, 29));
        builder.add(buildTypeChoice(),       cc.xyw(3, 29, 5));

        return builder.getPanel();
    }


    /**
     * Builds and returns a row of radio buttons for the type choice.
     *
     * @return a row of radio buttons for the type choice.
     */
    private JComponent buildTypeChoice() {
        FormLayout layout = new FormLayout("p, $rgap, p, $rgap, p");
        DefaultFormBuilder builder = new DefaultFormBuilder(layout);
        builder.append(typeNewBuildingRadio,
                       typeConversionRadio,
                       typeRepairRadio);
        return builder.getPanel();
    }


}