/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view.editor;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JTextField;
import javax.swing.text.JTextComponent;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.skeleton.domain.Constants;
import com.jgoodies.skeleton.domain.Flange;
import com.jgoodies.skeleton.model.Resources;

/**
 * An implementation of {@link Editor} that displays instances of
 * class {@link Flange}.<p>
 *
 * This class uses a {@link FormLayout} to lay out the panel and a
 * <code>DefaultFormBuilder</code> to add components. Columns are specified
 * <em>before</em> the panel is filled with components, rows are added
 * dynamically. The builder is used to update a cursor, to add rows dynamically,
 * and to fill components. The builder's convenience methods are used to add
 * labels and separators.<p>
 *
 * This panel building style is recommended unless you have a more
 * powerful builder or don't want to add rows dynamically.
 * See the {@link SegmentEditor} for an implementation that specifies
 * rows before the panel is filled with components. For learning purposes
 * you may check out the other {@link Editor} implementations too.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.12 $
 *
 * @see     com.jgoodies.skeleton.domain.Flange
 * @see     DescriptionEditor
 * @see     ShaftEditor
 * @see     SegmentEditor
 */
public final class FlangeEditor extends AbstractEditor {

    private JTextComponent  identifierField;
    private JTextComponent  ptiField;
    private JTextComponent  powerField;
    private JTextComponent  sField;
    private JTextComponent  daField;
    private JTextComponent  diField;
    private JTextComponent  da2Field;
    private JTextComponent  di2Field;
    private JTextComponent  rField;
    private JTextComponent  dField;
    private JComboBox       locationCombo;
    private JTextComponent  kFactorField;
    private JComboBox       materialCombo;
    private JTextComponent  numbersField;
    private JTextComponent  dsField;


    // Instance Creation ****************************************************

    /**
     * Constructs a <code>FlangeEditor</code>.
     */
    public FlangeEditor() {
        super(Resources.FLANGE_ICON);
    }


    // Storing and Restoring Models *****************************************

    /**
     * Writes the editor contents to the given model.
     *
     * @param model   the object to write this view's values to
     */
    public void updateModel(Object model) {
        Flange flange = (Flange) model;

        flange.setIdentifier(identifierField.getText());
        flange.setLocation((String) locationCombo.getSelectedItem());
        flange.setMaterial((String) materialCombo.getSelectedItem());
    }


    /**
     * Reads the editor contents from the given model.
     *
     * @param model    the object to read the values from
     */
    public void updateView(Object model) {
        Flange flange = (Flange) model;

        setTitleSuffix(flange.getIdentifier());
        identifierField.setText(flange.getIdentifier());
        locationCombo.setSelectedItem(flange.getLocation());
        materialCombo.setSelectedItem(flange.getMaterial());
    }


    // Component Creation and Configuration **********************************

    /**
     *  Creates and intializes the UI components.
     */
    private void initComponents() {
        identifierField = new JTextField();
        ptiField        = new JTextField();
        powerField      = new JTextField();
        sField          = new JTextField();
        daField         = new JTextField();
        diField         = new JTextField();
        da2Field        = new JTextField();
        di2Field        = new JTextField();
        rField          = new JTextField();
        dField          = new JTextField();
        locationCombo   = new JComboBox(Constants.LOCATION_ARRAY);
        kFactorField    = new JTextField();
        materialCombo   = new JComboBox(Constants.MATERIAL_ARRAY);
        numbersField    = new JTextField();
        dsField         = new JTextField();
    }


    // Building *************************************************************

    /**
     * Builds and returns this editor's panel.
     * Columns are specified before components are added to the form,
     * rows are added dynamically using the {@link DefaultFormBuilder}.<p>
     *
     * The builder combines a step that is done again and again:
     * add a label, proceed to the next data column and add a component.
     */
    @Override
    protected JComponent buildPanel() {
        initComponents();

        FormLayout layout = new FormLayout(
                "$label, $lcgap, $component, $colgap, "
              + "$label, $lcgap, $component");
        DefaultFormBuilder builder = new DefaultFormBuilder(layout);
        builder.setDefaultDialogBorder();

        builder.appendSeparator("Flange");

        builder.append("Id&entifier:", identifierField);
        builder.nextLine();

        builder.append("&PTI/kW:",   ptiField);
        builder.append("Po&wer/kW:", powerField);

        builder.append("&s/mm:",     sField);
        builder.nextLine();

        builder.appendSeparator("Diameters");

        builder.append("da/mm:",    daField);
        builder.append("di/mm:",    diField);

        builder.append("da2/mm:",   da2Field);
        builder.append("di2/mm:",   di2Field);

        builder.append("&R/mm:",     rField);
        builder.append("&D/mm:",     dField);

        builder.appendSeparator("Criteria");

        builder.append("&Location:", locationCombo);
        builder.append("&k-factor:", kFactorField);

        builder.appendSeparator("Bolts");

        builder.append("&Material:", materialCombo);
        builder.nextLine();

        builder.append("&Numbers:",  numbersField);
        builder.nextLine();

        builder.append("ds/mm:",    dsField);

        return builder.getPanel();
    }


}