/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view.editor;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

/**
 * Associates and selects <code>Editor</code>s for particular domain objects.
 * This is a simple form of the
 * <a href="http://martinfowler.com/eaaDev/PresentationChooser.html">Presentation
 * Chooser</a> pattern. It holds a Map that maps domain object types
 * to Editor instances. You can register an Editor for a given type
 * using <code>#register(Class, Editor)</code> and can look it up later
 * using <code>#lookup(Object)</code>.<p>
 *
 * Applications that require a more sophisticated association between
 * domain objects and presentations will likely need a more complex registry.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.9 $
 *
 * @see Editor
 */

final class PresentationChooser {


    /**
     * Maps domain classes to <code>Editor</code> instances.
     *
     * @see #register(Class, Editor)
     */
    private final Map editorRegistry;


    // Instance Creation ******************************************************

    /**
     * Constructs a PresentationChooser that has no Editors registered.
     */
    PresentationChooser() {
        editorRegistry = new HashMap();
    }


    // Managing Editors *******************************************************

    /**
     * Registers an <code>Editor</code> for a domain class.
     *
     * @param domainType   the key used to lookup the editor later
     * @param editor        the value stored under the given domain class
     */
    void register(Class domainType, Editor editor) {
        Object oldValue = editorRegistry.put(domainType, editor);
        if (oldValue != null) {
            Logger logger = Logger.getLogger("com.jgoodies.skeleton.view.editor.PresentationChooser");
            logger.warning("Duplicate editor registered for " + domainType);
        }
    }

    /**
     * Returns the <code>Editor</code> associated with the given domain object,
     * or {@code null} if there's none.
     *
     * @param domain   the object to be edited
     * @return the editor associated with the given domain object
     */
    Editor lookup(Object domain) {
        Class domainType = domain.getClass();
        return (Editor) editorRegistry.get(domainType);
    }


}