/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view.editor;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JTextField;
import javax.swing.text.JTextComponent;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.skeleton.domain.Constants;
import com.jgoodies.skeleton.domain.Segment;
import com.jgoodies.skeleton.model.Resources;

/**
 * An implementation of {@link Editor} that displays instances of
 * {@link Segment}.<p>
 *
 * This class uses a <code>FormLayout</code> to lay out the panel
 * and a <code>DefaultFormBuilder</code> to add components.
 * Columns and rows are specified before the panel is filled with components.
 * The builder is used to update a cursor and to fill components.
 * The builder's convenience methods are used to add labels and separators.<p>
 *
 * This panel building style is intended for learning purposes only.
 * The recommended style is demonstrated in the {@link FlangeEditor}.
 * For learning purposes you may check out the other {@link Editor}
 * implementations too.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.13 $
 *
 * @see com.jgoodies.skeleton.domain.Segment
 * @see DescriptionEditor
 * @see ShaftEditor
 * @see FlangeEditor
 */
public final class SegmentEditor extends AbstractEditor {

    private JTextComponent  identifierField;
    private JTextComponent  ptiField;
    private JTextComponent  powerField;
    private JTextComponent  lenField;
    private JTextComponent  daField;
    private JTextComponent  diField;
    private JTextComponent  da2Field;
    private JTextComponent  di2Field;
    private JTextComponent  rField;
    private JTextComponent  dField;
    private JComboBox       locationCombo;
    private JTextComponent  kFactorField;
    private JComboBox       connectionCombo;
    private JCheckBox       holesCheckBox;
    private JCheckBox       slotsCheckBox;


    // Instance Creation ****************************************************

    /**
     * Constructs a <code>SegmentEditor</code>.
     */
    public SegmentEditor() {
        super(Resources.SEGMENT_ICON);
    }


    // Storing and Restoring Models *****************************************

    /**
     * Writes view contents to the given model.
     *
     * @param model  the object to write this editor's value to
     */
    public void updateModel(Object model) {
        Segment segment = (Segment) model;

        segment.setIdentifier(identifierField.getText());
        segment.setLocation((String) locationCombo.getSelectedItem());
        segment.setConnection((String) connectionCombo.getSelectedItem());
    }


    /**
     * Reads view contents from the given model.
     *
     * @param model    the object to read the values from
     */
    public void updateView(Object model) {
        Segment segment = (Segment) model;

        setTitleSuffix(segment.getIdentifier());
        identifierField.setText(segment.getIdentifier());
        locationCombo.setSelectedItem(segment.getLocation());
        connectionCombo.setSelectedItem(segment.getConnection());
        holesCheckBox.setSelected(segment.getRadialHoles());
        slotsCheckBox.setSelected(segment.getLongitudinalSlots());
    }


    // Component Creation and Configuration **********************************

    /**
     *  Creates and configures the UI components.
     */
    private void initComponents() {
        identifierField = new JTextField();
        ptiField        = new JTextField();
        powerField      = new JTextField();
        lenField        = new JTextField();
        daField         = new JTextField();
        diField         = new JTextField();
        da2Field        = new JTextField();
        di2Field        = new JTextField();
        rField          = new JTextField();
        dField          = new JTextField();
        locationCombo   = new JComboBox(Constants.LOCATION_ARRAY);
        kFactorField    = new JTextField();
        connectionCombo = new JComboBox(Constants.MATERIAL_ARRAY);
        holesCheckBox   = new JCheckBox("Has radial holes");
        holesCheckBox.setMnemonic('o');
        slotsCheckBox   = new JCheckBox("Has longitudinal slots");
        slotsCheckBox.setMnemonic('s');
        slotsCheckBox.setDisplayedMnemonicIndex(17);
    }


    // Building *************************************************************

    /**
     * Builds and returns this editor's panel.
     */
    @Override
    protected JComponent buildPanel() {
        initComponents();

        FormLayout layout = new FormLayout(
                "$label, $lcgap, $component, $colgap, "
              + "$label, $lcgap, $component",
                "3*(p, $lg), p, $pg, "
              + "3*(p, $lg), p, $pg, "
              + "2*(p, $lg), p, 6dlu, p, $lg, p");

        DefaultFormBuilder builder = new DefaultFormBuilder(layout);
        builder.setDefaultDialogBorder();

        builder.appendSeparator("Segment");
        builder.nextLine(2);

        builder.append("Id&entifier:", identifierField);
        builder.nextLine(2);

        builder.append("&PTI/kW:",     ptiField);
        builder.append("Po&wer/kW:",   powerField);
        builder.nextLine(2);

        builder.append("len/mm:",     lenField);
        builder.nextLine(2);

        builder.appendSeparator("Diameters");
        builder.nextLine(2);

        builder.append("da/mm:",      daField);
        builder.append("di/mm:",      diField);
        builder.nextLine(2);

        builder.append("da2/mm:",     da2Field);
        builder.append("di2/mm:",     di2Field);
        builder.nextLine(2);

        builder.append("&R/mm:",       rField);
        builder.append("&D/mm:",       dField);
        builder.nextLine(2);

        builder.appendSeparator("Criteria");
        builder.nextLine(2);

        builder.append("&Location:",   locationCombo);
        builder.append("&k-factor:",   kFactorField);
        builder.nextLine(2);

        builder.append("Co&nnection:", connectionCombo);
        builder.nextLine(2);

        builder.nextColumn(2);
        builder.append(holesCheckBox, 5);
        builder.nextLine(2);

        builder.nextColumn(2);
        builder.append(slotsCheckBox, 5);

        return builder.getPanel();
    }

}