/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view.editor;

import java.awt.Component;

import javax.swing.*;
import javax.swing.text.JTextComponent;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.skeleton.domain.Constants;
import com.jgoodies.skeleton.domain.Shaft;
import com.jgoodies.skeleton.model.Resources;
import com.jgoodies.uif_lite.builder.PanelBuilder2;

/**
 * An implementation of {@link Editor} that displays a {@link Shaft}.<p>
 *
 * This container uses a <code>FormLayout</code> and the panel building
 * is done with the <code>PanelBuilder</code> class. Columns and rows
 * are specified before the panel is filled with components.<p>
 *
 * This panel building style is simple and works quite well. However, you may
 * consider using a more sophisticated form builder to fill the panel and/or
 * add rows dynamically; see the {@link SegmentEditor} for this alternative.
 * For learning purposes you max check out the other {@link Editor}
 * implementations.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.13 $
 *
 * @see com.jgoodies.skeleton.domain.Shaft
 * @see	DescriptionEditor
 * @see	SegmentEditor
 * @see	FlangeEditor
 */

public final class ShaftEditor extends AbstractEditor {

    private JTextComponent      identifierField;
    private JTextComponent      powerField;
    private JTextComponent      speedField;
    private JComboBox           materialComboBox;
    private JComboBox           iceClassComboBox;
    private JFormattedTextField lengthField;
    private JTextComponent      machineryCommentArea;
    private JTextComponent      inspectionCommentArea;


    // Instance Creation ****************************************************

    /**
     * Constructs a <code>ShaftEditor</code>.
     */
    public ShaftEditor() {
        super(Resources.SHAFT_ICON);
    }


    // Storing and Restoring Models *****************************************

    /**
     * Writes view contents to the given model.
     *
     * @param model  the object to write this editor's value to
     */
    public void updateModel(Object model) {
        Shaft shaft = (Shaft) model;

        shaft.setIdentifier(identifierField.getText());
        shaft.setPower(powerField.getText());
        shaft.setSpeed(speedField.getText());
        shaft.setMaterial((String) materialComboBox.getSelectedItem());
        shaft.setIceClass((String) iceClassComboBox.getSelectedItem());
        shaft.setLength(((Long) lengthField.getValue()).longValue());
        shaft.setMachinery(machineryCommentArea.getText());
        shaft.setInspection(inspectionCommentArea.getText());
    }


    /**
     * Reads view contents from the given model.
     *
     * @param model    the object to read the values from
     */
    public void updateView(Object model) {
        Shaft shaft = (Shaft) model;

        setTitleSuffix(shaft.getIdentifier());
        identifierField.setText(shaft.getIdentifier());
        powerField.setText(shaft.getPower());
        speedField.setText(shaft.getSpeed());
        materialComboBox.setSelectedItem(shaft.getMaterial());
        iceClassComboBox.setSelectedItem(shaft.getIceClass());
        lengthField.setValue(new Long(shaft.getLength()));
        machineryCommentArea.setText(shaft.getMachinery());
        inspectionCommentArea.setText(shaft.getInspection());
    }


    // Component Creation and Initialization **********************************

    /**
     *  Creates and configures the UI components.
     */
    private void initComponents() {
        identifierField       = new JTextField();
        powerField            = new JTextField();
        speedField            = new JTextField();
        materialComboBox      = new JComboBox(Constants.MATERIAL_ARRAY);
        iceClassComboBox      = new JComboBox(Constants.ICE_CLASS_ARRAY);
        lengthField           = new JFormattedTextField();
        machineryCommentArea  = new JTextArea();
        inspectionCommentArea = new JTextArea();
    }


    // Building *************************************************************

    /**
     * Builds the content pane.
     */
    @Override
    protected JComponent buildPanel() {
        initComponents();
        Component machineryPane = new JScrollPane(machineryCommentArea);
        Component inspectionPane = new JScrollPane(inspectionCommentArea);

        FormLayout layout = new FormLayout(
                "$label, $lcgap, $component, $colgap, "
              + "$label, $lcgap, $component",
                "4*(p, $lg), p, $pg, "
              + "4*(p, $lg), p");
        layout.setRowGroups(new int[][] { { 3, 13, 15, 17, 19 } });

        PanelBuilder builder = new PanelBuilder2(layout);
        builder.setDefaultDialogBorder();

        CellConstraints cc = new CellConstraints();

        builder.addSeparator("Shaft",    cc.xyw (1, 1, 7));

        builder.addLabel("Id&entifier:",   cc.xy  (1, 3));
        builder.add(identifierField,       cc.xy  (3, 3));

        builder.addLabel("&Power:",        cc.xy  (1, 5));
        builder.add(powerField,            cc.xy  (3, 5));
        builder.addLabel("&Speed:",        cc.xy  (5, 5));
        builder.add(speedField,            cc.xy  (7, 5));

        builder.addLabel("&Material:",     cc.xy  (1, 7));
        builder.add(materialComboBox,      cc.xy  (3, 7));
        builder.addLabel("&Ice class:",    cc.xy  (5, 7));
        builder.add(iceClassComboBox,      cc.xy  (7, 7));

        builder.addLabel("&Length:",       cc.xy  (1, 9));
        builder.add(lengthField,           cc.xy  (3, 9));

        builder.addSeparator("Comments", cc.xyw (1, 11, 7));

        builder.addLabel("M&achinery:",    cc.xy  (1, 13));
        builder.add(machineryPane,         cc.xywh(3, 13, 5, 3, "f, f"));

        builder.addLabel("I&nspection:",   cc.xy  (1, 17));
        builder.add(inspectionPane,        cc.xywh(3, 17, 5, 3, "f, f"));

        return builder.getPanel();
    }

}