/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif_lite;

import java.awt.Dimension;
import java.awt.Image;

import javax.swing.JComponent;
import javax.swing.JFrame;

import com.jgoodies.skeleton.Skeleton;
import com.jgoodies.uif_lite.application.ResourceMap;

/**
 * A light version of the JGoodies <code>AbstractFrame</code> class.
 * It provides the following features:
 * <ul>
 * <li>doesn't store and restore its state, for example, the window bounds
 * <li>has no minimum width
 * <li>has no support for aesthetic default sizes
 * <li>has no support for different logo icon sizes
 * </ul>
 *
 * @author Karsten Lentzsch
 */
public abstract class AbstractFrame extends JFrame {

    private static final String APPLICATION_ICON_ID  = "application.icon";
    private static final ResourceMap RESOURCES =
        new ResourceMap(null, Skeleton.class);



    // Instance Creation ******************************************************

    protected AbstractFrame(String label) {
        super(label);
    }


    // Building ***************************************************************

    /**
     * Builds the frame's content pane, packs it, locates it on the screen,
     * sets the icon, restores the saved state, and registers listeners.
     * It then is ready to be opened.<p>
     *
     * Subclasses should rarely override this method.
     */
    public void build() {
        JComponent content = buildContentPane();
        setContentPane(content);
        pack();
        locateOnScreen();
        configureWindowIcon();
        configureCloseOperation();
    }

    // Abstract Behavior ****************************************************

    /**
     * Builds and returns this frame's content pane.
     *
     * @return the built content pane
     */
    protected abstract JComponent buildContentPane();

    /**
     * Configures the behavior that will happen
     * when the user initiates a "close" on this frame.
     */
    protected abstract void configureCloseOperation();


    // Default Behavior *****************************************************

    /**
     * Locates the frame on the screen center; subclasses may override.
     */
    protected void locateOnScreen() {
        Dimension paneSize   = getSize();
        Dimension screenSize = getToolkit().getScreenSize();
        setLocation(
            (screenSize.width  - paneSize.width)  / 2,
            (screenSize.height - paneSize.height) / 2);
    }

    /**
     * Sets this frame's icon, which is fetched via the
     * <code>ResourceUtils</code>.
     */
    protected void configureWindowIcon() {
        Image logo = RESOURCES.getImage(APPLICATION_ICON_ID);
        if (logo != null) {
            setIconImage(logo);
        }
    }

}