/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif_lite.application;

import java.util.HashMap;
import java.util.Map;
import java.util.MissingResourceException;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.KeyStroke;

import com.jgoodies.skeleton.Skeleton;

/**
 * A light version of the JGoodies <code>ActionManager</code> class;
 * lacks lazy initialization and support for gray icons.<p>
 *
 * A repository for <code>Action</code> instances and manager that
 * reads and initializes action values from a <code>ResourceBundle</code>.<p>
 *
 * Once an action has been registered via #register, it can be retrieved
 * with #get, or #getAll.
 *
 * @author Karsten Lentzsch
 */
public final class ActionManager {

    private static final ResourceMap RESOURCES =
        new ResourceMap(null, Skeleton.class);

    private static final ActionManager INSTANCE = new ActionManager();


    private final Map actions;

    // Suppresses default constructor, ensuring non-instantiability.
    private ActionManager() {
        this.actions = new HashMap(50);
    }

    /**
     * Registers an <code>Action</code> for the given <code>id</code>.
     *
     * @param id       the key used to store and retrieve the Action
     * @param action   the Action to be registered
     * @return the registered Action
     */
    public static Action register(String id, Action action) {
        if (action == null)
            throw new NullPointerException("Registered actions must not be null.");

        ActionReader.readAndPutValues(action, RESOURCES, id);
        Object oldValue = INSTANCE.actions.put(id, action);
        if (oldValue != null)
            System.err.println("Duplicate action id: " + id);

        return action;
    }

    /**
     * Looks up and returns the <code>Action</code> for the given id.
     *
     * @param id   the key used to lookup the Action
     * @return the Action for the given id, null if none was registered
     *    for that id
     */
    public static Action get(String id) {
        Action action = (Action) INSTANCE.actions.get(id);
        if (null == action) {
            System.err.println("No action found for id: " + id);
            return null;
        }
        return action;
    }

    /**
     * The ActionReader reads action values for label, descriptions, etc. from
     * a resource bundle for a given id, and puts these values into an action.
     */
    private static final class ActionReader {

        private static final String LABEL             = "label";
        private static final char   MNEMONIC_MARKER   = '&';
        private static final String SHORT_DESCRIPTION = "tooltip";
        private static final String LONG_DESCRIPTION  = "helptext";
        private static final String ICON              = "icon";
        private static final String ACCELERATOR       = "accelerator";

        private static final String MNEMONIC_INDEX_KEY = "MnemonicIndexKey";

        private final String  id;
        private final String  name;
        private final Integer mnemonic;
        private final Integer aMnemonicIndex;
        private final String  shortDescription;
        private final String  longDescription;
        private final Icon    icon;
        private final KeyStroke accellerator;

        /**
         * Reads properties for <code>id</code> in <code>bundle</code> and
         * sets the approriate values in the given <code>action</code>.
         */
        static void readAndPutValues(
            Action action,
            ResourceMap resourceMap,
            String id) {
            ActionReader reader = new ActionReader(resourceMap, id);
            reader.putValues(action);
        }

        private ActionReader(ResourceMap resourceMap, String id) {
            this.id = id;
            String nameWithMnemonic = getString(resourceMap, id + "." + LABEL, id);
            int index = mnemonicIndex(nameWithMnemonic);
            name = stripName(nameWithMnemonic, index);
            mnemonic = stripMnemonic(nameWithMnemonic, index);
            aMnemonicIndex = new Integer(index);

            shortDescription =
                getString(
                        resourceMap,
                    id + '.' + SHORT_DESCRIPTION,
                    null);
            longDescription =
                getString(resourceMap, id + '.' + LONG_DESCRIPTION, null);

            Icon normalIcon;
            try {
                normalIcon = resourceMap.getIcon(id + '.' + ICON);
            } catch (MissingResourceException e) {
                normalIcon = null;
            }
            icon = normalIcon;

            String shortcut = getString(resourceMap, id + '.' + ACCELERATOR, null);
            accellerator = getKeyStroke(shortcut);
        }

        /**
         * Put the ActionReader's properties as values in the Action.
         */
        private void putValues(Action action) {
            action.putValue(Action.NAME, name);
            action.putValue(Action.SHORT_DESCRIPTION, shortDescription);
            action.putValue(Action.LONG_DESCRIPTION, longDescription);
            action.putValue(Action.SMALL_ICON, icon);
            action.putValue(Action.ACCELERATOR_KEY, accellerator);
            action.putValue(Action.MNEMONIC_KEY, mnemonic);
            action.putValue(MNEMONIC_INDEX_KEY, aMnemonicIndex);
        }

        private int mnemonicIndex(String nameWithMnemonic) {
            return nameWithMnemonic.indexOf(MNEMONIC_MARKER);
        }

        private String stripName(
            String nameWithMnemonic,
            int mnemonicIndex) {
            return mnemonicIndex == -1
                ? nameWithMnemonic
                : nameWithMnemonic.substring(0, mnemonicIndex)
                    + nameWithMnemonic.substring(mnemonicIndex + 1);
        }

        private Integer stripMnemonic(
            String nameWithMnemonic,
            int mnemonicIndex) {
            return mnemonicIndex == -1
                ? null
                : new Integer(Character.toUpperCase(
                        nameWithMnemonic.charAt(mnemonicIndex + 1)));
        }


        private KeyStroke getKeyStroke(String accelleratorString) {
            if (accelleratorString == null)
                return null;
            KeyStroke keyStroke =
                KeyStroke.getKeyStroke(accelleratorString);
            if (keyStroke == null)
                System.err.println(
                    "Action "
                        + id
                        + " has an invalid accellerator "
                        + accelleratorString);
            return keyStroke;
        }

        private String getString(
            ResourceMap resourceMap,
            String key,
            String defaultString) {
            try {
                return resourceMap.getString(key);
            } catch (MissingResourceException e) {
                return defaultString;
            }
        }

    }

}