/*
 * Copyright (c) 2006-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif_lite.application;

import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.text.Format;
import java.text.MessageFormat;
import java.text.ParseException;
import java.util.*;
import java.util.List;
import java.util.logging.Logger;

import javax.imageio.ImageIO;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.KeyStroke;

/**
 * Manages and provides access to the resource converters and defines
 * a default converter that handles several frequently used types.
 * Custom converters can be registered to convert types, if they
 * are not already supported, see {@link #register(ResourceConverter)}.
 * A custom converter overrides all previously registered converters
 * for the associated type, including the default converter.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.4 $
 *
 * @see ResourceConverter
 * @see ResourceMap#getObject(String, Class)
 */
final class ResourceConverters {

    private static final Logger LOGGER = Logger.getLogger(ResourceConverters.class.getName());

    /**
     * Holds all converters; used to lookup a converter for a given type
     * in case it has not been cached.
     */
    private static final List CONVERTERS =
        new ArrayList();


    /**
     * Maps classes to resource converters. Used to lookup the converter
     * associated with a given type in {@link ResourceMap#getObject}.
     */
    private static final Map CONVERTER_MAP =
        new HashMap();


    static {
        ResourceConverter defaultConverter = new DefaultConverter();
        register(defaultConverter);
    }

    private ResourceConverters() {
        // Overrides default constructor prevents instantiation.
    }


    // Converters *************************************************************

    /**
     * Registers a ResourceConverter for the given type. Overrides all
     * previously registered converters for its supported types.<p>
     *
     * Invalidates the cached converter map that will be re-filled
     * in {@link #forType(Class)}.
     *
     * @param converter     the converter to be associated with the given type;
     *     converts Strings to the given type
     *
     * @throws IllegalArgumentException  if the converter is null
     */
    public static synchronized void register(ResourceConverter converter) {
        if (converter == null) {
            throw new IllegalArgumentException("The converter must not be null.");
        }
        LOGGER.fine("Registering a resource converter:" + converter);
        CONVERTERS.add(0, converter);
        invalidateCache();
    }


    /**
     * Returns the ResourceConverter associated with the given type.
     * Accesses the converter cache first, and iterates over all converters
     * only if the converter has not been cached before.
     *
     * @param type   the class the result is associated with
     * @return a ResourceConverter that can convert strings to the given type
     *
     * @throws IllegalArgumentException  if the type is null
     */
    public static synchronized ResourceConverter forType(Class type) {
        if (type == null) {
            throw new IllegalArgumentException("The type must not be null.");
        }
        ResourceConverter converter = (ResourceConverter) CONVERTER_MAP.get(type);
        if (converter == null) {
            converter = lookupConverter(type);
            CONVERTER_MAP.put(type, converter);
        }
        return converter;
    }


    private static ResourceConverter lookupConverter(Class type) {
        for (Iterator iter = CONVERTERS.iterator(); iter.hasNext();) {
            ResourceConverter converter = (ResourceConverter) iter.next();
            if (converter.supportsType(type)) {
                return converter;
            }
        }
        return null;
    }


    // Cache Management *******************************************************

    private static void invalidateCache() {
        CONVERTER_MAP.clear();
    }


    // Default Converter ******************************************************

    private static final class DefaultConverter implements ResourceConverter {

        private static final Class[] SUPPORTED_TYPES = new Class[]{
              Boolean.class, boolean.class,
              Byte.class, byte.class,
              Calendar.class,
              Character.class, char.class,
              Color.class,
              Date.class,
              Dimension.class,
              Double.class, double.class,
              File.class,
              Float.class, float.class,
              Font.class,
              Icon.class, ImageIcon.class,
              Image.class,
              Insets.class,
              Integer.class, int.class,
              KeyStroke.class,
              Long.class, long.class,
              MessageFormat.class,
              Short.class, short.class,
              URL.class
        };


        /**
         * Iterates over the supporte types and returns
         * the first that matches.
         */
        public boolean supportsType(Class type) {
            for (Class element : SUPPORTED_TYPES) {
                if (type == element)
                    return true;
            }
            return false;
        }


        /**
         * {@inheritDoc}<p>
         *
         * This implementation converts a bunch of types.
         */
        public Object convert(String key, String value, Class type, ResourceMap r) {
            try {
                if (type == Boolean.class || type == boolean.class) {
                    return convertBoolean(key, value, r);
                } else if (type == Byte.class || type == byte.class) {
                    return Byte.valueOf(value);
                } else if (type == Calendar.class) {
                    return convertCalendar(key, value, r);
                } else if (type == Character.class || type == char.class) {
                    if (value.length() != 1) {
                        throw new ResourceConversionException(
                                "Character value too long.", key, value);
                    }
                    return new Character(value.charAt(0));  // #valueOf in 1.5
                } else if (type == Color.class) {
                    return Color.decode(value);
                } else if (type == Date.class) {
                    return convertDate(key, value, r);
                } else if (type == Dimension.class) {
                    return convertDimension(key, value, r);
                } else if (type == Double.class || type == double.class) {
                    return Double.valueOf(value);
                } else if (type == File.class) {
                    return new File(value);
                } else if (type == Float.class  || type == float.class) {
                    return Float.valueOf(value);
                } else if (type == Font.class) {
                    return Font.decode(value);
                } else if (type == Icon.class || type == ImageIcon.class) {
                    return new ImageIcon(convertImage(key, value, r));
                } else if (type == Image.class) {
                    return convertImage(key, value, r);
                } else if (type == Insets.class) {
                    return convertInsets(key, value, r);
                } else if (type == Integer.class || type == int.class) {
                    return Integer.valueOf(value);
                } else if (type == KeyStroke.class) {
                    return KeyStroke.getKeyStroke(value);
                } else if (type == Long.class || type == long.class) {
                    return Long.valueOf(value);
                } else if (type == MessageFormat.class) {
                    return new MessageFormat(value);
                } else if (type == Short.class || type == short.class) {
                    return Short.valueOf(value);
                } else if (type == URL.class) {
                    return convertURL(key, value, r);
                } else
                    throw new IllegalStateException("Unknown type " + type);
            } catch (ResourceConversionException e) {
                throw e;
            } catch (Throwable t) {
                throw new ResourceConversionException("Invalid " + type + " format.", key, value, t);
            }
        }


        private Boolean convertBoolean(String key, String value, ResourceMap r) {
            if (value.equalsIgnoreCase("true")) {
                return Boolean.TRUE;
            } else if (value.equalsIgnoreCase("false")) {
                return Boolean.FALSE;
            } else if (   value.equalsIgnoreCase("on")
                       || value.equalsIgnoreCase("yes")) {
                return Boolean.TRUE;
            } else if (   value.equalsIgnoreCase("off")
                       || value.equalsIgnoreCase("no")) {
                return Boolean.FALSE;
            }
            throw new ResourceConversionException(
                    "Illegal boolean format. Must be one of: true, false, yes, no, on, off", key, value);
        }


        private Calendar convertCalendar(String key, String value, ResourceMap r) {
            Date date = convertDate(key, value, r);
            if (date == null) {
                return null;
            }
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            return calendar;
        }


        private Date convertDate(String key, String value, ResourceMap r) {
            try {
                Format mediumFormat = DateFormat.getDateInstance(DateFormat.MEDIUM, Locale.ENGLISH);
                return (Date) mediumFormat.parseObject(value);
            } catch (ParseException e1) {
                Format shortFormat = DateFormat.getDateInstance(DateFormat.SHORT, Locale.ENGLISH);
                try {
                    return (Date) shortFormat.parseObject(value);
                } catch (Exception e2) {
                    throw new ResourceConversionException("Invalid date format.", key, value, e2);
                }
            }
        }


        private Dimension convertDimension(String key, String value, ResourceMap r) {
            try {
                int[] ints = parseInts(value);
                if (ints.length != 2) {
                    throw new ResourceConversionException("Invalid Dimension format. Must be: <int>, <int>.", key, value, null);
                }
                return new Dimension(ints[0], ints[1]);
            } catch (NumberFormatException e) {
                throw new ResourceConversionException("Invalid Dimension format. Must be: <int>, <int>.", key, value, null);
            }
        }


        private Image convertImage(String key, String value, ResourceMap r) {
            String path = r.resolvePath(value);
            URL url = r.getClassLoader().getResource(path);
            if (url == null) {
                throw new ResourceConversionException(
                        "Invalid image path.\nParent path=" + r.getResourceParentPath(), key, value, null);
            }
            try {
                return ImageIO.read(url);
            } catch (IOException e) {
                throw new ResourceConversionException(
                        "Can't read image.", key, value, e);
            }
        }


        private Insets convertInsets(String key, String value, ResourceMap r) {
            try {
                int[] ints = parseInts(value);
                if (ints.length != 4) {
                    throw new ResourceConversionException("Invalid Insets format. Must be: <int>, <int>.", key, value, null);
                }
                return new Insets(ints[0], ints[1], ints[2], ints[3]);
            } catch (NumberFormatException e) {
                throw new ResourceConversionException(
                        "Invalid Insets format. Must be: <int>, <int>, <int>, <int>.", key, value, null);
            }
        }


        private URL convertURL(String key, String value, ResourceMap r) {
            try {
                return new URL(value);
            } catch (MalformedURLException e) {
                URL url = r.getClassLoader().getResource(r.resolvePath(value));
                if (url != null) {
                    return url;
                }
                throw new ResourceConversionException(
                        "Invalid URL.", key, value, e);
            }
        }


        // Helper Code -------------------------------------------------------

        private int[] parseInts(String value) throws NumberFormatException {
            String[] token = value.split(",");
            int[] result = new int[token.length];
            for (int i = 0; i < result.length; i++) {
                result[i] = Integer.parseInt(token[i].trim());
            }
            return result;
        }

    }

}