/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.domain;

/**
 * A factory that can create a sample project with a sample description,
 * some shafts and child segments and flanges.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.8 $
 *
 * @see Project
 * @see Description
 * @see Flange
 * @see Segment
 * @see Shaft
 */
public final class ProjectFactory {

    private ProjectFactory() {
        // Overrides default constructor; prevents instantiation.
    }

    /**
     * Creates and returns a sample data model with sample components.
     *
     * @return a sample data model with sample components.
     */
    public static Project createSample() {
        Shaft shaft;

        Project project = new Project("Unnamed Project");

        fillDescription(project.getDescription());

        shaft = createShaft("Propeller Shaft1");
        shaft.addComponent(createSegment("Segment 1"));
        shaft.addComponent(createFlange("Flange 1a"));
        shaft.addComponent(createFlange("Flange 1b"));
        project.addShaft(shaft);

        shaft = createShaft("Intermediate Shaft");
        shaft.addComponent(createSegment("Segment 2a"));
        shaft.addComponent(createSegment("Segment 2b"));
        shaft.addComponent(createFlange("Flange 2a"));
        shaft.addComponent(createFlange("Flange 2b"));
        project.addShaft(shaft);

        shaft = createShaft("Propeller Shaft2");
        shaft.addComponent(createSegment("Segment 3a"));
        shaft.addComponent(createSegment("Segment 3b"));
        shaft.addComponent(createSegment("Segment 3c"));
        shaft.addComponent(createFlange("Flange 3a"));
        project.addShaft(shaft);

        return project;
    }


    /**
     * Sets sample data in the given description.
     */
    private static void fillDescription(Description description) {
        description.setName("Sample Project");
        description.setCompanyName("Hapag Lloyd");
        description.setOrderNo("583-992/2002");
        description.setContactPerson("Buzz Lightyear");
        description.setReferenceNo("32098");
        description.setInspector("Clouseau");
        description.setStatus(Description.STATUS_IN_PROGRESS);
        description.setRegisterNo("22067");
        description.setShipyard("HDW");
        description.setHullNumbers("472");
        description.setProjectType(Description.TYPE_NEW_BUILDING);
    }


    /**
     * Creates and returns a sample shaft with the specified identifier.
     */
    private static Shaft createShaft(String identifier) {
        Shaft shaft = new Shaft(identifier);
        shaft.setMaterial(Constants.MATERIAL_ARRAY[0]);
        shaft.setIceClass(Constants.ICE_CLASS_ARRAY[1]);
        return shaft;
    }


    /**
     * Creates and returns a sample shaft with the specified identifier.
     */
    private static Segment createSegment(String identifier) {
        Segment segment = new Segment(identifier);
        segment.setLocation(Constants.LOCATION_ARRAY[0]);
        segment.setConnection(Constants.MATERIAL_ARRAY[0]);
        segment.setRadialHoles(true);
        return segment;
    }


    /**
     * Creates and returns a sample shaft with the specified identifier.
     */
    private static Flange createFlange(String identifier) {
        Flange flange = new Flange(identifier);
        flange.setLocation(Constants.LOCATION_ARRAY[0]);
        flange.setMaterial(Constants.MATERIAL_ARRAY[0]);
        return flange;
    }

}