/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.model;

import com.jgoodies.binding.beans.Model;
import com.jgoodies.skeleton.domain.Flange;

/**
 * Provides bound properties for the model and proposed model.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.13 $
 */
public final class EditorModel extends Model {

    // Names of the Bound Bean Properties *************************************

    public static final String PROPERTYNAME_EDITED_OBJECT =
        "editedObject";

    public static final String PROPERTYNAME_VALIDATION_TARGET =
        "validationTarget";


    // Fields *****************************************************************

  /**
   * Holds a reference to this model's parent model, the MainModel.
   */
  private final MainModel mainModel;

    /**
     * Holds the object edited by the active editor.
     *
     * @see #getEditedObject()
     */
    private Object editedObject;

    /**
     * Holds the object to be validated.
     * This is used during the transition from an old selection
     * to a proposed new selection.
     *
     * @see #validateBeforeSelectionChange(Object)
     */
    private Object validationTarget;


    // Instance Creation ******************************************************

    /**
     * Constructs the editor sub-model for the given parent main model.
     */
    EditorModel(MainModel mainModel) {
        this.mainModel = mainModel;
    }


    // Handling Selection Changes *********************************************

    /**
     * Validates the given old value and returns the validation result string,
     * or {@code null} if the value is valid.
     * If you don't want to validate, just return {@code null}
     * to accept all selection changes.<p>
     *
     * Since this implementation validates domain objects,
     * we first need to copy the GUI values from the active editor
     * to the domain object to be validated.
     * This implementation validates the old selection
     * and assumes that the domain object temporarily accepts
     * invalid property values; and it assumes that all invalid
     * values will be fixed before the selection can change,
     * and before the value is saved.<p>
     *
     * As an alternative, you may clone the old selection,
     * and use the clone as validation target. This approach
     * could be used to flush invalid editor changes, if
     * the users wants to.
     *
     * @param oldSelection       the value before the change
     * @return the validation result string, or {@code null}
     *     if the object is valid
     */
    String validateBeforeSelectionChange(Object oldSelection) {
        Object validationObject = oldSelection;

        // Update the validationObject from the active editor's content.
        setValidationTarget(validationObject);

        return validate(validationObject);
    }


    /**
     * The selection has changed from the old to the new selection.
     * First updates the edited object, which in turn updates
     * the oldSelection from the old editor, and updates the new
     * editor from the new selection. Then saves the old selection.
     *
     * @param oldSelection   the object that has been edited before
     * @param newSelection   the object to become the new edited object
     */
    void selectionChanged(Object oldSelection, Object newSelection) {
        setEditedObject(newSelection);
        if (oldSelection != null) {
            mainModel.save(oldSelection);
        }
    }


    // Properties Accessors ***************************************************

    /**
     * Returns the domain object edited by the active editor.
     *
     * @return the domain object in the active editor.
     */
    public Object getEditedObject() {
        return editedObject;
    }

    /**
     * Sets the given domain object as edited object.
     *
     * @param newValue   the model to become the edited object
     */
    private void setEditedObject(Object newValue) {
        Object oldValue = getEditedObject();
        editedObject = newValue;
        firePropertyChange(PROPERTYNAME_EDITED_OBJECT, oldValue, newValue);
    }


    /**
     * Returns the domain object that shall be validated.
     *
     * @return the domain object to be validated
     */
    public Object getValidationTarget() {
        return validationTarget;
    }


    /**
     * Sets the given domain object as validation target.
     * Notifies the views that this target shall be updated
     * from the active editor. Uses an old value of {@code null}
     * to ensure that a PropertyChangeEvent is fired whenever
     * the new value is non-null.
     *
     * @param newValue   a domain object to be validated.
     */
    private void setValidationTarget(Object newValue) {
        validationTarget = newValue;
        firePropertyChange(PROPERTYNAME_VALIDATION_TARGET, null, newValue);
    }


    // Misc *******************************************************************

    /**
     * Validates the given object and returns a validation result string,
     * or {@code null} if the object is valid.
     *
     * @param model   the object to validate
     * @return the validation result string, or {@code null}
     *     if the object is valid
     */
    private String validate(Object model) {
        if (!(model instanceof Flange))
            return null;
        Flange flange = (Flange) model;
        StringBuilder builder = new StringBuilder();
        if (flange.getIdentifier().length() >= 10) {
            builder.append("Use an identifier with less than 10 characters.");
        }
        return builder.length() != 0
            ? builder.toString()
            : null;
    }


}