/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.model;

import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.io.File;
import java.util.EventObject;
import java.util.prefs.Preferences;

import javax.swing.ActionMap;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import com.jgoodies.binding.beans.Model;
import com.jgoodies.skeleton.domain.Project;
import com.jgoodies.skeleton.domain.ProjectFactory;
import com.jgoodies.skeleton.view.preferences.PreferencesDialog;
import com.jgoodies.uif.application.Action;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ApplicationContext;
import com.jgoodies.uif.application.ExitListener;
import com.jgoodies.uif.util.WindowUtils;
import com.jgoodies.uifextras.convenience.DefaultAboutDialog;
import com.jgoodies.uifextras.convenience.DefaultTipOfTheDayModel;
import com.jgoodies.uifextras.convenience.SetupManager;
import com.jgoodies.uifextras.convenience.TipOfTheDayDialog;
import com.jgoodies.uifextras.help.HelpBroker;
import com.jgoodies.uifextras.printing.PrintManager;

/**
 * Provides bound bean properties for the project, navigation tree,
 * navigation tree selection, selection type. Refers to the
 * presentation settings and a submodel for the dynamic help.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.33 $
 *
 * @see NavigationModel
 * @see UISettingsModel
 * @see DynamicHelpModel
 */
public final class MainModel extends Model implements ExitListener {

    // Names of the Bound Bean Properties *************************************

    public static final String PROPERTYNAME_PROJECT = "project";


    // Action Resource Keys ***************************************************

    public static final String ACTION_NEW_PROJECT         = "newProject";
    public static final String ACTION_OPEN_PROJECT        = "openProject";
    public static final String ACTION_SAVE                = "save";
    public static final String ACTION_SAVE_AS             = "saveAs";
    public static final String ACTION_OPEN_PAGE_SETUP     = "openPageSetup";
    public static final String ACTION_PRINT               = "print";
    public static final String ACTION_OPEN_PRINT_DIALOG   = "openPrintDialog";
    public static final String ACTION_OPEN_PREFERENCES    = "openPreferences";
    public static final String ACTION_EXIT                = "exit";
    public static final String ACTION_OPEN_TIP_OF_THE_DAY = "openTipOfTheDay";
    public static final String ACTION_OPEN_HELP_CONTENTS  = "openHelpContents";
    public static final String ACTION_OPEN_ABOUT_DIALOG   = "openAboutDialog";
    public static final String ACTION_CLOSE_HELP_VIEW     = "closeHelpView";


    // References to Submodels ************************************************

    /**
     * Refers to the navigation submodel that provides
     * bound properties, operations and Actions for the
     * navigation tree.
     *
     * @see #getNavigationModel()
     */
    private final NavigationModel navigationModel;

    /**
     * Refers to the editor submodel that provides
     * bound properties for the edited and validated object.
     */
    private final EditorModel editorModel;

    /**
     * Refers to the submodel that provides bound properties
     * around the dynamic help tree..
     *
     * @see #getHelpModel()
     */
    private final DynamicHelpModel helpModel;


    /**
     * Refers to the submodel for the UI-related settings.
     *
     * @see #getUiSettingsModel()
     */
    private final UISettingsModel uiSettingsModel;


    // Model State Fields *****************************************************

    /**
     * Holds the current project.
     *
     * @see #getProject()
     * @see #setProject(Project)
     */
    private Project project;


    // Instance Creation ******************************************************

    /**
     * Constructs a MainModel that has no project set,
     * no selection and no tree model.
     */
    public MainModel() {
        navigationModel = new NavigationModel(this);
        editorModel = new EditorModel(this);
        helpModel = new DynamicHelpModel();
        uiSettingsModel = new UISettingsModel();
        restoreState();
        initEventHandling();
        updateProjectActionEnablement(null);
    }


    // Initialization *********************************************************

    private void initEventHandling() {
        ApplicationContext.getInstance().getApplication().addExitListener(this);
    }


    // Accessing Submodels ****************************************************

    /**
     * Returns the navigation submodel.
     *
     * @return the navigation submodel.
     */
    public NavigationModel getNavigationModel() {
        return navigationModel;
    }


    /**
     * Returns the editor submodel.
     *
     * @return the editor submodel.
     */
    public EditorModel getEditorModel() {
        return editorModel;
    }


    /**
     * Returns the submodel that provides bound properties
     * for the help tree model, selection, help page and help visibility.
     *
     * @return the submodel for the dynamic help
     */
    public DynamicHelpModel getHelpModel() {
        return helpModel;
    }


    /**
     * Returns the UI related settings.
     *
     * @return the UI related settings
     */
    public UISettingsModel getUiSettingsModel() {
        return uiSettingsModel;
    }


    // Managing Projects ******************************************************

    /**
     * Returns the current project.
     *
     * @return the current project.
     */
    public Project getProject() {
        return project;
    }


    /**
     * Sets a new project.
     *
     * @param newProject   the project to set
     *
     * @throws NullPointerException if the new project is null
     */
    public void setProject(Project newProject) {
        if (newProject == null)
            throw new NullPointerException("The project must not be null.");

        Project oldProject = getProject();
        project = newProject;
        firePropertyChange(PROPERTYNAME_PROJECT, oldProject, newProject);
        getNavigationModel().updateNavigationTreeModel();
        updateProjectActionEnablement(newProject);
    }


    /**
     * Checks and answers if a project is loaded.
     *
     * @return true if a project is loaded.
     */
    public boolean hasProject() {
        return getProject() != null;
    }


    /**
     * Checks and answers whether the project's file path is valid.
     *
     * @return true if the project's file path is valid.
     */
    public boolean isProjectFilePathValid() {
        return false;
    }


    // Application ExitListener Implementation **************************

    /**
     * Stores this model's state if the application is closing.
     */
    public void applicationExiting() {
        storeState();
    }


    /**
     * Checks and answers whether we accept a close request.
     * The default behavior just says: true = OK to close.
     * You may ask: "Do you really want to close the application?".<p>
     *
     * TODO: Warn if the edited object is invalid and offer
     * to cancel the exit operation.
     *
     * @return true to accept the window close, false to reject it
     */
    public boolean applicationExitAllowed(EventObject event) {
        return true;
    }


    // Actions **************************************************************

    /**
     * Creates a new project and sets it in the main model.
     * Uses a sample project for demoing purposes.
     */
    @Action
    public void newProject() {
        setProject(ProjectFactory.createSample());
    }


    /**
     * Opens a file chooser, loads a project from a file,
     * and sets it in the main model.
     * Uses a sample project in this demo.
     */
    @Action
    public void openProject(ActionEvent e) {
        FileDialog dialog = new FileDialog(
                (Frame) WindowUtils.getParent(e),
                "Open project file");
        dialog.setVisible(true);
        dialog.dispose();
        if (dialog.getFile() == null)
            return;

        System.out.println(
            "You've chosen to open this file: " + dialog.getFile());
        setProject(Project.readFrom(new File(dialog.getFile())));
    }


    /**
     * Saves the project to the current file.
     */
    @Action
    public void save(ActionEvent e) {
        showMessage(e, "Save performed.");
    }


    /**
     * Asks the user for a file name and saves the project to that file.
     */
    @Action
    public void saveAs(ActionEvent e) {
        showMessage(e, "Save as performed.");
    }


    @Action
    public void openPageSetup(ActionEvent e) {
        PrintManager.openPageSetupDialog();
    }


    /**
     * Prints a project summary.
     */
    @Action
    public void print(ActionEvent e) {
        showMessage(e, "Print performed.");
    }


    /**
     * Opens a print dialog, then prints a project summary to the choosen printer.
     */
    @Action
    public void openPrintDialog(ActionEvent e) {
        showMessage(e, "Open print dialog performed.");
    }


    /**
     * Opens the preferences dialog.
     */
    @Action
    public void openPreferences(ActionEvent e) {
        new PreferencesDialog(
                (Frame) WindowUtils.getParent(e),
                getUiSettingsModel()).open();
    }


    @Action
    public void exit(ActionEvent e) {
        ApplicationContext.getInstance().getApplication().exit(e);
    }


    /**
     * Checks if we shall show a tip of the day: asks the TipOfTheDayDialog
     * whether it is enabled, and the SetupManager, if we are not running
     * for the first time. We don't want to disturb the user the first time,
     * where we already have opened some extra panels from the setup process.
     * <p>
     * Opens the tip of the day dialog in the event dispatch thread.
     */
    public void checkForOpenTipOfTheDayDialog() {
        if ((SetupManager.usageCount() > 1)
            && (DefaultTipOfTheDayModel.getInstance().isShowingTips())) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    openTipOfTheDay(null);
                }
            });
        }
    }


    /**
     * Opens the tip-of-the-day dialog. The ActionEvent may be null
     *
     * @param e   the ActionEvent in user interaction,
     *     or {@code null} if invoked at application startup
     */
    @Action
    public void openTipOfTheDay(ActionEvent e) {
        new TipOfTheDayDialog(
                (Frame) WindowUtils.getParent(e),
                DefaultTipOfTheDayModel.getInstance()).open();
    }


    @Action
    public void openHelpContents(ActionEvent e) {
        HelpBroker.openDefault();
    }


    /**
     * Opens the about dialog.
     */
    @Action
    public void openAboutDialog(ActionEvent e) {
        new DefaultAboutDialog((Frame) WindowUtils.getParent(e)).open();
    }


    @Action
    public void closeHelpView(ActionEvent e) {
        getHelpModel().closeDynamicHelp();
    }


    // Helper Code **********************************************************

    /**
     * Show a message.
     */
    void showMessage(EventObject e, String what) {
        JOptionPane.showMessageDialog(WindowUtils.getParent(e), what);
    }


    /**
     * Opens a message dialog that shows the validation result,
     * things the user must fix, before the selection can change.
     *
     * @param e                  the event that initiated this message
     * @param validationResult   the validation result to be displayed
     * @param error              true for an error message, false for a warning
     */
    void showValidationMessage(EventObject e, String validationResult, boolean error) {
        String messageText = validationResult + "\n\n";
        String titleText   = "Validation " + (error ? "Error" : "Warning");
        int messageType    = error ? JOptionPane.ERROR_MESSAGE : JOptionPane.WARNING_MESSAGE;
        JOptionPane.showMessageDialog(
                WindowUtils.getParent(e),
                messageText,
                titleText,
                messageType);
    }


    /**
     * Saves the given object to a database or whatever.
     */
    void save(Object model) {
        // save the given model.
        // System.out.println("Saving " + model);
    }


    // Storing State ********************************************************

    /**
     * Restores the application state from the user preferences.
     */
    public void restoreState() {
        getUiSettingsModel().restoreFrom(ApplicationContext.getInstance().getUserPreferences());
    }


    /**
     * Stores the application state to the user preferences.
     */
    public void storeState() {
        Preferences prefs = ApplicationContext.getInstance().getUserPreferences();
        getUiSettingsModel().storeIn(prefs);
        SetupManager.incrementUsageCounter();
    }


    // Updating the Action Enablement *****************************************

    /**
     * Updates the enablement of actions that are related to the project state.
     *
     * @param aProject    the current project
     */
    private void updateProjectActionEnablement(Project aProject) {
        ActionMap map = Application.getActionMap(this);
        boolean enabled = aProject != null;
        map.get(ACTION_SAVE).setEnabled(enabled);
        map.get(ACTION_SAVE_AS).setEnabled(enabled);
        map.get(ACTION_PRINT).setEnabled(enabled);
        map.get(ACTION_OPEN_PRINT_DIALOG).setEnabled(enabled);
    }


}