/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.model;

import java.util.prefs.Preferences;

import com.jgoodies.binding.beans.Model;

/**
 * Provides bound properties for presentation related settings.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.7 $
 */
public final class UISettingsModel extends Model {

    // Names of the Bound Bean Properties *************************************

    public static final String PROPERTYNAME_EXPAND_SELECTED_PATHS =
        "expandSelectedPaths";

    public static final String PROPERTYNAME_COLLAPSE_DESELECTED_PATHS =
        "collapseDeselectedPaths";


    // Default Values *********************************************************

    /**
     * The default value (true) for property 'expandSelectedPaths'.
     */
    private static final boolean DEFAULT_EXPAND_SELECTED_PATHS   = true;

    /**
     * The default value (false) for property 'collapseDeselectedPaths'.
     */
    private static final boolean DEFAULT_COLLAPSE_DESELECTED_PATHS = false;


    // Fields *****************************************************************

    /**
     * Describes whether a path shall be expanded on selection.
     */
    private boolean expandSelectedPaths = DEFAULT_EXPAND_SELECTED_PATHS;

    /**
     * Describes whether a path shall be collapsed on deselection.
     */
    private boolean collapseDeselectedPaths = DEFAULT_COLLAPSE_DESELECTED_PATHS;


    // Properties Accessors ***************************************************

    /**
     * Returns if the navigator collapses deselected paths.
     *
     * @return true if deselected paths will be collapsed
     */
    public boolean getCollapseDeselectedPaths() {
        return collapseDeselectedPaths;
    }


    /**
     * Returns if the navigator expands paths on selection.
     *
     * @return true if selected paths are expanded, false otherwise
     */
    public boolean getExpandSelectedPaths() {
        return expandSelectedPaths;
    }


    /**
     * Enables or disables the automatic compression of deselected paths.
     *
     * @param newValue   true to enable the auto collapse, false to disable it
     */
    public void setCollapseDeselectedPaths(boolean newValue) {
        boolean oldValue = getCollapseDeselectedPaths();
        collapseDeselectedPaths = newValue;
        firePropertyChange(PROPERTYNAME_COLLAPSE_DESELECTED_PATHS, oldValue, newValue);
    }


    /**
     * Enables or disables the automatic expansion of selected paths.
     *
     * @param newValue   true to enable the auto expansion, false to disable it
     */
    public void setExpandSelectedPaths(boolean newValue) {
        boolean oldValue = getExpandSelectedPaths();
        expandSelectedPaths = newValue;
        firePropertyChange(PROPERTYNAME_EXPAND_SELECTED_PATHS, oldValue, newValue);
    }


    // Property Persistency ***************************************************


    private static final String KEY_EXPAND_SELECTED_PATHS =
        "state.expandSelectedPaths";

    private static final String KEY_COLLAPSE_DESELECTED_PATHS =
        "state.collapseDeselectedPaths";


    /**
     * Restores the persistent properties from the specified UISettingsModel.
     *
     * @param prefs   the UISettingsModel object that holds the property values
     */
    void restoreFrom(Preferences prefs) {
        setExpandSelectedPaths(
            prefs.getBoolean(
                KEY_EXPAND_SELECTED_PATHS,
                DEFAULT_EXPAND_SELECTED_PATHS));

        setCollapseDeselectedPaths(
            prefs.getBoolean(
                KEY_COLLAPSE_DESELECTED_PATHS,
                DEFAULT_COLLAPSE_DESELECTED_PATHS));
    }

    /**
     * Stores the persistent properties in the specified UISettingsModel.
     *
     * @param prefs   the UISettingsModel object that holds the property values
     */
    void storeIn(Preferences prefs) {
        prefs.putBoolean(
            KEY_EXPAND_SELECTED_PATHS,
            getExpandSelectedPaths());

        prefs.putBoolean(
            KEY_COLLAPSE_DESELECTED_PATHS,
            getCollapseDeselectedPaths());
    }

}