/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view;

import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.net.URL;
import java.util.logging.Logger;

import javax.swing.*;

import com.jgoodies.skeleton.model.DynamicHelpModel;
import com.jgoodies.skeleton.model.MainModel;
import com.jgoodies.uif.AbstractView;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.builder.ToolBarBuilder;
import com.jgoodies.uif.component.ToolBarButton;
import com.jgoodies.uif.panel.SimpleInternalFrame;
import com.jgoodies.uifextras.util.HTMLUtils;
import com.jgoodies.uifextras.util.UIFactory;

/**
 * Display the contents of the currently selected dynamic help node.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.18 $
 *
 * @see DynamicHelpModel
 */
final class HelpContentView extends AbstractView {

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(HelpContentView.class);


    private final MainModel mainModel;

    /**
     * Refers to the model that provides a bound property for the help page.
     */
    private final DynamicHelpModel dynamicHelpModel;

    private JEditorPane editorPane;


    // Instance Creation ******************************************************

    /**
     * Constructs a <code>HelpContentView</code> for the given dynamicHelpModel.
     *
     * @param mainModel  provides the tree model and tree selection model
     */
    HelpContentView(MainModel mainModel) {
        this.mainModel = mainModel;
        this.dynamicHelpModel = mainModel.getHelpModel();
    }


    private void initEventHandling() {
        dynamicHelpModel.addPropertyChangeListener(
                DynamicHelpModel.PROPERTYNAME_HELP_PAGE,
                new PageChangeHandler());
    }


    @Override
    protected JComponent buildPanel() {
        initEventHandling();

        SimpleInternalFrame panel = new SimpleInternalFrame(
            RESOURCES.getIcon("helpContentView.icon"),
            RESOURCES.getString("helpContentView.text"));
        panel.setToolBar(buildToolBar());
        panel.setContent(buildContent());
        panel.setSelected(true);
        panel.setMinimumSize  (RESOURCES.getDimension("helpContentView.minSize"));
        panel.setPreferredSize(RESOURCES.getDimension("helpContentView.prefSize"));
        return panel;
    }


    // Building ***************************************************************

    /**
     * Builds and returns the tool bar.
     */
    private JToolBar buildToolBar() {
        ActionMap actionMap = Application.getActionMap(mainModel);
        ToolBarBuilder builder = new ToolBarBuilder("Help Contents");
        builder.add(new CloseButton(actionMap.get(MainModel.ACTION_CLOSE_HELP_VIEW)));
        return builder.getToolBar();
    }

    /**
     * Builds and answers the content pane.
     */
    private JComponent buildContent() {
        editorPane = new JEditorPane();
        editorPane.setMargin(new Insets(10, 10, 5, 2));
        editorPane.setEditable(false);
        editorPane.setContentType("text/html");
        JScrollPane scrollPane = UIFactory.createStrippedScrollPane(editorPane);
        scrollPane.setMinimumSize(new Dimension(100, 60));
        scrollPane.setPreferredSize(new Dimension(400, 60));
        return scrollPane;
    }


    // Handling Changes *******************************************************

    /**
     * Sets the page for the specified URL.
     */
    private void setPage(URL helpURL) {
        try {
            editorPane.setPage(helpURL);
            HTMLUtils.addDefaultStyleSheet(editorPane);
        } catch (IOException e) {
            if (helpURL != null)
                Logger.getLogger("HelpContentView").severe(
                    "Can't show help URL: " + helpURL);
        }
    }

    /**
     * Updates the help page if the dynamicHelpModel's page has changed.
     */
    private final class PageChangeHandler implements PropertyChangeListener {

        /**
         * The dynamicHelpModel's help page has changed. Sets the new page.
         *
         * @param evt   describes the property change
         */
        public void propertyChange(PropertyChangeEvent evt) {
            setPage((URL) evt.getNewValue());
        }
    }

    /**
     * A button that resets the rollover state after the
     * action has been performed and if the button is not showing.
     */
    private final class CloseButton extends ToolBarButton {
        private CloseButton(Action action) {
            super(action);
        }
        @Override
        protected void fireActionPerformed(ActionEvent event) {
            super.fireActionPerformed(event);
            if (!isShowing()) {
                getModel().setRollover(false);
            }
        }
    }

}