/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JComponent;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeModel;

import com.jgoodies.skeleton.model.DynamicHelpModel;
import com.jgoodies.skeleton.model.MainModel;
import com.jgoodies.skeleton.model.help.HelpNode;
import com.jgoodies.uif.AbstractView;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.panel.SimpleInternalFrame;
import com.jgoodies.uif.util.TreeUtils;
import com.jgoodies.uifextras.util.UIFactory;

/**
 * Builds the help navigation panel that displays a tree of help nodes.
 * The help node tree changes with the selection type of the navigation tree.<p>
 *
 * This panel is embedded in a stripped scrollpane that in turn is contained
 * in a <code>SimpleInternalFrame</code> that in turn is contained
 * in the main page.<p>
 *
 * Observes the MainModel's selection type to update the displayed HelpSet.
 * The associated HelpSet is requested from the HelpSets.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.16 $
 *
 * @see DynamicHelpModel
 * @see MainPageView
 */
final class HelpNavigationView extends AbstractView {

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(HelpNavigationView.class);


    /**
     * Refers to the model that holds the tree model and tree selection model.
     */
    private final DynamicHelpModel dynamicHelpModel;

    /**
     * Refers to the tree that displays the dynamic help nodes.
     *
     * @see #initComponents()
     */
    private JTree tree;


    // Instance Creation ******************************************************

    /**
     * Constructs a HelpNavigationView for the given dynamicHelpModel.
     *
     * @param mainModel  provides the tree model and tree selection model
     */
    HelpNavigationView(MainModel mainModel) {
        this.dynamicHelpModel = mainModel.getHelpModel();
    }


    // Initialization *********************************************************

    /**
     * Builds the tree utilized for this <code>HelpNavigator</code>.
     */
    private void initComponents() {
        tree = new RefreshedTree(dynamicHelpModel.getHelpTreeModel());
        tree.setSelectionModel(dynamicHelpModel.getHelpTreeSelectionModel());
        tree.setRootVisible(false);
        tree.setShowsRootHandles(false);
        tree.setCellRenderer(new TreeCellRenderer());
        TreeUtils.expandTopLevel(tree);
    }


    private void initEventHandling() {
        dynamicHelpModel.addPropertyChangeListener(
                DynamicHelpModel.PROPERTYNAME_HELP_TREE_MODEL,
                new TreeModelChangeHandler());
    }


    // Building ***************************************************************

    /**
     * Initializes and returns the help navigation tree.
     *
     * @return the help navigation tree
     */
    @Override
    protected JComponent buildPanel() {
        initComponents();
        initEventHandling();

        SimpleInternalFrame panel = new SimpleInternalFrame(
                RESOURCES.getString("helpNavigationView.text"));
        panel.setContent(UIFactory.createStrippedScrollPane(tree));
        panel.setSelected(true);
        panel.setMinimumSize  (new Dimension(100, 100));
        panel.setPreferredSize(new Dimension(100, 100));

        return panel;
    }


    // Event Handling *********************************************************

    /**
     * Sets a new tree model.
     *
     * @param newModel   the new tree model
     */
    private void updateModel(TreeModel newModel) {
        tree.setModel(newModel);
        TreeUtils.expandTopLevel(tree);
    }


    /**
     * Listens to changes in the dynamicHelpModel's navigation tree model and rebuilds the tree.
     */
    private final class TreeModelChangeHandler implements PropertyChangeListener {

        /**
         * The dynamicHelpModel's help tree model has changed.
         * Update this navigator's tree model and expand the top-level nodes.
         *
         * @param evt   describes the property change
         */
        public void propertyChange(PropertyChangeEvent evt) {
            TreeModel newModel = (TreeModel) evt.getNewValue();
            updateModel(newModel);
        }
    }


    // Helper Classes *********************************************************

    /**
     * A tree cell renderer that uses special HelpNode properties.
     */
    private static final class TreeCellRenderer extends DefaultTreeCellRenderer {

        private Font cachedOriginalFont;
        private Font boldFont;

        @Override
        public Component getTreeCellRendererComponent(JTree tree, Object value,
                                                      boolean sel,
                                                      boolean expanded,
                                                      boolean leaf, int row,
                                                      boolean focused) {
            super.getTreeCellRendererComponent(tree, value, selected, expanded,
                    false, row, focused);
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
            HelpNode item = (HelpNode) node.getUserObject();
            if (null == item) return this;
            this.setForeground(sel
                    ? getTextSelectionColor()
                    : getTextNonSelectionColor());
            selected = sel;
            this.setFont(item.isChapter()
                    ? getBoldFont(tree.getFont())
                    : tree.getFont());
            this.setIcon(item.getIcon(sel));
            return this;
        }

        private Font getBoldFont(Font originalFont) {
            if (originalFont != cachedOriginalFont) {
                boldFont = originalFont.deriveFont(Font.BOLD);
                cachedOriginalFont = originalFont;
            }
            return boldFont;
        }

    }


    /**
     * A tree that refreshes the cell renderer when the UI changes.
     */
    private static final class RefreshedTree extends JTree {

        private RefreshedTree(TreeModel treeModel) {
            super(treeModel);
        }

        /**
         * Updates the UI; updates the selection path and cell renderer.
         */
        @Override
        public void updateUI() {
            super.updateUI();
            setCellRenderer(new TreeCellRenderer());
        }

    }
}