/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view;

import java.awt.Dimension;
import java.util.prefs.Preferences;

import javax.swing.JComponent;
import javax.swing.JMenuBar;

import com.jgoodies.skeleton.model.MainModel;
import com.jgoodies.uif.AbstractFrame;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ApplicationContext;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.util.Resizer;
import com.jgoodies.uif.util.ScreenUtils;

/**
 * The main frame of the Skeleton Pro application.
 * Creates the menu bar, menus, tool bar and subpanels.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.20 $
 *
 * @see MainModel
 */

public final class MainFrame extends AbstractFrame {

    private static final Dimension MINIMUM_SIZE =
        new Dimension(500, 400);

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(MainFrame.class);


    /**
     * Refers to the model that provides all high-level submodels.
     * Used to build this frame's main page.
     *
     * @see #buildContentPane()
     */
    private final MainModel mainModel;

    private MainPageView mainPageView;


    // Instance Creation ****************************************************

    /**
     * Constructs an instance of the Skeleton Pro app's main frame.
     *
     * @param mainModel   provides bound properties and high-level models
     */
    public MainFrame(MainModel mainModel) {
        super(RESOURCES.getString("application.title"));
        setName("main");
        this.mainModel = mainModel;
    }


    // Building *************************************************************

    /**
     * Builds this <code>MainFrame</code>. Firstly, it executes the
     * superclass behavior, then sets the menu bar, registers help sets,
     * and finally makes the help view invisible.
     */
    @Override
    public void build() {
        super.build();
        setJMenuBar(buildMenuBar());
    }

    /**
     * Builds this frame's content pane.
     */
    @Override
    protected JComponent buildContentPane() {
        mainPageView = new MainPageView(mainModel);
        return mainPageView.getPanel();
    }

    /**
     * Builds and returns the menu bar.
     */
    private JMenuBar buildMenuBar() {
        return new MenuBarView(mainModel).getMenuBar();
    }


    // Implementing Abstract Behavior ****************************************

    /**
     * Returns the frame's minimum size. It is used by the WindowUtils
     * to resize the window if the user has shrinked the window below
     * this given size.
     *
     * @return the frame's minimum size
     */
    public Dimension getWindowMinimumSize() {
        return MINIMUM_SIZE;
    }


    /**
     * Tries to achieve a 5:4 aspect ratio.
     *
     * @param component   the component to be resized
     */
    @Override
    protected void resizeHook(JComponent component) {
        Resizer.FIVE2FOUR.resizeDialogContent(component);
    }


    @Override
    protected void configureCloseOperation() {
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(ApplicationContext.getInstance().getApplication().getApplicationExitOnWindowClosingHandler());
    }


    // Storing and Restoring State *******************************************

    /**
     * Stores this frame's state from the user preferences:
     * frame bounds and the main page's internal state.
     */
    @Override
    protected void storeState() {
        super.storeState();
        Preferences prefs = ScreenUtils.screenConfigurationNode(
                ApplicationContext.getInstance().getUserPreferences());
        mainPageView().storeIn(prefs);
    }


    /**
     * Restores this frame's state from the user preferences:
     * frame bounds and the main page's internal state.
     */
    @Override
    protected void restoreState() {
        super.restoreState();
        Preferences prefs = ScreenUtils.screenConfigurationNode(
                ApplicationContext.getInstance().getUserPreferences());
        mainPageView().restoreFrom(prefs);
    }


    // Accessing Collaborators **********************************************

    /*
     * Returns the main page view.
     */
    private MainPageView mainPageView() {
        return mainPageView;
    }

}