/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.LayoutManager;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.prefs.Preferences;

import javax.swing.*;

import com.jgoodies.looks.LookUtils;
import com.jgoodies.skeleton.domain.Description;
import com.jgoodies.skeleton.domain.Flange;
import com.jgoodies.skeleton.domain.Segment;
import com.jgoodies.skeleton.domain.Shaft;
import com.jgoodies.skeleton.model.DynamicHelpModel;
import com.jgoodies.skeleton.model.MainModel;
import com.jgoodies.skeleton.view.editor.*;
import com.jgoodies.uif.AbstractView;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.util.ComponentTreeUtils;
import com.jgoodies.uifextras.util.UIFactory;

/**
 * Builds the main page of the Skeleton Pro application:
 * the tool bar, navigator, help navigation, all editors,
 * the dynamic help viewer, and the status bar.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.19 $
 *
 * @see MainModel
 * @see MainFrame
 */

final class MainPageView extends AbstractView {

    private static final Dimension PREFERRED_SIZE =
        LookUtils.IS_LOW_RESOLUTION
            ? new Dimension(620, 510)
            : new Dimension(760, 570);

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(MainPageView.class);


    // UISettingsModel Key
    private static final String MAIN_DIVIDER_LOCATION_KEY =
        "main.main_divider_location";

    private static final String HELP_DIVIDER_LOCATION_KEY =
        "main.help_divider_location";


    /**
     * Refers to the model that provides all high-level models.
     * Used to build this frame's main page.
     */
    private final MainModel mainModel;

    private ToolBarView         toolBarView;
    private NavigationView      navigationView;
    private HelpNavigationView  helpNavigationView;
    private EditorView          editorView;
    private HelpContentView     helpContentView;
    private JSplitPane          mainSplitPane;
    private JSplitPane          editorHelpSplitPane;
    private JLabel              statusField;


    // Instance Creation ******************************************************

    /**
     * Constructs a MainPageView for the given main model.
     *
     * @param mainModel   provides high-level models
     */
    MainPageView(MainModel mainModel) {
        this.mainModel = mainModel;
    }


    // Initialization *********************************************************

    /**
     * Creates, binds and configures the subviews and components.
     */
    private void initComponents() {
        toolBarView        = new ToolBarView(mainModel);
        navigationView     = new NavigationView(mainModel);
        helpNavigationView = new HelpNavigationView(mainModel);
        editorView         = new EditorView(mainModel);
        initEditorView();
        helpContentView    = new HelpContentView(mainModel);
        statusField        = UIFactory.createPlainLabel(
                RESOURCES.getString("application.copyright"));
    }


    /**
     * Adds and registers editors to the editor view,
     * and sets the active editor to the Welcome panel.
     */
    private void initEditorView() {
        WelcomeView welcomePanel = new WelcomeView(mainModel);

        editorView.addEditor(null,              welcomePanel);
        editorView.addEditor(Description.class, new DescriptionEditor());
        editorView.addEditor(Shaft.class,       new ShaftEditor());
        editorView.addEditor(Segment.class,     new SegmentEditor());
        editorView.addEditor(Flange.class,      new FlangeEditor());

        editorView.setActiveEditor(welcomePanel);
    }


    private void initEventHandling() {
        mainModel.getHelpModel().addPropertyChangeListener(
                DynamicHelpModel.PROPERTYNAME_HELP_VISIBLE,
                new HelpVisibilityChangeHandler());
    }


    // Building *************************************************************

    /**
     * Builds this panel with the horizontal <code>JSplitPane</code> in the
     * center and a status bar in the south.
     */
    @Override
    protected JComponent buildPanel() {
        initComponents();
        initEventHandling();

        JPanel mainPage = new RefreshedPanel(new BorderLayout());
        mainPage.add(toolBarView.getToolBar(), BorderLayout.NORTH);
        mainPage.add(buildMainSplitPane(),     BorderLayout.CENTER);
        mainPage.add(buildStatusBar(),         BorderLayout.SOUTH);
        mainPage.setPreferredSize(PREFERRED_SIZE);

        setHelpVisible(false);

        return mainPage;
    }


    /**
     * Builds and answers the main <code>JSplitPane</code> that contains
     * the navigation elements on the left, and the view panels on the right.
     */
    private JComponent buildMainSplitPane() {
        mainSplitPane =
            UIFactory.createStrippedSplitPane(
                JSplitPane.HORIZONTAL_SPLIT,
                buildSplittedNavigationPanels(),
                buildEditorHelpPanel(),
                0.25);
        mainSplitPane.setBorder(BorderFactory.createEmptyBorder(6, 4, 0, 4));
        return mainSplitPane;
    }

    /**
     * Builds and returns a split pane that contains the
     * NavigationView and the HelpNavigationView.
     */
    private JComponent buildSplittedNavigationPanels() {
        return UIFactory.createStrippedSplitPane(
            JSplitPane.VERTICAL_SPLIT,
            navigationView.getPanel(),
            helpNavigationView.getPanel(),
            0.667);
    }

    /**
     * Builds the <code>EditorView</code>, the <code>HelpContentView</code>
     * and answers them wrapped by a stripped <code>JSplitPane</code>.
     */
    private JComponent buildEditorHelpPanel() {
        editorHelpSplitPane =
            UIFactory.createStrippedSplitPane(
                JSplitPane.VERTICAL_SPLIT,
                editorView.getPanel(),
                helpContentView.getPanel(),
                0.667);
        return editorHelpSplitPane;
    }

    /**
     * Builds and returns the status bar.
     */
    private JPanel buildStatusBar() {
        JPanel panel = new JPanel(new BorderLayout());
        panel.setBorder(BorderFactory.createEmptyBorder(2, 6, 2, 3));
        panel.add(statusField, BorderLayout.WEST);
        return panel;
    }


    // Storing and Restoring State ******************************************

    /**
     * Restores the frame's state from the user preferences.
     */
    public void restoreFrom(Preferences userPrefs) {
        int mainDividerLocation =
            userPrefs.getInt(MAIN_DIVIDER_LOCATION_KEY, -1);
        int verticalDividerLocation =
            userPrefs.getInt(HELP_DIVIDER_LOCATION_KEY, -1);

        if ((mainDividerLocation > 100)
            && (mainDividerLocation < getPanel().getWidth() - 50)) {
            mainSplitPane.setDividerLocation(mainDividerLocation);
        }
        if ((verticalDividerLocation > 100)
            && (verticalDividerLocation < getPanel().getHeight() - 50)) {
            editorHelpSplitPane.setDividerLocation(verticalDividerLocation);
        }
    }

    /**
     * Stores the frame's state in the user preferences.
     */
    public void storeIn(Preferences userPrefs) {
        int mainDividerLocation = mainSplitPane.getDividerLocation();
        int verticalDividerLocation =
            editorHelpSplitPane.getDividerLocation();
        userPrefs.putInt(MAIN_DIVIDER_LOCATION_KEY, mainDividerLocation);
        userPrefs.putInt(
            HELP_DIVIDER_LOCATION_KEY,
            verticalDividerLocation);
    }


    // Updating the Help View Visibility **************************************

    /**
     * Answers if the dynamic help contents panel is visible.
     */
    private boolean isHelpVisible() {
        return editorHelpSplitPane.getTopComponent() != null;
    }

    /**
     * Sets the visiblity of the dynamic help contents panel.
     */
    private void setHelpVisible(boolean b) {
        if (isHelpVisible() == b) {
            return;
        }
        int dividerLocation = mainSplitPane.getDividerLocation();
        if (b) {
            mainSplitPane.setRightComponent(editorHelpSplitPane);
            editorHelpSplitPane.setTopComponent(editorView.getPanel());
        } else {
            editorHelpSplitPane.setTopComponent(null);
            mainSplitPane.setRightComponent(editorView.getPanel());
        }
        mainSplitPane.setDividerLocation(dividerLocation);
        editorHelpSplitPane.validate();
    }


    /**
     * Listens to changes in the mainModel's navigation tree model and rebuilds the tree.
     */
    private final class HelpVisibilityChangeHandler implements PropertyChangeListener {

        /**
         * The mainModel's help visibility has changed.
         * Show or hide the help viewer.
         *
         * @param evt   describes the property change
         */
        public void propertyChange(PropertyChangeEvent evt) {
            boolean visible = ((Boolean) evt.getNewValue()).booleanValue();
            setHelpVisible(visible);
        }
    }


    // Helper Class ***********************************************************

    /**
     * In addition to the superclass behavior this class updates the UIs
     * of the help content that may not be contained in the component tree.
     */
    private final class RefreshedPanel extends JPanel {

        private RefreshedPanel(LayoutManager layout) {
            super(layout);
        }

        /**
         * In case some panels are invisible, we explicitly update their UIs.
         */
        @Override
        public void updateUI() {
            super.updateUI();
            if (getComponentCount() == 0)
                return;
            if (editorHelpSplitPane == null || isHelpVisible())
                return;
            ComponentTreeUtils.updateComponentTreeUI(editorHelpSplitPane);
        }

    }

}