/*
 * Copyright (c) 2002-2006 Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view;

import javax.swing.ActionMap;
import javax.swing.JMenu;
import javax.swing.JMenuBar;

import com.jgoodies.looks.BorderStyle;
import com.jgoodies.looks.HeaderStyle;
import com.jgoodies.looks.Options;
import com.jgoodies.looks.plastic.PlasticLookAndFeel;
import com.jgoodies.looks.windows.WindowsLookAndFeel;
import com.jgoodies.skeleton.model.MainModel;
import com.jgoodies.skeleton.model.NavigationModel;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.builder.MenuBuilder;
import com.jgoodies.uif.osx.OSXApplicationMenu;

/**
 * Builds the JMenuBar and pull-down menus in Skeleton Pro.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.10 $
 *
 * @see MainFrame
 * @see MainModel
 */
final class MenuBarView {

    /**
     * Refers to this view's presentation mainModel that provides
     * the Actions used to build the menu items.
     */
    private final MainModel mainModel;

    /**
     * Holds the lazily created menu bar.
     */
    private JMenuBar menuBar;


    // Instance Creation ******************************************************

    /**
     * Constructs a MenuBarView for the given mainModel that
     * provides the Actions used to build the menu items.
     *
     * @param mainModel    provides the Actions used to build the menu items
     */
    MenuBarView(MainModel mainModel) {
        this.mainModel = mainModel;
    }


    // API ********************************************************************

    /**
     * Lazily builds and returns this view's menu bar.
     *
     * @return the lazily built menu bar
     */
    JMenuBar getMenuBar() {
        if (menuBar == null) {
            menuBar = buildMenuBar();
        }
        return menuBar;
    }


    // Building ***************************************************************


    /**
     * Configures, composes, and returns the menu bar.
     *
     * @return the menu bar
     */
    private JMenuBar buildMenuBar() {
		JMenuBar bar = new JMenuBar();

		// Set a hint so that JGoodies Looks will detect it as being in the header.
		bar.putClientProperty(Options.HEADER_STYLE_KEY,
								  HeaderStyle.BOTH);
		// Unlike the default, use a separator border.
		bar.putClientProperty(WindowsLookAndFeel.BORDER_STYLE_KEY,
								  BorderStyle.SEPARATOR);
		bar.putClientProperty(PlasticLookAndFeel.BORDER_STYLE_KEY,
								  BorderStyle.SEPARATOR);

		bar.add(buildFileMenu());
		bar.add(buildNavigatorMenu());
		bar.add(buildHelpMenu());
		return bar;
	}


	/**
	 * Builds and returns the File menu.
	 */
	private JMenu buildFileMenu() {
        ActionMap mainMap = Application.getActionMap(mainModel);

        MenuBuilder builder = new MenuBuilder("File", 'F');
        builder.add(mainMap.get(MainModel.ACTION_NEW_PROJECT));
        builder.add(mainMap.get(MainModel.ACTION_OPEN_PROJECT));
        builder.addSeparator();
        builder.add(mainMap.get(MainModel.ACTION_SAVE));
        builder.add(mainMap.get(MainModel.ACTION_SAVE_AS));
        builder.addSeparator();
        builder.add(mainMap.get(MainModel.ACTION_OPEN_PAGE_SETUP));
        builder.add(mainMap.get(MainModel.ACTION_OPEN_PRINT_DIALOG));
        if (!OSXApplicationMenu.isRegisteredPreferences()) {
            builder.addSeparator();
            builder.add(mainMap.get(MainModel.ACTION_OPEN_PREFERENCES));
        }
        if (!OSXApplicationMenu.isRegisteredQuit()) {
            builder.addSeparator();
            builder.add(mainMap.get(MainModel.ACTION_EXIT));
        }
		return builder.getMenu();
	}


	/**
	 * Builds and returns the Component menu.
	 */
	private JMenu buildNavigatorMenu() {
        ActionMap navigationMap = Application.getActionMap(mainModel.getNavigationModel());

		MenuBuilder builder = new MenuBuilder("Components", 'C');
        builder.add(navigationMap.get(NavigationModel.ACTION_ADD_SHAFT));
        builder.add(navigationMap.get(NavigationModel.ACTION_ADD_FLANGE));
        builder.add(navigationMap.get(NavigationModel.ACTION_ADD_SEGMENT));
        builder.add(navigationMap.get(NavigationModel.ACTION_DELETE_ITEM));
		return builder.getMenu();
	}


	/**
	 * Builds and returns the Help menu.
	 */
	private JMenu buildHelpMenu() {
        ActionMap mainMap = Application.getActionMap(mainModel);
        MenuBuilder builder = new MenuBuilder("Help", 'H');
        builder.add(mainMap.get(MainModel.ACTION_OPEN_HELP_CONTENTS));
        builder.addSeparator();
        builder.add(mainMap.get(MainModel.ACTION_OPEN_TIP_OF_THE_DAY));
        if (!OSXApplicationMenu.isRegisteredAbout()) {
            builder.addSeparator();
            builder.add(mainMap.get(MainModel.ACTION_OPEN_ABOUT_DIALOG));
        }
		return builder.getMenu();
	}

}