/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view;

import javax.swing.ActionMap;
import javax.swing.JToolBar;

import com.jgoodies.looks.BorderStyle;
import com.jgoodies.looks.HeaderStyle;
import com.jgoodies.looks.plastic.PlasticLookAndFeel;
import com.jgoodies.looks.windows.WindowsLookAndFeel;
import com.jgoodies.skeleton.model.MainModel;
import com.jgoodies.skeleton.model.NavigationModel;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.builder.ToolBarBuilder;

/**
 * Builds the tool bar of the Skeleton Pro application.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.13 $
 *
 * @see MainFrame
 */
final class ToolBarView {

    /**
     * Refers to this view's presentation mainModel that provides
     * the Actions used to build the menu items.
     */
    private final MainModel mainModel;

    /**
     * Holds the lazily created tool bar.
     */
    private JToolBar toolBar;


    // Instance Creation ******************************************************

    /**
     * Constructs a ToolBarView for the given mainModel that
     * provides the Actions used to build the menu items.
     *
     * @param mainModel    provides the Actions used to build the menu items
     */
    ToolBarView(MainModel mainModel) {
        this.mainModel = mainModel;
    }


    // API ********************************************************************

    /**
     * Lazily builds and returns this view's tool bar.
     *
     * @return the lazily built menu bar
     */
    JToolBar getToolBar() {
        if (toolBar == null) {
            toolBar = buildToolBar();
        }
        return toolBar;
    }


    // Building ***************************************************************


    /**
     * Creates, configures, composes and returns the tool bar.
     *
     * @return the application's tool bar.
     */
    private JToolBar buildToolBar() {
        ActionMap mainMap = Application.getActionMap(mainModel);
        ActionMap navigationMap = Application.getActionMap(mainModel.getNavigationModel());

        // Set a hint so that JGoodies Looks will detect it as being in the header.
        ToolBarBuilder builder = new ToolBarBuilder("ToolBar", HeaderStyle.BOTH);
        // Unlike the default, use a separator border.
        builder.getToolBar().putClientProperty(
            WindowsLookAndFeel.BORDER_STYLE_KEY,
            BorderStyle.SEPARATOR);
        builder.getToolBar().putClientProperty(
            PlasticLookAndFeel.BORDER_STYLE_KEY,
            BorderStyle.SEPARATOR);

        builder.addGap(2);
        //toolBar.add(history().createGoBackButton());
        //toolBar.add(history().createGoNextButton());
        //toolBar.addGap(LARGE_GAP);
        builder.add(mainMap.get(MainModel.ACTION_OPEN_PROJECT));
        builder.add(mainMap.get(MainModel.ACTION_SAVE));
        builder.add(mainMap.get(MainModel.ACTION_PRINT));
        builder.addLargeGap();
        builder.add(navigationMap.get(NavigationModel.ACTION_ADD_SHAFT));
        builder.add(navigationMap.get(NavigationModel.ACTION_ADD_FLANGE));
        builder.add(navigationMap.get(NavigationModel.ACTION_ADD_SEGMENT));
        builder.addLargeGap();
        builder.addGlue();
        builder.add(mainMap.get(MainModel.ACTION_OPEN_HELP_CONTENTS));
        builder.addGap(2);
        return builder.getToolBar();
    }


}