/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view.editor;

import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JToolBar;


/**
 * This interface describes general editors that have an Icon,
 * title, ToolBar, and that can set and return a model.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.11 $
 *
 * @see AbstractEditor
 */

public interface Editor {


    /**
     * Returns this editor's icon. The icon can be shared
     * by all editors for a domain type, or it can change
     * for every individual edited domain object.
     *
     * @return this editor's icon
     */
    Icon getIcon();


    /**
     * Returns this editor's title. The title can be shared
     * by all editors for a domain type, or it can change
     * with every individual edited domain object.
     *
     * @return this editor's title.
     */
    String getTitle();


    /**
     * Returns this editor's tool bar.
     *
     * @return this editor's tool bar
     */
    JToolBar getToolBar();


    /**
     * Returns this editor's panel.
     *
     * @return this editor's built panel
     */
    JComponent getPanel();


    /**
     * Activates this editor. This method is invoked if this editor
     * shall become the active editor. In Skeleton (Pro) this happens
     * whenever the selection type in the navigation panel has changed
     * and this editor is associated with the new selection.
     * For example if the selection changes from type Flange
     * to type Shaft, this method will be invoked. It will not
     * be invoked if the selection changes from Flange1 to Flange2.<p>
     *
     * Implementors may choose to do nothing or update the view.
     * This depends on who can change domain objects. If only
     * editors can change the domain, you likely need not update
     * the view on editor activation. If the domain objects can
     * be changed by third parties, you may need to update the view
     * if an editor gets activated.<p>
     *
     * Implementors may also need to perform additional actions
     * on activation. For example, scroll a list or table to make
     * the current selection visible.
     */
    void activate();


    /**
     * Deactivates the editor. This method is invoked
     * if the selection type in the navigation panel has changed
     * and this editor is associated with the old selection.<p>
     *
     * Typically an implementor needs to update the model data
     * by copying values from this editor's components to the model.
     */
    void deactivate();


    /**
     * Reads the values from this editor's components
     * and sets them in the given model.
     *
     * @param model  the object to write this editor's value to
     */
    void updateModel(Object model);


    /**
     * Reads the values from the given model
     * and sets them in this editor's components.
     *
     * @param model    the object to read the values from
     */
    void updateView(Object model);


}
