/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view.editor;

import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import com.jgoodies.skeleton.model.EditorModel;
import com.jgoodies.skeleton.model.MainModel;
import com.jgoodies.skeleton.model.NavigationModel;
import com.jgoodies.uif.AbstractView;
import com.jgoodies.uif.panel.SimpleInternalFrame;
import com.jgoodies.uif.util.ComponentTreeUtils;
import com.jgoodies.uifextras.util.UIFactory;

/**
 * A container that wraps <code>Editor</code> instances with a JScrollPane,
 * which in turn is wrapped by a <code>SimpleInternalFrame</code>.<p>
 *
 * It keeps track of a collection of editors to be able to update
 * the UIs of invisible editors when the look&amp;feel changes.
 * <code>SwingUtilities#updateComponentTreeUI</code> updates
 * only editors that are in the component tree. Since we have only
 * the active editor in the component tree, we must update other
 * editors <em>by hand</em>.
 * An alternative implementation is to use a <code>CardPanel</code>
 * that has all editors in the component tree, but displays one of them
 * and hides all others.<p>
 *
 * Refers to a <code>PresentationChooser</code> that selects
 * an Editor appropriate for the currently selected domain object.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.14 $
 *
 * @see NavigationModel
 * @see Editor
 * @see javax.swing.SwingUtilities#updateComponentTreeUI(java.awt.Component)
 */
public final class EditorView extends AbstractView {

    /**
     * Reports changes for the edited and validated object.
     * Used to observe these changes to update the domain,
     * to switch editors and to change the edited object.
     */
    private final EditorModel editorModel;

    /**
     * Maps domain classes to <code>Editor</code> instances.
     *
     * @see #addEditor(Class, Editor)
     */
    private final PresentationChooser presentationChooser;

    /**
     * Holds a list of all editors to update the UI of invisible editors
     * in case the look&amp;feel changes. Contains some editors that
     * are not registered with the editor registry, for example the
     * WelcomeView that has no associated domain class.
     */
    private final List editors;

    private JScrollPane scrollPane;

    private Editor activeEditor;


    // Instance Creation ******************************************************

    /**
     * Constructs an EditorView for the given main model.
     *
     * @param mainModel   used to observe selection changes
     */
    public EditorView(MainModel mainModel) {
        this.editorModel = mainModel.getEditorModel();
        presentationChooser = new PresentationChooser();
        editors = new LinkedList();
    }


    // Building ***************************************************************

    private void initEventHandling() {
        editorModel.addPropertyChangeListener(
                EditorModel.PROPERTYNAME_EDITED_OBJECT,
                new EditedObjectChangeHandler());
        editorModel.addPropertyChangeListener(
                EditorModel.PROPERTYNAME_VALIDATION_TARGET,
                new ValidationTargetChangeHandler());
    }


    @Override
    protected JComponent buildPanel() {
        initEventHandling();

        SimpleInternalFrame panel = new RefreshedSimpleInternalFrame("Editor");
        panel.setContent(buildContent());
        panel.setMinimumSize  (new Dimension(200, 100));
        panel.setPreferredSize(new Dimension(400, 200));
        return panel;
    }


    /**
     * Builds the content, which is a panel wrapped by a stripped
     * <code>JScrollPane</code>. The panel will be replaced later
     * by the different viewers, where we access the viewer via
     * the scrollpane's viewport view.
     */
    private JComponent buildContent() {
        return scrollPane = UIFactory.createStrippedScrollPane(new JPanel(null));
    }


    /**
     * In addition to the superclass behavior this class updates the UIs
     * of the editors that are not contained in the component tree.
     */
    private final class RefreshedSimpleInternalFrame extends SimpleInternalFrame {

        private RefreshedSimpleInternalFrame(String title) {
            super(title);
        }

        /**
         * The look&amp;feel has changed. Updates the UI.
         * In addition to the superclass behavior, this method
         * updates the component trees for the invisible editors.
         */
        @Override
        public void updateUI() {
            super.updateUI();
            if (editors == null)
                return;

            for (Iterator i = editors.iterator(); i.hasNext();) {
                Editor editor = (Editor) i.next();

                // The Swing Suite provides class ComponentTreeUtils
                // that fixes bugs with l&f switching at runtime.
                ComponentTreeUtils.updateComponentTreeUI(editor.getPanel());
            }
        }

    }


    // Managing Editors *******************************************************

    /**
     * Adds an <code>Editor</code> to this <code>EditorView</code>.
     *
     * @param domainType   the key used to register the editor for lookup
     * @param editor       the editor to add
     */
    public void addEditor(Class domainType, Editor editor) {
        if (domainType != null)
            presentationChooser.register(domainType, editor);

        editors.add(editor);
    }


    // Changing the Editor ****************************************************

    /**
     * Returns the active <code>Editor</code>.
     *
     * @return the active <code>Editor</code>.
     */
    private Editor getActiveEditor() {
        return activeEditor;
    }

    /**
     * Shows the specified <code>Editor</code>: sets the icon,
     * title, tool bar, and finally switches the viewport's view.
     *
     * @param newEditor   the editor to be set
     */
    public void setActiveEditor(Editor newEditor) {
        SimpleInternalFrame panel = (SimpleInternalFrame) getPanel();
        panel.setFrameIcon(newEditor.getIcon());
        panel.setTitle(newEditor.getTitle());
        panel.setToolBar(newEditor.getToolBar());
        scrollPane.setViewportView(newEditor.getPanel());
        activeEditor = newEditor;
    }


    /**
     * Deactivates the current editor, looks up the editor associated
     * with the new selection, updates the view from the new selection,
     * and activates the chosen editor, which in turn makes it visible.
     *
     * @param oldEditedObject   the old domain selection
     * @param newEditedObject   the new domain selection
     */
    private void updateEditor(Object oldEditedObject, Object newEditedObject) {
        updateModel(oldEditedObject);
        if (newEditedObject == null)
            return;

        Editor oldEditor = getActiveEditor();
        Editor newEditor = presentationChooser.lookup(newEditedObject);
        if (oldEditor != newEditor) {
            oldEditor.deactivate();
        }

        if (newEditor == null)
            return;

        newEditor.updateView(newEditedObject);
        if (oldEditor != newEditor) {
            newEditor.activate();
            setActiveEditor(newEditor);
        }
    }


    /**
     * Copies the GUI state from the active editor to the given model.
     * Does nothing if the model is {@code null}.
     *
     * @param model   the object to be updated
     */
    private void updateModel(Object model) {
        if (model == null)
            return;

        getActiveEditor().updateModel(model);
    }


    // Event Handling *********************************************************

    /**
     * Listens to changes in the editor model's
     * edited object and updates the old edited object from the view,
     * activates a new editor - if necessary - and updates the new editor
     * from the new edited object.
     */
    private final class EditedObjectChangeHandler implements PropertyChangeListener {

        /**
         * The main model's selection has changed. Update the old model
         * from the view, look up the editor registered for the selection's
         * domain class, deactivate the active editor, activate the new editor
         * and set the latter as active editor.
         *
         * @param evt   describes the property change
         */
        public void propertyChange(PropertyChangeEvent evt) {
            Object oldEditedObject = evt.getOldValue();
            Object newEditedObject = evt.getNewValue();
            updateEditor(oldEditedObject, newEditedObject);
        }
    }


    /**
     * Listens to changes in the editor model's proposed selection
     * to update the domain, so that it can be validated and stored.
     */
    private final class ValidationTargetChangeHandler implements PropertyChangeListener {

        /**
         * The editor model indicates that there's a new validation target.
         * Copies the GUI state from the active editor to the new
         * validation target. After this update operation,
         * the validation target can be validated.
         *
         * @param evt   describes the property change
         */
        public void propertyChange(PropertyChangeEvent evt) {
            updateModel(editorModel.getValidationTarget());
        }
    }


}