/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.skeleton.view.preferences;

import java.awt.BorderLayout;
import java.awt.Frame;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.jgoodies.binding.PresentationModel;
import com.jgoodies.skeleton.model.UISettingsModel;
import com.jgoodies.uif.AbstractDialog;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.util.Resizer;
import com.jgoodies.uifextras.convenience.ResourceIDs;
import com.jgoodies.uifextras.laf.LafChoiceModel;
import com.jgoodies.uifextras.laf.LafChoiceView;
import com.jgoodies.uifextras.panel.HeaderPanel;

/**
 * Builds the preferences dialog.<p>
 *
 * This dialog's PresentationModel registers a PropertyChangeListener
 * with the settings object. To remove this listener, we set the model's bean
 * to null if the dialog is hidden. In case you reuse this dialog by
 * holding a reference to a dialog instance, you don't need to reset the bean.
 * If you keep a bunch of reusable dialogs in a history, you may set the
 * model's bean to the settings before the dialog is shown,
 * and can set it to null if the dialog is hidden.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.15 $
 *
 * @see com.jgoodies.binding.PresentationModel
 */
public final class PreferencesDialog extends AbstractDialog {

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(PreferencesDialog.class);


    /**
     * Refers to the model that holds the settings and that vends
     * ValueModels that adapt the bound bean properties of the settings.
     */
    private final PresentationModel<UISettingsModel> model;


    // Instance Creation ******************************************************

    /**
     * Constructs a PreferencesDialog for the given parent and settings.
     *
     * @param owner      this dialog's parent frame
     * @param settings   the settings to edit
     */
    public PreferencesDialog(Frame owner, UISettingsModel settings) {
        super(owner);
        this.model = new PresentationModel<UISettingsModel>(settings, getTriggerChannel());
    }


    // Building *************************************************************

    /**
     * Builds and returns the preference's header.
     */
    @Override
    protected JComponent buildHeader() {
        return new HeaderPanel(
            "Preferences",
            "You can see and edit the tool's persistent preferences."
                + "\nClick on a tab to select a different category.",
            RESOURCES.getIcon(ResourceIDs.PREFERENCES_ICON));
    }


    /**
     * Builds and returns the preference's content pane.
     */
    @Override
    protected JComponent buildContent() {
        JPanel content = new JPanel(new BorderLayout());
        content.add(buildTabbedPane(),                 BorderLayout.CENTER);
        content.add(buildButtonBarWithOKCancelApply(), BorderLayout.SOUTH);
        return content;
    }


    /**
     * Builds and returns the tabbed pane.
     */
    protected JTabbedPane buildTabbedPane() {
        LafChoiceView lafChoiceView = new LafChoiceView(new LafChoiceModel(getTriggerChannel()));

        JTabbedPane pane = new JTabbedPane();
        pane.addTab("General",     new GeneralTabView(model).getPanel());
        pane.addTab("Look & Feel", lafChoiceView.getPanel());
        return pane;
    }


    // Misc *****************************************************************

    /**
     * Closes the window.
     */
    @Override
    protected void doCloseWindow() {
        doCancel();
    }


    /**
     * Unlike the default try to get an aspect ratio of 1:1.
     */
    @Override
    protected void resizeHook(JComponent component) {
        Resizer.ONE2ONE.resizeDialogContent(component);
    }


    /**
     * In addition to the superclass behavior removes the PresentationModel's
     * PropertyChangeListener from its target bean, which is the settings.
     */
    @Override
    public void setVisible(boolean visible) {
        /*
         * In case you want to reuse the dialog instance you may consider
         * removing the code that sets the model's bean to null.
         * The settings and this dialog will then reference each other.
         *
         * Or you can add the commented code given below.
         * In the latter case, you must keep a reference to
         * the settings, for example in an instance field.
         */
        // if (visible) {
        //    model.setBean(settings);
        //}
        super.setVisible(visible);
        if (!visible) {
            model.setBean(null);
        }
    }

}