/*
 * Copyright (c) 2003-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.validation.demo.view;

import javax.swing.ActionMap;
import javax.swing.JMenu;
import javax.swing.JMenuBar;

import com.jgoodies.looks.BorderStyle;
import com.jgoodies.looks.HeaderStyle;
import com.jgoodies.looks.Options;
import com.jgoodies.looks.plastic.PlasticLookAndFeel;
import com.jgoodies.looks.windows.WindowsLookAndFeel;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.builder.MenuBuilder;
import com.jgoodies.uif.osx.OSXApplicationMenu;
import com.jgoodies.validation.demo.model.MainModel;

/**
 * A class that builds the menu bar and pull-down menus
 * of the JGoodies Forms Demo.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.9 $
 */
final class MenuBarView {

    /**
     * Refers to this view's presentation model that provides
     * the Actions used to build the menu items.
     */
    private final MainModel mainModel;

    /**
     * Holds the lazily created menu bar.
     */
    private JMenuBar menuBar;


    // Instance Creation ******************************************************

    /**
     * Constructs a MenuBarView for the given model that
     * provides the Actions used to build the menu items.
     *
     * @param mainModel    provides the Actions used to build the menu items
     */
    MenuBarView(MainModel mainModel) {
        this.mainModel = mainModel;
    }


    // API ********************************************************************

    /**
     * Lazily builds and returns this view's menu bar.
     *
     * @return the lazily built menu bar
     */
    JMenuBar getMenuBar() {
        if (menuBar == null) {
            menuBar = buildMenuBar();
        }
        return menuBar;
    }


    // Building ***************************************************************

	/**
	 * Configures, composes, and returns the menu bar.
	 */
	private JMenuBar buildMenuBar() {
		JMenuBar bar = new JMenuBar();

		// Set a hint so that JGoodies Looks will detect it as being in the header.
		bar.putClientProperty(Options.HEADER_STYLE_KEY,
								  HeaderStyle.BOTH);
		// Unlike the default, use a separator border.
		bar.putClientProperty(WindowsLookAndFeel.BORDER_STYLE_KEY,
								  BorderStyle.SEPARATOR);
		bar.putClientProperty(PlasticLookAndFeel.BORDER_STYLE_KEY,
								  BorderStyle.SEPARATOR);

        if (   !OSXApplicationMenu.isRegisteredPreferences()
            || !OSXApplicationMenu.isRegisteredQuit()) {
            bar.add(buildFileMenu());
        }

		bar.add(buildHelpMenu());
		return bar;
	}


	/**
	 * Builds and returns the File menu.
	 */
	private JMenu buildFileMenu() {
        ActionMap map = Application.getActionMap(mainModel);
        MenuBuilder builder = new MenuBuilder("File", 'F');
        builder.add(map.get(MainModel.ACTION_OPEN_PREFERENCES));
		builder.addSeparator();
        builder.add(map.get(MainModel.ACTION_EXIT));
		return builder.getMenu();
	}


	/**
	 * Builds and returns the Help menu.
	 */
	private JMenu buildHelpMenu() {
        ActionMap map = Application.getActionMap(mainModel);
        MenuBuilder builder = new MenuBuilder("Help", 'H');
        builder.add(map.get(MainModel.ACTION_OPEN_HELP_CONTENTS));
        if (!OSXApplicationMenu.isRegisteredAbout()) {
            builder.addSeparator();
            builder.add(map.get(MainModel.ACTION_OPEN_ABOUT_DIALOG));
        }
		return builder.getMenu();
	}

}
