/*
 * Copyright (c) 2003-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  o Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  o Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  o Neither the name of JGoodies Karsten Lentzsch nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.jgoodies.validation.tutorial.performance;

import java.util.HashMap;
import java.util.Map;

import com.jgoodies.validation.ValidationResult;
import com.jgoodies.validation.util.PropertyValidationSupport;
import com.jgoodies.validation.util.ValidationUtils;


/**
 * Validates containers, performs simple validation on the client-side,
 * and caches valid container numbers.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.7 $
 */
class ContainerValidatorWithCache extends ContainerValidator {

    /**
     * Maps container numbers to Booleans.
     *
     * @see #isValidContainerNumber(String)
     */
    private final Map<String, Boolean> cachedValidationResults;


    // Instance Creation ******************************************************

    /**
     * Constructs a ContainerValidator.
     */
    ContainerValidatorWithCache(String prefix) {
        super(prefix);
        cachedValidationResults = new HashMap<String, Boolean>();
    }


    // Validation *************************************************************

    /**
     * Validates this Validator's Container and returns the result
     * as an instance of {@link ValidationResult}.
     *
     * @param container  the container to be validated
     * @return the ValidationResult of the container validation
     */
    @Override
    public ValidationResult validate(Container container) {
        PropertyValidationSupport support =
            new PropertyValidationSupport(container, prefix);

        String containerNo = container.getContainerNo();
        System.out.println("Client validation: " + containerNo);
        if (ValidationUtils.isBlank(containerNo))
            support.addError("Container No", "is mandatory");
        else if (!ValidationUtils.hasBoundedLength(containerNo, 5, 5))
            support.addError("Container No", "length must be 5");
        else if (!isValidContainerNumber(containerNo))
            support.addError("Container No", "invalid");

        System.out.println();
        return support.getResult();
    }


    private boolean isValidContainerNumber(String containerNumber) {
        Boolean result = cachedValidationResults.get(containerNumber);
        if (result == null) {
            boolean valid = BackendService.isValidContainerNumber(containerNumber);
            result = Boolean.valueOf(valid);
            cachedValidationResults.put(containerNumber, result);
        }
        return result.booleanValue();
    }


}
