/*
 * Copyright (c) 2003-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  o Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  o Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  o Neither the name of JGoodies Karsten Lentzsch nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.jgoodies.validation.tutorial.performance;

import java.beans.PropertyChangeEvent;
import java.util.concurrent.ExecutionException;

import org.jdesktop.swingworker.SwingWorker;

import com.jgoodies.binding.beans.DelayedPropertyChangeHandler;
import com.jgoodies.validation.ValidationResult;

/**
 * Executes a given performance sub example in a background thread using a
 * SwingWorker. Validates the container using the example's container validator,
 * and adds the EDT time spent (should be close to 0).
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.10 $
 */
final class ValidationHandlerInBackground extends DelayedPropertyChangeHandler {

    private final PerformanceSubExample subExample;


    /**
     * Constructs a change handler that handles container no changes
     * with a slight delay, saving unnecessary validation operations.
     * The validation is performed in the background using a Swing Worker.
     *
     * @param subExample  provides the container, validator, and result model
     * @param delay       the milliseconds to wait before the delayed property change
     *     will be performed
     * @param coalesce    {@code true} to coalesce all pending changes,
     *     {@code false} to fire changes with the delay when an update
     *     has been received
     */
    ValidationHandlerInBackground(PerformanceSubExample subExample, int delay,
                                  boolean coalesce) {
        super(delay, coalesce);
        this.subExample = subExample;
    }


    @Override
    public void delayedPropertyChange(PropertyChangeEvent evt) {
        long t0 = System.currentTimeMillis();
        new ContainerNoWorker(subExample).execute();
        long t1 = System.currentTimeMillis();
        // The time spent will be close to 0.
        subExample.addTime(t1 - t0);
    }


    // Helper Class ***********************************************************

    static final class ContainerNoWorker extends SwingWorker<ValidationResult, Object> {

        private final PerformanceSubExample subExample;

        ContainerNoWorker(PerformanceSubExample subExample) {
            this.subExample = subExample;
        }

        @Override
        protected ValidationResult doInBackground() {
            Container container = subExample.getContainer();
            ContainerValidator validator = subExample.getContainerValidator();
            return validator.validate(container);
        }


        @Override
        protected void done() {
            try {
                subExample.getValidationResultModel().setResult(get());
            } catch (InterruptedException e) {
                e.printStackTrace();
            } catch (ExecutionException e) {
                e.getCause().printStackTrace();
            }
        }
    }

}
