/*
 * Copyright (c) 2003-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  o Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  o Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  o Neither the name of JGoodies Karsten Lentzsch nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.jgoodies.validation.tutorial.validator;

import com.jgoodies.validation.Validatable;
import com.jgoodies.validation.ValidationResult;
import com.jgoodies.validation.tutorial.shared.Order;
import com.jgoodies.validation.util.PropertyValidationSupport;
import com.jgoodies.validation.util.ValidationUtils;


/**
 * An Order subclass that can validate itself.
 * This class is intended for learning purposes.
 * The first choice for a domain object validation, is to have
 * a separate Validator class per domain class. See for example
 * the {@link OrderValidator}.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.10 $
 *
 * @see     OrderValidator
 */
final class ValidatingOrder extends Order implements Validatable {


    // Validation *************************************************************

    /**
     * Validates the order and returns a ValidationResult.
     *
     * @return the ValidationResult of the pure order validation
     */
    public ValidationResult validate() {
        PropertyValidationSupport support =
            new PropertyValidationSupport(this, "Order");

        if (ValidationUtils.isBlank(getOrderNo()))
            support.addError("Order No", "is mandatory");
        else if (!ValidationUtils.hasBoundedLength(getOrderNo(), 5, 10))
            support.addError("Order No", "length must be in [5, 10]");

        if (getOrderDate() == null)
            support.addError("Order Date", "is mandatory");

        if (getOrderDate() != null
            && getDeliveryDate() != null
            && getOrderDate().after(getDeliveryDate()))
            support.addWarning("Delivery Date", "shall be after the order date");

        if (getDeliveryNotes() != null
            && !ValidationUtils.hasMaximumLength(getDeliveryNotes(), 30))
            support.addWarning("Notes", "length shall be in [0, 30]");

        return support.getResult();
    }


}
