/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.MissingResourceException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import com.jgoodies.binding.value.BufferedValueModel;
import com.jgoodies.binding.value.Trigger;
import com.jgoodies.binding.value.ValueModel;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.component.UIFButton;
import com.jgoodies.uif.util.*;

/**
 * An abstract superclass that minimizes the effort required to build
 * consistent Swing dialogs quickly. It forms a template process
 * to build a standardized dialog and provides convenience behavior
 * to create frequently used dialog buttons, button bars and actions.
 * Also, this class can assist you in buffering dialog input values;
 * therefore it use the the {@link BufferedValueModel} from the
 * JGoodies Binding library. See below for more information on buffering.<p>
 *
 * The dialog build process begins with the <code>#build</code> method that
 * builds the content pane, applies a resize hook, sets the content pane,
 * packs the dialog, sets the resizable state and locates it on the screen.
 * Subclasses will rarely override the #build method. Instead, subclasses
 * must implement <code>#buildContent</code> to build the dialog's
 * main content. This content shall include the button bar
 * or button stack but not the header and not the dialog border.
 * The optional dialog header is built in <code>#buildHeader</code>
 * which answers {@code null} by default to indicate that the dialog
 * has no header. The border returned by <code>#getDialogBorder</code>
 * is wrapped around the content, not the header. Most dialogs will use a
 * default border for a plain dialog and a different border dialog with tabs.
 * The <code>#resizeHook</code> allows to resize the dialog to achieve
 * an esthetic aspect ratio. See {@link com.jgoodies.uif.util.Resizer} for
 * an example how to resize a dialog consistently.
 * <code>#locateOnScreen</code> enables you to change the dialog's location
 * on the screen. By default a dialog is located relative to its parent
 * frame or dialog. Other useful locations are the screen center or
 * a screen corner; class {@link com.jgoodies.uif.util.ScreenUtils} can assist you
 * in setting such a location.<p>
 *
 * To open a dialog invoke <code>#open</code>, to close it you can call
 * <code>#close</code>. Once it has been closed you can call
 * <code>#hasBeenCanceled</code> to understand whether this
 * dialog has been canceled or not.<p>
 *
 * AbstractDialog provides methods to create the following frequently used
 * buttons: <i>OK, Cancel, Accept, Apply, Reset, Close</i>. The button texts
 * are internationalized and can be localized under the keys mentioned in the
 * associated <code>#createXXXButton</code> method. If you click on such
 * a button, the associated <code>#doXXX</code> method will be invoked.
 * For example, if you click on the <i>Apply</i> button, <code>#doApply</code>
 * is invoked. If you override a <code>#doXXX</code> method, be sure to
 * call the super behavior. There are three methods that build button bars
 * from the standardized buttons mentioned before, where
 * <code>#createButtonBarWithOKCancel</code> is the most frequently used.<p>
 *
 * Find below a sample subclass that has a header, a button bar with <i>OK</i>
 * and <i>Cancel</i> and is resized to an esthetic aspect ratio:
 * <pre>
 * public final class MyDialog extends AbstractDialog {
 *
 *     protected JComponent buildHeader() {
 *         return new HeaderPanel(
 *             "My Dialog",
 *             "You can do click on the tabs below to ...",
 *             ResourceUtils.getIcon(MyResourceIDs.MY_DIALOG_ICON)
 *         );
 *     }
 *
 *     public JComponent buildContent() {
 *         JPanel content = new JPanel(new BorderLayout());
 *         content.add(buildMainPanel(),             BorderLayout.CENTER);
 *         content.add(buildButtonBarWithOKCancel(), BorderLayout.SOUTH);
 *         return content;
 *     }
 *
 *     private JComponent buildMainPanel() {
 *         // Your custom code to build the main panel...
 *     }
 *
 *     protected void resizeHook(JComponent component) {
 *         Resizer.DEFAULT.resize(component);
 *     }
 *
 * }
 * </pre><p>
 *
 * <strong>Buffering Input Values</strong><br>
 * This class provides optional behavior to delay the commit of dialog
 * input values. You can create buffered models that hold back value changes
 * until you either commit or flush the buffered values. You can find more
 * information about binding and buffering values in the documentation
 * for the JGoodies Binding library. This library is used to implement
 * buffering in this class.<p>
 *
 * AbstractDialog holds a <code>triggerChannel</code> that can be shared
 * by instances of BufferedValueModel to commit and flush buffered values.
 * This way changes of dialog values can be delayed until the trigger channel
 * triggers a commit or flush of the buffered values.<p>
 *
 * By default the trigger channel changes to true in <code>#doApply</code>
 * and <code>#doAccept</code> and changes to false in <code>#doReset</code>
 * and <code>#doCancel</code>. And so, clicking on <i>Apply</i> or <i>OK</i>
 * will commit buffered dialog values; while clicking on <i>Reset</i> or
 * <i>Cancel</i> will reset the buffered values to the original values.
 * See the documentation f the JGoodies Data Binding framework for details.<p>
 *
 * The trigger channel is provided as the bound Java bean property
 * <i>triggerChannel</i> that must be a non-{@code null}
 * <code>ValueModel</code> with values of type <code>Boolean</code>.
 * Attempts to read or write other value types may be rejected
 * with runtime exceptions. The trigger channel is initialized as an
 * instance of {@link Trigger}.<p>
 *
 * Find below a sample subclass, a customer editor, that buffers
 * the values of two input fields for the customer's last name and first name:
 * <pre>
 * public final class CustomerDialog extends AbstractDialog {
 *
 *     private Component lastNameField;
 *     private Component firstNameField;
 *     private Component titleField;
 *     ...
 *
 *     public CustomerDialog(Frame owner, Customer customer) {
 *         super(owner, "Edit Customer");
 *         initComponents(customer);
 *     }
 *
 *     private void initComponents(Customer customer) {
 *
 *         // Bound field using a DocumentAdapter
 *         lastNameField = new JTextField(40);
 *         lastNameField.setDocument(
 *             new DocumentAdapter(
 *                 buffer(new PropertyAdapter(customer, "lastName"))
 *             )
 *         );
 *
 *         // Bound field using a DocumentToValueConnector
 *         firstNameField = new JTextField(40);
 *         new DocumentToValueConnector(
 *             firstNameField,
 *             buffer(new PropertyAdapter(customer, "firstName"))
 *         );
 *
 *         // Bound field created by a factory
 *         titleField = BasicComponentFactory.createTextField(
 *             buffer(new PropertyAdapter(customer, "title"))
 *         );
 *         titleField.setColumns(20);
 *     }
 *
 *     public JComponent buildContent() {
 *         JPanel content = new JPanel(new BorderLayout());
 *         content.add(buildMainPanel(),             BorderLayout.CENTER);
 *         content.add(buildButtonBarWithOKCancel(), BorderLayout.SOUTH);
 *         return content;
 *     }
 *
 *     private JComponent buildMainPanel() {
 *         FormLayout layout = new FormLayout(
 *             "pref, 4dlu, default",  // columns
 *             ""                      // add rows dynamically
 *         );
 *         DefaultFormBuilder builder = new DefaultFormBuilder(layout);
 *         builder.setDefaultDialogBorder();
 *         builder.append("Last name",  lastNameField);
 *         builder.append("First name", firstNameField);
 *         builder.append("Title",      titleField);
 *         ...
 *         return builder.getPanel();
 *     }
 *
 * }
 * </pre>
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.10 $
 *
 * @see     ButtonBarFactory
 * @see     com.jgoodies.uif.util.Resizer
 * @see     com.jgoodies.binding.value.ValueModel
 * @see     com.jgoodies.binding.value.BufferedValueModel
 * @see     com.jgoodies.binding.value.Trigger
 */
public abstract class AbstractDialog extends JDialog {

    // Names of Bound Bean Properties *****************************************

    /**
     * The name of the bound read-write property <em>triggerChannel</em>.
     *
     * @see #getTriggerChannel()
     * @see #setTriggerChannel(ValueModel)
     */
    public static final String PROPERTYNAME_TRIGGER_CHANNEL = "triggerChannel";


    // Static Fields **********************************************************

    /**
     * Holds the default value for the <em>escapeCancelsMode</em> property.
     *
     * @see #getEscapeCancelsDefaultMode()
     * @see #setEscapeCancelsDefaultMode(Mode)
     */
    private static Mode escapeCancelsDefaultMode = Modes.ALWAYS;


    // Instance Fields ********************************************************

    /**
     * Indicates whether the dialog has been canceled or not.
     * The value is reset to {@code false} in the #open method.
     */
    private boolean canceled = false;

    /**
     * Holds a three-state trigger channel that can be used to trigger
     * commit and reset events in instances of {@link BufferedValueModel}.
     * The trigger value is changed to true in <code>#doApply</code> and
     * <code>#doOK</code> and changed to false in <code>#doReset</code> and
     * <code>#doCancel</code>.<p>
     *
     * The trigger channel is initialized as a <code>Trigger</code>
     * but may be replaced with any other <code>ValueModel</code>
     * that accepts booleans.
     */
    private ValueModel triggerChannel = new Trigger();

    /**
     * Refers to this dialogs ResourceMap that is used to localize
     * the texts for the output shortcuts: Yesterday, Today, Tomorrow.
     */
    private ResourceMap resourceMap;

    /**
     * Describes whether this dialog will be cancel if the Escape key
     * is pressed or not.
     *
     * @see #getEscapeCancelsMode()
     * @see #setEscapeCancelsMode(Mode)
     */
    private Mode escapeCancelsMode = getEscapeCancelsDefaultMode();


    // Fields for the lazily created frequently used dialog Actions ***********

    private Action applyAction;
    private Action cancelAction;
    private Action closeAction;
    private Action okAction;
    private Action resetAction;


    // Instance Creation ****************************************************

    public AbstractDialog() {
        this((Frame) null);
    }


    /**
     * Constructs a modal <code>AbstractDialog</code> with the given Frame
     * as its owner, using the application's default window title.
     *
     * @param owner   the dialog's parent frame
     */
    public AbstractDialog(Frame owner) {
        this(owner, getDefaultWindowTitle());
    }


    /**
     * Constructs a modal <code>AbstractDialog</code> with the given Frame
     * as its owner using the given window title.
     *
     * @param owner   the dialog's parent frame
     * @param title   the window title
     */
    public AbstractDialog(Frame owner, String title) {
        this(owner, title, true);
    }


    /**
     * Constructs an <code>AbstractDialog</code> with the given Frame
     * as its owner using the given window title  and the specified modal state.
     *
     * @param owner   the dialog's parent frame
     * @param title   the window title
     * @param modal   true for a model dialog, false for non-modal
     */
    public AbstractDialog(Frame owner, String title, boolean modal) {
        super(owner, title, modal);
        configureWindowClosing();
    }


    /**
     * Constructs a modal <code>AbstractDialog</code>  with the given Dialog
     * as its owner using the application's default window title.
     *
     * @param owner   the dialog's parent dialog
     */
    public AbstractDialog(Dialog owner) {
        this(owner, getDefaultWindowTitle());
    }


    /**
     * Constructs a modal <code>AbstractDialog</code> with the given
     * <code>Dialog</code> as its owner using the given window title.
     *
     * @param owner   the dialog's parent dialog
     * @param title   the window title
     */
    public AbstractDialog(Dialog owner, String title) {
        this(owner, title, true);
    }


    /**
     * Constructs an <code>AbstractDialog</code> with the given
     * <code>Dialog</code> as its owner using the given window title
     * and the specified modal state.
     *
     * @param owner   the dialog's parent dialog
     * @param title   the window title
     * @param modal   true for a model dialog, false for non-modal
     */
    public AbstractDialog(Dialog owner, String title, boolean modal) {
        super(owner, title, modal);
        configureWindowClosing();
    }


    // Default Window Title ***************************************************

    private static String getDefaultWindowTitle() {
        ResourceMap resourceMap = Application.getResourceMap(AbstractDialog.class);
        return resourceMap.getString("application.title");
    }


    // Internationalization ***************************************************

    /**
     * Returns the ResourceMap used to lookup localized button texts
     * and other resources.
     *
     * @return the ResourceMap used to lookup localized button texts
     *    and other resources
     */
    public ResourceMap getResourceMap() {
        if (resourceMap == null) {
            resourceMap = Application.getResourceMap(getClass());
        }
        return resourceMap;
    }


    /**
     * Retrieves and returns a String for the given key from this dialog's
     * resource map. If the resource is absent, the key is returned instead.
     *
     * @param key  the key used to lookup the localized string
     * @return the localized text or default text
     *
     * @see #getResourceMap()
     */
    protected String getString(String key) {
        try {
            return getResourceMap().getString(key);
        } catch (MissingResourceException e) {
                Logger.getLogger(getClass().getName()).
                    log(Level.WARNING, "AbstractDialog", e);
                return key;
        }
    }


    // Escape Cancels *********************************************************

    /**
     * Returns the default value for the <em>escapeCancelsMode</em> property.
     * The value will be used to initialize this property. The default can be
     * overridden per dialog using <code>setEscapeCancelsMode</code>.
     *
     * @return the default value for the <em>escapeCancelsMode</em> property.
     *
     * @see #setEscapeCancelsDefaultMode(Mode)
     * @see #getEscapeCancelsMode()
     * @see #setEscapeCancelsMode(Mode)
     * @since 1.4
     */
    public static Mode getEscapeCancelsDefaultMode() {
        return escapeCancelsDefaultMode;
    }

    /**
     * Sets the application-wide default for the <em>escapeCancelsMode</em>
     * property. The default value can be overridden per dialog using
     * <code>setEscapeCancelsMode</code>.
     *
     * @param defaultValue   the value to be set as default
     *
     * @see #getEscapeCancelsDefaultMode()
     * @see #getEscapeCancelsMode()
     * @see #setEscapeCancelsMode(Mode)
     * @since 1.4
     */
    public static void setEscapeCancelsDefaultMode(Mode defaultValue) {
        escapeCancelsDefaultMode = defaultValue;
    }


    /**
     * Answers whether Escape cancels this dialog. If so, this dialog's
     * cancel Action will be performed if the Escape key is pressed
     * while any component in this dialog has the focus.<p>
     *
     * The default value for this property can be specified using
     * <code>#setEscapeCancelsDefaultMode</code>.
     *
     * @return the default value for the <em>escapeCancelsMode</em> property.
     *
     * @see #setEscapeCancelsMode(Mode)
     * @see #getEscapeCancelsDefaultMode()
     * @see #setEscapeCancelsDefaultMode(Mode)
     * @since 1.4
     */
    public Mode getEscapeCancelsMode() {
        return escapeCancelsMode;
    }

    /**
     * Specifies whether this dialog will be canceled if Escape is pressed.
     * If the given Mode is enabled in the current environment, this key
     * binding will be added, otherwise it'll be removed. If enabled,
     * the Escape key is mapped to this dialog's root pane using this
     * dialog's cancel Action.<p>
     *
     * The default value for this property can be specified using
     * <code>#setEscapeCancelsDefaultMode</code>.
     *
     * @param escapeCancelsMode  true to bind the Escape key to the cancel action
     *
     * @see #getEscapeCancelsMode()
     * @see #getEscapeCancelsDefaultMode()
     * @see #setEscapeCancelsDefaultMode(Mode)
     * @since 1.4
     */
    public void setEscapeCancelsMode(Mode escapeCancelsMode) {
        Mode oldEscapeCancelsMode = getEscapeCancelsMode();
        if (oldEscapeCancelsMode == escapeCancelsMode)
            return;
        this.escapeCancelsMode = escapeCancelsMode;
        // Register or deregister only if there's a root pane.
        if (getRootPane() != null) {
            if (escapeCancelsMode.enabled())
                registerCancelOnEscapeKey();
            else
                deregisterCancelOnEscapeKey();
        }
        firePropertyChange(
                "escapeCancelsMode",
                oldEscapeCancelsMode,
                getEscapeCancelsMode());
    }


    /**
     * Maps the escape key to this dialog's cancel Action.
     *
     * @see #deregisterCancelOnEscapeKey
     * @see #setEscapeCancelsMode(Mode)
     * @see #setEscapeCancelsDefaultMode(Mode)
     */
    private void registerCancelOnEscapeKey() {
        ComponentUtils.addAction(
                getRootPane(),
                getCancelAction(),
                KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
                JComponent.WHEN_IN_FOCUSED_WINDOW);
    }


    /**
     * Removes the mapping for the escape key to this dialog's cancel Action.
     *
     * @see #registerCancelOnEscapeKey
     * @see #setEscapeCancelsMode(Mode)
     * @see #setEscapeCancelsDefaultMode(Mode)
     */
    private void deregisterCancelOnEscapeKey() {
        ComponentUtils.addAction(
                getRootPane(),
                getCancelAction(),
                KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
                JComponent.WHEN_IN_FOCUSED_WINDOW);
    }


    // The Initially Focused Component ****************************************

    /**
     * Sets a Component that should receive the focus when this dialog is
     * made visible for the first time. Therefore it sets an instance
     * of {@link UIFFocusTraversalPolicy} as this dialog's focus traversal
     * policy, that will return the given component as initial component.<p>
     *
     * Useful if Swing default choice for the initial component as returned
     * by this dialog's FocusTraversalPolicy is not appropriate. For example,
     * if you want to set the initial focus to the first component of the
     * first tab of a dialog's tabbed pane, you can use this method.<p>
     *
     * <strong>Note:</strong> As a side-effect this method changes this
     * dialog's FocusTraversalPolicy to an instance of UIFFocusTraversalPolicy.
     * If you want to set a different FocusTraversalPolicy, do not invoke this
     * method.
     *
     * @param initialComponent  the Component to get the focus if this dialog
     *    becomes visible the first time
     * @see java.awt.FocusTraversalPolicy#getInitialComponent(Window)
     * @see UIFFocusTraversalPolicy#getInitialComponent(Window)
     * @since 1.4
     */
    public void setInitialComponent(Component initialComponent) {
        setFocusTraversalPolicy(new UIFFocusTraversalPolicy(initialComponent));
    }


    // Building ***************************************************************

    /**
     * Builds the dialog's content pane, packs it, sets the resizable property,
     * and locates it on the screen. The dialog is then ready to be opened.<p>
     *
     * Subclasses should rarely override this method.
     */
    protected void build() {
        JComponent content = buildContentPane();

        // Work around the JRE's gray rect problem.
        // TODO: Remove this if we require Java 6.
        if (SystemUtils.IS_JAVA_5) {
            setBackground(content.getBackground());
        }

        resizeHook(content);
        setContentPane(content);
        pack();
        setResizable();
        locateOnScreen();
        if (getEscapeCancelsMode().enabled()) {
            registerCancelOnEscapeKey();
        }
    }

    /**
     * Builds and returns the content pane, sets the border and
     * puts an optional header component in the dialog's north.<p>
     *
     * Subclasses will rarely override this method.
     *
     * @return the dialog's content pane with a border set and
     *     optional header in the north
     */
    protected JComponent buildContentPane() {
        JComponent center = buildContent();
        center.setBorder(getDialogBorder());

        JComponent header = buildHeader();
        if (header == null)
            return center;

        JPanel panel = new JPanel(new BorderLayout());
        panel.add(header, BorderLayout.NORTH);
        panel.add(center, BorderLayout.CENTER);
        return panel;
    }


    /**
     * Builds and returns the optional header component, which will be put in
     * the dialog's north by the default #buildContentPane implementation.
     * By default this method answers {@code null} and so, no header
     * will be used.<p>
     *
     * A typical implementation answers an instance of <code>HeaderPane</code>
     * or a similar component, see the following code example:
     * <pre>
     * protected JComponent buildHeader() {
     *     return new HeaderPanel(
     *         "My Dialog",
     *   	   "You can do click on the tabs below to ...",
     *		   ResourceUtils.getIcon(MyResourceIDs.MY_DIALOG_ICON)
     *		);
     * }
     * </pre>
     *
     * @return the built header component
     */
    protected JComponent buildHeader() {
        return null;
    }


    // Abstract Building Behavior *********************************************

    /**
     * Builds and answers the dialog's main content without header and border.
     * Subclass must override this method. See the class comment for a detailed
     * description how this method is part of the build process.
     *
     * @return the dialog's main content without header and border
     */
    protected abstract JComponent buildContent();


    // Dialog Life Cycle ******************************************************

    /**
     * Builds the dialog content, marks it as not canceled and makes it visible.
     */
    public void open() {
        build();
        canceled = false;
        setVisible(true);
    }


    /**
     * Closes the dialog: makes it invisible and disposes it,
     * which in turn releases all required OS resources.
     */
    public void close() {
        dispose();
    }


    /**
     * Checks and answers whether the dialog has been canceled.
     * This indicator is set in #doAccept and #doCancel.
     *
     * @return true indicates that the dialog has been canceled
     */
    public boolean hasBeenCanceled() {
        return canceled;
    }


    // Configuration **********************************************************

    /**
     * Configures the closing behavior: invokes #doCloseWindow
     * instead of just closing the dialog. This allows subclasses
     * to hook into the close process and perform a custom code sequence.
     */
    protected void configureWindowClosing() {
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowClosingHandler());
    }

    /**
     * Resizes the specified component. This method is called during the build
     * process and enables subclasses to achieve a better aspect ratio,
     * by applying a resizer, e.g. the <code>Resizer</code>.
     *
     * @param component   the component to be resized
     */
    protected void resizeHook(JComponent component) {
        // Do nothing; subclasses will likely override.
    }


    /**
     * TODO: Move this constant to the Forms Borders class.
     */
    protected static final Border DIALOG_BORDER =
        new EmptyBorder(12, 10, 10, 10);

    /**
     * TODO: Move this constant to the Forms Borders class.
     */
    protected static final Border CARD_DIALOG_BORDER =
        new EmptyBorder(10, 6, 7, 6);


    /**
     * Returns the border that will be put around the content, which
     * has been created by #buildContent.<p>
     *
     * Subclasses will typically use <code>DIALOG_BORDER</code> or
     * <code>CARD_DIALOG_BORDER</code>, which is more narrow than the default.
     *
     * @return the dialog's border
     */
    protected Border getDialogBorder() {
        return DIALOG_BORDER;
    }


    /**
     * Sets the dialog's resizable state. By default dialogs are non-resizable;
     * subclasses may override.
     */
    protected void setResizable() {
        setResizable(false);
    }


    /**
     * Locates the dialog on the screen. The default implementation
     * sets the location relative to the parent.<p>
     *
     * Subclasses may choose to center the dialog or put it in a screen corner.
     */
    protected void locateOnScreen() {
        setLocationRelativeTo(getParent());
    }


    // Buffering Support ******************************************************

    /**
     * Creates and returns a {@link BufferedValueModel} on the given ValueModel
     * using the dialog's trigger channel to trigger commit and flush events.
     * This is just a convenience method to minimize the code necessary
     * to buffer underlying models.
     *
     * @param valueModel  the value model to be buffered
     * @return a BufferedValueModel triggered by the dialog's trigger channel
     * @see BufferedValueModel
     * @see ValueModel
     * @see Trigger
     */
    public final BufferedValueModel buffer(ValueModel valueModel) {
        return new BufferedValueModel(valueModel, getTriggerChannel());
    }


    /**
     * Returns a ValueModel that can be used to trigger commit and flush events
     * in BufferedValueModels. The trigger channel's value changes to true
     * in <code>#doApply</code> which is invoked by <code>#doAccept</code>,
     * and it changes to false in <code>#doReset</code> and
     * <code>#doCancel</code>.
     *
     * @return the dialog's trigger channel
     * @see BufferedValueModel
     * @see ValueModel
     * @see #setTriggerChannel(ValueModel)
     */
    public final ValueModel getTriggerChannel() {
        return triggerChannel;
    }

    /**
     * Sets the given ValueModel as the dialog's new trigger channel.
     * Subsequent invocations of <code>#doApply, #doAccept, #doReset</code>
     * and <code>#doCancel</code> will trigger commit and flush events
     * using the new trigger channel.
     *
     * @param newTriggerChannel  the ValueModel to be set as the dialog's trigger channel
     * @see BufferedValueModel
     * @see ValueModel
     * @see #getTriggerChannel()
     * @throws NullPointerException  if the new trigger channel is {@code null}
     */
    public final void setTriggerChannel(ValueModel newTriggerChannel) {
        if (newTriggerChannel == null)
            throw new NullPointerException("The trigger channel must not be null");

        ValueModel oldTriggerChannel = getTriggerChannel();
        triggerChannel = newTriggerChannel;
        firePropertyChange(PROPERTYNAME_TRIGGER_CHANNEL, oldTriggerChannel, newTriggerChannel);
    }


    /**
     * Ensures that the trigger channel changes to true which in turn triggers
     * commit events in all {@link BufferedValueModel} instances that share
     * this trigger. By default this method is invoked in {@link #doApply()}.
     *
     * @see #triggerFlush()
     * @see #doApply()
     */
    protected final void triggerCommit() {
        if (Boolean.TRUE.equals(getTriggerChannel().getValue()))
            getTriggerChannel().setValue(null);
        getTriggerChannel().setValue(Boolean.TRUE);
    }

    /**
     * Ensures that the trigger channel changes to false which in turn triggers
     * flush events in all {@link BufferedValueModel} instances that share
     * this trigger. By default this method is invoked in {@link #doReset()}.
     *
     * @see #triggerCommit()
     * @see #doReset()
     */
    protected final void triggerFlush() {
        if (Boolean.FALSE.equals(getTriggerChannel().getValue()))
            getTriggerChannel().setValue(null);
        getTriggerChannel().setValue(Boolean.FALSE);
    }


    // Common Buttons *******************************************************

    /**
     * Creates and returns an Accept button with the given label text
     * and default state. Clicking this button invokes #doAccept,
     * which by default closes this dialog.<p>
     *
     * While this method allows to use a custom label, method
     * <code>#createOKButton</code> uses a standardized internationalized
     * <i>OK</i> text.<p>
     *
     * Subclasses that use this method to create the dialog OK or Accept button
     * shall ensure that <code>#doAccept</code> performs the appropriate actions.
     *
     * @param text       the button's label text
     * @param isDefault  true to make the button the default button
     * @return an accept button with the given label text and default state
     *
     * @see #doAccept()
     * @see #createOKButton(boolean)
     */
    protected UIFButton createAcceptButton(String text, boolean isDefault) {
        UIFButton button = new UIFButton(new DispatchingAction(ACCEPT_ID, text));
        if (isDefault)
            setDefaultButton(button);
        return button;
    }


    /**
     * Creates and returns an Apply button. Clicking this button invokes
     * #doApply which by default lets the dialog remain open.
     * The button's label text can be localized under key
     * <code>ResourceID.BUTTON_APPLY_TEXT</code>.
     *
     * @return an Apply button
     *
     * @see #doApply()
     * @see #getApplyAction()
     */
    protected UIFButton createApplyButton() {
        return new UIFButton(getApplyAction());
    }


    /**
     * Creates and returns a Cancel button. Clicking this button invokes
     * #doCancel which by default closes the dialog.
     * The button's label text can be localized under key
     * <code>ResourceID.BUTTON_CANCEL_TEXT</code>.<p>
     *
     * The button created by this method has the property
     * <em>verifyInputWhenFocusTarget</em> set to {@code false}.
     *
     * @return a Cancel button
     *
     * @see #doCancel()
     */
    protected UIFButton createCancelButton() {
        UIFButton button = new UIFButton(getCancelAction());
        button.setVerifyInputWhenFocusTarget(false);
        return button;
    }


    /**
     * Creates and returns a Close button. Clicking this button invokes
     * #doClose which by default just closes the dialog.
     * The button's label text can be localized under key
     * <code>ResourceID.BUTTON_CLOSE_TEXT</code>.
     *
     * @param isDefault  true to make this the default button
     * @return a Close button
     *
     * @see #doClose()
     */
    protected UIFButton createCloseButton(boolean isDefault) {
        UIFButton closeButton = new UIFButton(getCloseAction());
        if (isDefault)
            setDefaultButton(closeButton);
        return closeButton;
    }


    /**
     * Creates and returns an OK button. Clicking this button invokes
     * #doAccept which by default performs a #doAccept, sets the
     * canceled marker to false and finally closes the dialog.<p>
     *
     * This method uses #createAcceptButton to create the button,
     * and uses a standardized internationalized <i>OK</i> label text
     * that you can be localize under key
     * <code>ResourceID.BUTTON_OK_TEXT</code>.
     *
     * @param isDefault   true to make this the default button
     * @return an OK button
     *
     * @see #doAccept()
     * @see #doApply()
     */
    protected UIFButton createOKButton(boolean isDefault) {
        UIFButton okButton = new UIFButton(getOKAction());
        if (isDefault)
            setDefaultButton(okButton);
        return okButton;
    }


    /**
     * Creates and returns a Reset button. Clicking this button invokes
     * #doReset which by default lets the dialog remain open.
     * The button's label text can be localized under key
     * <code>ResourceID.BUTTON_RESET_TEXT</code>.
     *
     * @return a Reset button
     * @see #doReset()
     */
    protected UIFButton createResetButton() {
        return new UIFButton(getResetAction());
    }


    /**
     * Sets a button as default button in the dialog's root pane.
     *
     * @param button   the button to be set as default
     */
    protected void setDefaultButton(JButton button) {
        getRootPane().setDefaultButton(button);
    }


    // Common Button Bars ***************************************************

    /**
     * Builds and returns a button bar with a Close button.
     * Uses #createCloseButton to create a standardized button.
     *
     * @return a button bar that contains a Close button
     * @see #createCloseButton(boolean)
     */
    protected JComponent buildButtonBarWithClose() {
        JPanel bar = ButtonBarFactory.buildCloseBar(createCloseButton(true));
        bar.setBorder(Borders.BUTTON_BAR_GAP_BORDER);
        return bar;
    }


    /**
     * Builds and returns a button bar with an OK and a Cancel button.
     * Uses #createOKButton and #createCancelButton to create
     * standardized buttons.
     *
     * @return a button bar that contains an OK and a Cancel button
     * @see #createOKButton(boolean)
     * @see #createCancelButton()
     */
    protected JComponent buildButtonBarWithOKCancel() {
        JPanel bar = ButtonBarFactory.buildOKCancelBar(
            createOKButton(true),
            createCancelButton());
        bar.setBorder(Borders.BUTTON_BAR_GAP_BORDER);
        return bar;
    }


    /**
     * Builds and returns a button bar with three buttons: OK, Cancel and Apply.
     * Uses #createOKButton, #createCancelButton and #createApplyButton
     * to create standardized buttons.
     *
     * @return a button bar that contains: an OK, a Cancel and an Apply button
     * @see #createOKButton(boolean)
     * @see #createCancelButton()
     * @see #createApplyButton()
     */
    protected JComponent buildButtonBarWithOKCancelApply() {
        JPanel bar = ButtonBarFactory.buildOKCancelApplyBar(
            createOKButton(true),
            createCancelButton(),
            createApplyButton());
        bar.setBorder(Borders.BUTTON_BAR_GAP_BORDER);
        return bar;
    }


    // Common Behavior ******************************************************

    /**
     * Invokes <code>#doApply</code>, marks the dialog as uncanceled and
     * finally closes it. This method is intended to apply value changes
     * and then quit the dialog. Typically the core of this action is
     * performed in <code>#doApply</code>.
     *
     * This method is invoked if an <i>Accept</i> or <i>OK</i> button created
     * by <code>#createAcceptButton</code> or <code>createOKButton</code>
     * has been clicked. Such buttons use a DispatchingActionListener
     * that in turn delegates the action perform to this method.
     */
    public void doAccept() {
        doApply();
        canceled = false;
        close();
    }

    /**
     * Applies changes in the dialog and leaves the dialog open.
     * Changes the trigger channel to true, which will commit values
     * buffered by BufferedValueModels that share this dialog's
     * trigger channel.<p>
     *
     * This method is invoked if an <i>Apply</i> button created by
     * <code>#createApplyButton</code> has been clicked.
     * Such buttons use a DispatchingActionListener that in turn
     * delegates the action perform to this method.
     * By default this method is called by <code>#doAccept</code>
     * which is performed when an <i>Accept</i> or <i>OK</i> button
     * has been clicked.<p>
     *
     * Subclasses that override this method shall typically invoke
     * this super method or shall ensure that the trigger is committed.
     *
     * @see #doAccept()
     * @see #doReset()
     * @see #triggerCommit()
     */
    public void doApply() {
        triggerCommit();
    }


    /**
     * Resets changed dialog values, marks the dialog as canceled and closes it.
     *
     * This method is invoked if a <i>Cancel</i> button created by
     * <code>#createCancelButton</code> has been clicked.
     * Such buttons use a DispatchingActionListener that in turn
     * delegates the action perform to this method.
     */
    public void doCancel() {
        doReset();
        canceled = true;
        close();
    }


    /**
     * Marks the dialog as uncanceled and closes it.
     *
     * This method is invoked if a <i>Close</i> button created by
     * <code>#createCloseButton</code> has been clicked.
     * Such buttons use a DispatchingActionListener that in turn
     * delegates the action perform to this method.
     */
    public void doClose() {
        canceled = false;
        close();
    }

    /**
     * Performs the close window action that is invoked if the dialog
     * is closing, for example by clicking the dialog's close button
     * (often in the upper right corner).
     * By default this methods performs a cancel via <code>#doCancel</code>.
     */
    protected void doCloseWindow() {
        doCancel();
    }


    /**
     * Resets changed dialog values and leaves the dialog open.
     * Changes the trigger channel to false, which will flush values
     * buffered by BufferedValueModels that share this dialog's
     * trigger channel.<p>
     *
     * This method is invoked if a <i>Reset</i> button created by
     * <code>#createResetButton</code> has been clicked.
     * Such buttons use a DispatchingActionListener that in turn
     * delegates the action perform to this method.
     * By default this method is called by <code>#doCancel</code>
     * which is performed when a <i>Cancel</i> button has been clicked.<p>
     *
     * Subclasses that override this method shall typically invoke
     * this super method or shall ensure that the trigger is flushed.
     *
     * @see #doAccept()
     * @see #doReset()
     * @see #triggerFlush()
     */
    public void doReset() {
        triggerFlush();
    }




    // Default Button Actions ***********************************************

    /**
     * Lazily creates and returns the Apply Action that invokes #doApply.
     *
     * @return the lazily created Apply Action
     *
     * @see #createApplyButton
     * @see #buildButtonBarWithOKCancelApply()
     */
    public Action getApplyAction() {
        if (applyAction == null) {
            applyAction = new DispatchingAction(
                    APPLY_ID, getString("button.apply.text"));
        }
        return applyAction;
    }

    /**
     * Lazily creates and returns the Cancel Action that invokes #doCancel.
     *
     * @return the lazily created Cancel Action
     *
     * @see #createCancelButton
     * @see #buildButtonBarWithOKCancel
     * @see #buildButtonBarWithOKCancelApply()
     */
    public Action getCancelAction() {
        if (cancelAction == null) {
            cancelAction = new DispatchingAction(
                    CANCEL_ID, getString("button.cancel.text"));
        }
        return cancelAction;
    }


    /**
     * Lazily creates and returns the Close Action that invokes #doClose.
     *
     * @return the lazily created Close Action
     *
     * @see #createCloseButton
     * @see #buildButtonBarWithOKCancelApply()
     */
    public Action getCloseAction() {
        if (closeAction == null) {
            closeAction = new DispatchingAction(
                    CLOSE_ID, getString("button.close.text"));
        }
        return closeAction;
    }


    /**
     * Lazily creates and returns the OK Action that invokes #doAccept.
     *
     * @return the lazily created OK Action
     *
     * @see #createOKButton
     * @see #buildButtonBarWithOKCancel
     * @see #buildButtonBarWithOKCancelApply()
     */
    public Action getOKAction() {
        if (okAction == null) {
            okAction = new DispatchingAction(
                    ACCEPT_ID, getString("button.ok.text"));
        }
        return okAction;
    }


    /**
     * Lazily creates and returns the Reset Action that invokes #doReset.
     *
     * @return the lazily created Reset Action
     *
     * @see #createResetButton
     */
    public Action getResetAction() {
        if (resetAction == null) {
            resetAction = new DispatchingAction(
                    RESET_ID, getString("button.reset.text"));
        }
        return resetAction;
    }


    // Private Helper Class ***************************************************

    /**
     * Invokes #doCloseWindow when the window is about to be closed.
     */
    private final class WindowClosingHandler extends WindowAdapter {

        @Override
        public void windowClosing(WindowEvent e) {
            doCloseWindow();
        }

    }


    // IDs for the DispatchingAction ******************************************

    private static final String ACCEPT_ID = "accept";
    private static final String APPLY_ID  = "apply";
    private static final String CANCEL_ID = "cancel";
    private static final String CLOSE_ID  = "close";
    private static final String RESET_ID  = "reset";


    /**
     * An implementation of the Action interface that dispatches using an ID.
     * Provides the behavior required for the default buttons created above.
     */
    private final class DispatchingAction extends AbstractAction {

        private final String id;

        private DispatchingAction(String id, String name) {
            super(name);
            this.id = id;
        }

        /**
         * Dispatches using this Action's stored id, then delegates to the
         * appropriate dialog method that does the real work.
         */
        public void actionPerformed(ActionEvent evt) {
            if (id.equals(ACCEPT_ID)) {
                doAccept();
            } else if (id.equals(APPLY_ID)) {
                doApply();
            } else if (id.equals(CANCEL_ID)) {
                doCancel();
            } else if (id.equals(CLOSE_ID)) {
                doClose();
            } else if (id.equals(RESET_ID)) {
                doReset();
            } else
                throw new IllegalStateException("Unknown action name" + id);
        }

        @Override
        public String toString() {
            return super.toString() + ": " + id;
        }
    }


}
