/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif;

import java.awt.Frame;
import java.awt.Image;
import java.util.EventObject;
import java.util.MissingResourceException;
import java.util.prefs.Preferences;

import javax.swing.JComponent;
import javax.swing.JFrame;

import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ApplicationContext;
import com.jgoodies.uif.application.ExitListener;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.util.Resizer;
import com.jgoodies.uif.util.ScreenUtils;
import com.jgoodies.uif.util.SystemUtils;
import com.jgoodies.uif.util.WindowUtils;

/**
 * Provides convenience code that is frequently used in application frames:
 * <ul>
 * <li>standardized build process,
 * <li>esthetic height-width ratio,
 * <li>good screen location,
 * <li>frame icons fetched from resource utils,
 * <li>storing and restoring bounds and state,
 * <li>a title composed of prefix and suffix.
 * </ul><p>
 *
 * AbstractFrame registers an ApplicationListener with the UIF's ApplicationContext.
 * Currently the ApplicationContext class keeps an indirect (but hard) reference
 * to AbstractFrames. And so AbstractFrame instances won't be garbage collected
 * before the ApplicationContext closes. A future version of the UIF will register
 * ApplicationContext listeners using a listener list based on WeakReferences.<p>
 *
 * TODO: Register the listener based on WeakReferences.<p>
 *
 * TODO: Remove the above note once the ApplicationContext uses WeakReferences.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.16 $
 *
 * @see com.jgoodies.uif.application.ApplicationContext
 * @see com.jgoodies.uif.util.WindowUtils
 */
public abstract class AbstractFrame extends JFrame {

    // Constants **************************************************************

    private static final String APPLICATION_ICON       = "application.icon";
    private static final String APPLICATION_ICON_12x12 = "application.icon.12x12";
    private static final String APPLICATION_ICON_16x16 = "application.icon.16x16";

    // Instance Fields ********************************************************

    /**
     * Holds an optional suffix for this frame's title. Will be appended
     * to a prefix if the title is set using #setTitlePrefix.
     *
     * @see #setTitlePrefix(String)
     * @see #setTitleSuffix(String)
     */
    private String titleSuffix;


    // Instance Creation ******************************************************

    /**
	 * Constructs an AbstractFrame using the given window title.
     * Also sets the title suffix to the given title.
     *
     * @param title    the window title to be used
	 */
	protected AbstractFrame(String title) {
		super(title);
        setTitleSuffix(title);
	}


	// Building *************************************************************

	/**
	 * Builds the frame's content pane, packs it, locates it on the screen,
	 * sets the icon, restores the saved state, and registers listeners.
	 * It then is ready to be opened.<p>
     *
	 * Subclasses should rarely override this method.
	 */
	public void build() {
		JComponent content = buildContentPane();

        // Work around the JRE's gray rect problem.
        // TODO: Remove this if we require Java 6.
        if (SystemUtils.IS_JAVA_5) {
            setBackground(content.getBackground());
        }

		resizeHook(content);
		setContentPane(content);
		pack();
		locateOnScreen();
		configureWindowIcon();
		restoreState();
		initEventHandling();
        configureCloseOperation();
	}


	// Abstract Behavior ****************************************************

	/**
	 * Subclasses must override this method to build and return the content pane.
     *
     * @return the built content pane
	 */
    protected abstract JComponent buildContentPane();

    /**
     * Configures the behavior that will happen
     * when the user initiates a "close" on this frame.<p>
     *
     * To properly shut down the application, you must invoke
     * <code>ApplicationContext#close</code>. If you just want to shut down
     * you can use a default handler from class <code>ApplicationContext</code>
     * as follows:<pre>
     * protected void configureCloseOperation() {
     *     setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
     *     addWindowListener(
     *         ApplicationContext.getApplicationCloseOnWindowClosingHandler());
     * }
     * </pre>
     *
     * @see JFrame#setDefaultCloseOperation(int)
     * @see Application#getApplicationExitOnWindowClosingHandler()
     */
    protected abstract void configureCloseOperation();


    // Convenience Behavior ***************************************************

    /**
     * Sets this frame's title by means of a prefix. The prefix is ensured
     * to have a length less than 40 characters. The title is then composed
     * of the prefix and the suffix. The suffix will often be the application's
     * name.<p>
     *
     * Example: The suffix is &quot;JDiskReport&quot;, if the prefix is empty,
     * the title is just <pre>JDiskReport</pre>.
     * If the prefix is &quot;Scanning&quot;, the title is
     * <pre>Scanning - JDiskReport</pre>.
     *
     * @param prefix   the window title's prefix
     *
     * @see #setTitleSuffix(String)
     */
    public void setTitlePrefix(String prefix) {
        String trimmedPrefix = prefix.trim();
        if (trimmedPrefix.length() > 40)
            trimmedPrefix = trimmedPrefix.substring(0, 37) + '\u2026';

        String middle = trimmedPrefix.length() == 0 ? "" : " - ";
        setTitle(trimmedPrefix + middle + titleSuffix);
    }

    /**
     * Returns the title's suffix, the part of the title that is
     * appended to the prefix - if any.
     *
     * @return the suffix of this frame's title
     */
    public String getTitleSuffix() {
        return titleSuffix;
    }

    /**
     * Sets this frame's title suffix, which is used to compose this frame's
     * title together with the title prefix.<p>
     *
     * Example: The suffix is &quot;JDiskReport&quot;, if the prefix is empty,
     * the title is just <pre>JDiskReport</pre>.
     * If the prefix is &quot;Scanning&quot;, the title is
     * <pre>Scanning - JDiskReport</pre>.
     *
     * @param newTitleSuffix   the suffix for this frame's title
     *
     * @see #setTitlePrefix(String)
     */
    public void setTitleSuffix(String newTitleSuffix) {
        titleSuffix = newTitleSuffix;
    }


	// Default Behavior *****************************************************

	/**
	 * Makes this frame visible. Allows subclasses to perform additional
     * operations immediately before or after the frame is made visible.
	 */
	public void open() {
		setVisible(true);
	}


	/**
	 * Resizes the specified component. This is called during the #build
	 * process and enables subclasses to achieve a better aspect ratio,
	 * by applying a resizer, e.g. the <code>Resizer</code>.
     *
     * @param component    the component to be resized
	 */
	protected void resizeHook(JComponent component) {
		Resizer.DEFAULT.resizeDialogContent(component);
	}


	/**
	 * Locates this frame on the screen, by default centered on the screen.
	 */
	protected void locateOnScreen() {
		ScreenUtils.locateOnOpticalScreenCenter(this);
	}


	/**
	 * Initializes the event handling. The default implementation
     * listens to frame resize events to ensure a minimum size and
     * to keep track of the stored bounds.<p>
     *
     * Subclasses that override this method will typically
     * invoke this default implementation.
	 */
	protected void initEventHandling() {
        addComponentListener(new WindowUtils.SizeChangeHandler(this));
        ApplicationContext.getInstance().getApplication().addExitListener(new ApplicationExitHandler());
	}


    /**
     * Sets this frame's icon that is fetched via the <code>ResourceUtils</code>.
     * Uses a 12x12 or 16x16 pixel image.
     *
     * @see WindowUtils#setImageIcon(java.awt.Frame, Image, Image)
     */
    protected void configureWindowIcon() {
        Image image12x12;
        Image image16x16;
        ResourceMap map = Application.getResourceMap(getClass());
        try {
            image12x12 = map.getImage(APPLICATION_ICON_12x12);
        } catch (MissingResourceException e) {
            image12x12 = null;
        }
        try {
            image16x16 = map.getImage(APPLICATION_ICON_16x16);
        } catch (MissingResourceException e) {
            image16x16 = map.getImage(APPLICATION_ICON);
        }
       	WindowUtils.setImageIcon(this, image12x12, image16x16);
    }


    // Storing and Restoring Persistent Frame State ***************************

    /**
     * Stores this frame's state in the user preferences
     * as provided by the <code>ApplicationContext</code>.<p>
     *
     * The default implementation does nothing.
     * Since we may add behavior in future versions,
     * subclasses that override this method shall call
     * this super implementation before their added behavior.
     */
    protected void storeState() {
        // The default implementation does nothing.
    }


    /**
     * Restores this frame's state from the user preferences
     * as provided by the <code>ApplicationContext</code>.<p>
     *
     * The default implementation restores the window bounds and a maximized
     * or normal state via the <code>WindowUtils</code>. Also the current bounds
     * are put into this frame's client properties.<p>
     *
     * Subclasses that override this method will likely call this
     * default behavior before adding custom actions.
     *
     * @see WindowUtils#restoreBounds(JFrame, Preferences)
     * @see WindowUtils#restoreState(Frame, Preferences, boolean)
     * @see Frame#getExtendedState()
     */
    protected void restoreState() {
        Preferences userPrefs = ApplicationContext.getInstance().getUserPreferences();
        WindowUtils.restoreBounds(this, userPrefs);
        WindowUtils.restoreState(this, userPrefs, false);
    }


    // Helper Classes *********************************************************

    private final class ApplicationExitHandler implements ExitListener {

        public boolean applicationExitAllowed(EventObject e) {
            return true;
        }

        public void applicationExiting() {
            storeState();
        }
    }


}
