/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 */

package com.jgoodies.uif.action;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import javax.swing.Action;

/**
 * Describes, holds and manages a set of Actions. Useful to group related
 * Actions, for example to enable and disable them all at the same time.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.6 $
 *
 * @see Action
 * @see javax.swing.AbstractAction
 */
public final class ActionSet {

    /**
     * Describes this ActionSet, for example to identify this set while debugging.
     *
     * @see #getName()
     */
    private final String name;

    /**
     * Holds this ActionSet's actions.
     *
     * @see #getActions()
     */
    private final List<Action> actions;


    // Instance Creation ******************************************************

    /**
     * Constructs an <code>ActionSet</code> with the given name and list of
     * <code>Actions</code>.
     *
     * @param name describes this ActionSet
     * @param actions The Actions that make this set
     *
     * @throws NullPointerException if the list of actions is null.
     */
    public ActionSet(String name, List<Action> actions) {
        this.name = name;
        this.actions = actions;
        if (actions == null)
            throw new NullPointerException(
                    "The list of actions must not be null.");
    }


    /**
     * Constructs an <code>ActionSet</code> with the given name and array of
     * <code>Actions</code>.
     *
     * @param name describes this ActionSet
     * @param actions The Actions that make this set
     *
     * @throws NullPointerException if the list of actions is null.
     */
    public ActionSet(String name, Action... actions) {
        this(name, Arrays.asList(actions));
        if (actions == null)
            throw new NullPointerException(
                    "The array of actions must not be null.");
    }


    // Public API ***********************************************************

    /**
     * Returns this set's name.
     *
     * @return this ActionSet's name
     */
    public String getName() {
        return name;
    }


    /**
     * Returns an unmodifiable list of this ActionSet's actions.
     *
     * @return an unmodifiable list of this ActionSet's actions
     */
    public List<Action> getActions() {
        return Collections.unmodifiableList(actions);
    }


    /**
     * Adds an Action to this set of Actions.
     *
     * @param action   the Action to add
     *
     * @throws NullPointerException if the action is null
     */
    public void add(Action action) {
        if (action == null)
            throw new NullPointerException("The action must not be null.");
        actions.add(action);
    }


    /**
     * Removes an Action from this set of Actions.
     *
     * @param action   the Action to remove
     *
     * @throws NullPointerException if the action is null
     */
    public void remove(Action action) {
        if (action == null)
            throw new NullPointerException("The action must not be null.");
        actions.remove(action);
    }


    /**
     * Enables or disables all actions in this set of actions.
     *
     * @param enabled true to enable all actions, false to disable.
     */
    public void setEnabled(boolean enabled) {
        for (Action action : actions) {
            action.setEnabled(enabled);
        }
    }


}
