/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */
package com.jgoodies.uif.action;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ButtonModel;

import com.jgoodies.binding.adapter.RadioButtonAdapter;
import com.jgoodies.binding.adapter.ToggleButtonAdapter;
import com.jgoodies.binding.value.ValueModel;
import com.jgoodies.uif.application.ActionReader;
import com.jgoodies.uif.application.ResourceMap;


/**
 * An Action implementation useful to to create JCheckBoxes, JRadioButtons,
 * JCheckBoxMenuItems and other toggle components. ToggleActions do not
 * perform an action. They provide a toggle state in the ButtonModel and
 * the data required to build toggle components: icon, text, mnemonic, etc.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.6 $
 *
 * @see ButtonModel
 */
public final class ToggleAction extends AbstractAction {

    private static enum Type {RADIO, CHECK}

    private final Type type;
    private final ValueModel subject;
    private final Object selectedValue;
    private final Object deselectedValue;


    // Instance Creation ******************************************************

    /**
     * Constructs a ToggleAction for the given Action name, ResourceMap,
     * Type, subject ValueModel, selected, and deselected value.
     *
     * @param actionName       used to look up action properties in the resource map
     * @param resourceMap      provides the action properties: name, icon, etc.
     * @param type             Type.RADIO or Type.CHECK
     * @param selectedValue   set to the subject ValueModel if selected
     * @param deselectedValue set to the subject ValueModel if deselected
     */
    private ToggleAction(
            String actionName,
            ResourceMap resourceMap,
            Type type,
            ValueModel subject,
            Object selectedValue,
            Object deselectedValue) {
        this.type = type;
        this.subject = subject;
        this.selectedValue = selectedValue;
        this.deselectedValue = deselectedValue;
        ActionReader.readAndPutValues(this, resourceMap, actionName);
    }


    /**
     * Creates and returns a ToggleAction for the given <code>ValueModel</code>.
     * Useful to create JCheckButtons and JCheckButtonMenuItems.
     *
     * @param actionName   used to look up action properties in the resource map
     * @param resourceMap  provides the action properties: name, icon, etc.
     * @param subject   holds the boolean toggle button state
     * @return an action with an associated ButtonModel that reflects
     *     the toggle state of the given subject ValueModel
     */
    public static ToggleAction createCheck(
            String actionName,
            ResourceMap resourceMap,
            ValueModel subject) {
        return new ToggleAction(
                actionName, resourceMap,
                Type.CHECK, subject, Boolean.TRUE, Boolean.FALSE);
    }


    /**
     * Creates and returns a ToggleAction for the given <code>ValueModel</code>.
     * Useful to create JCheckButtons and JCheckButtonMenuItems.
     *
     * @param actionName   used to look up action properties in the resource map
     * @param resourceMap  provides the action properties: name, icon, etc.
     * @param subject   holds the boolean toggle button state
     * @param selectedValue   the subject value that represents the selected state
     * @param deselectedValue the subject value that represents the deselected state
     * @return an action with an associated ButtonModel that reflects
     *     the toggle state of the given subject ValueModel
     */
    public static ToggleAction createCheck(
            String actionName,
            ResourceMap resourceMap,
            ValueModel subject,
            Object selectedValue,
            Object deselectedValue) {
        return new ToggleAction(
                actionName, resourceMap,
                Type.CHECK, subject, selectedValue, deselectedValue);
    }


    /**
     * Creates and returns a ToggleAction for the given <code>ValueModel</code>
     * and choice value. Useful to create JRadioButtons and JRadioButtonMenuItems.
     *
     * @param actionName   used to look up action properties in the resource map
     * @param resourceMap  provides the action properties: name, icon, etc.
     * @param subject   holds the boolean toggle button state
     * @param choice   the subject value that represents the selected state
     * @return an action with an associated ButtonModel that reflects
     *     the toggle state of the given subject ValueModel
     */
    public static ToggleAction createRadio(
            String actionName,
            ResourceMap resourceMap,
            ValueModel subject,
            Object choice) {
        return new ToggleAction(
                actionName, resourceMap,
                Type.RADIO, subject, choice, null);
    }


    // Action API *************************************************************

//    /**
//     * Enables or disables the action. In addition to the superclass behavior,
//     * this method update the enablement of the associated ButtonModel.
//     *
//     * @param enabled  true to enable the action, false to disable it
//     * @see javax.swing.Action#setEnabled(boolean)
//     */
//    @Override
//    public void setEnabled(boolean enabled) {
//        getButtonModel().setEnabled(enabled);
//        super.setEnabled(enabled);
//    }


    /**
     * Does nothing. ToggleActions do not perform an action.
     * They provide a state in the ButtonModel and the data
     * required to build toggle components: icon, text, mnemonic, etc.
     *
     * @param evt   the ignored event
     */
    public void actionPerformed(ActionEvent evt) {
        // Just toggle the model state.
    }


    // Public API ************************************************************

    /**
     * Creates and returns a ButtonModel for this ToggleAction.
     *
     * @return the created ButtonModel
     */
    public ButtonModel createButtonModel() {
        ButtonModel model = isCheckAction()
            ? new ToggleButtonAdapter(subject, selectedValue, deselectedValue)
            : new RadioButtonAdapter (subject, selectedValue);
        Integer mnemonicValue = (Integer) getValue(Action.MNEMONIC_KEY);
        if (mnemonicValue != null) {
            model.setMnemonic(mnemonicValue.intValue());
        }
        model.setEnabled(isEnabled());
        return model;
    }


    /**
     * Checks and answers if this is an action intended for check components.
     * Radio-like actions are used for JRadioButtons and JRadioButtonMenuItems,
     * check-like actions are used for JCheckBoxes and JCheckBoxMenuItems.
     *
     * @return true for check ToggleActions, false otherwise
     */
    public boolean isCheckAction() {
        return type == Type.CHECK;
    }


    /**
     * Checks and answers if this is an action intended for radio components.
     * Radio-like actions are used for JRadioButtons and JRadioButtonMenuItems,
     * check-like actions are used for JCheckBoxes and JCheckBoxMenuItems.
     *
     * @return true for radio ToggleActions, false otherwise
     */
    public boolean isRadioAction() {
        return type == Type.RADIO;
    }


}