/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.application;

import java.awt.Frame;
import java.awt.Window;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.MissingResourceException;
import java.util.logging.FileHandler;
import java.util.logging.LogManager;
import java.util.logging.Logger;

import javax.swing.JFrame;

import com.jgoodies.uif.splash.Splash;
import com.jgoodies.uif.util.SystemUtils;

/**
 * An abstract class that organizes the application startup process.
 * It therefore defines a default startup sequence and provides some
 * convenience code, useful for this task.<p>
 *
 * This class uses the JGoodies foundation classes only. In contrast,
 * the <code>DefaultApplicationStarter</code> uses more of the JGoodies
 * convenience classes.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.24 $
 *
 * @see Application
 * @see	Splash
 *
 * @since 1.8
 */
public abstract class AbstractUIFApplication extends Application {


    // Constants **************************************************************

    /**
     * The MS Windows environment variable for the application data directory.
     */
    private static final String WINDOWS_APPDATA = "APPDATA";

    /**
     * The Mac OS application data subdirectory name.
     */
    private static final String MAC_LIBRARY_APP_SUPPORT =
        "Library/Application Support";


    // Instance Fields ********************************************************

    /**
     * Holds the directory that can be used to store application data.
     * Lazily initialized in {@link #getApplicationDataDirectory()}.
     */
    private File applicationDataDirectory;


    // Default Implementation *************************************************

    /**
     * Does a minimal preparation before running the application load process:
     * initializes fields and catches errors.
     */
    @Override
    protected void startup(String[] args) {
        try {
            Splash.resetStartTime();
            load();
        } finally {
            Splash.ensureClosed();
        }
    }


    @Override
    protected void shutdown() {
        disposeFramesAndWindows();
    }


    /**
     * Loads, configures, initializes, and brings up the application.
     */
    protected void load() {
        // Add some properties to get more precise environment info
        // in case of an error.
        addApplicationProperties();

        // Configure logging, for example set up a file log handler.
        configureLogging();

        configureSplash();
        Splash.open();
        //System.out.println(Splash.elapsedTime());

        configureHelp();

        // Configure all UI related properties: look&feel, system properties, etc.
        Splash.setNote("Configuring UI");
        configureUI();

        launchApplication();
    }


    // Overriding Defaults ****************************************************

    protected void exitSystem() {
        Splash.ensureClosed();
        if (SystemUtils.systemExitAllowed())
            System.exit(0); // exit the application
    }


    // Abstract Behavior ******************************************************

    /**
     * Configures the logging.
     */
    protected abstract void configureLogging();

    /**
     * Configures the splash component.
     */
    protected abstract void configureSplash();


    /**
     * Configures the application wide help.
     */
    protected abstract void configureHelp();


    /**
     * Configures all UI related properties: look&feel, system properties, etc.
     */
    protected abstract void configureUI();


    /**
     * Brings up the application, it therefore initializes the main frame,
     * checks the setup process, initializes all actions, then builds
     * the main frame, and finally opens it.<p>
     *
     * TODO: Consider renaming this method to <code>#createAndShowGUI</code>
     * to make it consistent with the examples used in Kathy's Java Tutorial.
     */
    protected abstract void launchApplication();


    // Standard Directory *****************************************************

    /**
     * Lazily looks up and returns the directory that shall be used
     * to store and retrieve application data. The real work is delegated
     * to {@link #lookupApplicationDataDirectory()} which in turn gets
     * the base dir from {@link #lookupApplicationDataBaseDirectory()}.<p>
     *
     * The default implementation of {@link #lookupApplicationDataDirectory()}
     * requires the following application resource values:
     * {@code application.id} and {@code application.vendor.id}.<p>
     *
     * Examples for the default implementation:
     * <pre>
     * application.id=Skeleton Pro
     * application.vendor.id=JGoodies
     *
     * C:\Documents and Files\Karsten\AppData\JGoodies\Skeleton Pro\                  # XP
     * C:\Dokumente und Einstellungen\Karsten\Anwendungsdaten\JGoodies\Skeleton Pro\  # XP German
     * C:\Users\Karsten\AppData\JGoodies\Skeleton Pro\                                # Vista
     * C:\Users\Karsten\Anwendungsdaten\JGoodies\Skeleton Pro\                        # Vista German
     *
     * /Users/Karsten/Library/Application Support/JGoodies/Skeleton Pro/              # Mac OS X
     *
     * /users/karsten/.jgoodies/skeletonpro/                                          # Solaris, Linux, all other OS
     * </pre>
     *
     * @return the application data directory
     *
     * @since 1.9.1
     */
    public final File getApplicationDataDirectory() {
        if (applicationDataDirectory == null) {
            applicationDataDirectory = lookupApplicationDataDirectory();
        }
        return applicationDataDirectory;
    }


    // Convenience Code *******************************************************

    /**
     * Returns the application's application resource map chain.
     *
     * @return the application's application resource map chain
     * @see ApplicationContext#getResourceMap()
     */
    protected final ResourceMap getAppResourceMap() {
        return ApplicationContext.getInstance().getResourceMap();
    }


    /**
     * Ensures that the parent directory exists for the file
     * associated with the specified logging pattern.
     *
     * @param pattern   a pattern as used with the Java logging
     */
    protected final void ensureParentDirectoryExists(String pattern) {
        File file = generate(pattern);
        File parentDirectory = file.getParentFile();
        if (!parentDirectory.exists())
            parentDirectory.mkdirs();
    }


    /**
     * Disposes all frames and their owned windows.
     */
    protected final void disposeFramesAndWindows() {
        Frame[] frames = Frame.getFrames();
        for (Frame frame : frames) {
            if (frame instanceof JFrame) {
                JFrame jframe = (JFrame) frame;
                Window[] ownedWindows = jframe.getOwnedWindows();
                for (Window element : ownedWindows) {
                    element.dispose();
                }
            }
            frame.dispose();
        }
    }


    /**
     * Looks up and returns the directory for application data such as
     * log files, project data, etc. This default implementation returns a
     * platform-specific value. Subclasses may override.<p>
     *
     * On Windows and Mac the <code>${application.vendor.id}</code> and
     * <code>${application.id}</code> are appended to the application base
     * directory. On all other platforms whitespace characters are stripped
     * from the vendor id and application id, and both are in lower case.
     * Also, the path aims to result in a hidden directory by prepending
     * a dot ('.').
     *
     * Examples:
     * <pre>
     * &lt;basedir&gt;/JGoodies/Skeleton Pro/   # Windows, Mac
     * &lt;basedir&gt;/.jgoodies/skeletonpro/   # Other platforms
     * </pre>
     *
     * @return the path to the application data
     *
     * @throws MissingResourceException if <code>${application.id}</code>
     *     or <code>${application.vendor.id}</code> is undefined
     *
     * @since 1.9.1
     */
    protected File lookupApplicationDataDirectory() {
        File baseDir = lookupApplicationDataBaseDirectory();
        String vendorId = getAppResourceMap().getString("application.vendor.id");
        String appId    = getAppResourceMap().getString("application.id");
        String vendorPath;
        String appPath;
        if (SystemUtils.IS_OS_WINDOWS || SystemUtils.IS_OS_MAC) {
            vendorPath = vendorId;
            appPath    = appId;
        } else {
            vendorPath = "." + vendorId.replaceAll("\\s", "").toLowerCase();
            appPath    = appId.replaceAll("\\s", "").toLowerCase();
        }
        return new File(new File(baseDir, vendorPath), appPath);
    }


    /**
     * Looks up and returns the base directory for application data
     * such as log files,project data, etc. This default implementation returns a
     * platform-specific value. Subclasses may override.<p>
     *
     * On Windows the <tt>%APPDATA%</tt> environment variable is requested
     * from the {@code System}. On the Mac the user home directory is
     * appended by <tt>Libraries/Application Support</tt>. All other platforms
     * just use the user home dir.
     *
     * Examples:
     * <pre>
     * C:\Documents and Files\Karsten\AppData\                 # XP
     * C:\Dokumente und Einstellungen\Karsten\Anwendungsdaten\ # XP German
     * C:\Users\Karsten\AppData\                               # Vista
     * C:\Users\Karsten\Anwendungsdaten\                       # Vista German
     *
     * /users/Karsten/Library/Application Support/             # Mac OS X
     *
     * /users/karsten/                                         # Other OS
     * </pre>
     *
     * @return the base directory for application data
     *
     * @since 1.9.1
     */
    protected File lookupApplicationDataBaseDirectory() {
        if (SystemUtils.IS_OS_WINDOWS) {
            try {
                return new File(System.getenv(WINDOWS_APPDATA));
            } catch (SecurityException e) {
                // Fall back to the user home dir.
            }
        }
        File userHome = new File(System.getProperty("user.home"));
        if (SystemUtils.IS_OS_MAC) {
            return new File(userHome, MAC_LIBRARY_APP_SUPPORT);
        }
        return userHome;
    }


    /**
     * Reads the specified logging configuration properties file and
     * configures the LogManager.
     *
     * @param configurationFileName   a properties file to configure the LogManager.
     *
     * @see LogManager#readConfiguration(InputStream)
     *
     * @since 1.9.1
     */
    protected void configureLoggingFromProperties(String configurationFileName) {
        InputStream in = getClass().getResourceAsStream(configurationFileName);
        if (in == null) {
            throw new IllegalStateException(
                    "Failed to find the logging configuration file: "
                    + configurationFileName);
        }
        try {
            LogManager.getLogManager().readConfiguration(in);
        } catch (SecurityException e) {
            System.err.println("Failed to read the logging properties from file: " + configurationFileName);
            e.printStackTrace();
        } catch (IOException e) {
            System.err.println("Failed to read the logging properties from file: " + configurationFileName);
            e.printStackTrace();
        }
    }


    /**
     * Adds a FileHandler to the root Logger that writes log files with the
     * specified pattern to the given directory.
     *
     * @param logDirectory     the directory where log files shall be located
     * @param logFilePattern   a pattern that shall not contain path information,
     *    for example "console%u.log".
     *
     * @since 1.9.1
     */
    protected void addLogFileHandler(File logDirectory, String logFilePattern) {
        if (!logDirectory.exists() && !logDirectory.mkdirs()) {
            System.err.println("Failed to create the logfile directory: " + logDirectory);
            return;
        }
        try {
            String pattern = logDirectory.getAbsolutePath() + "/" + logFilePattern;
            FileHandler handler = new FileHandler(pattern);
            Logger.getLogger("").addHandler(handler);
        } catch (IOException e) {
            System.err.println("Failed to configure the logging: " + e.getLocalizedMessage());
        }
    }


    // Helper Code ************************************************************

    /**
     * Adds properties about the application and environment that
     * help us identify problems in case of a runtime error.
     */
    private void addApplicationProperties() {
        ResourceMap resourceMap = getAppResourceMap();
        String[] keys = new String[]{
                "application.name",
                "application.vendor",
                "application.vendor.url",
                "application.vendor.mail",
                "application.fullVersion"
        };
        for (String key : keys) {
            System.setProperty(key, resourceMap.getString(key));
        }
    }


    /**
     * Generates a filename from the given logging pattern
     * and creates and returns a File with that name.
     *
     * @param pattern   a pattern as used with the Java logging
     * @return a File named as described by the given logging pattern
     */
    private File generate(String pattern) {
        File file = null;
        String word = "";
        int ix = 0;
        while (ix < pattern.length()) {
            char ch = pattern.charAt(ix);
            ix++;
            char ch2 = 0;
            if (ix < pattern.length()) {
                ch2 = Character.toLowerCase(pattern.charAt(ix));
            }
            if (ch == '/') {
                if (file == null) {
                    file = new File(word);
                } else {
                    file = new File(file, word);
                }
                word = "";
                continue;
            } else  if (ch == '%') {
                if (ch2 == 't') {
                    String tmpDir = System.getProperty("java.io.tmpdir");
                    if (tmpDir == null) {
                        tmpDir = System.getProperty("user.home");
                    }
                    file = new File(tmpDir);
                    ix++;
                    word = "";
                    continue;
                } else if (ch2 == 'h') {
                    file = new File(System.getProperty("user.home"));
                    ix++;
                    word = "";
                    continue;
                } else if (ch2 == 'g') {
                    ix++;
                    continue;
                } else if (ch2 == 'u') {
                    ix++;
                    continue;
                } else if (ch2 == '%') {
                    word = word + "%";
                    ix++;
                    continue;
                }
            }
            word = word + ch;
        }
        if (word.length() > 0) {
            if (file == null) {
                file = new File(word);
            } else {
                file = new File(file, word);
            }
        }
        return file;
    }


}
