/*
 * Copyright (c) 2006-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.application;

import java.util.MissingResourceException;
import java.util.logging.Logger;

import javax.swing.Icon;
import javax.swing.KeyStroke;

import com.jgoodies.uif.util.SystemUtils;

/**
 * Sets Action properties from values that are specified in a ResourceMap.
 * Looks up the action's name, icon, short and long descriptions, mnemonic,
 * and mnemonic index. Values may be specialized for different platforms
 * using a platform suffix. Currently only the <tt>.mac</tt> suffix for
 * Mac OS X is supported.<p>
 *
 * Find below the resource key suffixes used to read values
 * <table border="1">
 * <tr>
 * <td>Suffix</td><td>Value</td><td>Type</td><td>Default</td>
 * </tr>
 * <tr>
 * <td><tt>.label</tt></td><td>name</td><td>mandatory</td><td>none</td>
 * </tr>
 * <tr>
 * <td><tt>.accelerator</tt></td><td>accelerator key</td><td>optional</td><td>none</td>
 * </tr>
 * <tr>
 * <td><tt>.tooltip</tt></td><td>short description</td><td>optional</td><td>name</td>
 * </tr>
 * <tr>
 * <td><tt>.helptext</tt></td><td>long description</td><td>optional</td><td>short description</td>
 * </tr>
 * <tr>
 * <td><tt>.icon</tt></td><td>small icon</td><td>optional</td><td>null</td>
 * </tr>
 * <tr>
 * <td><tt>.largeIcon</tt></td><td>large icon</td><td>optional</td><td>null</td>
 * </tr>
 * </table><p>
 *
 * The action name may contain the mnemonic marker character <tt>&amp;</tt>
 * to indicate the mnemonic and mnemonic index. The &amp; character itself
 * can be described by a double ampersand, <tt>&amp;&amp;</tt>.
 * Find below an example of properties used to describe three actions:
 * <pre>
 * print.label=&Print
 * print.icon=images/print.gif
 *
 * openPrintDialog.label=&Print\u2026
 * openPrintDialog.icon=images/print.gif
 * openPrintDialog.tooltip=Open print dialog
 *
 * openHelpContents.label=&Contents
 * openHelpContents.label.mac=JDiskReport Help
 * openHelpContents.icon=images/help.gif
 * openHelpContents.tooltip=Show Help Contents
 * openHelpContents.helptext=Opens the help viewer and shows the help contents.
 * </pre><p>
 *
 * This class is used by the Actions class to initialize Actions that
 * are looked up from methods annotated with the Action annotation.
 * The icon lookup and conversion from icon paths to icons is done
 * by a ResourceMap.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.7 $
 *
 * @see     javax.swing.Action
 * @see     Action
 * @see     Actions
 * @see     ResourceMap
 *
 * @since 1.9
 */
public final class ActionReader {

    private static final String NAME              = "label";
    private static final char   MNEMONIC_MARKER   = '&';
    private static final String SHORT_DESCRIPTION = "tooltip";
    private static final String LONG_DESCRIPTION  = "helptext";
    private static final String SMALL_ICON        = "icon";
    private static final String LARGE_ICON        = "largeIcon";
    private static final String ACCELERATOR       = "accelerator";

    /**
     * Specifies whether platform-specific resources are enabled or disabled.
     * If enabled, the resource lookup needs more time to search both the
     * platform-specific resource, and the generic fallback.
     */
    private static boolean platformResourcesEnabled = false;

    /**
     * Holds the suffix used for the current operating system,
     * one of: ".linux", ".mac", ".windows", ".solaris".
     */
    private static String SUFFIX_OS_CURRENT;

    static {
        if (SystemUtils.IS_OS_LINUX) {
            SUFFIX_OS_CURRENT = ".linux";
        } else if (SystemUtils.IS_OS_MAC) {
            SUFFIX_OS_CURRENT = ".mac";
        } else if (SystemUtils.IS_OS_WINDOWS) {
            SUFFIX_OS_CURRENT = ".windows";
        } else if (SystemUtils.IS_OS_SOLARIS) {
            SUFFIX_OS_CURRENT = ".solaris";
        }
    }


    private ActionReader() {
        // Suppresses default constructor, prevents instantiation.
    }


    public static boolean getPlatformResourcesEnabled() {
        return platformResourcesEnabled;
    }

    public static void setPlatformResourcesEnabled(boolean b) {
        platformResourcesEnabled = b;
    }

    /**
     * Reads action properties for <code>prefix</code> in <code>map</code>
     * and sets the associated values in the given <code>action</code>.
     *
     * @param action    the Action to be initialized
     * @param map       the ResourceMap used to resolve texts and paths
     * @param prefix    the prefix used for the resource map lookup
     *
     * @throws MissingResourceException if the action name and the action icon
     *    are missing
     */
    public static void readAndPutValues(
        javax.swing.Action action,
        ResourceMap map,
        String prefix) {
        String nameWithMnemonic = getString(map, prefix + "." + NAME, null);
        String name = null;
        Integer mnemonic = null;
        Integer mnemonicIndex = null;
        if (nameWithMnemonic != null) {
            int index = nameWithMnemonic.indexOf(MNEMONIC_MARKER);
            name = stripName(nameWithMnemonic, index);
            mnemonic = stripMnemonic(nameWithMnemonic, index);
            mnemonicIndex = Integer.valueOf(index);
        }

        String shortDescription = getString(
                map,
                prefix + '.' + SHORT_DESCRIPTION,
                null);

        String longDescription = getString(
                map,
                prefix + '.' + LONG_DESCRIPTION,
                null);

        Icon smallIcon = getObject(map, prefix + '.' + SMALL_ICON, Icon.class, null);
        Icon largeIcon = getObject(map, prefix + '.' + LARGE_ICON, Icon.class, smallIcon);

        String shortcut = getString(map, prefix + '.' + ACCELERATOR, null);
        KeyStroke accelerator = getKeyStroke(prefix, shortcut);

        if ((name == null) && (smallIcon == null)) {
            String nameKey = prefix + "." + NAME;
            String iconKey = prefix + "." + SMALL_ICON;
            String baseName = map.getBundleBaseName();
            throw new MissingResourceException(
                    "Missing action name and action icon for bundle " +
                    baseName +
                    "\nname key=" + nameKey +
                    "\nicon key=" + iconKey,
                    baseName,
                    nameKey);
        }

        // Put the ActionReader's properties as values in the Action.
        action.putValue(javax.swing.Action.NAME,                 name);
        action.putValue(javax.swing.Action.SHORT_DESCRIPTION,    shortDescription);
        action.putValue(javax.swing.Action.LONG_DESCRIPTION,     longDescription);
        action.putValue(javax.swing.Action.SMALL_ICON,           smallIcon);
        action.putValue(ActionKeys.LARGE_ICON_KEY,               largeIcon);
        action.putValue(javax.swing.Action.ACCELERATOR_KEY,      accelerator);
        action.putValue(ActionKeys.DISPLAYED_MNEMONIC_INDEX_KEY, mnemonicIndex);

        /*
         * The following conditional works around a bug in AbstractButton
         * that can configure itself from an Action with mnemonic set to null
         * but throws an NPE if the Action's mnemonic becomes null afterwards.
         *
         * @see AbstractButton.ButtonActionPropertyChangeListener
         */
        if (mnemonic != null) {
            action.putValue(javax.swing.Action.MNEMONIC_KEY, mnemonic);
        }
    }

    private static String stripName(String nameWithMnemonic, int aMnemonicIndex) {
        return aMnemonicIndex == -1
            ? nameWithMnemonic
            : nameWithMnemonic.substring(0, aMnemonicIndex)
                + nameWithMnemonic.substring(aMnemonicIndex + 1);
    }

    private static Integer stripMnemonic(String nameWithMnemonic, int aMnemonicIndex) {
        return aMnemonicIndex == -1
            ? null
            : Integer.valueOf(Character.toUpperCase(
                    nameWithMnemonic.charAt(aMnemonicIndex + 1)));
    }


    private static KeyStroke getKeyStroke(String aPrefix, String acceleratorString) {
        if (acceleratorString == null)
            return null;

        KeyStroke keyStroke = KeyStroke.getKeyStroke(acceleratorString);
        if (keyStroke == null)
            Logger.getLogger("ActionReader").warning(
                "Action "
                    + aPrefix
                    + " has an invalid accelerator "
                    + acceleratorString);
        return keyStroke;
    }


    /**
     * Looks up and returns a string value from the given resource map
     * for the specified key. The lookup is performed in two steps:
     * first an operating system specific key is used.
     * If that succeeds, we're done.
     * If it fails, the generic key will be tried.
     * If this resource is missing too, the default string is returned.
     *
     * @param map             the resource map used for the lookup
     * @param key             the generic key
     * @param defaultString   used if no value is found
     * @return the platform specific string, generic string, or default string
     */
    private static String getString(
        ResourceMap map,
        String key,
        String defaultString) {
        return getObject(map, key, String.class, defaultString);
    }

    /**
     * Looks up and returns a value from the given resource map
     * for the specified key. The lookup is performed in two steps:
     * first an operating system specific key is used.
     * If that succeeds, we're done.
     * If it fails, the generic key will be tried.
     * If this resource is missing too, the default value is returned.
     *
     * @param map             the resource map used for the lookup
     * @param key             the generic key
     * @param defaultValue   used if no value is found
     * @return the platform specific value, generic value, or default value
     */
    private static <T> T getObject(
        ResourceMap map,
        String key,
        Class<T> type,
        T defaultValue) {
        if (!platformResourcesEnabled) {
            return getGenericObject(map, key, type, defaultValue);
        }
        try {
            return map.getObject(key + SUFFIX_OS_CURRENT, type);
        } catch (MissingResourceException e) {
            return getGenericObject(map, key, type, defaultValue);
        }
    }

    private static <T> T getGenericObject(ResourceMap map, String key, Class<T> type, T defaultValue) {
        try {
            return map.getObject(key, type);
        } catch (MissingResourceException e) {
            return defaultValue;
        }
    }


}
