/*
 * Copyright (c) 2006-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.application;

import java.awt.event.ActionEvent;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;


/**
 * An internal app framework class that looks up and implements Actions
 * for methods that are annotated with the <code>@Action</code> annotation.
 * Initializes these actions from  resource maps, and checks for valid
 * action method signatures.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.11 $
 *
 * @see ActionReader
 */
final class Actions {

    private static Logger logger = Logger.getLogger(Actions.class.getName());


    // Instance Creation ******************************************************

    private Actions() {
        // Overrides default constructor; prevents instantiation.
    }


    // Convenience Methods ****************************************************

    static ActionMap lookupActionMap(Object target, ResourceMap customResourceMap) {
        ApplicationActionMap map = new ApplicationActionMap(target, customResourceMap);
        // Used to identify duplicate action method names.
        Map<String, Method> actionMethods = new HashMap<String, Method>();

        for (Method method : target.getClass().getMethods()) {
            Action annotation = method.getAnnotation(Action.class);
            if (annotation == null) {
                continue;
            }
            String actionName = annotation.name().length() == 0
                ? method.getName()
                : annotation.name();
            ensureValidTypes(method);
            Method oldMethod = actionMethods.put(actionName, method);
            if (oldMethod != null) {
                throw new ActionLookupException("Duplicate action method for name \"" + actionName + "\"."
                        + "\n" + oldMethod
                        + "\n" + method
                        );
            }
            map.addAction(method, actionName, annotation);
        }
        return map;
    }


    private static void ensureValidTypes(Method method) {
        Class<?> returnType = method.getReturnType();
        if (returnType != Void.TYPE) {
            throw new IllegalArgumentException("Methods annotated with @Action must return void.");
        }
        Class<?>[] parameterTypes = method.getParameterTypes();
        if (    (parameterTypes.length > 1)
            || (    (parameterTypes.length == 1)
                 && (parameterTypes[0] != ActionEvent.class))) {
            throw new IllegalArgumentException("Methods annotated with @Action must have no parameter or a single ActionEvent.");
        }
    }


    // Helper Classes *********************************************************

    static final class ApplicationActionMap extends ActionMap {

        final Object target;
        private final ResourceMap targetResourceMap;
        private final ResourceMap customResourceMap;

        ApplicationActionMap(Object target, ResourceMap customResourceMap) {
            this.target = target;
            this.targetResourceMap = Application.getResourceMap(target.getClass());
            this.customResourceMap = customResourceMap;
        }

        void addAction(
                Method method,
                String actionName,
                Action annotation) {
            Class<?> targetClass = target.getClass();
            Class<?> declaringClass = method.getDeclaringClass();
            boolean hasCustomResourceMap = customResourceMap != null;
            ResourceMap resourceMap;
            if (hasCustomResourceMap) {
                resourceMap = customResourceMap;
            } else if (declaringClass == targetClass) {
                resourceMap = targetResourceMap;
            } else {
                resourceMap =
                    Application.getResourceMap(declaringClass);
            }
            javax.swing.Action action = new ApplicationAction(this, method);
            action.setEnabled(annotation.enabled());
            // action.putValue(javax.swing.Action.NAME, actionName);
            readAndPutValues(action, actionName, resourceMap, customResourceMap,
                    target, method);

            super.put(actionName, action);
        }


        private static void readAndPutValues(
                javax.swing.Action action,
                String actionName,
                ResourceMap resourceMap,
                ResourceMap customResourceMap,
                Object target,
                Method method) {
            try {
                ActionReader.readAndPutValues(action, resourceMap, actionName);
            } catch (MissingResourceException e) {
                String message = "Missing action name resource." +
                    "\nAction name=" + actionName +
                    "\nMethod name=" + method.getName() +
                    "\nTarget class   =" + target.getClass().getName() +
                    "\nDeclaring class=" + method.getDeclaringClass().getName();
                if (customResourceMap != null) {
                    message = message +
                        "\nCustom resource map bundle=" + customResourceMap.getBundleBaseName();
                }
                throw new ActionReadException(message, e);
            }
        }


        // Disabling Superclass Behavior **************************************

        @Override
        public void clear() {
            throw new UnsupportedOperationException(
                    "Clearing an ApplicationActionMap is prohibited.");
        }

        @Override
        public void put(Object key, javax.swing.Action action) {
            throw new UnsupportedOperationException(
                    "Putting actions to an ApplicationActionMap is prohibited.");
        }

        @Override
        public void remove(Object key) {
            throw new UnsupportedOperationException(
                    "Removing actions from an ApplicationActionMap is prohibited.");
        }

    }


    /**
     * An Action implementation that invokes a method that is annotated with
     * the <code>@Action</code> Annotation. If the action method returns a Tasks
     * it is executed.
     */
    static final class ApplicationAction extends AbstractAction {

        private static final String INVOCATION_FAILED_FORMAT =
            "Action method invocation failed.\nClass=%1$s\nObject=%2$s\nMethod=%3$s";

        private final ApplicationActionMap actionMap;
        private final Method method;

        ApplicationAction(ApplicationActionMap actionMap, Method method) {
            this.actionMap = actionMap;
            this.method = method;
        }

        public void actionPerformed(ActionEvent e) {
            if (!isEnabled()) {
                throw new IllegalStateException(
                        "You must not perform disabled Actions.");
            }
            invokeMethod(actionMap.target, method, e);
        }

        /**
         * Invokes an action method with a given ActionEvent on a given object.
         * If an InvocationTargetExceptions is caught, the cause is thrown
         * if it is a runtime exception, otherwise the cause will be wrapped
         * in a runtime exception.
         *
         * @param object    the object the method is invoked from
         * @param m    the action method to invoke
         * @param e         the ActionEvent
         * @return a worker
         */
        private Object invokeMethod(Object object, Method m, ActionEvent e) {
            try {
                Object[] arguments = (m.getParameterTypes().length == 0)
                    ? null
                    : new Object[] {e};
                return m.invoke(object, arguments);
            } catch (IllegalArgumentException e1) {
                // This should not happen, because valid types and
                // parameters have been checked in #ensureValidTypes.
                logger.log(Level.SEVERE, methodInvocationFailureString(object, m), e1);
                throw new RuntimeException(methodInvocationFailureString(object, m), e1);
            } catch (IllegalAccessException e1) {
                // This should not happen, because the method
                // has been made accessible before the invocation.
                logger.log(Level.SEVERE, methodInvocationFailureString(object, m), e1);
                throw new RuntimeException(methodInvocationFailureString(object, m), e1);
            } catch (InvocationTargetException e1) {
                logger.log(Level.SEVERE, methodInvocationFailureString(object, m), e1);
                Throwable cause = e1.getCause();
                if (cause instanceof Error) {
                    throw (Error) cause;
                }
                if (cause instanceof RuntimeException) {
                    throw (RuntimeException) cause;
                }
                throw new RuntimeException(methodInvocationFailureString(object, m), cause);
            }
        }

        private String methodInvocationFailureString(Object t, Method m) {
            Class<?> targetClass = t != null ? t.getClass() : null;
            return String.format(
                    INVOCATION_FAILED_FORMAT,
                    targetClass, t, m);
        }


    }


    // Exception **************************************************************

    /**
     * A runtime exception used to describe action method lookup failures.
     */
    static final class ActionLookupException extends RuntimeException {

        // Instance Creation ******************************************************

        /**
         * Constructs a new exception instance with the specified detail message
         * and cause.
         *
         * @param message the detail message (which is saved for later retrieval by
         *        the {@link #getMessage()}method).
         */
        ActionLookupException(String message) {
            super(message);
        }

    }


    /**
     * A runtime exception used to describe ActionReader failures.
     */
    static final class ActionReadException extends RuntimeException {

        // Instance Creation ******************************************************

        /**
         * Constructs a new exception instance with the specified detail message
         * and cause.
         *
         * @param message   the detail message which is saved for later
         *        retrieval by the {@link #getMessage()} method.
         * @param cause     the cause which is saved for later retrieval by the
         *        {@link #getCause()} method. A {@code null} value is permitted,
         *        and indicates that the cause is nonexistent or unknown.
         */
        ActionReadException(String message, Throwable cause) {
            super(message, cause);
        }

    }


}
