/*
 * Copyright (c) 2006-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.application;

import java.awt.EventQueue;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.EventObject;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.Action;
import javax.swing.ActionMap;

import com.jgoodies.binding.beans.Model;

/**
 * A base class for Swing applications. It addresses the following issues:
 * <ul>
 * <li>Application startup. Launches an app in the Event-Dispatch-Thread (EDT).
 * <li>Application shutdown. Asks registered ExitListeners whether
 *     the application may exit or not, and notifies these listeners
 *     if the application is exiting.
 * </ul><p>
 *
 * TODO: Consider registering the application as UncaughtExceptionHandler.
 * If an uncaught exception is encountered, the application shall be able
 * to handle it, for example it could show a message dialog or shut down
 * the application.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.22 $
 *
 * @since 1.8
 */
public abstract class Application extends Model {

    private static final Logger LOGGER = Logger.getLogger(Application.class.getName());

    /**
     * Holds the list of ShutdownListener instances that observe
     * changes in the application state.
     *
     * @see #addExitListener(ExitListener)
     * @see #removeExitListener(ExitListener)
     * @see #exitAllowed(EventObject)
     * @see #fireApplicationExiting()
     */
    private final List<ExitListener> listenerList;


    // Instance Creation ******************************************************

    protected Application() {
        listenerList = new CopyOnWriteArrayList<ExitListener>();
    }


    // Convenience Code *******************************************************

    /**
     * Returns the action named {@code actionName} of the given target object.
     * Equivalent to:
     * {@code Application.getActionMap(target).get(actionName)}.<p>
     *
     * <strong>Note:</strong> Requesting an Action will create
     * a hard reference to the given target, so it cannot be cleared
     * by the garbage collector until the ActionMap entry is cleared
     * using {@link #clearActionMap(Object)}.
     *
     * @param target   the instance that will perform the Action
     * @param actionName  used to look up the associated Action method
     * @return the Action for name {@code actionName} of the given target.
     *
     * @throws IllegalArgumentException if the action is absent.
     *
     * @since 1.9.1
     */
    public static Action getAction(Object target, String actionName) {
        Action action = Application.getActionMap(target).get(actionName);
        if (action == null) {
            throw new IllegalArgumentException("No action found for name " + actionName);
        }
        return action;
    }


    /**
     * Returns the ActionMap for the given target. Equivalent to:
     * {@code ApplicationContext.getInstance().getActionMap(target)}.<p>
     *
     * <strong>Note:</strong> Requesting an ActionMap will create
     * a hard reference to the given target, so it cannot be cleared
     * by the garbage collector until the ActionMap entry is cleared
     * using {@link #clearActionMap(Object)}.
     *
     * @param target  the object that shall execute the Actions
     * @return an ActionMap with Actions that perform the target's
     *    action methods.
     * @see ApplicationContext#getActionMap(Object)
     *
     * @since 1.9.1
     */
    public static ActionMap getActionMap(Object target) {
        return ApplicationContext.getInstance().getActionMap(target);
    }


    /**
     * Clears the ActionMap cache entry for the given target.
     * Used to avoid memory leaks when the target isn't required
     * anymore.
     *
     * @param target  the key in the ActionMap cache
     *
     * @throws NullPointerException if {@code target} is {@code null}
     *
     * @since 1.9.1
     */
    public static void clearActionMap(Object target) {
        ApplicationContext.getInstance().clearActionMap(target);
    }


    /**
     * Returns the ResourceMap for the given type. Equivalent to:
     * {@code ApplicationContext.getInstance().getResourceMap(type)}.
     *
     * @param type   a class to lookup resources for
     * @return the type's class or package resource map - if available and
     *     {@code type} is not {@code null} - otherwise this application's
     *     resource map
     * @see ApplicationContext#getResourceMap(Class)
     */
    public static ResourceMap getResourceMap(Class<?> type) {
        return ApplicationContext.getInstance().getResourceMap(type);
    }


    // Lifecycle **************************************************************

    /**
     * Instantiates the given Application class, initializes
     * the ApplicationContext by setting the Application instance,
     * then invokes {@code #startup} with the given arguments.
     * Typically this method is called from an application's #main method.
     *
     * @param appClass the class of the application to launch
     * @param args optional launch arguments, often the the main method's arguments.
     *
     * @see #shutdown()
     * @see ApplicationContext#setApplication(Application)
     */
    public static synchronized void launch(final Class<? extends Application> appClass, final String... args) {
        Runnable runnable = new Runnable() {
            public void run() {
                Application application = null;
                try {
                    application = appClass.newInstance();
                } catch (InstantiationException e) {
                    e.printStackTrace();
                    LOGGER.log(Level.SEVERE, "Can't instantiate " + appClass, e);
                    return;
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                    LOGGER.log(Level.SEVERE, "Illegal Access during launch of " + appClass, e);
                    return;
                }
                try {
                    ApplicationContext.getInstance().setApplication(application);
                    application.startup(args);
                } catch (Exception e) {
                    LOGGER.log(Level.SEVERE, "Failed to launch " + appClass, e);
                }
            }
        };
        if (EventQueue.isDispatchThread()) {
            runnable.run();
        } else {
            EventQueue.invokeLater(runnable);
        }
    }


//    /**
//     * Initializes the application before the GUI is constructed by
//     * {@code startup}. This method runs on the event dispatching thread.<p>
//     *
//     * Subclasses that want to initialize something before the startup is
//     * performed, shall override this method. This default implementation
//     * does nothing.
//     *
//     * @param args optional launch arguments, often the main method's arguments.
//     *
//     * @see #launch(Class, String[])
//     * @see #startup()
//     * @see #shutdown()
//     */
//    protected void initialize(String[] args) {
//        // Does nothing; subclasses may override.
//    }


    /**
     * Starts this application when the application is launched.
     * A typical application creates and shows the GUI in this method.
     * This method runs on the event dispatching thread.<p>
     *
     * Called by the static {@code launch} method.
     *
     * @see #launch(Class, String[])
     * @see #shutdown()
     */
    protected abstract void startup(String[] args);


    /**
     * Performs cleanup tasks when the application exits.
     * This default implementation does nothing; subclasses may override.
     * This method runs on the event dispatching thread.
     *
     * @see #startup(String[])
     * @see #exit(EventObject)
     * @see #addExitListener(ExitListener)
     */
    protected void shutdown() {
        // This default implementation does nothing.
    }


    /**
     * Shuts down this application by calling {@code exit(null)}.
     * This version of #exit() is intended if the application exit
     * wasn't triggered by an event. Consequently exit listeners
     * lack the optional event object useful to open dialogs at
     * the right position or screen.
     *
     * @see #exit(EventObject)
     * @see #shutdown()
     * @see #addExitListener(ExitListener)
     * @see #removeExitListener(ExitListener)
     */
    public final void exit() {
        exit(null);
    }


    /**
     * Notifies registered exit listeners that the application is exiting,
     * then shuts down the application by calling {@code shutdown()}.
     * If an exit listener vetoes against the exit, this method does nothing.<p>
     *
     * Exceptions thrown during the exit listener notification and
     * during the shutdown are logged only.<p>
     *
     * The optional event object is useful for exit listeners that
     * show a dialog; these can use the event object to determine
     * on which screen the dialog shall pop up.
     *
     * @param event an optional EventObject that triggered the shutdown
     *
     * @see #exit()
     * @see #shutdown()
     * @see #addExitListener(ExitListener)
     * @see #removeExitListener(ExitListener)
     */
    public void exit(EventObject event) {
        if (!exitAllowed(event)) {
            return;
        }
        try {
            fireApplicationExiting();
        } catch (Exception e) {
            LOGGER.log(Level.WARNING, "Failed to notify listeners about the application exit.");
        }
        try {
            shutdown();
        } catch (Exception e) {
            LOGGER.log(Level.SEVERE, "Failed to shutdown the application.");
        } finally {
            end();
        }
    }


    /**
     * Called by the {@link #exit(EventObject)} default implementation
     * to terminate the application. This default {@code end} implementation
     * just invokes {@code System.exit(0)}, which terminates the currently
     * running Java Virtual Machine.<p>
     *
     * Subclasses may want to override this default behavior to enable
     * multiple Applications running in a JVM, or to let the JVM continue
     * to run after an Application end, for example if the Application is
     * an IDE plug-in.
     *
     * @see #exit(EventObject)
     *
     * @since 1.9.1
     */
    protected void end() {
        System.exit(0);
    }


    /**
     * Checks and answers if the application is allowed to exit.
     * Notifies all exit listeners so they can veto against exiting.
     *
     * @param event an optional event that provides the (event) source
     *    that originated the application exit, useful when opening
     *    a dialog that asks the user whether the exit is allowed or not
     * @return true if no one vetoed against the exit, false if someone
     *    vetoed against the exit
     */
    public final boolean exitAllowed(EventObject event) {
        for (ExitListener listener : listenerList) {
            if (!listener.applicationExitAllowed(event))
                return false;
        }
        return true;
    }


    // Managing Exit Listeners ************************************************

    /**
     * Adds an <code>ExitListener</code> that observes
     * changes in the application state.
     *
     * @param listener  the <code>ExitListener</code> to add
     * @throws IllegalArgumentException if the listener is null
     */
    public final void addExitListener(ExitListener listener) {
        if (listener == null)
            throw new IllegalArgumentException("The exit listener must not be null.");

        listenerList.add(listener);
    }


    /**
     * Removes the given <code>ExitListener</code>.
     *
     * @param listener  the <code>ExitListener</code> to remove
     * @throws IllegalArgumentException if the listener is null
     */
    public final void removeExitListener(ExitListener listener) {
        if (listener == null)
            throw new IllegalArgumentException("The exit listener must not be null.");

        listenerList.remove(listener);
    }


    /**
     * Fires an event that indicates that the application is about to exit.
     * Allows interested parties to perform operations before the shutdown.
     */
    protected final void fireApplicationExiting() {
        for (ExitListener listener : listenerList) {
            listener.applicationExiting();
        }
    }


    // Window Closing ********************************************************

    private WindowListener applicationShutdownOnWindowClosingHandler;

    /**
     * Lazily creates and returns a handler that invokes
     * <code>Application#exit()</code> on the <em>window closing</em> event.
     *
     * @return a handler that exits the application on the
     *     <em>window closing</em> event.
     */
    public final WindowListener getApplicationExitOnWindowClosingHandler() {
        if (applicationShutdownOnWindowClosingHandler == null) {
            applicationShutdownOnWindowClosingHandler =
                new ApplicationExitOnWindowClosingHandler();
        }
        return applicationShutdownOnWindowClosingHandler;
    }


    // Helper Class ***********************************************************

    /**
     * Exits the application on window closing.
     */
    private static final class ApplicationExitOnWindowClosingHandler extends WindowAdapter {

        @Override
        public void windowClosing(WindowEvent e) {
            ApplicationContext.getInstance().getApplication().exit(e);
        }

    }


}
