/*
 * Copyright (c) 2006-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.application;

import java.util.Collections;
import java.util.Locale;
import java.util.Map;
import java.util.WeakHashMap;
import java.util.logging.Logger;
import java.util.prefs.Preferences;

import javax.swing.ActionMap;

import com.jgoodies.binding.beans.Model;

/**
 * Provides shared access for Swing application data.
 * Addresses the following topics:
 * <ul>
 * <li>Resource management. Returns ResourceMaps per class, package, and application.
 *     These resource maps add type conversions to property resource bundles.
 *     So the app can request icons, colors, fonts, and other types
 *     frequently used in desktop applications.
 * <li>Action management. Returns ActionMaps per target object, where the
 *     Actions are lookup from methods annotated with the <code>@Action</code>
 *     annotation.
 * <li>Preferences access. Returns a preferences node per class that can
 *     be used to persist application state, for example window bounds.
 * </ul><p>
 *
 * TODO: Consider re-reading action values to enable users to switch
 * the Locale and update action views automatically. The current code
 * just invalidates the ActionMap cache and requires to rebuild the UI.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.14 $
 */
public final class ApplicationContext extends Model {

    private static Logger logger = Logger.getLogger(ApplicationContext.class.getName());

    private static ApplicationContext instance;

    // Resources --------------------------------------------------------------

    /**
     * Holds the class-level ResourceMaps that have been requested.
     */
    private final Map<Class<?>, ResourceMap> classResourceMapCache;

    /**
     * Holds the ResourceMaps that have been requested for a package.
     */
    private final Map<Package, ResourceMap> packageResourceMapCache;

    /**
     * This application's top-level resource map,
     * the parent of all package-level resource maps.
     */
    private ResourceMap applicationResourceMap;


    // Actions ----------------------------------------------------------------

    private final Map<Object, ActionMap> actionMapCache;

    /**
     * Holds the Locale that was used while caching ActionMaps.
     * If the current default locale differs from this stored locale,
     * the cache is invalid.
     */
    private Locale cachedLocale;


    // Instance Creation ******************************************************

    private ApplicationContext() {
        classResourceMapCache =
            Collections.synchronizedMap(new WeakHashMap<Class<?>, ResourceMap>());
        packageResourceMapCache =
            Collections.synchronizedMap(new WeakHashMap<Package, ResourceMap>());
        actionMapCache =
            Collections.synchronizedMap(new WeakHashMap<Object, ActionMap>());
    }


    /**
     * Lazily creates and returns the single instance of this application
     * context.
     *
     * @return the single instance
     */
    public static synchronized ApplicationContext getInstance() {
        if (instance == null) {
            instance = new ApplicationContext();
        }
        return instance;
    }


    // Application Management *************************************************

    private Class<? extends Application> applicationClass;

    private Application application;


    /**
     * Returns the class of this context's application.
     * Used by the resource management to look up bundles
     * that are related to the application class.
     *
     * @return the class of this context's application
     */
    public Class<? extends Application> getApplicationClass() {
        return applicationClass;
    }


    private void setApplicationClass(Class<? extends Application> applicationClass) {
        this.applicationClass = applicationClass;
    }


    /**
     * Returns this context's Application.
     *
     * @return this context's Application
     */
    public Application getApplication() {
        return application;
    }


    synchronized void setApplication(Application application) {
        if (application == null) {
            throw new IllegalArgumentException("The application must not be null.");
        }
        if (getApplication() != null) {
            throw new IllegalStateException("The application has already been launched.");
        }
        this.application = application;
        applicationResourceMap = null;
        classResourceMapCache.clear();
        packageResourceMapCache.clear();
        setApplicationClass(application.getClass());
    }


    // Resources **************************************************************

    /**
     * Lazily creates and returns the application-level ResourceMap.
     * An Application <tt>mypackage.MyApplication</tt> must provide
     * a resource bundle named
     * <tt>mypackage.MyApplication.resources.MyApplication</tt>.
     *
     * @return the application-wide ResourceMap
     */
    public ResourceMap getResourceMap() {
        if (applicationResourceMap == null) {
            applicationResourceMap = createApplicationResourceMap();
        }
        return applicationResourceMap;
    }


    /**
     * Looks up and returns a ResourceMap for the given class.
     * It looks for a class-level, package-level and application-level
     * ResourceBundle to build a hierarchy of ResourceMaps.
     * This hierarchy is used for a fallback mechanism during
     * the ResourceMap resource lookup.<p>
     *
     * The class-level and package-level ResourceBundles are optional,
     * the application-level bundle is mandatory. The application-level
     * ResourceMap is the parent of all package-level ResourceMaps.
     * A package-level ResourceMap - if any - is the parent of the class-level
     * ResourceMaps for all classes in that package. If the package-level
     * bundle is absent, the application-wide ResourceMap is used as
     * direct parent for the class-level ResourceMap - if any.<p>
     *
     * For class <tt>mypackage.MyClass</tt> and Application
     * <tt>apppackage.MyApplication</tt> the following resource bundles
     * are searched to build ResourceMaps:
     * <tt>mypackage.resources.MyClass</tt> (optional class-level),
     * <tt>mypackage.resources.Package</tt> (optional package-level),
     * <tt>apppackage.resources.MyApplication</tt> (mandatory
     * application-level).<p>
     *
     * When looking for a resource, the ResourceMap first uses its own
     * ResourceBundle. In case the resource is missing there, the lookup
     * continues with the ResourceMap's parent ResourceMap.
     * A resource lookup fails, if all ResourceMaps in this path
     * miss the resource.
     *
     * @param type   a class to lookup resources for
     * @return the type's package resource map - if available and <code>type</code>
     *     is not {@code null}, otherwise this application's resource map
     *
     * @see ResourceMap
     */
    public ResourceMap getResourceMap(Class<?> type) {
        if (type == null) {
            return getResourceMap();
        }
        ResourceMap resourceMap = classResourceMapCache.get(type);
        if (resourceMap != null) {
            logger.fine("Class ResourceMap cache hit: " + type);
            return resourceMap;
        }
        resourceMap = new ResourceMap(
                getResourceMap(type.getPackage(), type),
                type);
        classResourceMapCache.put(type, resourceMap);
        return resourceMap;
   }


    private ResourceMap getResourceMap(Package aPackage, Class<?> type) {
        ResourceMap resourceMap = packageResourceMapCache.get(aPackage);
        if (resourceMap != null) {
            logger.fine("Package ResourceMap cache hit: " + aPackage);
            return resourceMap;
        }
        resourceMap = new ResourceMap(
                getResourceMap(),
                type.getClassLoader(),
                packageResourceBundleBaseName(type));
        packageResourceMapCache.put(aPackage, resourceMap);
        return resourceMap;
    }


    /**
     * Creates and returns the application-level ResourceMap.
     * This implementation creates a ResourceMap chain that starts with
     * the launched Application class and stops at the Application class.
     * Each resource maps parent is the resource map of its superclass.<p>
     *
     * If the application has not been launched, this methods returns
     * the resource map for Application.class.<p>
     *
     * Subclasses may override this method to set a parent ResourceMap
     * that is shared by multiple applications.
     *
     * @return the application-level ResourceMap chain
     */
    protected ResourceMap createApplicationResourceMap() {
        return getApplicationClass() == null
            ? new ResourceMap(Application.class)
            : createApplicationResourceMap(getApplicationClass());
    }


    private ResourceMap createApplicationResourceMap(Class<?> type) {
        return type == Application.class
            ? new ResourceMap(type)
            : new ResourceMap(createApplicationResourceMap(type.getSuperclass()), type);
    }


    private static String packageResourceBundleBaseName(Class<?> type) {
        return type.getPackage().getName() + ".resources.Package";
    }


    // Actions ****************************************************************

    /**
     * Returns the action named {@code actionName} of the given target object.
     * This a convenience method for
     * {@code ApplicationContext.getInstance().getActionMap(target).get(actionName)}.<p>
     *
     * <strong>Note:</strong> Requesting an Action will create
     * a hard reference to the given target, so it cannot be cleared
     * by the garbage collector until the ActionMap entry is cleared
     * using {@link #clearActionMap(Object)}.
     *
     * @param target   the instance that will perform the Action
     * @param actionName  used to look up the associated Action method
     * @return the Action for name {@code actionName} of the given target.
     *
     * @throws IllegalArgumentException if the action is absent.
     *
     * @deprecated Replaced by {@link Application#getAction(Object, String)}
     */
    @Deprecated
    public static javax.swing.Action getAction(Object target, String actionName) {
        javax.swing.Action action = getInstance().getActionMap(target).get(actionName);
        if (action == null) {
            throw new IllegalArgumentException("No action found for name " + actionName);
        }
        return action;
    }


    /**
     * Looks up and returns the ActionMap for the given target.
     * This map contains Actions for the target's method that are
     * annotated with the <code>@Action</code> annotation.<p>
     *
     * The actions text, icon, description, etc. are initialized
     * from the target's resource map. If you want to use a custom
     * resource map, use {@link #getActionMap(Object, ResourceMap)}
     * instead.<p>
     *
     * <strong>Note:</strong> Requesting an ActionMap will create
     * a hard reference to the given target, so it cannot be cleared
     * by the garbage collector until the ActionMap entry is cleared
     * using {@link #clearActionMap(Object)}.
     *
     * @param target  the object that shall execute the Actions
     * @return an ActionMap with Actions that perform the target's
     *    action methods.
     */
    public ActionMap getActionMap(Object target) {
        return getActionMap(target, null);
    }


    /**
     * Looks up and returns the ActionMap for the given target.
     * This map contains Actions for the target's method that are
     * annotated with the <code>@Action</code> annotation.<p>
     *
     * The actions text, icon, description, etc. are initialized
     * from the given resource map. If you can use the target's
     * resource map, use {@link #getActionMap(Object)} instead.<p>
     *
     * <strong>Note:</strong> Requesting an ActionMap will create
     * a hard reference to the given target, so it cannot be cleared
     * by the garbage collector until the ActionMap entry is cleared
     * using {@link #clearActionMap(Object)}.
     *
     * @param target        the object that shall execute the Actions
     * @param resourceMap   used to initialize the action text, icon, etc
     * @return an ActionMap with Actions that perform the target's
     *    action methods.
     */
    public ActionMap getActionMap(Object target, ResourceMap resourceMap) {
        if (target == null) {
            throw new NullPointerException("The target must not be null.");
        }
        ensureValidCache();
        ActionMap actionMap = actionMapCache.get(target);
        if (actionMap != null) {
            logger.fine("ActionMap cache hit: " + target);
            return actionMap;
        }
        actionMap = Actions.lookupActionMap(target, resourceMap);
        actionMapCache.put(target, actionMap);
        return actionMap;
    }


    /**
     * Clears the ActionMap cache entry for the given target.
     * Used to avoid memory leaks when the target isn't required
     * anymore.
     *
     * @param target  the key in the ActionMap cache
     *
     * @throws NullPointerException if {@code target} is {@code null}
     *
     * @since 1.9.1
     */
    public void clearActionMap(Object target) {
        if (target == null) {
            throw new NullPointerException("The target must not be null.");
        }
        ensureValidCache();
        actionMapCache.put(target, null);
    }


    private synchronized void ensureValidCache() {
        if (cachedLocale != Locale.getDefault()) {
            actionMapCache.clear();
            cachedLocale = Locale.getDefault();
        }
    }


    // Preferences ************************************************************

    /**
     * Returns this application's root preference node for the calling user.
     * Useful for information that is specific to a user and an application.
     * User-specific data that shall be shared between applications should
     * use {@link Preferences#userRoot()} instead.<p>
     *
     * For example the user preferences root of the Application class
     * <tt>com.package1.MyApp</tt> uses the path
     * <tt>&lt;user-root&gt;/&lt;application-root&gt;</tt> which is
     * <tt>&lt;user-root&gt;/com/package1/MyApp</tt>
     * by default.
     *
     * @return the user preference node associated with this application.
     * @throws SecurityException if a security manager is present and
     *         it denies <tt>RuntimePermission("preferences")</tt>.
     * @see    RuntimePermission
     * @see    Preferences
     */
    public Preferences getUserPreferences() {
        return Preferences.userRoot().node(getApplicationPreferencesNodeName());
    }


    /**
     * Returns this application's root preference node for the system.
     * Useful for information that is shared by all users but associated
     * with an application. System data that shall be shared between
     * applications should use {@link Preferences#systemRoot()} instead.<p>
     *
     * For example the system preferences root of the Application class
     * <tt>com.package1.MyApp</tt> uses the path
     * <tt>&lt;system-root&gt;/&lt;application-root&gt;</tt> which is
     * <tt>&lt;system-root&gt;/com/package1/MyApp</tt> by default.
     *
     * @return the system preference node associated with this application.
     * @throws SecurityException if a security manager is present and
     *         it denies <tt>RuntimePermission("preferences")</tt>.
     * @see    RuntimePermission
     * @see    Preferences
     */
    public Preferences getSystemPreferences() {
        return Preferences.systemRoot().node(getApplicationPreferencesNodeName());
    }


    /**
     * Returns the preferences path name for this application. For example
     * the path name for <tt>com.mydomain.mypackage.MyApplication</tt> is
     * <tt>"/com/mydomain/mypackage/MyApplication"</tt><p>
     *
     * Subclasses may override to use a simpler name
     *
     * @return the preferences node name associated with this application
     */
    protected String getApplicationPreferencesNodeName() {
        return preferencesNodeName(getApplicationClass());
    }


    /**
     * Returns the preferences path name for the given class. For example
     * the path name for class <tt>com.mydomain.mypackage.MyClass</tt> is
     * <tt>"/com/mydomain/mypackage/MyClass"</tt>
     *
     * @param type  the type to look up a preferences node name for
     * @return the preferences node name associated with the given type
     * @throws IllegalArgumentException if the class is an array.
     */
    private static String preferencesNodeName(Class<?> type) {
        if (type.isArray())
            throw new IllegalArgumentException(
                "Arrays have no associated preferences node name.");
        String className = type.getName();
        return "/" + className.replace('.', '/');
//        int pkgEndIndex = className.lastIndexOf('.');
//        if (pkgEndIndex < 0)
//            return "/<unnamed>";
//        String packageName = className.substring(0, pkgEndIndex);
//        return "/" + packageName.replace('.', '/');
    }


}
