/*
 * Copyright (c) 2006-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.application;


/**
 * Describes how resource strings are converted to a given type.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.11 $
 *
 * @since 1.8
 */
interface ResourceConverter {

    /**
     * Checks and answers whether this converter supports the given type,
     * i.e. it can convert a String to the given type.
     *
     * @param type    the type to convert to
     * @return {@code true} if this converter can convert to the given type,
     *    {@code false} otherwise
     */
    boolean supportsType(Class<?> type);


    /**
     * Converts the given String value to the specified return type.
     * Implementations will typically just use the String value and
     * convert it to an instance of the return type. The key is useful
     * for conversion failure messages. The resource map can be used
     * to access the map's class loader (e.g. for loading resources) and
     * to resolve relative paths. Also, the resource map can be used
     * to request conversion configuration data, such as
     * a <code>Format</code> used to parse Strings.<p>
     *
     * The string conversion is invoked by the resource map object lookup
     * in {@link ResourceMap#getObject(String, Class)},
     * if a resource value is a String and requires conversion. For example
     * if a boolean is requested, but the ResourceBundle returns the
     * String {@code true}.<p>
     *
     * @param key         the resource key associated with the String value
     * @param value       the String value found for the resource key
     * @param returnType  the type of the converted object
     * @param r           the resource map that provides the class loader,
     *                    that resolves relative paths and potentially has
     *                    conversion configuration information
     * @return the converted value of type <code>returnType</code>
     *
     * @throws ResourceConversionException if the converter fails to convert
     *    the string to the specified return type, for example in case
     *    of a parse error
     *
     * @see ResourceMap#getObject(String, Class)
     */
    Object convert(String key, String value, Class<?> returnType, ResourceMap r);


    /**
     * A runtime exception used to describe string conversion failures,
     * e.g. a parse error.
     */
    public static final class ResourceConversionException extends RuntimeException {

        // Instance Creation ******************************************************

        /**
         * Constructs a new exception instance with the specified detail message.
         * The cause is not initialized.
         *
         * @param message  the detail message which is saved for later
         *        retrieval by the {@link #getMessage()} method.
         * @param key     the resource bundle key used to lookup the value
         *        that could not be converted
         * @param value   the value that couldn't be converted
         */
        public ResourceConversionException(String message, String key, String value) {
            this(message, key, value, null);
        }


        /**
         * Constructs a new exception instance with the specified detail message
         * and cause.
         *
         * @param message  the detail message which is saved for later
         *        retrieval by the {@link #getMessage()} method.
         * @param key      the resource bundle key used to lookup the value
         *        that could not be converted
         * @param value    the value that couldn't be converted
         * @param cause    the cause which is saved for later retrieval by the
         *        {@link #getCause()} method. A {@code null} value is permitted,
         *        and indicates that the cause is nonexistent or unknown.
         */
        public ResourceConversionException(String message, String key, String value, Throwable cause) {
            super(message + "\nKey=\"" + key + "\"\nValue=" + value + "\"", cause);
        }

    }


}
