/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  o Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  o Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  o Neither the name of JGoodies Karsten Lentzsch nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.jgoodies.uif.builder;

import java.awt.Component;
import java.lang.ref.WeakReference;
import java.util.MissingResourceException;

import javax.swing.JLabel;
import javax.swing.JPanel;

import com.jgoodies.forms.builder.AbstractI15dPanelBuilder;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif.application.ResourceMap;

/**
 * A general purpose panel builder that uses the {@link FormLayout}
 * to lay out <code>JPanel</code>s. In addition to its superclass
 * {@link PanelBuilder} this class provides convenience behavior to map
 * resource keys to their associated internationalized (i15d) strings
 * when adding labels, titles and titled separators.<p>
 *
 * The localized texts used in methods <code>#addI15dLabel</code>
 * and <code>#addI15dTitle</code> can contain an optional mnemonic marker.
 * The mnemonic and mnemonic index are indicated by a single ampersand
 * (<tt>&amp;</tt>). For example <tt>&quot;&amp;Save&quot</tt>, or
 * <tt>&quot;Save&nbsp;&amp;as&quot</tt>. To use the ampersand itself,
 * duplicate it, for example <tt>&quot;Look&amp;&amp;Feel&quot</tt>.<p>
 *
 * <strong>Note:</strong> This class has been temporarily extended to add
 * support for automatically associating added focusable components with
 * a previously added label that has a mnemonic. This support will be moved
 * to the Forms library core in the Forms 1.3.
 *
 * @author	Karsten Lentzsch
 * @version $Revision: 1.4 $
 *
 * @see	ResourceMap
 */
public class I15dPanelBuilder2 extends AbstractI15dPanelBuilder {

    // Instance Fields ********************************************************

    /**
     * Holds the <code>ResourceMap</code> used to look up internationalized
     * (i15d) String resources.
     */
    private final ResourceMap resourceMap;


    /**
     * The instance value for the setLabelFor feature.
     * Is initialized using the global default.
     *
     * @see #setLabelForFeatureEnabled(boolean)
     * @see PanelBuilder2#setLabelForFeatureEnabledDefault(boolean)
     */
    private boolean labelForFeatureEnabled;


    /**
     * Refers to the most recently label that has a mnemonic set - if any.
     * Used to invoke {@link JLabel#setLabelFor(java.awt.Component)}
     * for the next component added to the panel. After the association
     * has been set, the reference will be cleared.
     */
    private WeakReference<JLabel> mostRecentlyAddedMnemonicLabelReference = null;


    // Instance Creation ******************************************************

    /**
     * Constructs a <code>I15dPanelBuilder2</code> for the given
     * layout and resource bundle. Uses an instance of <code>JPanel</code>
     * as layout container.
     *
     * @param layout        the <code>FormLayout</code> used to layout the container
     * @param resourceMap   the <code>ResourceMap</code> used to look up i15d strings
     *
     * @throws NullPointerException if resourceMap is null
     */
    public I15dPanelBuilder2(FormLayout layout, ResourceMap resourceMap){
        this(layout, resourceMap, new JPanel(null));
    }


    /**
     * Constructs a <code>I15dPanelBuilder2</code>
     * for the given FormLayout, resource bundle, and layout container.
     *
     * @param layout  the <code>FormLayout</code> used to layout the container
     * @param resourceMap  the <code>ResourceMap</code> used to look up i15d strings
     * @param panel   the layout container
     *
     * @throws NullPointerException if resourceMap is null
     */
    public I15dPanelBuilder2(FormLayout layout, ResourceMap resourceMap, JPanel panel){
        super(layout, panel);
        if (resourceMap == null) {
            throw new NullPointerException("The resource map must not be null");
        }
        this.resourceMap = resourceMap;
        labelForFeatureEnabled = PanelBuilder2.getLabelForFeatureEnabledDefault();
    }


    // Configuration **********************************************************

    /**
     * Returns whether the setLabelFor feature is enabled for this PanelBuilder.
     * The value is initialized from the global default value for this feature
     * {@link PanelBuilder2#getLabelForFeatureEnabledDefault()}.
     * It is globally disabled by default.
     *
     * @return true for enabled, false for disabled
     */
    public boolean isLabelForFeatureEnabled() {
        return labelForFeatureEnabled;
    }


    /**
     * Enables or disables the setLabelFor feature for this PanelBuilder.
     * The value is initialized from the global default value
     * {@link PanelBuilder2#getLabelForFeatureEnabledDefault()}.
     * It is globally disabled by default.
     *
     * @param b true for enabled, false for disabled
     */
    public void setLabelForFeatureEnabled(boolean b) {
        labelForFeatureEnabled = b;
    }


    // Implementing Abstract Behavior *****************************************

    /**
     * Looks up and returns the internationalized (i15d) string for the given
     * resource key from the <code>ResourceMap</code>.
     *
     * @param resourceKey  the key to look for in the resource map
     * @return the associated internationalized string, or the resource key
     *     itself in case of a missing resource
     */
    @Override
    protected String getI15dString(String resourceKey) {
        try {
            return resourceMap.getString(resourceKey);
        } catch (MissingResourceException mre) {
            return resourceKey;
        }
    }


    // Overriding Superclass Behavior *****************************************

    /**
     * Adds a component to the panel using the given cell constraints.
     * In addition to the superclass behavior, this implementation
     * tracks the most recently label that has mnemonic, and associates
     * it with the next added focusable component.<p>
     *
     * TODO: Consider to clear the most recently added mnemonic label
     * if another label is added - even if the latter has no mnemonic set.
     *
     * @param component        the component to add
     * @param cellConstraints  the component's cell constraints
     * @return the added component
     *
     * @see #isLabelForFeatureEnabled()
     * @see #isLabelForApplicable(Component)
     */
    @Override
    public Component add(Component component, CellConstraints cellConstraints) {
        Component result = super.add(component, cellConstraints);
        if (!isLabelForFeatureEnabled()) {
            return result;
        }
        JLabel mostRecentlyAddedMnemonicLabel = getMostRecentlyAddedMnemonicLabel();
        if (   (mostRecentlyAddedMnemonicLabel != null)
            && isLabelForApplicable(component)) {
            mostRecentlyAddedMnemonicLabel.setLabelFor(component);
            clearMostRecentlyAddedMnemonicLabel();
        }
        setMostRecentlyAddedMnemonicLabel(component);
        return result;
    }


    // Default Behavior *******************************************************

    /**
     * Checks and answers whether the given component shall be set
     * as component for a previously added label with mnemonic using
     * {@link JLabel#setLabelFor(Component)}.
     * This default implementation just checks whether the component is
     * focusable. Subclasses may override.
     *
     * @param component    the component to be checked
     * @return true if focusable, false otherwise
     */
    protected boolean isLabelForApplicable(Component component) {
        return component.isFocusable();
    }


    // Helper Code ************************************************************

    /**
     * Returns the most recently added JLabel that has a mnemonic set
     * - if any, <code>null</code>, if none has been set, or if it has
     * been cleared after setting an association before, or if it has been
     * cleared by the garbage collector.
     *
     * @return the most recently added JLabel that has a mnemonic set
     *     and has not been associated with a component applicable for this
     *     feature. <code>null</code> otherwise.
     */
    private JLabel getMostRecentlyAddedMnemonicLabel() {
        if (mostRecentlyAddedMnemonicLabelReference == null) {
            return null;
        }
        JLabel label = mostRecentlyAddedMnemonicLabelReference.get();
        if (label == null) {
            return null;
        }
        return label;
    }


    /**
     * Sets a JLabel that has a mnemonic set as most recently added label.
     * Does nothing otherwise.
     *
     * @param component  the label to be set - if at all
     */
    private void setMostRecentlyAddedMnemonicLabel(Component component) {
        if (!(component instanceof JLabel)) {
            return;
        }
        JLabel label = (JLabel) component;
        if (label.getDisplayedMnemonic() != -1) {
            mostRecentlyAddedMnemonicLabelReference =
                new WeakReference<JLabel>(label);
        }
    }


    /**
     * Clears the reference to the most recently added mnemonic label.
     */
    private void clearMostRecentlyAddedMnemonicLabel() {
        mostRecentlyAddedMnemonicLabelReference = null;
    }


}
