/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  o Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  o Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  o Neither the name of JGoodies Karsten Lentzsch nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.jgoodies.uif.builder;

import java.awt.Component;
import java.lang.ref.WeakReference;

import javax.swing.JLabel;
import javax.swing.JPanel;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

/**
 * Adds a feature to its superclass PanelBuilder that labels with a mnemonic
 * are associated with the next added focusable component.
 * This feature shall be moved to the PanelBuilder.<p>
 *
 * <strong>Note:</strong> This class has been temporarily added to the UIF.
 * The support for automatically associating added focusable components with
 * a previously added label that has a mnemonic will be moved to the
 * Forms library core in the Forms 1.3.
 *
 * @author	Karsten Lentzsch
 * @version $Revision: 1.2 $
 */
public class PanelBuilder2 extends PanelBuilder {

    // Static Fields **********************************************************

    /**
     * The global default for the enablement of the setLabelFor feature.
     * Turned off by default.
     *
     * @see #setLabelForFeatureEnabledDefault(boolean)
     * @see #setLabelForFeatureEnabled(boolean)
     */
    private static boolean labelForFeatureEnabledDefault = false;


    // Instance Fields ********************************************************

    /**
     * The instance value for the setLabelFor feature.
     * Is initialized using the global default.
     *
     * @see #setLabelForFeatureEnabled(boolean)
     * @see #setLabelForFeatureEnabledDefault(boolean)
     */
    private boolean labelForFeatureEnabled;


    /**
     * Refers to the most recently label that has a mnemonic set - if any.
     * Used to invoke {@link JLabel#setLabelFor(java.awt.Component)}
     * for the next component added to the panel. After the association
     * has been set, the reference will be cleared.
     */
    private WeakReference<JLabel> mostRecentlyAddedMnemonicLabelReference = null;


    // Instance Creation ****************************************************

    /**
     * Constructs a <code>PanelBuilder2</code> for the given
     * layout. Uses an instance of <code>JPanel</code> as layout container
     * with the given layout as layout manager.
     *
     * @param layout  the FormLayout to use
     */
    public PanelBuilder2(FormLayout layout){
        this(layout, new JPanel(null));
    }

    /**
     * Constructs a <code>PanelBuilder2</code> for the given
     * FormLayout and layout container.
     *
     * @param layout  the FormLayout to use
     * @param panel   the layout container to build on
     */
    public PanelBuilder2(FormLayout layout, JPanel panel){
        super(layout, panel);
        labelForFeatureEnabled = getLabelForFeatureEnabledDefault();
    }


    // Global Defaults ********************************************************

    /**
     * Returns the global default for the enablement of the setLabelFor feature.
     * This can be overridden per PanelBuilder using
     * {@link #setLabelForFeatureEnabled(boolean)}.
     * The feature is globally disabled by default.
     *
     * @return true for globally enabled, false for globally disabled
     */
    public static boolean getLabelForFeatureEnabledDefault() {
        return labelForFeatureEnabledDefault;
    }


    /**
     * Sets the default value for the setLabelFor feature enablement.
     * This can be overridden per PanelBuilder using
     * {@link #setLabelForFeatureEnabled(boolean)}.
     * The default value is used to set the initial PanelBuilder
     * setting for this feature.
     * The feature is globally disabled by default.
     *
     * @param b true for globally enabled, false for globally disabled
     */
    public static void setLabelForFeatureEnabledDefault(boolean b) {
        labelForFeatureEnabledDefault = b;
    }


    // Configuration **********************************************************

    /**
     * Returns whether the setLabelFor feature is enabled for this PanelBuilder.
     * The value is initialized from the global default value for this feature
     * {@link #getLabelForFeatureEnabledDefault()}. It is globally disabled
     * by default.
     *
     * @return true for enabled, false for disabled
     */
    public boolean isLabelForFeatureEnabled() {
        return labelForFeatureEnabled;
    }


    /**
     * Enables or disables the setLabelFor feature for this PanelBuilder.
     * The value is initialized from the global default value
     * {@link #getLabelForFeatureEnabledDefault()}. It is globally disabled
     * by default.
     *
     * @param b true for enabled, false for disabled
     */
    public void setLabelForFeatureEnabled(boolean b) {
        labelForFeatureEnabled = b;
    }


    // Overriding Superclass Behavior *****************************************

    /**
     * Adds a component to the panel using the given cell constraints.
     * In addition to the superclass behavior, this implementation
     * tracks the most recently label that has mnemonic, and associates
     * it with the next added focusable component.<p>
     *
     * TODO: Consider to clear the most recently added mnemonic label
     * if another label is added - even if the latter has no mnemonic set.
     *
     * @param component        the component to add
     * @param cellConstraints  the component's cell constraints
     * @return the added component
     *
     * @see #isLabelForFeatureEnabled()
     * @see #isLabelForApplicable(Component)
     */
    @Override
    public Component add(Component component, CellConstraints cellConstraints) {
        Component result = super.add(component, cellConstraints);
        if (!isLabelForFeatureEnabled()) {
            return result;
        }
        JLabel mostRecentlyAddedMnemonicLabel = getMostRecentlyAddedMnemonicLabel();
        if (   (mostRecentlyAddedMnemonicLabel != null)
            && isLabelForApplicable(component)) {
            mostRecentlyAddedMnemonicLabel.setLabelFor(component);
            clearMostRecentlyAddedMnemonicLabel();
        }
        setMostRecentlyAddedMnemonicLabel(component);
        return result;
    }


    // Default Behavior *******************************************************

    /**
     * Checks and answers whether the given component shall be set
     * as component for a previously added label with mnemonic using
     * {@link JLabel#setLabelFor(Component)}.
     * This default implementation just checks whether the component is
     * focusable. Subclasses may override.
     *
     * @param component    the component to be checked
     * @return true if focusable, false otherwise
     */
    protected boolean isLabelForApplicable(Component component) {
        return component.isFocusable();
    }


    // Helper Code ************************************************************

    /**
     * Returns the most recently added JLabel that has a mnemonic set
     * - if any, <code>null</code>, if none has been set, or if it has
     * been cleared after setting an association before, or if it has been
     * cleared by the garbage collector.
     *
     * @return the most recently added JLabel that has a mnemonic set
     *     and has not been associated with a component applicable for this
     *     feature. <code>null</code> otherwise.
     */
    private JLabel getMostRecentlyAddedMnemonicLabel() {
        if (mostRecentlyAddedMnemonicLabelReference == null) {
            return null;
        }
        JLabel label = mostRecentlyAddedMnemonicLabelReference.get();
        if (label == null) {
            return null;
        }
        return label;
    }


    /**
     * Sets a JLabel that has a mnemonic set as most recently added label.
     * Does nothing otherwise.
     *
     * @param component  the label to be set - if at all
     */
    private void setMostRecentlyAddedMnemonicLabel(Component component) {
        if (!(component instanceof JLabel)) {
            return;
        }
        JLabel label = (JLabel) component;
        if (label.getDisplayedMnemonic() != -1) {
            mostRecentlyAddedMnemonicLabelReference =
                new WeakReference<JLabel>(label);
        }
    }


    /**
     * Clears the reference to the most recently added mnemonic label.
     */
    private void clearMostRecentlyAddedMnemonicLabel() {
        mostRecentlyAddedMnemonicLabelReference = null;
    }


}
