/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.builder;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import com.jgoodies.uif.action.ToggleAction;
import com.jgoodies.uif.component.UIFCheckBoxMenuItem;
import com.jgoodies.uif.component.UIFMenuItem;
import com.jgoodies.uif.component.UIFRadioButtonMenuItem;
import com.jgoodies.uif.util.Mode;
import com.jgoodies.uif.util.Modes;

/**
 * A builder that reduces the effort required to build, configure
 * and fill a JPopupMenu with menu items and separators.
 * Provides several convenience methods that implicitly create
 * extended menu item implementations or implicitly configure items
 * to automatically handle typical platforms differences.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.7 $
 *
 * @see javax.swing.JPopupMenu
 * @see javax.swing.Action
 * @see ToggleAction
 * @see com.jgoodies.uif.component.UIFMenuItem
 * @see com.jgoodies.uif.component.UIFCheckBoxMenuItem
 * @see com.jgoodies.uif.component.UIFRadioButtonMenuItem
 * @see com.jgoodies.uif.builder.ToolBarBuilder
 */
public final class PopupMenuBuilder {

    /**
     * Holds the mode that is used to initialize the icon visible mode.
     *
     * @see #getIconVisibleDefaultMode()
     * @see #setIconVisibleDefaultMode(Mode)
     */
    private static Mode iconVisibleDefaultMode = Modes.LAF_NON_AQUA;

    /**
     * Holds the current icon visible mode.
     */
    private Mode iconVisibleMode;

    /**
     * Refers to the popup menu that is built by this builder.
     */
    private final JPopupMenu popupMenu;


	// Instance Creation *****************************************************

	/**
	 * Constructs a popup menu builder.
     * Implicitly creates a popup menu with the given label.
	 *
	 * @param label     the label for the built menu
	 */
	public PopupMenuBuilder(String label) {
		this(new JPopupMenu(label));
	}


	/**
	 * Constructs a popup menu builder using the given JPopupMenu.
	 *
	 * @param menu   the menu used with this builder
	 */
	public PopupMenuBuilder(JPopupMenu menu) {
	    this.popupMenu = menu;
	    setIconVisibleMode(getIconVisibleDefaultMode());
	}


	// Setting Defaults ******************************************************

	/**
	 * Returns the default icon mode that is used as initial value
	 * of the <em>iconVisibleMode</em> property.
	 *
	 * @return the current icon mode
	 */
	public static Mode getIconVisibleDefaultMode() {
	    return iconVisibleDefaultMode;
	}


	/**
	 * Sets a new default icon mode that is used as initial value
	 * for the <em>iconVisibleMode</em> property.
	 *
	 * @param newDefaultMode   the default icon mode to be set
	 */
	public static void setIconVisibleDefaultMode(Mode newDefaultMode) {
	    iconVisibleDefaultMode = newDefaultMode;
	}


	// Accessing Properties **************************************************

	/**
	 * Returns the current icon mode that determines whether menu items created
	 * by <code>createActionComponent(Action)</code> get an icon or not.
	 *
	 * @return the current icon mode
	 */
	public Mode getIconVisibleMode() {
	    return iconVisibleMode;
	}


	/**
	 * Sets a new icon mode that determines whether menu items created
	 * by <code>createActionComponent(Action)</code> get an icon or not.
	 *
	 * @param newMode   the icon mode to be set
	 */
	public void setIconVisibleMode(Mode newMode) {
	    iconVisibleMode = newMode;
	}


	/**
	 * Returns the popup menu that is built by this builder.
	 *
	 * @return this builder's menu
	 */
	public JPopupMenu getPopupMenu() {
	    return popupMenu;
	}


	// Adding Components ******************************************************

	/**
	 * Creates a menu item for the specified text, adds it to this
	 * builder's menu and returns the new menu item.
	 *
     * @param text   the text for the menu item
     * @return the new menu item
	 */
	public UIFMenuItem add(String text) {
	    return add(new UIFMenuItem(text));
	}


	/**
	 * Creates a menu item for the specified text and icon, adds it
	 * to this builder's menu and returns the new menu item.
	 *
     * @param text   the text for the menu item
     * @param icon   the icon for the menu item
     * @return the new menu item
	 */
	public UIFMenuItem add(String text, Icon icon) {
	    return add(new UIFMenuItem(text, icon));
	}


	/**
	 * Creates a menu item for the specified text and icon, adds it
	 * to this builder's menu and returns the new menu item.
	 *
	 * @param text       the text for the menu item
	 * @param mnemonic   the mnemonic for the menu item
	 * @return the new menu item
	 */
	public UIFMenuItem add(String text, int mnemonic) {
	    return add(new UIFMenuItem(text, mnemonic));
	}


	/**
	 * Creates a menu item for the specified text and icon, adds it
	 * to this builder's menu and returns the new menu item.
	 *
	 * @param text       the text for the menu item
     * @param icon       the icon for the menu item
	 * @param mnemonic   the mnemonic for the menu item
	 * @return the new menu item
	 */
	public UIFMenuItem add(String text, Icon icon, int mnemonic) {
	    return add(new UIFMenuItem(text, icon, mnemonic));
	}


    /**
     * Creates, adds and returns a <code>UIFMenuItem</code> that is
     * configured using the given plain Action.
     *
     * @param action the <code>Action</code> for the menu item to be added
     * @return the new menu item
     *
     * @throws IllegalArgumentException if the action is an instance
     *     of <code>ToggleAction</code>
     */
    public JMenuItem add(Action action) {
        if ((action instanceof ToggleAction))
            throw new IllegalArgumentException("Use #addToggle for ToggleActions.");

        return add(new UIFMenuItem(action));
    }


    /**
     * Creates a radio button or check box menu item for the given
     * <code>ToggleAction</code>, adds it to this builder's menu
     * and returns the new menu item.
     *
     * @param toggleAction the <code>Action</code> for the menu item to be added
     * @return the new menu item
     */
    public JMenuItem addToggle(ToggleAction toggleAction) {
        JMenuItem item = toggleAction.isRadioAction()
            ? new UIFRadioButtonMenuItem(toggleAction)
            : new UIFCheckBoxMenuItem(toggleAction);
        return add(item);
    }


    /**
     * Adds the given JMenuItem to this builder's menu.
     *
     * @param item   the menu item to add
     * @return the added menu item
     */
    public JMenuItem add(JMenuItem item) {
        return getPopupMenu().add(item);
    }


    /**
     * Adds the given UIFMenuItem to this builder's menu.
     * Sets the item's icon visible mode to this builder's mode.
     *
     * @param item   the menu item to add
     * @return the added menu item
     */
    public UIFMenuItem add(UIFMenuItem item) {
        item.setIconVisibleMode(getIconVisibleMode());
        return (UIFMenuItem) add((JMenuItem) item);
    }


    /**
     * Adds the given UIFRadioButtonMenuItem to this builder's menu.
     * Sets the item's icon visible mode to this builder's mode.
     *
     * @param item   the menu item to add
     * @return the added menu item
     */
    public UIFRadioButtonMenuItem add(UIFRadioButtonMenuItem item) {
        item.setIconVisibleMode(getIconVisibleMode());
        return (UIFRadioButtonMenuItem) add((JMenuItem) item);
    }


    /**
     * Adds the given UIFRadioButtonMenuItem to this builder's menu.
     * Sets the item's icon visible mode to this builder's mode.
     *
     * @param item   the menu item to add
     * @return the added menu item
     */
    public UIFCheckBoxMenuItem add(UIFCheckBoxMenuItem item) {
        item.setIconVisibleMode(getIconVisibleMode());
        return (UIFCheckBoxMenuItem) add((JMenuItem) item);
    }


    /**
     * Adds a separator to this builder's menu.
     */
    public void addSeparator() {
        getPopupMenu().addSeparator();
    }


}
