/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.builder;

import java.awt.Component;
import java.awt.Dimension;

import javax.swing.*;

import com.jgoodies.uif.action.ToggleAction;
import com.jgoodies.uif.component.PopupButton;
import com.jgoodies.uif.component.ToolBarButton;
import com.jgoodies.uif.component.ToolBarToggleButton;
import com.jgoodies.uif.util.Mode;
import com.jgoodies.uif.util.Modes;

/**
 * A builder that reduces the effort required to build, configure
 * and fill a JToolBar with buttons and gaps.
 * Provides several convenience methods that implicitly create
 * buttons specialized for toolbars or implicitly configure buttons
 * to automatically handle typical platforms differences.<p>
 *
 * Subclasses can override the default toolbar configuration in
 * <code>#configureToolBar</code> and can override the creation methods
 * <code>#add*</code>.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.6 $
 *
 * @see	javax.swing.Action
 * @see	ToggleAction
 * @see	com.jgoodies.uif.component.ToolBarButton
 * @see	com.jgoodies.uif.component.ToolBarToggleButton
 * @see com.jgoodies.uif.builder.MenuBuilder
 */
public class ToolBarBuilder {

    /**
     * Holds the mode that is used to initialize the icon visible mode.
     *
     * @see #getBorderPaintedDefaultMode()
     * @see #setBorderPaintedDefaultMode(Mode)
     */
    private static Mode borderPaintedDefaultMode = Modes.LAF_NON_AQUA;

    /**
     * Holds the mode that is used to determine the margin size
     * if the button is configured.
     *
     * @see #getWideMarginMode()
     * @see #setWideMarginMode(Mode)
     */
    private static Mode wideMarginDefaultMode = Modes.LAF_AQUA;

    /**
     * Holds the mode that is used to set the <em>borderPainted</em>
     * property if the UI changes.
     *
     * @see #getBorderPaintedMode()
     * @see #setBorderPaintedMode(Mode)
     */
    private Mode borderPaintedMode = getBorderPaintedDefaultMode();

    /**
     * Holds the mode that is used to determine the margin size
     * if the button is configured.
     *
     * @see #getWideMarginMode()
     * @see #setWideMarginMode(Mode)
     */
    private Mode wideMarginMode = getWideMarginDefaultMode();

    /**
     * Refers to the tool bar that is built by this builder.
     */
    private final JToolBar toolBar;


    // Instance Creation ******************************************************

    /**
     * Constructs a tool bar builder for a tool bar without name.
     * The tool bar has the rollover property set and is not floatable.
     */
    public ToolBarBuilder() {
        this("", null);
    }

    /**
     * Constructs a tool bar builder for a tool bar with the given name.
     * The tool bar has the rollover property set and is not floatable.
     *
     * @param name   the name for the built tool bar
     */
    public ToolBarBuilder(String name) {
        this(name, null);
    }

    /**
     * Constructs a tool bar builder for tool bar
     * with the given name and header style.
     * The tool bar has the rollover property set and is not floatable.
     *
     * @param name               the name for the built tool bar
     * @param headerStyleOrName  the style applied to the built tool bar
     */
    public ToolBarBuilder(String name, Object headerStyleOrName) {
        toolBar = new JToolBar(name);
        configureToolBar();
        setHeaderStyle(headerStyleOrName);
    }

    /**
     * Constructs a tool bar builder for the given tool bar.
     * The toolbar won't be configured using <code>#configureToolBar</code>.
     *
     * @param toolBar            the toolbar used to build
     */
    public ToolBarBuilder(JToolBar toolBar) {
        this.toolBar = toolBar;
    }


    // Setting Defaults ******************************************************

    /**
     * Returns the border painted default mode that is used to initialize
     * the <em>borderPaintedMode</em> property.
     *
     * @return the border painted default mode
     */
    public static Mode getBorderPaintedDefaultMode() {
        return borderPaintedDefaultMode;
    }


    /**
     * Sets a new border painted default mode that is used to initialize
     * the <em>borderPaintedMode</em> property.
     *
     * @param newDefaultMode   the border painted default mode to be set
     */
    public static void setBorderPaintedDefaultMode(Mode newDefaultMode) {
        borderPaintedDefaultMode = newDefaultMode;
    }


    /**
     * Returns the wide margin default mode that is used to initialize
     * the <em>wideMarginMode</em> property.
     *
     * @return the wide margin default mode
     */
    public static Mode getWideMarginDefaultMode() {
        return wideMarginDefaultMode;
    }


    /**
     * Sets a new wide margin default mode that is used to initialize
     * the <em>wideMarginMode</em> property.
     *
     * @param newDefaultMode   the wide margin default mode to be set
     */
    public static void setWideMarginDefaultMode(Mode newDefaultMode) {
        wideMarginDefaultMode = newDefaultMode;
    }


    // Accessing Properties ***************************************************

    /**
     * Returns this builder's border painted mode that is used to set
     * the border painted mode in implicitly created tool bar buttons.
     *
     * @return the current border painted mode
     */
    public final Mode getBorderPaintedMode() {
        return borderPaintedMode;
    }


    /**
     * Sets a new border painted mode that is used to set
     * the border painted mode in implicitly created tool bar buttons.
     *
     * @param newMode   the border painted mode to be set
     */
    public final void setBorderPaintedMode(Mode newMode) {
        borderPaintedMode = newMode;
    }


    /**
     * Returns this builder's wide margin mode that is used to set
     * the wide margin mode in implicitly created tool bar buttons.
     *
     * @return the current wide margin mode
     */
    public final Mode getWideMarginMode() {
        return wideMarginMode;
    }


    /**
     * Sets a new wide margin mode that is used to set
     * the border painted mode in implicitly created tool bar buttons.
     *
     * @param newMode   the wide margin mode to be set
     */
    public final void setWideMarginMode(Mode newMode) {
        wideMarginMode = newMode;
    }


    /**
     * Returns the tool bar that is built by this builder.
     *
     * @return this builder's tool bar
     */
    public final JToolBar getToolBar() {
        return toolBar;
    }


    // ToolBar Configuration *************************************************

    /**
     * Sets default settings for the tool bar's rollover and floatable
     * properties: rollover and non-floatable.<p>
     *
     * Subclasses may override to implement a different default configuration.
     */
    protected void configureToolBar()  {
        getToolBar().setRollover(true);
        getToolBar().setFloatable(false);
    }


    /**
     * Sets the JGoodies Looks border style as object or by name.
     *
     * @param borderStyleOrName   a com.jgoodies.looks.BorderStyle or its name
     */
    public final void setBorderStyle(Object borderStyleOrName) {
        getToolBar().putClientProperty("Plastic.borderStyle",
                borderStyleOrName);
        getToolBar().putClientProperty("jgoodies.windows.borderStyle",
                borderStyleOrName);
    }


    /**
     * Sets the JGoodies Looks header style as object or by name.
     *
     * @param headerStyleOrName   a com.jgoodies.looks.HeaderStyle or its name
     */
    public final void setHeaderStyle(Object headerStyleOrName) {
        getToolBar().putClientProperty("jgoodies.headerStyle",
                headerStyleOrName);
    }


    // Adding Components ******************************************************

    /**
     * Adds a gap to the tool bar.
     */
    public void addGap() {
        addGap(6);
    }


    /**
     * Adds a large gap to the tool bar. Useful to separate sections.
     */
    public void addLargeGap() {
        addGap(12);
    }


    /**
     * Adds a gap to the tool bar.
     *
     * @param size   the gap size in pixel
     */
    public void addGap(int size) {
        add(Box.createRigidArea(new Dimension(size, size)));
    }


    /**
     * Adds a glue component to this builder's tool bar.
     * The glue is a gap that grows if the tool bar grows.
     */
    public void addGlue() {
         add(Box.createGlue());
    }


    /**
     * Adds a separator to this builder's tool bar.
     */
    public void addSeparator() {
        getToolBar().addSeparator();
    }


    /**
     * Adds a popup button to this builder's tool bar.
     * Let the popup button add itself to this tool bar,
     * because it may consists of multiple parts
     *
     * @param button    the popup button to be added
     */
    public void add(PopupButton button) {
        button.addTo(getToolBar());
    }

    /**
     * Adds a component to this builder's tool bar and returns the component.
     *
     * @param component    the component to be added
     * @return the component
     */
    public Component add(Component component) {
        return getToolBar().add(component);
    }


    /**
     * Creates, adds and returns a ToolBarButton that is configured
     * using the given <code>Action</code>.
     *
     * @param action   an action that describes a button
     * @return the added ToolBarButton
     *
     * @throws IllegalArgumentException if the action is an instance
     *     of <code>ToggleAction</code>
     */
    public JButton add(Action action) {
        if ((action instanceof ToggleAction))
            throw new IllegalArgumentException("Use #addToggle for ToggleActions.");

        ToolBarButton button = new ToolBarButton(action);
        button.setBorderPaintedMode(getBorderPaintedMode());
        button.setWideMarginMode(getWideMarginMode());
        add(button);
        return button;
    }


    /**
     * Creates, adds and returns a ToolBarToggleButton that is configured
     * using the given <code>ToggleAction</code>.
     *
     * @param toggleAction an action that describes a toggle and provides a model
     * @return the added ToolBarToggleButton
     */
    public JToggleButton addToggle(ToggleAction toggleAction) {
        ToolBarToggleButton button = new ToolBarToggleButton(toggleAction);
        button.setBorderPaintedMode(getBorderPaintedMode());
        button.setWideMarginMode(getWideMarginMode());
        add(button);
        return button;
    }


}
