/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.component;

import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;

import javax.swing.JComponent;

/**
 * A component that paints a horizontal or vertical gradient background.
 * The gradient consists of two sections that are specified by three colors:
 * start, middle and stop.<p>
 *
 * The location of the middle color can be defined by a float value
 * in the interval [0, 1], with 0f for the left hand side,
 * 0.5f for the center, and 1f for the right hand side.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.2 $
 */
public final class GradientSeparator extends JComponent {

    /** Constant for horizontal gradients. */
    public static final int HORIZONTAL =  0;

    /** Constant for vertical gradients. */
    public static final int VERTICAL   = 90;


    /** Left hand middle color proportion in horizontal gradients. */
    public static final float LEFT = 0f;

    /** Top middle color proportion in vertical gradients. */
    public static final float TOP = 0f;

    /** Middle middle color proportion in horizontal gradients. */
    public static final float MIDDLE = 0.5f;

    /** Centered middle color proportion in vertical gradients. */
    public static final float CENTER = 0.5f;

    /** Right hand middle color proportion in horizontal gradients. */
    public static final float RIGHT = 1f;

    /** Bottom middle color proportion in vertical gradients. */
    public static final float BOTTOM = 1f;



    /** Orientation is either horizontal or vertical. */
    private final int     orientation;

    /** Start color for the first gradient section. */
    private final Color   startColor;

    /** End color for the first gradient section and start for the second. */
    private final Color   middleColor;

    /** End color for the second gradient section. */
    private final Color   stopColor;

    /**
     * A float value from [0, 1] that defines the proportion of the
     * middle gradient: 0f for the left hand side, 0.5f for the center,
     * and 1f for the right hand side.
     */
    private final float   middleProportion;


    // Instance creation ************************************************

	/**
	 * Constructs a GradientSeparator with the specified
     * orientation, gradient colors, and gradient middle location.
	 */
	public GradientSeparator(int orientation,
                          Color startColor,
                          Color middleColor,
                          Color stopColor,
                          float middleProportion) {
		this.orientation      = orientation;
        this.startColor       = startColor;
        this.middleColor      = middleColor;
        this.stopColor        = stopColor;
        this.middleProportion = middleProportion;

        if (orientation != HORIZONTAL &&
            orientation != VERTICAL)
            throw new IllegalArgumentException("The gradient orientation must be horizontal or vertical.");
        if (startColor == null)
            throw new NullPointerException("The gradient start color must not be null.");
        if (middleColor == null)
            throw new NullPointerException("The gradient middle color must not be null.");
        if (stopColor == null)
            throw new NullPointerException("The gradient stop color must not be null.");
        if (middleProportion < 0 || middleProportion > 1)
            throw new IllegalArgumentException("The middle location must be in [0, 1].");
	}


	/**
	 * Creates and returns a GradientSeparator
     * with a single horizontal gradient described by two colors.
	 */
	public static GradientSeparator createSingleHorizontalGradient(
                        Color startColor, Color stopColor) {
		return new GradientSeparator(HORIZONTAL,
                                      startColor, stopColor, stopColor,
                                      LEFT);
	}


    /**
     * Creates and returns a GradientSeparator
     * with a centered horizontal gradient that has the specified
     * color in the middle and is transparent on both sides.
     */
    public static GradientSeparator createHorizontalGradient(Color color) {
        return createHorizontalGradient(asTransparentColor(color),
                                         color);
    }


    /**
     * Creates and returns a GradientSeparator
     * with a two-section horizontal gradient described by two colors.
     */
    public static GradientSeparator createHorizontalGradient(
                Color startColor, Color middleColor) {
        return createHorizontalGradient(startColor, middleColor, startColor);
    }


    /**
     * Creates and returns a GradientSeparator
     * with a two-section horizontal gradient described by three colors.
     */
    public static GradientSeparator createHorizontalGradient(
                Color startColor, Color middleColor, Color stopColor) {
        return new GradientSeparator(HORIZONTAL,
                                      startColor, middleColor, stopColor,
                                      MIDDLE);
    }


    /**
     * Creates and returns a GradientSeparator
     * with a single vertical gradient described by two colors.
     */
    public static GradientSeparator createSingleVerticalGradient(
                            Color startColor, Color stopColor) {
        return new GradientSeparator(VERTICAL,
                                      startColor, stopColor, stopColor,
                                      BOTTOM);
    }


    /**
     * Creates and returns a GradientSeparator
     * with a two-section vertical gradient that has the specified
     * color in the center and is transparent on both ends.
     */
    public static GradientSeparator createVerticalGradient(Color baseColor) {
        return createVerticalGradient(asTransparentColor(baseColor), baseColor);
    }


    /**
     * Creates and returns a GradientSeparator
     * with a two-section vertical gradient described by two colors.
     */
    public static GradientSeparator createVerticalGradient(Color startColor,
                                                            Color middleColor) {
        return createVerticalGradient(startColor, middleColor, startColor);
    }


    /**
     * Creates and returns a GradientSeparator
     * with a two-section vertical gradient described by three colors.
     */
    public static GradientSeparator createVerticalGradient(Color startColor,
                                                            Color middleColor,
                                                            Color stopColor) {
        return new GradientSeparator(VERTICAL,
                                      startColor, middleColor, stopColor,
                                      CENTER);
    }


    // Painting *************************************************************

	/**
	 * Paints a background composed of two gradients.
	 */
	@Override
    protected void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		int width  = getWidth();
		int height = getHeight();
		if (orientation == HORIZONTAL) {
			int mid = Math.round(width * middleProportion);
            if (middleProportion != 0f) {
                g2.setPaint(new GradientPaint(0, 0, startColor, mid, 0, middleColor));
                g2.fillRect(0, 0, mid, height);
            }
            if (middleProportion != 1f) {
                g2.setPaint(new GradientPaint(mid, 0, middleColor, width, 0, stopColor));
                g2.fillRect(mid, 0, width, height);
            }
		} else {
			int mid = Math.round(height * middleProportion);
            if (middleProportion != 0f) {
				g2.setPaint(new GradientPaint(0, 0, startColor, 0, mid, middleColor));
				g2.fillRect(0, 0, width, mid);
            }
            if (middleProportion != 1f) {
				g2.setPaint(new GradientPaint(0, mid, middleColor, 0, height, stopColor));
				g2.fillRect(0, mid, width, height - mid);
            }
		}
	}


    // Helper Code **********************************************************

    /**
     * Returns a transparent color for the given base color.
     */
    private static Color asTransparentColor(Color baseColor) {
        return new Color(baseColor.getRed(),
                          baseColor.getGreen(),
                          baseColor.getBlue(),
                          0);
    }


}