/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.component;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import com.jgoodies.uif.util.ComponentTreeUtils;
import com.jgoodies.uif.util.CompoundIcon;
import com.jgoodies.uif.util.NullIcon;


/**
 * Combines an action button and a popup menu.<p>
 *
 * It is intended to be used in cases where you have a list of
 * frequently used items, which you want to choose from a menu.
 * To quickly choose the most recently used element, you click
 * the action button. This is popular in recent Web browsers.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.5 $
 */

public final class PopupButton extends JPanel {

	private final JButton mainButton;
	private final JPopupMenu popupMenu;

	private AbstractButton arrowButton;
	private boolean mouseIsOver = false;


    // Instance Creation ****************************************************

	/**
	 * Constructs a popup button using the given main button and popup menu.
     *
     * @param mainButton  the button that performs the main action
     * @param popupMenu   the popup menu to choose from
	 */
	public PopupButton(JButton mainButton, JPopupMenu popupMenu) {
        super(null);
		this.mainButton = mainButton;
		this.popupMenu  = popupMenu;
		build();
	}


    // Add components to a tool bar *****************************************

    /**
     * Adds this popup button's two components to the given tool bar
     * individually. Useful because the tool bar may handle buttons
     * as special components and would not work well with a panel.
     * This is the case with the Windows toolbar in XP style.<p>
     *
     * Tries to adjust the arrow button with the main button's height
     * in case the main button is larger due to larger insets.
     *
     * @param toolBar   the tool bar to add the components to
     */
    public void addTo(JToolBar toolBar) {
        toolBar.add(mainButton);
        toolBar.add(arrowButton);
        adjustButtonHeights();
    }



	/**
	 * Builds the popup button component.
	 */
	private void build() {
		arrowButton = createArrowButton();

		mainButton.getModel().addChangeListener(new MainButtonChangeListener());
		mainButton.addMouseListener(new MainButtonMouseListener());
		popupMenu.addPopupMenuListener(new RolloverPopupMenuListener());

		setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.VERTICAL;
		gbc.anchor = GridBagConstraints.WEST;

		gbc.gridwidth = 1;
		gbc.gridheight = GridBagConstraints.REMAINDER;
		gbc.weightx = 0.0;
		add(mainButton, gbc);

		gbc.gridwidth = GridBagConstraints.REMAINDER;
		add(arrowButton, gbc);

		setOpaque(false);
		//updateSize();
	}


	/**
	 * Creates and returns the arrow button.
	 */
	private AbstractButton createArrowButton() {
        Icon mainButtonIcon = mainButton.getIcon();
        int iconHeight = mainButtonIcon != null
            ? mainButtonIcon.getIconHeight()
            : 16;
        Icon arrowIcon = new ArrowIcon();
        Icon compoundIcon = new CompoundIcon(
            new NullIcon(arrowIcon.getIconWidth(), iconHeight),
            arrowIcon,
            CompoundIcon.CENTER);
		AbstractButton button = new ToolBarButton(compoundIcon);
		button.setModel(new DelegatingButtonModel(mainButton.getModel()));
		button.addActionListener(new ArrowButtonActionListener());
		button.addMouseListener(new ArrowButtonMouseListener());
        Insets insets = button.getMargin();
		button.setMargin(new Insets(insets.top, 0, insets.bottom, 0));
		return button;
	}


	@Override
    public float getAlignmentX() {
		return mainButton.getAlignmentX();
	}


	@Override
    public Dimension getMaximumSize() {
		return getPreferredSize();
	}


    /**
     * In addition to the superclass behavior, we update the
     * popup menu that is not in the component tree.
     * And we need readjust the main button and arrow button heights.
     */
	@Override
    public void updateUI() {
		super.updateUI();
		if (null == popupMenu)
			return;

		ComponentTreeUtils.updateComponentTreeUI(popupMenu);
        adjustButtonHeights();
    }


    private void adjustButtonHeights() {
        Dimension d = mainButton.getMinimumSize();
        d.width = arrowButton.getMinimumSize().width;
        arrowButton.setMaximumSize(d);
    }


    // Event Handling *********************************************************


    /**
     * Shows the menu and sets the button to armed.
     */
    private final class ArrowButtonActionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            popupMenu.show(mainButton, 0, mainButton.getHeight());
            arrowButton.getModel().setArmed(true);
        }
    }


    /**
     * Listens to arrow button mouse events and switches
     * the main button's rollover property on and off.
     */
    private final class ArrowButtonMouseListener extends MouseAdapter {
        @Override
        public void mouseEntered(MouseEvent e) {
            mouseIsOver = true;
            mainButton.getModel().setRollover(true);
        }
        @Override
        public void mouseExited(MouseEvent e) {
            mouseIsOver = false;
            mainButton.getModel().setRollover(popupMenu.isVisible());
        }
    }


    /**
     * Listens to main button state change events and
     * repaints the arrow button.
     */
    private final class MainButtonChangeListener implements ChangeListener {
        public void stateChanged(ChangeEvent e) {
            arrowButton.repaint();
        }
    }


    /**
     * Listens to main button mouse events and switches
     * the arrow button's rollover property.
     */
    private final class MainButtonMouseListener extends MouseAdapter {
        @Override
        public void mouseEntered(MouseEvent e) {
            mouseIsOver = true;
            arrowButton.getModel().setRollover(true);
        }
        @Override
        public void mouseExited(MouseEvent e) {
            mouseIsOver = false;
            arrowButton.getModel().setRollover(popupMenu.isVisible());
        }
    }


    /**
     * Listens to popup menu events to update
     * the arrow button's rollover  and pressed state.
     */
    private final class RolloverPopupMenuListener implements PopupMenuListener {
        public void popupMenuCanceled(PopupMenuEvent e) {
            // Implements PopupMenuListener; do nothing
        }
        public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
            arrowButton.getModel().setRollover(mouseIsOver);
            arrowButton.getModel().setPressed(false);
        }
        public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
            arrowButton.getModel().setRollover(true);
            arrowButton.getModel().setPressed(true);
        }
    }


	// Helper Classes *********************************************************

	/**
     * An icon implementation for the arrow button.
     */
    private static final class ArrowIcon implements Icon {

        private static final int ICON_HEIGHT = 4;
        private static final int ICON_WIDTH  = 2 * ICON_HEIGHT + 1;

        public int getIconWidth() {
            return ICON_WIDTH;
        }
        public int getIconHeight() {
            return ICON_HEIGHT;
        }

        public void paintIcon(Component c, Graphics g, int x, int y) {
            AbstractButton b = (AbstractButton) c;
            ButtonModel m = b.getModel();
            int w = getIconWidth() - 2;
            int h = ICON_HEIGHT;

            g.translate(x, y);

            g.setColor(
                UIManager.getColor(
                    m.isEnabled() ? "controlText" : "textInactiveText"));
            for (int i = 0; i < h; i++) {
                g.drawLine(i + 1, i, w - i, i);
            }

            g.translate(-x, -y);
        }
    }


	/**
     * A button model that delegates everything to a delegate.
     */
	private static final class DelegatingButtonModel extends DefaultButtonModel {

		private final ButtonModel delegate;

		private DelegatingButtonModel(ButtonModel delegate) { this.delegate = delegate; }

		@Override
        public boolean isEnabled()	{ return delegate.isEnabled(); 					 }
		@Override
        public boolean isRollover()	{ return delegate.isRollover(); 					 }
		@Override
        public boolean isArmed()	{ return super.isArmed()   || delegate.isArmed();   }
		@Override
        public boolean isPressed()	{ return super.isPressed() || delegate.isPressed(); }

		@Override
        public void setRollover(boolean b) {
            delegate.setRollover(b);
        }

		@Override
        public void setEnabled(boolean b) {
			if (delegate != null)
				delegate.setEnabled(b);
		}

	}

}