/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.component;

import java.awt.Dimension;
import java.awt.Insets;

import javax.swing.*;
import javax.swing.plaf.InsetsUIResource;
import javax.swing.plaf.UIResource;

import com.jgoodies.uif.util.Mode;
import com.jgoodies.uif.util.Modes;

/**
 * A JButton subclass intended for use in tool bars.
 * Adds a special configuration to its superclass.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.8 $
 */
public class ToolBarButton extends JButton {

    private static final char   ELLIPSIS         = '\u2026';
    private static final String ELLIPSIS_STRING  = "...";


    /**
     * Reused for tool bar buttons.
     */
    private static final Insets EMPTY_INSETS = new InsetsUIResource(0, 0, 0, 0);

    /**
     * Holds the mode that is used to set the <em>borderPainted</em>
     * property if the UI changes.
     *
     * @see #getBorderPaintedMode()
     * @see #setBorderPaintedMode(Mode)
     */
    private Mode borderPaintedMode = Modes.LAF_NON_AQUA;

    /**
     * Holds the mode that is used to determine the margin size
     * if the button is configured.
     *
     * @see #getWideMarginMode()
     * @see #setWideMarginMode(Mode)
     */
    private Mode wideMarginMode = Modes.LAF_AQUA;


    // Instance Creation ******************************************************

    /**
     * Constructs a tool bar button without icon.
     * This constructor is intended only for visual builder tools
     * that require an empty constructor. Before this instance is
     * used in the UI, you should set an icon or Action.
     */
    public ToolBarButton() {
        this((Icon) null);
    }

    /**
     * Constructs a tool bar button with the specified icon.
     *
     * @param icon   the icon for the button
     */
    public ToolBarButton(Icon icon) {
        super(icon);
        configureButton();
    }

    /**
     * Constructs tool bar button and sets properties as described
     * by the given action.
     *
     * @param action    provides the properties for this button
     */
    public ToolBarButton(Action action) {
        super(action);
        configureButton();
    }


    // Accessing Properties ***************************************************

    /**
     * Returns this button's border painted mode that can be used to
     * change the <em>borderPainted</em> property if the UI changes.
     *
     * @return the current border painted mode
     */
    public Mode getBorderPaintedMode() {
        return borderPaintedMode;
    }


    /**
     * Sets a new border painted mode that can be used to change the
     * <em>borderPainted</em> property if the UI changes.
     *
     * @param newMode   the mode to be set
     * @throws NullPointerException if the new mode is {@code null}
     */
    public void setBorderPaintedMode(Mode newMode) {
    	if (newMode == null)
    		throw new NullPointerException("The border painted mode must not be null.");
        borderPaintedMode = newMode;
        configureButton();
    }


    /**
     * Returns this button's wide margin mode that can be used
     * to change the button's margin during the configuration.
     *
     * @return the current wide margin mode
     */
    public Mode getWideMarginMode() {
    	return wideMarginMode;
    }


    /**
     * Sets a new border painted mode that can be used to change
     * the <em>borderPainted</em> property if the UI changes.
     *
     * @param newMode   the mode to be set
     * @throws NullPointerException if the new mode is {@code null}
     */
    public void setWideMarginMode(Mode newMode) {
    	if (newMode == null)
    		throw new NullPointerException("The wide margin mode must not be null.");
    	wideMarginMode = newMode;
    	configureButton();
    }


    // Updating the UI *******************************************************

    /**
     * In addition to the superclass behavior that updates the UI
     * this method configures several button properties. For details see
     * {@link ToolBarButton#configureButton(AbstractButton, boolean, boolean)}.
     * This configuration honors the border painted mode
     * and the wide margin mode.<p>
     *
     * This method is invoked during the superclass construction
     * and therefore we check for an uninitialized field.
     */
    @Override
    public void updateUI() {
        super.updateUI();
        if (getBorderPaintedMode() != null) {
        	configureButton();
        }
    }


    // Configuration **********************************************************

    /**
     * Configures an AbstractButton for use in a tool bar.
     *
     * @param button          the button to configure
     * @param borderPainted   true to paint a border, false to not paint it
     * @param wideMargin      true for a wide margin, false for narrow
     */
    public static void configureButton(
    		AbstractButton button,
			boolean borderPainted,
			boolean wideMargin) {
        button.setHorizontalTextPosition(CENTER);
        button.setVerticalTextPosition(BOTTOM);
        button.setAlignmentY(CENTER_ALIGNMENT);
        button.setBorderPainted(borderPainted);
        if (button.getMargin() instanceof UIResource) {
            button.setMargin(getButtonMargin(button, wideMargin));
        }
        button.setMnemonic(0);
        button.setFocusable(false);
    }

    /**
     * Returns the button's margin.
     */
    private static Insets getButtonMargin(AbstractButton button, boolean wideMargin) {
        Dimension defaultIconSize =
            UIManager.getDimension("jgoodies.defaultIconSize");
        Icon icon = button.getIcon();
        if (defaultIconSize == null || icon == null)
            return EMPTY_INSETS;

        int hpad = Math.max(0, defaultIconSize.width  - icon.getIconWidth());
        int vpad = Math.max(0, defaultIconSize.height - icon.getIconHeight());
        int top    = vpad / 2;
        int left   = hpad / 2;
        int bottom = top  + vpad % 2;
        int right  = left + hpad % 2;
        if (wideMargin) {
            top    += 2;
            left   += 2;
            bottom += 2;
            right  += 2;
        }
        return new InsetsUIResource(top, left, bottom, right);
    }

    /*
     * Configures this button.
     */
    private void configureButton() {
        configureButton(
        		this,
				getBorderPaintedMode().enabled(),
				getWideMarginMode().enabled()
        );
        setDefaultCapable(false);
    }

    /**
     * Configures the button's properties from the given action.
     */
    @Override
    protected void configurePropertiesFromAction(Action a) {
        super.configurePropertiesFromAction(a);
        String shortDescription = (String) a.getValue(Action.SHORT_DESCRIPTION);
        if (shortDescription == null) {
            String name = (String) a.getValue(Action.NAME);
            if (name != null) {
                String defaultShortDescription = stripEllipsis(name);
                setToolTipText(defaultShortDescription);
            }
        }
        Icon icon = getIcon();
        if (icon != null) {
            putClientProperty("hideActionText", Boolean.TRUE);
        }
    }


    @Override
    public void setText(String text) {
        // Unlike the superclass, do not set a text in tool bars.
    }


    // Helper Code ************************************************************

    /**
     * Strips a trailing ellipsis - if any - from the given string
     * and returns the stripped string. The ellipsis can be described
     * by three dots "..." or by the ellipsis character &hellip;.<p>
     *
     * Examples: "Print..." -> "Print", "Print\u2026" -> "Print",
     * "About" -> "About"
     *
     * @param withEllipsis  the String with optional ellipsis
     * @return the String without ellipsis
     */
    static String stripEllipsis(String withEllipsis) {
        int length = withEllipsis.length();
        if (withEllipsis.charAt(length - 1) == ELLIPSIS)
            return withEllipsis.substring(0, length - 1);
        else if (withEllipsis.endsWith(ELLIPSIS_STRING))
            return withEllipsis.substring(0, length - ELLIPSIS_STRING.length());
        else
            return withEllipsis;
    }


}