/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.component;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JToggleButton;

import com.jgoodies.uif.action.ToggleAction;
import com.jgoodies.uif.util.Mode;
import com.jgoodies.uif.util.Modes;

/**
 * Provides a default configuration suitable for most toolbar toggle buttons.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.7 $
 */
public final class ToolBarToggleButton extends JToggleButton {

    /**
     * Holds the mode that is used to set the <em>borderPainted</em>
     * property if the UI changes.
     *
     * @see #getBorderPaintedMode()
     * @see #setBorderPaintedMode(Mode)
     */
    private Mode borderPaintedMode = Modes.LAF_NON_AQUA;

    /**
     * Holds the mode that is used to determine the margin size
     * if the button is configured.
     *
     * @see #getWideMarginMode()
     * @see #setWideMarginMode(Mode)
     */
    private Mode wideMarginMode = Modes.LAF_AQUA;


    // Instance Creation ******************************************************

    /**
     * Constructs a toggle button for use in tool bars.
     */
    public ToolBarToggleButton() {
        configureButton();
    }

    /**
     * Constructs a toggle button with the specified icon for use in tool bars.
     *
     * @param icon   the icon for the button
     */
    public ToolBarToggleButton(Icon icon) {
    	super(icon);
        configureButton();
    }

    /**
     * Constructs a <code>ToolBarButton</code> using
     * the specified <code>ToggleAction</code>.
     *
     * @param toggleAction   a ToggleAction that describes what to perform
     *     and how to label the button
     */
    public ToolBarToggleButton(ToggleAction toggleAction) {
        super(toggleAction);
        configureButton();
        int displayedMnemonicIndex = getDisplayedMnemonicIndex();
        setModel(toggleAction.createButtonModel());
        setDisplayedMnemonicIndex(displayedMnemonicIndex);
    }

    // Accessing Properties ***************************************************

    /**
     * Returns this button's border painted mode that can be used to
     * change the <em>borderPainted</em> property if the UI changes.
     *
     * @return the current border painted mode
     */
    public Mode getBorderPaintedMode() {
    	return borderPaintedMode;
    }


    /**
     * Sets a new border painted mode that can be used to change the
     * <em>borderPainted</em> property if the UI changes.
     *
     * @param newMode   the mode to be set
     * @throws NullPointerException if the new mode is {@code null}
     */
    public void setBorderPaintedMode(Mode newMode) {
    	if (newMode == null)
    		throw new NullPointerException("The border painted mode must not be null.");
    	borderPaintedMode = newMode;
    	configureButton();
    }


    /**
     * Returns this button's wide margin mode that can be used
     * to change the button's margin during the configuration.
     *
     * @return the current wide margin mode
     */
    public Mode getWideMarginMode() {
    	return wideMarginMode;
    }


    /**
     * Sets a new border painted mode that can be used to change
     * the <em>borderPainted</em> property if the UI changes.
     *
     * @param newMode   the mode to be set
     * @throws NullPointerException if the new mode is {@code null}
     */
    public void setWideMarginMode(Mode newMode) {
    	if (newMode == null)
    		throw new NullPointerException("The wide margin mode must not be null.");
    	wideMarginMode = newMode;
    	configureButton();
    }


    // Updating the UI *******************************************************

    /**
     * In addition to the superclass behavior that updates the UI
     * this method configures several button properties. For details see
     * <code>ToolBarButton#configureButton(AbstractButton, boolean, boolean)</code>.
     * This configuration honors the border painted mode
     * and the wide margin mode.<p>
     *
     * This method is invoked during the superclass construction
     * and therefore we check for an uninitialized field.
     */
    @Override
    public void updateUI() {
        super.updateUI();
        if (getBorderPaintedMode() != null) {
        	configureButton();
        }
    }


    // Configuration *********************************************************

    /*
     * Configures this button.
     */
    private void configureButton() {
    	ToolBarButton.configureButton(
    			this,
				getBorderPaintedMode().enabled(),
				getWideMarginMode().enabled()
    	);
    }

    /**
     * Configures the button's properties from the given action.
     */
    @Override
    protected void configurePropertiesFromAction(Action a) {
        super.configurePropertiesFromAction(a);
        String shortDescription = (String) a.getValue(Action.SHORT_DESCRIPTION);
        if (shortDescription == null) {
            String name = (String) a.getValue(Action.NAME);
            String defaultShortDescription = ToolBarButton.stripEllipsis(name);
            setToolTipText(defaultShortDescription);
        }
        Icon icon = getIcon();
        if (icon != null) {
            setText(null); // Would like to say: putClientProperty("hideActionText", Boolean.TRUE);
        }
    }

}