/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.component;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JButton;

import com.jgoodies.uif.util.Mode;

/**
 * A JButton subclass that adds support for narrow margins
 * and mode-dependent opaqueness.
 *
 * Narrow margins are useful to save screen real estate if a button
 * is embedded in a form, or if you assign the same width to multiple
 * buttons, but some buttons have large text labels.
 * The narrow setting is implemented using the Swing client properties.
 * Note that this feature is currently only supported by the JGoodies
 * look&amp;feel implementations.<p>
 *
 * If the opaque mode is set to a non-{@code null} value,
 * the opaque state will be set according to the mode's enablement
 * each time the UI changes.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.8 $
 *
 * @see com.jgoodies.uif.util.Mode
 * @see com.jgoodies.uif.util.Modes
 */

public class UIFButton extends JButton {

    /**
     * Holds the mode that is used to set the <em>opaque</em>
     * property if the UI changes.
     *
     * @see #getOpaqueMode()
     * @see #setOpaqueMode(Mode)
     */
    private Mode opaquenessMode;


    // Instance Creation ******************************************************

    /**
     * Constructs a button with no set text or icon.
     */
    public UIFButton() {
        super();
    }

    /**
     * Constructs a button with an icon.
     *
     * @param icon  the Icon image to display on the button
     */
    public UIFButton(Icon icon) {
        super(icon);
    }


    /**
     * Constructs a button with text.
     *
     * @param text  the text of the button
     */
    public UIFButton(String text) {
        super(text);
    }


    /**
     * Constructs a button where properties are taken from the
     * <code>Action</code> supplied.
     *
     * @param a the <code>Action</code> used to specify the new button
     */
    public UIFButton(Action a) {
        super(a);
    }


    /**
     * Constructs a button with the given initial text and icon.
     *
     * @param text  the text of the button
     * @param icon  the Icon image to display on the button
     */
    public UIFButton(String text, Icon icon) {
        super(text, icon);
    }


    // Accessing Properties ***************************************************

    /**
     * Returns this button's opaqueness mode that can be used to
     * change the <em>opaque</em> property if the UI changes.
     *
     * @return the current opaqueness mode
     */
    public final Mode getOpaqueMode() {
        return opaquenessMode;
    }


    /**
     * Sets a new opaqueness mode that can be used to change the
     * <em>opaque</em> property if the UI changes.
     *
     * @param newMode   the mode to be set
     */
    public final void setOpaqueMode(Mode newMode) {
        opaquenessMode = newMode;
        setOpaque(getOpaqueMode().enabled());
    }


    /**
     * In addition to the superclass behavior that updates the UI
     * this method sets the opaque property to the opaque's mode
     * current enabled value - if an opaque mode exists at all.
     */
    @Override
    public void updateUI() {
        super.updateUI();
        if (getOpaqueMode() != null) {
            setOpaque(getOpaqueMode().enabled());
        }
    }


}