/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.component;

import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import javax.swing.JLabel;
import javax.swing.plaf.FontUIResource;

/**
 * In addition to the superclass JLabel, this class can render
 * the label's text with anti-aliasing (AA) enabled. And it can use
 * a custom font style and a custom font size. These attributes
 * will be restored if the look&amp;feel changes.
 * Provides bound Java Bean properties <em>fontStyle</em>, <em>fontSizeDelta</em>
 * and <em>antiAliased</em>.<p>
 *
 * If AA is enabled, it'll switch on the text anti-aliasing before
 * painting the label. Subpixel AA is not used and not detected.
 * And so, this label's AA mode will replace subpixel AA with plain AA,
 * leading to a potentially decreased readability. Therefore the AA
 * should be used only for larger font sizes where AA increases the readability,
 * and the differences between AA and subpixel AA are minimal.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.6 $
 */

public class UIFLabel extends JLabel {

    // Names of Bound Bean Properties *****************************************

    public static final String PROPERTYNAME_FONT_STYLE      = "fontStyle";
    public static final String PROPERTYNAME_FONT_SIZE_DELTA = "fontSizeDelta";
    public static final String PROPERTYNAME_ANTI_ALIASED    = "antiAliased";


    // Private Fields *********************************************************

    /**
     * The style of the font used to render this label's text.
     */
    private int fontStyle;

    /**
     * A delta in pixels that is added to the default font size.
     * Useful to display the text larger or smaller than the default.
     */
    private int fontSizeDelta;

    /**
     * Indicates whether the text will be rendered with anti alias enabled or not.
     */
    private boolean antiAliased;


    // Instance Creation ******************************************************

    /**
     * Constructs a UIFLabel for the given text, font style,
     * font extra size, and anti-aliasing mode.
     *
     * @param text          the initial text
     * @param antiAliased   true enables anti alias, false disables it
     */
    public UIFLabel(String text, boolean antiAliased) {
        this(text, Font.PLAIN, 0, antiAliased);
    }

    /**
     * Constructs a UIFLabel for the given text, font style,
     * font size delta, and anti-aliasing mode.
     *
     * @param text           the initial text
     * @param fontStyle      the style of the font to use: plain, bold, etc.
     * @param fontSizeDelta  a delta in pixel relative to the default font size
     * @param antiAliased    true enables anti alias, false disables it
     */
    public UIFLabel(
        String text,
        int fontStyle,
        int fontSizeDelta,
        boolean antiAliased) {
        super(text);
        this.fontStyle     = fontStyle;
        this.fontSizeDelta = fontSizeDelta;
        this.antiAliased   = antiAliased;
        updateUI();
    }


    // Accessors **************************************************************

    /**
     * Returns this label's font style. Useful to force a bold text style.
     *
     * @return this label's font style
     * @see Font
     */
    public int getFontStyle() {
        return fontStyle;
    }

    /**
     * Sets a new font style that will be used to render this label's text.
     *
     * @param newFontStyle   the font style to set
     */
    public void setFontStyle(int newFontStyle) {
        int oldFontStyle = getFontStyle();
        fontStyle = newFontStyle;
        updateUI();
        revalidate();
        repaint();
        firePropertyChange(PROPERTYNAME_FONT_STYLE, oldFontStyle, newFontStyle);
    }


    /**
     * Returns the delta of this label's text font relative to the default font.
     *
     * @return the delta between the default font size and this label's font size
     */
    public int getFontSizeDelta() {
        return fontSizeDelta;
    }

    /**
     * Sets a new font size delta that will make this label's text larger
     * or smaller than the default font.
     *
     * @param newFontSizeDelta   the font size delta to set
     */
    public void setFontSizeDelta(int newFontSizeDelta) {
        int oldFontSizeDelta = getFontSizeDelta();
        fontSizeDelta = newFontSizeDelta;
        updateUI();
        repaint();
        revalidate();
        firePropertyChange(PROPERTYNAME_FONT_SIZE_DELTA,
                oldFontSizeDelta,
                newFontSizeDelta);
    }


    /**
     * Returns whether this label's text will be rendered
     * with anti-aliasing enabled.
     *
     * @return true if anti-aliasing is enabled, false if disabled
     */
    public boolean isAntiAliased() {
        return antiAliased;
    }

    /**
     * Enables or disabled the anti-aliasing of this label's text.
     *
     * @param antiAliased   true to enable anti-aliasing, false to disable it
     */
    public void setAntiAliased(boolean antiAliased) {
        boolean oldAntiAliased = isAntiAliased();
        this.antiAliased = antiAliased;
        revalidate();
        repaint();
        firePropertyChange(PROPERTYNAME_ANTI_ALIASED, oldAntiAliased, antiAliased);
    }


    // Overridden Superclass Behavior *****************************************

    /**
     * Paints the label: switches on anti-aliasing if necessary.
     *
     * @param g    the Graphics object to render on
     */
    @Override
    public void paint(Graphics g) {
        if (!isAntiAliased()) {
            super.paint(g);
            return;
        }

        Graphics2D g2 = (Graphics2D) g;

        final RenderingHints.Key key = RenderingHints.KEY_TEXT_ANTIALIASING;
        Object newAAHint = RenderingHints.VALUE_TEXT_ANTIALIAS_ON;
        Object oldAAHint = g2.getRenderingHint(key);
        if (newAAHint != oldAAHint) {
            g2.setRenderingHint(key, newAAHint);
        } else {
            oldAAHint = null;
        }
        super.paint(g2);
        if (oldAAHint != null) {
            g2.setRenderingHint(key, oldAAHint);
        }
    }


    /**
     * Restores the fonts after the UI has changed.
     */
    @Override
    public void updateUI() {
        super.updateUI();
        Font font = getFont();
        if (0 == getFontSizeDelta()) {
            if (font.getStyle() != fontStyle)
                setFont(new FontUIResource(font.deriveFont(fontStyle)));
        } else
            setFont(
                new FontUIResource(
                    new Font(
                        font.getName(),
                        fontStyle,
                        font.getSize() + getFontSizeDelta())));
    }

}