/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.component;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JRadioButtonMenuItem;

import com.jgoodies.uif.action.ToggleAction;
import com.jgoodies.uif.util.Mode;

/**
 * A <code>JRadioButtonMenuItem</code> subclass that adds supports for
 * temporarily invisible icons. Useful to hide icons on a specific platform or
 * for a specific look&amp;feel. For example the Mac Aqua style guide
 * recommend to hide icons unless there's a special context such as
 * a history menu.<p>
 *
 * The menu item's icon set can be made visible or invisible directly.
 * But typically you should set the icon visible mode and use a
 * {@link com.jgoodies.uif.util.Mode} that describes when to
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.6 $
 *
 * @see com.jgoodies.uif.util.Mode
 * @see com.jgoodies.uif.util.Modes
 */

public final class UIFRadioButtonMenuItem extends JRadioButtonMenuItem {

    /**
     * Describes whether icons can be shown or are hidden.
     * By default
     */
    private boolean iconVisible = true;

    private Icon backedIcon;
    private Icon backedSelectedIcon;
    private Icon backedDisabledIcon;
    private Icon backedDisabledSelectedIcon;
    private Icon backedPressedIcon;
    private Icon backedRollOverIcon;
    private Icon backedRollOverSelectedIcon;

    /**
     * Holds the mode that is used to set the <em>iconVisible</em>
     * property each time the UI changes. By default this mode
     * isn't set at all, and so the icon visibility won't be changed.
     *
     * @see #getIconVisibleMode()
     * @see #setIconVisibleMode(Mode)
     */
    private Mode iconVisibleMode;


	// Instance Creation *****************************************************

    /**
     * Constructs a menu item with no set text or icon.
     */
    public UIFRadioButtonMenuItem() {
        this(null, null, false);
    }


    /**
     * Constructs a menu item with the specified icon.
     *
     * @param icon   the icon of the menu item
     */
    public UIFRadioButtonMenuItem(Icon icon) {
        this(null, icon, false);
    }


    /**
     * Constructs a menu item with the specified text.
     *
     * @param text    the text of the menu item
     */
    public UIFRadioButtonMenuItem(String text) {
        this(text, null, false);
    }


    /**
     * Constructs a radio button menu item with the specified text
     * and selection state.
     *
     * @param text the text of the menu item
     * @param selected the selected state of the menu item
     */
    public UIFRadioButtonMenuItem(String text, boolean selected) {
        this(text, null, selected);
    }

    /**
     * Constructs a menu item with the specified icon.
     *
     * @param icon   the icon of the menu item
     * @param selected the selected state of the menu item
     */
    public UIFRadioButtonMenuItem(Icon icon, boolean selected) {
        this(null, icon, selected);
    }


    /**
     * Constructs a radio button menu item whose properties are taken
     * from the specified <code>Action</code>.
     *
     * @param action  the action that provides the properties
     */
    public UIFRadioButtonMenuItem(Action action) {
        super(action);
    }


    /**
     * Constructs a radio button menu item whose properties are taken
     * from the specified <code>ToggleAction</code>. Also sets
     * the menu item's model to the model provided by the toggle action.
     *
     * @param toggleAction  the action that provides the properties
     */
    public UIFRadioButtonMenuItem(ToggleAction toggleAction) {
        super(toggleAction);
        int displayedMnemonicIndex = getDisplayedMnemonicIndex();
        setModel(toggleAction.createButtonModel());
        setDisplayedMnemonicIndex(displayedMnemonicIndex);
    }


    /**
     * Constructs a menu item with the specified text and icon.
     *
     * @param text    the text of the menu item
     * @param icon    the icon of the menu item
     */
    public UIFRadioButtonMenuItem(String text, Icon icon) {
        this(text, icon, false);
    }


    /**
     * Constructs a radio button menu item that has the specified
     * text, image, and selection state.  All other constructors
     * defer to this one.
     *
     * @param text  the string displayed on the radio button
     * @param icon  the image that the button should display
     * @param selected  if true, the button is initially selected;
     *                  otherwise, the button is initially unselected
     */
    public UIFRadioButtonMenuItem(String text, Icon icon, boolean selected) {
        super(text, icon, selected);
    }


    /**
     * Constructs a menu item with the specified text and
     * keyboard mnemonic.
     *
     * @param text     the text of the menu item
     * @param mnemonic the keyboard mnemonic for the menu item
     */
    public UIFRadioButtonMenuItem(String text, int mnemonic) {
        this(text, null, false);
        setMnemonic(mnemonic);
    }


    /**
     * Constructs a menu item with the specified text and
     * keyboard mnemonic.
     *
     * @param text     the text of the menu item
     * @param mnemonic the keyboard mnemonic for the menu item
     * @param selected  if true, the button is initially selected;
     *                  otherwise, the button is initially unselected
     */
    public UIFRadioButtonMenuItem(String text, int mnemonic, boolean selected) {
        this(text, null, selected);
        setMnemonic(mnemonic);
    }


    /**
     * Constructs a menu item with the specified text, icon and
     * keyboard mnemonic.
     *
     * @param text     the text of the menu item
     * @param icon    the icon of the menu item
     * @param mnemonic the keyboard mnemonic for the menu item
     */
    public UIFRadioButtonMenuItem(String text, Icon icon, int mnemonic) {
        this(text, icon, false);
        setMnemonic(mnemonic);
    }


    /**
     * Constructs a menu item with the specified text, icon and
     * keyboard mnemonic.
     *
     * @param text     the text of the menu item
     * @param icon    the icon of the menu item
     * @param mnemonic the keyboard mnemonic for the menu item
     * @param selected  if true, the button is initially selected;
     *                  otherwise, the button is initially unselected
     */
    public UIFRadioButtonMenuItem(String text, Icon icon, int mnemonic, boolean selected) {
        this(text, icon, selected);
        setMnemonic(mnemonic);
    }


	// Accessing Properties **************************************************

    /**
     * Answers whether icons are currently visible or hidden.
     *
     * @return true if icons are visible, false if invisible
     */
    public boolean isIconVisible() {
        return iconVisible;
    }


    /**
     * Shows or hides this item's icons. In case the icons are hidden
     * they are stored in private fields so they can be restored later.<p>
     *
     * The current implementations override icons during the restoring proces
     * that have been set between the hide and show actions.
     *
     * @param newVisible  true for visible, false for invisible
     */
    public void setIconVisible(boolean newVisible) {
        boolean oldVisible = isIconVisible();
        if (oldVisible == newVisible)
            return;
        iconVisible = newVisible;
        if (newVisible) {
            setIcon                (backedIcon);
            setSelectedIcon        (backedSelectedIcon);
            setDisabledIcon        (backedDisabledIcon);
            setDisabledSelectedIcon(backedDisabledSelectedIcon);
            setPressedIcon         (backedPressedIcon);
            setRolloverIcon        (backedRollOverIcon);
            setRolloverSelectedIcon(backedRollOverSelectedIcon);
        } else {
            backedIcon                 = getIcon();
            backedSelectedIcon         = getSelectedIcon();
            backedDisabledIcon         = getDisabledIcon();
            backedDisabledSelectedIcon = getDisabledSelectedIcon();
            backedPressedIcon          = getPressedIcon();
            backedRollOverIcon         = getRolloverIcon();
            backedRollOverSelectedIcon = getRolloverSelectedIcon();
            setIcon                (null);
            setSelectedIcon        (null);
            setDisabledIcon        (null);
            setDisabledSelectedIcon(null);
            setPressedIcon         (null);
            setRolloverIcon        (null);
            setRolloverSelectedIcon(null);
        }
        firePropertyChange("iconVisible", oldVisible, newVisible);
    }


    /**
	 * Returns the current icon mode that determines
	 * whether this item shows or hides an icon - if any.
	 *
	 * @return the current icon mode
	 */
	public Mode getIconVisibleMode() {
	    return iconVisibleMode;
	}


	/**
	 * Sets a new icon visible mode that determines whether
	 * this item shows or hides an icon - if any.
	 *
	 * @param newMode   the icon visible mode to be set
	 */
	public void setIconVisibleMode(Mode newMode) {
	    Mode oldMode = getIconVisibleMode();
	    iconVisibleMode = newMode;
	    if (getIconVisibleMode() != null) {
	    	setIconVisible(getIconVisibleMode().enabled());
	    }
	    firePropertyChange("iconVisibleMode", oldMode, newMode);
	}


	// Overriding Superclass Behavior ****************************************

	/**
	 * In addition to the superclass behavior, this class
	 * sets the displayed mnemonic index if and only if the
	 * action provides one under key <code>"mnemonicIndex"</code>.
	 *
	 * @param a the <code>Action</code> from which to get the properties,
	 *		or {@code null}
	 *
	 * @see Action
	 */
	@Override
    protected void configurePropertiesFromAction(Action a) {
	    super.configurePropertiesFromAction(a);
	    UIFMenuItem.setDisplayedMnemonicIndex(this, a);
	}


	/**
	 * In addition to the superclass behavior that updates the UI
	 * this method adds or removes the item's icon according to the
	 * icon visible mode's current enabled value - if any.
	 */
	@Override
    public void updateUI() {
	    super.updateUI();
	    if (getIconVisibleMode() != null) {
	        setIconVisible(getIconVisibleMode().enabled());
	    }
	}


}