/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.laf;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.prefs.Preferences;

import javax.swing.LookAndFeel;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.UIManager.LookAndFeelInfo;

import com.jgoodies.looks.LookUtils;
import com.jgoodies.looks.Options;
import com.jgoodies.uif.application.ApplicationContext;
import com.jgoodies.uif.util.ComponentTreeUtils;

/**
 * A helper class that configures Swing related look issues:<ol>
 * <li>installs new look and feels.</li>
 * <li>sets system properties,</li>
 * <li>restores a look and feel with an optional theme, and</li>
 * <li>overrides UI defaults.</li>
 * </ol>
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.12 $
 *
 * @see     LookChoiceStrategy
 * @see     LookChoiceStrategies
 * @see     LookConfiguration
 * @see     LookConfigurations
 * @see     javax.swing.UIManager
 */
public final class ExtUIManager {

    /**
     * A key for the user preferences used to store and restore
     * the classname of the default Look&amp;Feel.
     *
     * @see #getDefaultLookClassName()
     * @see #getDefaultLookAndFeel()
     * @see #getStoredLookConfiguration()
     */
    public static final String DEFAULT_LAF_KEY = "laf.default";

    private static final Logger LOGGER = Logger.getLogger("ExtUIManager");

    /**
     * Holds the current LookChoiceStrategy that is used
     * to provide a class name for the default L&amp;F
     * in case there's no default class name stored in the preferences.
     *
     * @see #getDefaultLookClassName()
     */
    private static LookChoiceStrategy lookChoiceStrategy =
        LookChoiceStrategies.SYSTEM;

    /**
     * Holds the lazily created LookConfigurations.
     */
    private static LookConfigurations lookConfigurations;


    // Instance Creation ******************************************************

    private ExtUIManager() {
        // Override the default constructor - prevents instantiation.
    }


    // Public API *************************************************************

    /**
     * Returns the current LookChoiceStrategy.
     *
     * @return the current LookChoiceStrategy.
     */
    public static LookChoiceStrategy getLookChoiceStrategy() {
        return lookChoiceStrategy;
    }

    /**
     * Sets a LookChoiceStrategy that will be used to lookup
     * the very first look, if no look has been stored.
     *
     * @param strategy the LookChoiceStrategy used to lookup the very first look
     */
    public static void setLookChoiceStrategy(LookChoiceStrategy strategy) {
        lookChoiceStrategy = strategy;
    }

    /**
     * Returns a clone of the LookConfigurations object which is lazily
     * initialized with default values.
     *
     * @return a clone of the LookConfigurations
     */
    public static LookConfigurations getLookConfigurations() {
        if (lookConfigurations == null) {
            lookConfigurations = LookConfigurations.restoreFrom(
                    ApplicationContext.getInstance().getUserPreferences());
        }
        return (LookConfigurations) lookConfigurations.clone();
    }


    /**
     * Sets a new LookConfigurations object, stores them in the
     * ApplicationContext's user preferences, sets the look and theme
     * and finally updates all component tree UIs.
     * Does nothing if the new LookConfigurations equals the old configurations
     *
     * @param newLookConfigurations  the LookConfigurations to set
     */
    public static void setLookConfigurations(LookConfigurations newLookConfigurations) {
        if (newLookConfigurations.equals(lookConfigurations)) {
            return;
        }
        lookConfigurations = newLookConfigurations;
        newLookConfigurations.storeIn(
                ApplicationContext.getInstance().getUserPreferences());
        setLookAndTheme(newLookConfigurations.getDefaultConfiguration());
        ComponentTreeUtils.updateAllUIs();
    }

    /**
     * Installs the Plastic L&amp;Fs, enables system fonts, restores
     * the LookConfigurations from the application preferences
     * and finally sets the selected look and theme.<p>
     *
     * This method shall be called before the any UI components are created.
     * Therefore we don't need to update the component tree UIs.
     * In contrast, if you set a new LookConfigurations object later,
     * we store it in the prefs and update the component tree UIs.
     *
     * @see #setLookConfigurations(LookConfigurations)
     * @see #setStoredLookConfiguration(LookConfiguration)
     */
    public static void setup() {
        // Install extra looks
        UIManager.installLookAndFeel("JGoodies Plastic",    Options.PLASTIC_NAME);
        UIManager.installLookAndFeel("JGoodies Plastic 3D", Options.PLASTIC3D_NAME);
        UIManager.installLookAndFeel("JGoodies Plastic XP", Options.PLASTICXP_NAME);

        UIManager.put("ApplicationContext.useSystemFontSettings", Boolean.TRUE);

        setLookAndTheme(getStoredLookConfiguration());
    }


    // Default and Stored Look & Feel *****************************************

    /**
     * Looks up and returns the class name of the Look&amp;Feel that
     * has been stored in the user preferences.
     *
     * @return the class name of the stored Look&amp;Feel.
     */
    public static String getStoredLookClassName() {
        return ApplicationContext.getInstance().getUserPreferences().get(
                DEFAULT_LAF_KEY, null);
    }


    /**
     * Looks up and returns the Look&amp;Feel name as provided by the current LookChoiceStrategy.
     *
     * @return the class name of the LookChoiceStrategy's default Look&amp;Feel.
     */
    public static String getDefaultLookClassName() {
        return getLookChoiceStrategy().getDefaultLookClassName();
    }


    /**
     * Looks up the class name of the default Look&amp;Feel,
     * then create and returns an instance of this class.
     * Returns the UIManager's current Look&amp;Feel in case the above fails.
     *
     * @return the default Look&amp;Feel
     */
    public static LookAndFeel getStoredLookAndFeel() {
        String storedLafClassName = getStoredLookClassName();
        if (storedLafClassName == null) {
            return getDefaultLookAndFeel();
        }
        LookAndFeel laf = createLookAndFeelInstance(storedLafClassName);
        if (laf != null) {
            return laf;
        }
        LOGGER.warning("Could not create the stored default L&F " + storedLafClassName);
        // Don't complain about it again. Store the default laf.
        LookAndFeel defaultLaf = getDefaultLookAndFeel();
        setStoredLookAndFeel(defaultLaf);
        return defaultLaf;
    }


    /**
     * Looks up the class name of the default Look&amp;Feel,
     * then create and returns an instance of this class.
     * Returns the UIManager's current Look&amp;Feel in case the above fails.
     *
     * @return the default Look&amp;Feel
     */
    public static LookAndFeel getDefaultLookAndFeel() {
        String defaultLafClassName = getDefaultLookClassName();
        LookAndFeel laf = createLookAndFeelInstance(defaultLafClassName);
        if (laf != null) {
            return laf;
        }
        LOGGER.info("Could not create the default L&F " + defaultLafClassName);
        return UIManager.getLookAndFeel();
    }


    /**
     * Looks up and returns the default LookConfiguration,
     * that is the default Look&amp;Feel plus theme - if any.
     *
     * @return the default LookConfiguration, i.e. L&amp;F plus theme.
     */
    public static LookConfiguration getStoredLookConfiguration() {
        return LookConfiguration.restoreFrom(
                getStoredLookAndFeel(),
                ApplicationContext.getInstance().getUserPreferences());
    }


    /**
     * Stores the given LookAndFeel as default in the user preferences.
     *
     * @param laf   the LookAndFeel to be set as default
     */
    public static void setStoredLookAndFeel(LookAndFeel laf) {
        if (laf != null) {
            Preferences prefs = ApplicationContext.getInstance().getUserPreferences();
            prefs.put(DEFAULT_LAF_KEY, laf.getClass().getName());
        }
    }


    /**
     * Sets the LookConfiguration used as default. Used in this class'
     * <code>#setup</code> to determine the default look&amp;feel plus theme.
     * Also used when restoring look configurations from the user preferences.
     *
     * @param lookConfiguration  the configuration used as default
     */
    public static void setStoredLookConfiguration(LookConfiguration lookConfiguration) {
        setStoredLookAndFeel(lookConfiguration.getLookAndFeel());
    }


    // Look and Feel Instantiation ******************************************

    /**
     * Creates and returns an instance of <code>LookAndFeel</code> for the
     * specified class name. Before the class is loaded, the given class name
     * may be replaced as defined in the JGoodies Looks <code>Options</code>
     * class. For example the Sun Windows L&amp;F is replaced by the
     * JGoodies Windows L&amp;F.
     *
     * @param className   the name of the class to be instantiated
     * @return the LookAndFeel instance for the given class name,
     *     a replacement applied
     */
    public static LookAndFeel createLookAndFeelInstance(String className) {
        String replacementClassName =
            Options.getReplacementClassNameFor(className);
        if (replacementClassName == null) {
            return null;
        }
        try {
            Class clazz = Class.forName(replacementClassName);
            return (LookAndFeel) clazz.newInstance();
        } catch (Throwable t) {
            String message =
                "Class name="
                    + className
                    + "\nReplacement class name="
                    + replacementClassName;
            LOGGER.log(Level.INFO, message, t);
        }
        return null;
    }


    /**
     * Looks up and returns a list of supported look&amp;feels.
     *
     * @return a list of supported look&amp;feels.
     */
    public static List lookupSupportedLookAndFeelInstances() {
        UIManager.LookAndFeelInfo[] lafInfos =
            UIManager.getInstalledLookAndFeels();
        List result = new ArrayList(lafInfos.length);
        for (LookAndFeelInfo element : lafInfos) {
            String className = element.getClassName();
            LookAndFeel laf = createLookAndFeelInstance(className);
            if ((laf != null) && (laf.isSupportedLookAndFeel())) {
                result.add(laf);
            }
        }
        return result;
    }


    // Setting the Look And Theme *******************************************

    private static void setLookAndTheme(LookConfiguration configuration) {
        setLookAndTheme(configuration.getLookAndFeel(), configuration.getTheme());
    }


    /**
     * Sets a look and theme. Wraps <code>LookUtils.setLookAndTheme</code>
     * with an exception handling.
     *
     * @see LookUtils#setLookAndTheme(javax.swing.LookAndFeel, java.lang.Object)
     */
    private static void setLookAndTheme(LookAndFeel laf, Object theme) {
        try {
            LOGGER.config("Setting L&F: " + laf.getName());
            LookUtils.setLookAndTheme(laf, theme);
        } catch (UnsupportedLookAndFeelException e) {
            LOGGER.warning("Can't set unsupported look and feel:" + laf.getName());
        }
    }


    // Accessing the Supported Look and Feels *******************************

    /**
     * Holds a lazily created List of the supported LookAndFeels.
     * This list can be eagerly initialized by means of the
     * EagerInitializer.
     *
     * @see #getSupportedLookAndFeelInstances()
     */
    private static List supportedLafs;

    /**
     * Lazily initializes and returns the list of supported look and feels
     * sorted by name.
     *
     * @return a List of the supported look&amp;feels
     */
    public static List getSupportedLookAndFeelInstances() {
        if (supportedLafs == null) {
            supportedLafs = ExtUIManager.lookupSupportedLookAndFeelInstances();
            Collections.sort(supportedLafs, new LafNameComparator());
        }
        return supportedLafs;
    }


    /**
     * Compares two LookAndFeel instances using their uppercase names.
     */
    private static final class LafNameComparator implements Comparator {
        public int compare(Object o1, Object o2) {
            LookAndFeel laf1 = (LookAndFeel) o1;
            LookAndFeel laf2 = (LookAndFeel) o2;
            return laf1.getName().toUpperCase().compareTo(
                laf2.getName().toUpperCase());
        }
    }


}