/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.laf;

import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.prefs.Preferences;

import javax.swing.LookAndFeel;

import com.jgoodies.binding.beans.Model;
import com.jgoodies.looks.LookUtils;
import com.jgoodies.looks.plastic.PlasticLookAndFeel;

/**
 * Describes a Look and Feel with its optional color theme.
 * Future versions may support font hints and font size hints.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.6 $
 *
 * @see	LookConfigurations
 * @see	java.util.prefs.Preferences
 * @see javax.swing.LookAndFeel
 */
public final class LookConfiguration extends Model implements Cloneable {

    // Names of the Bound Bean Properties *************************************

    /**
     * The name of the bound read-write property <em>theme</em>.
     */
    public static final String PROPERTYNAME_THEME  = "theme";

    /**
     * The name of the bound read-only property <em>themes</em>.
     */
    public static final String PROPERTYNAME_THEMES = "themes";


    // ************************************************************************

    private static final String THEME_KEY_PREFIX = "laf.themeName.";
    private static final Logger LOGGER = Logger.getLogger("LookConfiguration");

    private static final List PLASTIC_THEMES =
        Collections.unmodifiableList(PlasticLookAndFeel.getInstalledThemes());

    private static final List NO_THEMES =
        Collections.EMPTY_LIST;


    // ************************************************************************

    /**
     * Refers to the LookAndFeel instance this configuration describes.
     */
    private final LookAndFeel laf;

    /**
     * Holds an optional theme associated with the LookAndFeel.
     */
    private Object theme;


    // Instance Creation ******************************************************

    /**
     * Constructs a LookConfiguration for the specified LookAndFeel.
     *
     * @param laf   the L&amp;f instance this configuration describes
     */
    public LookConfiguration(LookAndFeel laf) {
        this(laf, null);
    }

    /**
     * Constructs a LookConfiguration for the specified LookAndFeel and theme.
     *
     * @param laf    the L&amp;f instance this configuration describes
     * @param theme  the L&amp;f's currently selected theme
     */
    public LookConfiguration(LookAndFeel laf, Object theme) {
        if (laf == null)
            throw new NullPointerException("LookAndFeel must not be null.");
        this.laf = laf;
        this.theme = theme != null ? theme : LookUtils.getDefaultTheme(laf);
    }


    /**
     * Returns a LookConfiguration for the specified LookAndFeel
     * with the theme - if any - read from the given Preferences.
     *
     * @param laf   the L&amp;f instance this configuration describes
     * @param prefs the Preferences to read the theme from
     *
     * @return the LookConfiguration read from the given Preferences
     */
    static LookConfiguration restoreFrom(LookAndFeel laf, Preferences prefs) {
        return new LookConfiguration(laf, restoreThemeFrom(laf, prefs));
    }


    // Accessors **************************************************************

    /**
     * Returns the LookAndFeel described by this configuration.
     *
     * @return the LookAndFeel described by this configuration.
     */
    public LookAndFeel getLookAndFeel() {
        return laf;
    }


    /**
     * Returns the list of themes available for this configuration's L&amp;F.
     * Currently only Plastic L&amp;Fs return a list, all other L&amp;Fs
     * return an empty list.
     *
     * @return a List of themes available for this configuration's L&amp;F.
     */
    public List getThemes() {
        return (laf instanceof PlasticLookAndFeel)
            ? PLASTIC_THEMES
            : NO_THEMES;
    }


    /**
     * Returns the theme associated with this configuration's L&amp;F,
     * null if none.
     *
     * @return the theme associated with this configuration's L&amp;F
     */
    public Object getTheme() {
        return theme;
    }


    /**
     * Associates the given theme with this configuration's L&amp;F.
     *
     * @param newTheme  the theme to be associated with the L&amp;F
     */
    public void setTheme(Object newTheme) {
        Object oldTheme = getTheme();
        this.theme = newTheme;
        firePropertyChange(PROPERTYNAME_THEME, oldTheme, newTheme);
    }


    // Misc *******************************************************************

    /**
     * Stores the configuration in the specified Preferences.
     *
     * @param prefs   the Preferences node to the this configuration in
     */
    void storeIn(Preferences prefs) {
        LOGGER.config("Storing the LookConfiguration for " + laf);

        if (getTheme() != null) {
            prefs.put(themeKey(laf), theme.getClass().getName());
        }
    }

    /**
     * Restores the configuration for the specified LookAndFeel
     * from the given Preferences.
     *
     * @return the restored theme - if any
     */
    private static Object restoreThemeFrom(LookAndFeel laf, Preferences prefs) {
        String themeName = prefs.get(themeKey(laf), null);
        if (themeName != null) {
            LOGGER.config(
                "Restoring theme for " + laf.getName() + ": " + themeName);
        }
        return themeName == null ? null : createThemeInstance(themeName);
    }

    /**
     * Creates and answers a theme instance for the specified class name.
     * Return {@code null} in case of an instantiation exception.
     */
    private static Object createThemeInstance(String themeClassName) {
        try {
            Class theClass = Class.forName(themeClassName);
            return theClass.newInstance();
        } catch (Exception e) {
            LOGGER.log(Level.WARNING, "Class name=" + themeClassName, e);
            return null;
        }
    }

    private static String themeKey(LookAndFeel laf) {
        return THEME_KEY_PREFIX + laf.getName();
    }

    private String getLafClassName() {
        return laf.getClass().getName();
    }


    // Overriding Object Behavior *********************************************

    /**
     * Returns a clone of this LookConfigurations instance.
     *
     * @return a cloned version of this object
     */
    @Override
    public Object clone() {
        // Safe, since this is a final class
        return new LookConfiguration(getLookAndFeel(), getTheme());
    }


    /**
     * Two LookConfiguration instance are equal if and only if
     * the class names of the LookAndFeels are equal and
     * the themes are equal.
     *
     * @return true if this configuration is equal to the given object
     */
    @Override
    public boolean equals(Object o) {
        if (this == o)
            return true;
        if (!(o instanceof LookConfiguration))
            return false;

        LookConfiguration config = (LookConfiguration) o;
        return getLafClassName().equals(config.getLafClassName())
            && (((theme == null) && (config.theme == null))
             || ((theme != null) && (theme.equals(config.theme))));
    }


    /**
     * Returns this configuration's hash code.
     * A poor but correct implementation. LookConfiguration instances
     * will likely not be used in HashMaps or HashSets.
     *
     * @return this configuation's hash code
     */
    @Override
    public int hashCode() {
        return laf.hashCode();
    }


    /**
     * Returns a string representation that lists the default configuration
     * and all configurations with look&amp;feel name and optional theme.
     *
     * @return a string representation that list the default configuration
     *    and all configurations with look&amp;feel name and optional theme.
     */
    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        builder.append("lafName=");
        builder.append(getLookAndFeel().getName());
        if (getTheme() != null) {
            builder.append("; theme=");
            builder.append(getTheme().getClass().getName());
        }
        return builder.toString();
    }


}