/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.laf;

import java.util.*;
import java.util.prefs.Preferences;

import javax.swing.LookAndFeel;

import com.jgoodies.binding.beans.Model;

/**
 * Describes the set of available instances of LookConfiguration
 * with one of them selected as default.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.6 $
 *
 * @see	LookConfiguration
 * @see ExtUIManager
 * @see	java.util.prefs.Preferences
 * @see javax.swing.LookAndFeel
 */

public final class LookConfigurations extends Model implements Cloneable {

    // Names of the Bound Bean Properties *************************************

    /**
     * The name of the bound read-only property <em>configurations</em>.
     */
    public static final String PROPERTYNAME_CONFIGURATIONS =
        "configurations";

    /**
     * The name of the bound read-write property <em>defaultConfiguration</em>.
     */
    public static final String PROPERTYNAME_DEFAULT_CONFIGURATION =
        "defaultConfiguration";


    // ************************************************************************

    /**
     * Maps Look&amp;Feel class names to instances of LookConfiguration.
     */
    private final Map configurations;

    /**
     * Holds the default LookConfiguration.
     *
     * @see #getDefaultConfiguration()
     * @see #setDefaultConfiguration(LookConfiguration)
     */
    private LookConfiguration defaultConfiguration;


    // Instance Creation ****************************************************

    /**
     * Constructs LookConfigurations object with the specified selection.
     *
     * @param selection  the selected LookConfiguration
     */
    public LookConfigurations(LookConfiguration selection) {
        this.configurations = new HashMap();
        this.defaultConfiguration = selection;
    }

    /**
     * Looks up and returns the LookConfigurations from the given Preferences.
     *
     * @param prefs   the Preferences node to restore from
     * @return the restored LookConfigurations object
     */
    static LookConfigurations restoreFrom(Preferences prefs) {
        LookConfiguration defaultConfig = ExtUIManager.getStoredLookConfiguration();
        LookConfigurations result = new LookConfigurations(defaultConfig);

        for (Iterator i = ExtUIManager.getSupportedLookAndFeelInstances().iterator(); i.hasNext();) {
            LookAndFeel laf = (LookAndFeel) i.next();
            result.putConfiguration(LookConfiguration.restoreFrom(laf, prefs));
        }
        return result;
    }


    // Public API *************************************************************

    /**
     * Returns the default LookConfiguration.
     *
     * @return the default LookConfiguration
     */
    public LookConfiguration getDefaultConfiguration() {
        return defaultConfiguration;
    }

    /**
     * Sets a new default LookConfiguration.
     *
     * @param newDefaultConfiguration  the new default configuration
     */
    public void setDefaultConfiguration(LookConfiguration newDefaultConfiguration) {
        LookConfiguration oldDefaultConfiguration = getDefaultConfiguration();
        this.defaultConfiguration = newDefaultConfiguration;
        firePropertyChange(
                PROPERTYNAME_DEFAULT_CONFIGURATION,
                oldDefaultConfiguration,
                newDefaultConfiguration);
    }


    /**
     * Returns the Collection of LookConfiguration instances
     * held by this LookConfigurations.
     *
     * @return the Collection of LookConfiguration instances
     *     held by this LookConfigurations.
     */
    public Collection getConfigurations() {
        List list = new ArrayList(configurations.values());
        Collections.sort(list, new ConfigurationComparator());
        return list;
    }


    // Implementation *********************************************************

    /**
     * Looks up and returns the <code>LookConfiguration</code>
     * for the specified <code>LookAndFeel</code>.
     */
    private LookConfiguration getConfiguration(LookAndFeel laf) {
        LookConfiguration config =
            (LookConfiguration) configurations.get(getKey(laf));
        return config != null ? config : new LookConfiguration(laf);
    }

    /**
     * Puts the given <code>LookConfiguration</code> in the map of
     * all configurations using the config's look and feel as key.
     */
    private void putConfiguration(LookConfiguration config) {
        configurations.put(getKey(config), config);
    }

    private String getKey(LookConfiguration config) {
        return getKey(config.getLookAndFeel());
    }

    private String getKey(LookAndFeel laf) {
        return laf.getClass().getName();
    }


    /**
     * Stores the selection and all configurations in the specified Preferences.
     *
     * @param prefs   the Preferences node to store this LookConfigurations in
     */
    void storeIn(Preferences prefs) {
        ExtUIManager.setStoredLookConfiguration(getDefaultConfiguration());
        for (Iterator i = configurations.values().iterator(); i.hasNext();) {
            LookConfiguration configuration = (LookConfiguration) i.next();
            configuration.storeIn(prefs);
        }
    }

    // Overriding Object Behavior *********************************************

    /**
     * Returns a clone of this LookConfigurations instance.
     *
     * @return a cloned version of this object
     */
    @Override
    public Object clone() {
        LookConfiguration defaultConfig =
            (LookConfiguration) defaultConfiguration.clone();
        // Safe, since this is a final class
        LookConfigurations result = new LookConfigurations(defaultConfig);
        for (Iterator i = configurations.values().iterator(); i.hasNext();) {
            LookConfiguration config = (LookConfiguration) i.next();
            result.putConfiguration(config.equals(defaultConfig)
                    ? defaultConfig
                    : (LookConfiguration) config.clone());
        }
        return result;
    }


    /**
     * Checks and answers if this LookConfigurations equals the given object.
     * Two LookConfigurations are equal if and only if they have equals
     * LookConfiguration instances as default, and if all LookConfiguration
     * instances are equal too.
     *
     * @return true if this LookConfigurations object equals the given object
     */
    @Override
    public boolean equals(Object o) {
        if (this == o)
            return true;
        if (!(o instanceof LookConfigurations))
            return false;

        LookConfigurations other = (LookConfigurations) o;
        if (!getDefaultConfiguration().equals(other.getDefaultConfiguration()))
            return false;

        for (Iterator i = configurations.values().iterator(); i.hasNext();) {
            LookConfiguration config1 = (LookConfiguration) i.next();
            LookConfiguration config2 = other.getConfiguration(config1.getLookAndFeel());
            if (config2 == null || !config1.equals(config2))
                return false;
        }
        return true;
    }

    /**
     * Returns this object's hash code. A poor but correct implementation.
     * Instances of LookConfigurations will likely not be used in HashMaps
     * or HashSets.
     *
     * @return this object's hash code
     */
    @Override
    public int hashCode() {
        return defaultConfiguration.hashCode();
    }


    /**
     * Returns a string representation that lists the default configuration
     * and all configurations with look&amp;feel name and optional theme.
     *
     * @return a string representation that list the default configuration
     *    and all configurations with look&amp;feel name and optional theme.
     */
    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder("LookConfigurations[");
        builder.append("\n    default=");
        builder.append(getDefaultConfiguration().toString());
        for (Iterator i = getConfigurations().iterator(); i.hasNext();) {
            LookConfiguration configuration = (LookConfiguration) i.next();
            builder.append("\n    ");
            builder.append(configuration.toString());
        }
        builder.append("\n]");
        return builder.toString();
    }


    // Helper Code ************************************************************

    private static class ConfigurationComparator implements Comparator {
        public int compare(Object o1, Object o2) {
            LookConfiguration config1 = (LookConfiguration) o1;
            LookConfiguration config2 = (LookConfiguration) o2;
            return config1.getLookAndFeel().getName().toUpperCase().compareTo(
                    config2.getLookAndFeel().getName().toUpperCase());
        }
    }




}