/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.osx;

import java.awt.event.ActionEvent;

import com.apple.eawt.Application;
import com.apple.eawt.ApplicationAdapter;
import com.apple.eawt.ApplicationEvent;


/**
 * Handles Max OS X application events for about, preferences and quit
 * as registered by the {@link OSXApplicationMenu} class. Delegates
 * the concrete behavior to the <code>ActionListener</code>s referenced
 * by the <code>OSXApplicationMenu</code>.<p>
 *
 * Since this class extends a Mac OS X AWT extension class,
 * it will typically fail to load - unless the environment is on a Mac.
 * Therefore the caller <code>OSXApplicationMenu</code> checks the platform
 * before it attempts to load and invoke the <code>#register()</code> method.<p>
 *
 * To enable a simple integration with IDEs, we use a jar provided by Apple
 * that consists stub classes for the Apple AWT extension classes.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.5 $
 * @since 1.3
 *
 * @see OSXApplicationMenu
 */

final class OSXApplicationHandler extends ApplicationAdapter {

    /**
     * Handles the Mac OS X application event for about.
     * Invoked if the user selects About in the application menu.<p>
     *
     * If an about <code>ActionListener</code> has been registered with
     * the <code>OSXApplicationMenu</code>, its <code>#actionPerformed</code>
     * will be called with a newly created <code>ActionEvent</code>.
     */
    @Override
    public void handleAbout(ApplicationEvent evt) {
        if (OSXApplicationMenu.aboutListener != null) {
            evt.setHandled(true);
            OSXApplicationMenu.aboutListener.actionPerformed(
                    new ActionEvent(this, 0, "about"));
        }
    }


    /**
     * Handles the Mac OS X application event for preferences.
     * Invoked if the user selects About in the application menu.<p>
     *
     * If a preferences <code>ActionListener</code> has been registered with
     * the <code>OSXApplicationMenu</code>, its <code>#actionPerformed</code>
     * will be called with a newly created <code>ActionEvent</code>.
     */
    @Override
    public void handlePreferences(ApplicationEvent evt) {
        if (OSXApplicationMenu.prefsListener != null) {
            evt.setHandled(true);
            OSXApplicationMenu.prefsListener.actionPerformed(
                    new ActionEvent(this, 0, "preferences"));
        }
    }


    /**
     * Handles the Mac OS X application event for quit.
     * Invoked if the user selects About in the application menu.<p>
     *
     * If a quit <code>ActionListener</code> has been registered with
     * the <code>OSXApplicationMenu</code>, its <code>#actionPerformed</code>
     * will be called with a newly created <code>ActionEvent</code>.<p>
     *
     * <code>#setHandled</code> is called with {@code false} to cancel
     * the AppleEvent-based quit. This gives the Java code a chance to delay
     * or cancel the termination process, for example by opening a request
     * window.
     */
    @Override
    public void handleQuit(ApplicationEvent evt) {
        if (OSXApplicationMenu.quitListener != null) {
            evt.setHandled(false);
            OSXApplicationMenu.quitListener.actionPerformed(
                    new ActionEvent(this, 0, "quit"));
        }
    }


    /**
     * Adds an instance of this class as listener for Mac OS X application
     * events. Also enables the preferences menu, if and only if a preferences
     * action listener has been registered with the
     * <code>OSXApplicationMenu</code> class.<p>
     *
     * This method will be invoked via reflection from method
     * <code>OSXApplicationMenu#lookupAndRegister()</code>.
     */
    static void register() {
        Application application = new Application();
        application.addApplicationListener(new OSXApplicationHandler());
        application.setEnabledPreferencesMenu(OSXApplicationMenu.prefsListener != null);
    }


}