/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.osx;

import java.awt.event.ActionListener;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.security.AccessControlException;
import java.util.logging.Logger;

import com.jgoodies.uif.util.SystemUtils;


/**
 * Hooks existing about, preferences and quit action listeners
 * into the Mac OS X application menu. Provides information which
 * functions have been hooked into the application menu, so an
 * application can request this if it conditionally builds menus.
 * Reduces the effort to support these Mac OS X features in
 * multi-platform applications.<p>
 *
 * The implementation first checks for a Mac OS X Java runtime
 * and does nothing if  it runs on a different platform.
 * Next, it tries to load a subclass of the <code>ApplicationAdapter</code>
 * from the Mac Java extensions in package <code>com.apple.eawt</code>.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.5 $
 * @since 1.3
 *
 * @see com.jgoodies.uif.osx.OSXApplicationHandler
 */

public final class OSXApplicationMenu {

    private OSXApplicationMenu() {
        // Overrides default constructor; prevents instantiation.
    }

    /**
     * The system property key used to set the application's name on Mac OS X.
     *
     * @see #setAboutName(String)
     */
    private static final String KEY_MAC_APPLICATION_MENU_ABOUT_NAME =
        "com.apple.mrj.application.apple.menu.about.name";

    /**
     * The classname used to safely create an instance of the
     * OSXApplicationHandler.
     */
    private static final String OSX_APPLICATION_HANDLER_CLASSNAME =
        "com.jgoodies.uif.osx.OSXApplicationHandler";

    /**
     * Refers to the listener that performs the about action.
     */
    static ActionListener aboutListener;

    /**
     * Refers to the listener that performs the preferences action.
     */
    static ActionListener prefsListener;

    /**
     * Refers to the listener that performs the quit action.
     */
    static ActionListener quitListener;

    /**
     * Describes whether listeners have been registered with
     * the Mac OS X application men.
     */
    private static boolean registered = false;


    // Public API ************************************************************

    /**
     * Sets the name that is used in the Mac OS X application menu
     * as title and suffix for the about and quit actions.
     * Catches an AccessControlException silently in case this method
     * has been invoked in an untrusted environment.<p>
     *
     * Must be called before a windows is opened. And so, a good place
     * is the application's main method or the application startup.
     * {@link com.jgoodies.uif.application.AbstractUIFApplication}#load
     * calls this method.<p>
     *
     * @param aboutName    the name to be used
     */
    public static void setAboutName(String aboutName) {
        try {
            System.setProperty(KEY_MAC_APPLICATION_MENU_ABOUT_NAME, aboutName);
        } catch (AccessControlException e) {
            Logger.getLogger(OSXApplicationMenu.class.getName()).warning(
                    "Can set the OS X application menu name only in trusted environments.");
        }
    }


    /**
     * Registers the given action listeners for about, preferences and
     * quit with the Mac OS X application menu - if we're on OS X.
     *
     * @param about        a listener that performs the about action
     * @param preferences  a listener that performs the preferences action
     * @param quit         a listener that performs the quit action
     *
     * @return true if an action has been registered
     */
    public static synchronized boolean register(ActionListener about,
                                 ActionListener preferences,
                                 ActionListener quit) {
        if (about == null && preferences == null && quit == null)
            return false;
        aboutListener = about;
        prefsListener = preferences;
        quitListener = quit;
        registered = lookupAndRegister();
        return registered;
    }


    /**
     * Checks and answers whether an about listener has been registered
     * with the Mac OS X application menu. Useful to conditionally build
     * a menu item for <em>About</em>.
     *
     * @return true if an about listener has been registered
     */
    public static boolean isRegisteredAbout() {
        return registered && (aboutListener != null);
    }


    /**
     * Checks and answers whether a preferences listener has been registered
     * with the Mac OS X application menu. Useful to conditionally build
     * a menu item for <em>Preferences</em>.
     *
     * @return true if a preferences listener has been registered
     */
    public static boolean isRegisteredPreferences() {
        return registered && (prefsListener != null);
    }


    /**
     * Checks and answers whether a quit listener has been registered
     * with the Mac OS X application menu. Useful to conditionally build
     * a menu item for <em>Quit</em>/<em>Exit</em>.
     *
     * @return true if a quit listener has been registered
     */
    public static boolean isRegisteredQuit() {
        return registered && (quitListener != null);
    }


    // Helper Code ************************************************************

    private static boolean lookupAndRegister() {
        if (!SystemUtils.IS_OS_MAC) return false;

        Class handlerClass;
        try {
            handlerClass = Class.forName(OSX_APPLICATION_HANDLER_CLASSNAME);
        } catch (ClassNotFoundException e) {
            System.err.println("Class not found OSXApplicationHandler");
            return false;
        }
        Method registerMethod;
        Class[] paramClass = new Class[]{};
        Object[] arguments = new Object[]{};
        try {
            registerMethod = handlerClass.getDeclaredMethod(
                    "register", paramClass);
            registerMethod.invoke(handlerClass, arguments);
        } catch (NoSuchMethodException e1) {
            e1.printStackTrace();
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }
        return true;
    }
}