/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.splash;

import java.awt.*;
import java.awt.image.BufferStrategy;

import com.jgoodies.uif.util.ScreenUtils;

/**
 * This implementation of the {@link SplashProvider} interface uses an
 * AWT Window to display a splash image and an optional progress
 * bar. It has been optimized for quick startup.<p>
 *
 * TODO: Add features for an optional drop shadow (platform dependent)
 *    and using translucent splash images showing the desktop background.
 *
 * @author	Karsten Lentzsch
 * @version $Revision: 1.16 $
 *
 * @see	com.jgoodies.uif.splash.Splash
 * @see	com.jgoodies.uif.splash.SplashProvider
 */
public final class ImageSplash extends Window implements SplashProvider {

    private static final int DEFAULT_BAR_WIDTH  = 100;
    private static final int DEFAULT_BAR_HEIGHT =  10;
    private static final int VPAD               =  10;

    private final Image  image;

    private String      note;
    private boolean     noteEnabled;
    private boolean     progressVisible;
    private Color       textColor;
    private Rectangle   progressBarBounds;
    private int         percent;


    // Instance Creation ****************************************************

    /**
     * Constructs an AWT based splash for the given {@code image} using a
     * default Frame. Progress is invisible and note changes will be ignored.
     *
     * @param image             the splash image to display
     */
    public ImageSplash(Image image) {
        this(image, false);
    }


    /**
     * Constructs an AWT based splash for the given {@code image}
     * using a default Frame that has an optional progress bar.
     * The initial note is 'Loading&hellip;'. Note changes will be ignored
     * by default, see {@link #setNoteEnabled(boolean)}.
     *
     * @param image             the splash image to display
     * @param progressVisible   {@code true} to show progress, {@code false} to hide it
     */
    public ImageSplash(Image image, boolean progressVisible) {
        this(new Frame(), image, "Loading\u2026", progressVisible);
    }


    /**
     * Constructs an AWT based splash for the given Frame, Image, initial note
     * and progress visibility. Note changes will be ignored by default,
     * see {@link #setNoteEnabled(boolean)}.
     *
     * @param owner             this window's parent frame
     * @param image             the splash image to display
     * @param initialNote       the note that will be displayed first
     * @param progressVisible   {@code true} to show progress, {@code false} to hide it
     */
    public ImageSplash(
        Frame owner,
        Image image,
        String initialNote,
        boolean progressVisible) {
        super(owner);
        this.image = image;
        this.note = initialNote;
        this.percent = 0;
        this.progressVisible = progressVisible;
        setSize(image.getWidth(null), image.getHeight(null));
        setProgressBarBounds(VPAD);
        setForeground(Color.DARK_GRAY);
        setBackground(Color.LIGHT_GRAY);
        textColor = Color.BLACK;
        ScreenUtils.locateOnOpticalScreenCenter(this);
    }


    // Setting Properties ***************************************************

    /**
     * Returns whether this splash honors or ignores note changes.
     * This feature is disabled by default.
     *
     * @return {@code true} if this splash changes the displayed note in
     *     {@link #setNote(String)}, {@code false} to ignore it
     */
    public boolean isNoteEnabled() {
        return noteEnabled;
    }


    /**
     * Sets whether this splash shall honor or ignore note changes.
     * This feature is disabled by default.
     *
     * @param noteEnabled    {@code true} if this splash changes the displayed
     *     note in {@link #setNote(String)}, {@code false} to ignore it
     */
    public void setNoteEnabled(boolean noteEnabled) {
        this.noteEnabled = noteEnabled;
    }


    /**
     * Returns whether this splash shows a progress bar or not.
     *
     * @return {@code true} if this splash shows a progress, {@code false} if not
     */
    public boolean isProgressVisible() {
        return progressVisible;
    }


    /**
     * Shows or hides the progress bar.
     *
     * @param progressVisible  {@code true} to show the progress, {@code false} to hide it
     */
    public void setProgressVisible(boolean progressVisible) {
        this.progressVisible = progressVisible;
    }


    /**
     * Returns the note color.
     *
     * @return the note color
     */
    public Color getTextColor() {
        return textColor;
    }

    /**
     * Sets a new note color.
     *
     * @param newTextColor   the new note color
     */
    public void setTextColor(Color newTextColor) {
        textColor = newTextColor;
    }


    /**
     * Sets the bounds for the progress bar using the given Rectangle.
     *
     * @param r   the Rectangle that describes the progress bar bounds
     */
    public void setProgressBarBounds(Rectangle r) {
        progressBarBounds = new Rectangle(r);
    }

    /**
     * Sets the bounds for the progress bar using a pad from the dialog's bottom.
     *
     * @param bottomPad   the distance in pixel from the splash's bottom
     */
    public void setProgressBarBounds(int bottomPad) {
        int x = (getWidth() - DEFAULT_BAR_WIDTH) / 2;
        int y = getHeight() - DEFAULT_BAR_HEIGHT - bottomPad;
        Rectangle defaultBounds =
            new Rectangle(x, y, DEFAULT_BAR_WIDTH, DEFAULT_BAR_HEIGHT);
        setProgressBarBounds(defaultBounds);
    }


    // SplashProvider Implementation ******************************************

    /**
     * Opens the splash window.
     */
    public void openSplash() {
        setVisible(true);
//        synchronized(this) {
//            while(!paintCalled) {
//                try {
//                    wait();
//                } catch (InterruptedException e) {
//                    // Do nothing
//                }
//            }
//        }
        update(getGraphics());
    }


    /**
     * Closes and disposes the splash window.
     */
    public void closeSplash() {
        dispose();
    }


    /**
     * Sets a new progress value.
     */
    public void setProgress(int percent) {
        if (isProgressVisible()) {
            this.percent = percent;
            repaintImmediately();
        }
    }


    /**
     * Sets a new note if and only if {@link #isNoteEnabled()} returns true.
     *
     * @param newNote    the note to set
     */
    public void setNote(String newNote) {
        if (isNoteEnabled()) {
            note = newNote;
            repaintImmediately();
        }
    }


    // Painting *************************************************************

    /**
     * Updates the display area of the window. To reduce flickering,
     * {@code paint} is called directly. This is possible, because
     * the splash fills its complete area.
     */
    @Override
    public void update(Graphics g) {
        paint(g);
    }


    /**
     * Renders the image and optionally a progress bar with a note.
     * Painting is delegated to {@code #paint0} that is used by
     * {@link #repaintImmediately()} too.
     */
    @Override
    public void paint(Graphics g) {
        paint0(g);
    }


    /**
     * We first ensure that we have a valid peer by checking the showing state.
     * We loose our peer, if {@code closeSplash} has been called, before
     * this repaint is initiated, for example because the progress or note
     * is set after the splash became invisible. As an alternative, we could
     * make the splash invisible without disposing the native resources.
     *
     * @see #closeSplash()
     */
    private void repaintImmediately() {
        if (!isShowing()) {
            return;
        }
        BufferStrategy strategy = getBufferStrategy();
        if (strategy == null) {
            createBufferStrategy(2);
            strategy = getBufferStrategy();
        }
        Graphics g = strategy.getDrawGraphics();
        // The strategy should return a non-null Graphics object.
        // However, the following check works around a bug in
        // the Java for Mac OS X 10.4 Release 5 (1.5.0_07)
        // where the Graphics object can be null.
        // This bug seems to be fixed in the OS X 10.4 Java 6 Preview.
        if (g != null) {
            paint0(g);
            g.dispose();
            strategy.show();
        }
    }


    /**
     * Invoked by {@link #paint(Graphics)} and {@link #repaintImmediately()}.
     */
    private void paint0(Graphics g) {
        if (image != null) {
            g.drawImage(image, 0, 0, this);
        }
        if (progressVisible) {
            paintProgress(g);
        }
        paintNote(g);
    }


    private void paintProgress(Graphics g) {
        int x = progressBarBounds.x;
        int y = progressBarBounds.y;
        int w = progressBarBounds.width;
        int h = progressBarBounds.height;
        int progressWidth = (w - 2) * percent / 100;
        int progressHeight = h - 2;

        g.translate(x, y);
        // Paint border
        g.setColor(Color.GRAY);
        g.drawLine(0, 0, w - 2, 0);
        g.drawLine(0, 0, 0, h - 1);
        g.setColor(Color.WHITE);
        g.drawLine(0, h - 1, w - 1, h - 1);
        g.drawLine(w - 1, 0, w - 1, h - 1);
        // Paint background
        g.setColor(getBackground());
        g.fillRect(1, 1, w - 2, progressHeight);
        // Paint progress bar
        g.setColor(getForeground());
        g.fillRect(1, 1, progressWidth, progressHeight);
        g.translate(-x, -y);
    }


    private void paintNote(Graphics g) {
        FontMetrics fm = getFontMetrics(g.getFont());
        int textWidth = fm.stringWidth(note);
        int textX = (getWidth() - textWidth) / 2;
        g.setColor(textColor);
        g.drawString(note, textX, progressBarBounds.y - VPAD / 2);
    }


}