/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.splash;

import java.util.logging.Logger;

/**
 * A singleton that provides access to a pluggable splash
 * to improve the application's feedback during startup.
 * You can open and close it, set messages and indicate progress.<p>
 *
 * The concrete work is done by pluggable splash providers, which you
 * can set via #setProvider, for example a splash window or splash panel.
 * The default provider is <code>NullSplash</code>, other providers
 * are <code>ImageSplash</code> and <code>JWindowSplash</code>.<p>
 *
 * This class logs splash events on <code>CONFIG</code> level with
 * the elapsed time. You must invoke <code>#initStartTime</code> in your
 * application's <code>#main</code> method to get accurate timing information.
 *
 * @author	Karsten Lentzsch
 * @version $Revision: 1.4 $
 *
 * @see	SplashProvider
 */
public final class Splash {

    private static final Logger LOGGER = Logger.getLogger("Splash");

    private static SplashProvider provider;
    private static long     t0 = System.currentTimeMillis();
    private static boolean  splashOpen = false;

    private Splash() {
        // Suppresses default constructor, ensuring non-instantiability.
    }

    /**
     * Resets the start time that is used to compute the elapsed time.
     */
    public static void resetStartTime() {
        t0 = System.currentTimeMillis();
    }

    /**
     * Computes and returns the elapsed time since the last invocation of
     * <code>#resetStartTime()</code>.
     *
     * @return the time that has been elapsed since the time has been reset
     */
    public static long elapsedTime() {
        return System.currentTimeMillis() - t0;
    }

    /**
     * Ensures that the splash is open, calls <code>#open</code> if necessary.
     */
    public static synchronized void ensureOpen() {
        if (!splashOpen)
            open();
    }

    /**
     * Ensures that the splash is closed, calls <code>#close</code> if necessary.
     */
    public static synchronized void ensureClosed() {
        if (splashOpen)
            close();
    }

    /**
     * Opens the splash by calling the providers <code>#openSplash</code> method.
     */
    public static synchronized void open() {
        if (splashOpen)
            throw new IllegalStateException("Splash is already open.");

        getProvider().openSplash();
        splashOpen = true;
        LOGGER.config(Splash.elapsedTime() + "ms - opened");
    }

    /**
     * Closes the splash by calling the providers <code>#closeSplash</code> method.
     */
    public static synchronized void close() {
        if (!splashOpen)
            throw new IllegalStateException("Splash has been closed before.");

        LOGGER.config(Splash.elapsedTime() + "ms - closed");
        getProvider().closeSplash();
        splashOpen = false;
    }

    /**
     * Sets the splash note.
     *
     * @param message   the text to be set as current splash note
     */
    public static void setNote(String message) {
        LOGGER.config(Splash.elapsedTime() + "ms - " + message);
        getProvider().setNote(message);
    }

    /**
     * Sets the splash progress percent value.
     *
     * @param percent   the progress in percent to be set as splash progress
     */
    public static void setProgress(int percent) {
        LOGGER.config(Splash.elapsedTime() + "ms " + percent + "%");
        getProvider().setProgress(percent);
    }

    /**
     * Sets the splash note and progress at the same time.
     *
     * @param message   the text to be set as current splash note
     * @param percent   the progress in percent to be set as splash progress
     */
    public static void setNote(String message, int percent) {
        setNote(message);
        setProgress(percent);
    }

    /**
     * Returns the current {@link SplashProvider}.
     *
     * @return the current SplashProvider.
     */
    public static SplashProvider getProvider() {
        if (provider == null) {
            provider = new NullSplash();
        }
        return provider;
    }

    /**
     * Sets a {@link SplashProvider}.
     *
     * @param newProvider   the SplashProvider to be set
     */
    public static void setProvider(SplashProvider newProvider) {
        provider = newProvider;
    }


    // Helper Class *********************************************************

    /**
     * This empty default splash provider does nothing.
     */
    private static final class NullSplash implements SplashProvider {
        public void openSplash() {
            // Do nothing
        }

        public void closeSplash() {
            // Do nothing
        }

        public void setNote(String message) {
            // Do nothing
        }

        public void setProgress(int percent) {
            // Do nothing
        }
    }

}