/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.util;

import java.awt.Component;
import java.awt.Graphics;

import javax.swing.Icon;

/**
 * An <code>Icon</code> implementation that paints two icons
 * as compound or overlaid icon using a specified alignment.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.5 $
 *
 * @see javax.swing.ImageIcon
 */

public final class CompoundIcon implements Icon {

    /**
     * Put the foreground icon at the top of its display area,
     * centered horizontally.
     */
    public static final int CENTER = 0;

    /**
     * Put the foreground icon at the top of its display area,
     * centered horizontally.
     */
    public static final int NORTH = 1;

    /**
     * Put the foreground icon at the top-right corner of its display area.
     */
    public static final int NORTHEAST = 2;

    /**
     * Put the foreground icon on the right side of its display area,
     * centered vertically.
     */
    public static final int EAST = 3;

    /**
     * Put the foreground icon at the bottom-right corner of its display area.
     */
    public static final int SOUTHEAST = 4;

    /**
     * Put the foreground icon at the bottom of its display area, centered
     * horizontally.
     */
    public static final int SOUTH = 5;

    /**
     * Put the foreground icon at the bottom-left corner of its display area.
     */
    public static final int SOUTHWEST = 6;

    /**
     * Put the foreground icon on the left side of its display area,
     * centered vertically.
     */
    public static final int WEST = 7;

    /**
     * Put the foreground icon at the top-left corner of its display area.
     */
    public static final int NORTHWEST = 8;

    /**
     * Refers to the icon that will be painted in the background.
     */
    private final Icon backgroundIcon;

    /**
     * Refers to the icon that will be painted in the foreground.
     * This will often be a smaller overlay icon.
     */
    private final Icon foregroundIcon;

    private final int  height;
    private final int  width;
    private int xOffset;
    private int yOffset;


    // Instance Creation ******************************************************

    /**
     * Constructs a compound icon for the given foreground and background icons,
     * using a default anchor.
     *
     * @param backgroundIcon   the icon in the centered background
     * @param foregroundIcon   the icon that overlays the background icon
     */
    public CompoundIcon(Icon backgroundIcon, Icon foregroundIcon) {
        this(backgroundIcon, foregroundIcon, SOUTHEAST);
    }

    /**
     * Constructs a compound icon for the given foreground and background icons,
     * using the specified anchor.
     *
     * @param backgroundIcon   the icon in the centered background
     * @param foregroundIcon   the icon that overlays the background icon
     * @param anchor           the position of the foreground icon relative
     *     to the background icon
     */
    public CompoundIcon(Icon backgroundIcon, Icon foregroundIcon, int anchor) {
        this.backgroundIcon = backgroundIcon;
        this.foregroundIcon = foregroundIcon;
        height = Math.max(
                backgroundIcon.getIconHeight(),
                foregroundIcon.getIconHeight());
        width  = Math.max(
                backgroundIcon.getIconWidth(),
                foregroundIcon.getIconWidth());
        setAnchor(anchor);
    }


    // Implementing the Icon Interface ****************************************

    /**
     * Returns this icon's width, which is the maximum of
     * the widths of the background and foreground icons.
     *
     * @return an int specifying the fixed width of this icon.
     */
    public int getIconWidth() {
        return width;
    }

    /**
     * Returns this icon's height, which is the maximum of
     * the heights of the background and foreground icons.
     *
     * @return an int specifying the fixed height of this icon.
     */
    public int getIconHeight() {
        return height;
    }

    /**
     * Draws this icon at the specified location. First paints
     * the background icon at the specified location, then paints
     * the foreground icon using the offsets computed in #setAnchor.
     */
    public void paintIcon(Component c, Graphics g, int x, int y) {
        backgroundIcon.paintIcon(c, g, x, y);
        foregroundIcon.paintIcon(c, g, x + xOffset, y + yOffset);
    }


    // Helper Code ************************************************************

    private void setAnchor(int anchor) {
        int xDiff = backgroundIcon.getIconWidth() - foregroundIcon.getIconWidth();
        int yDiff = backgroundIcon.getIconHeight() - foregroundIcon.getIconHeight();

        xOffset = ((anchor == NORTHWEST)
                || (anchor == WEST)
                || (anchor == SOUTHWEST))
                ? 0
                : (((anchor == NORTH)
                    || (anchor == CENTER)
                    || (anchor == SOUTH))
                    ? xDiff / 2
                    : xDiff);

        yOffset = ((anchor == NORTHWEST)
                || (anchor == NORTH)
                || (anchor == NORTHEAST))
                ? 0
                : (((anchor == WEST) || (anchor == CENTER) || (anchor == EAST))
                    ? yDiff / 2
                    : yDiff);
    }

}