/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.util;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;

import javax.swing.LookAndFeel;
import javax.swing.UIManager;

import com.sun.java.swing.plaf.windows.WindowsLookAndFeel;


/**
 * A typesafe enumeration for a bunch of <code>Mode</code>s useful for
 * handling differences between platforms and looks.
 *
 * Note that some values require access to protected System properties
 * that are only readable in safe environments.
 * <p>
 * The available modes are:
 * <table>
 * <tr>
 *     <td>Name</td><td>Meaning</td>
 * </tr>
 * <tr>
 *     <td><code>ALWAYS</code></td>
 *     <td>Enabled on all operating systems and look&amp;feels</td>
 * </tr>
 * <tr>
 *     <td><code>NEVER</code></td>
 *     <td>Disabled on all operating systems and look&amp;feels</td>
 * </tr>
 * <tr>
 *     <td><code>LAF_AQUA</code></td>
 *     <td>Enabled for the Mac OS X Aqua L&amp;F only,
 *         disabled for all other l&amp;fs</td>
 * </tr>
 * <tr>
 *     <td><code>LAF_WINDOWS</code></td>
 *     <td>Enabled for the Windows L&amp;Fs,
 *         disabled for all other l&amp;fs</td>
 * </tr>
 * <tr>
 *     <td><code>LAF_NON_AQUA</code></td>
 *     <td>Disabled for the Mac OS X Aqua L&amp;F only,
 *         enabled for all other l&amp;fs</td>
 * </tr>
 * <tr>
 *     <td><code>LAF_NON_WINDOWS</code></td>
 *     <td>Disabled for the Windows L&amp;Fs,
 *         enabled for all other l&amp;fs</td>
 * </tr>
 * <tr>
 *     <td><code>OS_LINUX</code></td>
 *     <td>Enabled on Linux only, disabled on all other platforms</td>
 * </tr>
 * <tr>
 *     <td><code>OS_MAC</code></td>
 *     <td>Enabled on Mac OS only, disabled on all other platforms</td>
 * </tr>
 * <tr>
 *     <td><code>OS_WINDOWS</code></td>
 *     <td>Enabled on Windows only, disabled on all other platforms</td>
 * </tr>
 * <tr>
 *     <td><code>OS_NON_LINUX</code></td>
 *     <td>Disabled on LINUX only, enabled on all other platforms</td>
 * </tr>
 * <tr>
 *     <td><code>OS_NON_MAC</code></td>
 *     <td>Disabled on Mac OS only, enabled on all other platforms</td>
 * </tr>
 * <tr>
 *     <td><code>OS_NON_WINDOWS</code></td>
 *     <td>Disabled on Windows only, enabled on all other platforms</td>
 * </tr>
 * </table>
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.6 $
 */

public final class Modes implements Mode, Serializable {

    // Public Constants *******************************************************
    // Note: new constants must be appended to this list. See also #readResolve

    /**
     * Describes a mode that is always enabled.
     */
    public static final Modes ALWAYS          = new Modes("Always");

    /**
     * Describes a mode that is always disabled.
     */
    public static final Modes NEVER           = new Modes("Never");

    /**
     * Describes a mode that is enabled if and only if
     * the current look&amp;feel is the Mac OS X Aqua look&amp;feel.
     */
    public static final Modes LAF_AQUA        = new Modes("Aqua l&f only");

    /**
     * Describes a mode that is enabled if and only if
     * the current look&amp;feel is a Windows look&amp;feel.
     */
    public static final Modes LAF_WINDOWS     = new Modes("Windows l&f only");

    /**
     * Describes a mode that is enabled if and only if the current
     * look&amp;feel is <em>not</em> the Mac OS X Aqua look&amp;feel.
     * In other words: it is disabled for Aqua and enabled
     * for all other l&amp;fs.
     */
    public static final Modes LAF_NON_AQUA    = new Modes("No Aqua l&f");

    /**
     * Describes a mode that is enabled if and only if the current
     * look&amp;feel is <em>not</em> a Windows look&amp;feel.
     * In other words: it is disabled for Windows l&amp;fs and enabled
     * for all other l&amp;fs.
     */
    public static final Modes LAF_NON_WINDOWS = new Modes("No Windows l&f");

    /**
     * Describes a mode that is enabled if and only if
     * the OS is Mac.
     */
    public static final Modes OS_MAC          = new Modes("Mac OS only");

    /**
     * Describes a mode that is enabled if and only if
     * the OS is Windows.
     */
    public static final Modes OS_WINDOWS      = new Modes("Windows OS Only");

    /**
     * Describes a mode that is enabled if and only if
     * the OS is <em>not</em> Mac.
     */
    public static final Modes OS_NON_MAC      = new Modes("No Mac OS");

    /**
     * Describes a mode that is enabled if and only if
     * the OS is <em>not</em> Windows.
     */
    public static final Modes OS_NON_WINDOWS  = new Modes("No Windows OS");

    /**
     * Describes a mode that is enabled if and only if
     * the OS is Linux.
     */
    public static final Modes OS_LINUX        = new Modes("Linux only");

    /**
     * Describes a mode that is enabled if and only if
     * the OS is <em>not</em> Linux.
     */
    public static final Modes OS_NON_LINUX    = new Modes("No Linux");


    // Private Declarations ***************************************************

    private final transient String name;

    private Modes(String name) {
        this.name = name;
    }


    // Public API *************************************************************

    /**
     * Checks and answers whether this mode is enabled or disabled.
     * Note that some enumeration values require to read protected
     * System properties and so will work properly only in safe environments.
     *
     * @return true if enabled, false if disabled
     */
    public boolean enabled() {
        if (this == ALWAYS)
            return true;
        else if (this == NEVER)
            return false;
        else if (this == OS_LINUX)
            return SystemUtils.IS_OS_LINUX;
        else if (this == OS_MAC)
            return SystemUtils.IS_OS_MAC;
        else if (this == OS_WINDOWS)
            return SystemUtils.IS_OS_WINDOWS;
        else if (this == OS_NON_LINUX)
            return !SystemUtils.IS_OS_LINUX;
        else if (this == OS_NON_MAC)
            return !SystemUtils.IS_OS_MAC;
        else if (this == OS_NON_WINDOWS)
            return !SystemUtils.IS_OS_WINDOWS;
        else if (this == LAF_AQUA)
            return isLafAqua();
        else if (this == LAF_WINDOWS)
            return isLafWindows();
        else if (this == LAF_NON_AQUA)
            return !isLafAqua();
        else if (this == LAF_NON_WINDOWS)
            return !isLafWindows();
        else
            throw new IllegalStateException("Unknown mode");
    }


    /**
     * Returns a human readable string representation for this mode.
     *
     * @return a human readable string representation for this mode
     */
    @Override
    public String toString() {
        return "Mode: " + name + " currently " + (enabled() ? "enabled" : "disabled");
    }


    // Serialization *****************************************************

    private static int nextOrdinal = 0;

    private final int ordinal = nextOrdinal++;

    private static final Modes[] VALUES =
        {ALWAYS, NEVER,
         LAF_AQUA, LAF_WINDOWS,
         LAF_NON_AQUA, LAF_NON_WINDOWS,
         OS_MAC, OS_WINDOWS,
         OS_NON_MAC, OS_NON_WINDOWS,
         OS_LINUX, OS_NON_LINUX};

    private Object readResolve() {
        return VALUES[ordinal];  // Canonicalize
    }


    // Caching and Lazily Computing the Laf State *****************************

    /**
     * Holds the cached result of the Aqua l&amp;f check.
     * Is invalidated by the <code>LookAndFeelChangeHandler</code>
     * if the look&amp;feel changes.
     */
    private static Boolean cachedIsLafAqua;

    /**
     * Holds the cached result of the Aqua l&amp;f check.
     * Is invalidated by the <code>LookAndFeelChangeHandler</code>
     * if the look&amp;feel changes.
     */
    private static Boolean cachedIsLafWindows;

    /**
     * Describes whether the <code>LookAndFeelChangeHandler</code>
     * has been registered with the <code>UIManager</code> or not.
     * It is registered lazily when the first cached l&amp;f state is computed.
     */
    private static boolean changeHandlerRegistered = false;


    private static void ensureLookAndFeelChangeHandlerRegistered() {
        if (!changeHandlerRegistered) {
            UIManager.addPropertyChangeListener(new LookAndFeelChangeHandler());
            changeHandlerRegistered = true;
        }
    }

    /**
     * Lazily checks and answers whether the Aqua look&amp;feel is active.
     *
     * @return true if the current look&amp;feel is Aqua
     */
    public static boolean isLafAqua() {
        if (cachedIsLafAqua == null) {
            cachedIsLafAqua = Boolean.valueOf(computeIsLafAqua());
            ensureLookAndFeelChangeHandlerRegistered();
        }
        return cachedIsLafAqua.booleanValue();
    }

    /**
     * Lazily checks and answers whether the Windows look&amp;feel is active.
     *
     * @return true if the current look&amp;feel is Windows
     */
    public static boolean isLafWindows() {
        if (cachedIsLafWindows == null) {
            cachedIsLafWindows = Boolean.valueOf(computeIsLafWindows());
            ensureLookAndFeelChangeHandlerRegistered();
        }
        return cachedIsLafWindows.booleanValue();
    }

    /**
     * Computes and answers whether an Aqua look&amp;feel is active.
     * This may be Apple's Aqua L&amp;f, or a sub-L&amp;f that
     * uses the same ID, because it doesn't substantially change the look.
     *
     * @return true if the current look&amp;feel is Aqua
     */
    private static boolean computeIsLafAqua() {
        return UIManager.getLookAndFeel().getID().equals("Aqua");
    }

    /**
     * Computes and answers whether the Windows look&amp;feel is active.
     *
     * @return true if the current look&amp;feel is Windows
     */
    private static boolean computeIsLafWindows() {
        LookAndFeel laf = UIManager.getLookAndFeel();
        return SystemUtils.IS_OS_WINDOWS && laf instanceof WindowsLookAndFeel;
    }


    /**
     * Invalidates cached L&amp;f information when the L&amp; changes.
     */
    private static final class LookAndFeelChangeHandler implements PropertyChangeListener {

        /**
         * Invalidates the cached laf states if the look&amp;feel changes.
         *
         * @param evt  describes the property change
         */
        public void propertyChange(PropertyChangeEvent evt) {
            cachedIsLafAqua    = null;
            cachedIsLafWindows = null;
        }
    }

}