/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.util;

import java.awt.Dimension;

import javax.swing.JComponent;

/**
 * Assists in getting aesthetic aspect ratios for panels and dialogs.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.4 $
 */

public final class Resizer {

    // Internal Constants *****************************************************

    private static final float SQRT_RATIO =
        1.0f / (float) Math.sqrt(2.0d);

    private static final int HORIZONTAL_DIALOG_MARGIN =  6;
    private static final int VERTICAL_DIALOG_MARGIN   = 25;


    // Public Constants *******************************************************

    /** A Resizer for an aspect ratio of 1:1.                                */
    public static final Resizer ONE2ONE      = new Resizer(1);

    /** A Resizer for an aspect ratio of 5:4.                                */
    public static final Resizer FIVE2FOUR    = new Resizer(5.0f / 4.0f);

    /** A Resizer for an aspect ratio of 4:3.                                */
    public static final Resizer FOUR2THREE   = new Resizer(4.0f / 3.0f);

    /** The default Resizer for an aspect ratio of 4:3.                      */
    public static final Resizer DEFAULT      = FOUR2THREE;

    /** A Resizer for an aspect ratio of sqrt(2):1.                          */
    public static final Resizer SQRT         = new Resizer(SQRT_RATIO);

    /** A Resizer for an aspect ratio of 1:sqrt(2).                          */
    public static final Resizer REVERSE_SQRT = new Resizer(1.0f / SQRT_RATIO);


    // Instance Fields ********************************************************

    /**
     * Holds this Resizer's aspect ratio.
     */
    private final float aspectRatio;


    // Instance Creation ******************************************************

    /**
	 * Constructs a Resizer with the specified aspect ratio.
     *
     * @param aspectRatio   the aspect ratio used to compute Dimensions
	 */
    public Resizer(float aspectRatio) {
        this.aspectRatio = aspectRatio;
    }


    // API ********************************************************************

    /**
	 * Returns a Dimension that has this Resizer's aspect ratio and
     * has been computed by adjusting the given dimension.
     * Favors expanding the width over expanding the height.
     *
     * @param original   the Dimension to be adjusted to be aesthetic
     * @return a Dimension that has this Resizer's aspect ratio
	 */
    public Dimension from(Dimension original) {
        return from(original, 0, 0);
    }

    /**
     * Returns a Dimension that is based on this Resizer's aspect ratio
     * and has been computed by adjusting the given dimension.
     * The given margins are added to the given width and height
     * and so build a larger dimension before the new aspect ratio
     * is achieved. This is useful if the given dimension describes
     * only part of a larger container. For example if the dimension
     * of a dialog's content pane is given and the larger dialog shall
     * get an aesthetic aspect ratio.<p>
     *
     * The adjustment algorithm favors expanding the width
     * over expanding the height.
     *
     * @param original         the inner Dimension
     * @param horizontalMargin the sum of the left and right margin to be added
     * @param verticalMargin   the sum of the top and bottom margin to be added
     * @return a Dimension that reflects the Resizer's aspect ratio if
     *     increased by the given margins
     */
    public Dimension from(Dimension original, int horizontalMargin, int verticalMargin) {
        int width  = original.width  + horizontalMargin;
        int height = original.height + verticalMargin;
        int expandedWidth  = Math.round(height * aspectRatio);
        int expandedHeight = Math.round(width / aspectRatio);
        return (width < expandedWidth)
            ? new Dimension(
                expandedWidth - horizontalMargin,
                height - verticalMargin)
            : new Dimension(
                width - horizontalMargin,
                expandedHeight - verticalMargin);
    }


    /**
	 * Returns a Dimension with the given height and a computed width
     * so that the result has this Resizer's aspect ratio.
     *
     * @param height   the height used to compute the width
     * @return a Dimension with the given height and a width so that
     *     the result has this Resizer's aspect ratio
	 */
    public Dimension fromHeight(int height) {
        return new Dimension(Math.round(height * aspectRatio), height);
    }


    /**
     * Returns a Dimension with the given width and a computed height
     * so that the result has this Resizer's aspect ratio.
     *
     * @param width   the width used to compute the height
     * @return a Dimension with the given width and a height so that
     *     the result has this Resizer's aspect ratio
	 */
    public Dimension fromWidth(int width) {
        return new Dimension(width, Math.round(width / aspectRatio));
    }


    /**
	 * Sets the component's preferred size to a Dimension that has
     * this Resizer's aspect ratio. First requests the component's current
     * preferred sized, uses this to compute an aesthetic Dimension,
     * which is finally set as new preferred size.
     *
     * @param component   the component to be resized
     * @see #from(Dimension)
	 */
    public void resize(JComponent component) {
        component.setPreferredSize(from(component.getPreferredSize()));
    }


    /**
     * Sets the component's preferred size to a Dimension that has
     * this Resizer's aspect ratio. First requests the component's current
     * preferred sized, uses this to compute an aesthetic Dimension,
     * which is finally set as new preferred size.
     *
     * @param component   the component to be resized
     * @see #from(Dimension, int, int)
     */
    public void resizeDialogContent(JComponent component) {
        component.setPreferredSize(
            from(component.getPreferredSize(),
                 HORIZONTAL_DIALOG_MARGIN,
                 VERTICAL_DIALOG_MARGIN));
    }

}