/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.util;

import java.util.concurrent.ExecutionException;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.jdesktop.swingworker.SwingWorker;


/**
 * Adds behavior to its superclass SwingWorker to properly handle
 * exceptions thrown during the background execution.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.5 $
 *
 * @param <T> the result type returned by this {@code SwingWorker's}
 *        {@code doInBackground} and {@code get} methods
 * @param <V> the type used for carrying out intermediate results by this
 *        {@code SwingWorker's} {@code publish} and {@code process} methods
 */
public abstract class SafeWorker<T, V> extends SwingWorker<T, V> {

    @Override
    protected final void done() {
        try {
            if (isCancelled()) {
                cancelled();
            } else {
                try {
                    succeeded(get());
                } catch (InterruptedException e) {
                    interrupted(e);
                } catch (ExecutionException e) {
                    failed(e.getCause());
                }
            }
        } finally {
            finished();
        }
    }


    /**
     * Called when the background operation has been successfully executed.
     * In other words, it hasn't been canceled, is not interrupted
     * and has not thrown an exception during the execution.
     *
     * @param result  the result of the worker execution as returned by
     *    {@link #get()}; may be {@code null}.
     */
    protected void succeeded(T result) {
        // Default implementation does nothing.
    }


    /**
     * Called when this worker has been canceled
     * by means of {@link #cancel(boolean)} method.
     */
    protected void cancelled() {
        // Default implementation does nothing.
    }


    /**
     * Called when a background execution of a worker throws an exception.
     * This default implementation throws Errors immediately, and
     * hands over exceptions to {@code Application#handleException}.
     *
     * @param throwable  the exception thrown during the background execution
     */
    protected void failed(Throwable throwable) {
        if (throwable instanceof Error) {
            throw (Error) throwable;
        }
        Logger.getLogger(getClass().getName()).log(Level.SEVERE,
                "Background execution failed", throwable);
    }


    /**
     * Called when a worker background execution has been interrupted.
     */
    protected void interrupted(InterruptedException e) {
        // Default implementation does nothing.
    }


    /**
     * Called as the last operation when a task is done.
     * This method is called when the background execution
     * succeeded, failed, has been cancelled or interrupted.
     * It is intended for cleaning up the task, task state
     * or releasing resources.
     */
    protected void finished() {
        // Default implementation does nothing.
    }


}
