/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.util;

import java.awt.*;
import java.util.prefs.Preferences;

/**
 * This class consists only of static convenience behavior for screen handling.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.9 $
 */
public final class ScreenUtils {

    private ScreenUtils() {
        // Override default constructor - prevents instantiation.
    }


    // Setting a Frequently used Screen Location ******************************

    /**
     * Locates the given component on the screen's center.
     *
     * @param component   the component to be centered
     *
     * @see #locateOnOpticalScreenCenter(Component)
     */
    public static void locateOnScreenCenter(Component component) {
        Dimension paneSize = component.getSize();
        Dimension screenSize = component.getToolkit().getScreenSize();
        component.setLocation(
            (screenSize.width  - paneSize.width)  / 2,
            (screenSize.height - paneSize.height) / 2);
    }


    /**
     * Locates the given component on the optical screen center.
     * "Centering" means to bias vertical placement slightly towards
     * the top of the screen, instead of placing exactly in the middle.
     * This is done because the user's eye is naturally biased towards
     * the top of the screen.
     *
     * @param component   the component to be optically centered
     *
     * @since 1.6
     */
    public static void locateOnOpticalScreenCenter(Component component) {
        Dimension paneSize = component.getSize();
        Dimension screenSize = component.getToolkit().getScreenSize();
        component.setLocation(
            (screenSize.width  - paneSize.width)  / 2,
            (int) ((screenSize.height - paneSize.height) * 0.45));
    }

    /**
     * Locates the given component on the screen's north-east.
     *
     * @param component   the component to be located
     */
    public static void locateOnScreenNorthEast(Component component) {
        Dimension paneSize = component.getSize();
        Dimension screenSize = component.getToolkit().getScreenSize();
        component.setLocation((screenSize.width - paneSize.width), 0);
    }

    /**
     * Locates the given component on the screen's west.
     *
     * @param component   the component to be located
     */
    public static void locateOnScreenWest(Component component) {
        Dimension paneSize = component.getSize();
        Dimension screenSize = component.getToolkit().getScreenSize();
        component.setLocation(0, (screenSize.height - paneSize.height) / 2);
    }

    /**
     * Locates the given component on the screen's east.
     *
     * @param component   the component to be located
     */
    public static void locateOnScreenEast(Component component) {
        Dimension paneSize = component.getSize();
        Dimension screenSize = component.getToolkit().getScreenSize();
        component.setLocation(
            (screenSize.width  - paneSize.width),
            (screenSize.height - paneSize.height) / 2);
    }


    // Screen Configurations **************************************************

    /**
     * Returns if the screen dimension is equal or larger than 1024x768 pixel.
     *
     * @return true if the screen is at least 1024x768 pixel
     */
    public static boolean has1024x768Screen() {
        Dimension screenSize = screenSize();
        return (screenSize.width >= 1024) && (screenSize.height >= 768);
    }

    /**
     * Returns if the screen dimension is equal or larger than 800x600 pixel.
     *
     * @return true if the screen is at least 800x600 pixel
     */
    public static boolean has800x600Screen() {
        Dimension screenSize = screenSize();
        return (screenSize.width >= 800) && (screenSize.height >= 600);
    }


    /**
     * Looks up and returns the screen configuration encoded as a String.
     * Examples: "[0,0,1024,768]" for a single 1024x768 screen,
     * "[0,0,1024,768][1024,0,1024,768]" for a dual-screen configuration
     * sitting side-by-side where each screen has 1024x768 pixels.<p>
     *
     * This String can be used to identify a configuration when storing
     * and restoring screen state such as bounds, table column width, etc.
     * If the screen size, the number of available screens, or their
     * position changes, the encoded configuration changes.
     *
     * @return the screen configuration encoded as a string
     */
    public static String encodedScreenConfiguration() {
        StringBuilder builder = new StringBuilder();
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        for (GraphicsDevice gd : ge.getScreenDevices()) {
            GraphicsConfiguration gc = gd.getDefaultConfiguration();
            builder.append(encodeRectangle(gc.getBounds()));
        }
        return builder.toString();
    }


    /**
     * Looks up and returns a preferences child node for the given preferences.
     *
     * @param prefs    the preferences parent node
     * @return a preferences child node for the screen configuration.
     *
     * @see #encodedScreenConfiguration()
     */
    public static Preferences screenConfigurationNode(Preferences prefs) {
        return prefs.node(ScreenUtils.encodedScreenConfiguration());
    }


    // Helper Code ************************************************************

    /**
     * Returns a short String representation of a Rectangle,
     * for example "[0,0,1024,768]".
     */
    private static String encodeRectangle(Rectangle r) {
        return "[" + r.x + "," + r.y + "," + r.width + "," + r.height + "]";
    }


    private static Dimension screenSize() {
        return Toolkit.getDefaultToolkit().getScreenSize();
    }


}
