/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.util;

import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.Toolkit;
import java.io.PrintStream;
import java.text.NumberFormat;
import java.util.Enumeration;
import java.util.Properties;
import java.util.logging.Logger;

import javax.swing.UIManager;

/**
 * Provides convenience behavior to determine the operating system
 * and a few other odds and ends.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.12 $
 */
public final class SystemUtils {

    // Requesting the OS and OS Version ***************************************

    /**
     * The <code>os.name</code> System Property. Operating system name.<p>
     *
     * Defaults to {@code null} if the runtime does not have security
     * access to read this property or the property does not exist.
     */
    private static final String OS_NAME = getSystemProperty("os.name");

    /**
     * The <code>os.version</code> System Property. Operating system version.<p>
     *
     * Defaults to {@code null} if the runtime does not have security
     * access to read this property or the property does not exist.
     */
    private static final String OS_VERSION = getSystemProperty("os.version");

    /**
     * Is true if this is Linux.
     */
    public static final boolean IS_OS_LINUX = startsWith(OS_NAME, "Linux")
                                           || startsWith(OS_NAME, "LINUX");

    /**
     * Is true if this is the Mac OS.
     */
    public static final boolean IS_OS_MAC =
        startsWith(OS_NAME, "Mac OS");

    /**
     * True if this is Solaris.
     */
    public static final boolean IS_OS_SOLARIS =
        startsWith(OS_NAME, "Solaris");

    /**
     * Is true if this is the Windows OS.
     */
    public static final boolean IS_OS_WINDOWS =
        startsWith(OS_NAME, "Windows");

    /**
     * Is true if this is the Windows 2000 OS.
     */
    public static final boolean IS_OS_WINDOWS_2000 =
        startsWith(OS_NAME, "Windows 2000");

    /**
     * Is true if this is the Windows XP OS.
     */
    public static final boolean IS_OS_WINDOWS_XP  =
        IS_OS_WINDOWS && startsWith(OS_VERSION, "5.1");


    // Requesting the Java Version ********************************************

    /**
     * The <code>os.name</code> System Property. Operating system name.<p>
     *
     * Defaults to {@code null} if the runtime does not have security
     * access to read this property or the property does not exist.
     */
    private static final String JAVA_VERSION = getSystemProperty("java.version");

    /**
     * True if this is Java 5.x. We check for a prefix of 1.5.
     */
    public static final boolean IS_JAVA_5 =
        startsWith(JAVA_VERSION, "1.5");

    /**
     * True if this is Java 6. We check for a prefix of 1.6.
     */
    public static final boolean IS_JAVA_6 =
        startsWith(JAVA_VERSION, "1.6");

    /**
     * True if this is Java 6.x or later. Since we don't support Java 1.4,
     * we can check that it's not 1.5.
     */
    public static final boolean IS_JAVA_6_OR_LATER =
        !IS_JAVA_5;


    // User Interface Requests ************************************************

    /**
     * Is true if this environment's default toolkit reports a screen resolution
     * below 120 dpi.
     */
    public static final boolean IS_LOW_RES = isLowRes();


    private SystemUtils() {
        // Override default constructor; prevents instantiation.
    }


    /**
     * Dumps the given <code>Properties</code> to the specified <code>PrintStream</code>.
     *
     * @param stream        the stream to write to
     * @param properties    the properties to dump
     */
    public static void dumpProperties(
        PrintStream stream,
        Properties properties) {
        for (Enumeration e = properties.propertyNames();
            e.hasMoreElements();
            ) {
            String key = (String) e.nextElement();
            if (key.equals("line.separator"))
                continue;
            stream.println(key + " = " + properties.getProperty(key));
        }
    }


    /**
     * Looks up and returns an extended set of system properties that
     * consist of the standard system properties plus those added
     * by <code>#addSpecialProperties</code>.<p>
     *
     * First calls <code>System.getProperties</code> that may result in
     * a security exception if there is a security manager that doesn't
     * allow access to the system properties.
     *
     * @return the extended system properties
     * @throws SecurityException  if a security manager exists and its
     *     <code>checkPropertiesAccess</code> method doesn't allow access
     *     to the system properties.
     * @see java.lang.SecurityException
     * @see java.lang.SecurityManager#checkPropertiesAccess()
     * @see java.util.Properties
     */
    public static Properties getExtendedSystemProperties() {
        Properties properties = (Properties) System.getProperties().clone();
        addSpecialProperties(properties);
        return properties;
    }


    /*
    A safe variant of the above method, contributed by Andrej Golovnin.

    public static Properties getExtendedSystemProperties() {
        Properties properties = new Properties();
        try {
            properties.putAll(System.getProperties());
        } catch (SecurityException e) {
            Logger.getLogger(SystemUtils.class.getName()).warning(
                    "Can't access the System properties.");
        }
        addSpecialProperties(properties);
        return properties;
    }
    */


    /**
     * Sleeps for the given <code>milliseconds</code>
     * catching a potential InterruptedException.
     *
     * @param milliseconds   the time to sleep in milliseconds
     */
    public static void sleep(long milliseconds) {
        try {
            Thread.sleep(milliseconds);
        } catch (InterruptedException e) {
            // Do nothing on interrupt.
        }
    }


    /**
     * Checks and answers if a system exit is allowed or not.
     * For example, as a plug-in you should not exit the system
     * but silently shut down your application threads.
     *
     * @return true if <code>System.exit()</code> is allowed
     */
    public static boolean systemExitAllowed() {
        return !"false".equals(
            getSystemProperty("jgoodies.SystemExitAllowed"));
    }


    // Private Helper Methods ***********************************************************

    /**
     * Tries to look up the System property for the given key.
     * In untrusted environments this may throw a SecurityException.
     * In this case we catch the exception and answer an empty string.
     *
     * @param key   the name of the system property
     * @return the system property's String value, or {@code null} if there's
     *     no such value, or an empty String when
     *     a SecurityException has been caught
     */
    private static String getSystemProperty(String key) {
        try {
            return System.getProperty(key);
        } catch (SecurityException e) {
            Logger.getLogger(SystemUtils.class.getName()).warning(
                  "Can't access the System property " + key + ".");
            return "";
        }
    }


    /**
     * Adds some properties to the specified <code>Properties</code>.
     * These are: screen resolution, screen size, color model, and
     * used/free/total runtime memory.
     *
     * @param properties    the properties to be added
     */
    private static void addSpecialProperties(Properties properties) {
        // Add AWT properties that we are interested in.
        Toolkit toolkit = Toolkit.getDefaultToolkit();
        properties.put(
            "awt.screen.resolution",
            Integer.valueOf(toolkit.getScreenResolution()).toString());
        Dimension screenSize = toolkit.getScreenSize();
        properties.put(
            "awt.screen.size",
            screenSize.width + " x " + screenSize.height);
        properties.put(
            "awt.colormodel",
            toolkit.getColorModel().getClass().getName());

        // Add Look & Feel data
        properties.put("look&feel.id",   UIManager.getLookAndFeel().getID());
        properties.put("look&feel.name", UIManager.getLookAndFeel().getName());

        // Add memory data
        long free = Runtime.getRuntime().freeMemory();
        long total = Runtime.getRuntime().totalMemory();
        long used = total - free;
        NumberFormat format = NumberFormat.getNumberInstance();
        properties.put("runtime.memory.used", format.format(used));
        properties.put("runtime.memory.free", format.format(free));
        properties.put("runtime.memory.total", format.format(total));
    }


    private static boolean startsWith(String str, String prefix) {
        return str != null && str.startsWith(prefix);
    }


    private static boolean isLowRes() {
        try {
            return Toolkit.getDefaultToolkit().getScreenResolution() < 120;
        } catch (HeadlessException e) {
            return true;
        }
    }


}
