/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.util;

import java.util.Enumeration;

import javax.swing.JTree;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

/**
 * Consists only of static methods for convenience tree operations.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.7 $
 *
 * @see JTree
 */
public final class TreeUtils {

    private TreeUtils() {
       // Overrides default constructor; prevents instantiation
    }


    /**
     * Collapses all siblings of the specified <code>TreePath</code>.
     *
     * @param tree   the tree to operate on
     * @param aPath   the path that defines which siblings are to be collapsed
     */
    public static void collapseAllSiblings(JTree tree, TreePath aPath) {
        TreeNode node = (TreeNode) aPath.getLastPathComponent();
        collapseChildren(tree, aPath);
        TreePath path = aPath.getParentPath();
        while (path != null) {
            TreeNode parent = (TreeNode) path.getLastPathComponent();

            Enumeration siblings = parent.children();
            while (siblings.hasMoreElements()) {
                TreeNode aSibling = (TreeNode) siblings.nextElement();
                if (aSibling != node) {
                    TreePath siblingPath = path.pathByAddingChild(aSibling);
                    tree.collapsePath(siblingPath);
                }
            }
            node = parent;
            path = path.getParentPath();
        }
    }


    /**
     * Collapses the children of the specified <code>TreePath</code>.
     *
     * @param tree   the tree to operate on
     * @param path   the path to the parent of the children to be collapsed
     */
    public static void collapseChildren(JTree tree, TreePath path) {
        TreeNode parent = (TreeNode) path.getLastPathComponent();
        Enumeration enumeration = parent.children();
        while (enumeration.hasMoreElements()) {
            TreeNode child = (TreeNode) enumeration.nextElement();
            TreePath childPath = path.pathByAddingChild(child);
            tree.collapsePath(childPath);
        }
        tree.collapsePath(path);
    }


    /**
     * Expands the nodes of the given tree up to the given level:
     * 0 does nothing, 1 expands the top level, 2 expands the first
     * two levels, 3 expands the first three levels, etc.
     *
     * @param tree              the tree to operate on
     * @param maxLevelToExpand  the maximum level to expand
     */
    public static void expand(JTree tree, int maxLevelToExpand) {
        for (int level = 0; level < maxLevelToExpand; level++) {
            for (int row = tree.getRowCount(); row >= 0; row--) {
                tree.expandRow(row);
            }
        }
    }


    /**
     * Expands the top level rows of the given tree.
     *
     * @param tree   the tree to operate on
     */
    public static void expandTopLevel(JTree tree) {
        expand(tree, 1);
    }


}
