/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.util;

import java.awt.Component;
import java.awt.FocusTraversalPolicy;
import java.awt.Window;

import javax.swing.JRootPane;
import javax.swing.JScrollBar;
import javax.swing.LayoutFocusTraversalPolicy;
import javax.swing.text.JTextComponent;

/**
 * A FocusTraversalPolicy that determines traversal order based on the order
 * of child Components in a Container. In addition to its superclass,
 * this class excludes non-editable text components, scroll bars, and the
 * root pane from the focus cycle. Also this policy offers to set an optional
 * initial component that overrides the Swing default initial component.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.7 $
 * @since 1.4
 *
 * @see JTextComponent#isEditable()
 */
public class UIFFocusTraversalPolicy extends LayoutFocusTraversalPolicy {

    /**
     * Holds a default instance of the focus traversal policy,
     * intended to be used as the global default policy.
     *
     * @see java.awt.KeyboardFocusManager#setDefaultFocusTraversalPolicy(java.awt.FocusTraversalPolicy)
     */
    public static final FocusTraversalPolicy DEFAULT
        = new UIFFocusTraversalPolicy();


    /**
     * Refers to the optional component that will be focused
     * if this policy's windows becomes visible the first time.
     *
     * @see #setInitialComponent(Component)
     * @see #getInitialComponent(Window)
     */
    private Component initialComponent;


    // Instance Creation ******************************************************

    /**
     * Constructs a UIFFocusTraversalPolicy with no initial component set.
     */
    public UIFFocusTraversalPolicy() {
         // Widen the visibility for this policy.
    }


    /**
     * Constructs a UIFFocusTraversalPolicy with the given Component
     * as initial component.
     *
     * @param initialComponent    the Component that will be returned
     *     as initial component instead of Swing default initial component
     */
    public UIFFocusTraversalPolicy(Component initialComponent) {
        this.initialComponent = initialComponent;
    }


    // ************************************************************************

    /**
     * Determines whether a Component is an acceptable choice as the new
     * focus owner. By default, this method will accept a Component if and
     * only if it is visible, displayable, enabled, and focusable,
     * no JScrollBar, no JRootPane, and in case of a JTextComponent,
     * it must be editable.
     *
     * @param aComponent the Component whose fitness as a focus owner is to
     *        be tested
     * @return {@code true} if aComponent is visible, displayable,
     *         enabled, and focusable; {@code false} otherwise
     */
    @Override
    protected boolean accept(Component aComponent) {
        if (!super.accept(aComponent)) {
            return false;
        } else if (aComponent instanceof JTextComponent) {
            JTextComponent textComponent = (JTextComponent) aComponent;
            return textComponent.isEditable();
        } else if (aComponent instanceof JScrollBar) {
            return false;
        } else if (aComponent instanceof JRootPane) {
            return false;
        } else {
            return true;
        }
    }


    /**
     * Sets the Component that should receive the focus when a Window is
     * made visible for the first time.
     *
     * @param initialComponent  the Component that should receive
     *      the focus when window is made visible for the first time,
     *      or null to use the super implementation's initial component
     *
     * @see #getInitialComponent(Window)
     */
    public void setInitialComponent(Component initialComponent) {
        this.initialComponent = initialComponent;
    }


    /**
     * Returns the Component that should receive the focus when a Window is
     * made visible for the first time. In addition to the super implementation
     * this method first checks, if this policy has an initial component set.
     *
     * @param window the Window whose initial Component is to be returned
     * @return the Component that should receive the focus when window is made
     *         visible for the first time, or null if no suitable Component can
     *         be found
     *
     * @see #setInitialComponent(Component)
     * @see FocusTraversalPolicy#getDefaultComponent
     * @see Window#getMostRecentFocusOwner
     * @throws IllegalArgumentException if window is null
     */
    @Override
    public Component getInitialComponent(Window window) {
        return initialComponent != null
            ? initialComponent
            : super.getInitialComponent(window);
    }


}
