/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uif.util;

import java.awt.*;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.EventObject;
import java.util.StringTokenizer;
import java.util.prefs.Preferences;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPopupMenu;
import javax.swing.JWindow;

import com.jgoodies.binding.value.DelayedReadValueModel;
import com.jgoodies.binding.value.ValueHolder;
import com.jgoodies.binding.value.ValueModel;
import com.jgoodies.uif.application.ApplicationContext;


/**
 * Provides static convenience behavior to store and restore window bounds
 * in a Preferences node and to ensure a minimum size for Swing windows.<p>
 *
 * Window bounds can be stored by the <code>#storeBounds</code> methods.
 * These require that the given JWindow, JDialog or JFrame provide
 * a unique window ID that is requested by <code>#getWindowID()</code>.<p>
 *
 * A second feature allows to ensure a window minimum size. The
 * <code>SizeChangeHandler</code> listens to window resize events
 * and sets a new size if the user has shrunken the window below
 * a given dimension. This mechanism requires that the Swing window
 * class implements a method <code>#getWindowMinimumSize()</code>.<p>
 *
 * Both features have been specialized for instances of JFrame, to
 * avoid storing the bounds of a maximized frame in the preferences.
 * Therefore the SizeChangeHandler stores bounds (in a frame client
 * property) but only if the frame is not maximized.
 * Using a ComponentAdapter that stores bounds every time the window
 * has been moved or resized fails, because sometimes expanding a frame
 * is splitted in a move event plus resize event, where the first move
 * does not yet indicate the expanded state - even if delayed using
 * <code>SwingUtilities#invokeLater</code>.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.16 $
 */

public final class WindowUtils {

    // Private Constants ******************************************************

    /**
     * The name of the method that is used to request a window's ID.
     */
    private static final String WINDOW_ID_METHOD_NAME = "getWindowID";

    /**
     * The name of the method that is used to request a window's minimum size.
     */
    private static final String MIN_SIZE_METHOD_NAME = "getWindowMinimumSize";

    /**
     * Part of the key that is used to persist window bounds in the preferences.
     */
    private static final String WINDOW_BOUNDS_KEYPART = "window_bounds";

    /**
     * Part of the key that is used to persist the extended frame state
     * in the preferences.
     */
    private static final String FRAME_STATE_KEYPART = "frame_state";



    private WindowUtils() {
        // Override default constructor - prevents instantiation.
    }


    // Setting a Frame's Icon ***********************************************

    /**
     * Chooses an appropriate image from the given image set and sets
     * it as image icon in the given frame. Checks the platform and
     * screen resolution to find the image size that fits best.
     *
     * @param frame       the frame to set an icon image for
     * @param image12x12  a 12x12 pixel image useful for Windows 2000
     * @param image16x16  a 16x16 pixel image used for most platforms
     *
     * @since 1.3
     */
    public static void setImageIcon(
                Frame frame,
                Image image12x12,
                Image image16x16) {
        Image primaryImage;
        Image secondaryImage;
        if (SystemUtils.IS_OS_MAC) {
            primaryImage   = null;
            secondaryImage = null;
        } else if (SystemUtils.IS_OS_WINDOWS_2000 && SystemUtils.IS_LOW_RES) {
            primaryImage   = image12x12;
            secondaryImage = image16x16;
        } else {
            primaryImage   = image16x16;
            secondaryImage = image12x12;
        }
        frame.setIconImage(primaryImage != null
                ? primaryImage
                : secondaryImage);
    }


    // Storing and Restoring Bounds *****************************************

    /**
     * Stores the window bounds in a child node of the given preferences,
     * that reflects the current screen configuration.
     * The bounds Rectangle is encoded as a String.<p>
     *
     * The window must provide an ID via method <code>#getWindowID</code>.
     *
     * @param prefs    the preferences to store the bounds in
     * @param window   the window that provides the bounds
     * @throws IllegalArgumentException   if the window provides no window ID
     */
    public static void storeBounds(Preferences prefs, JWindow window) {
        storeBounds0(prefs, window);
    }


    /**
     * Stores the dialog bounds in a child node of the given preferences,
     * that reflects the current screen configuration.
     * The bounds Rectangle is encoded as a String.<p>
     *
     * The dialog must provide an ID via method <code>#getWindowID</code>.
     *
     * @param prefs    the preferences to store the bounds in
     * @param dialog   the dialog that provides the bounds
     * @throws IllegalArgumentException   if the dialog provides no window ID
     */
    public static void storeBounds(Preferences prefs, JDialog dialog) {
        storeBounds0(prefs, dialog);
    }


    /**
     * Stores the frame bounds in a child node of the given preferences,
     * that reflects the current screen configuration.
     * The bounds are stored, if and only if the frame is visible
     * and not maximized in both directions.
     * The bounds Rectangle is encoded as a String.<p>
     *
     * The frame must provide an ID via method <code>#getWindowID</code>.
     *
     * @param prefs    the preferences to store the bounds in
     * @param frame    the frame that provides the bounds
     * @throws IllegalArgumentException   if the frame provides no window ID
     */
    public static void storeBounds(Preferences prefs, Frame frame) {
        boolean maximizedBoth = frame.getExtendedState() == Frame.MAXIMIZED_BOTH;
        if (frame.isVisible() && !maximizedBoth) {
            storeBounds0(prefs, frame);
        }
    }


    /**
     * Stores the frame's extended state in a child node of given preferences
     * that reflects the current screen configuration.
     * The state is encoded as a String that represents the state's bit mask.<p>
     *
     * The frame must provide an ID via method <code>#getWindowID</code>.
     *
     * @param prefs    the preferences to store the extended state in
     * @param frame    the frame that provides the window ID, state and toolkit
     * @throws IllegalArgumentException   if the frame provides no window ID
     *
     * @since 1.4
     */
    public static void storeState(Preferences prefs, Frame frame) {
        Preferences screenConfigNode =
            ScreenUtils.screenConfigurationNode(prefs);
        screenConfigNode.put(
                frameStateKey(frame),
                Integer.toString((frame.getExtendedState())));
    }


    /**
     * Restores the window's bounds from the given preferences.
     * Decodes the String encoding of the bounds rectangle, and sets
     * the bounds if the bounds are present and have a valid encoding.<p>
     *
     * The window must provide an ID via method <code>#getWindowID</code>.
     *
     * @param window   the window to be restored
     * @param prefs    the preferences that provide the bounds
     * @throws IllegalArgumentException   if the window provides no window ID
     */
    public static void restoreBounds(JWindow window, Preferences prefs) {
        Rectangle bounds = getStoredBounds(prefs, window);
        if (bounds != null)
            window.setBounds(bounds);
    }


    /**
     * Restores the window's bounds from the given preferences.
     * Decodes the String encoding of the bounds rectangle,
     * and sets the bounds if the bounds are present and have
     * a valid encoding.<p>
     *
     * The dialog must provide an ID via method <code>#getWindowID</code>.
     *
     * @param dialog   the dialog to be restored
     * @param prefs    the preferences that provide the bounds
     * @throws IllegalArgumentException   if the dialog provides no window ID
     */
    public static void restoreBounds(JDialog dialog, Preferences prefs) {
        Rectangle bounds = getStoredBounds(prefs, dialog);
        if (bounds != null)
            dialog.setBounds(bounds);
    }


    /**
     * Restores the frame's bounds from the given preferences.
     * Decodes the String encoding of the bounds rectangle,
     * and sets the bounds if the bounds are present and have
     * a valid encoding.<p>
     *
     * The frame must provide an ID via method <code>#getWindowID</code>.
     *
     * @param frame    the frame to be restored
     * @param prefs    the preferences that provide the bounds
     * @throws IllegalArgumentException   if the frame provides no window ID
     */
    public static void restoreBounds(JFrame frame, Preferences prefs) {
        Rectangle bounds = getStoredBounds(prefs, frame);
        if (bounds != null)
            frame.setBounds(bounds);
    }


    /**
     * Restores the frame's state from the given preferences.
     * Decodes the stored encoded state and sets it - even if invalid or
     * not supported by the platform. In the latter two cases, the frame
     * will do nothing.<p>
     *
     * The frame must provide an ID via method <code>#getWindowID</code>.
     *
     * @param frame    the frame to be restored
     * @param prefs    the preferences that provide the state
     * @throws IllegalArgumentException   if the frame provides no window ID
     *
     * @since 1.4
     */
    public static void restoreState(Frame frame, Preferences prefs) {
        restoreState(frame, prefs, true);
    }


    /**
     * Restores the frame's state from the given preferences.
     * Decodes the stored encoded state and sets it - even if invalid or
     * not supported by the platform. In the latter two cases, the frame
     * will do nothing. The iconified state is restored iff the boolean
     * parameter is set to {@code true}.<p>
     *
     * The frame must provide an ID via method <code>#getWindowID</code>.
     *
     * @param frame    the frame to be restored
     * @param prefs    the preferences that provide the state
     * @param restoreIconified  true to restore the iconified state,
     *      false to not restore the iconified state
     * @throws IllegalArgumentException   if the frame provides no window ID
     *
     * @since 1.4
     */
    public static void restoreState(Frame frame, Preferences prefs, boolean restoreIconified) {
        int state = getStoredState(frame, prefs);
        if (state == -1)
            return;
        if (state != Frame.ICONIFIED || !restoreIconified) {
            frame.setExtendedState(state);
        }
    }


    // Getting the Parent Window **********************************************

    /**
     * Returns the parent window for a given event.
     *
     * @param e   the event that may have a component as source.
     * @return the parent window of the event's source
     *
     * @since 1.7
     */
    public static Window getParent(EventObject e) {
        if (e == null) {
            return null;
        }
        return getWindowAncestor(e.getSource());
    }


    // Helper Methods *********************************************************

    private static void storeBounds0(Preferences prefs, Window window) {
        Preferences screenConfigNode =
            ScreenUtils.screenConfigurationNode(prefs);
        screenConfigNode.put(windowBoundsKey(window), encodeRectangle(window.getBounds()));
    }


    private static Rectangle getStoredBounds(Preferences prefs, Window window) {
        Preferences screenConfigNode =
            ScreenUtils.screenConfigurationNode(prefs);
        return decodeRectangle(screenConfigNode.get(windowBoundsKey(window), ""));
    }


    /**
     * Looks up and returns the frame's stored state from the given preferences.
     * Decodes the stored encoded state and returns it - even if invalid or
     * not supported by the platform. Returns <code>-1</code> if no state
     * has been stored or the encoding cannot be parsed.<p>
     *
     * The frame must provide an ID via method <code>#getWindowID</code>.<p>
     *
     * The node used to read the state changes with the screen configuration.
     * This ensures that window bounds are valid for the
     * current screen configuration. For example if the screen size has been
     * shrunk, the bounds stored for a larger screen won't be used.
     *
     * @param frame    the frame that provides the window ID and toolkit
     * @param prefs    the preferences that provide the state
     * @return the frame's decoded stored state, or <code>-1</code>
     *      if no state has been stored, or the stored state cannot be parsed
     * @throws IllegalArgumentException   if the frame provides no window ID
     */
    private static int getStoredState(Frame frame, Preferences prefs) {
        Preferences screenConfigNode =
            ScreenUtils.screenConfigurationNode(prefs);
        String encodedState = screenConfigNode.get(frameStateKey(frame), "");
        try {
            return Integer.parseInt(encodedState);
        } catch (NumberFormatException e) {
            return -1;
        }
    }


    /**
     * Computes and returns a key used to store and restore window bounds
     * to/from Preferences. The key consists of a unique ID
     * that is requested from the window via <code>#getWindowID</code>,
     * a constant suffix.
     *
     * @param window   the window that provides the ID and toolkit
     * @return a key that is used to store and restore window bounds
     * @throws IllegalArgumentException if the window provides no window ID
     * @throws NullPointerException if the window is {@code null}
     */
    private static String windowBoundsKey(Window window) {
        String windowID = getWindowID(window);
        if (windowID == null)
            throw new IllegalArgumentException("The window must provide an ID:" + window);

        return windowID + '.' + WINDOW_BOUNDS_KEYPART;
    }


    /**
     * Computes and returns a key used to store and restore window bounds
     * to/from Preferences. The key consists of a unique ID
     * that is requested from the window via <code>#getWindowID</code>,
     * and a constant suffix.
     *
     * @param frame   the Frame that provides the ID and toolkit
     * @return a key that is used to store and restore window bounds
     * @throws IllegalArgumentException if the window provides no window ID
     * @throws NullPointerException if the frame is {@code null}
     */
    private static String frameStateKey(Frame frame) {
        String windowID = getWindowID(frame);
        if (windowID == null)
            throw new IllegalArgumentException("The frame must provide an ID:" + frame);

        return windowID + '.' + FRAME_STATE_KEYPART;
    }


    /**
     * Looks up and returns the unique ID of the given window - if it has any.
     * Used to compose preferences keys to store window state.
     * The ID is requested using the method <code>#getWindowID</code>
     * - if available. As a fallback the window's name is returned.
     *
     * @param window   the window that provides the ID
     * @return the window's unique ID or {@code null} if it provides none
     * @throws NullPointerException if the window is {@code null}
     */
    private static String getWindowID(Window window) {
        try {
            Method method = window.getClass().getMethod(WINDOW_ID_METHOD_NAME, (Class[])null);
            return (String) method.invoke(window,  (Object[])null);
        } catch (NoSuchMethodException e) {
            // Ignore and proceed with fallback
        } catch (IllegalAccessException e) {
            // Ignore and proceed with fallback
        } catch (InvocationTargetException e) {
            // Ignore and proceed with fallback
        }
        return window.getName();
    }


    /**
     * Looks up and returns the minimum size of the given window - if any.
     * The minimum size is requested using the method
     * <code>#getWindowMinimumSize</code>.
     *
     * @param window   the window that provides the minimum sizes
     * @return the window's minimum size or {@code null} if it provides none
     */
    private static Dimension getWindowMinimumSize(Window window) {
        try {
            Method method = window.getClass().getMethod(MIN_SIZE_METHOD_NAME, (Class[]) null);
            return (Dimension) method.invoke(window, (Object[]) null);
        } catch (NoSuchMethodException e) {
            return null;
        } catch (IllegalArgumentException e) {
            return null;
        } catch (IllegalAccessException e) {
            return null;
        } catch (InvocationTargetException e) {
            return null;
        }
    }


    /**
     * Encodes the given Rectangle and returns it as a String.
     * The reverse operation is <code>#decodeRectangle</code>.
     *
     * @param r   the Rectangle to be encoded
     * @return a String encoding of the given Rectangle
     */
    private static String encodeRectangle(Rectangle r) {
        return Integer.toString(r.x) + ", " + r.y + ", " + r.width + ", " + r.height;
    }

    /**
     * Decodes the given encoded Rectangle and returns a Rectangle
     * if it could be successfully decoded, or {@code null} otherwise.
     * The reverse operation is <code>#encodeRectangle</code>.
     *
     * @param encodedRectangle  the String encoded Rectangle
     * @return the decoded Rectangle, or {@code null} if the
     *     encoding is invalid
     */
    private static Rectangle decodeRectangle(String encodedRectangle) {
        StringTokenizer tokenizer = new StringTokenizer(encodedRectangle, " ,");
        try {
            int x = Integer.parseInt(tokenizer.nextToken());
            int y = Integer.parseInt(tokenizer.nextToken());
            int w = Integer.parseInt(tokenizer.nextToken());
            int h = Integer.parseInt(tokenizer.nextToken());
            return new Rectangle(x, y, w, h);
        } catch (Exception e) {
            return null;
        }
    }


    // Helper Code ************************************************************

    /**
     * Looks up and returns the root Window or Applet
     * for the given event source.
     *
     * @param eventSource   the event source to get the root for
     * @return the root Window or Applet for the event source
     */
    private static Window getWindowAncestor(Object eventSource) {
        if (eventSource instanceof Component) {
            return getWindowAncestor((Component) eventSource);
        } else if (eventSource instanceof MenuComponent) {
            MenuContainer mParent = ((MenuComponent) eventSource).getParent();
            if (mParent instanceof Component)
                return getWindowAncestor((Component) mParent);
        }
        return null;
    }


    private static Window getWindowAncestor(Component c) {
        Component p = c;
        while (p != null) {
            if (p instanceof JPopupMenu) {
                p = ((JPopupMenu) p).getInvoker();
            } else if (p instanceof Window) {
                return (Window)p;
            } else {
                p = p.getParent();
            }
        }
        return null;
    }


    // Helper Classes *********************************************************

    /**
     * A ComponentListener that handles size changes in Windows
     * to ensure a minimum size. For Frames it also stores the
     * bounds and state, if the frame has been moved or resized.<p>
     *
     * This class works around a nasty issue with AWT component moved events.
     * We want to store the frame bounds only if the frame is not maximized.
     * If a frame is maximized by the user, AWT send a component moved event
     * <em>before</em> it indicates the frame maximized state.
     * So we need to re-order of these two events.<p>
     *
     * This implementation handles component moved events a bit later,
     * using a DelayedReadValueModel from the JGoodies Binding.
     * If the component moves, the componentMovedModel is updated immediately.
     * But the DelayedComponentMovedHandler doesn't listen to this model directly,
     * it observes its delayed wrapper. This has two effects:
     * 1) multiple component moved events are coalesced to one, and
     * 2) after the delay the frame maximization state is already updated.
     * So in <code>#componentMoved</code> we can check the maximization state
     * and it'll report if the component moved event was a maximized event
     * or not.
     *
     * @see WindowUtils#storeBounds(Preferences, Frame)
     * @see WindowUtils#storeState(Preferences, Frame)
     */
    public static final class SizeChangeHandler extends ComponentAdapter {

        private static final int COMPONENT_MOVED_DELAY = 50;

        /**
         * Holds the observed window, one of: JWindow, JDialog or JFrame.
         */
        private final Window window;

        /**
         * Holds the most recent windows bounds.
         * Used to store frame bounds after frame moved events.
         */
        private ValueModel componentMovedModel;


        // Instance Creation --------------------------------------------------

        /**
         * Constructs a SizeChangeHandler for the given Swing window.
         *
         * @param window    the window to be observed
         */
        public SizeChangeHandler(JWindow window) {
            this.window = window;
            init();
        }

        /**
         * Constructs a SizeChangeHandler for the given Swing dialog.
         *
         * @param dialog    the dialog to be observed
         */
        public SizeChangeHandler(JDialog dialog) {
            this.window = dialog;
            init();
        }

        /**
         * Constructs a SizeChangeHandler for the given Swing frame.
         *
         * @param frame    the frame to be observed
         */
        public SizeChangeHandler(JFrame frame) {
            this.window = frame;
            init();
        }


        // Initialization -----------------------------------------------------

        private void init() {
            componentMovedModel = new ValueHolder();
            ValueModel delayedComponentMovedModel =
                new DelayedReadValueModel(componentMovedModel, COMPONENT_MOVED_DELAY, true);
            delayedComponentMovedModel.addValueChangeListener(new DelayedComponentMovedHandler());
        }


        // ComponentListener Implementation -----------------------------------

        /**
         * If a frame has been moved, we update the componentMovedModel.
         * This in turn will notify the DelayedComponentMovedHandler after a delay.
         */
        @Override
        public void componentMoved(ComponentEvent evt) {
            if (window instanceof JFrame) {
                JFrame frame = (JFrame) window;
                componentMovedModel.setValue(frame.getBounds());
            }
        }

        /**
         * If this window provides a minimum size the window's size
         * is adjusted after a resize to ensure this minimum size.
         * If the window is a JFrame, the frame bounds are stored
         * if the frame doesn't cover the full screen.
         *
         * @param evt   the given component event
         */
        @Override
        public void componentResized(ComponentEvent evt) {
            Dimension minimumSize = getWindowMinimumSize(window);
            if (minimumSize != null) {
                window.setSize(
                    Math.max(minimumSize.width,  window.getWidth()),
                    Math.max(minimumSize.height, window.getHeight()));
            }
            if (window instanceof JFrame) {
                JFrame frame = (JFrame) window;
                Preferences userPrefs = ApplicationContext.getInstance().getUserPreferences();
                WindowUtils.storeBounds(userPrefs, frame);
                WindowUtils.storeState(userPrefs, frame);
            }
        }


        // --------------------------------------------------------------------

        private void delayedComponentMoved() {
            if (window instanceof JFrame) {
                JFrame frame = (JFrame) window;
                Preferences userPrefs = ApplicationContext.getInstance().getUserPreferences();
                WindowUtils.storeBounds(userPrefs, frame);
                WindowUtils.storeState(userPrefs, frame);
            }
        }


        /**
         * Handles component moved events "a bit later".
         * See the SizeChangeHandler's JavaDoc class comment.
         */
        private final class DelayedComponentMovedHandler implements PropertyChangeListener {
            public void propertyChange(PropertyChangeEvent evt) {
                delayedComponentMoved();
            }
        }


    }


}
