/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.convenience;

import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.net.URL;

import javax.swing.*;

import com.jgoodies.forms.builder.ButtonBarBuilder2;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.Sizes;
import com.jgoodies.uif.AbstractDialog;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.util.Resizer;
import com.jgoodies.uifextras.help.HelpBroker;
import com.jgoodies.uifextras.panel.HeaderPanel;

/**
 * Provides the layout and actions for a generic about dialog.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.12 $
 *
 * @see DefaultAboutToolView
 */

public class DefaultAboutDialog extends AbstractDialog {

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(DefaultAboutDialog.class);


    private final JComponent toolTab;

    /**
     * Constructs a default about dialog using the given owner.
     *
     * @param owner   the dialog's owner
     */
    public DefaultAboutDialog(Frame owner) {
        this(owner, null);
    }

    /**
     * Constructs a default about dialog using the given owner and tool tab.
     *
     * @param owner   the dialog's owner
     * @param toolTab a custom component used as the <em>Tool</em> tab
     */
    public DefaultAboutDialog(Frame owner, JComponent toolTab) {
        super(owner);
        this.toolTab = toolTab != null
            ? toolTab : buildDefaultToolTab();
    }

    /**
     * Creates and returns the default tool tab.
     *
     * @return the default tool tab component
     */
    protected JComponent buildDefaultToolTab() {
        return new DefaultAboutToolView().getPanel();
    }

    /**
     * Builds and returns the dialog's content.
     *
     * @return the dialog's content with tabbed pane and button bar
     */
    @Override
    protected JComponent buildContent() {
        JPanel content = new JPanel(new BorderLayout());
        content.add(buildTabbedPane(), BorderLayout.CENTER);
        content.add(buildButtonBar(),  BorderLayout.SOUTH);
        return content;
    }

    /**
     * Builds and returns the dialog's header.
     *
     * @return the dialog's header component
     */
    @Override
    protected JComponent buildHeader() {
        return new HeaderPanel(
            "About " + RESOURCES.getString("application.name"),
            "You can click on the tabs below to see information\n" +
            "about the tool's version and the system's state.",
            RESOURCES.getIcon(ResourceIDs.ABOUT_ICON));
    }

    /**
     * Builds and returns the dialog's tabbed pane.
     *
     * @return the dialog's tabbed pane component
     */
    protected JComponent buildTabbedPane() {
        JTabbedPane tabbedPane = new JTabbedPane();
        tabbedPane.add("Tool",   toolTab);
        tabbedPane.add("System", buildSystemTab());
        return tabbedPane;
    }

    protected JComponent buildButtonBar() {
        ButtonBarBuilder2 builder = new ButtonBarBuilder2();
        builder.addGlue();
        builder.addButton(getShowLicenseAction());
        builder.addUnrelatedGap();
        builder.addButton(createCloseButton(true));
        builder.getPanel().setBorder(Borders.BUTTON_BAR_GAP_BORDER);
        return builder.getPanel();
    }

    /**
     * Resizes the given component to give it a quadratic aspect ratio.
     *
     * @param component   the component to be resized
     */
    @Override
    protected void resizeHook(JComponent component) {
        Resizer.ONE2ONE.resizeDialogContent(component);
    }


    /**
     * Builds and returns the dialog's system tab.
     *
     * @return the system tab component
     */
    protected JComponent buildSystemTab() {
        JPanel systemTable = new SystemTable();
        int width = Sizes.dialogUnitXAsPixel(100, systemTable);
        systemTable.setPreferredSize(Resizer.DEFAULT.fromWidth(width));
        systemTable.setBorder(Borders.DIALOG_BORDER);
        systemTable.setOpaque(false);
        return systemTable;
    }

    /**
     * Returns an Action that shows the license.
     *
     * @return the Action that shows license button
     */
    protected Action getShowLicenseAction() {
        return new ShowLicenseAction();
    }


    protected void doShowLicense() {
        close();
        URL url = RESOURCES.getURL(ResourceIDs.LICENSE_AGREEMENT_PATH);
        HelpBroker.openURL(url);
    }


    private final class ShowLicenseAction extends AbstractAction {
        private ShowLicenseAction() {
            super("Show License");
            putValue(Action.MNEMONIC_KEY, Integer.valueOf('S'));
        }
        public void actionPerformed(ActionEvent e) {
            doShowLicense();
        }
    }


}