/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.convenience;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.*;
import javax.swing.border.Border;

import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.uif.AbstractDialog;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.component.UIFButton;
import com.jgoodies.uif.panel.CardPanel;
import com.jgoodies.uif.util.Modes;
import com.jgoodies.uif.util.Resizer;

/**
 * Builds the default setup dialog, which consists of two pages:
 * a welcome page and the license acceptance page.
 *
 * @see	DefaultSetupWelcomePanel
 * @see	SetupLicensePanel
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.11 $
 */

public class DefaultSetupDialog extends AbstractDialog {

	private static final int   PREFERRED_WIDTH	= 600;
	private static final String WELCOME_PANEL		= "Welcome";
	private static final String LICENSE_PANEL		= "License";

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(DefaultSetupDialog.class);


    private final JComponent		welcomePanel;
	private final SetupLicensePanel	licensePanel;


	/**
	 * Constructs a default setup dialog using the given owner frame,
	 * application name, welcome panel, and license panel.
	 */
	public DefaultSetupDialog(JFrame owner, String applicationName,
		JComponent welcomePanel, SetupLicensePanel licensePanel) {
		super(owner, applicationName + " Setup");
		this.welcomePanel = welcomePanel;
		this.licensePanel = licensePanel;
	}


	// Building *************************************************************************

	/**
	 * Builds and answers the panels content.
	 */
	@Override
    protected JComponent buildContent() {
		CardPanel panel = new CardPanel();
		panel.add(buildWelcomePanel(), WELCOME_PANEL);
		panel.add(buildLicensePanel(), LICENSE_PANEL);

		Dimension size = Resizer.DEFAULT.fromWidth(PREFERRED_WIDTH);
		panel.setMinimumSize(size);
		panel.setPreferredSize(size);
		panel.setSize(size);

		return panel;
	}


	/**
	 * Builds and answers the welcome panel; hands over the button bar.
	 */
	private JComponent buildWelcomePanel() {
		((SetupManager.WizardPanel) welcomePanel).build(buildWelcomeButtonBar());
		return welcomePanel;
	}


	/**
	 * Builds and answer the license panel; hand over the button bar.
	 */
	private JComponent buildLicensePanel() {
		licensePanel.build(buildLicenseButtonBar());
		return licensePanel;
	}


	/**
	 * Answers the dialog's border. Since the default is a Wizard style
	 * we use an empty border.
	 */
	@Override
    protected Border getDialogBorder() {
		return Borders.EMPTY_BORDER;
	}


	/**
	 * Uses the default <code>Resizer</code>.
	 */
	@Override
    protected void resizeHook(JComponent component) {
		Resizer.DEFAULT.resizeDialogContent(component);
	}


	// Building Buttons *****************************************************************

	/**
	 * Creates and answers the back button.
	 */
	private JButton createBackButton() {
		JButton button = new JButton("< Back");   // TODO: I18n
		button.setMnemonic('B');
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				doBack();
			}
		});
		return button;
	}


	/**
	 * Creates and answers the next button.
	 */
	private JButton createNextButton() {
		UIFButton button = new UIFButton("Next >");  // TODO: I18n
		button.setMnemonic('N');
		button.setOpaqueMode(Modes.LAF_NON_AQUA);
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				doNext();
			}
		});
		return button;
	}


	/**
	 * Builds and answers the button bar for the welcome panel.
	 */
	private JComponent buildWelcomeButtonBar() {
		UIFButton cancelButton = createCancelButton();
		cancelButton.setOpaqueMode(Modes.LAF_NON_AQUA);
		JPanel bar = ButtonBarFactory.buildRightAlignedBar(
			     createNextButton(),
                 cancelButton);
        bar.setOpaque(false);
        return bar;
	}


	/**
	 * Builds and answers the button bar for the license panel.
	 */
	private JComponent buildLicenseButtonBar() {
		JButton finishButton = createAcceptButton("Finish", false);
		finishButton.setMnemonic('F');
		return ButtonBarFactory.buildRightAlignedBar(
			         createBackButton(),
                     finishButton,
                     createCancelButton());
	}


	// Command execution ****************************************************************

	/**
	 * Performs the accept operation.
	 */
	@Override
    public void doAccept() {
		if (licensePanel.licenseAccepted()) {
			super.doAccept();
			return;
		}
		if (licensePanel.licenseDeclined()) {
			doDecline();
			return;
		}
		JOptionPane.showMessageDialog(this,
				"You must select to either accept or decline the license agreement.",
				//sourceManager.getString(ResourceIDs.LICENSE_MUST_SELECT_TEXT),
				getTitle(), JOptionPane.INFORMATION_MESSAGE);
	}


	/**
	 * Performs the close window operation.
	 */
	@Override
    protected void doCloseWindow() {
		if (LICENSE_PANEL.equals(getCardPanel().getVisibleChildName()))
			doCancel();
		else
			doNext();
	}


	/**
	 * Performs the cancel operation.
	 */
	@Override
    public void doCancel() {
		int choice = JOptionPane.showConfirmDialog(this,
				"Do you really want to cancel the setup process?",
				//ResourceUtils.getString(ResourceIDs.LICENSE_CONFIRM_DECLINE_TEXT),
				getTitle(), JOptionPane.YES_NO_OPTION);

		if (JOptionPane.YES_OPTION == choice)
			super.doCancel();
	}


	/**
	 * Declines the license.
	 */
	private void doDecline() {
		int choice = JOptionPane.showConfirmDialog(this,
				RESOURCES.getString(ResourceIDs.LICENSE_CONFIRM_DECLINE_TEXT),
				getTitle(), JOptionPane.YES_NO_OPTION);

		if (JOptionPane.YES_OPTION == choice)
			super.doCancel();
	}


	/**
	 * Goes back to the previous wizard page.
	 */
	private void doBack() {
		getCardPanel().showPreviousCard();
	}


	/**
	 * Goes forward to the next wizard page.
	 */
	private void doNext() {
		getCardPanel().showNextCard();
		setLicensePage();
	}


	// Misc *****************************************************************************

	/**
	 * Answers the card panel that is used to hold the wizard pages.
	 */
	private CardPanel getCardPanel() {
		return (CardPanel) getContentPane();
	}


	/**
	 * Answers the license page.
	 */
	private void setLicensePage() {
		URL url = RESOURCES.getURL(ResourceIDs.LICENSE_AGREEMENT_PATH);
		try {
			licensePanel.setPage(url);
		} catch (IOException e) {
			Logger.getLogger("LicenseAcceptanceDialog").log(
				Level.WARNING, "Can't open URL " + url + ".", e);
		}
	}


}