/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.convenience;

import java.net.URL;
import java.util.prefs.Preferences;

import com.jgoodies.binding.beans.Model;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ApplicationContext;
import com.jgoodies.uif.application.ResourceMap;

/**
 * Provides a dialog and managing behavior for a tip of the day functionality.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.8 $
 */
public final class DefaultTipOfTheDayModel extends Model implements TipOfTheDayModel {

    private static final String  KEY_PREFIX = "com.jgoodies.uif.TipOfTheDay";
    private static final String  TIP_COUNT_RESOURCE_KEY = KEY_PREFIX + ".tipCount";
    private static final String  IS_SHOWING_KEY = KEY_PREFIX + ".is_showing";
    private static final boolean IS_SHOWING_DEFAULT = true;
    private static final String  TIP_INDEX_KEY = KEY_PREFIX + ".index";
    private static final int     TIP_INDEX_DEFAULT = 0;

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(DefaultTipOfTheDayModel.class);

    private static final DefaultTipOfTheDayModel INSTANCE = new DefaultTipOfTheDayModel();

    private final int tipCount;
    private final Preferences preferences;


    // Instance  Creation ****************************************************

    /**
     * Constructs the default tip of the day model and initializes
     * the current tip index.
     */
    private DefaultTipOfTheDayModel() {
        tipCount = RESOURCES.getInt(TIP_COUNT_RESOURCE_KEY);
        preferences = ApplicationContext.getInstance().getUserPreferences();
    }


    public static DefaultTipOfTheDayModel getInstance() {
        return INSTANCE;
    }


    // TipOfTheDayModel Implementation ****************************************

    /**
     * Returns the next tip, either a plain or HTML String.
     *
     * @return the next tip
     */
    public URL nextTip() {
        int currentIndex = getCurrentIndex();
        if (++currentIndex >= tipCount) {
            currentIndex = 0;
        }
        setCurrentIndex(currentIndex);
        return getTipAt(currentIndex);
    }


    /**
     * Returns the next tip, either a plain or HTML String.
     *
     * @return the next tip
     */
    public URL previousTip() {
        int currentIndex = getCurrentIndex();
        if (--currentIndex < 0) {
            currentIndex = tipCount -1;
        }
        setCurrentIndex(currentIndex);
        return getTipAt(currentIndex);
    }


    /**
     * Returns if we shall show tips at application startup.
     *
     * @return true to show tips, false to not show tips
     */
    public boolean isShowingTips() {
        return preferences.getBoolean(IS_SHOWING_KEY, IS_SHOWING_DEFAULT);
    }

    /**
     * Sets if we shall show tips at application startup.
     *
     * @param b    true to enable the tips, false to disable
     */
    public void setShowingTips(boolean b) {
        boolean oldValue = isShowingTips();
        preferences.putBoolean(IS_SHOWING_KEY, b);
        firePropertyChange(SHOWING_TIPS_PROPERTY, oldValue, b);
    }


    // Helper Code ************************************************************

    private int getCurrentIndex() {
        return preferences.getInt(TIP_INDEX_KEY, TIP_INDEX_DEFAULT);
    }

    private void setCurrentIndex(int index) {
        preferences.putInt(TIP_INDEX_KEY, index);
    }

    private URL getTipAt(int index) {
        String tipPathKey = KEY_PREFIX + ".[" + index + ']';
        return RESOURCES.getURL(tipPathKey);
   }


}