/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.convenience;

import java.awt.Image;
import java.awt.KeyboardFocusManager;
import java.io.IOException;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JOptionPane;
import javax.swing.UIManager;

import com.jgoodies.looks.LookUtils;
import com.jgoodies.uif.AbstractFrame;
import com.jgoodies.uif.application.AbstractUIFApplication;
import com.jgoodies.uif.laf.ExtUIManager;
import com.jgoodies.uif.splash.ImageSplash;
import com.jgoodies.uif.splash.Splash;
import com.jgoodies.uif.util.UIFFocusTraversalPolicy;
import com.jgoodies.uifextras.help.HelpBroker;

/**
 * This class is used during the startup process and extends
 * the more generic superclass behavior of the
 * <code>AbstractUIFApplication</code>.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.15 $
 *
 * @see	com.jgoodies.uif.splash.Splash
 * @see	com.jgoodies.uifextras.help.HelpBroker
 *
 * @since 1.8
 */
public abstract class DefaultUIFApplication extends AbstractUIFApplication {

	/**
	 * Configure logging to write a <tt>console.log</tt> file.
	 */
	@Override
    protected void configureLogging() {
        Logger aLogger = Logger.getLogger("");
        aLogger.setLevel(Level.WARNING);
        try {
            String pattern = getDefaultLogFilePattern();
            ensureParentDirectoryExists(pattern);
            FileHandler handler = new FileHandler(pattern);
            aLogger.addHandler(handler);
        } catch (IOException e) {
            System.err.println("Failed to configure the logging: " + e.getLocalizedMessage());
        }
	}


    /**
     * Configures the logging.<p>
     *
     * <strong>This will become the default logging behavior in the
     * next major UIF version.</strong>. For now it resides as an
     * optional default behavior and must be invoked manually from
     * the {@code #configureLogging} method.
     *
     * This default implementation reads the logging configuration from
     * a file <tt>resources/logging.&lt;LOGGING TYPE&gt;.properties</tt>.
     * The <tt>&lt;LOGGING TYPE&gt;</tt> is read from the system property
     * <i>logging.type</i>, and is <tt>default</tt> by default. Hence,
     * the default logging configuration file is
     * <tt>resources/logging.default.properties</tt>.
     * If for example you set the logging type to <tt>debug</tt>,
     * the logging configuration file will be
     * <tt>resources/logging.debug.properties</tt>.<p>
     *
     * Also adds a FileHandler that logs to
     * <code>&lt;applicationDataDirectory&gt;/console%u.log</code>.<p>
     *
     * Subclasses may override this implementation.
     *
     * @see #getApplicationDataDirectory()
     *
     * @since 1.9.1
     */
    protected void configureLogging2() {
        String logType = System.getProperty("logging.type", "default");
        String configurationFileName = "resources/logging." + logType + ".properties";
        configureLoggingFromProperties(configurationFileName);
        addLogFileHandler(getApplicationDataDirectory(), "console%u.log");
    }


    /**
     * Returns the default log file pattern.
     * This implementation uses the application configuration's app filename
     * as directory name under the user home directory. The log file name is
     * <tt>console.log</tt>.
     *
     * @return the default log file pattern
     *
     * @see #getDefaultLogFilePrefix()
     */
	protected String getDefaultLogFilePattern() {
        return getDefaultLogFilePrefix() +  "/console%u.log";
	}


    /**
     * Computes and returns a pattern for the log file directory.
     * This default implementation uses the absolute path of the
     * application data directory.
     *
     * @return the absolute path of the application data directory
     *
     * @see #getApplicationDataDirectory()
     *
     * @since 1.8
     */
    protected String getDefaultLogFilePrefix() {
        return getApplicationDataDirectory().getAbsolutePath();
    }


	/**
	 * Configures the splash component: reads the splash image, then opens an ImageSplash.
	 */
	@Override
    protected void configureSplash() {
		Image image = getAppResourceMap().getImage(ResourceIDs.SPLASH_IMAGE);
		Splash.setProvider(new ImageSplash(image, true));
	}

    protected void addMessageHandler() {
        Logger.getLogger("").addHandler(new MessageLogHandler(
                getAppResourceMap().getString("application.vendor.mail")));
    }


    /**
     * Configures the application wide help.
     */
    @Override
    protected void configureHelp() {
        HelpBroker.setHelpSetPath(getAppResourceMap().getURL("application.helpSetPath"));
    }

	/**
	 * Brings up the application, it therefore initializes the main frame,
	 * checks the setup process, initializes all actions, then builds
	 * the main frame, and finally opens it.
	 */
	@Override
    protected void launchApplication() {
        // Now add a more sophisticated handler.
        addMessageHandler();

	    Splash.setNote("Creating MainFrame", 30);
		AbstractFrame mainFrame = createMainFrame();
		JOptionPane.setRootFrame(mainFrame);

		Splash.setNote("Initializing Actions\u2026", 40);
		initializeActions();

		checkSetup();

		mainFrame.build();
        Splash.setNote("Opening\u2026", 90);
		mainFrame.open();
	}


	/**
	 * Creates and returns the main frame.
     *
     * @return the created main frame
	 */
    protected abstract AbstractFrame createMainFrame();


	/**
	 * Initializes the application's actions.
	 */
    protected abstract void initializeActions();


	/**
	 * Configures all UI related properties: look&amp;feel,
     * system properties, etc.
     * By default we set the Swing class loader to class
     * <code>LookUtils</code> of the JGoodies L&amp;F library.
     * Also the menu bar is moved to the screen in Aqua look&amp;feel.<p>
     *
     * Subclasses that override this method are encouraged to invoke
     * this super behavior.
	 */
	@Override
    protected void configureUI() {
	    System.setProperty("apple.laf.useScreenMenuBar", "true");

	    ExtUIManager.setup();
        UIManager.put("ClassLoader", LookUtils.class.getClassLoader());

        KeyboardFocusManager.getCurrentKeyboardFocusManager().
            setDefaultFocusTraversalPolicy(UIFFocusTraversalPolicy.DEFAULT);
	}


	/**
	 * Checks whether a setup is necessary. For example, one can check
	 * whether the user has accepted the license agreement.
	 */
	protected void checkSetup() {
		configureSetupManager();
		if (!SetupManager.checkLicense())
			exit();
	}


	/**
	 * Configures the <code>SetupManager</code>. The default does nothing.
	 * Subclasses can, for example modify the welcome panel.
	 */
	protected void configureSetupManager() {
        // Do nothing by default; subclasses may override.
	}


}