/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.convenience;


import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;

import javax.swing.JOptionPane;

/**
 * This class pops up message dialogs to log messages with
 * a level greater or equal to Level.WARNING. In addition,
 * a <code>ConsoleHandler</code> is used to write
 * all messages to the console, too.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.7 $
 */

public final class MessageLogHandler extends Handler {

    private static final String OK_LABEL = "OK";
    private static final String FEEDBACK_LABEL = "Send Feedback\u2026";
    private static final Object[] OPTIONS =
        new Object[] { OK_LABEL, FEEDBACK_LABEL };

    private final String feedbackReceiver;

    public MessageLogHandler(String feedbackReceiver) {
        this.feedbackReceiver = feedbackReceiver;
    }

    @Override
    public void publish(LogRecord logRecord)  {
        int level = logRecord.getLevel().intValue();

        // Don't pop up Windows for Level < WARNING.
        if (level < Level.WARNING.intValue())
            return;

        String msg = logRecord.getMessage();
        if (logRecord.getThrown() == null)
            JOptionPane.showMessageDialog(
                null,
                msg,
                getTitle(logRecord.getLevel()),
                getMessageType(logRecord.getLevel()));
        else {
            showOptionWithFeedbackDialog(logRecord.getLevel(), msg, logRecord.getThrown());
        }
    }

    @Override
    public void close() {
        // Implements Handler; this implementation does nothing
    }

    @Override
    public void flush() {
        // Implements Handler; this implementation does nothing
    }

    private int getMessageType(Level level) {
        if (Level.SEVERE.equals(level))
            return JOptionPane.ERROR_MESSAGE;
        else if (Level.WARNING.equals(level))
            return JOptionPane.WARNING_MESSAGE;
        else
            return JOptionPane.INFORMATION_MESSAGE;
    }

    private String getSubject(Level level) {
        return "Execution " + getTitle(level);
    }

    private String getTitle(Level level) {
        if (Level.SEVERE.equals(level))
            return "Error";
        else if (Level.WARNING.equals(level))
            return "Warning";
        else
            return "Message";
    }

    private void sendFeedback(Level level, String msg, Throwable thrown) {
        StringWriter out = new StringWriter();
        out.write(msg);

        out.write("\n");
        writeSystemProperties(
            out,
            new String[] {
                "os.name",
                "os.version",
                "java.vm.vendor",
                "java.vm.version",
                "application.fullversion" });

        if (thrown != null) {
            out.write("\n\n");
            thrown.printStackTrace(new PrintWriter(out));
        }
        new SendFeedbackDialog(
            null,
            feedbackReceiver,
            getSubject(level),
            out.toString())
            .open();
    }

    private void writeSystemProperties(StringWriter out, String[] keys) {
        for (String key : keys) {
            String value = System.getProperty(key);
            if (value != null) {
                out.write("\n");
                out.write(key);
                out.write("=");
                out.write(value);
            }
        }
    }

    private void showOptionWithFeedbackDialog(
        Level level,
        String msg,
        Throwable thrown) {
        int messageType = getMessageType(level);
        String title = getTitle(level);
        String fullMessage = msg + "\n" + thrown.getLocalizedMessage();

        int choice =
            JOptionPane.showOptionDialog(
                null,
                fullMessage,
                title,
                -1,
                messageType,
                null,
                OPTIONS,
                OK_LABEL);
        if (choice == 1)
            sendFeedback(level, msg, thrown);
    }

}