/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.convenience;

import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;
import javax.swing.border.Border;

import com.jgoodies.forms.builder.ButtonBarBuilder2;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.uif.AbstractDialog;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.util.Resizer;
import com.jgoodies.uifextras.panel.HeaderPanel;
import com.jgoodies.uifextras.util.UIFactory;

/**
 * A dialog that is used to send feedback. Consists of a tabbed pane
 * with two tabs: help and mail contents.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.11 $
 */

public final class SendFeedbackDialog extends AbstractDialog {

    private static final int    PANEL_WIDTH = 280;
    private static final String COPY_TO_CLIPBOARD_LABEL = "Copy to Clipboard";

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(SendFeedbackDialog.class);


    private final String receiver;
    private final String subject;
    private final String mailContents;

    /**
     * Constructs a <code>SendFeedbackDialog</code> using the given
     * owner frame, mail receiver, mail subject, and mail contents.
     */
    public SendFeedbackDialog(
        Frame owner,
        String receiver,
        String subject,
        String mailContents) {
        super(owner);
        this.receiver = receiver;
        this.subject = subject;
        this.mailContents = mailContents;
    }

    // Building *************************************************************

    /**
     * Builds and answer the dialog's header.
     */
    @Override
    protected JComponent buildHeader() {
        return new HeaderPanel(
            "Send Feedback: " + subject,
            "You can send feedback to help improve this tool.\n" +
            "Click on the 'Contents' tab to see the mail contents.",
            RESOURCES.getIcon("feedbackDialog.icon"));
    }

    /**
     * Builds and answers the content.
     */
    @Override
    protected JComponent buildContent() {
        JPanel content = new JPanel(new BorderLayout());
        content.add(buildTabbedPane(), BorderLayout.CENTER);
        content.add(buildButtonBar(),  BorderLayout.SOUTH);
        return content;
    }

    /**
     * Builds and answers the tabbed pane with two tabs.
     */
    protected JComponent buildTabbedPane() {
        JTabbedPane pane = new JTabbedPane();
        pane.add("Help", buildHelpTab());
        pane.add("Mail Contents", buildMailContentsTab());
        return pane;
    }

    /**
     * Builds and answers the help tab.
     */
    private JPanel buildHelpTab() {
        JPanel panel = new JPanel(new BorderLayout());
        JTextArea area =
            UIFactory.createWrappedMultilineLabel(getHelpContents());
        panel.add(area);
        panel.setPreferredSize(Resizer.DEFAULT.fromWidth(PANEL_WIDTH));
        panel.setBorder(Borders.DIALOG_BORDER);
        panel.setOpaque(false);
        return panel;
    }

    /**
     * Builds and answers the mail contents tab.
     */
    private JPanel buildMailContentsTab() {
        JPanel panel = new JPanel(new BorderLayout());
        JTextArea area = new JTextArea(getContentsWithHeader());
        area.setEditable(false);
        panel.add(new JScrollPane(area));
        panel.setPreferredSize(Resizer.DEFAULT.fromWidth(PANEL_WIDTH));
        panel.setBorder(Borders.DIALOG_BORDER);
        panel.setOpaque(false);
        return panel;
    }

    /**
     * Builds and answers the button bar with "Copy to Clipboard" and "Close"  buttons.
     */
    private JComponent buildButtonBar() {
        ButtonBarBuilder2 builder = new ButtonBarBuilder2();
        builder.setDefaultButtonBarGapBorder();
        builder.addGlue();
        builder.addButton(createCopyToClipboardButton());
        builder.addUnrelatedGap();
        builder.addButton(createCloseButton(true));
        return builder.getPanel();
    }

    /**
     * Creates and answers the copy-to-clipboard button.
     */
    private JButton createCopyToClipboardButton() {
        JButton button = new JButton(COPY_TO_CLIPBOARD_LABEL);
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                StringSelection selection =
                    new StringSelection(getContentsWithHeader());
                getToolkit().getSystemClipboard().setContents(
                    selection,
                    selection);
                //copyButton.setEnabled(false);
            }
        });
        return button;
    }

    /**
     * Returns the dialog's border, here, the default card dialog border.
     */
    @Override
    protected Border getDialogBorder() {
        return CARD_DIALOG_BORDER;
    }

    /**
     * Use the default resizer to get an aesthetic aspect ratio.
     */
    @Override
    protected void resizeHook(JComponent component) {
        Resizer.DEFAULT.resizeDialogContent(component);
    }

    // Helper Code **********************************************************

    /**
     * Returns the help contents.
     */
    private String getHelpContents() {
        return RESOURCES.getString(
                "feedbackDialog.message",
                receiver, subject);
    }

    /**
     * Returns the mail contents as concatenated string.
     */
    private String getContentsWithHeader() {
        return "To: "
            + receiver
            + "\n"
            + "Subject: "
            + subject
            + "\n\n"
            + mailContents;
    }

}