/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.convenience;

import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.net.URL;
import java.util.logging.Logger;

import javax.swing.*;

import com.jgoodies.binding.adapter.BasicComponentFactory;
import com.jgoodies.binding.adapter.PreferencesAdapter;
import com.jgoodies.binding.beans.PropertyAdapter;
import com.jgoodies.forms.builder.ButtonBarBuilder2;
import com.jgoodies.forms.layout.Sizes;
import com.jgoodies.uif.AbstractDialog;
import com.jgoodies.uif.util.Resizer;
import com.jgoodies.uifextras.util.HTMLUtils;
import com.jgoodies.uifextras.util.UIFactory;

/**
 * Provides a dialog and managing behavior for a tip of the day functionality.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.9 $
 *
 * @see PreferencesAdapter
 */
public final class TipOfTheDayDialog extends AbstractDialog {

    private static final Logger LOGGER = Logger.getLogger("TipOfTheDayDialog");

    private final TipOfTheDayModel model;
    private JEditorPane htmlPane;


    // Instance  Creation ****************************************************

    /**
     * Constructs the dialog for the given owner and tip model.
     *
     * @param owner   the optional owner of the tip of the day dialog
     * @param model   the model that provides the tips
     */
    public TipOfTheDayDialog(Frame owner, TipOfTheDayModel model) {
        super(owner, "Tip of the Day");
        this.model = model;
    }


    // Building *************************************************************

    /**
     * Builds and returns the dialog's content.
     */
    @Override
    protected JComponent buildContent() {
        JPanel content = new JPanel(new BorderLayout());
        content.add(buildMainPanel(), BorderLayout.CENTER);
        content.add(buildButtonBar(), BorderLayout.SOUTH);
        return content;
    }

    /**
     * Builds and returns the main panel.
     */
    private JPanel buildMainPanel() {
        JPanel panel = new JPanel(new BorderLayout());
        htmlPane = UIFactory.createHTMLPane(false, false);
        htmlPane.setMargin(new Insets(0, 10, 0, 5));
        showTip(model.nextTip());
        panel.add(new JScrollPane(htmlPane));
        panel.setPreferredSize(Resizer.DEFAULT.fromWidth(320));
        return panel;
    }

    /**
     * Creates and returns the button bar, which consists of: CheckBox, glue,
     * Back, Next, and Close buttons.
     */
    private JComponent buildButtonBar() {
        JCheckBox showTipsCheckBox = BasicComponentFactory.createCheckBox(
                new PropertyAdapter<TipOfTheDayModel>(model, TipOfTheDayModel.SHOWING_TIPS_PROPERTY),
                "Show tips on startup");
        showTipsCheckBox.setMnemonic('S');

        ButtonBarBuilder2 builder = new ButtonBarBuilder2();
        builder.setDefaultButtonBarGapBorder();
        builder.addFixed(showTipsCheckBox);
        builder.addUnrelatedGap();
        builder.addGlue();
        builder.addStrut(Sizes.dluX(50));  // or a back button
        builder.addRelatedGap();
        builder.addButton(new NextAction());
        builder.addRelatedGap();
        builder.addButton(createCloseButton(true));
        return builder.getPanel();
    }

    /**
     * Use the default resizer to get an aesthetic aspect ratio.
     */
    @Override
    protected void resizeHook(JComponent component) {
        Resizer.DEFAULT.resizeDialogContent(component);
    }


    // Behavior ***************************************************************

    /**
     * Performs the close operation.
     */
    @Override
    public void close() {
        doApply();
        super.close();
    }


    public void showNextTip() {
        showTip(model.nextTip());
    }


    // Misc *******************************************************************

    /**
     * Shows the tip for the specified tip path.
     */
    private void showTip(URL tipURL) {
        try {
            htmlPane.setPage(tipURL);
            HTMLUtils.addDefaultStyleSheet(htmlPane);
        } catch (IOException e) {
            LOGGER.warning("Can't show tip URL: " + tipURL);
        }
    }

    private final class NextAction extends AbstractAction {

        NextAction() {
            super("Next");
            putValue(Action.MNEMONIC_KEY, Integer.valueOf('N'));
        }

        public void actionPerformed(ActionEvent e) {
            showNextTip();
        }
    }

}
