/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.fileaccess;

import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;

import javax.swing.*;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreeSelectionModel;

import com.jgoodies.uif.AbstractDialog;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.component.UIFTree;
import com.jgoodies.uif.util.Resizer;
import com.jgoodies.uifextras.convenience.ResourceIDs;
import com.jgoodies.uifextras.panel.HeaderPanel;

/**
 * A dialog for choosing a directory. All access to the file system
 * is delegated to an <code>ExtFileSystemView</code>.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.11 $
 *
 * @see ExtFileSystemView
 */
public final class DirectoryChooser extends AbstractDialog {

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(DirectoryChooser.class);


	private boolean canSelectLeafDirectory;
	private File	selection = null;
	private JTree	directoryTree;


	// Instance Creation ******************************************************

	/**
	 * Constructs a directory chooser for the given owner frame.
	 */
	private DirectoryChooser(Frame owner) {
		super(owner);
		canSelectLeafDirectory = false;
	}


	/**
	 * Lets the user choose a directory, which is then returned.
	 * Opens a directory chooser dialog using the owner frame.
     * Offers only directories that are not a leaf.
     *
     * @param parent    the optional owner of this dialog
     * @return the chosen directory
     *
     * @see #chooseDirectory(Component, boolean)
	 */
	public static File chooseDirectory(Component parent) {
		return chooseDirectory(parent, false);
	}


	/**
	 * Lets the user choose and directory, which is then returned.
	 * Opens a directory chooser dialog using the owner frame
	 * and the given select-leaf-directory property.
     *
     * @param parent    the optional owner of this dialog
     * @param canSelectLeafDirectory   true to be able to select directories
     *    that are a file system leaf, false to disallow to select such dirs
     * @return the chosen directory
     *
     * @see #chooseDirectory(Component)
	 */
	public static File chooseDirectory(Component parent, boolean canSelectLeafDirectory) {
        Frame frame = parent instanceof Frame
                ? (Frame) parent
                : (Frame) SwingUtilities
                        .getAncestorOfClass(Frame.class, parent);

		DirectoryChooser chooser = new DirectoryChooser(frame);
		chooser.setCanSelectLeafDirectory(canSelectLeafDirectory);
		chooser.open();
		return chooser.getSelectedDirectory();
	}


	// Building *************************************************************************

	/**
	 * Builds and returns the dialog's header panel.
	 */
	@Override
    protected JComponent buildHeader() {
	    return new HeaderPanel(
	   		"Choose Directory",
	   		"Select a directory from the list below, then click \"OK\"."
			+ "\nYou can expand directories by clicking on the anchors.",
			RESOURCES.getIcon(ResourceIDs.CHOOSE_DIRECTORY_ICON)
			);
	}


	/**
	 * Builds and returns the dialog's content panel.
	 */
	@Override
    protected JComponent buildContent() {
		JPanel content = new JPanel(new BorderLayout());
		content.add(createDirectoryTreePane(), BorderLayout.CENTER);
		content.add(buildButtonBarWithOKCancel(), BorderLayout.SOUTH);
		getOKAction().setEnabled(false);

		content.setPreferredSize(new Dimension(200, 310));
		return content;
	}


	/**
	 * Uses the default <code>Resizer</code> to give the dialog an aesthetic
	 * aspect ratio.
	 */
	@Override
    protected void resizeHook(JComponent component) {
		Resizer.DEFAULT.resizeDialogContent(component);
	}


	/**
	 * Creates, configures, and answers the directory tree pane.
	 */
	private JComponent createDirectoryTreePane() {
		TreeNode root = DirectoryTreeNode.createRoot();
		directoryTree = new UIFTree(root);
		directoryTree.setRootVisible(false);
		directoryTree.setShowsRootHandles(true);
		directoryTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		if (root.getChildCount() == 1)
			directoryTree.expandRow(0);

		// Set a JFileChooser-like Renderer
		directoryTree.setCellRenderer(new DirectoryTreeRenderer());

		// Listen to selection changes.
		directoryTree.addTreeSelectionListener(new TreeSelectionListener() {
			public void valueChanged(TreeSelectionEvent e) {
				updateOKActionEnablement((DirectoryTreeNode) e.getPath().getLastPathComponent());
			}
		});

		// Listen to double click; doOk iff okButton is enabled.
		directoryTree.addMouseListener(new MouseAdapter() {
			@Override
            public void mousePressed(MouseEvent e) {
				if ((e.getClickCount() == 2) && getOKAction().isEnabled())
					doAccept();
			}
		});

		// Load DirectoryTreeNode's children before the tree will expand.
		directoryTree.addTreeWillExpandListener(new TreeWillExpandListener() {
			public void treeWillExpand(TreeExpansionEvent e) {
				DirectoryTreeNode node = (DirectoryTreeNode) e.getPath().getLastPathComponent();
				try {
					setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
					node.loadChildren();
				} finally {
					setCursor(Cursor.getDefaultCursor());
				}
			}
			public void treeWillCollapse(TreeExpansionEvent e) {
                // Implements TreeWillExpandListener; ignore
            }
		});

		//
		return new JScrollPane(directoryTree);
	}


	// Actions **************************************************************************

	/**
	 * Closes the dialog.
	 */
	@Override
    public void close() {
		setVisible(false);
	}


	/**
	 * Cancels the choice and closes the dialog.
	 */
	@Override
    public void doCancel() {
		selection = null;
		super.doCancel();
	}


	// Misc *****************************************************************************

	/**
	 * Returns the selected directory.
	 */
	private File getSelectedDirectory() {
		return selection;
	}


	/**
	 * Enables or disables the selection of leaf directories.
     *
     * @param canSelectLeafDirectory   true to be able to select directories
     *    that are a file system leaf, false to disallow to select such dirs
	 */
	public void setCanSelectLeafDirectory(boolean canSelectLeafDirectory) {
		this.canSelectLeafDirectory = canSelectLeafDirectory;
	}


	/**
	 * Updates the enablement of the OK button.
	 */
	private void updateOKActionEnablement(DirectoryTreeNode selectedNode) {
		selection = selectedNode.getDirectory();
		boolean hasChildren = canSelectLeafDirectory || (selectedNode.getChildCount() > 0);
		getOKAction().setEnabled(hasChildren && selectedNode.isFileSystem());
	}


	// Helper Class *********************************************************************

	// Helper class that renders directory tree nodes.
	private static final class DirectoryTreeRenderer extends DefaultTreeCellRenderer {

		@Override
        public Component getTreeCellRendererComponent(
					JTree tree, Object value,
					boolean isSelected, boolean expanded, boolean leaf,
					int row, boolean focused) {
			super.getTreeCellRendererComponent(tree, value, isSelected, expanded, false, row, focused);
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
			Object userObject = node.getUserObject();
			if ((null == userObject) || !(userObject instanceof File))
				return this;

			File directory = (File) userObject;
			ExtFileSystemView v = ExtFileSystemView.getInstance();
			setText(v.getDirectoryName(directory));
			setIcon(v.getIcon(directory, isSelected));
			return this;
		}
	}


}
