/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.fileaccess;

import java.io.File;
import java.util.*;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;

/**
 * A TreeNode implementation that describes nodes in a tree of
 * directories. It is used in the DirectoryChooser tree.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.7 $
 *
 * @see     TreeNode
 * @see     DirectoryChooser
 */

final class DirectoryTreeNode extends DefaultMutableTreeNode {

    /** Have the children been loaded?         */
    private boolean childrenAreLoaded;

    /** Shall I defer the loading of children? */
    private final boolean deferLoadingChildren;


    // Instance Creation ******************************************************

    /**
     * Constructs a DirectoryTreeNode for the given directory.
     */
    private DirectoryTreeNode(File directory) {
        this(directory, true);
    }


    /**
     * Constructs a DirectoryTreeNode for the given directory
     * either by eagerly or lazily loading the children.
     */
    private DirectoryTreeNode(File directory, boolean deferLoadingChildren) {
        super(directory);
        childrenAreLoaded = false;
        this.deferLoadingChildren = deferLoadingChildren;
    }


    /**
     * Constructs a DirectoryTreeNode for the specified name and children.
     */
    private DirectoryTreeNode(String name, File[] children) {
        super(name);
        addChildren(children);
        deferLoadingChildren = true;
    }


    /**
     * Creates and returns a root node.
     */
    public static DirectoryTreeNode createRoot() {
        return new DirectoryTreeNode("Filesystem Roots",
                ExtFileSystemView.getInstance().getRoots());
    }


    // ************************************************************************

    /**
     * Adds the given children to this node.
     */
    protected void addChildren(File[] theChildren) {
        childrenAreLoaded = true;

        for (File element : theChildren) {
            add(new DirectoryTreeNode(element, true));
        }
    }


    // TreeNode Implementation *************************************************

    /**
     * Returns an Enumeration of this node's children.
     */
    @Override
    public Enumeration children() {
        ensureChildrenAreLoaded();
        return super.children();
    }


    /**
     * Returns the child node at the specified index.
     */
    @Override
    public TreeNode getChildAt(int index) {
        ensureChildrenAreLoaded();
        return super.getChildAt(index);
    }


    /**
     * Returns the number of chilren.
     */
    @Override
    public int getChildCount() {
        ensureChildrenAreLoaded();
        return super.getChildCount();
    }


    /**
     * Checks and answers if the node is a leaf.
     */
    @Override
    public boolean isLeaf() {
        return !childrenAreLoaded && shouldDeferLoadingChildren()
                ? false
                : super.isLeaf();
    }


    /**
     * Ensures that the node's children are loaded.
     */
    private void ensureChildrenAreLoaded() {
        if (!childrenAreLoaded) loadChildren();
    }


    /**
     * Returns the directory for this node.
     */
    public File getDirectory() {
        return (File) getUserObject();
    }


    /**
     * Checks and answers if the node belongs to a directory in the file system;
     * note that some shell folders may not correspond to a file.
     */
    public boolean isFileSystem() {
        return ExtFileSystemView.getInstance().isFileSystem(getDirectory());
    }


    /**
     * Loads the children if this node - if they have not been loaded before.
     */
    public void loadChildren() {
        // Do nothing if children are already loaded.
        if (childrenAreLoaded) return;

        File directory = getDirectory();
        childrenAreLoaded = true;

        File[] fileList = ExtFileSystemView.getInstance().getFiles(directory,
                false);
        if (null == fileList) return;

        List childDirectories = new LinkedList();
        // Iterate over the directory's content. Collect child directories.
        for (File file : fileList) {
            if (file.isDirectory()) childDirectories.add(file);
        }

        // Sort the child directories by name;
        Collections.sort(childDirectories);

        // Create and add child nodes.
        Iterator i = childDirectories.iterator();
        while (i.hasNext()) {
            File aDirectory = (File) i.next();
            add(new DirectoryTreeNode(aDirectory));
        }
    }


    /**
     * Returns if children shall be loaded lazily.
     */
    protected boolean shouldDeferLoadingChildren() {
        return deferLoadingChildren;
    }
}