/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.fileaccess;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;

import javax.swing.Icon;
import javax.swing.JFileChooser;
import javax.swing.UIManager;
import javax.swing.filechooser.FileSystemView;

import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.util.CompoundIcon;
import com.jgoodies.uif.util.SystemUtils;

/**
 * An extended version of the Swing {@link FileSystemView} class.
 * It is implemented as a singleton, that reinitializes itself each time
 * the look&amp;feel changes.<p>
 *
 * TODO: Remove the LookChangeListener from unused instances.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.12 $
 * @see FileSystemView
 */

public final class ExtFileSystemView {

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(ExtFileSystemView.class);


    private static PropertyChangeListener lookChangeHandler;
    private static ExtFileSystemView instance;

    private Icon            absentFolderIcon;
    private Icon            absentLeafIcon;
    private Icon            floppyDriveIcon;
    private Icon            hardDriveIcon;
    private Icon            homeFolderIcon;
    private Icon            leafIcon;
    private final JFileChooser    fileChooser;
    private final FileSystemView  fileSystemView;


    // Instance Creation / Access *********************************************

    /**
     * Constructs an extended file system view.
     */
    private ExtFileSystemView() {
        fileChooser = new JFileChooser();
        fileSystemView = fileChooser.getFileSystemView();
        installCachedIcons();
        ensureLookChangeHandlerRegistered();
    }

    /**
     * Lazily creates and returns the sole instance.
     *
     * @return the sole instance of the class
     */
    public static ExtFileSystemView getInstance() {
        if (instance == null) {
            instance = new ExtFileSystemView();
        }
        return instance;
    }

    /**
     * Ensures that a listener is registered with the UIManager
     * that handles Look&emp;Feel changes. The handler listens to
     * look changes and then resets the instance, so that a new request
     * will construct a new instance and automatically refresh all
     * look-related properties.
     */
    private static void ensureLookChangeHandlerRegistered() {
        if (lookChangeHandler == null) {
            lookChangeHandler = new LookAndFeelChangeHandler();
            UIManager.addPropertyChangeListener(lookChangeHandler);
        }
    }

    // ***********************************************************************

    /**
     * Returns the current directory. Useful for dialogs that operate on
     * directories to show the directory the user has worked with recently
     *
     * @return the current directory
     */
    public File getCurrentDirectory() {
        return fileChooser.getCurrentDirectory();
    }

    /**
     * Returns the system's display name for the specified directory.
     *
     * @param directory    the directory File to be checked
     * @return the directory's system display name
     */
    public String getDirectoryName(File directory) {
        return fileSystemView.getSystemDisplayName(directory);
    }

    /**
     * Returns the <code>File</code>s that are shown as childs of
     * the specified <code>File</code>. A child is either contained
     * in the file system, or a <code>ShellFolder</code> child.
     *
     * @param file           the parent file
     * @param useFileHiding  true honors the hidden file state
     * @return the files that are shown as childs of the given parent file
     */
    public File[] getFiles(File file, boolean useFileHiding) {
        return fileSystemView.getFiles(file, useFileHiding);
    }

    /**
     * Looks up and returns an icon for the given file and selection state.
     * The selection state can be used in renderers to show different icons
     * for the selected and deselected states.
     *
     * @param file   the associated file
     * @param isSelected   true if we're looking for an icon in selected state
     * @return an icon that represents the file (type)
     *
     * @see #getLeafIcon(File)
     */
    public Icon getIcon(File file, boolean isSelected) {
        if (!file.exists())
            return absentFolderIcon;

        try {
            return fileChooser.getIcon(file);
        } catch (Exception e) {
            /*
             System.out.println("Can't get system icon for: " + file);
             System.out.println("isTraversable=" + isTraversable(file));
             System.out.println("isFileSystem="  + isFileSystem(file));
             System.out.println("isRoot="  + fileSystemView.isRoot(file));
             */
            return UIManager.getIcon(isSelected
                    ? "Tree.openIcon"
                    : "Tree.closedIcon");
        }
    }


    /**
     * Fetches and returns a leaf icon for the given file.
     * If we know that the file is a leaf, we can guess
     * a better icon, in case the file is absent.
     *
     * @param file   the leaf file to lookup the icon for
     * @return an icon that represents the given file
     *
     * @see #getIcon(File, boolean)
     */
    public Icon getLeafIcon(File file) {
        // Workaround for problem with "pagefile.sys" and "hyberfil.sys"
        File parent = file.getParentFile();
        if (parent != null) {
            boolean isRootChild = null == parent.getParentFile();
            String name = file.getName();
            if (isRootChild
                && (   ("pagefile.sys".equals(name))
                    || ("hiberfil.sys".equals(name))))
                return leafIcon;
        }

        return !file.exists() ? absentLeafIcon : getIcon(file, false);
    }

    /**
     * Looks up and returns an array of filesystem roots.
     *
     * @return an array of the filesystem's roots
     */
    public File[] getRoots() {
        File[] roots = fileSystemView.getRoots();
        if (SystemUtils.IS_OS_WINDOWS)
            return roots;

        File[] nodes = new File[roots.length + 1];
        System.arraycopy(roots, 0, nodes, 1, roots.length);
        nodes[0] = getCurrentDirectory();
        return nodes;
    }

    /**
     * Installs a set of icons that we cache to improve the performance.
     */
    private void installCachedIcons() {
        leafIcon = UIManager.getIcon("FileView.fileIcon");
        if (leafIcon == null) {
            leafIcon = UIManager.getIcon("Tree.leafIcon");
        }
        Icon folderIcon = UIManager.getIcon("FileView.directoryIcon");
        if (folderIcon == null) {
            folderIcon = UIManager.getIcon("Tree.closedIcon");
        }
        Icon absentIcon = RESOURCES.getIcon("ExtFileSystemView.absentIcon");
        absentFolderIcon = new CompoundIcon(folderIcon, absentIcon);
        absentLeafIcon = new CompoundIcon(leafIcon, absentIcon);

        hardDriveIcon = UIManager.getIcon("FileView.hardDriveIcon");
        if (null == hardDriveIcon)
            hardDriveIcon = UIManager.getIcon("Tree.closedIcon");

        floppyDriveIcon = UIManager.getIcon("FileView.floppyDriveIcon");
        if (null == floppyDriveIcon)
            floppyDriveIcon = hardDriveIcon;

        homeFolderIcon = UIManager.getIcon("FileChooser.homeFolderIcon");
        if (null == homeFolderIcon)
            homeFolderIcon = UIManager.getIcon("Tree.closedIcon");
    }

    /**
     * Checks and returns whether the given file is backed up by
     * the file system. Otherwise it is a virtual file or directory,
     * for example, a shell folder like <i>Shared Documents</i> on Windows.
     *
     * @param file    the file to check
     * @return true if the file is backed by the file system, false otherwise
     */
    public boolean isFileSystem(File file) {
        return fileSystemView.isFileSystem(file);
    }

    private static final class LookAndFeelChangeHandler implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent e) {
            instance = null;
        }
    }

}