/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.help;

import java.net.URL;

/**
 * This singleton class provides access to a HelpViewer,
 * which in turn uses a HelpSet.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.8 $
 */
public final class HelpBroker {

    private static ClassLoader classLoader;
    private static URL helpSetPath;
    private static HelpBroker instance;

    private final HelpModel  model;
    private final HelpViewer viewer;


    // Instance Creation ******************************************************

    /**
     * Creates a HelpBroker and a HelpViewer on the given HelpSet.
     *
     * @param helpSet   the HelpSet used to create the HelpViewer
     */
    HelpBroker(HelpSet helpSet) {
        this.model  = new HelpModel(helpSet);
        this.viewer = new HelpViewer(model);
        viewer.build();
    }


    // Public API *************************************************************

    /**
     * Sets the path to the HelpSet, which will be used during
     * the lazy initialization process, see <code>#createInstance(URL)</code>.
     *
     * @param path    the path to the HelpSet
     *
     * @throws IllegalArgumentException if the path is {@code null}
     */
    public static void setHelpSetPath(URL path) {
        if (path == null)
            throw new IllegalArgumentException("The HelpSet path must not be null.");
        helpSetPath = path;
    }


    public static void setClassLoader(ClassLoader loader) {
        classLoader = loader;
    }


    /**
     * Shows the help page at the given URL in the HelpViewer.
     *
     * @param url   the URL of the help page to open
     */
    public static void openURL(URL url) {
        viewer().show(url);
    }


    /**
     * Shows the default help page in the HelpViewer.
     */
    public static void openDefault() {
        viewer().setDisplayed(true);
    }

    /**
     * Returns whether this broker has already created an instance.
     *
     * @return true if an instance has been created, false otherwise
     */
    public static boolean hasInstance() {
        return instance != null;
    }

    static URL getSelectedURL() {
        return model().getSelection();
    }

    static void closeViewer() {
        viewer().setVisible(false);
    }

    /**
     * Returns the single HelpBroker instance; lazily creates an instance.
     *
     * @return the single HelpBroker instance
     */
    public static HelpBroker getInstance() {
        if (!hasInstance()) {
            instance = createInstance(helpSetPath);
        }
        return instance;
    }


    // Helper Code ************************************************************

    /**
     * Finds HelpSet's URL, creates a HelpSet, and finally creates
     * a HelpBroker instance for this HelpSet.
     */
    private static HelpBroker createInstance(URL aHelpSetPath) {
        ClassLoader loader = classLoader != null
                ? classLoader
                : HelpBroker.class.getClassLoader();
        HelpSet hs = new HelpSet(loader, aHelpSetPath);
        HelpBroker helpBroker = hs.createHelpBroker();
        return helpBroker;
    }

    /**
     *  Returns the HelpViewer.
     */
    private static HelpViewer viewer() {
        return getInstance().viewer;
    }

    private static HelpModel model() {
        return getInstance().model;
    }


}