/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.help;

import java.net.URL;

import javax.swing.Icon;

import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;

/**
 * An immutable class that describes help chapters and
 * help topics in the JGoodies tiny help.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.9 $
 */

public final class HelpItem {

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(HelpItem.class);


    private static final int  CHAPTER_ITEM     = 0;
    private static final int  TOPIC_ITEM       = 1;

    private static final Icon CLOSED_BOOK_ICON =
        RESOURCES.getIcon("com.jgoodies.help.closedBook.icon");
    private static final Icon OPEN_BOOK_ICON   =
        RESOURCES.getIcon("com.jgoodies.help.openBook.icon");
    private static final Icon TOPIC_ICON       =
        RESOURCES.getIcon("com.jgoodies.help.topic.icon");

    private final String name;
    private final int    type;
    private final URL    url;


    // Instance Creation ******************************************************

    /**
     * Constructs a help item for the given name, type, and URL.
     *
     * @throws NullPointerException if the name is {@code null}
     */
    private HelpItem(String name, int type, URL url) {
        if (name == null)
            throw new NullPointerException("The name must not be null.");
        this.name = name;
        this.type = type;
        this.url = url;
    }


    /**
     * Creates and returns a help chapter using the given name.
     *
     * @param name   the chapter's name
     * @return a HelpItem that represents a chapter
     *
     * @throws NullPointerException if the name is {@code null}
     */
    static HelpItem createChapter(String name) {
        return new HelpItem(name, CHAPTER_ITEM, null);
    }


    /**
     * Creates and returns a help topic for the given name and URL.
     *
     * @param name   the topic's name
     * @return a HelpItem that represents a topic
     *
     * @throws NullPointerException if the name is {@code null}
     */
    static HelpItem createTopic(String name, URL url) {
        return new HelpItem(name, TOPIC_ITEM, url);
    }


    // ************************************************************************

    /**
     * Returns this item's URL.
     *
     * @return this item's URL
     */
    URL getURL() {
        return url;
    }


    /**
     * Answers whether this item's URL matches the given URL.
     *
     * @throws NullPointerException if the given URL is {@code null}
     */
    boolean matches(URL aUrl) {
        return aUrl.equals(getURL());
    }


    /**
     * Answers if this item represents a chapter.
     */
    boolean isChapter() {
        return type == CHAPTER_ITEM;
    }


    /**
     * Returns the icon for this item using the specified selection state.
     */
    Icon getIcon(boolean sel) {
        return isChapter()
            ? (sel ? OPEN_BOOK_ICON : CLOSED_BOOK_ICON)
            : TOPIC_ICON;
    }


    @Override
    public String toString() {
        return name;
    }
}