/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.help;

import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.net.URL;
import java.util.Enumeration;

import javax.swing.ActionMap;
import javax.swing.JPopupMenu;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.*;

import com.jgoodies.binding.beans.Model;
import com.jgoodies.uif.application.Action;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.builder.PopupMenuBuilder;
import com.jgoodies.uif.component.UIFMenuItem;
import com.jgoodies.uifextras.printing.PrintManager;
import com.jgoodies.uifextras.printing.PrintableDocument;
import com.jgoodies.uifextras.util.PopupAdapter;

/**
 * Provides the models used in a HelpViewer, its tree and HTML view.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.9 $
 */

public final class HelpModel extends Model {

    // Names of the Bound Bean Properties *************************************

    static final String PROPERTYNAME_SELECTION = "selection";


    // ************************************************************************

    private final HelpSet helpSet;
    private final HelpHistory history;

    /**
     * Holds the model for the navigation tree.
     *
     * @see #getNavigationTreeModel()
     * @see #getNavigationTreeSelectionModel()
     */
    private final TreeModel navigationTreeModel;


    /**
     * Refers to the single selection model for the navigation tree.
     * The selection is reset if the navigation tree model changes.
     *
     * @see #getNavigationTreeSelectionModel()
     * @see #getNavigationTreeModel()
     */
    private final TreeSelectionModel navigationTreeSelectionModel;


    /**
     * Holds the selected help URL.
     *
     * @see #getSelection()
     * @see #setSelection(URL)
     */
    private URL selection;

    private MouseListener popupAdapter;


    // Instance Creation ******************************************************

    HelpModel(HelpSet helpSet) {
        this.helpSet = helpSet;

        navigationTreeModel = new DefaultTreeModel(helpSet.getRoot());
        navigationTreeSelectionModel = new DefaultTreeSelectionModel();
        navigationTreeSelectionModel
                .setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

        history = new HelpHistory(helpSet.getHome());
        initEventHandling();
    }


    // Initialization *********************************************************

    /**
     * Registers listeners for the navigation selection, and history.
     */
    private void initEventHandling() {
        history.addPropertyChangeListener(
                HelpHistory.PROPERTYNAME_SELECTION,
                new HistoryChangeHandler());
        navigationTreeSelectionModel.addTreeSelectionListener(
                new NavigationTreeSelectionChangeHandler());
    }


    // Managing the Navigation Tree Model *************************************

    /**
     * Returns the tree model for the navigation tree.
     *
     * @return the tree model for the navigation tree.
     */
    TreeModel getNavigationTreeModel() {
        return navigationTreeModel;
    }


    /**
     * Returns the fixed selection model for the navigation tree.
     *
     * @return the fixed selection model for the navigation tree.
     */
    TreeSelectionModel getNavigationTreeSelectionModel() {
        return navigationTreeSelectionModel;
    }


    private void setTreeSelection(URL newURL) {
        DefaultMutableTreeNode root = (DefaultMutableTreeNode) getRoot();
        DefaultMutableTreeNode matchingNode = findNode(newURL, root);
        if (null == matchingNode)
            return;
        TreePath pathToRoot = new TreePath(matchingNode.getPath());
        navigationTreeSelectionModel.setSelectionPath(pathToRoot);
    }


    private DefaultMutableTreeNode findNode(URL newURL, DefaultMutableTreeNode node) {
        // Check whether this node is the one we're looking for.
        HelpItem item = (HelpItem) node.getUserObject();
        if ((item != null) && item.matches(newURL))
            return node;

        // Otherwise check the node's children.
        Enumeration enumeration = node.children();
        while (enumeration.hasMoreElements()) {
            DefaultMutableTreeNode child = (DefaultMutableTreeNode) enumeration.nextElement();
            DefaultMutableTreeNode matchingNode = findNode(newURL, child);
            if (matchingNode != null)
                return matchingNode;
        }

        // We haven't found a matching node.
        return null;
    }


    // Accessors **************************************************************

    /**
     * Returns the selected URL.
     *
     * @return the selected URL
     */
    public URL getSelection() {
        return selection;
    }


    /**
     * Sets the given URL as new selection. Adds the new selection
     * to the history and notifies all observers about the change.
     *
     * @param newURL   the URL to be set as new selection
     */
    public void setSelection(URL newURL) {
        URL oldURL = getSelection();
        selection = newURL;
        getHistory().setSelection(newURL);
        setTreeSelection(newURL);
        firePropertyChange(PROPERTYNAME_SELECTION, oldURL, newURL);
    }


    URL getBaseURL() {
        return helpSet.getBaseURL();
    }

    HelpHistory getHistory() {
        return history;
    }


    URL getHome() {
        return history.getHome();
    }


    TreeNode getRoot() {
        return helpSet.getRoot();
    }


    String getTitle() {
        return helpSet.getTitle();
    }


    // Actions ****************************************************************

    @Action(name="com.jgoodies.help.close")
    public void closeViewer() {
        HelpBroker.closeViewer();
    }


    @Action(name="com.jgoodies.help.pageSetup")
    public void openPageSetup(ActionEvent e) {
        PrintManager.openPageSetupDialog();
    }


    @Action(name="com.jgoodies.help.print")
    public void printHelp(ActionEvent e) {
        URL url = HelpBroker.getSelectedURL();
        PrintableDocument.createFrom(url).printWithDialog();
    }


    // MouseListener **********************************************************

    MouseListener getPopupListener() {
        if (popupAdapter == null) {
            popupAdapter = new PopupAdapter(true) {
                @Override
                protected JPopupMenu buildPopupMenu(MouseEvent e) {
                    return HelpModel.this.buildPopupMenu();
                }
            };
        }
        return popupAdapter;
    }

    private JPopupMenu buildPopupMenu() {
        ActionMap map = Application.getActionMap(this);
        PopupMenuBuilder builder = new PopupMenuBuilder(getTitle() + " Menu");
        builder.add(new UIFMenuItem(getHistory().getGoBackAction()));
        builder.add(new UIFMenuItem(getHistory().getGoNextAction()));
        builder.add(new UIFMenuItem(getHistory().getGoHomeAction()));
        builder.addSeparator();
        builder.add(new UIFMenuItem(map.get("printHelp")));

        return builder.getPopupMenu();
    }


    // Event Handling *********************************************************

    /**
     * Listens to history changes and sets the selection.
     */
    private final class HistoryChangeHandler implements PropertyChangeListener {

        public void propertyChange(PropertyChangeEvent event) {
            setSelection(history.getSelection());
        }

    }


    /**
     * Listens to changes in the tree node selection and updates the selection.
     */
    private final class NavigationTreeSelectionChangeHandler implements TreeSelectionListener {

        /**
         * The selected node in the navigation tree has changed.
         * Updates this model's selected URL.<p>
         *
         * Does nothing if the new tree selection is {@code null}.
         *
         * @param evt    the event that describes the change
         */
        public void valueChanged(TreeSelectionEvent evt) {
            TreePath selPath = navigationTreeSelectionModel.getSelectionPath();
            if (selPath == null)
                return;
            Object newSelection = selPath.getLastPathComponent();
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) newSelection;
            HelpItem item = (HelpItem) node.getUserObject();
            URL newURL = item.getURL();
            if (newURL != null) {
                setSelection(newURL);
            }
        }

    }


}