/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.help;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Implements a subset of the Java Help API <code>HelpSet</code> class.
 * It provides an API that is very similiar to Java Help.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.8 $
 */
public final class HelpSet {

    private static Logger logger = Logger.getLogger("HelpSet");

    private final URL baseURL;
    private final Map<String, URL> idsToURL;
    private String title;
    private URL home;
    private DefaultMutableTreeNode root;


    // Instance Creation ******************************************************

    /**
	 * Creates a HelpSet using the given ClassLoader and base URL.
     *
     * @param loader    the class loader used to lookup resources
     * @param baseURL   the baseURL used to append path suffixes to
	 */
    public HelpSet(ClassLoader loader, URL baseURL) {
        this.baseURL = baseURL;
        this.idsToURL = new HashMap<String, URL>();
        readHelpSet(baseURL);
    }


    // API ********************************************************************

    /**
	 * Creates and returns a HelpBroker.
     *
     * @return the created help broker
	 */
    public HelpBroker createHelpBroker() {
        return new HelpBroker(this);
    }

    URL getBaseURL() {
        return baseURL;
    }

    URL getHome() {
        return home;
    }

    TreeNode getRoot() {
        return root;
    }

    String getTitle() {
        return title;
    }

    /**
	 * Looks up and returns the URL that is associated with
	 * the specified help id.
	 *
	 * @param helpID   the id of the help item to look for
	 * @return the URL associated with the given help id
	 */
    URL idToURL(String helpID) {
        URL url = idsToURL.get(helpID);
        if (null == url)
            logger.warning("HelpID " + helpID + " has no target URL.");
        return url;
    }


    // Helper Code ************************************************************

    private DefaultMutableTreeNode createChapterNode(String name) {
        return new DefaultMutableTreeNode(HelpItem.createChapter(name));
    }

    private DefaultMutableTreeNode createTopicNode(String name, String helpID) {
        return new DefaultMutableTreeNode(HelpItem.createTopic(name, idToURL(
                helpID)));
    }

    private URL getFullPath(String path) {
        try {
            return new URL(baseURL, path);
        } catch (java.net.MalformedURLException e) {
            logger.warning("Can't form URL" + e.getLocalizedMessage());
            return null;
        }
    }

    /**
     * Reads the HelpSet's main file, the map, and the table of contents.
     */
    private void readHelpSet(URL helpSetURL) {
        Document document = getDocument(helpSetURL, "HelpSet");
        if (document != null) {
            parseHelpSet(document);
        }
    }

    private void parseHelpSet(Document document) {
        Node documentElement = document.getDocumentElement();
        NodeList nodes = documentElement.getChildNodes();
        for (int i = 0; i < nodes.getLength(); i++) {
            Node node = nodes.item(i);
            String nodeName = node.getNodeName();
            if (nodeName.equals("title")) {
                parseTitle(node);
            } else if (nodeName.equals("maps")) {
                parseMaps(node);
            } else if (nodeName.equals("view")) {
                parseViews(node);
            }
        }
    }

    private void parseTitle(Node titleNode) {
        title = getTextValue(titleNode);
        // System.out.println("Title=" + title);
    }

    private void parseMaps(Node mapsNode) {
        String homeID = null;
        NodeList nodes = mapsNode.getChildNodes();
        for (int i = 0; i < nodes.getLength(); i++) {
            Node node = nodes.item(i);
            String nodeName = node.getNodeName();
            if (nodeName.equals("homeID")) {
                homeID = getTextValue(node);
                // System.out.println("homeID=" + homeID);
            } else if (nodeName.equals("mapref")) {
                String location = getAttributeValue(node, "location");
                // System.out.println("Map file location=" + location);
                readMap(location);
            }
        }
        home = idToURL(homeID);
        // System.out.println("Home=" + home);
    }

    private void parseViews(Node viewsNode) {
        NodeList nodes = viewsNode.getChildNodes();
        for (int i = 0; i < nodes.getLength(); i++) {
            Node node = nodes.item(i);
            String nodeName = node.getNodeName();
            if (nodeName.equals("data")) {
                readTOC(getTextValue(node));
            }
        }
    }




    private void readMap(String mapLocation) {
        URL mapURL = getFullPath(mapLocation);
        Document document = getDocument(mapURL, "HelpMap");
        if (document != null) {
            parseHelpMap(document);
        }
    }


    private void parseHelpMap(Document document) {
        Node documentElement = document.getDocumentElement();
        NodeList nodes = documentElement.getChildNodes();
        for (int i = 0; i < nodes.getLength(); i++) {
            Node node = nodes.item(i);
            String nodeName = node.getNodeName();
            if (nodeName.equals("mapID")) {
                parseMapID(node);
            }
        }
    }

    private void parseMapID(Node mapIDNode) {
        String target = getAttributeValue(mapIDNode, "target");
        String url    = getAttributeValue(mapIDNode, "url");
        //System.out.println("MapID target=" + target + "; url=" + url);
        Object oldURL = idsToURL.put(target, getFullPath(url));
        if (oldURL != null) {
            logger.severe("Duplicate entry in HelpSet map " + mapIDNode.getOwnerDocument().getNamespaceURI()
                    + ": target=" + target + " url=" + url);
        }
    }





    /**
     * Reads the HelpSet's table of contents.
     */
    private void readTOC(String tocName) {
        URL tocURL = getFullPath(tocName);
        Document document = getDocument(tocURL, "HelpTOC");
        if (document != null) {
            parseHelpTOC(document);
        }
    }


    private void parseHelpTOC(Document document) {
        root = new DefaultMutableTreeNode(null);

        Node documentElement = document.getDocumentElement();
        parseTOCItems(documentElement, root);
    }

    private void parseTOCItems(Node parentNode, DefaultMutableTreeNode parent) {
        NodeList nodes = parentNode.getChildNodes();
        for (int i = 0; i < nodes.getLength(); i++) {
            Node node = nodes.item(i);
            String nodeName = node.getNodeName();
            if (nodeName.equals("tocitem")) {
                String text   = getAttributeValue(node, "text");
                String image  = getAttributeValue(node, "image");
                String target = getAttributeValue(node, "target");
                //System.out.println("text=" + text + "; image=" + image + "; target=" + target);
                if ("chapter".equals(image)) {
                    DefaultMutableTreeNode treeNode = createChapterNode(text);
                    parent.add(treeNode);
                    parseTOCItems(node, treeNode);
                } else {
                    parent.add(createTopicNode(text, target));
                }
            }
        }
    }


    private static Document getDocument(URL url, String documentDescription) {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        // Configure the factory
        factory.setIgnoringComments(true);
        factory.setCoalescing(true);
        try {
            DocumentBuilder builder = factory.newDocumentBuilder();
            builder.setEntityResolver(new MyResolver());
            return builder.parse(url.toExternalForm());
        } catch (ParserConfigurationException e) {
            logger.log(Level.WARNING,
                    "Can't build " + documentDescription + " parser.",
                    e);
        } catch (IOException e) {
            logger.log(Level.WARNING,
                    "Can't open " + documentDescription + " from " + url,
                    e);
        } catch (SAXException e) {
            logger.log(Level.WARNING,
                    "Can't parse " + documentDescription + " from " + url,
                    e);
        }
        return null;
    }


    private static String convertNodeValue(Node node) {
        if (node.getNodeValue() == null)
            return null;
        String printValue = node.getNodeValue().trim();
        int x = printValue.indexOf("\n");
        if (x >= 0) printValue = printValue.substring(0, x);
        return printValue;
    }

    private static String getTextValue(Node node) {
        return node == null
            ? null
            : convertNodeValue(node.getFirstChild());
    }

    private static String getAttributeValue(Node node, String attributeName) {
        NamedNodeMap attributes = node.getAttributes();
        Node attributeNode = attributes.getNamedItem(attributeName);
        return attributeNode == null
            ? null
            : attributeNode.getNodeValue();
    }


    /**
     * An EntityResolver implementation that uses local copies
     * of the most frequently used JavaHelp DTDs.
     * We failed to parse the official public TOC 1.0 DTD,
     * and therefore this replacement seems to be necessary.
     */
    private static final class MyResolver implements EntityResolver {

        private static final String HELPSET_1_0_DTD_PATH =
            "http://java.sun.com/products/javahelp/helpset_1_0.dtd";

        private static final String HELPSET_1_0_DTD_REPLACEMENT_PATH =
            "/com/jgoodies/uifextras/help/resources/helpset_1_0.dtd";

        private static final String MAP_1_0_DTD_PATH =
            "http://java.sun.com/products/javahelp/map_1_0.dtd";

        private static final String MAP_1_0_DTD_REPLACEMENT_PATH =
            "/com/jgoodies/uifextras/help/resources/map_1_0.dtd";

        private static final String TOC_1_0_DTD_PATH =
            "http://java.sun.com/products/javahelp/toc_1_0.dtd";

        private static final String TOC_1_0_DTD_REPLACEMENT_PATH =
            "/com/jgoodies/uifextras/help/resources/toc_1_0.dtd";


        public InputSource resolveEntity(String publicId, String systemId) {
            String replacementPath;
            if (systemId.equals(HELPSET_1_0_DTD_PATH)) {
                replacementPath = HELPSET_1_0_DTD_REPLACEMENT_PATH;
            } else if (systemId.equals(MAP_1_0_DTD_PATH)) {
                replacementPath = MAP_1_0_DTD_REPLACEMENT_PATH;
            } else if (systemId.equals(TOC_1_0_DTD_PATH)) {
                replacementPath = TOC_1_0_DTD_REPLACEMENT_PATH;
            } else {
                // The default behavior.
                return null;
            }

            InputStream in = getInputStream(replacementPath);
            return new InputSource(in);
        }
    }


    /**
     * Gets and returns an InputStream for the given path
     * using the default ClassLoader.
     *
     * @param path   the path used to lookup the input stream's URL
     * @return an InputStream for the contents of the URL described by
     *     the specified path
     */
    private static InputStream getInputStream(String path) {
        URL url = HelpSet.class.getResource(path);
        if (url != null) {
            try {
                return url.openStream();
            } catch (IOException e) {
                logger.log(
                    Level.INFO,
                    "Can't open URL " + url.toString(),
                    e);
            }
        }
        return null;
    }


}
