/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.help;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.net.URL;
import java.util.prefs.Preferences;

import javax.swing.*;

import com.jgoodies.looks.HeaderStyle;
import com.jgoodies.uif.AbstractFrame;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ApplicationContext;
import com.jgoodies.uif.builder.MenuBuilder;
import com.jgoodies.uif.builder.ToolBarBuilder;
import com.jgoodies.uif.component.UIFSplitPane;
import com.jgoodies.uif.util.ScreenUtils;
import com.jgoodies.uif.util.SystemUtils;

/**
 * Presents a HelpSet in a frame with a navigation tree and HTML display.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.12 $
 *
 * @see HelpModel
 */
final class HelpViewer extends AbstractFrame {

    private static final String    WINDOW_ID = "help";

    private static final Dimension WINDOW_MINIMUM_SIZE =
        new Dimension(200, 200);

    private static final String DIVIDER_LOCATION_KEY =
        WINDOW_ID + ".divider_location";

    private final HelpModel model;

    private NavigationView   navigationView;
    private PresentationView presentationView;
    private JSplitPane       splitPane;


    // Instance Creation ******************************************************

    HelpViewer(HelpModel model) {
        super(model.getTitle());
        setName(WINDOW_ID);
        this.model = model;
    }


    // Help API ***************************************************************

    /**
     * Shows or hides this help viewer.
     */
    void setDisplayed(boolean display) {
        if (display)
            show(model.getHome());
        else
            setVisible(false);
    }

    /**
     * Makes the help viewer visible and opens the specified <code>URL</code>.
     */
    void show(URL url) {
        if (!isVisible())
            open();
        else {
            setState(Frame.NORMAL);
            setVisible(true);
        }
        model.setSelection(url);
    }


    // Abstract Building ******************************************************

    /**
     * Builds the help viewer.
     */
    @Override
    public void build() {
        super.build();
        if (SystemUtils.IS_OS_MAC) {
            setJMenuBar(buildMenuBar());
        }
        setTitle(getTitle());
    }

    /**
     * Unlike the default frames, we are located on screen north-east by
     * default.
     */
    @Override
    protected void locateOnScreen() {
        ScreenUtils.locateOnScreenNorthEast(this);
    }


    private void initComponents() {
        navigationView   = new NavigationView(model);
        presentationView = new PresentationView(model);
    }


    /**
     * Builds and returns this help viewer's content pane.
     */
    @Override
    protected JComponent buildContentPane() {
        initComponents();

        JPanel content = new JPanel(new BorderLayout(0, 3));
        content.add(buildToolBar(),                     BorderLayout.NORTH);
        content.add(buildNavigationPresentationPanel(), BorderLayout.CENTER);
        return content;
    }


    // Concrete Building ******************************************************

    private JMenuBar buildMenuBar() {
        ActionMap helpModelMap = Application.getActionMap(model);

        JMenuBar menuBar = new JMenuBar();
        MenuBuilder builder;

        builder = new MenuBuilder("File", 'F');
        builder.add(helpModelMap.get("com.jgoodies.help.close"));
        builder.addSeparator();
        builder.add(helpModelMap.get("com.jgoodies.help.pageSetup"));
        builder.add(helpModelMap.get("com.jgoodies.help.print"));
        menuBar.add(builder.getMenu());

        builder = new MenuBuilder("History", 'H');
        builder.add(model.getHistory().getGoBackAction());
        builder.add(model.getHistory().getGoNextAction());
        builder.add(model.getHistory().getGoHomeAction());
        menuBar.add(builder.getMenu());

        return menuBar;
    }

    /**
     * Builds and returns the tool bar.
     */
    private JToolBar buildToolBar() {
        ActionMap helpModelMap = Application.getActionMap(model);

        ToolBarBuilder builder = new ToolBarBuilder(getTitle(), HeaderStyle.SINGLE);
        builder.getToolBar().putClientProperty("Plastic.borderStyle", "Separator");
        builder.addGap();
        builder.add(model.getHistory().getGoBackAction());
        builder.addGap();
        builder.add(model.getHistory().getGoNextAction());
        builder.addGap();
        builder.add(model.getHistory().getGoHomeAction());
        builder.addGap();
        builder.add(helpModelMap.get("com.jgoodies.help.print"));
        return builder.getToolBar();
    }

    /**
     * Builds and returns the navigation and presentation split panel.
     */
    private JComponent buildNavigationPresentationPanel() {

        // Add a splitpane with tree in the left and graph in the right.
        splitPane = UIFSplitPane.createStrippedSplitPane(
                JSplitPane.HORIZONTAL_SPLIT,
                navigationView.getPanel(),
                presentationView.getPanel());

        JPanel panel = new JPanel(new BorderLayout());
        panel.add(splitPane, BorderLayout.CENTER);
        panel.setBorder(BorderFactory.createEmptyBorder(2, 0, 0, 0));
        return panel;
    }


    // Overriding superclass behavior *****************************************

    /**
     * Returns the frame's minimum size. It is used by the WindowUtils to resize
     * the window if the user has shrinked the window below this given size.
     *
     * @return the frame's minimum size
     */
    public Dimension getWindowMinimumSize() {
        return WINDOW_MINIMUM_SIZE;
    }

    @Override
    protected void configureCloseOperation() {
        setDefaultCloseOperation(HIDE_ON_CLOSE);
    }

    @Override
    protected void resizeHook(JComponent c) {
        // Unlike the superclass, we don't resize the viewer.
    }


    // Storing and Restoring Window State ************************************

    /**
     * In addition to the superclass behavior
     * this method stores the divider location.
     */
    @Override
    protected void storeState() {
        if (splitPane == null)
            return;

        super.storeState();
        Preferences prefs = ScreenUtils.screenConfigurationNode(
                ApplicationContext.getInstance().getUserPreferences());
        prefs.putInt(DIVIDER_LOCATION_KEY, splitPane.getDividerLocation());
    }


    /**
     * In addition to the superclass behavior
     * this method restore the divider location.
     */
    @Override
    protected void restoreState() {
        super.restoreState();
        Preferences prefs = ScreenUtils.screenConfigurationNode(
                ApplicationContext.getInstance().getUserPreferences());
        restoreDividerLocation(prefs);
    }


    /**
     * Set the frame's split pane divider location from the Preferences.
     */
    private void restoreDividerLocation(Preferences prefs) {
        // Read the raw layout data.
        int dividerLocation = prefs.getInt(DIVIDER_LOCATION_KEY, -1);

        // Do nothing if the data couldn't be read.
        if (dividerLocation == -1)
            return;

        // Do nothing if the divider location exceeds the frame's width.
        if (dividerLocation > getSize().width)
            return;

        // Set the divider location.
        splitPane.setDividerLocation(dividerLocation);
    }


}