/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */


package com.jgoodies.uifextras.help;

import java.awt.Component;
import java.awt.Font;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeModel;

import com.jgoodies.uif.AbstractView;
import com.jgoodies.uif.util.TreeUtils;

/**
 * Builds the navigation panel that displays a tree of DefaultMutableTreeNodes
 * that in turn represent the HelpItems. Observes the HelpModel selection
 * to scroll the tree to the selected node.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.7 $
 *
 * @see HelpModel
 * @see PresentationView
 */
final class NavigationView extends AbstractView {

    /**
     * Refers to the HelpModel that holds the tree model and tree selection model.
     */
    private final HelpModel model;

    /**
     * Refers to the tree that displays the navigation nodes.
     *
     * @see #initComponents()
     */
    private JTree tree;


    // Instance Creation ******************************************************

    /**
     * Constructs a NavigationView for the given HelpModel.
     *
     * @param model  provides the tree model and tree selection model
     */
    NavigationView(HelpModel model) {
        this.model = model;
    }


    // Initialization *********************************************************

    /**
     * Creates, binds and configures the navigation tree
     * used to display the help items. Requests the tree model
     * and the tree selection model from the HelpModel.
     */
    private void initComponents() {
        tree = new RefreshedTree(model.getNavigationTreeModel());
        tree.setSelectionModel(model.getNavigationTreeSelectionModel());
        //tree.putClientProperty("JTree.lineStyle", "None");
        tree.setRootVisible(false);
        tree.setCellRenderer(new TreeCellRenderer());
        TreeUtils.expandTopLevel(tree);
    }


    private void initEventHandling() {
        tree.addMouseListener(model.getPopupListener());

        model.addPropertyChangeListener(
                HelpModel.PROPERTYNAME_SELECTION,
                new SelectedURLChangeHandler());
    }


    // Building ***************************************************************

    /**
     * Builds the tree, initializes events and returns the navigation tree.
     */
    @Override
    protected JComponent buildPanel() {
        initComponents();
        initEventHandling();

        JComponent panel  = new JScrollPane(tree);
        panel.putClientProperty("jgoodies.isEtched", Boolean.TRUE);
        panel.updateUI();
        return panel;
    }


    // Event Handling *********************************************************

    private class SelectedURLChangeHandler implements PropertyChangeListener {

        public void propertyChange(PropertyChangeEvent event) {
            tree.scrollPathToVisible(tree.getSelectionPath());
        }
    }


    // Helper Classes *********************************************************

    /**
     * Renders icons and labels for the help nodes.
     */
    private static final class TreeCellRenderer extends DefaultTreeCellRenderer {

        private Font cachedOriginalFont;
        private Font boldFont;

        @Override
        public Component getTreeCellRendererComponent(JTree tree, Object value,
            boolean sel, boolean expanded, boolean leaf, int row, boolean focused) {
            super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, focused);
            String stringValue = tree.convertValueToText(value, sel, expanded, leaf, row, focused);
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
            HelpItem item = (HelpItem) node.getUserObject();
            if (null == item)
                return this;
            setText(stringValue);
            selected = item.isChapter() ? false : sel;
            this.setForeground(selected ? getTextSelectionColor() : getTextNonSelectionColor());
            this.setIcon(item.getIcon(expanded));
            this.setFont(item.isChapter() ? getBoldFont(tree.getFont()) : tree.getFont());
            return this;
        }

        private Font getBoldFont(Font originalFont) {
            if (originalFont != cachedOriginalFont) {
                boldFont = originalFont.deriveFont(Font.BOLD);
                cachedOriginalFont = originalFont;
            }
            return boldFont;
        }
    }


    /**
     * Refreshes the selection path and cell renderer after UI changes.
     */
    private static final class RefreshedTree extends JTree {

        private RefreshedTree(TreeModel treeModel) {
            super(treeModel);
        }

        /**
         * In addition to the superclass behavior
         * this method updates the cell renderer.
         */
        @Override
        public void updateUI() {
            super.updateUI();
            setCellRenderer(new TreeCellRenderer());
        }

    }
}