/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.history;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;

import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import com.jgoodies.binding.beans.Model;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;

/**
 * A class that provides the state and behavior for a back-next navigation.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.22 $
 *
 * @param <E>    the type of the elements in this history
 */
public class History<E> extends Model {

    // Names of Bound Bean Properties *****************************************

    public static final String PROPERTYNAME_HOME      = "home";
    public static final String PROPERTYNAME_SELECTION = "selection";


    // Action Resource Keys ***************************************************

    public static final String ACTION_GO_BACK = "history.goBack";
    public static final String ACTION_GO_NEXT = "history.goNext";
    public static final String ACTION_GO_HOME = "history.goHome";


    // ************************************************************************

    private static final int MAX_TOOLTIP_LENGTH = 60;

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(History.class);


    // Instance Fields ********************************************************

    private final int capacity;
    private final Action     goBackAction;
    private final Action     goNextAction;
    private final Action     goHomeAction;
    private final JPopupMenu backPopupMenu;
    private final JPopupMenu nextPopupMenu;

    private E home;
    private HistoryList<E> list;


    // Instance Creation ******************************************************

    /**
     * Constructs a History with the given maximum capacity.
     *
     * @param capacity the maximum number of elements in the List.
     */
    public History(int capacity) {
        this(capacity, ACTION_GO_BACK, ACTION_GO_NEXT, ACTION_GO_HOME);
    }


    /**
     * Constructs a History with the given maximum capacity.
     * Expects custom action resource keys that can be used to
     * override the default action configuration.
     *
     * @param capacity the maximum number of elements in the List.
     * @param goBackActionName  the resource key for the go-back Action
     * @param goNextActionName  the resource key for the go-next Action
     * @param goHomeActionName  the resource key for the go-home Action
     */
    public History(int capacity,
            String goBackActionName,
            String goNextActionName,
            String goHomeActionName) {
        this.capacity = capacity;
        ActionMap actionMap = Application.getActionMap(this);
        goBackAction = actionMap.get(goBackActionName);
        goNextAction = actionMap.get(goNextActionName);
        goHomeAction = actionMap.get(goHomeActionName);
        backPopupMenu = new JPopupMenu();
        backPopupMenu.putClientProperty("jgoodies.noIcons", Boolean.TRUE);
        nextPopupMenu = new JPopupMenu();
        nextPopupMenu.putClientProperty("jgoodies.noIcons", Boolean.TRUE);
        reset();
    }


    // Aceessors **************************************************************

    /**
     * @return the element that has been set as home element, which becomes
     *         the new selection if the go-home operation is performed.
     *
     * @see #setHome(Object)
     */
    public E getHome() {
        return home;
    }


    /**
     * Sets the given element as new home element.
     * It is set when the go-home operation is performed.
     *
     * @param newHome   the element to become the new home element
     */
    public void setHome(E newHome) {
        E oldHome = getHome();
        this.home = newHome;
        firePropertyChange(PROPERTYNAME_HOME, oldHome, newHome);
    }


    /**
     * @return the current selection
     */
    public E getSelection() {
        return list.getLastAdded();
    }


    /**
     * Sets the given element as new selection.
     *
     * @param newSelection   the object to be set as new selection
     */
    public void setSelection(E newSelection) {
        E oldSelection = getSelection();
        if (equals(oldSelection, newSelection)) {
            return;
        }
        setSelection0(newSelection);
    }


    // Operations *************************************************************

    /**
     * Removes all occurrences of {@code element} in this history.
     *
     * @param element  the element to be removed
     */
    public void removeAll(E element) {
        list.removeAll(element);
        updateActions();
    }


    /**
     * Removes all occurrences of {@code element} and goes back
     * to the previous element.
     *
     * @param element  the element to be removed
     */
    public void removeAndGoBack(E element) {
        list.removeAll(element);
        setSelection0(list.getLastAdded());
        //setSelection0(list.getAndGoPrevious());
    }


    /**
     * Resets the history: clears the list, and disables all actions.
     */
    public void reset() {
        list = new HistoryList<E>(capacity);
        goBackAction.setEnabled(false);
        goNextAction.setEnabled(false);
        goHomeAction.setEnabled(false);
    }


    // Action Operations ******************************************************

    @com.jgoodies.uif.application.Action(name=ACTION_GO_BACK)
    public void goBack() {
        setSelection0(list.getAndGoPrevious());
    }


    @com.jgoodies.uif.application.Action(name=ACTION_GO_NEXT)
    public void goNext() {
        setSelection0(list.getAndGoNext());
    }


    @com.jgoodies.uif.application.Action(name=ACTION_GO_HOME)
    public void goHome() {
        setSelection0(home);
    }


    // Exposing Actions *******************************************************
    // The following methods are not necessary; these actions can be accessed
    // via the Application framework. However, we provide these methods
    // to make the migration to this version easier.
    // ************************************************************************

    /**
     * @return the Action that performs the {@link #goBack()} operation
     */
    public Action getGoBackAction() {
        return goBackAction;
    }


    /**
     * @return the Action that performs the {@link #goNext()} operation
     */
    public Action getGoNextAction() {
        return goNextAction;
    }


    /**
     * @return the Action that performs the {@link #goHome()} operation
     */
    public Action getGoHomeAction() {
        return goHomeAction;
    }


    // Exposing Popup Menus ***************************************************

    /**
     * @return the popup menu that consists of menu items for back operations
     */
    public JPopupMenu getBackPopupMenu() {
        return backPopupMenu;
    }


    /**
     * @return the popup menu that consists of menu items for next operations
     */
    public JPopupMenu getNextPopupMenu() {
        return nextPopupMenu;
    }


    // Default Behavior *******************************************************

    /**
     * Creates and returns a tool tip for the given object;
     * clips the string's center, if it exceeds a maximum length.
     *
     * @param element    the object to extract the tooltip text from
     * @return the tooltip string extracted from the given target
     */
    protected String createToolTip(E element) {
        return centerClippedString(element.toString(), MAX_TOOLTIP_LENGTH);
    }


    // Helper Code ************************************************************

    private String getBackText() {
        E previous = list.getPrevious();
        return (null == previous)
            ? RESOURCES.getString("history.back")
            : RESOURCES.getString("history.backTo", createToolTip(previous));
    }


    private String getNextText() {
        E next = list.getNext();
        return (null == next)
            ? RESOURCES.getString("history.next")
            : RESOURCES.getString("history.nextTo", createToolTip(next));
    }


    private void setSelection0(E newSelection) {
        if (null == newSelection) {
            return;
        }
        list.add(newSelection);
        updateMenu(backPopupMenu, list.getBackIterator(), true);
        updateMenu(nextPopupMenu, list.getNextIterator(), false);
        updateActions();
        firePropertyChange(PROPERTYNAME_SELECTION, null, newSelection);
    }


    /**
     * Returns a string that has a maximum size and contains <i>&hellip;</i>
     * if the given string was too large to fit into the maximum size.
     */
    private static String centerClippedString(String str, int maxSize) {
        int length = str.length();
        if (length <= maxSize)
            return str;

        int headLength = maxSize / 2;
        int tailLength = maxSize - headLength - 3;
        String head = str.substring(0, headLength);
        String tail = str.substring(length - tailLength, length - 1);
        return head + "\u2026" + tail;
    }


    // Event Handling *********************************************************

    private void updateActions() {
        E selection = getSelection();
        E previous = list.getPrevious();
        E next = list.getNext();

        String goBackToolTip = getBackText();
        String goNextToolTip = getNextText();

        goBackAction.putValue(Action.SHORT_DESCRIPTION, goBackToolTip);
        goBackAction.putValue(Action.NAME, goBackToolTip);
        goNextAction.putValue(Action.SHORT_DESCRIPTION, goNextToolTip);
        goNextAction.putValue(Action.NAME, goNextToolTip);

        goBackAction.setEnabled(previous != null);
        goNextAction.setEnabled(next != null);
        goHomeAction.setEnabled((home != null) && !home.equals(selection));
    }


    private void updateMenu(JPopupMenu menu, Iterator<E> items, boolean goBack) {
        menu.removeAll();
        for (int steps = 1; items.hasNext(); steps++) {
            JMenuItem menuItem = new JMenuItem(createToolTip(items.next()));
            menuItem.putClientProperty("noIcon", Boolean.TRUE);
            menuItem.addActionListener(new GoMenuActionListener(steps, goBack));
            menu.add(menuItem);
        }
    }


    private void updateState() {
        updateMenu(backPopupMenu, list.getBackIterator(), true);
        updateMenu(nextPopupMenu, list.getNextIterator(), false);
        updateActions();
        firePropertyChange(PROPERTYNAME_SELECTION, null, getSelection());
    }


    /**
     * Implements the go back and go next behavior for the menu items.
     */
    private final class GoMenuActionListener implements ActionListener {

        private final int steps;
        private final boolean back;

        private GoMenuActionListener(int steps, boolean back) {
            this.steps = steps;
            this.back = back;
        }

        public void actionPerformed(ActionEvent e) {
            if (back) {
                list.goBack(steps);
            } else {
                list.goNext(steps);
            }
            updateState();
        }
    }


}
