/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.history;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

/**
 * Provides behavior for getting forward and backward in a fixed list
 * of recently used items. Used by back and forward navigation buttons.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.9 $
 *
 * @param <E>  the type of the list elements
 */
public final class HistoryList<E> {

    private final int  limit;
    private final List<E> list;
    private int indexOfNextAdd;


    /**
     * Constructs a <code>HistoryList</code> with the given capacity limit.
     *
     * @param limit    the upper bound for the list's capacity
     */
    public HistoryList(int limit) {
        this.limit = limit;
        this.list  = new ArrayList<E>(limit);
        indexOfNextAdd = 0;
    }

    /**
     * Adds an element to the history if it is not the previous element.
     * Returns whether the history changed.
     *
     * @param o   the object to add
     * @return true if the history changed, false if it's unchanged
     */
    public boolean add(E o) {
        // If the previous element equals this one, do nothing and answer false.
        if (o.equals(getLastAdded()))
            return false;

        // Trim from the indexOfNextAdd to the end, as we'll
        // never reach these elements once the new one is added.
        // This is the only operation where we would benefit
        // from subclassing ArrayList; the latter provides #removeRage.
        for (int index = list.size() - 1; index > 0 && index >= indexOfNextAdd; index--) {
            list.remove(index);
        }
        list.add(indexOfNextAdd, o);

        // If the limit is exceeded remove the least recently added element.
        // Otherwise increase the indexOfNextAdd.
        if (list.size() > limit)
            list.remove(0);
        else
            indexOfNextAdd++;

        // Answer true to indicate that the element has been added.
        return true;
    }

    public void removeAll(E item) {
        for (int index = list.size() - 1; index > 0; index--) {
            E element = list.get(index);
            if (element.equals(item)) {
                list.remove(index);
                if (index < indexOfNextAdd) {
                    indexOfNextAdd--;
                }
            }
        }
    }

    /**
     * Returns the previous element and goes back - if possible.
     *
     * @return the previous element
     */
    public E getAndGoPrevious() {
        return (hasPrevious()) ? list.get(--indexOfNextAdd - 1) : null;
    }

    /**
     * Returns the next element and goes next - if possible.
     *
     * @return the next element
     */
    public E getAndGoNext() {
        return (hasNext()) ? list.get(indexOfNextAdd++) : null;
    }

    /**
     * Returns an <code>Iterator</code> for the available backward elements.
     *
     * @return an iterator that iterates over the available backward elements
     */
    public Iterator<E> getBackIterator() {
        List<E> sublist = subListNoDuplicates(0, indexOfNextAdd - 1);
        Collections.reverse(sublist);
        return sublist.iterator();
    }

    /**
     * Returns an <code>Iterator</code> for the available next elements.
     *
     * @return an iterator that iterates over the available next elements
     */
    public Iterator<E> getNextIterator() {
        return subListNoDuplicates(indexOfNextAdd, list.size()).iterator();
    }

    /**
     * Returns the element that was added last.
     *
     * @return the most recently added element
     */
    public E getLastAdded() {
        return (indexOfNextAdd > 0) ? list.get(indexOfNextAdd - 1) : null;
    }

    /**
     * Returns the previous element.
     *
     * @return the previous element
     */
    public E getPrevious() {
        return (hasPrevious()) ? list.get(indexOfNextAdd - 2) : null;
    }

    /**
     * Returns the next element.
     *
     * @return the next element
     */
    public E getNext() {
        return (hasNext()) ? list.get(indexOfNextAdd) : null;
    }

    /**
     * Goes back the specified number of steps.
     *
     * @param steps    the number of steps to go back
     */
    public void goBack(int steps) {
        indexOfNextAdd = Math.max(0, indexOfNextAdd - steps);
    }

    /**
     * Goes forward the specified number of steps.
     *
     * @param steps    the number of steps to go forward
     */
    public void goNext(int steps) {
        indexOfNextAdd = Math.min(indexOfNextAdd + steps, list.size());
    }

    /**
     * Checks and answers if there's a previous element.
     *
     * @return true if there's a previous element
     */
    public boolean hasPrevious() {
        return indexOfNextAdd > 1;
    }

    /**
     * Checks and answer if there's a next element.
     *
     * @return true if there's a next element
     */
    public boolean hasNext() {
        return indexOfNextAdd < list.size();
    }

    private List<E> subListNoDuplicates(int startIndex, int stopIndex) {
        E lastAdded = getLastAdded();
        ArrayList<E> result = new ArrayList<E>();
        for (int i = startIndex; i < stopIndex; i++) {
            E o = list.get(i);
            if (o != lastAdded)
                result.add(o);
        }
        return result;
    }

    @Override
    public String toString() {
        return "(Limit: "
            + limit
            + "; indexOfNextAdd: "
            + indexOfNextAdd
            + ")"
            + list.toString();
    }


}
