/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.laf;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import com.jgoodies.binding.beans.BeanAdapter;
import com.jgoodies.binding.beans.Model;
import com.jgoodies.binding.list.SelectionInList;
import com.jgoodies.binding.value.ValueModel;
import com.jgoodies.uif.laf.ExtUIManager;
import com.jgoodies.uif.laf.LookConfiguration;
import com.jgoodies.uif.laf.LookConfigurations;

/**
 * A model for choosing a LookAndFeel and color theme. Provides two lists
 * with selection: one for the available LookConfiguration instances,
 * another for the themes associated with the selected LookConfiguration.<p>
 *
 * This whole model operates on a clone of the ExtUIManager's
 * LookConfigurations, held in a BeanAdapter's bean channel.
 * If this model's trigger channel changes to true, this LookConfigurations
 * is set to the ExtUIManager, which will update the app's user interface.
 * If the trigger indicates a flush, the LookConfigurations is read (and copied)
 * again from the ExtUIManager.<p>
 *
 * Implementation Note: We could potentially use the configurationSelection's
 * selectionHolder as bean channel for the themeListHolder and the
 * themeSelectionHolder. This would work fine, if we would have no other
 * listeners observing changes in the configurationSelection. But we observe
 * this selection with a ConfigurationChangeHandler. Since we need to handle
 * changes in the configuration selection in a known order, we cannot rely
 * on the automatic updates. Therefore this implementation sets the new bean
 * for the selectedConfigurationAdapter "by hand" in the
 * ConfigurationChangeHandler, to ensure the order we want.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.4 $
 *
 * @see LafChoiceView
 * @see com.jgoodies.binding.list.SelectionInList
 */
public final class LafChoiceModel extends Model {

    // Name of the Bound Bean Property ****************************************

    /**
     * The name of the bound read-only property that is intended
     * to indicate changes in the selected configuration.
     *
     * @see #getSelectedConfiguration()
     */
    public static final String PROPERTYNAME_SELECTED_CONFIGURATION =
        "selectedConfiguration";


    // ************************************************************************

    /**
     * Refers to the trigger that is observed to commit or forget
     * the changes in this model's LookConfigurations copy.
     * If this trigger channel changes to true,
     * the copied LookConfigurations is set to the ExtUIManager.
     */
    private final ValueModel triggerChannel;

    /**
     * Holds a copy of the ExtUIManager's LookConfigurations.
     * This whole LafChoiceModel operates on the copy.
     * If the trigger indicates a commit, the copy is set as new
     * LookConfigurations to the ExtUIManager. In case the trigger
     * indicates a flush, this copy is read again from the ExtUIManager.
     *
     * @see ExtUIManager#getLookConfigurations()
     * @see ExtUIManager#setLookConfigurations(LookConfigurations)
     */
    private final BeanAdapter configurationsAdapter;

    /**
     * Adapts the bound properties of the currently selected LookConfiguration.
     * Used to get the list of themes and theme selection of this configuration.
     */
    private final BeanAdapter selectedConfigurationAdapter;

    /**
     * A SelectionInList of LookConfiguration instances.
     * The selection holder is the LookConfiguration's
     * "defaultConfiguration".
     */
    private final SelectionInList configurationSelection;

    /**
     * A SelectionInList of the themes associated with
     * the LookConfiguration selected in the configurationSelection.
     */
    private final SelectionInList themeSelection;

    /**
     * Listens to changes in the selected theme to notify observers about
     * changes in the LookConfiguration. This in turn can be used to update
     * a L&amp;F preview.
     * This listener will be temporarily removed from the themeSelection
     * if the LookConfiguration changes. This avoids duplicate notifications
     * of LookConfiguration changes.
     */
    private PropertyChangeListener themeChangeHandler;


    // Instance Creation ******************************************************

    /**
     * Constructs a LookAndFeelChoiceModel using the given trigger.
     * First sets up the list with selection of the LookConfiguration instances.
     * These are adapted from the LookConfigurations by means of a BeanAdapter.
     *
     * In a second step the list with selection of color themes is initialized
     * much like the LookConfigurations instances.
     *
     * @param triggerChannel    changes to <code>Boolean.TRUE</code> or
     *      <code>Boolean.FALSE</code> to indicate a commit or flush event
     */
    public LafChoiceModel(ValueModel triggerChannel) {
        this.triggerChannel = triggerChannel;

        // Setup the configuration selection in list
        configurationsAdapter = new BeanAdapter(ExtUIManager.getLookConfigurations(), true);
        ValueModel configurationListHolder =
            configurationsAdapter.getValueModel(LookConfigurations.PROPERTYNAME_CONFIGURATIONS);
        ValueModel configurationSelectionHolder =
            configurationsAdapter.getValueModel(LookConfigurations.PROPERTYNAME_DEFAULT_CONFIGURATION);

        configurationSelection = new SelectionInList(
                configurationListHolder,
                configurationSelectionHolder);

        // Setup the theme selection in list.
        selectedConfigurationAdapter = new BeanAdapter(configurationSelectionHolder, true);
        ValueModel themeListHolder =
            selectedConfigurationAdapter.getValueModel(LookConfiguration.PROPERTYNAME_THEMES);
        ValueModel themeSelectionHolder =
            selectedConfigurationAdapter.getValueModel(LookConfiguration.PROPERTYNAME_THEME);

        themeSelection = new SelectionInList(themeListHolder, themeSelectionHolder);

        initEventHandling();
    }


    // Initialization *********************************************************

    /**
     * Initialized the events handled by this model: 1) changes in
     * the chosen LookConfiguration, 2) changes in the selected theme,
     * 3) a commit indicated by the trigger channel.<p>
     *
     * If the LookConfiguration changes, we update the themeSelection's
     * list and selection holder. During this update, we do not handle
     * changes in the theme, because the new selection holder may fire
     * a theme change.<p>
     *
     * If the theme changes, we just fire a LookConfiguration change.<p>
     *
     * If the trigger channel changes to Boolean.TRUE, we commit
     * this model's backed LookConfiguration to the UIManager.
     */
    private void initEventHandling() {
        getConfigurationSelection().addPropertyChangeListener(
                SelectionInList.PROPERTYNAME_SELECTION,
                new ConfigurationChangeHandler());

        themeChangeHandler = new ThemeChangeHandler();
        getThemeSelection().addPropertyChangeListener(
                SelectionInList.PROPERTYNAME_SELECTION,
                themeChangeHandler);

         triggerChannel.addValueChangeListener(new TriggerChangeHandler());
    }


    // Exposing Models ********************************************************

    /**
     * Returns the list of LookConfiguration instances with a single selection.
     *
     * @return the selection of LookConfiguration instances.
     */
    public SelectionInList getConfigurationSelection() {
        return configurationSelection;
    }


    /**
     * Returns the list of themes associated with the currently selected
     * LookConfiguration plus the selected theme.
     *
     * @return the selection of themes associated with the current LookConfiguration
     */
    public SelectionInList getThemeSelection() {
        return themeSelection;
    }


    /**
     * Returns the currently selected LookConfiguration.
     *
     * @return the currently selected LookConfiguration.
     */
    public LookConfiguration getSelectedConfiguration() {
        return (LookConfiguration) configurationSelection.getSelection();
    }


    // Handling Selection Changes *********************************************

    private void fireSelectedConfigurationChanged(LookConfiguration newConfiguration) {
        firePropertyChange(PROPERTYNAME_SELECTED_CONFIGURATION, null, newConfiguration);
    }

    // The chosen configuration has changed, update the theme selection.
    // While updating do not handle theme changes to avoid
    // duplicate notification of observers of the LookConfiguration property.
    private class ConfigurationChangeHandler implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent evt) {
            LookConfiguration newConfiguration = (LookConfiguration) evt.getNewValue();
            themeSelection.removePropertyChangeListener(
                    SelectionInList.PROPERTYNAME_SELECTION,
                    themeChangeHandler);
            selectedConfigurationAdapter.setBean(newConfiguration);
            themeSelection.addPropertyChangeListener(
                    SelectionInList.PROPERTYNAME_SELECTION,
                    themeChangeHandler);
            fireSelectedConfigurationChanged(newConfiguration);
        }
    }

    private class ThemeChangeHandler implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent evt) {
            fireSelectedConfigurationChanged(getSelectedConfiguration());
        }
    }


    // Handling Trigger Events ************************************************

    /**
     * Sets the current bean of the configurationsAdapter as
     * new LookConfigurations to the ExtUIManager. After the
     * commit, we read a clone from the ExtUIManager, to make
     * sure we don't operate on the ExtUIManagers values.
     */
    private void commitLookConfigurations() {
        LookConfigurations backedConfigurations =
            (LookConfigurations) configurationsAdapter.getBean();
        ExtUIManager.setLookConfigurations(backedConfigurations);
        configurationsAdapter.setBean(ExtUIManager.getLookConfigurations());
    }


    /**
     * Reads the LookConfigurations from the ExtUIManager
     * and sets it as new bean of the configurationsAdapter.
     * Since the ExtUIManager returns a clone of the current LookConfigurations
     * this will change the configurationSelection and in turn
     * the themeSelection.
     */
    private void flushLookConfigurations() {
        configurationsAdapter.setBean(ExtUIManager.getLookConfigurations());
    }


    // Listens to changes in the trigger channel to commit or flush changes.
    private class TriggerChangeHandler implements PropertyChangeListener {

        public void propertyChange(PropertyChangeEvent evt) {
            if (Boolean.TRUE.equals(evt.getNewValue())) {
                commitLookConfigurations();
            } else if (Boolean.FALSE.equals(evt.getNewValue())) {
                flushLookConfigurations();
            }
        }
    }

}