/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.laf;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;

import javax.swing.*;

import com.jgoodies.looks.plastic.PlasticLookAndFeel;
import com.jgoodies.looks.plastic.PlasticTheme;
import com.jgoodies.uif.panel.CardPanel;
import com.jgoodies.uif.util.ComponentTreeUtils;

/**
 * A panel that provides a preview for a look using a bunch of sample widgets.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.4 $
 */
final class LafChoicePreview extends CardPanel {

    private static final Dimension PREFERRED_SIZE = new Dimension(300, 170);
    private static final String    SAMPLE_CARD    = "sample";
    private static final String    FAKE_CARD      = "fake";

    private final JComponent samplePanel;
    private Image buffer;


    // Instance Creation ****************************************************

    /**
     * Constructs a LafChoicePreview.
     */
    LafChoicePreview() {
        samplePanel = new LafChoiceSampleView().getPanel();
        samplePanel.setPreferredSize(PREFERRED_SIZE);
        add(samplePanel, SAMPLE_CARD);
        add(new JLabel("Hallo"), FAKE_CARD);
    }


    // Updating *************************************************************

    void update(LookAndFeel newLaf, PlasticTheme newTheme) {
        // Save the old L&F.
        LookAndFeel oldLaf = UIManager.getLookAndFeel();
        PlasticTheme oldTheme = null;
        try {
            if (newLaf instanceof PlasticLookAndFeel) {
                oldTheme = PlasticLookAndFeel.getPlasticTheme();
                if (newTheme != null)
                    PlasticLookAndFeel.setPlasticTheme(newTheme);
            }
            UIManager.setLookAndFeel(newLaf);
            updateAndValidate();

            if (newLaf instanceof PlasticLookAndFeel)
                PlasticLookAndFeel.setPlasticTheme(oldTheme);

            // Restore the old L&F.
            UIManager.setLookAndFeel(oldLaf);
            updateComponentTree();
        } catch (UnsupportedLookAndFeelException e) {
            // Ignore unsupported looks
        }
    }


    /**
     * Updates the UI delegates of all sample widgets.
     */
    private void updateComponentTree() {
        ComponentTreeUtils.updateComponentTreeUI(samplePanel);
    }


    /**
     * Updates and repaints the UI delegates of all sample widgets.
     */
    private synchronized void updateAndValidate() {
        updateComponentTree();
        ensureHasBuffer();
        updateBuffer();
    }


    @Override
    public void paint(Graphics g) {
        ensureHasBuffer();
        g.drawImage(buffer, 0, 0, null);
    }


    /**
     * Ensures that a background image buffer has been created.
     */
    private synchronized void ensureHasBuffer() {
        if (buffer != null)
            return;
        buffer = createImage(getWidth(), getHeight());
        updateBuffer();
    }


    /**
     * Updates the background image buffer. Therefore switches to
     * the preview card, paints to the buffer, and finally switches back
     * to the fake card.
     */
    private void updateBuffer() {
        validate();
        showCard(SAMPLE_CARD);

        int w = buffer.getWidth(null);
        int h = buffer.getHeight(null);
        Graphics g = buffer.getGraphics();
        g.setClip(0, 0, w, h);
        super.paint(g);
        g.dispose();

        showCard(FAKE_CARD);
    }

}