/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.laf;

import java.awt.Dimension;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif.AbstractView;
import com.jgoodies.uif.component.UIFTree;
import com.jgoodies.uifextras.util.ExtTable;
import com.jgoodies.uifextras.util.ThinBevelBorder;

/**
 * A panel that presents frequently used Swing components.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.5 $
 */
final class LafChoiceSampleView extends AbstractView {

    private JTree        tree;
    private JTable       table;
    private JScrollPane  tableScrollPane;
    private JComponent   textField;
    private JComponent   formattedTextField;
    private JComboBox    editableCombo;
    private JComboBox    nonEditableCombo;
    private JComponent   spinner;
    private JButton      button;
    private JRadioButton radioButton;
    private JCheckBox    checkBox;
    private JLabel       label;
    private JProgressBar progressBar;
    private JSlider      slider;


    // Building the Sample Widgets ******************************************************

    /**
     * Creates and initializes the sample components.
     */
    private void initComponents() {
        tree = new UIFTree(createTreeModel());
        tree.expandRow(2);

        table = new ExtTable(createTableModel());
        table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        table.getColumnModel().getColumn(0).setPreferredWidth(60);
        table.getColumnModel().getColumn(1).setPreferredWidth(105);
        table.setRowSelectionInterval(2, 2);
        tableScrollPane =
            new JScrollPane(
                table,
                ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER,
                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);

        radioButton = new JRadioButton("Radio", true);
        radioButton.setContentAreaFilled(false);
        checkBox = new JCheckBox("Check", true);
        checkBox.setContentAreaFilled(false);
        label = new JLabel("Label");

        final int reducedHeight = 50;
        progressBar = new JProgressBar(SwingConstants.VERTICAL, 0, 100) {
            @Override
            public Dimension getPreferredSize() {
                return new Dimension(
                    super.getPreferredSize().width,
                    reducedHeight);
            }
        };
        progressBar.setValue(50);
        slider = new JSlider(SwingConstants.VERTICAL) {
            @Override
            public Dimension getPreferredSize() {
                Dimension size =
                    new Dimension(
                        super.getPreferredSize().width,
                        reducedHeight);
                //System.out.println("pref size=" + size);
                return size;
            }
        };
        slider.setOpaque(false);


        textField = new JTextField("Text field");
        formattedTextField = new JFormattedTextField("Formatted");

        editableCombo = new JComboBox(new String[] { "Editable", "Two", "Three" });
        editableCombo.setEditable(true);
        nonEditableCombo = new JComboBox(new String[] { "Choice", "Three", "Four" });

        spinner = new JSpinner();
        button = new JButton("Button");
        button.setMnemonic('B');
    }


    /**
     * Builds and returns the sample panel.
     */
    @Override
    protected JComponent buildPanel() {
        initComponents();

        FormLayout layout = new FormLayout(
                "0:g, 3dlu, 0:g, 7dlu, 0:g, 3dlu, 0:g, 7dlu, 0:g, 3dlu, 0:g",
                "fill:min:grow, 7dlu, pref");
        layout.setColumnGroups(new int[][] { { 1, 3, 5, 7, 9, 11 } });
        PanelBuilder builder = new PanelBuilder(layout);
        builder.setBorder(BorderFactory.createCompoundBorder(
                            new ThinBevelBorder(ThinBevelBorder.LOWERED),
                            Borders.DIALOG_BORDER));

        CellConstraints cc = new CellConstraints();
        builder.add(new JScrollPane(tree),  cc.xyw(1, 1, 3));
        builder.add(tableScrollPane,        cc.xyw(5, 1, 3));
        builder.add(buildTabbedPane(),      cc.xyw(9, 1, 3));

        builder.add(textField,              cc.xy (1, 3));
        builder.add(formattedTextField,     cc.xy (3, 3));
        builder.add(editableCombo,          cc.xy (5, 3));
        builder.add(nonEditableCombo,       cc.xy (7, 3));
        builder.add(spinner,                cc.xy (9, 3));
        builder.add(button,                 cc.xy (11,3));

        return builder.getPanel();
    }


    /**
     * Builds and returns the sample tabbed pane.
     */
    private JComponent buildTabbedPane() {
        FormLayout layout = new FormLayout(
                "pref:grow, 4dlu, pref, 4dlu, min",
                "pref, pref, 2dlu, top:pref:grow");
        PanelBuilder builder = new PanelBuilder(layout);
        builder.setBorder(BorderFactory.createEmptyBorder(4, 10, 4, 10));
        builder.setOpaque(false);

        CellConstraints cc = new CellConstraints();
        builder.add(radioButton, cc.xy(1, 1));
        builder.add(checkBox,    cc.xy(1, 2));
        builder.add(label,       cc.xy(1, 4));
        builder.add(progressBar, cc.xywh(3, 1, 1, 4));
        builder.add(slider,      cc.xywh(5, 1, 1, 4));

        JTabbedPane tabbedPane = new JTabbedPane();
        tabbedPane.add("Tab1", builder.getPanel());
        tabbedPane.add("Tab2", new JLabel("Hidden"));
        return tabbedPane;
    }


    /**
     * Creates and returns a sample tree model.
     */
    private TreeModel createTreeModel() {
        DefaultMutableTreeNode root = new DefaultMutableTreeNode("Musicians");
        DefaultMutableTreeNode parent;

        parent = new DefaultMutableTreeNode("Piano");
        root.add(parent);
        parent.add(new DefaultMutableTreeNode("Alexander von Schlippenbach"));
        parent.add(new DefaultMutableTreeNode("McCoy Tyner"));
        parent.add(new DefaultMutableTreeNode("Sun Ra"));

        parent = new DefaultMutableTreeNode("Saxophon");
        root.add(parent);
        parent.add(new DefaultMutableTreeNode("Albert Ayler"));
        parent.add(new DefaultMutableTreeNode("Archie Shepp"));
        parent.add(new DefaultMutableTreeNode("Charlie Parker"));
        parent.add(new DefaultMutableTreeNode("John Coltrane"));
        parent.add(new DefaultMutableTreeNode("Ornette Coleman"));
        parent.add(new DefaultMutableTreeNode("Sonny Rollins"));

        return new DefaultTreeModel(root);
    }


    private TableModel createTableModel() {
        return new DefaultTableModel(
                new String[][] { { "Ayler",    "Greenwich Village" },
                                 { "Coltrane", "Ascension" },
                                 { "Davis",    "In a Silent Way" },
                                 { "Sanders",  "Karma" },
                                 { "Shorter",  "Juju" } },
                new String[] { "Artist", "Title      " });
    }


}