/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.laf;

import java.awt.Component;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.*;
import javax.swing.plaf.metal.MetalTheme;

import com.jgoodies.binding.adapter.BasicComponentFactory;
import com.jgoodies.binding.list.ArrayListModel;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.looks.plastic.PlasticTheme;
import com.jgoodies.uif.AbstractView;
import com.jgoodies.uif.builder.PanelBuilder2;
import com.jgoodies.uif.laf.LookConfiguration;

/**
 * A panel for choosing a LookAndFeel and optional color theme.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.7 $
 *
 * @see LafChoiceModel
 * @see ListModel
 * @see ArrayListModel
 */
public final class LafChoiceView extends AbstractView {

    private final LafChoiceModel lafChoiceModel;

    private JList lafList;
    private JList themesList;
    private LafChoicePreview preview;


    // Instance Creation ******************************************************

    /**
     * Constructs a LafChoiceView for the given LafChoiceModel.
     *
     * @param lafChoiceModel    provides the models for the look&amp;feel choice
     */
    public LafChoiceView(LafChoiceModel lafChoiceModel) {
        this.lafChoiceModel = lafChoiceModel;
    }


    // Initialization *********************************************************

    private void initComponents() {
        lafList = BasicComponentFactory.createList(
                lafChoiceModel.getConfigurationSelection(),
                new LookConfigurationListCellRenderer());

        themesList = BasicComponentFactory.createList(
                lafChoiceModel.getThemeSelection(),
                new ThemeListCellRenderer());

        preview = new LafChoicePreview();
    }


    /**
     * Registers listeners to respond to selections in the
     * look-and-feel and themes lists.
     */
    private void initEventHandling() {
        lafChoiceModel.addPropertyChangeListener(
                LafChoiceModel.PROPERTYNAME_SELECTED_CONFIGURATION,
                new LookConfigurationChangeHandler());
    }


    // Building *************************************************************

    /**
     * Builds and returns a panel for choosing a Look&amp;Feel.
     * Consists of the l&amp;f and themes list on top,
     * and the preview at the bottom.
     */
    @Override
    protected JComponent buildPanel() {
        initComponents();
        initEventHandling();

        FormLayout layout = new FormLayout(
                "fill:default:grow, $ugap, fill:default:grow",
                "p, $lg, fill:100dlu, $pg, p, 1dlu, p");
        layout.setColumnGroups(new int[][]{{1, 3}});
        layout.setRowGroups(   new int[][]{{3, 7}});

        PanelBuilder builder = new PanelBuilder2(layout);
        builder.setBorder(Borders.DLU14_BORDER);
        builder.setOpaque(false);

        CellConstraints cc = new CellConstraints();
        builder.addLabel("&Look && Feel:",       cc.xy (1, 1));
        builder.add(new JScrollPane(lafList),    cc.xy (1, 3));
        builder.addLabel("&Theme:",              cc.xy (3, 1));
        builder.add(new JScrollPane(themesList), cc.xy (3, 3));
        builder.addLabel("Preview:",             cc.xyw(1, 5, 3));
        builder.add(preview,                     cc.xyw(1, 7, 3));

        JPanel panel = builder.getPanel();
        panel.addComponentListener(new SelectionsVisibleHandler());
        return panel;
    }


    // Event Handling *********************************************************

    private class LookConfigurationChangeHandler implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent evt) {
            LookConfiguration newSelection = (LookConfiguration) evt.getNewValue();
            boolean hasThemes = !newSelection.getThemes().isEmpty();
            themesList.ensureIndexIsVisible(themesList.getSelectedIndex());
            themesList.setEnabled(hasThemes);
            preview.update(
                    newSelection.getLookAndFeel(),
                    (PlasticTheme) newSelection.getTheme());
        }
    }


    private class SelectionsVisibleHandler extends ComponentAdapter {
        /**
         * The choice panel is now visible.
         * Ensures that the selected indices are visible.
         */
        @Override
        public void componentShown(ComponentEvent e) {
            lafList.ensureIndexIsVisible(lafList.getSelectedIndex());
            themesList.ensureIndexIsVisible(themesList.getSelectedIndex());
        }
    }

    // Cell Renderers *********************************************************

    // Renders instances of LookConfiguration displaying their L&amp;f name
    private static class LookConfigurationListCellRenderer extends DefaultListCellRenderer {
        @Override
        public Component getListCellRendererComponent(
                JList list, Object value, int index,
                boolean isSelected,
                boolean cellHasFocus) {
            return super.getListCellRendererComponent(
                list,
                ((LookConfiguration) value).getLookAndFeel().getName(),
                index, isSelected, cellHasFocus);
        }
    }

    // Renders instances of MetalTheme
    private static class ThemeListCellRenderer extends DefaultListCellRenderer {
        @Override
        public Component getListCellRendererComponent(
                JList list, Object value, int index,
                boolean isSelected,
                boolean cellHasFocus) {
            return super.getListCellRendererComponent(
                list,
                (value instanceof MetalTheme
                    ? ((MetalTheme) value).getName()
                    : value),
                index, isSelected, cellHasFocus);
        }
    }

}