/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.panel;

import java.awt.Color;
import java.awt.Font;
import java.awt.Insets;

import javax.swing.*;

import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.component.ToolBarButton;
import com.jgoodies.uif.util.CompoundIcon;
import com.jgoodies.uif.util.NullIcon;
import com.jgoodies.uifextras.convenience.ResourceIDs;
import com.jgoodies.uifextras.util.ActionLabel;
import com.jgoodies.uifextras.util.UIFactory;

/**
 * A panel that stacks action components generated from
 * Action instances as decorated components.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.10 $
 */
public final class ActionPanel extends JPanel {

    public static final Color DEFAULT_ENABLED_COLOR =
        Color.DARK_GRAY.brighter();
    public static final Color DEFAULT_DISABLED_COLOR =
        Color.LIGHT_GRAY.darker();
    private static final ResourceMap RESOURCES =
        Application.getResourceMap(ActionPanel.class);



    private final Icon     defaultIcon;
    private final boolean forceDefaultIcon;

    private boolean showingHelpTexts = true;

    // Instance Creation ******************************************************

    /**
     * Constructs an <code>ActionPanel</code> without default icon using
     * the mode that allows individual icons.
     */
    public ActionPanel() {
        this(null, false);
    }

    /**
     * Constructs an <code>ActionPanel</code> using the specified default icon
     * and hint, whether we will always force to use the default icon instead
     * of the individual icons as provided by the actions.
     *
     * @param defaultIcon   an icon to be used if the action provides none
     * @param forceDefaultIcon  true uses the defaultIcon for all actions
     */
    public ActionPanel(Icon defaultIcon, boolean forceDefaultIcon) {
        super(new FormLayout("fill:pref:grow", ""));
        this.forceDefaultIcon = forceDefaultIcon;
        setOpaque(false);
        if (defaultIcon != null)
            this.defaultIcon = defaultIcon;
        else {
            Icon rawDefaultIcon =
                RESOURCES.getIcon(ResourceIDs.ACTIONPANEL_DEFAULT_ICON);
            this.defaultIcon =
                new CompoundIcon(
                    rawDefaultIcon,
                    new NullIcon(14, 10));
        }
    }


    // Configuration *********************************************************

    public boolean isShowingHelpTexts() {
        return showingHelpTexts;
    }

    public void setShowingHelpTexts(boolean b) {
        showingHelpTexts = b;
    }

    /**
     * Creates an action component and adds it to the panel.
     *
     * @param action    the action to be added
     * @return the associated action component
     * @see #add(Action, Icon)
     */
    public JComponent add(Action action) {
        return add(action, null);
    }

    /**
     * Creates an action component and adds it to the panel.
     *
     * @param action    the action to be added
     * @param icon      the icon used for this action
     * @return the associated action component
     * @see #add(Action)
     */
    public JComponent add(Action action, Icon icon) {
        if (!action.isEnabled())
            return null;
        JComponent actionComponent = createActionComponent(action, icon);
        addAction(actionComponent);
        return actionComponent;
    }

    /**
     * Adds an action component to the panel.
     *
     * @param actionComponent the action's associated component to be added
     */
    private void addAction(JComponent actionComponent) {
        FormLayout layout = (FormLayout) getLayout();
        layout.appendRow(FormFactory.UNRELATED_GAP_ROWSPEC);
        layout.appendRow(FormFactory.PREF_ROWSPEC);
        add(actionComponent, new CellConstraints(1, layout.getRowCount()));
    }

    /**
     * Adds a separator component to the panel.
     */
    public void addSeparator() {
        FormLayout layout = (FormLayout) getLayout();
        layout.appendRow(FormFactory.UNRELATED_GAP_ROWSPEC);
    }

    /**
     * Creates and returns an action component.
     *
     * @param action   the action that provides texts and icons
     * @param icon     an optional icon that overrides both the individual
     *     action icon as well as the default icon
     * @return the associated action component
     */
    private JComponent createActionComponent(Action action, Icon icon) {
        String name     = (String) action.getValue(Action.SHORT_DESCRIPTION);
        String helptext = (String) action.getValue(Action.LONG_DESCRIPTION);

        JButton button = new ToolBarButton(action);
        button.setContentAreaFilled(false);
        button.setIcon(icon != null ? icon : getIconFor(action));
        if (forceDefaultIcon) {
            button.setRolloverIcon(null);
        }
        button.setBorder(Borders.EMPTY_BORDER);
        button.setMargin(new Insets(0, 0, 0, 0));

        FormLayout layout = new FormLayout(
                "left:pref, 3dlu, left:pref",
                "pref, 1dlu, pref");
        JPanel panel = new JPanel(layout);
        panel.setOpaque(false);
        CellConstraints cc = new CellConstraints();
        panel.add(button,                           cc.xy(1, 1));
        panel.add(createActionLabel(name, action),  cc.xy(3, 1));
        if (isShowingHelpTexts()) {
            cc.insets = new Insets(0, 1, 0, 0);  // Align perceived label bounds
            panel.add(UIFactory.createPlainLabel(helptext, Color.BLACK),
                      cc.xy(3, 3));
        }

        return panel;
    }

    /**
     * Creates and returns an action label for the given anchor and action.
     *
     * @param actionText   the text for the action label
     * @param action       the action that provides the action perform behavior
     * @return an action label that changes the cursor on fly-over
     */
    private JLabel createActionLabel(String actionText, Action action) {
        ActionLabel label = new ActionLabel(action);
        label.setText(actionText);
        label.setForeground(action.isEnabled()
                ? DEFAULT_ENABLED_COLOR
                : DEFAULT_DISABLED_COLOR);
        label.setIcon(null);
        label.setFontStyle(Font.BOLD);
        label.setFontSizeDelta(6);
        label.setDisplayedMnemonic(-1);
        label.setLinkColor(Color.BLACK);
        label.setToolTipText(null);
        label.setAntiAliased(true);
        return label;
    }

    /**
     * Looks up and returns an icon for the given action. In case
     * the action has no individual icon set a default icon is returned.
     *
     * @param action   the action that may provide an icon
     * @return the action's small icon, or the default icon
     */
    private Icon getIconFor(Action action) {
        Icon icon = (Icon) action.getValue(Action.SMALL_ICON);
        return forceDefaultIcon || null == icon ? defaultIcon : icon;
    }

}