/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.panel;

import java.awt.Color;
import java.awt.Image;

import javax.swing.*;
import javax.swing.text.JTextComponent;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.*;
import com.jgoodies.uifextras.util.UIFactory;

/**
 * A panel that shows a a bold title, a description and an optional icon
 * in front of a gradient background.<p>
 *
 * <strong>Note:</strong> The API of this class is work in progress.
 * Likely the visibility of the protected methods will be reduced
 * to private.<p>
 *
 * TODO: Describe the constraints for the title and description in
 * this class comment, e.g. pull up the JavaDoc comment from
 * <code>#setDescription</code>.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.6 $
 */
public class HeaderPanel extends JPanel {

    // Names of Bound Bean Properties *****************************************

    public static final String PROPERTYNAME_TITLE       = "title";
    public static final String PROPERTYNAME_DESCRIPTION = "description";
    public static final String PROPERTYNAME_ICON        = "icon";


    // Constants **************************************************************

    public static final ConstantSize DEFAULT_MINIMUM_WIDTH  =
        Sizes.dluX(200);
    public static final ConstantSize DEFAULT_MINIMUM_HEIGHT =
        Sizes.pixel(70);


    // Instance Fields ********************************************************

    private final ConstantSize minimumWidth;
    private final ConstantSize minimumHeight;

    private JLabel titleLabel;
    private JLabel iconLabel;
    private JTextComponent descriptionArea;
    private final JComponent backgroundComponent;


    // Instance Creation ****************************************************

    /**
     * Constructs a HeaderPanel with the given title and description
     * that has no icon.
     *
     * @param title        the short bold title
     * @param description  the plain text shown under the title
     */
    public HeaderPanel(String title, String description) {
        this(title, description, null);
    }


    /**
     * Constructs a HeaderPanel with the given title, description and icon.
     *
     * @param title        the short bold title
     * @param description  the plain text shown under the title
     * @param icon         the icon shown in the right-hand side
     */
    public HeaderPanel(String title, String description, Icon icon) {
        this(title, description, icon,
                DEFAULT_MINIMUM_WIDTH,
                DEFAULT_MINIMUM_HEIGHT);
    }


    /**
     * Constructs a HeaderPanel for the given title, description, icon,
     * and minimum width and minimum height.
     *
     * @param title         the short bold title
     * @param description   the 2-line plain description shown under the title
     * @param icon          the icon shown in the right center of the panel
     * @param minimumWidth  the lower bound for this header's width
     * @param minimumHeight the lower bound for this header's center height
     */
    public HeaderPanel(
            String title,
            String description,
            Icon icon,
            ConstantSize minimumWidth,
            ConstantSize minimumHeight) {
        this(title, description, icon,
                new GradientBackgroundPanel(true),
                minimumWidth,
                minimumHeight);
    }


    /**
     * Constructs a HeaderPanel for the given title, description, icon,
     * and minimum width and minimum height.
     *
     * @param title         the short bold title
     * @param description   the 2-line plain description shown under the title
     * @param icon          the icon shown in the right center of the panel
     * @param minimumWidth  the lower bound for this header's width
     * @param minimumHeight the lower bound for this header's center height
     */
    public HeaderPanel(
            String title,
            String description,
            Icon icon,
            JComponent backgroundComponent,
            ConstantSize minimumWidth,
            ConstantSize minimumHeight) {
        super(true);
        this.minimumWidth  = minimumWidth;
        this.minimumHeight = minimumHeight;
        initComponents();
        setTitle(title);
        setDescription(description);
        setIcon(icon);
        this.backgroundComponent = backgroundComponent;
        build();
    }


    // Accessors ************************************************************

    /**
     * Returns the title text that will be shown in bold.
     *
     * @return the title text.
     */
    public String getTitle() {
        return titleLabel.getText();
    }

    /**
     * Sets the title text that will be shown in bold.
     *
     * @param newTitle   new title shown in bold
     */
    public void setTitle(String newTitle) {
        String oldTitle = getTitle();
        titleLabel.setText(newTitle);
        firePropertyChange(PROPERTYNAME_TITLE, oldTitle, newTitle);
    }


    /**
     * Returns the description text.
     *
     * @return the description text.
     */
    public String getDescription() {
        return descriptionArea.getText();
    }

    /**
     * Sets this header's description text that will be shown as
     * plain text under the bold title. The description must contain
     * line wraps and shall be limited to two lines.
     *
     * @param newDescription   the text shown in plain under the title
     */
    public void setDescription(String newDescription) {
        String oldDescription = getDescription();
        descriptionArea.setText(newDescription);
        firePropertyChange(PROPERTYNAME_DESCRIPTION, oldDescription, newDescription);
    }


    /**
     * Returns this header's optional icon.
     *
     * @return the icon.
     */
    public Icon getIcon() {
        return iconLabel.getIcon();
    }

    /**
     * Sets the optional icon that will be shown on the right hand side.
     * In case the icon is an ImageIcon with a width less than 20 pixel,
     * the image used will be scaled to be twice as large as the original.
     *
     * @param icon   the icon to be shown on the right-hand side
     */
    public void setIcon(Icon icon) {
        Icon oldIcon = getIcon();
        Icon newIcon = getScaledIcon(icon);
        iconLabel.setIcon(newIcon);
        firePropertyChange(PROPERTYNAME_ICON, oldIcon, newIcon);
    }

    private Icon getScaledIcon(Icon icon) {
        if (icon == null)
            return null;
        if (icon.getIconWidth() > 20 || !(icon instanceof ImageIcon))
            return icon;
        ImageIcon imgIco = (ImageIcon) icon;
        Image image   = imgIco.getImage();
        int newWidth  = 2 * icon.getIconWidth();
        int newHeight = 2 * icon.getIconHeight();
        image = image.getScaledInstance(newWidth, newHeight, Image.SCALE_SMOOTH);
        return new ImageIcon(image);
    }


    // Building *************************************************************

    /**
     * Creates and configures the UI components.
     */
    private void initComponents() {
        titleLabel = createTitleLabel();
        descriptionArea = createDescriptionArea();
        iconLabel = new JLabel();
    }


    /**
     * Creates and returns the JLabel used for the title.
     *
     * @return the JLabel used for the title.
     */
    protected JLabel createTitleLabel() {
        return UIFactory.createBoldLabel("", 0, Color.BLACK);
    }


    /**
     * Creates and returns the JTextComponent used for the description.
     *
     * @return the JTextComponent used for the description.
     */
    protected JTextComponent createDescriptionArea() {
        JTextArea area = UIFactory.createWrappedMultilineLabel("");
        area.setLineWrap(false);
        area.setForeground(Color.BLACK);
        //area.setBackground(Color.GRAY);
        //area.setOpaque(true);
        area.setRows(2);
        return area;
    }


    protected JComponent createDefaultBackgroundComponent() {
        return new GradientBackgroundPanel(true);
    }


    // Building *************************************************************

    /**
     * Builds this header panel. This default implementation stacks the
     * center and bottom components and honors the minimum width and height.
     */
    protected void build() {
        ColumnSpec columnSpec = new ColumnSpec(
                ColumnSpec.FILL,
                Sizes.bounded(Sizes.PREFERRED, minimumWidth, null),
                ColumnSpec.DEFAULT_GROW);
        RowSpec rowSpecCenter = new RowSpec(
                RowSpec.FILL,
                Sizes.bounded(Sizes.PREFERRED, minimumHeight, null),
                RowSpec.NO_GROW);
        RowSpec rowSpecBottom = new RowSpec(Sizes.PREFERRED);
        FormLayout layout = new FormLayout(
                new ColumnSpec[]{columnSpec},
                new RowSpec[]{rowSpecCenter, rowSpecBottom});
        setLayout(layout);
        CellConstraints cc = new CellConstraints();
        add(buildCenterComponent(), cc.xy  (1, 1));
        add(buildBottomComponent(), cc.xy  (1, 2));
        add(backgroundComponent,    cc.xywh(1, 1, 1, 2));
    }


    /**
     * Builds and returns this header's center component.
     *
     * @return this header's center component.
     */
    protected JComponent buildCenterComponent() {
        FormLayout layout = new FormLayout(
            "7dlu, 9dlu, 175dlu:grow, 7dlu, pref, 4dlu",
            "4dlu, 3dlu, pref, 2dlu, pref, 0:grow, 3dlu, 4dlu");

        PanelBuilder builder = new PanelBuilder(layout); //, new FormDebugPanel());
        builder.setOpaque(false);

        CellConstraints cc = new CellConstraints();
        builder.add(titleLabel,      cc.xyw (2, 3, 2));
        builder.add(descriptionArea, cc.xywh(3, 5, 1, 2));
        builder.add(iconLabel,       cc.xywh(5, 2, 1, 6));
        //FormDebugUtils.dumpAll(builder.getPanel());
        return builder.getPanel();
    }


    /**
     * Builds and returns the bottom component, a separator by default.
     *
     * @return this header's bottom component
     */
    protected JComponent buildBottomComponent() {
        return new JSeparator();
    }


}
