/*
 * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.panel;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import javax.swing.*;
import javax.swing.border.EmptyBorder;

import com.jgoodies.uif.application.Application;
import com.jgoodies.uif.application.ResourceMap;
import com.jgoodies.uif.util.CompoundIcon;
import com.jgoodies.uif.util.NullIcon;
import com.jgoodies.uifextras.convenience.ResourceIDs;
import com.jgoodies.uifextras.util.ActionLabel;
import com.jgoodies.uifextras.util.GradientSeparator;

/**
 * A panel that stacks action components generated from Actions.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.9 $
 *
 * @see javax.swing.Action
 */
public final class ItemPanel extends JPanel {

    private static final ResourceMap RESOURCES =
        Application.getResourceMap(ItemPanel.class);


    /**
     * The icon used as fallback if an Action provides no icon.
     */
	private final Icon	defaultIcon;

    /**
     * If true, all items use this panel's default icon,
     * even if their Action provides an icon.
     */
	private final boolean forceDefaultIcon;


    // Instance Creation ******************************************************

	/**
	 * Constructs an ItemPanel with black labels that use an icon
     * if and only if the Action provides an icon.
	 */
	public ItemPanel() {
	    this(null, false, Color.BLACK);
	}


    /**
     * Constructs an ItemPanel with black labels and the given default icon.
     * Items use the default icon if the Action provides no icon.
     */
    public ItemPanel(Icon defaultIcon) {
        this(defaultIcon, false, Color.BLACK);
    }


	/**
	 * Constructs an ItemPanel using the given default icon, text foreground
     * color, and icon hint. If items are forced to use the default icon,
     * all items use the default icon. Otherwise they use the default only
     * if their Action doesn't provide an icon.
	 */
	public ItemPanel(Icon defaultIcon, boolean forceDefaultIcon,
                     Color textForeground) {
		super(new GridBagLayout());
		this.forceDefaultIcon = forceDefaultIcon;
        //this.textForeground = textForeground;
		setOpaque(false);
		if (defaultIcon != null)
			this.defaultIcon = defaultIcon;
		else {
			Icon rawDefaultIcon =
                RESOURCES.getIcon(ResourceIDs.ACTIONPANEL_DEFAULT_ICON);
			this.defaultIcon = rawDefaultIcon.getIconWidth() < 16
                ? new CompoundIcon(rawDefaultIcon, new NullIcon(16, 10))
                : rawDefaultIcon;
		}

	}


    // ************************************************************************

	/**
	 * Creates an action component and adds it to the panel.
     *
     * @param action  the action that will be used to create the component
     * @return the action component
	 */
	public JComponent add(Action action) {
		JComponent actionComponent = createActionComponent(action);
		addAction(actionComponent);
		return actionComponent;
	}


    /**
     * Adds the actions from the given list to the panel.
     *
     * @param actions	a list of actions to add to the panel
     */
    public void addAll(List actions) {
        for (Iterator i = actions.iterator(); i.hasNext();) {
            Action action = (Action) i.next();
            add(action);
        }
    }


    /**
     * Adds the actions from the given list to the panel.
     *
     * @param actions   an array of actions to add to the panel
     */
    public void addAll(Action[] actions) {
        addAll(Arrays.asList(actions));
    }


	/**
	 * Adds an action component to the panel.
	 */
	private void addAction(JComponent actionComponent) {
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill		= GridBagConstraints.VERTICAL;
		gbc.anchor		= GridBagConstraints.WEST;
		gbc.gridwidth	= GridBagConstraints.REMAINDER;
		gbc.gridheight 	= 1;
		gbc.weightx = 1.0;
		gbc.weighty = 0.0;
		gbc.insets = new Insets(0, 0, 0, 0);
		add(actionComponent, gbc);
	}


	/**
	 * Adds a separator component to the panel.
	 */
	public void addStrut(int height) {
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.gridheight = 1;
		gbc.weightx = 0.0;
		gbc.weighty = 0.0;
		gbc.insets = new Insets(height, 0, 0, 0);
		add(Box.createVerticalStrut(height), gbc);
	}


	/**
	 * Adds a separator component to the panel.
	 */
	public void addSeparator() {
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.gridheight = 1;
		gbc.weightx = 0.0;
		gbc.weighty = 0.0;
		gbc.insets = new Insets(7, 10, 7, 10);
		JComponent c = GradientSeparator.createHorizontalGradient(Color.DARK_GRAY, Color.DARK_GRAY);
		add(c, gbc);
	}


    /**
     * Creates and returns an action component for the given Action.
     */
    private JComponent createActionComponent(Action action) {
        ActionLabel label = new ActionLabel(action);
        label.setBorder(new EmptyBorder(4, 7, 3, 7));
        label.setIcon(getIconFor(action));
        label.setToolTipText(null);
        return label;
    }

	/**
	 * Looks up the icon from the given action and returns it,
     * or the default icon, if the action has no individual icon set.
	 */
	private Icon getIconFor(Action action) {
		Icon icon = (Icon) action.getValue(Action.SMALL_ICON);
		return forceDefaultIcon || null == icon ? defaultIcon : icon;
	}

}