/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.panel;

import java.awt.GridLayout;
import java.awt.event.MouseListener;
import java.util.*;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellRenderer;

import com.jgoodies.uifextras.util.TableUtilities;

/**
 * A panel that displays a table reflecting an instance of <code>Properties</code>.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.6 $
 */
public class PropertiesTable extends JPanel {

	protected JTable table;

	protected String[] headers;
	protected Object[][] data;

	private final MyTableModel model;


	/**
	 * Constructs a <code>PropertiesTable</code> for the given
	 * <code>Properties</code> and column headers.
	 */
	public PropertiesTable(Properties properties, String[] headers) {
		super(new GridLayout(0, 1));
		this.headers = headers;
		model = new MyTableModel();
		setData(properties);
		build();
	}


	/**
	 * Creates, configures, and wraps the table.
	 */
	protected void build() {
		table = new JTable(model);
		table.setAutoResizeMode(getAutoResizeMode());

		// Resize the columns to its preferred width.
		TableUtilities.resizeColumnsToPreferredWidth(table);

		// The value column and so, should have a left-aligned header.
        // Since this column has no renderer, we change the default renderer.
        TableCellRenderer defaultRenderer = table.getTableHeader().getDefaultRenderer();
        if (defaultRenderer instanceof JLabel) {
            ((JLabel) defaultRenderer).setHorizontalAlignment(SwingConstants.LEFT);
        }

		table.setRowSelectionAllowed(true);
		table.setColumnSelectionAllowed(false);
		table.setCellSelectionEnabled(false);
		table.setEnabled(true);
		table.setOpaque(false);
		JScrollPane scrollpane = new JScrollPane(table);
		add(scrollpane);
	}


	/**
	 * Adds a mouse listener to the table.
	 */
	@Override
    public void addMouseListener(MouseListener l) {
		table.addMouseListener(l);
	}


	/**
	 * Answers the table's auto resize mode.
	 */
	protected int getAutoResizeMode() {
		return JTable.AUTO_RESIZE_ALL_COLUMNS;
	}


	/**
	 * Returns the table's model.
	 */
	public MyTableModel getTableModel() {
		return model;
	}


	/**
	 * Sets a new <code>Properties</code> instance as underlying model data.
	 */
	public void setData(Properties properties) {
		data = new Object[properties.size()][2];
		int row = 0;
		Enumeration e = sort(properties.keys());
		while (e.hasMoreElements()) {
			String key = (String) e.nextElement();
			data[row][0] = key;
			data[row][1] = properties.get(key);
			row++;
		}
		model.updateData();
	}

    /**
     * Sorts an <code>Enumeration</code> and answers the sorted Enumeration.
     *
     * @param enumeration   the enumeration of all elements
     * @return an enumeration of the sorted elements
     */
    private Enumeration sort(Enumeration enumeration) {
        List list = new ArrayList(20);
        Enumeration e = enumeration;
        while (e.hasMoreElements()) {
            list.add(e.nextElement());
        }
        Collections.sort(list);
        return Collections.enumeration(list);
    }

	// Helper Class *********************************************************************

	// The table model
	private class MyTableModel extends AbstractTableModel {
		public int getColumnCount() {
			return headers.length;
		}
		public int getRowCount() {
			return data.length;
		}
		public Object getValueAt(int row, int col) {
			return row < data.length ? data[row][col] : null;
		}
		@Override
        public String getColumnName(int col) {
			return headers[col];
		}
		public void updateData() {
			fireTableDataChanged();
		}
	}

}
