/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.util;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.*;
import javax.swing.plaf.IconUIResource;

import com.jgoodies.uif.component.UIFLabel;

/**
 * A UIFLabel subclass that paints a hyperlink-like underline on mouseover.
 * Mouse clicks can perform an action either by setting this label's Action
 * or by adding an ActionListener to this label.<p>
 *
 * If the mouse enters this label the hand cursor is set; this cursor
 * will be removed either if the mouse exited or if the action performed
 * makes the label not showing.<p>
 *
 * TODO: Consider adding support for mnemonics.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.10 $
 *
 * @see javax.swing.Action
 * @see java.awt.event.ActionListener
 * @see Component#isShowing()
 */
public final class ActionLabel extends UIFLabel {

    private Color linkColor;
    private Color backgroundColor;
    private Color linkColorSave;
    private Color backgroundColorSave;

    /**
     * Describes the activation state: true for a hyperlink-like appearance,
     * false for a standard label.
     */
    private boolean active = false;

    private ActionListener actionListener = null;
    private Action action;
    private PropertyChangeListener actionPropertyChangeListener;


    // Instance Creation ******************************************************

    /**
     * Creates an ActionLabel with the given text.
     *
     * @param text    the text to be displayed
     */
    public ActionLabel(String text) {
        this(text, Color.BLUE, null);
    }


    /**
     * Creates an ActionLabel with the given text, icon, and horizontal alignment.
     *
     * @param text    the text to be displayed
     * @param icon    the icon to be displayed
     * @param horizontalAlignment  one of the following constants
     *           defined in <code>SwingConstants</code>:
     *           <code>LEFT</code>,
     *           <code>CENTER</code>,
     *           <code>RIGHT</code>,
     *           <code>LEADING</code> or
     *           <code>TRAILING</code>
     */
    public ActionLabel(String text, Icon icon, int horizontalAlignment) {
        this(text, Color.BLUE, null, icon, horizontalAlignment);
    }


    /**
     * Creates an ActionLabel with the given text, and colors for the link
     * and label background.
     *
     * @param text             the text to be displayed
     * @param linkColor        the color of the activated label
     * @param backgroundColor  the background color of the activated label
     */
    public ActionLabel(String text, Color linkColor, Color backgroundColor) {
        this(text, linkColor, backgroundColor, null, LEADING);
    }


    /**
     * Creates an ActionLabel with the given text, link color, background
     * color, icon, and horizontal alignment.
     *
     * @param text                 the text to be displayed
     * @param linkColor            the color of the activated label
     * @param backgroundColor      the background color of activated label
     * @param icon                 the icon to be displayed
     * @param horizontalAlignment  one of the following constants
     *           defined in <code>SwingConstants</code>:
     *           <code>LEFT</code>,
     *           <code>CENTER</code>,
     *           <code>RIGHT</code>,
     *           <code>LEADING</code> or
     *           <code>TRAILING</code>
     */
    public ActionLabel(
        String text,
        Color linkColor,
        Color backgroundColor,
        Icon icon,
        int horizontalAlignment) {
        super(text, false);
        setLinkColor(linkColor);
        setBackgroundColor(backgroundColor);
        addMouseListener(new ActionMouseAdapter());
        setIcon(icon);
        setHorizontalAlignment(horizontalAlignment);
    }


    /**
     * Creates an ActionLabel where properties are taken from the
     * Action supplied.
     *
     * @param action  the Action used to specify the new ActionLabel
     */
    public ActionLabel(Action action) {
        this(
            (String) action.getValue(Action.SHORT_DESCRIPTION),
            (Icon) action.getValue(Action.SMALL_ICON),
            SwingConstants.LEFT);
        configurePropertiesFromAction(action);
        setAction(action);
    }


    // Public API *************************************************************

    /**
     * Returns the color for the activated label.
     *
     * @return color of activated label
     */
    public Color getLinkColor() {
        return linkColor;
    }

    /**
     * Sets the color for the activated label.
     *
     * @param color    color of activated label
     */
    public void setLinkColor(Color color) {
        this.linkColor = color;
    }


    /**
     * Returns the background color for the activated label.
     *
     * @return background color of activated label
     */
    public Color getBackgroundColor() {
        return backgroundColor;
    }

    /**
     * Sets the background color for the activated label.
     *
     * @param color   background color of activated label
     */
    public void setBackgroundColor(Color color) {
        this.backgroundColor = color;
    }


    /**
     * Returns the currently set <code>Action</code> for this
     * <code>ActionEvent</code> source, or {@code null}
     * if no <code>Action</code> is set.
     *
     * @return the <code>Action</code> for this <code>ActionEvent</code>
     *      source, or {@code null}
     * @see Action
     * @see #setAction(Action)
     */
    public Action getAction() {
        return action;
    }

    /**
     * Sets the <code>Action</code> for the <code>ActionEvent</code> source.
     * The new <code>Action</code> replaces any previously set
     * <code>Action</code> but does not affect <code>ActionListeners</code>
     * independently added with <code>addActionListener</code>.
     * If the <code>Action</code> is already a registered
     * <code>ActionListener</code> for the button, it is not re-registered.
     * <p>
     * A side-effect of setting the <code>Action</code> is that the
     * <code>ActionEvent</code> source's properties  are immediately
     * set from the values in the <code>Action</code> (performed by the
     * method <code>configurePropertiesFromAction</code>) and
     * subsequently updated as the <code>Action</code>'s properties change
     * (via a <code>PropertyChangeListener</code> created by the method
     * <code>createActionPropertyChangeListener</code>.
     *
     * @param a the <code>Action</code> for the <code>AbstractButton</code>,
     *      or {@code null}
     * @see Action
     * @see #getAction()
     */
    public void setAction(Action a) {
        Action oldValue = action;
        if (action == null || !action.equals(a)) {
            action = a;
            if (oldValue != null) {
                removeActionListener(oldValue);
                oldValue.removePropertyChangeListener(
                    actionPropertyChangeListener);
                actionPropertyChangeListener = null;
            }
            configurePropertiesFromAction(action);
            if (action != null) {
                // Don't add if it is already a listener
                if (!isListener(ActionListener.class, action)) {
                    addActionListener(action);
                }
                // Reverse linkage:
                actionPropertyChangeListener =
                    new ActionPropertyChangeHandler(this);
                action.addPropertyChangeListener(actionPropertyChangeListener);
            }
            firePropertyChange("action", oldValue, action);
            revalidate();
            repaint();
        }
    }


    /**
     * Adds an action listener to receive action events from this label.
     *
     * @param listener  ActionListener to add to the event queue
     */
    public synchronized void addActionListener(ActionListener listener) {
        actionListener = AWTEventMulticaster.add(actionListener, listener);
    }

    /**
     * Removes the given action listener from the list of action listeners
     * associated with this label.
     *
     * @param listener  ActionListener to add to the event queue
     */
    public synchronized void removeActionListener(ActionListener listener) {
        actionListener = AWTEventMulticaster.remove(actionListener, listener);
    }


    // ************************************************************************

    private boolean isListener(Class<?> c, ActionListener a) {
        boolean isListener = false;
        Object[] listeners = listenerList.getListenerList();
        for (int i = listeners.length - 2; i >= 0; i -= 2) {
            if (listeners[i] == c && listeners[i + 1] == a) {
                isListener = true;
            }
        }
        return isListener;
    }

    /**
     * Factory method which sets the <code>ActionEvent</code>
     * source's properties according to values from the
     * <code>Action</code> instance.  The properties
     * which are set may differ for subclasses.  By default,
     * the properties which get set are <code>Text, Icon,
     * Enabled, ToolTipText, ActionCommand</code>, and <code>Mnemonic</code>.
     * <p>
     * If the <code>Action</code> passed in is {@code null},
     * the following things will occur:
     * <ul>
     * <li>the text is set to {@code null},
     * <li>the icon is set to {@code null},
     * <li>enabled is set to true,
     * <li>the tooltip text is set to {@code null}
     * </ul>
     *
     * @param a the <code>Action</code> from which to get the properties,
     *      or {@code null}
     * @see Action
     * @see #setAction(Action)
     */
    private void configurePropertiesFromAction(Action a) {
        configurePropertiesFromAction(a, null);
    }

    /**
     * Configures the AbstractButton's properties according to values
     * from the <code>Action</code> instance.  Which properties to set
     * is determined by the <code>types</code> parameter.
     * <code>types</code> may hold the following keys:
     * <ul>
     * <li><code>Action.NAME</code> - set the <code>Text</code> property
     *     from the <code>Action</code>,
     * <li><code>Action.SHORT_DESCRIPTION</code> - set the
     *     <code>ToolTipText</code> property from the <code>Action</code>,
     * <li><code>Action.SMALL_ICON</code> - set the <code>Icon</code> property
     *     from the <code>Action</code>,
     * <li><code>Action.MNEMONIC</code> - set the <code>Mnemonic</code>
     *     property from the <code>Action</code>,
     * <li><code>Action.ACTION_COMMAND_KEY</code> - set the
     *     <code>ActionCommand</code> property from the <code>Action</code>,
     * <li><code>"enabled"</code> - set <code>Enabled</code> property
     *     from the <code>Action</code>
     * </ul>
     * <p>
     * If the <code>Action</code> passed in is {@code null},
     * the following things will occur:
     * <ul>
     * <li>the text is set to {@code null},
     * <li>the icon is set to {@code null},
     * <li>enabled is set to true,
     * <li>the tooltip text is set to {@code null}
     * <li>the mnemonic is set to <code>'\0'</code>
     * </ul>
     *
     * @param a the <code>Action</code> from which to get the properties,
     *      or {@code null}
     * @param types determines which properties to set from the
     *          <code>Action</code>
     * @see Action
     * @see #setAction(Action)
     * @see #configurePropertiesFromAction(javax.swing.Action)
     */
    private void configurePropertiesFromAction(Action a, String[] types) {
        if (types == null) {
            String[] alltypes = { /* Action.MNEMONIC_KEY, */ Action.NAME,
                                  Action.SHORT_DESCRIPTION, Action.SMALL_ICON,
                                  Action.ACTION_COMMAND_KEY, "enabled" };
            types = alltypes;
        }
        for (String type : types) {
            if (type == null) continue;

            if (type.equals(Action.MNEMONIC_KEY)) {
                Integer n = (a == null) ? null : (Integer) a.getValue(type);
                setDisplayedMnemonic(n == null ? '\0' : n.intValue());
            } else if (type.equals(Action.NAME)) {
                setText(a != null ? (String) a.getValue(type) : null);
            } else if (type.equals(Action.SHORT_DESCRIPTION)) {
                setToolTipText(a != null ? (String) a.getValue(type) : null);
            } else if (type.equals(Action.SMALL_ICON)) {
                setIcon(a != null ? (Icon) a.getValue(type) : null);
            } else if (type.equals("enabled")) {
                setEnabled(a != null ? a.isEnabled() : true);
            }
        }
    }

    /**
     * Returns the icon used by this label when it's disabled.
     * If no disabled icon has been set, the label tries to construct
     * one from the default icon.
     *
     * @return the <code>disabledIcon</code> property
     * @see JLabel#setDisabledIcon(javax.swing.Icon)
     */
    @Override
    public Icon getDisabledIcon() {
        Icon superDisabledIcon = super.getDisabledIcon();
        if (superDisabledIcon != null)
            return superDisabledIcon;
        Icon defaultIcon = getIcon();
        if (defaultIcon != null && defaultIcon instanceof ImageIcon) {
            Icon icon = new IconUIResource(new ImageIcon(
                GrayFilter.createDisabledImage(
                    ((ImageIcon) defaultIcon).getImage())));
            setDisabledIcon(icon);
            return icon;
        }
        return defaultIcon;
    }


    /**
      * Paints the label.
      *
      * @param g    Graphics object for painting
      */
    @Override
    public void paint(Graphics g) {
        super.paint(g);
        if (active && isEnabled()){
            g.setColor(linkColor);
            paintUnderline(g);
        }
    }

    private void paintUnderline(Graphics g) {
        Insets i = getInsets();
        FontMetrics fm = g.getFontMetrics();

        Rectangle textRect = new Rectangle();
        Rectangle viewRect =
            new Rectangle(
                i.left,
                i.top,
                getWidth()  - (i.right + i.left),
                getHeight() - (i.bottom + i.top));

        SwingUtilities.layoutCompoundLabel(
            this,
            fm,
            getText(),
            getIcon(),
            getVerticalAlignment(),
            getHorizontalAlignment(),
            getVerticalTextPosition(),
            getHorizontalTextPosition(),
            viewRect,
            new Rectangle(),
            textRect,
            getText() == null
                ? 0
                : UIManager.getInt("Button.textIconGap"));

//        int shift =
//            ((Integer) UIManager.get("Label.textShiftOffset")).intValue();
//
        g.fillRect(
            textRect.x,
            textRect.y + fm.getAscent() + 1,
            textRect.width,
            1);
    }

    // Helper Classes *********************************************************

    private class ActionMouseAdapter extends MouseAdapter {

        /**
         * Handles the event when the mouse is released on the label.
         * Does nothing if the label is disabled or inactive (the mouse
         * already left the label's area). Otherwise it creates and
         * performs an ActionEvent using the actionListener - if any.<p>
         *
         * In case an action has been performed, we check whether this label
         * is still showing (it is visible and its parent is visible
         * and showing). If not, this label is deactivated.
         *
         * @param evt   the mouse event to handle
         *
         * @see Component#isShowing()
         */
        @Override
        public void mouseReleased(MouseEvent evt) {
            if (!isEnabled() || !active)
                return;
            if (actionListener != null) {
                actionListener.actionPerformed(
                    new ActionEvent(
                        ActionLabel.this,
                        ActionEvent.ACTION_PERFORMED,
                        "Clicked"));
                deactivateIfNotShowing();
            }
        }


        /**
         * Handles the event when the cursor enters the label.
         *
         * @param evt   the mouse event to handle
         */
        @Override
        public void mouseEntered(MouseEvent evt) {
            activateLink();
        }

        /**
          * Handles the event when the mouses exits the label.
          *
          * @param evt   the mouse event to handle
          */
        @Override
        public void mouseExited(MouseEvent evt) {
            deactivateLink();
        }


        /**
         * Deactivates this ActionLabel if it is not showing.
         * A first check is done immediately. A second check
         * is done later to handle situations where an Action
         * returns immediately, but performs later.
         */
        private void deactivateIfNotShowing() {
            if (!isShowing()) {
                deactivateLink();
                return;
            }
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    if (!isShowing())
                        deactivateLink();
                }
            });
        }

        private void activateLink() {
            active = true;

            linkColorSave = getForeground();
            setForeground(linkColor);

            backgroundColorSave = getBackground();
            if (backgroundColor != null)
                setBackground(backgroundColor);

            setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
            repaint();
        }


        private void deactivateLink() {
            active = false;
            setForeground(linkColorSave);
            setBackground(backgroundColorSave);
            setCursor(null);
            repaint();
        }


    }

    private static class ActionPropertyChangeHandler
        implements PropertyChangeListener {

        private final ActionLabel label;

        ActionPropertyChangeHandler(ActionLabel l) {
            this.label  = l;
        }

        public void propertyChange(PropertyChangeEvent e) {
            String propertyName = e.getPropertyName();
                if (e.getPropertyName().equals(Action.NAME)) {
                    Object hide = label.getClientProperty("hideActionText");
                    if (!Boolean.TRUE.equals(hide)) {
                        String text = (String) e.getNewValue();
                        label.setText(text);
                        label.repaint();
                    }
                } else if (
                    e.getPropertyName().equals(Action.SHORT_DESCRIPTION)) {
                    String text = (String) e.getNewValue();
                    label.setToolTipText(text);
                } else if (propertyName.equals("enabled")) {
                    Boolean enabledState = (Boolean) e.getNewValue();
                    label.setEnabled(enabledState.booleanValue());
                    label.repaint();
                } else if (e.getPropertyName().equals(Action.SMALL_ICON)) {
                    Icon icon = (Icon) e.getNewValue();
                    label.setIcon(icon);
                    label.invalidate();
                    label.repaint();
                }
//                else if (e.getPropertyName().equals(Action.MNEMONIC_KEY)) {
//                    Integer mn = (Integer) e.getNewValue();
//                    label.setDisplayedMnemonic(mn.intValue());
//                    label.invalidate();
//                    label.repaint();
//                }
//                  else if (
//                    e.getPropertyName().equals(Action.ACTION_COMMAND_KEY)) {
//                    label.setActionCommand((String) e.getNewValue());
//                }
        }
    }

}
