/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.util;

import java.awt.Component;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.List;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;

import com.jgoodies.uif.util.SystemUtils;

/**
 * A JTable subclass that can align and size columns.
 * Therefore it utilizes two arrays and a List for the column alignment,
 * header alignment, and the column size hints.
 * It also configures the table renderers, selection behavior,
 * row height, etc.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.7 $
 */
public class ExtTable extends JTable {

	private int[] columnAlignments	= null;
	private int[] headerAlignments	= null;
	private List<Integer>  columnSizeHints;


    // Instance Creation ******************************************************

	/**
	 * Constructs an ExtTable.
	 */
	public ExtTable() {
		configureTable();
	}


    /**
     * Constructs an ExtTable on the given TableModel.
     *
     * @param tableModel  this table's model
     */
    public ExtTable(TableModel tableModel) {
        super(tableModel);
        configureTable();
    }


	/**
	 * Constructs an ExtTable for the specified row data and column names.
	 */
    public ExtTable(Object[][] rowData, Object[] columnNames) {
    	super(rowData, columnNames);
    	configureTable();
    }


    // Initialization / Configuration *****************************************

    /**
     * Configures this table.
     */
    protected void configureTable() {
		setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

		// Modify the cell renderer.
		setDefaultRenderer(Object.class, createDefaultTableCellRenderer());
		setDefaultRenderer(Boolean.class, new BooleanRenderer());
		setDefaultEditor  (Boolean.class, new BooleanEditor());

		// Set the selection configuration.
		setRowSelectionAllowed(false);
		setColumnSelectionAllowed(false);
		setCellSelectionEnabled(false);
		setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		setEnabled(false);
		//setOpaque(false);
    }


	/**
	 * Configures the column headers: sets the alignment if available.
	 */
	private void configureColumnHeaders() {
		if (null == headerAlignments)
			return;

		for (int col = 0; col < getModel().getColumnCount(); col++) {
			if (headerAlignments[col] == SwingConstants.LEFT)
				TableUtilities.leftAlignColumn(getColumnModel().getColumn(col));
		}
	}


	/**
	 * Configures the column sizes - if available.
	 */
	private void configureColumnSizes() {
		if (null == columnSizeHints)
			return;

		for (int col = 0; col < getModel().getColumnCount(); col++) {
			int sizeHint = columnSizeHints.get(col).intValue();
			if (sizeHint != -1)
				getColumnModel().getColumn(col).setPreferredWidth(sizeHint);
		}
	}


    // ************************************************************************

	/**
	 * Creates and returns the default table cell renderer.
	 */
	protected TableCellRenderer createDefaultTableCellRenderer() {
		return new MyDefaultTableCellRenderer();
	}


	/**
	 * Returns the column alignment for the given column index.
	 */
	public int getColumnAlignment(int col) {
		return null == columnAlignments ? SwingConstants.LEFT : columnAlignments[col];
	}


	/**
	 * Returns an array of all column alignments.
	 */
	public int[] getColumnAlignments() {
		return columnAlignments;
	}


	/**
	 * Sets all column alignments.
	 */
	public void setColumnAlignments(int[] alignments) {
		this.columnAlignments = alignments;
	}


	/**
	 * Eagerly releases resources, here: the model.
	 */
	public void release() {
		setModel(new DefaultTableModel(2, 2));
	}


	/**
	 * Resets the selection.
	 */
	public void resetSelection() {
		getSelectionModel().clearSelection();
	}


	/**
	 * Sets all columns sizes using the given array of column size hints.
	 */
	public void setColumnSizes(int[] columnSizeHints) {
		List<Integer> hints = new ArrayList<Integer>();
		for (int element : columnSizeHints) {
			hints.add(Integer.valueOf(element));
        }
		setColumnSizes(hints);
	}


	/**
	 * Sets all columns sizes using the given List of column size hints.
	 */
	public void setColumnSizes(List<Integer> columnSizeHints) {
		this.columnSizeHints = columnSizeHints;
	}


	/**
	 * Sets all header alignments using the given array of alignments.
	 */
	public void setHeaderAlignments(int[] alignments) {
		this.headerAlignments = alignments;
	}


	/**
	 * Sets a new model. In addition to the superclass,
	 * we have to update the alignment and column sizes.
	 */
	@Override
    public void setModel(TableModel model) {
		super.setModel(model);
		configureColumnHeaders();
		configureColumnSizes();
		revalidate();
	}


	/**
	 * Updates the UI. In addition to the superclass behavior,
	 * this method computes and sets the row height.
	 */
	@Override
    public void updateUI() {
		configureEnclosingScrollPane();
		super.updateUI();
		setRowHeight(computeRowHeight());
	}


    protected int computeRowHeight() {
        int tableFontSize    = getFont().getSize();
        int minimumRowHeight = tableFontSize + 6;
        int defaultRowHeight = SystemUtils.IS_LOW_RES ? 17 : 18;
        return Math.max(minimumRowHeight, defaultRowHeight);
    }


    // Renderers **************************************************************

	/**
	 * A default renderer that honors the columns alignment.
	 */
	public static class MyDefaultTableCellRenderer extends DefaultTableCellRenderer {

		private static final Border EMPTY_BORDER = BorderFactory.createEmptyBorder(0, 4, 0, 4);

		@Override
        public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {
			JLabel label =
				(JLabel) super.getTableCellRendererComponent(table, value, isSelected, false, row, column);
			ExtTable decTable = (ExtTable) table;
			label.setHorizontalAlignment(decTable.getColumnAlignment(table.convertColumnIndexToModel(column)));
			label.setBorder(EMPTY_BORDER);
			return label;
		}
	}


    /**
     * Unlike the default BooleanEditor, we set the "border painted flat" property.
     */
	public static class BooleanEditor extends DefaultCellEditor {
		public BooleanEditor() {
		    super(new JCheckBox());
		    JCheckBox checkBox = (JCheckBox) getComponent();
		    checkBox.setHorizontalAlignment(SwingConstants.CENTER);
		    checkBox.setBorderPaintedFlat(true);
		}
    }


    /**
     * Unlike the default BooleanRenderer, we set the "border painted flat" property.
     */
    public static class BooleanRenderer extends JCheckBox implements TableCellRenderer {
		public BooleanRenderer() {
		    setHorizontalAlignment(SwingConstants.CENTER);
		    setBorderPaintedFlat(true);
		}

        public Component getTableCellRendererComponent(JTable table, Object value,
						       boolean isSelected, boolean hasFocus, int row, int column) {
		    if (isSelected) {
		        setForeground(table.getSelectionForeground());
		        super.setBackground(table.getSelectionBackground());
		    } else {
		        setForeground(table.getForeground());
		        setBackground(table.getBackground());
		    }
	        setSelected((value != null && ((Boolean) value).booleanValue()));
	        return this;
	    }

	    /**
	     * Overridden for performance reasons.
	     * See the <a href="#override">Implementation Note</a>
	     * for more information.
	     */
	    @Override
        public void validate() {
	        // Overridden for performance reason
        }

	    /**
	     * Overridden for performance reasons.
	     * See the <a href="#override">Implementation Note</a>
	     * for more information.
	     */
	    @Override
        public void revalidate() {
            // Overridden for performance reason
        }

	    /**
	     * Overridden for performance reasons.
	     * See the <a href="#override">Implementation Note</a>
	     * for more information.
	     */
	    @Override
        public void repaint(long tm, int x, int y, int width, int height) {
            // Overridden for performance reason
        }

	    /**
	     * Overridden for performance reasons.
	     * See the <a href="#override">Implementation Note</a>
	     * for more information.
	     */
	    @Override
        public void repaint(Rectangle r) {
            // Overridden for performance reason
        }

	    /**
	     * Overridden for performance reasons.
	     * See the <a href="#override">Implementation Note</a>
	     * for more information.
	     */
	    @Override
        protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
			// Strings get interned...
			if (propertyName == "text") {
		    	super.firePropertyChange(propertyName, oldValue, newValue);
			}
	    }

	    /**
	     * Overridden for performance reasons.
	     * See the <a href="#override">Implementation Note</a>
	     * for more information.
	     */
	    @Override
        public void firePropertyChange(String propertyName, boolean oldValue, boolean newValue) {
            // Overridden for performance reason
        }

    }


}