/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.util;

import java.awt.Font;

import javax.swing.JEditorPane;
import javax.swing.UIManager;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.StyleSheet;

import com.jgoodies.uif.util.Modes;



/**
 * Consists only of static convenience methods for HTML operations.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.4 $
 * @since 1.4
 */
public final class HTMLUtils {

    /**
     * Holds the control font that is lazily initialized
     * in <code>#getDefaultStyleSheet()</code>.
     */
    private static Font cachedTextFont;

    /**
     * Holds the default style sheet that is lazily initialized
     * in <code>#getDefaultStyleSheet()</code>.
     */
    private static StyleSheet cachedDefaultStyleSheet;


    // Instance Creation ******************************************************

	private HTMLUtils() {
		// Suppresses default constructor, ensuring non-instantiability.
	}


    // Changing the JEditorPane StyleSheet ************************************

    /**
     * Adds the default style sheet to the editor pane's document.
     * That default style sheet sets the font to a body text font
     * requested from the look&amp;feel, the tree font for Aqua,
     * the label font otherwise.
     *
     * @param editorPane         the editor pane to be modified
     *
     * @throws NullPointerException if editorPane is null
     */
    public static void addDefaultStyleSheet(JEditorPane editorPane) {
        addStyleSheet(editorPane, getDefaultStyleSheet());
    }


    /**
     * Adds a default style sheet to the editor pane's document
     * that uses the given font as text font.
     *
     * @param editorPane   the editor pane to be modified
     * @param textFont     the font used to set the name and size in the style
     *
     * @throws NullPointerException if editorPane is null
     */
    public static void addDefaultStyleSheet(JEditorPane editorPane, Font textFont) {
        addStyleSheet(editorPane, createDefaultStyleSheet(textFont));
    }


    /**
     * Adds the given style sheet to the editor pane's document.
     *
     * @param editorPane         the editor pane to be modified
     * @param newStyleSheet   the style sheet to set
     *
     * @throws NullPointerException if editorPane is null
     */
    public static void addStyleSheet(JEditorPane editorPane, StyleSheet newStyleSheet) {
        HTMLDocument document = (HTMLDocument) editorPane.getDocument();
        StyleSheet styleSheet = document.getStyleSheet();
        styleSheet.addStyleSheet(newStyleSheet);
    }


    // Helper Code ************************************************************

    private static Font getDefaultTextFont() {
        String fontKey = Modes.isLafAqua()
            ? "Tree.font"
            : "Label.font";
        return UIManager.getFont(fontKey);
    }


    private static StyleSheet getDefaultStyleSheet() {
        Font textFont = getDefaultTextFont();
        if (textFont != cachedTextFont) {
            cachedDefaultStyleSheet = createDefaultStyleSheet(textFont);
            cachedTextFont = textFont;
        }
        return cachedDefaultStyleSheet;
    }


    private static StyleSheet createDefaultStyleSheet(Font font) {
        String rule =
            "body, p, td, a, li { font-family: " + font.getName() + "; "
                               + "font-size: "   + font.getSize() + "pt;  }";
        StyleSheet styleSheet = new StyleSheet();
        styleSheet.addRule(rule);
        return styleSheet;
    }


}

