/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.util;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JPopupMenu;
import javax.swing.UIManager;

/**
 * A MouseListener that minimizes the effort required to add a JPopupMenu
 * to a JComponent. Handles mouse pressed and mouse released events and
 * shows the popup menu if the event indicates a <em>popupTrigger</em>.
 * The JPopupMenu is either rebuilt dynamically or lazily created and reused.<p>
 *
 * TODO: Consider using the Look&amp;Feel dependent popup trigger check
 * in {@link JPopupMenu#isPopupTrigger(java.awt.event.MouseEvent)}.
 * See also the commented code in <code>#popupMenuIfTriggered</code>.
 *
 * @author  Karsten Lentzsch
 * @version $Revision: 1.4 $
 */

public abstract class PopupAdapter extends MouseAdapter {

    /**
     * Indicates whether the popup menu shall be reused or rebuild everytime.
     */
    private final boolean reusePopupMenu;

    /**
     * Holds the lazily created popup menu in case it is reused.
     */
    private JPopupMenu popupMenu;

    private PropertyChangeListener lookChangeHandler;


    // Instance Creation ******************************************************

    /**
     * Creates a PopupAdapter that rebuilds its popup menu everytime.
     */
    public PopupAdapter() {
        this(false);
    }


    /**
     * Creates a PopupAdapter with the specified reuse policy.
     * Either the popup menu is reused or it is rebuild everytime.
     *
     * @param reusePopupMenu  true to lazily create and reuse the popup,
     *     false to rebuild the popup everytime
     */
    public PopupAdapter(boolean reusePopupMenu) {
        this.reusePopupMenu = reusePopupMenu;
    }


    // Abstract Method ********************************************************

	/**
	 * Builds and returns this adapter's popup menu.
     * If this adapter has been configured to rebuild the popup menu everytime,
     * this method is invoked everytime before the menu shall be shown.
     * Otherwise it'll be called once, the menu is stored and will be
     * reused until the look&amp;feel changes.
     *
     * @param e  the MouseEvent that initiated the popup menu build process
	 */
    protected abstract JPopupMenu buildPopupMenu(MouseEvent e);


    // Showing the Popup ******************************************************

	@Override
    public void mousePressed(MouseEvent e) {
		popupMenuIfTriggered(e);
	}


	@Override
    public void mouseReleased(MouseEvent e) {
		popupMenuIfTriggered(e);
	}


	private void popupMenuIfTriggered(MouseEvent e) {
        // This popup trigger check is look&feel dependent.
        // JPopupMenu menu = getPopupMenu(e);
		// if (menu.isPopupTrigger(e)) {

        // This popup trigger check is platform dependent.
        if (e.isPopupTrigger()) {
            JPopupMenu menu = getPopupMenu(e);
            menu.show(e.getComponent(), e.getX(), e.getY());
		}
	}


    // Accessing the Potentially Cached Popup Menu ****************************

    private JPopupMenu getPopupMenu(MouseEvent evt) {
        if (popupMenu == null || !reusePopupMenu) {
            popupMenu = buildPopupMenu(evt);
            ensureLookChangeHandlerRegistered();
        }
        return popupMenu;
    }

    /**
     * Ensures that a listener is registered with the UIManager
     * that handles Look&emp;Feel changes. The handler listens to
     * look changes and then resets the popup menu, so that a new request
     * will construct a new instance and automatically refresh all
     * look-related properties.
     */
    private void ensureLookChangeHandlerRegistered() {
        if (lookChangeHandler == null) {
            lookChangeHandler = new LookAndFeelChangeHandler();
            UIManager.addPropertyChangeListener(lookChangeHandler);
        }
    }


    private class LookAndFeelChangeHandler implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent e) {
            popupMenu = null;
        }
    }



}