/*
 * Copyright (c) 2000-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package com.jgoodies.uifextras.util;

import java.awt.*;
import java.awt.event.MouseEvent;

import javax.swing.*;
import javax.swing.plaf.UIResource;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.text.Document;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

import com.jgoodies.forms.factories.Borders;
import com.jgoodies.uif.component.UIFLabel;
import com.jgoodies.uif.component.UIFSplitPane;

/**
 * A factory class that consists only of static methods to create frequently
 * used Swing components.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.9 $
 */
public final class UIFactory {

    private UIFactory() {
        // Overrides default constructor; prevents instantiation.
    }

    // Labels *****************************************************************

    /**
	 * Creates and returns a <code>JLabel</code> with a bold font for the
	 * specified text.
     *
     * @param text    the label's initial text
     * @return a label that uses a bold font
	 */
    public static JLabel createBoldLabel(String text) {
        return createBoldLabel(text, 0);
    }

    /**
	 * Creates and returns a <code>JLabel</code> with a bold font for the
	 * specified text and size increment, e.g. +3 is 3 pixel larger than the
	 * default.
     *
     * @param text          the label's initial text
     * @param sizeIncrement a pixel delta that is added to the default font size
     * @return a label that uses a bold font
	 */
    public static JLabel createBoldLabel(String text, int sizeIncrement) {
        return createBoldLabel(text, sizeIncrement, UIManager.getColor(
                "controlText"));
    }

    /**
	 * Creates and returns a <code>JLabel</code> with a bold font for the
	 * specified text, size increment, and foreground color.
     *
     * @param text          the label's initial text
     * @param sizeIncrement a pixel delta that is added to the default font size
     * @param foreground    the label's foreground color
     * @return a label that uses a bold font
	 */
    public static JLabel createBoldLabel(String text, int sizeIncrement,
            Color foreground) {
        return createBoldLabel(text, sizeIncrement, foreground, false);
    }

    /**
	 * Creates and returns a <code>JLabel</code> with a bold font for the
	 * specified text, size increment, foreground color, and anti-alias
	 * setting.
     *
     * @param text          the label's initial text
     * @param sizeIncrement a pixel delta that is added to the default font size
     * @param foreground    the label's foreground color
     * @param antiAliased   true to enable anti-aliasing
     * @return a label that uses a bold font
	 */
    public static JLabel createBoldLabel(String text, int sizeIncrement,
            Color foreground, boolean antiAliased) {
        JLabel label = new UIFLabel(text, Font.BOLD, sizeIncrement, antiAliased);
        label.setForeground(foreground);
        return label;
    }

    /**
	 * Creates and returns a <code>JLabel</code> that uses a plain font.
	 * Should be used judiciously - overrides l&f settings.
     *
     * @param text    the label's initial text
     * @return a plain label that uses the control text color
	 */
    public static JLabel createPlainLabel(String text) {
        return createPlainLabel(text, UIManager.getColor("controlText"));
    }

    /**
	 * Creates and returns a <code>JLabel</code> that uses a plain font and
	 * the given foreground color.
	 * Should be used judiciously - overrides l&f settings.
     *
     * @param text         the label's initial text
     * @param foreground   the label's foreground color
     * @return a plain label that uses the control text color
	 */
    public static JLabel createPlainLabel(String text, Color foreground) {
        return createPlainLabel(text, foreground, false);
    }

    /**
	 * Creates and returns a <code>JLabel</code> that uses a plain font and
	 * the given foreground color.
	 * Should be used judiciously - override l&f settings.
     *
     * @param text         the label's initial text
     * @param foreground   the label's foreground color
     * @param antiAliased  true to enable anti-aliasing
     * @return a plain label that uses the control text color
	 */
    public static JLabel createPlainLabel(String text, Color foreground,
            boolean antiAliased) {
        JLabel label = new UIFLabel(text, Font.PLAIN, 0, antiAliased);
        if (foreground != null)
            label.setForeground(foreground);
        label.setVerticalAlignment(SwingConstants.BOTTOM);
        return label;
    }

    /**
	 * Creates and returns a JTextField, that is read-only.
	 * <p>
	 * TODO: move to BasicComponentFactory
	 *
	 * @return a read-only text field
	 */
    public static JTextField createReadOnlyTextField() {
        JTextField field = new JTextField() {
            @Override
            protected void processEvent(AWTEvent e) {
                // Ignore all events in read-only fields.
            }
        };
        field.setFocusable(false);
        field.setEditable(false);
        field.setOpaque(false);
        return field;
    }

    /**
	 * Creates and returns a <code>JTextArea</code> that can be used as a
	 * label that spans over multiple lines. Requires hand-wrapped text.
     *
     * @param text    the text component's initial text - requires newlines
     * @return a text component that renders the given text in multiple lines
     * @see #createWrappedMultilineLabel(String)
	 */
    public static JTextArea createMultilineLabel(String text) {
        return new ReadOnlyTextArea(text);
    }

    /**
	 * Creates and returns a <code>JTextArea</code> that can be used as a
	 * label that spans over multiple lines. Will wrap lines at words.
     *
     * @param text    the text component's initial text
     * @return a text component that renders the given text in multiple lines
     * @see #createMultilineLabel(String)
	 */
    public static JTextArea createWrappedMultilineLabel(String text) {
        JTextArea area = createMultilineLabel(text);
        area.setLineWrap(true);
        area.setWrapStyleWord(true);
        return area;
    }


    /**
     * Creates and returns a non-editable JEditorPane with editable background.
     *
     * @return a non-editable JEditorPane
     *
     * @since 1.6
     */
    public static JEditorPane createInfoEditorPane() {
        return new InfoEditorPane();
    }


    /**
     * Creates and returns a non-editable JTextArea with editable background.
     *
     * @return a non-editable JTextArea
     *
     * @since 1.6
     */
    public static JTextArea createInfoTextArea() {
        return new InfoTextArea();
    }


    /**
	 * Creates and returns an editor pane that shall be used for displaying
	 * HTML pages. Uses the specified properties to set whether we load
	 * synchronously or asynchronously, and process events, or shield the pane
	 * from processing events.
     *
     * @param asynchronousLoad  true to load asynchronously, false for synchronous
     * @param processEvents     true to process events
     * @return an editor pane that can display HTML pages
	 */
    public static JEditorPane createHTMLPane(boolean asynchronousLoad,
            boolean processEvents) {
        JEditorPane htmlPane;
        if (processEvents) {
            htmlPane = new InfoEditorPane() {
                @Override
                protected void processMouseMotionEvent(MouseEvent e) {
                    // Ignore only the mouse motion events
                }
            };
        } else {
            htmlPane = new InfoEditorPane() {
                @Override
                protected void processEvent(AWTEvent e) {
                    // Ignore all events in read-only mode.
                }
            };
            htmlPane.setFocusable(false);
        }
        HTMLEditorKit htmlEditorKit = asynchronousLoad
                ? new HTMLEditorKit()
                : new HTMLEditorKit() {
                    @Override
                    public Document createDefaultDocument() {
                        HTMLDocument doc = (HTMLDocument) super
                                .createDefaultDocument();
                        doc.setAsynchronousLoadPriority( -1);
                        return doc;
                    }
                };
        htmlPane.setEditorKit(htmlEditorKit);
        return htmlPane;
    }

    // Table ******************************************************************
    /**
	 * Creates and returns a component that wraps a <code>JTable</code> with
	 * a <code>JScrollPane</code>, sets the viewport's background and
	 * appropriate corners.
     *
     * @param table   the table to be wrapped
     * @return the table wrapped by a scrollpane
	 */
    public static JComponent createTablePanel(JTable table) {
        Color background = UIManager.getColor("window");

        JScrollPane scrollPane = new JScrollPane(table);
        scrollPane.getViewport().setOpaque(false);
        scrollPane.getViewport().setBackground(background);
        scrollPane.setOpaque(false);

        scrollPane.setCorner(ScrollPaneConstants.UPPER_RIGHT_CORNER,
                new JPanel(null));
        scrollPane.setCorner(ScrollPaneConstants.LOWER_RIGHT_CORNER,
                new JPanel(null));

        JPanel panel = new JPanel(new GridLayout(0, 1)) {
            @Override
            public void updateUI() {
                super.updateUI();
                setBackground(UIManager.getColor("window"));
            }
        };
        panel.setPreferredSize(new Dimension(220, 100));
        panel.add(scrollPane);

        return panel;
    }

    /**
	 * Creates and returns a left aligned table header renderer.
     *
     * @return a table cell renderer for left aligned headers
	 */
    public static TableCellRenderer createLeftAlignedDefaultHeaderRenderer() {
        DefaultTableCellRenderer label = new DefaultTableCellRenderer() {
            @Override
            public Component getTableCellRendererComponent(JTable table,
                    Object value, boolean isSelected, boolean hasFocus,
                    int row, int column) {
                if (table != null) {
                    JTableHeader header = table.getTableHeader();
                    if (header != null) {
                        this.setForeground(header.getForeground());
                        this.setBackground(header.getBackground());
                        this.setFont(header.getFont());
                    }
                }
                setText((value == null) ? "" : value.toString());
                this.setBorder(UIManager.getBorder("TableHeader.cellBorder"));
                return this;
            }
        };
        label.setHorizontalAlignment(SwingConstants.LEFT);
        return label;
    }

    // Misc *******************************************************************
    /**
	 * Creates and returns a <code>JSplitPane</code> that has empty borders.
     * Useful to avoid duplicate decorations, for example if the split pane
     * is contained by other components that already provide a border.
     *
     * @param orientation    the split pane's orientation: horizontal or vertical
     * @param comp1          the top/left component
     * @param comp2          the bottom/right component
     * @param resizeWeight   indicates how to distribute extra space
     * @return a split panes that has an empty border
	 */
    public static JSplitPane createStrippedSplitPane(int orientation,
            Component comp1, Component comp2, double resizeWeight) {
        JSplitPane split = UIFSplitPane.createStrippedSplitPane(orientation, comp1, comp2);
        split.setResizeWeight(resizeWeight);
        return split;
    }

    /**
	 * Creates and returns a <code>JScrollPane</code> that has an empty border.
     * Useful to avoid duplicate decorations, for example if the scrollpane
     * is contained by other components that already provide a border.
     *
     * @param component   the component to wrap
     * @return the scrollpane that has no border
	 */
    public static JScrollPane createStrippedScrollPane(Component component) {
        JScrollPane scrollPane = new JScrollPane(component);
        scrollPane.setBorder(BorderFactory.createEmptyBorder());
        return scrollPane;
    }

    /**
	 * Returns a light background <code>Color</code>. Computes and answers a
	 * <code>Color</code> that has a minimum and maximum brightness and is
	 * slightly darker than the specified <code>Color</code>.
     *
     * @return a color used as a background that is slightly darker than
     * the color used by controls
	 */
    public static Color getLightBackground() {
        Color color = UIManager.getColor("control");
        float[] hsbValues = new float[3];
        Color.RGBtoHSB(color.getRed(), color.getGreen(), color.getBlue(),
                hsbValues);
        float hue = hsbValues[0];
        //float saturation = hsbValues[1];
        float brightness = hsbValues[2];
        return brightness > 0.95f
            ? Color.WHITE
            : (Color.getHSBColor(hue, hue < 0.01f ? 0 : 0.03f, 0.97f));
    }


    // Helper Class ***********************************************************


    /**
     * A non-editable JTextArea with the editable background.
     */
    private static final class InfoTextArea extends JTextArea {

        InfoTextArea() {
            setEditable(false);
            setBackground(UIManager.getColor("TextArea.background"));
        }

        @Override
        public void updateUI() {
            super.updateUI();
            if (getBackground() instanceof UIResource) {
                setBackground(UIManager.getColor("TextArea.background"));
            }
        }

    }


    /**
     * A non-editable JEditorPane with the editable background.
     */
    private static class InfoEditorPane extends JEditorPane {

        InfoEditorPane() {
            setEditable(false);
            setBackground(UIManager.getColor("EditorPane.background"));
        }

        @Override
        public void updateUI() {
            super.updateUI();
            if (getBackground() instanceof UIResource) {
                setBackground(UIManager.getColor("EditorPane.background"));
            }
        }
    }


    private static class ReadOnlyTextArea extends JTextArea {

        ReadOnlyTextArea(String text) {
            super(text);
            setEditable(false);
            setFocusable(false);
            setBorder(Borders.EMPTY_BORDER);
        }

        @Override
        public void updateUI() {
            super.updateUI();
            setOpaque(false);
            setMargin(new Insets(0, 0, 0, 0));
            if (getForeground() instanceof UIResource)
                setForeground(UIManager.getColor("controlText"));
        }

    }


}
